/*******************************************************************************
**
** The source file for the Jahshaka core object file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

JahLayer::JahLayer(void) 
{

    //////////////////////////////////////////////////////////////
    //flag to let you know if a mediaobject was created
    //we only create mediaobject when a media file is
    //assigned to the layer to keep layers light
    mediaExchangePresent  = false;

    //initialize JahBasePath and JahMediaPath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath  = jprefs.getBasePath();
    JahMediaPath = jprefs.getMediaPath();

    ///////////////////////////////////////////////////////////////
    //this will be moved out into the world and sent that way
    //for now we start up using default resolution
    int jahres = jprefs.getJahResolution();

    updateResolution(jahres);


    ////////////////////////////////////////////////////////////////
    //set up default variables 
    setupDefaultVariables();

    usedagpu = false;

    ////////////////////////////////////////////////////////////////
    //set up default variables 
    setupKeyframeVariables();

}

/////////////////////////////////////////////////////
// its doing this for EVERY layer on construction
//but we only need to do it for background layers

void JahLayer::updateResolution(int res)
{

    projectData thedata;

    X_RESOLUTION = thedata.getXres(res);
    Y_RESOLUTION = thedata.getYres(res);

    widthOffset  = (GLfloat)X_RESOLUTION/2;
    heightOffset = (GLfloat)Y_RESOLUTION/2;

}

JahLayer::~JahLayer(void)
{
    delete node;

    //see void QPtrCollection::setAutoDelete ( bool enable )
    //autodelete isnt working and needs to be cleaned up
    layernodes->clear();
    delete layernodes;

    if (mediaExchangePresent)
        delete mediaExchange;
}

void JahLayer::setupKeyframeVariables()
{
    //set up the nodes for keyframes

    // node is always equal to the current node
    node        = new motionNode;

    layernodes  = new JahKeyframeObject;

    clear();
}

//////////////////////////////////////////////////////////////
// routines necessary to initialize external objects

void JahLayer::setupDefaultVariables() 
{

    //layer flags and default settings
    layerAssetStatus     = 0;        
    keyAssetStatus       = 0;
    foreground           = 0;

    selected        = FALSE;
    drawKeyframes   = FALSE;

    showEdges       = true;
    loop			= false;
    ping			= false;

    pixelshader     = FALSE;
    layerfog        = FALSE;
    drawtheKey      = FALSE;

    //add these to the save/load routines
    blend           = 1;
    blendmode1      = GL_DST_COLOR;
    blendmode2      = GL_ZERO;
    depth           = FALSE;
    reflect			= FALSE;   
    smooth			= TRUE;

    cull            = 0;        
    lighting        = 0;

    //for the built in layer effects
    layerEffect     = FALSE;
    blur            = FALSE;
    hblur           = 0.0;
    vblur           = 0.0;

    negative        = FALSE;
    swaprgb         = FALSE;
    mirror          = FALSE;

    //Set default color
    Red = Green = Blue = 1.0f;

    //true means its textured, false is wireframe
    ObjectDisplayMode = FALSE;

    //layer mesh for effects
    MeshFirsttime   = FALSE;
    foreground      = FALSE;   
    keyAssetStatus  = FALSE;
    invertKey       = FALSE;
    blend           = FALSE;   
    depth           = FALSE;

    //3d obj object data defaults
    model_list      = 0;	             // display list for object
    smoothing_angle = 90.0;	             // smoothing angle
    model_file      = NULL;              // name of the obect file
    material_mode   = 0;
    facet_normal    = GL_FALSE;

    sceneExists = false;
    modelExists = false;

    //set up particle defaults
    ParticleDraw    = PARLINE;
    ParticleColors  = FALSE;

    renderStatus = false;

    qtfont			= FALSE; //for qt gl fonts

    option1    = false;
    option2    = false;
    option3    = false;
    option4    = false;

}

//we have to initialize the textobject first before using it
void JahLayer::addTextObj() 
{

    text            = new textObj;
    text->fontshere = false;
    text->fontstyle = FPOLYGON;

    if (qtfont)
    {
        text->theqfont = new QFont("helvetica");

        text->bold      = false;
        text->italic    = false;
        text->underline = false;
        text->strikeout = false;

        text->pointsize = 24;
        text->pixelsize = 0;
        text->weight    = 0;
        text->pitch     = 0;
        text->stretch   = 0;
    }
}

//we have to initialize the mediaobject first before using it
void JahLayer::addMediaObj() 
{

    if (!mediaExchangePresent)
    {
        mediaExchange = new assetExchange;
        mediaExchangePresent=true;
    }
}


//////////////////////////////////////////////////////////////
// miscellaneous routines


//routine to print out all info on object
void JahLayer::printInfo(void) 
{
    QString data;

    debug("Printing out the layer data");
    debug("---------------------------");
    debug( "LayerName   :" + layername );
    debug( "LayerType   :" + data.setNum(layertype   ) );
    debug( "ObjectType  :" + data.setNum(objtype     ) );
    debug( "Status      :" + data.setNum(layerStatus ) );
    debug( "Selected    :" + data.setNum(selected    ) );
    debug( " " );
    debug( "Key         :" + data.setNum(keyAssetStatus   ) );
    debug( "DrawKey     :" + data.setNum(drawtheKey  ) );
    debug( "InvertKey   :" + data.setNum(invertKey   ) );
    //debug( "VideoTexKey :" + data.setNum(videotexkey ) );
    debug( " " );
    debug( "Blending    :" + data.setNum(blend       ) );
    debug( "Depth       :" + data.setNum(depth       ) );
    debug( "Lighting    :" + data.setNum(lighting    ) );
    debug( "Reflection  :" + data.setNum(reflect     ) );
    debug("---------------------------");
    //asset.printinfo();
    debug("---------------------------");

}






