/*******************************************************************************
**
** The header file for the Jahshaka core structures file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef CORESTRUCTURES_H
#define CORESTRUCTURES_H

//FTGL headers for fonts
#include <FTGLOutlineFont.h>
#include <FTGLPolygonFont.h>
#include <FTGLBitmapFont.h>
#include <FTGLTextureFont.h>
#include <FTGLPixmapFont.h>
#include "FTGLExtrdFont.h"

#include "qimage.h"

/////////////////////////////////////////////////////////
//core data structures
/////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
//core layer structures

enum LayerCategory     { LAYER, PARTICLE, OBJECTS, FRAME, TEXT, GRID, AXIS, LIGHT };

enum ObjectCategory    { JAHLAYER, MESHLAYER, MAINBACK, BACKDROP,
                        BEZIERS, OBJOBJECTS, MAXOBJECTS, CUBE, SPHERE, CYLINDER,
                        JAHTEXT, TITLEGRID, JAHPARTICLE, JAHAXIS, JAHLIGHT, JAHFOG, 
                        EFFECT };  //effects will soon be broken out and flagged
                                    //ie if (effectson) then see the qptlis in the object of effects applied

/////////////////////////////////////////////////////////
//core texture structure
//objectimage is the source image and texture is the image after formatting
//this makes the layers twice the size but is necessary since
//opengl doesnt suport non ^2 texture maps 
//maybe we can use the texID ? 
//opengl 1.2 supports non ^2 texture maps we can fix it then

struct TextureImage {

//can we get rid of this qimage here?
//replace with pointer to database???
QImage       objectImage;                            // Image Data (Up To 32 Bits)
GLuint       bpp;                                    // Image Color Depth In Bits Per Pixel.
GLfloat      wresolution;                            // resolution for texture map
GLfloat      hresolution;                            // resolution for texture map
GLfloat      width;                                  // Image Width
GLfloat      height;                                 // Image Height
GLfloat      x;                                      // 3dspace x cord
GLfloat      y;                                      // 3dspace y cord

//can we get rid of this qimage here?
//replace with pointer to database???
QImage       texture;                                // the opengl texture
GLfloat      texwidth;                               // texture Width for map
GLfloat      texheight;                              // texture Height for map
GLfloat      texwidthratio;                          // ratio of texture width to next largest power of two
GLfloat      texheightratio;                         // ratio of texture height to next largest power of two
GLuint       texID;                                  // Texture ID Used To Select A Texture

GLfloat      mapwidth;                               // Image mapping width
GLfloat      mapheight;                              // Image mapping height


};



/////////////////////////////////////////////////////////
//core font data structures

enum FontDrawstyle     { FBITMAP, FPIXMAP, FOUTLINE, FPOLYGON, FEXTRUDE, FTEXTURE };

struct textObj {

    QString     text;
    QString     font;
    double      extrude;
    int         size;
    QString     getText() { return text; };
    FontDrawstyle fontstyle;

    //for freetype fonts
    FTFont* fonts[6];
    bool fontshere;
    int width;
    int height;

    //for Qt font support
    QFont			*theqfont;

    bool bold;
    bool italic;
    bool underline;
    bool strikeout;

    int  pointsize;
    int  pixelsize;
    int  weight;
    int  pitch;
    int  stretch;

};




////////////////////////////////////////////////////////////
// for particle objects

enum ParticleCategory  { ATOMIC, JETSPRAY, WATERFALL, ROCKET, EXPLOSION, SWIRL,
                        BALLOONS, SHAPE, FIREFLIES, SHOWER, FOUNTAIN };

enum ParticleDrawstyle { PARLINE, PARPOINT, PARGAUSS, PARTEXTURE };


#endif





