/*******************************************************************************
**
** The source file for the Jahshaka core 3dobject file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"
#include <qfiledialog.h>

////////////////////////////////////////////////
// 3d object loaders

//should be bool so we know if it worked
bool JahLayer::loadOBJ(const QString filename)
{
//debug("loading obj");
//first check if filename is there...

    QFile f( filename );
    if (f.exists())
    {

        //this hack is used to reload models
        //we delete one if its there and create a new one
    if (modelExists)
    {
        glmDelete(model);
		model = new GLMmodel;
    }
    else
    {
        modelExists = true;
		model = new GLMmodel;

    }

    bool returnval = false;

    //#ifndef __sgi
    //if (model) glmDelete(model);
    //#endif

    const char* text1= filename;
    int   text1len =  filename.length();
    char* rere = new char[text1len];
    strcpy(rere,text1);

    model = glmReadOBJ(rere);   //was model_file

    scale = glmUnitize(model);
    glmFacetNormals(model);
    glmVertexNormals(model, smoothing_angle);

    if (model->nummaterials > 0) material_mode = 2;
                            else material_mode = 0;

    glmScale(model, 10);

    if (model) returnval = true;

    return returnval;
    }
    else
    return false;

}


void JahLayer::makeObj()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug("making obj");

    if (model)
    {

    //jtrace.debug("material_mode",material_mode);
    //jtrace.debug("facet_normal",facet_normal);

    //just got to figure out texturing properly!
    glShadeModel( GL_SMOOTH );

    glPushMatrix(); //first we correct the orientation
    glNormal3f( 0.0, 0.0, 0.0 );

    glScalef ( 40, 40, 40 );

    if (material_mode == 0)
    {
        if (facet_normal)
		{
            glmDraw(model, GLM_FLAT);
		}
        else
		{
            glmDraw(model, GLM_SMOOTH);
		}
    }
    else if (material_mode == 1)
    {
        if (facet_normal)
		{
            glmDraw(model, GLM_FLAT | GLM_COLOR);
		}
        else
		{
            glmDraw(model, GLM_SMOOTH | GLM_COLOR);
		}
    }
    else if (material_mode == 2)
    {
        if (facet_normal)
		{
            glmDraw(model, GLM_FLAT | GLM_MATERIAL);
		}
        else
		{
            glmDraw(model, GLM_SMOOTH | GLM_MATERIAL);
		}
    }

    else if (material_mode == 4)
    {
        if (facet_normal)
		{
            glmDraw(model, GLM_FLAT | GLM_TEXTURE);
		}
        else
		{
            glmDraw(model, GLM_SMOOTH | GLM_TEXTURE);
		}
    }
    else if (material_mode == 5)
    {
        if (facet_normal)
		{
            glmDraw(model, GLM_FLAT | GLM_NONE);
		}
        else
		{
            glmDraw(model, GLM_SMOOTH | GLM_NONE);
		}
    }

	glPopMatrix();
    //jtrace.debug("made obj");
    }
}

bool JahLayer::loadMaxOBJ(const QString filename)
{
    QFile f( filename );
    if (f.exists())
    {

    bool status;

	//this hack is used to reload models
	//we delete one if its there and create a new one
    if (sceneExists)
    {
        delete scene;
        scene = new L3DS;
    }
    else
    {
        scene = new L3DS;
        sceneExists = true;
    }

	//here we have to convert the
	//file name to const char*
    const char* text1= filename;
    int   text1len =  filename.length();
    char* rere = new char[text1len];
    strcpy(rere,text1);

	//and here we load the object into the
	//maxobject
    if (!scene->LoadFile(rere))
    {
        status = false;
    }
    else
    {
        status = true;
    }

    return status;
    }
    else
    return false;
}


void JahLayer::makeMaxObj()
{
    //glShadeModel( GL_SMOOTH );
    //glEnable(GL_LIGHTING);
    //glEnable(GL_NORMALIZE);
    //glEnable(GL_TEXTURE_2D);

    if (scene)
    {

    glPushMatrix(); //first we correct the orientation
    glScalef ( 40, 40, 40 );
    glNormal3f( 0.0, 0.0, 0.0 );

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    int a = scene->GetMeshCount();

    //printf("There are %d meshes in the 3ds file\n", a);

    for (uint i= 0; i<scene->GetMeshCount(); i++)
    {
        LMesh &mesh = scene->GetMesh(i);

        glVertexPointer(4, GL_FLOAT, 0, &mesh.GetVertex(0));
        glTexCoordPointer(4, GL_FLOAT, 0, &mesh.GetVertex(0));
        glNormalPointer(GL_FLOAT, 0, &mesh.GetNormal(0));
        glColorPointer(3, GL_FLOAT, 0, &mesh.GetBinormal(0));
        glDrawElements(GL_TRIANGLES, mesh.GetTriangleCount()*3,
        GL_UNSIGNED_SHORT, &mesh.GetTriangle(0));
    }

    glPopMatrix(); //first we correct the orientation

    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_COLOR_ARRAY);

    //glDisable(GL_NORMALIZE);
    //glDisable(GL_LIGHTING);
    //if textureing is on... need a flag for this
    //glDisable(GL_TEXTURE_2D);
    }
}


