/*******************************************************************************
**
** The source file for the Jahshaka core asset file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

//////////////////////////////////////////////////////////
//assign a asset to the layer object
//this can be audio, video, etc
void JahLayer::assignAsset(videoData clip, VideoCategory cliptype)
{

    if (cliptype==IMAGE)
    {
        asset = clip;

        //if there is a slip
        if (clip.clipinframe>0 || clip.clipinframe<0)
        {
            inframe	    = clip.clipinframe;
            outframe    = clip.clipoutframe;
        }
        else
        {
            inframe	    = clip.startframe;
            outframe    = clip.endframe;
        }

        layerAssetStatus = true;
    }

    if (cliptype==KEY)
    {
        keyasset = clip;

        //if there is a slip
        if (clip.clipinframe>0 || clip.clipinframe<0)
        {
            keyinframe	    = clip.clipinframe;
            keyoutframe     = clip.clipoutframe;
        }
        else
        {
            keyinframe	    = clip.startframe;
            keyoutframe     = clip.endframe;
        }

        //this should be renamed to keyHasAsset;
        keyAssetStatus = TRUE;
    }

}

//////////////////////////////////////////////////////////////////
// initalizes a layer  as clip or key with a new media asset
// imagetype is either CLIP or KEY

bool JahLayer::LoadLayerAsset( VideoCategory imagetype )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahLayer::","Loading Asset into Layer");

    QImage   buf;

    //initialize the media exchange if its not already here
    //this should be moved out so its not checked every time

    if (!mediaExchangePresent)
    {
        addMediaObj();
    }

    if (imagetype==IMAGE)
    {
		
        buf=mediaExchange->getImage(asset, currentframe);

        updateGlTexture (buf);
        InitializeTexture(textureData);
    }

    if (imagetype==NOCATEGORY)
    {
		//if its no-category we need to override
		//the image displayed
		//as we are using this for new layers
        if ( !buf.load( JahBasePath+"Pixmaps/jahlayer.png" ) ) {	// Load first image from file
            jtrace.error( "Could not read default image, using single-color instead." );
            QImage dummy( 256, 256, 32 );
            dummy.fill( Qt::blue.rgb() );
            buf = dummy;
        }

        updateGlTexture (buf);
        InitializeTexture(textureData);
    }

    if (imagetype==KEY)
    {
        //this should use the keyasset
        buf=mediaExchange->getImage(keyasset, currentframe);

        if (invertKey)
        {
            buf.invertPixels();
        }

        textureKey =  buildGlTexture ( buf );
        InitializeTexture(textureKey);
    }

    return true;

}

////////////////////////////////////////////////////////////
// video and image-sequencing routines
// updates the textures in 3d space from the asset texture

void JahLayer::updateVidTex(int theframe)
{

    ///////////////////////////////////////////////////
    //figure out the current frame with the offset applied
    //this is the frame we are at in the timeline
    //debug(">>called update video tex");

    bool flag, keyflag = false;

    //////////////////////////////////////////////////////////////////
    // if there is a layer asset

    if ( layerAssetStatus )
    {
        flag = updateVidTexAsset(theframe);
    }
    //////////////////////////////////////////////////////////////////
    // if there is a key asset

    if ( keyAssetStatus )
    {
        keyflag = updateVidTexKeyasset(theframe);
    }

    //////////////////////////////////////////////////////////////////
    //now update the texture with the new image
    if (flag || keyflag)
    {
        updateTexture();
    }
}

bool JahLayer::updateVidTexAsset(int frame)
{
    currentframe = frame;
    bool flag = false;

    ////////////////////////////////////////////////////////////
    //if its a video texture it can have a slip or offset

    if ( asset.videotex )
    {
        int offset;

        if      (inframe>0)  { offset = inframe-1; }
        else if (inframe<=0) { offset = inframe; }

        //apply the offset to curentframe
        currentframe -= offset;
    }

    /////////////////////////////////////////////////////////
    //the first routine checks to see if we are inside the bounds of the clip

    if ( currentframe >= asset.startframe && currentframe<=asset.endframe )
    {
        //debug("drawing the image");
        flag = true;

        textureData.objectImage = mediaExchange->getStreamImage(asset, currentframe);
        InitializeTexture(textureData);
    }
    else
    {
        //////////////////////////////////////////////////////////
        //if we are outside the limits then we can either
        //hold start and end frame (default) or loop or ping pong

        if (loop)
        {
            //this is currently assuming that we are > endFrame
            int cf = currentframe;
            int cnt = 0;  //counter used for ping pong
            int nf = asset.endframe - asset.startframe;  //number of frames

            if (nf>0)
            {
                while (cf>=asset.endframe)
                {
                    cf = cf - nf;
                    cnt += 1;
                }

            if (ping && (cnt % 2 != 0) )
                cf = nf - cf;
            }

            //update texturedata
            flag = true;
            textureData.objectImage = mediaExchange->getStreamImage(asset, cf);
            InitializeTexture(textureData);

        }
    }

    return flag;
}

bool JahLayer::updateVidTexKeyasset(int frame)
{
    int currentkeyframe = frame;
    bool flag = false;

    //QImage keyframe;

    //is the key a videolayer or a still?
    if ( keyasset.videotex )
    {
        //this is where we compute the offset for the key
        //same as above for videolayer
        int keyoffset;

        if (keystartframe>0)       { keyoffset = keystartframe-1; }
        else if (keystartframe<=0) { keyoffset = keystartframe; }

        //apply the offset to curentframe
        currentkeyframe -= keyoffset;
    }


    if (currentkeyframe >= keyasset.startframe && currentkeyframe<=keyasset.endframe)
    {
       //debug("drawing the image");
        flag = true;

        textureKey.objectImage = mediaExchange->getStreamImage(keyasset, currentkeyframe);
        if (invertKey) { textureKey.objectImage.invertPixels(); }
        InitializeTexture(textureKey);

    }
    else
    {
        //////////////////////////////////////////////////////////
        //if we are outside the limits then we can either
        //hold start and end frame (default) or loop or ping pong

        if (loop)
        {
            //this is currently assuming that we are > endFrame
            int cf = currentkeyframe;
            int cnt = 0;  //counter used for ping pong
            int nf = keyasset.endframe - keyasset.startframe;  //number of frames

            if (nf>0)
            {
                while (cf>=keyasset.endframe)
                {
                    cf = cf - nf;
                    cnt += 1;
                }

            if (ping && (cnt % 2 != 0) )
                cf = nf - cf;
            }

            //update texturedata
            flag = true;

            textureKey.objectImage = mediaExchange->getStreamImage(keyasset, cf);
            if (invertKey) { textureKey.objectImage.invertPixels(); }
            InitializeTexture(textureKey);
        }
    }

    return flag;
}
