/*******************************************************************************
**
** The source file for the Jahshaka core font file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

#include "qfont.h"

//////////////////////////////////////////////////////////
// Font Control routines

void JahLayer::drawText(void) 
{

    int current_font;

    if (text->fontshere) 
    {

        glEnable( GL_LINE_SMOOTH);

        //if textureing is on... need a flag for this also FTGL doesnt support it yet
        //glEnable(GL_TEXTURE_2D);

        switch ( text->fontstyle ) {

            case FOUTLINE     :     {    current_font = 0;                 break; }
            case FPOLYGON     :     {    current_font = 1;                 break; }
            case FTEXTURE     :     {    current_font = 2;                 break; }
            case FBITMAP      :     {    current_font = 3;                 break; }
            case FEXTRUDE     :     {    current_font = 4;
                                            text->fonts[current_font]->Depth(text->extrude);
                                    break; }
            case FPIXMAP      :     {    current_font = 5;                 break; }
            default           :     {    current_font = 1;                 break; }

        }

    
        //a embarasing positioning rig!
        glPushMatrix(); //first we correct the orientation
        glTranslatef( -40.0, 0.0, 0.0 );
        glScalef ( 5, 5, 5 );

        glNormal3f( 0.0, 0.0, 1.0);
        glColor3f(Red, Green, Blue );

        //this command actually draws the object
        text->fonts[current_font]->Render( text->text);

        glDisable( GL_LINE_SMOOTH);

        glPopMatrix();

    }
}


void JahLayer::draw3dTextOutline(void) 
{

    if (text->fontshere) 
    {

        int current_font = 4;

        glLineWidth( 0.5 );
        glEnable( GL_LINE_SMOOTH);

        float x1, y1, z1, x2, y2, z2;
        text->fonts[current_font]->BBox( text->text, x1, y1, z1, x2, y2, z2);

        // Draw the bounding box
        glDisable( GL_LIGHTING);
        
        //glDisable( GL_TEXTURE_2D);

        glEnable(GL_BLEND);
        glBlendFunc( GL_SRC_ALPHA, GL_ONE); // GL_ONE_MINUS_SRC_ALPHA
    
        //a embarasing positioning rig!
        glPushMatrix(); //first we correct the orientation
        glTranslatef( -40.0, 0.0, 0.0 );
        glScalef ( 5, 5, 5 );

        glColor3f( 0.0, 1.0, 0.0);

        // Draw the front face
        glBegin( GL_LINE_LOOP);
            glVertex3f( x1, y1, z1);
            glVertex3f( x1, y2, z1);
            glVertex3f( x2, y2, z1);
            glVertex3f( x2, y1, z1);
        glEnd();
        
        // Draw the back face
        if( current_font == 4 && z1 != z2)
        {
            glBegin( GL_LINE_LOOP);
                glVertex3f( x1, y1, z2);
                glVertex3f( x1, y2, z2);
                glVertex3f( x2, y2, z2);
                glVertex3f( x2, y1, z2);
            glEnd();
            // Join the faces
            glBegin( GL_LINES);
                glVertex3f( x1, y1, z1);
                glVertex3f( x1, y1, z2);

                glVertex3f( x1, y2, z1);
                glVertex3f( x1, y2, z2);

                glVertex3f( x2, y2, z1);
                glVertex3f( x2, y2, z2);

                glVertex3f( x2, y1, z1);
                glVertex3f( x2, y1, z2);
            glEnd();
        }

            // Draw the baseline, Ascender and Descender
        glBegin( GL_LINES);
            glColor3f( 0.0, 0.0, 1.0);
            glVertex3f( 0.0, 0.0, 0.0);
            glVertex3f( text->fonts[current_font]->Advance( text->text), 0.0, 0.0);
            glVertex3f( 0.0, text->fonts[current_font]->Ascender(), 0.0);
            glVertex3f( 0.0, text->fonts[current_font]->Descender(), 0.0);

        glEnd();

        // Draw the origin
        glColor3f( 1.0, 0.0, 0.0);
        glPointSize( 5.0);
        glBegin( GL_POINTS);
        glVertex3f( 0.0, 0.0, 0.0);
        glEnd();
        glPointSize( 0.5);

        glPopMatrix();    

        glDisable(GL_BLEND);
        glDisable( GL_LINE_SMOOTH);
    }
}


bool JahLayer::loadFontText(void) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    //need to delete and recreate object here if it exists
    //array delete crashes linux ie delete [] text->fonts;
    if (text->fontshere) { delete text->fonts[0]; delete text->fonts[1];
                            delete text->fonts[2]; delete text->fonts[3];
                            delete text->fonts[4]; delete text->fonts[5];  }

    //check if the file exists first ie look for text->font
    FILE	*check; bool fileok;

    check = fopen((const char*)text->font, "r");
    if (!check) { fileok = false; text->fontshere = false; } else { fileok = true; fclose(check); }

    //if the files there then load it in
    if (fileok)
    {
        text->fontshere = true;

        text->fonts[0] = new FTGLOutlineFont( text->font );
        text->fonts[1] = new FTGLPolygonFont( text->font );
        text->fonts[2] = new FTGLTextureFont( text->font );
        text->fonts[3] = new FTGLBitmapFont( text->font );
        text->fonts[4] = new FTGLExtrdFont( text->font );
        text->fonts[5] = new FTGLPixmapFont( text->font );

        for (int i=0; i<= 5; i++) 
        {

            jtrace.debug ("Reading font",text->font );

            int point_size = 24;

            text->fonts[i]->Depth(text->extrude);
            text->fonts[i]->CharMap(ft_encoding_unicode);

            if (!text->fonts[i]->FaceSize(point_size)) 
            {
                jtrace.error( "ERROR: Unable to set font face size " );
                fileok = false; text->fontshere = false;
            }
        }

    }

  return fileok;

}

void JahLayer::makeText(void) 
{

	//all this jumbo only if font data is new or changed
	//and font is not already loaded

	//debug("making font text");

  if (loadFontText()) {	drawText(); }

}

///////////////////////////////////////////////////////////
// for Qt font support

bool JahLayer::loadQtFontText() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

	//load the font    
	//should only do this if the font has changed...
	//use a static to test this one
	text->theqfont->setFamily("Helvetica");

	//set up the font
	text->theqfont->setItalic(text->italic);
	text->theqfont->setUnderline(text->underline);
	text->theqfont->setBold(text->bold);
	text->theqfont->setPointSize(text->pointsize);
    
	//You can specify the foundry you want in the family name. Both fonts, f1 and f2, in the above example will be set to "Helvetica [Cronyx]". 
	//To determine the attributes of the font actually used in the window system, use a QFontInfo object, e.g. 

    QFontInfo info( *text->theqfont );
    QString family = info.family();
	debug(family);
    
	//To find out font metrics use a QFontMetrics object, e.g. 

    QFontMetrics fm( *text->theqfont );
    int pixelWidth = fm.width( "How many pixels wide is this text?" );
    int pixelHeight = fm.height();

	return true;

}

