/*******************************************************************************
**
** The source file for the Jahshaka core model file
** The Jahshaka Project
** Copyright (C) 2000-2004 Visual Media, LLC.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"


extern unsigned char* loadshaderfile(const char *name);
extern void find_shader_program_error(unsigned char * source, char* program_name);
extern void create_pbuffer(int width, int height);

//#warning  FIXME    The mesh definition needs to be cleaned up
#define MESH_X_DIMENSION                            72               
#define MESH_Y_DIMENSION                            48
#define IMAGE_SCALE                                 1.25


//////////////////////////////////////////////////////////
// Mesh Control objects

void JahLayer::initializeMeshLayer( void) 
{ 
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "JahLayer::initializeMeshLayer","Initializing MESH Object");

    GLfloat                 scene_width;
    GLfloat                 scene_height;
    GLfloat                 aspect_ratio;
    GLfloat                 width_scale;
    GLfloat                 height_scale;

    scene_width = getImageWidth();
    scene_height = getImageHeight();


    // I'm not sure where the 0.5 factor comes from, but it seems to work
    aspect_ratio = ((float)scene_width / (float)scene_height) * 0.5;

    height_scale = scene_height / (float)MESH_Y_DIMENSION;
    width_scale = scene_width / (float)MESH_X_DIMENSION;

    for (int i = 0; i <= MESH_X_DIMENSION; i++) 
    {
        for (int j = 0; j <= MESH_Y_DIMENSION; j++) 
        {
            //set x and y coords in mesh
            MeshXCoord[i][j] = (float)(i - (MESH_X_DIMENSION / 2)) * width_scale; 
            MeshYCoord[i][j] = (float)(j - (MESH_Y_DIMENSION / 2)) * height_scale;
            MeshZCoord[i][j] = 0.0;
        }
    }

}

void JahLayer::setMeshCoord( int i, int j, GLfloat x, GLfloat y) {

    MeshXCoord[i][j] = x;
    MeshYCoord[i][j] = y;

}

void JahLayer::setMeshCoord( int i, int j, GLfloat x, GLfloat y, GLfloat z) {

    MeshXCoord[i][j] = x;
    MeshYCoord[i][j] = y;
    MeshZCoord[i][j] = z;
}

void JahLayer::getMeshCoord( int i, int j, GLfloat& x, GLfloat& y, GLfloat& z)
{
    x = MeshXCoord[i][j];
    y = MeshYCoord[i][j];
    z = MeshZCoord[i][j];
}

//should this be ifdef gpgpu ? 
struct float4
{
    float                x;
    float                y;
    float                z;
    float                w;
};

void JahLayer::makeMeshLayer( void) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.info( "JahLayer::makeMeshLayer","making mesh textured layer");

    //do we need this here? we use the smooth variable in jahrender.cpp to set smooth shading
    //we set a override at the end of the routine to set it back to what it supposed to be
    glShadeModel( GL_SMOOTH );   // was GL_SMOOTH
    
    //bool usedagpu = false;

    glNormal3f( 0.0, 0.0, 0.0 );

    if (ObjectDisplayMode) 
    {
        glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
    }
    
    int                         ii, jj;

    GLfloat                     i, j;


// The camera distance calculation should be passed in from the GL resize
// routine.  This is a HACK which should be fixed FIXME

#define Y_RESOLUTION 		486
#define PI                  3.14159265358979323846
#define VIEW_ANGLE			45.0
#define VIEW_ANGLE_RADIANS 	((VIEW_ANGLE / 180.0) * PI)

	GLfloat camera_distance = ((float)Y_RESOLUTION / 2.0) / tan(VIEW_ANGLE_RADIANS / 2.0);


    glBegin(GL_QUADS);

    for (jj = 0; jj < MESH_Y_DIMENSION; jj++) 
    {
        j = (float)jj;

        for (ii = 0; ii < MESH_X_DIMENSION; ii++) 
        {   
            i = (float)ii;

            glTexCoord2f(	
                        (i / MESH_X_DIMENSION) * textureData.texwidthratio, 
                        (j / MESH_Y_DIMENSION) * textureData.texheightratio);
            glVertex3f( 
                        MeshXCoord[ii][jj], 
                        MeshYCoord[ii][jj], 
                        MeshZCoord[ii][jj]);

            glTexCoord2f(
                        ((i + 1) / MESH_X_DIMENSION) * textureData.texwidthratio, 
                        ((j + 0) / MESH_Y_DIMENSION) * textureData.texheightratio);
            glVertex3f( 
                        MeshXCoord[ii + 1][jj], 
                        MeshYCoord[ii + 1][jj], 
                        MeshZCoord[ii + 1][jj]);

            glTexCoord2f(
                        ((i + 1) / MESH_X_DIMENSION) * textureData.texwidthratio, 
                        ((j + 1) / MESH_Y_DIMENSION) * textureData.texheightratio); 
            glVertex3f( 
                        MeshXCoord[ii + 1][jj + 1],   
                        MeshYCoord[ii + 1][jj + 1],   
                        MeshZCoord[ii + 1][jj + 1]);

            glTexCoord2f(
                        ((i + 0) / MESH_X_DIMENSION) * textureData.texwidthratio, 
                        ((j + 1) / MESH_Y_DIMENSION) * textureData.texheightratio);
            glVertex3f( 
                        MeshXCoord[ii][jj + 1],   
                        MeshYCoord[ii][jj + 1],   
                        MeshZCoord[ii][jj + 1]);
        }
    }

    glEnd(); //GL_QUADS


    if (ObjectDisplayMode) 
    {
        glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    }
    
    //this is used to set the GL_SMOOTH at the top of the routine back to what it supposed to be
    if (smooth)
    {
        //glShadeModel( GL_SMOOTH ); 
    }
    else
    {
        glShadeModel( GL_FLAT ); 
    }

}


