/*******************************************************************************
**
** The source file for the Jahshaka core model file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

//////////////////////////////////////////////////////////
// Layer Control objects

void JahLayer::makeLayer( void) 
{
    glNormal3f( 0.0, 0.0, 0.0 );

    glBegin(GL_QUADS);		                // begin drawing a cube

    // Front Face (note that the texture's corners have to match the quad's corners)
    //need to calculate the height/2 and width/2 on texture creation so its not a calculation 
    //thats happening all the time but is store in mapwidth and mapheight

    glTexCoord2f(0.0f, 0.0);           
    glVertex3f(-textureData.mapwidth, -textureData.mapheight,  0.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio,  0.0f);   
    glVertex3f( textureData.mapwidth, -textureData.mapheight,  0.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio,  textureData.texheightratio);                    
    glVertex3f( textureData.mapwidth,  textureData.mapheight,  0.0f);	// Top Right Of The Texture and Quad     1.0f 1.0f

    glTexCoord2f(0.0f, textureData.texheightratio);  
    glVertex3f(-textureData.mapwidth,  textureData.mapheight,  0.0f);	// Top Left Of The Texture and Quad

    glEnd();

}



void JahLayer::makeKeyLayer( void) 
{
    glNormal3f( 0.0, 0.0, 0.0 );

    glBegin(GL_QUADS);		                // begin drawing a cube

    // Front Face (note that the texture's corners have to match the quad's corners)
    glTexCoord2f(0.0f, 0.0f);              
    glVertex3f(-textureKey.mapwidth, -textureKey.mapheight,  0.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureKey.texwidthratio, 0.0f);   
    glVertex3f( textureKey.mapwidth, -textureKey.mapheight,  0.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureKey.texwidthratio,  textureKey.texheightratio);                   
    glVertex3f( textureKey.mapwidth,  textureKey.mapheight,  0.0f);	// Top Right Of The Texture and Quad     1.0f 1.0f

    glTexCoord2f(0.0f, textureKey.texheightratio);                              
    glVertex3f(-textureKey.mapwidth,  textureKey.mapheight,  0.0f);	// Top Left Of The Texture and Quad

    glEnd();

}

// black background layer is the default black background with red trim

void JahLayer::makeBlackBackground( void ) 
{

    glNormal3f( 0.0, 0.0, 0.0 );
    glColor3f(0.0f, 0.0f, 0.0f);

    if (depth) glDisable(GL_DEPTH_TEST);

    glDisable(GL_DEPTH_TEST);
    glDisable( GL_TEXTURE_2D);

    glBegin(GL_QUADS);		               

        glVertex3f(-widthOffset, -heightOffset,  0.0f);	// Bottom Left Of The Texture and Quad
        glVertex3f( widthOffset, -heightOffset,  0.0f);	// Bottom Right Of The Texture and Quad
        glVertex3f( widthOffset,  heightOffset,  0.0f);	// Top Right Of The Texture and Quad     1.0f 1.0f
        glVertex3f(-widthOffset,  heightOffset,  0.0f);	// Top Left Of The Texture and Quad

    glEnd();

    if (!renderStatus)
    {
        makeLayerOutline();
    }

    if (depth) glEnable(GL_DEPTH_TEST);


}

// backgound layer is the background with a texture map

void JahLayer::makeBackground( void) 
{
    glNormal3f( 0.0, 0.0, 0.0 );

    glBegin(GL_QUADS);		                // begin drawing a cube

    glTexCoord2f(0.0f, 0.0f);   
    glVertex3f(-widthOffset, -heightOffset,  0.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio,  0.0f);   
    glVertex3f( widthOffset, -heightOffset,  0.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio,  textureData.texheightratio);                    
    glVertex3f( widthOffset,  heightOffset,  0.0f);	// Top Right Of The Texture and Quad     1.0f 1.0f

    glTexCoord2f(0.0f, textureData.texheightratio);                    
    glVertex3f(-widthOffset,  heightOffset,  0.0f);	// Top Left Of The Texture and Quad

    glEnd();

}

void JahLayer::makeTitleGrid() 
{

    glNormal3f( 0.0, 0.0, 0.0 );

    glColor3f(0.0f, 1.0f, 0.0f);
    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

    glBegin(GL_POLYGON);

    GLfloat titleWidth  = widthOffset;// * (8/10);
    GLfloat titleHeight = heightOffset;// * (8/10);

    glVertex3f(-titleWidth, -titleHeight,  0.0f);	// Bottom Left Of The Texture and Quad
    glVertex3f( titleWidth, -titleHeight,  0.0f);	// Bottom Right Of The Texture and Quad
    glVertex3f( titleWidth,  titleHeight,  0.0f);	// Top Right Of The Texture and Quad     1.0f 1.0f
    glVertex3f(-titleWidth,  titleHeight,  0.0f);	// Top Left Of The Texture and Quad

    glEnd();

    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

}



//////////////////////////////////////////////////////////////
// Basic Objects

void JahLayer::makeCube( void) 
{

    glNormal3f( 0.0, 0.0, 0.0 );

    glBegin(GL_QUADS);		                // begin drawing a cube

    // Front Face (note that the texture's corners have to match the quad's corners)
    glNormal3f( 0.0f, 0.0f, 1.0f);                              // front face points out of the screen on z.

    glTexCoord2f(0.0f, 0.0f);					
    glVertex3f(-100.0f, -100.0f,  100.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f( 100.0f, -100.0f,  100.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f( 100.0f,  100.0f,  100.0f);	// Top Right Of The Texture and Quad

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f(-100.0f,  100.0f,  100.0f);	// Top Left Of The Texture and Quad

    // Back Face
    glNormal3f( 0.0f, 0.0f,-1.0f);                              // back face points into the screen on z.

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f(-100.0f, -100.0f, -100.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f(-100.0f,  100.0f, -100.0f);	// Top Right Of The Texture and Quad

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f( 100.0f,  100.0f, -100.0f);	// Top Left Of The Texture and Quad

    glTexCoord2f(0.0f, 0.0f);				  
    glVertex3f( 100.0f, -100.0f, -100.0f);	// Bottom Left Of The Texture and Quad

    // Top Face
    glNormal3f( 0.0f, 1.0f, 0.0f);                              // top face points up on y.

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f(-100.0f,  100.0f, -100.0f);	// Top Left Of The Texture and Quad

    glTexCoord2f(0.0f, 0.0f);					
    glVertex3f(-100.0f,  100.0f,  100.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f( 100.0f,  100.0f,  100.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f( 100.0f,  100.0f, -100.0f);	// Top Right Of The Texture and Quad

    // Bottom Face
    glNormal3f( 0.0f, -1.0f, 0.0f);                             // bottom face points down on y.

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f(-100.0f, -100.0f, -100.0f);	// Top Right Of The Texture and Quad

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f( 100.0f, -100.0f, -100.0f);	// Top Left Of The Texture and Quad

    glTexCoord2f(0.0f, 0.0f);					
    glVertex3f( 100.0f, -100.0f,  100.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f(-100.0f, -100.0f,  100.0f);	// Bottom Right Of The Texture and Quad

    // Right face
    glNormal3f( 1.0f, 0.0f, 0.0f);                              // right face points right on x.

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f( 100.0f, -100.0f, -100.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f( 100.0f,  100.0f, -100.0f);	// Top Right Of The Texture and Quad

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f( 100.0f,  100.0f,  100.0f);	// Top Left Of The Texture and Quad

    glTexCoord2f(0.0f, 0.0f);					
    glVertex3f( 100.0f, -100.0f,  100.0f);	// Bottom Left Of The Texture and Quad

    // Left Face
    glNormal3f(-1.0f, 0.0f, 0.0f);                              // left face points left on x.

    glTexCoord2f(0.0f, 0.0f);					
    glVertex3f(-100.0f, -100.0f, -100.0f);	// Bottom Left Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, 0.0f);  
    glVertex3f(-100.0f, -100.0f,  100.0f);	// Bottom Right Of The Texture and Quad

    glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);					
    glVertex3f(-100.0f,  100.0f,  100.0f);	// Top Right Of The Texture and Quad

    glTexCoord2f(0.0f, textureData.texheightratio);									
    glVertex3f(-100.0f,  100.0f, -100.0f);	// Top Left Of The Texture and Quad

    glEnd();                                    // done with the polygon.

}


void JahLayer::makeCylinder( void ) {


    GLUquadricObj* q = gluNewQuadric();

    glPushMatrix();

    glTranslatef( 0.0, 0.0, -1.0 );
    glNormal3f( 0.0, 0.0, 0.0 );

    // cylinder
    gluQuadricNormals( q, GLU_SMOOTH );
    gluQuadricTexture( q, GL_TRUE );
    //gluQuadricDrawStyle( q, GLU_FILL );
    gluCylinder(q, 6.0, 6.0, 20.0, 24, 1);

    // end cap
    glTranslatef( 0.0, 0.0, 20.0 );
    gluDisk( q, 0.0, 6.0, 24, 1 );

    // other end cap
    glTranslatef( 0.0, 0.0, -20.0 );
    gluQuadricOrientation( q, (GLenum)GLU_INSIDE );
    gluDisk( q, 0.0, 6.0, 24, 1 );

    glPopMatrix();

}

void JahLayer::makeSphere( void ) {

    GLUquadricObj* q = gluNewQuadric();

    glNormal3f( 0.0, 0.0, 0.0 );

    // cylinder
    gluQuadricNormals( q, GLU_SMOOTH );
    gluQuadricTexture( q, GL_TRUE );

    gluSphere(q,150.0f,32,32);

}




void JahLayer::makeBezier() {

    GLfloat ctrlpoints[4][4][3] =
    {
        {
            {-150.0,  -150.0,  400.0},
            { -50.0,  -150.0,  200.0},
            {  50.0,  -150.0, -100.0},
            { 150.0,  -150.0,  200.0}},
        {
            {-150.0,  -50.0,  100.0},
            { -50.0,  -50.0,  300.0},
            {  50.0,  -50.0,   0.0},
            { 150.0,  -50.0, -100.0}},
        {
            {-150.0,   50.0,  400.0},
            { -50.0,   50.0,   0.0},
            {  50.0,   50.0,  300.0},
            { 150.0,   50.0,  400.0}},
        {
            {-150.0,  150.0, -200.0},
            { -50.0,  150.0, -200.0},
            {  50.0,  150.0,   0.0},
            { 150.0,  150.0, -100.0}}
    };

    glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4,
        0, 1, 12, 4, &ctrlpoints[0][0][0]);

    glEnable(GL_MAP2_VERTEX_3);
    glEnable(GL_AUTO_NORMAL);
    glEnable(GL_NORMALIZE);

    glMapGrid2f(100, 0.0, 1.0, 100, 0.0, 1.0);

    glEvalMesh2(GL_FILL, 0, 100, 0, 100);  //100 was 20

    glDisable(GL_MAP2_VERTEX_3);
    glDisable(GL_AUTO_NORMAL);
    glDisable(GL_NORMALIZE);



}

//fogColor
//GL_FOG_START
//GL_FOG_END

//makeFogWorld
void JahLayer::makeFog() 
{
#ifndef WIN32
    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    bool HWFOG = jprefs.getHwFog();

if (HWFOG)
{
    //debug("drawing fog");
    GLfloat	fogColor[4] = {0.3f, 0.3f, 6.0f, 1.0f};							// Fog Colour

    //turns fog env on
    glEnable(GL_FOG);													// Enable Fog
    glFogi(GL_FOG_MODE, GL_LINEAR);										// Fog Fade Is Linear
    glFogfv(GL_FOG_COLOR, fogColor);									// Set The Fog Color
    glFogf(GL_FOG_START,  1.0f);										// Set The Fog Start
    glFogf(GL_FOG_END,    0.0f);										// Set The Fog End
    glHint(GL_FOG_HINT, GL_NICEST);										// Per-Pixel Fog Calculation
    glFogi(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);		// Set Fog Based On Vertice Coordinates

    //glTexCoord2f(0.0f,					textureData.texheightratio);   glVertex3f(-textureData.mapwidth, -textureData.mapheight,  0.0f);
    //glTexCoord2f(textureData.texwidthratio,  textureData.texheightratio);   glVertex3f( textureData.mapwidth, -textureData.mapheight,  0.0f);
    //glTexCoord2f(textureData.texwidthratio,  1.0f);                    glVertex3f( textureData.mapwidth,  textureData.mapheight,  0.0f);
    //glTexCoord2f(0.0f,					1.0f);                    glVertex3f(-textureData.mapwidth,  textureData.mapheight,  0.0f);	// Top Left Of The Texture and Quad


    // Front Face (note that the texture's corners have to match the quad's corners)
    //glNormal3f( 0.0f, 0.0f, 1.0f);                              // front face points out of the screen on z.
    //glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f(-100.0f, -100.0f,  100.0f);
    //glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f( 100.0f, -100.0f,  100.0f);
    //glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f( 100.0f,  100.0f,  100.0f);
    //glTexCoord2f(0.0f, 1.0f);									glVertex3f(-100.0f,  100.0f,  100.0f);

    // Back Face
    //glNormal3f( 0.0f, 0.0f,-1.0f);
    /*

    /////////////////////////////////////////////////////////
    // this code works... but needs to be cleaned up as per the code below

        glBegin(GL_QUADS);
                                    // back face points into the screen on z.
    glFogCoordfEXT( 0.0f); glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f(-100.0f, -100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f(-100.0f,  100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, 1.0f);									glVertex3f( 100.0f,  100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f( 100.0f, -100.0f, -100.0f);

    // Top Face
    //glNormal3f( 0.0f, 1.0f, 0.0f);                              // top face points up on y.
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, 1.0f);									glVertex3f(-100.0f,  100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f(-100.0f,  100.0f,  100.0f);
    glFogCoordfEXT( 1.0f); glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f( 100.0f,  100.0f,  100.0f);
    glFogCoordfEXT( 1.0f); glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f( 100.0f,  100.0f, -100.0f);

    // Bottom Face
    //glNormal3f( 0.0f, -1.0f, 0.0f);                             // bottom face points down on y.
    glFogCoordfEXT( 0.0f); glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f(-100.0f, -100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, 1.0f);									glVertex3f( 100.0f, -100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f( 100.0f, -100.0f,  100.0f);
    glFogCoordfEXT( 1.1f); glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f(-100.0f, -100.0f,  100.0f);

    // Right face
    //glNormal3f( 1.0f, 0.0f, 0.0f);                              // right face points right on x.
    glFogCoordfEXT( 1.0f); glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f( 100.0f, -100.0f, -100.0f);
    glFogCoordfEXT( 1.0f); glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f( 100.0f,  100.0f, -100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, 1.0f);									glVertex3f( 100.0f,  100.0f,  100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f( 100.0f, -100.0f,  100.0f);

    // Left Face
    //glNormal3f(-1.0f, 0.0f, 0.0f);                              // left face points left on x.
    glFogCoordfEXT( 1.0f); glTexCoord2f(0.0f, textureData.texheightratio);					glVertex3f(-100.0f, -100.0f, -100.0f);
    glFogCoordfEXT( 1.0f); glTexCoord2f(textureData.texwidthratio, textureData.texheightratio);  glVertex3f(-100.0f, -100.0f,  100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(textureData.texwidthratio, 1.0f);					glVertex3f(-100.0f,  100.0f,  100.0f);
    glFogCoordfEXT( 0.0f); glTexCoord2f(0.0f, 1.0f);									glVertex3f(-100.0f,  100.0f, -100.0f);
*/

    /*
    glBegin(GL_QUADS);													// Back Wall
        glFogCoordfEXT( 0.0f);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-2.5f,-2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 0.0f);	glVertex3f( 2.5f,-2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 1.0f);	glVertex3f( 2.5f, 2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-2.5f, 2.5f,-15.0f);
    glEnd();

    glBegin(GL_QUADS);													// Floor
        glFogCoordfEXT( 0.0f);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-2.5f,-2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 0.0f);	glVertex3f( 2.5f,-2.5f,-15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(1.0f, 1.0f);	glVertex3f( 2.5f,-2.5f, 15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-2.5f,-2.5f, 15.0f);
    glEnd();

    glBegin(GL_QUADS);													// Roof
        glFogCoordfEXT( 0.0f);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-2.5f, 2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 0.0f);	glVertex3f( 2.5f, 2.5f,-15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(1.0f, 1.0f);	glVertex3f( 2.5f, 2.5f, 15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-2.5f, 2.5f, 15.0f);
    glEnd();

    glBegin(GL_QUADS);													// Right Wall
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 0.0f);	glVertex3f( 2.5f,-2.5f, 15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 1.0f);	glVertex3f( 2.5f, 2.5f, 15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 1.0f);	glVertex3f( 2.5f, 2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 0.0f);	glVertex3f( 2.5f,-2.5f,-15.0f);
    glEnd();

    glBegin(GL_QUADS);													// Left Wall
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-2.5f,-2.5f, 15.0f);
        glFogCoordfEXT( 1.0f);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-2.5f, 2.5f, 15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-2.5f, 2.5f,-15.0f);
        glFogCoordfEXT( 0.0f);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-2.5f,-2.5f,-15.0f);
    glEnd();
    */
    glEnd();
    glFlush ();															// Flush The GL Rendering Pipeline
    glDisable(GL_FOG);													// Enable Fog
    }

    #endif
}





void JahLayer::makeParticle() {

        //glMatrixMode(GL_MODELVIEW);
        glPushMatrix(); //first we correct the orientation

        glTranslatef( 0.0, 0.0, -5.0 );
        glRotatef( -90, 1.0, 0.0, 0.0 );
        glScalef ( 40, 40, 40 );

        glDisable(GL_TEXTURE_2D);
        bool ConstColor = ParticleColors;  //otherwise it roatates the colors

        switch(ParticleDraw)
        {
            case PARLINE:      pDrawGroupp(GL_LINES, true, ConstColor);          break;
            case PARPOINT:     pDrawGroupp(GL_POINTS, true, ConstColor);         break;

            case PARGAUSS:		{	glEnable(GL_TEXTURE_2D); MakeGaussianSpotTexture();
                                    pVector view = pVector(0, 0, 0) - pVector(0, -23.0, 10);
                                    view.normalize();
                                    pVector up(0, 0, 1);
                                    DrawGroupQuadSplat(view, up, 0.16, true, true, ConstColor); break; }

            case PARTEXTURE:	{	glEnable(GL_TEXTURE_2D);
                                    makeLayerTexture();
                                    pVector view = pVector(0, 0, 0) - pVector(0, -23.0, 10);
                                    view.normalize();
                                    pVector up(0, 0, 1);
                                    DrawGroupQuadSplat(view, up, 0.16, true, true, ConstColor); break; }
        }

        switch(ParticleStyle)
        {
            case ATOMIC:       Atom(ParticleFirsttime);          break;
            case JETSPRAY:     JetSpray(ParticleFirsttime);      break;
            case WATERFALL:    Waterfall(ParticleFirsttime);     break;
            case ROCKET:       { SteerShape = PDPlane; Rocket(ParticleFirsttime);        break; }
            case EXPLOSION:    Explosion(ParticleFirsttime);     break;
            case SWIRL:        Swirl(ParticleFirsttime);         break;
            case BALLOONS:     Balloons(ParticleFirsttime);      break;
            case SHAPE:        Shape(ParticleFirsttime);         break;
            case FIREFLIES:    FireFlies(ParticleFirsttime) ;    break;
            case SHOWER:       { SteerShape = PDPlane; Shower(ParticleFirsttime);        break; }
            case FOUNTAIN:     Fountain(ParticleFirsttime);      break;
        }

        glEnable(GL_TEXTURE_2D);

ParticleFirsttime=FALSE;

//pDrawGroupp(GL_LINES, true);

glPopMatrix();

}

