#include "coreobject.h"

#include <p_vector.h>

void JahLayer::addParticleObj() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "JahLayer::addParticleObj","Creating particle object");

    // we need to make sure this is done only once...
    //particle_handle;
    action_handle   =-1;
    maxParticles    = 20000;
    demoNum         = 10;
    numSteps        = 1;
    prim            = GL_LINES;
    listID          = -1;
    SpotTexID       = -1;
    BlurRate        = 0.09;
    SteerShape      = PDSphere;

    //do we need this? or just in the routines when called between changes
    ParticleFirsttime=TRUE;

    //is this 1000 particles?
    particle_handle = pGenParticleGroups(1, 1000);
    pCurrentGroup(particle_handle);

}


void JahLayer::Atom(bool first_time = true)
{
    if(first_time && action_handle<0) 
    { 
        action_handle = pGenActionLists(1);
    //	pGenParticleGroups(1, 100);
    }

    pVelocityD(PDBlob, 0.02, -0.2, 0, 0.015);
    pSize(1.0);

    if(first_time)
        pNewActionList(action_handle);

    static float jetx=-4, jety=0, jetz=-2.4;

    static float djx = drand48() * 0.05;
    static float djy = drand48() * 0.05;
    static float djz = drand48() * 0.05;

    jetx += djx;
    jety += djy;
    jetz += djz;

    if(jetx > 10 || jetx < -10) djx = -djx;
    if(jety > 10 || jety < -10) djy = -djy;
    if(jetz > 10 || jetz < -10) djz = -djz;

    pCopyVertexB(false, true);

    int LifeTime = 350;

    pKillOld(LifeTime);

    pColorD(1, PDSphere, 0.4+fabs(jetx*0.1), 0.4+fabs(jety*0.1), 0.4+fabs(jetz*0.1), 0.1);
    pSource(maxParticles / LifeTime, PDPoint, jetx, jety, jetz);

    pOrbitPoint(2, 0, 3, 0.1, 0.1);

    pOrbitPoint(-2, 0, -3, 0.1, 0.1);

    pDamping(0.994, 0.994, 0.994, 0.2);

    //pSink(false, PDSphere, 0, 0, 0, 15);

    pMove();

    if(first_time)
        pEndActionList();
}

void JahLayer::JetSpray(bool first_time = true)
{
    static pVector Jet;
    static pVector dJet;
    static bool FirstTime = true;

    if(FirstTime)
    {
        FirstTime = false;
        double Ang = drand48() * 2.0 * M_PI;
        dJet = pVector(cos(Ang), sin(Ang), 0);
        dJet *= 0.1;
    }
    Jet += dJet;
    if(Jet.x > 10 || Jet.x < -10) {dJet.x = -dJet.x; dJet.y += drand48() * 0.005;}
    if(Jet.y > 10 || Jet.y < -10) {dJet.y = -dJet.y; dJet.x += drand48() * 0.005;}

    pColorD(1.0, PDLine, 1,0,0, 1,1,0.3);

    pVelocityD(PDBlob, 0,0,0.1, 0.01);
    pStartingAge(0);
    pSize(1);

    int LifeTime = 100;

    pKillOld(LifeTime);

    pSource(maxParticles / LifeTime, PDDisc, Jet.x, Jet.y, Jet.z, 0, 0, 1, 0.5);

    pGravity(.000, .01, .000);

    pDamping(0.9, 0.97, 0.9);

    float BOX = .017;

    pRandomAccel(PDBox, -BOX, -BOX, -BOX, BOX, BOX, BOX);

    pMove();
}


void JahLayer::Waterfall(bool first_time = true)
{
    pVelocityD(PDBlob, 0.1, 0, 0.1, 0.004);
    pColorD(1.0, PDLine, 0.8, 0.9, 1.0, 1.0, 1.0, 1.0);
    pSize(1.5);
    pStartingAge(0);

    if(first_time)
    {
        action_handle = pGenActionLists(1);
        pNewActionList(action_handle);
    }

    pCopyVertexB(false, true);

    pSource(50, PDPoint, -4, 0, 6);

    pGravity(0.0, 0.0, -0.01);

    pKillOld(250);

    pBounce(0, 0.01, 0, PDSphere, -1, 0, 4, 1);

    pBounce(0, 0.01, 0, PDSphere, -2.5, 0, 2, 1);

    pBounce(0, 0.01, 0, PDSphere, 0.7, -0.5, 2, 1);

    pBounce(-0.01, 0.35, 0, PDPlane, 0,0,0, 0,0,1);

    pMove();

    if(first_time)
        pEndActionList();
}

void JahLayer::Rocket(bool first_time = true)
{
#define MAX_ROCKETS 15
    static int RocketSystem = -1;

    if(RocketSystem < 0)
        RocketSystem = pGenParticleGroups(1, MAX_ROCKETS);

    // Move the rockets.
    pCurrentGroup(RocketSystem);
    pVelocityD(PDCylinder, 0,0,0.15, 0,0,0.2, 0.11, 0.07);
    pColorD(1, PDBox, 0,0.5,0, 1,1,1);

    pSource(0.02, PDDisc, 0,0,0, 0,0,1, 6);
    pSink(false, PDPlane, 0,0,-1, 0,0,1);
    pGravity(0,0,-0.003);
    pMove();

    // Read back the position of the rockets.
    float rocketp[MAX_ROCKETS][3], rocketc[MAX_ROCKETS][3], rocketv[MAX_ROCKETS][3];
    int rcount = pGetParticles(0, MAX_ROCKETS, (float *)rocketp, (float *)rocketc, (float *)rocketv);

    pCurrentGroup(particle_handle);

    pSize(1.0);
    pStartingAge(0, 6);

    for(int i=0; i<rcount; i++)
    {
        pVector rv(rocketv[i][0], rocketv[i][1], rocketv[i][2]);
        rv.normalize();
        rv *= -0.026;
        //cerr << i << " " <<rocketp[i][0]<<" "<<rocketp[i][1]<<" "<<rocketp[i][2]<<"\n";
        //cerr << "c " <<rocketc[i][0]<<" "<<rocketc[i][1]<<" "<<rocketc[i][2]<<"\n";

        pColorD(1.0, PDLine, rocketc[i][0], rocketc[i][1], rocketc[i][2], 1,.5,.5);
        pVelocityD(PDBlob, rv.x, rv.y, rv.z, 0.006);
        pSource(30, PDPoint, rocketp[i][0], rocketp[i][1], rocketp[i][2]);
    }

    pGravity(0,0,-0.001);
    //pDamping(0.999, 0.999, 0.999);
    pTargetColor(0,1,0,0, 0.02);
    pKillOld(90);

    pMove();
}

void JahLayer::Balloons(bool first_time = true)
{
    float x=0, y=0, z=-1;
    float qty = maxParticles / 6.0;	// Because of 6 colors

    float BBOX = 0.5;

    pStartingAge(0, 5);
    pVelocity(0,0,0);
    pColor(1,0,0);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pColor(1,1,0);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pColor(0,1,0);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pColor(0,1,1);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pColor(0,0,1);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pColor(1,0,1);
    pSource(qty, PDBox, x-BBOX, y-BBOX, z-BBOX, x+BBOX, y+BBOX, z+BBOX);

    pGravity(.0005, .005, .0005);

    pDamping(0.9, 0.67, 0.9);

    float BOX = .005;

    pRandomAccel(PDBox, -BOX, -BOX, -BOX, BOX, BOX, BOX);
    pKillOld(400);

    pMove();
    pStartingAge(0);
}


void JahLayer::Shape(bool first_time = true)
{
    if(first_time)
    {
        pKillOld(0);
        int dim = int(pow(float(maxParticles), (float)0.33333333));
#define XX 8
#define YY 12
#define ZZ 8

        float dx = 2*XX / float(dim);
        float dy = 2*YY / float(dim);
        float dz = 2*ZZ / float(dim);

        float z = -ZZ;
        for(int k=0; k<dim; k++, z += dz)
        {
            float y = -YY;
            for(int l=0; l<dim; l++, y += dy)
            {
                float x = -XX;
                for(int j=0; j<dim; j++, x += dx)
                {
                    // Make the particles.
                    pVelocityD(PDBlob, 0, 0, 0, 0.01);
                    pColor(0.5 + z*0.05, 0.5 + x*0.05, 0.5 + x*0.05);
                    pVertex(x, y, z);

                }
            }
        }

        action_handle = pGenActionLists(1);
        pNewActionList(action_handle);
        pEndActionList();
    }
}



void JahLayer::Explosion(bool first_time = true)
{
    pVelocityD(PDSphere, 0,0,0,0.01,0.01);
    pColorD(1.0, PDSphere, 0.5, 0.7, 0.5, .3);
    pSize(1.0);
    pStartingAge(0);

    pCopyVertexB(false, true);

    pDamping(0.999, 0.999, 0.999, 0.2);

    static float i=0;

    if(first_time) i=0;

    pOrbitPoint(0, 0, 0, .02, 0.1);

    pExplosion(0, 0, 0, 1, 8, 3, 0.1, i += (1.0f / float(numSteps)));

    pSink(false, PDSphere, 0, 0, 0, 30);

    pMove();
}


void JahLayer::Swirl(bool first_time = true)
{
    pVelocityD(PDBlob, 0.02, -0.2, 0, 0.015);
    pSize(1.0);
    pStartingAge(0);

    static float jetx=-4, jety=0, jetz=-2.4;

    static float djx = drand48() * 0.05;
    static float djy = drand48() * 0.05;
    static float djz = drand48() * 0.05;

    jetx += djx;
    jety += djy;
    jetz += djz;

    if(jetx > 10 || jetx < -10) djx = -djx;
    if(jety > 10 || jety < -10) djy = -djy;
    if(jetz > 10 || jetz < -10) djz = -djz;

    pCopyVertexB(false, true);

    int LifeTime = 300;

    pKillOld(LifeTime);

    pColorD(1.0, PDSphere, 0.4+fabs(jetx*0.1), 0.4+fabs(jety*0.1), 0.4+fabs(jetz*0.1), 0.1);
    pSource(maxParticles / LifeTime, PDPoint, jetx, jety, jetz);

    pOrbitLine(0, 0, 1, 1, 0, 0, 0.1, 0.1);

    pDamping(1, 0.994, 0.994);

    pSink(false, PDSphere, 0, 0, 0, 15);

    pMove();
}

void JahLayer::Shower(bool first_time = true)
{
    static float jetx=0, jety=0, jetz=0;
    static float djx, djy;

    if(first_time)
    {
        jetx = 1;
        jety = 1;
        djx = drand48() * 0.1;
        djy = drand48() * 0.1;
    }

    jetx += djx;
    jety += djy;

    if(jetx > 1 || jetx < 0) {djx = -djx; djy += drand48() * 0.0005;}
    if(jety > 2 || jety < -2) {djy = -djy; djx += drand48() * 0.0005;}

    pVelocity(0, 0, 0);
    pSize(1.5);
    pStartingAge(0);

    pColor(jetx, jety, 1);

    pSource(120, PDLine, -5,jety,8, 5,jety,8);

    pGravity(0.0, 0.0, -0.004);

    glColor3f(1,1,0);
    if(SteerShape == PDSphere)
    {
        pAvoid(0.2, 1.0, 20, PDSphere, 0,0,0, 1.1);
#ifdef JAHIRIX
debug("fix glut under Irix please");
#else
        glutSolidSphere(1, 16, 8);
#endif
    }
    else if(SteerShape == PDTriangle)
    {
        pAvoid(2, 1.0, 20, PDTriangle, 0,-1,0, 2,0,0, 0,2,0);

        glBegin(GL_TRIANGLES);
        glVertex3f(0,-1,0);
        glVertex3f(2,0,0);
        glVertex3f(0,2,0);
        glEnd();
    }
    else if(SteerShape == PDRectangle)
    {
        pAvoid(2, 1.0, 20, PDRectangle, 0,-1,0, 2,1,0, 0,2,0);

        glBegin(GL_QUADS);
        glVertex3f(0,-1,0);
        glVertex3f(2,0,0);
        glVertex3f(2,2,0);
        glVertex3f(0,1,0);
        glEnd();
    }
    else if(SteerShape == PDPlane)
    {
        pAvoid(2, 1.0, 20, PDPlane, 0,0,0, 0,0,1);

        glBegin(GL_QUADS);
        glVertex3f(-2,-2,0);
        glVertex3f(2,-2,0);
        glVertex3f(2,2,0);
        glVertex3f(-2,2,0);
        glEnd();
    }

    pKillOld(130);

    pMove();
}

// A fountain spraying up in the middle of the screen
void JahLayer::FireFlies(bool first_time = true)
{
    pSize(1.0);
    pVelocityD(PDPoint, 0,0,0);
    pColorD(1.0, PDSphere, .5, .4, .1, .1);
    pStartingAge(0);

    pCopyVertexB(false, true);

    pSource(1, PDBlob, 0, 0, 2, 2);

    pRandomAccel(PDSphere, 0, 0, 0, 0.02);

    pKillOld(20);

    pMove();
}


void JahLayer::Fountain(bool first_time = true)
{

// Set up the state.
    pVelocityD(PDCylinder, 0.01, 0.0, 0.35, 0.01, 0.0, 0.37, 0.021, 0.019);
    pColorD(1.0, PDLine, 0.8, 0.9, 1.0, 1.0, 1.0, 1.0);
    pSize(1.5);

    // Generate particles along a very small line in the nozzel.
    pSource(100, PDLine, 0.0, 0.0, 0.401, 0.0, 0.0, 0.405);

    // Gravity.
    pGravity(0.0, 0.0, -0.01);

    // Bounce particles off a disc of radius 5.
    pBounce(-0.05, 0.35, 0, PDDisc, 0, 0, 0,  0, 0, 1,  5);

    // Kill particles below Z=-3.
    pSink(false, PDPlane, 0,0,-3, 0,0,1);

    // Move particles to their new positions.
    pMove();
}




// Draw each particle as a screen-aligned triangle with texture.
// Doesn't make the texture current. Just emits texcoords, if specified.
// If size_scale is 1 and const_size is true then the textured square
// will be 2x2 in world space (making the triangle sides be 4x4).
// view and up must be normalized and unequal.

// Draw as a splat texture on a quad.
void JahLayer::DrawGroupTriSplat(const pVector &view, const pVector &up,
                    float size_scale = 1.0f, bool draw_tex=false,
                    bool const_size=false, bool const_color=false)
{
    int cnt = pGetGroupCount();

    if(cnt < 1)
        return;

    pVector *ppos = new pVector[cnt];
    float *color = const_color ? NULL : new float[cnt * 4];
    pVector *size = const_size ? NULL : new pVector[cnt];

    pGetParticles(0, cnt, (float *)ppos, color, NULL, (float *)size);

    // Compute the vectors from the particle to the corners of its tri.
    // 2
    // |\ The particle is at the center of the x.
    // |-\ V0, V1, and V2 go from there to the vertices.
    // |x|\ The texcoords are (0,0), (2,0), and (0,2) respectively.
    // 0-+-1 We clamp the texture so the rest is transparent.

    pVector right = view ^ up;
    right.normalize();
    pVector nup = right ^ view;
    right *= size_scale;
    nup *= size_scale;

    pVector V0 = -(right + nup);
    pVector V1 = V0 + right * 4;
    pVector V2 = V0 + nup * 4;

    //cerr << "x " << view.x << " " << view.y << " " << view.z << endl;
    //cerr << "x " << nup.x << " " << nup.y << " " << nup.z << endl;
    //cerr << "x " << right.x << " " << right.y << " " << right.z << endl;
    //cerr << "x " << V0.x << " " << V0.y << " " << V0.z << endl;

    glBegin(GL_TRIANGLES);

    for(int i = 0; i < cnt; i++)
    {
        pVector &p = ppos[i];
        //cerr << p.x << " " << p.y << " " << p.z << endl;
        // cerr << color[i*4+3] << endl;

        if(!const_color)
            glColor4fv((GLfloat *)&color[i*4]);

        pVector sV0 = V0;
        pVector sV1 = V1;
        pVector sV2 = V2;

        if(!const_size)
        {
        sV0 *= size[i].x;
        sV1 *= size[i].x;
        sV2 *= size[i].x;
        }

        if(draw_tex) glTexCoord2f(0,0);
        pVector ver = p + sV0;
        glVertex3fv((GLfloat *)&ver);

        if(draw_tex) glTexCoord2f(2,0);
        ver = p + sV1;
        glVertex3fv((GLfloat *)&ver);

        if(draw_tex) glTexCoord2f(0,2);
        ver = p + sV2;
        glVertex3fv((GLfloat *)&ver);
    }

    glEnd();

    delete [] ppos;
    if(color) delete [] color;
    if(size) delete [] size;
}

// Draw as a splat texture on a quad.
void JahLayer::DrawGroupQuadSplat(const pVector &view, const pVector &up,
                    float size_scale = 1.0f, bool draw_tex=false,
                    bool const_size=false, bool const_color=false)
{
    int cnt = pGetGroupCount();

    if(cnt < 1)
        return;

    pVector *ppos = new pVector[cnt];
    float *color = const_color ? NULL : new float[cnt * 4];
    pVector *size = const_size ? NULL : new pVector[cnt];

    pGetParticles(0, cnt, (float *)ppos, color, NULL, (float *)size);

    // Compute the vectors from the particle to the corners of its quad.
    // The particle is at the center of the x.
    // 3-2 V0, V1, V2 and V3 go from there to the vertices.
    // |x| The texcoords are (0,0), (1,0), (1,1), and (0,1) respectively.
    // 0-1 We clamp the texture so the rest is transparent.

    pVector right = view ^ up;
    right.normalize();
    pVector nup = right ^ view;
    right *= size_scale;
    nup *= size_scale;

    pVector V0 = -(right + nup);
    pVector V1 = right - nup;
    pVector V2 = right + nup;
    pVector V3 = nup - right;

    //cerr << "x " << view.x << " " << view.y << " " << view.z << endl;
    //cerr << "x " << nup.x << " " << nup.y << " " << nup.z << endl;
    //cerr << "x " << right.x << " " << right.y << " " << right.z << endl;
    //cerr << "x " << V0.x << " " << V0.y << " " << V0.z << endl;

    glBegin(GL_QUADS);

    for(int i = 0; i < cnt; i++)
    {
        pVector &p = ppos[i];
        //cerr << p.x << " " << p.y << " " << p.z << endl;
        // cerr << color[i*4+3] << endl;

        if(!const_color)
            glColor4fv((GLfloat *)&color[i*4]);

        pVector sV0 = V0;
        pVector sV1 = V1;
        pVector sV2 = V2;
        pVector sV3 = V3;

        if(!const_size)
        {
        sV0 *= size[i].x;
        sV1 *= size[i].x;
        sV2 *= size[i].x;
        sV3 *= size[i].x;
        }

        if(draw_tex) glTexCoord2f(0,0);
        pVector ver = p + sV0;
        glVertex3fv((GLfloat *)&ver);

        if(draw_tex) glTexCoord2f(1,0);
        ver = p + sV1;
        glVertex3fv((GLfloat *)&ver);

        if(draw_tex) glTexCoord2f(1,1);
        ver = p + sV2;
        glVertex3fv((GLfloat *)&ver);

        if(draw_tex) glTexCoord2f(0,1);
        ver = p + sV3;
        glVertex3fv((GLfloat *)&ver);
    }

    glEnd();

    delete [] ppos;
    if(color) delete [] color;
    if(size) delete [] size;
}


///////////////////////////////////////////////////////////////////////////////
// for the particle system

inline float JahLayer::Gaussian2(float x, float y, float sigma)
{
    // The sqrt of 2 pi.
    #define SQRT2PI 2.506628274631000502415765284811045253006

    //exp is the exponential function
    return exp(-0.5 * (x*x + y*y) / (sigma*sigma)) / (SQRT2PI * sigma);
}

////////////////////////////////////////////////////
// we can also use this to create brushes in paint
void JahLayer::MakeGaussianSpotTexture()
{
    #define DIM 32
    #define DIM2 (DIM>>1)
    #define TEX_SCALE 7.0

    //glGenTextures(1, (GLuint *)&texture);
    //glBindTexture(GL_TEXTURE_2D, texture);
    
    glGenTextures(1, &textureData.texID);
    glBindTexture(GL_TEXTURE_2D, textureData.texID);


    float *img = new float[DIM*DIM];

    for(int y=0; y<DIM; y++)
    {
        for(int x=0; x<DIM; x++)
        {
            if(x==0 || x==DIM-1 || y==0 || y==DIM-1)
            img[y*DIM+x] = 0;
            else
            {
                img[y*DIM+x] = TEX_SCALE * Gaussian2(x-DIM2, y-DIM2, (DIM*0.15));
            }
        }
    }

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    float col[4] = {1.f, 1.f, 1.f, 1.f};
    glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, col);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    //GL_ASSERT();

    gluBuild2DMipmaps(GL_TEXTURE_2D, GL_ALPHA16, DIM, DIM, GL_ALPHA, GL_FLOAT, img);
}

