/*******************************************************************************
**
** The source file for the Jahshaka Core jahtexture module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"


////////////////////////////////////////////////////////////
// build, initialize and rebuild textures

////////////////////////////////////////////////////////////////////////////////
// heard this was a faster way to do things, plus the textures dont have to be a power of 2
// and so they can be computed directly from the image! this is great!!!
// glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, textureData.texture.width(), textureData.texture.height(),
//	                            GL_RGBA,  GL_UNSIGNED_BYTE, textureData.texture.bits());




//this one works directly on the motionPath's predefined textureData
//and is quicker
void JahLayer::updateGlTexture (  QImage & buf  ) 
{
    //we should check if a images is 24 or 32 bit here for the texture module
    int mapwidth =  buf.width();
    int mapheight = buf.height();

    int mapvalw, mapvalh;

    //here we determine the resolution of the opengl texture
    //since opengl textures must be an ordxer of 2^8 
    if      (mapwidth<=128)   { mapvalw = 128; } 
    else if (mapwidth<=256)   { mapvalw = 256; } 
    else if (mapwidth<=512)   { mapvalw = 512; } 
    else if (mapwidth<=1024)  { mapvalw = 1024;} 
    else if (mapwidth<=2048)  { mapvalw = 2048;}

    if      (mapheight<=128)  { mapvalh = 128; } 
    else if (mapheight<=256)  { mapvalh = 256; } 
    else if (mapheight<=512)  { mapvalh = 512; } 
    else if (mapheight<=1024) { mapvalh = 1024;} 
    else if (mapheight<=2048) { mapvalh = 2048;}

    textureData.objectImage   = buf;

    //fix this to save on texture ram ie test image for bitdepth first
    int depth                 = buf.depth();
    textureData.bpp           = depth;        
    //textured.bpp            = 32;         // default was to 32bit but we need to test for this for efficiency

    textureData.wresolution   = mapvalw;    // was layerres
    textureData.hresolution   = mapvalh;    // was layerres
    textureData.width         = mapwidth;
    textureData.height        = mapheight;

    textureData.texture       = QImage(mapvalw, mapvalh, depth);

    textureData.texwidth      = ( textureData.width / textureData.wresolution);
    textureData.texheight     =  1- ( textureData.height / textureData.hresolution );
    
	textureData.texwidthratio = textureData.width / textureData.wresolution;
	textureData.texheightratio = textureData.height / textureData.hresolution;

    //mapping ratio to product video aspect 4:3
    //calculation used is 640/2=320 /10 = 32
    textureData.x = (mapwidth/2)/10;
    textureData.y = ((mapheight/2)/10) * 1.24; //aspect ratio corrections here

    textureData.mapwidth = (GLfloat)mapwidth / 2;
    textureData.mapheight= (GLfloat)mapheight / 2;

}

//havent figured out exactly why we need 2 of these routines
//this routine returns a TextureImage... where is it used
TextureImage JahLayer::buildGlTexture (  QImage & buf  ) {

    TextureImage textured; // builds a texture object from the qimage buf

    //we should check if a images is 24 or 32 bit here for the texture module
    int mapwidth =  buf.width();
    int mapheight = buf.height();
    int mapvalw, mapvalh;

    //here we determine the resolution of the opengl texture
    //since opengl textures must be an ordxer of 2^8
    if      (mapwidth<=128)   { mapvalw = 128; }
    else if (mapwidth<=256)   { mapvalw = 256; }
    else if (mapwidth<=512)   { mapvalw = 512; }
    else if (mapwidth<=1024)  { mapvalw = 1024;}
    else if (mapwidth<=2048)  { mapvalw = 2048;}

    if      (mapheight<=128)  { mapvalh = 128; }
    else if (mapheight<=256)  { mapvalh = 256; }
    else if (mapheight<=512)  { mapvalh = 512; }
    else if (mapheight<=1024) { mapvalh = 1024;}
    else if (mapheight<=2048) { mapvalh = 2048;}

    textured.objectImage   = buf;

    //fix this to save on texture ram ie test image for bitdepth first
    int depth         = buf.depth();
    textured.bpp           = depth;
    //textured.bpp           = 32;         // default was to 32bit but we need to test for this for efficiency

    textured.wresolution    = mapvalw;    // was layerres
    textured.hresolution    = mapvalh;    // was layerres
    textured.width         = mapwidth;
    textured.height        = mapheight;

    textured.texture       = QImage(mapvalw, mapvalh, depth);

    textured.texwidth      = ( textured.width / textured.wresolution);
    textured.texheight     =  1- ( textured.height / textured.hresolution );
    
	textured.texwidthratio = textureData.width / textureData.wresolution;
	textured.texheightratio = textureData.height / textureData.hresolution;

    //  calculation is           640/2=320 /10 = 32
    textured.x = (mapwidth/2)/10;
    textured.y = ((mapheight/2)/10) * 1.24; //aspect ratio corrections here

    //mapping ration to product video aspect 4:3
    textured.mapwidth = (GLfloat)mapwidth / 2;
    textured.mapheight= (GLfloat)mapheight / 2;

    return textured;
}

//would be great to make this faster with bitBlt
//but it doesnt seem to be working
void JahLayer::InitializeTexture(TextureImage & textured) 
{
    //debug("calling InitializeTexture");
    //bitBlt(&textured.texture,0,0,&textureData.objectImage,0,0,(int)textured.wresolution,(int)textured.hresolution);

    //need to release the old texture image here first
    if (!textured.texture.isNull()) {textured.texture.reset();}

    textured.texture = textured.objectImage.copy(0,0,(int)textured.width,(int)textured.height,0);
    textured.texture = QGLWidget::convertToGLFormat( textured.texture );  // flipped 32bit RGBA

}

//this is called by the effects module and layer effects
void JahLayer::rebuildTextureData (  QImage & buf  ) 
{
//    debug("calling rebuildTextureData");
    textureData.objectImage = buf;

    InitializeTexture(textureData);
        
    updateTexture();
    
}

void JahLayer::rebuildTextureDataFx (  QImage & buf, bool conv  )
{

    //new code by Tony for effects module
    //only works with meshlayers and not with non ^2 clips using cpu effects ie try Polar effect as 1st fx
    //is not compatable with effects in the core ie animation module
    //but it doesnt reallocate the texture again unnecessarily
    //by working directly on the texture itself vs using the source image do we loose quality ?
    
    //textureData.objectImage = buf;
	if (conv)
      buf = QGLWidget::convertToGLFormat(buf); 
	
    //updateTexture(textureData.texture.width(), );
    glBindTexture(GL_TEXTURE_2D,  textureData.texID);   // 2d texture (x and y size)

	// scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); 
	// scale linearly when image smalled than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); 

    //QString conv;
    //debug("texturedata width :"+conv.setNum((GLint)textureData.width));
    //debug("texturedata height:"+conv.setNum((GLint)textureData.height));
    
    //debug("buf width :"+conv.setNum((GLint)buf.width()));
    //debug("buf height:"+conv.setNum((GLint)buf.height()));
    
    //use GLRGBA only if its so...
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (GLint)textureData.width, 
					(GLint)textureData.height, GL_RGBA, GL_UNSIGNED_BYTE, buf.bits());                   
                       
}

void JahLayer::rebuildTextureKey (  QImage & buf  ) 
{

//debug("build key");
    textureKey.objectImage = buf;

    InitializeTexture(textureKey);
    updateTexture();
}


void JahLayer::changeKeyStatus()
{
//debug("change key");
    invertKey = !(invertKey);
    textureKey.objectImage.invertPixels();

    textureKey =  buildGlTexture ( textureKey.objectImage );
    InitializeTexture(textureKey);  

    makeTexture();

}


void JahLayer::setLayerTexture(void) 
{
//debug("set layer");
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,  textureData.texID );
}

void JahLayer::setKeyTexture(void) 
{
//debug("set key");
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,  textureKey.texID );
}

////////////////////////////////////////////////////////////
// make the texture itself

void JahLayer::makeTexture( void )
{

    //erase texture if it already exists
	//theres a memory leak in makeLayerTexture maybe the delete isnt working?

    if (textureData.texID)     {   glDeleteTextures(1,& textureData.texID); }

    makeLayerTexture();

    if (  keyAssetStatus == TRUE) {
      if (textureKey.texID)     {   glDeleteTextures(1,& textureKey.texID);  }

          makeKeyTexture();
    }

}


void JahLayer::makeLayerTexture( void ) 
{   

    //procedure to map textureimage to texture

	//this routine is causing a major memory leak for some reason
	//hiting get desktop for a layer repeatedly will show this

    glGenTextures(1, &textureData.texID);
    glBindTexture(GL_TEXTURE_2D,  textureData.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    // Sgi systems (o2's at least) wont allow a 32bit texture and alpha blending to work simultaneously
    // the work around is to disable the alpha channel on 32 bit image files
    // maybe this will be fixed on ocatnes and or tga files

   //change this to a global ifdef later...
	
   #ifdef JAHIRIX
     //glTexImage2D( GL_TEXTURE_2D, 0, 3,        textureData.texture.width(), textureData.texture.height(), 0,      //last value is the border
     //                                GL_RGBA, GL_UNSIGNED_BYTE,
     //                                textureData.texture.bits() );

    glTexImage2D( GL_TEXTURE_2D, 0, 3, (GLint)textureData.wresolution, 
                            (GLint)textureData.hresolution, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (GLint)textureData.width, 
                            (GLint)textureData.height, GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits());                   

   #else
 
   //for some reason avi's on linux only paint this way without the alpha
   //see mediaobject.cpp line 219 and jahtexture.cpp line
   //if (asset->extension==".avi") {  
    if (asset.theType==MOVIE) 
    {
    //glTexImage2D( GL_TEXTURE_2D, 0, 3,        textureData.texture.width(), textureData.texture.height(), 0,      //last value is the border
    //                                 GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits() );

        glTexImage2D( GL_TEXTURE_2D, 0, 3, (GLint)textureData.wresolution, 
                            (GLint)textureData.hresolution, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (GLint)textureData.width, 
                            (GLint)textureData.height, GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits());                   
    
    } 
    else
    {
    //glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,  textureData.wresolution, textureData.hresolution, 0,      //last value is the border
    //                                 GL_RGBA, GL_UNSIGNED_BYTE,
    //                                 textureData.texture.bits() );

        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,  (GLint)textureData.wresolution, 
                            (GLint)textureData.hresolution, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL );

        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (GLint)textureData.width, 
            (GLint)textureData.height, GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits());                   

    }
   #endif



}

void JahLayer::makeKeyTexture( void ) {  //procedure to map textureimage to keytexture

    glGenTextures(1, & textureKey.texID);
    glBindTexture(GL_TEXTURE_2D,  textureKey.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    //glTexImage2D( GL_TEXTURE_2D, 0, 3, textureKey.texture.width(), textureKey.texture.height(), 0,      //last value is the border
    //                                   GL_RGBA, GL_UNSIGNED_BYTE, textureKey.texture.bits() );
        
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,  (GLint)textureKey.wresolution, 
                            (GLint)textureKey.hresolution, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL );

        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (GLint)textureKey.width, 
            (GLint)textureKey.height, GL_RGBA, GL_UNSIGNED_BYTE, textureKey.texture.bits());                   

}




void JahLayer::updateTexture( void ) 
{
    //used for video-clips we delete the old textures and create new ones
    //can we can SPEEEED this up by looking into
    //glCopyTexImage2D() and  copy the new data into the texture

    //its safe to assume the texture was already built once
    //glDeleteTextures(1,& textureData.texID); //should be taken care of in image loader?

   updateLayerTexture();

    if (  keyAssetStatus ) 
    {
           //glDeleteTextures(1,& textureKey.texID);
           updateKeyTexture();
    }

}

void JahLayer::updateLayerTexture( void ) 
{   
    //procedure to update texture in a sequence
    glBindTexture(GL_TEXTURE_2D,  textureData.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    //use GLRGBA only if its so...
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, textureData.texture.width(), textureData.texture.height(),
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits());


}

void JahLayer::updateKeyTexture( void ) {   //procedure to update texture in a sequence

    glBindTexture(GL_TEXTURE_2D,  textureKey.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, textureKey.texture.width(), textureKey.texture.height(),
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureKey.texture.bits());

}























