/*******************************************************************************
**
** The header file for the Jahshaka render module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "jahrender.h"

//#include "GL/glext.h"


JahRender::JahRender(void)
{ 

}

JahRender::~JahRender(void) { }

void JahRender::setToolMode(MouseMode mmode) 
{ 
    theToolMode = mmode; 
}

void JahRender::positionObject( motionNode * node ) 
{

    glTranslatef( node->tx, node->ty, node->tz );
    glRotatef( node->rx, 1.0, 0.0, 0.0 );
    glRotatef( node->ry, 0.0, 1.0, 0.0 );
    glRotatef( node->rz, 0.0, 0.0, 1.0 );

    glScalef ( node->sx, node->sy, node->sz );
    //glScalef ( (node->sx / 10.0) + 0.9, (node->sy / 10.0) + 0.9, (node->sz / 10.0) + 0.9 );

}


void JahRender::drawLayer(JahLayer & layer) 
{
// we need ways to speed this up as its the core routine

    if ( layer.layerStatus )
    {
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();

        //position the object acording to the keyframe set in node
        positionObject(layer.node);

        //now we set up drawing variables and draw the object
        if ( layer.layerfog ) { glEnable(GL_FOG); }                     

        // used for object selection - background is excluded
        //if (layer.objtype!=BACKDROP || layer.objtype!=MAINBACK) {  glLoadName(layer.layernumber); }
        if (layer.objtype!=BACKDROP || layer.objtype!=MAINBACK) 
		{  
			glLoadName(layer.asset.clipnumber); 
		}

        // set object color and transparency
        glColor4f( layer.Red,  layer.Green,  layer.Blue, ( layer.node->Alpha/100) );

        if ( layer.blend )     {    glEnable(GL_BLEND);
                                    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); }

        if ( layer.depth)      {    //debug("enabling depth"); 
                                    glEnable(GL_DEPTH_TEST);       }

        if ( layer.lighting )  {    glEnable(GL_LIGHTING);         }

        if ( layer.reflect)    {    glEnable(GL_TEXTURE_GEN_S);    
                                    glEnable(GL_TEXTURE_GEN_T);}

        if ( layer.smooth)     {    glShadeModel( GL_SMOOTH ); } else { glShadeModel(GL_FLAT); }

        ////////////////////////////////
        //Object drawing routines
        //#ifdef PIXELSHADERS
        if ( layer.pixelshader) {   glEnable(GL_VERTEX_PROGRAM_NV); }
        //#endif

        //when culling is involved we draw the cull first
        if ( layer.cull )      {    glEnable(GL_CULL_FACE); 
                                    glCullFace(GL_FRONT);
                                    makeObject(layer);
                                    glCullFace(GL_BACK); }

        //we make the key first
        if ( layer.drawtheKey ) {   glBlendFunc(layer.blendmode1,layer.blendmode2);
                                    makeKey(layer);
                                    glBlendFunc(GL_ONE, GL_ONE); }

        //draw the main object here
        if ( layer.layerStatus ) {  //debug("making obj"); 
                                    makeObject(layer); }

        //for the pixelshader
        //#ifdef PIXELSHADERS
        if ( layer.pixelshader) {   glDisable(GL_VERTEX_PROGRAM_NV); }
        //#endif

        // clean up environment
        //if ( layer.keyStatus ) { glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); }
        if ( layer.blend )     {    glDisable(GL_BLEND);           }
        if ( layer.lighting )  {    glDisable(GL_LIGHTING);        }
        if ( layer.cull)       {    glDisable(GL_CULL_FACE);       }

        if ( layer.reflect)    {    glDisable(GL_TEXTURE_GEN_S);   glDisable(GL_TEXTURE_GEN_T);}

        //if object is selected we draw the selection profile here
        if ( layer.selected )  {    makeLayerOutline(layer);       }

        if ( layer.depth )     {    glDisable(GL_DEPTH_TEST);      }

        if ( layer.layerfog ) {     glDisable(GL_FOG);             }                     // Enables GL_FOG

        glPopMatrix();

    }
}

void JahRender::drawNodes(JahLayer & layer) {
// we need ways to speed this up as its the core routine

    if ( layer.layerStatus )  
    {

        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();

        layer.makeKeyFrames();

        glPopMatrix();

    }

}

