/*******************************************************************************
**
** The source file for the Jahshaka RenderObject file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include <jahrender.h>

////////////////////////////////////////////////////
// this routine draws layers and objects
// it calls the necessary object routine which then draws itself

void JahRender::makeObject( JahLayer & layer ) 
{

    //first we select the texture map for the layer this should be in the loops so its not called 2 much
    //only also if a asset is present...

    if (layer.layerAssetStatus || layer.keyAssetStatus)
    {
    //	debug("layer has asset entering");
        layer.setLayerTexture(); 
    }

    switch ( layer.layertype ) {

    case LAYER     :  {  if (layer.drawtheLayer) {

                            switch ( layer.objtype ) {

                                //background layers
                                case BACKDROP     :     {    layer.makeBackground();          break; } // no trim
                                case MAINBACK     :     {    layer.makeBlackBackground();   
                                                            //we want this to be here not renderStatus in the layer
                                                            //if (!GLWorld::renderStatus) layer.makeLayerOutline();
                                                            //layer.makeLayerOutline();
                                                            break; } //w red trim

                                case JAHLAYER     :     {    layer.makeLayer();               break; }
                                case MESHLAYER    :     {    layer.makeMeshLayer();           break; }
                                case CUBE         :     {	 layer.makeCube();				  break; }
                                case SPHERE       :     {	 layer.makeSphere();              break; }
                                case CYLINDER     :     {    layer.makeCylinder();            break; }
                                case BEZIERS      :     {    layer.makeBezier();              break; }
                                case TITLEGRID    :     {    layer.makeTitleGrid();           break; }
                                case JAHFOG       :     {    layer.makeFog();                 break; }
                                default           :     {                                     break; }
                                    }

                            break; }
                    }

    case PARTICLE  :    {  layer.makeParticle();            break; }

    case LIGHT     :    {  layer.drawLight();               break; }

    case OBJECTS   :    { 
                            switch ( layer.objtype ) {

                                case OBJOBJECTS   :     {    layer.makeObj();                 break; }
                                case MAXOBJECTS   :     {    layer.makeMaxObj();              break; }
                                default           :     {                                     break; }
                            }

                            break; 
                        }

        //frame objecttype is obsolete was replaced by titlegrid
    case FRAME     :  {  layer.makeTitleGrid();           break; }

    case TEXT      :  {  layer.drawText();                break; }

    case GRID      :  {  layer.makeGrid();                break; }
    default        :  {                                   break; }
}

    //layer should clean up after itself
    if (layer.layerAssetStatus || layer.keyAssetStatus)
    {
    //	debug("layer has asset exiting");
        //glDisable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D,0); //heard this is a faster routine than the above

    }
}

////////////////////////////////////////////////////
// this routine draws the key since it is actually
// a separate layer

void JahRender::makeKey( JahLayer & layer ) {

    layer.setKeyTexture(); //first we select the texture map for the key

    switch ( layer.objtype ) {

        case JAHLAYER    :     {    layer.makeKeyLayer();                 break; }
        default          :     {                                          break; }
    }

    //layer should clean up after itself
    glDisable(GL_TEXTURE_2D);
}


////////////////////////////////////////////////////
// this routine draws the outline or object control / GUI widgets
// these subroutines shouldnt be in the layer code...
// should be here or in the world as its a gui element
// but currently they are attached to the layers themselves

// we could query the layer and get its position and draw the elements here...

void JahRender::makeLayerOutline( JahLayer & layer) {

    switch ( layer.layertype ) { //add .layertype for more options


    case LAYER     :  {  
                        if (layer.objtype==MAINBACK || layer.objtype==BACKDROP) 
                        {
                            //makes the blue one
                            layer.makeBackdropOutline(); 
                        }
                        else 
                        {

                            switch(theToolMode)
                            {
                            case MTRANSLATE : { layer.makeLayerTranslate();   break; } //makeLayerTranslate( void);
                            case MSCALE     : { layer.makeLayerScale();   break; } //makeLayerScale( void);
                            case MROTATE    : { layer.makeLayerRotate();    break; } //makeLayerRotate( void);
                            case MOFF       : { break; }
                            }
                        }
                    }

    case TEXT      :  {    
                        if (layer.objtype==JAHTEXT) {
                        switch ( layer.text->fontstyle ) {
                            case FPOLYGON     :     {                            break; }
                            case FEXTRUDE     :     { layer.draw3dTextOutline(); break; }
                            default           :     {                            break; }
                        }
                        }
                    }

    default        :  {                               break; }

}

}
