/*******************************************************************************
**
** The source file for the Jahshaka glworld.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"



//notes from qt tutorials for when we implement shared ui objects
// Shared display list id:
//GLuint GLWorld::sharedDisplayList = 0;

// Counter keeping track of number of GLBox instances sharing 
// the display list, so that the last instance can delete it:
//int GLWorld::sharedListUsers = 0;


GLWorld::GLWorld( GLCore* jahcore, const char* name, QHBox* controller, int* globalclipnumber )
{  

    //a pointer to the new shared opengl core
    core = jahcore;

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("Initializing Module",name);

    //initalize the global vars from the prefs
    JahPrefs& jprefs = JahPrefs::getInstance();

    JahBasePath  = jprefs.getBasePath();
    JahMediaPath = jprefs.getMediaPath();
    JahRenderer  = jprefs.getRenderer();

	//start up using default resolution
	int jahres = jprefs.getJahResolution();
	hasResMenu = false;

	JahResolutionValue = jahres;

	projectData thedata;
	X_RESOLUTION = thedata.getXres(jahres);
    Y_RESOLUTION = thedata.getYres(jahres);

    videohead    = jprefs.getVideohead();

    HWFOG        = jprefs.getHwFog();
    HWALIASING   = jprefs.getHwAliasing();

    //initialize all global variables
    numberLayers = 0;
    minFrame=1; 
	maxFrame=100;
    gridval = 0;

    renderpath = JahMediaPath+"media/renders/";

    //need to check this here as we dont have a JahScenesPAth pref yet
    QDir d( JahMediaPath+"scenes/" );                        // "./example"
    
    if ( d.exists() )
    {
        scenepath  = JahMediaPath+"scenes/";
    }
    else
    {
        scenepath = JahBasePath+"scenes/";
    }


    mmode = MOFF;
    forceplay = false;
    ModuleName  = name;
    updateEffect = false;
    useBuffering = false;

    //for hardware anti aliasing support
    hwaliasing = false;

    //set the module clipnumber to shared global
    clipnumber = globalclipnumber;

    //the main camrea object
    camera      = new JahLayer;

    //note that these dont have  to be layers
    //and should be display lists
    thegrid     = new JahLayer;
    titlesafe   = new JahLayer;
    textsafe    = new JahLayer;

    //the renderspace
    renderSpace = new JahRender;

	//the active layers list
    activeLayers = new ActiveLayerList;
    multiSelect = FALSE;

	//for ui options
	numUiOptions = 0;

    //builds the controller
    buildController(controller);

    //initalize world objects
    initializeWorld();

    //set default render quality to uncompressed BMP
    renderquality = 2;

    //used for slideshows need to move this out
    //renderrepeatlength = 1;

    //fog settings for the world view
    hfog = 0.0; 
	vfog = 0.0;

	//for the ui
	hasLabels = false;
    hasSliders = false;
	hasButtons = false;

	//for various interface widgets
    haskeyframer = false;

    /////////////////////////////////////////////////////////////////////
    //need to track and monitor this as well as control it for playback
    //controls the fps for scene playback
    redrawWait=30;
    timer = new QTimer( this );
    //connect( timer, SIGNAL(timeout()), SLOT(paintGL()) );
    connect( timer, SIGNAL(timeout()), core, SLOT(update()) );
    timer->start( redrawWait, TRUE );

    ///////////////////////////////////////////////
    // the linked list is here
    layerlist = new QPtrList<NewLayerList>;
    layerlist->setAutoDelete( TRUE );     // delete items when they are removed

    //we should be able to sort the layerlist now
    //but this is here for backwards code compatability until we can
    newworldorder = new QPtrList<worldSort>;
    newworldorder->setAutoDelete( TRUE );     // delete items when they are removed

    jtrace.debug("Initialized Module",name);

};


GLWorld::~GLWorld(void) 
{
    //autodelete isnt working and needs to be cleaned up this is a BAD hack
    //see void QPtrCollection::setAutoDelete ( bool enable )
    //void QPtrCollection::deleteItem ( Item d ) [pure virtual protected]
    //file:///usr/share/doc/qt-devel-3.1.1/html/qptrcollection.html#deleteItem
    for ( uint i = 0; i <= layerlist->count(); ++i )
    {
        if ( layerlist->at(i) ) delete layerlist->at(i)->thelayer;   
        //if ( effects->at(i) ) delete effects->at(i)->fxlayer;   
    }

    layerlist->clear();
    delete layerlist;

    newworldorder->clear();
    delete newworldorder;

    delete thegrid;
    delete titlesafe;
    delete textsafe;
    delete camera;
};


void GLWorld::setResolution(int val)
{

	///////////////////////////////////
	// projectData object
	projectData thedata;

	///////////////////////////////////
	// error checking
	int numSettings;

	numSettings = thedata.getNumformats();
	if (val>numSettings)
	{
		val = thedata.getDefaultformat();
	}

	////////////////////////////////////
	//update system and ui
	JahResolutionValue = val;

	X_RESOLUTION = thedata.getXres(JahResolutionValue);
    Y_RESOLUTION = thedata.getYres(JahResolutionValue);


	//a hack for the paint module here since it doesnt use a layer yet
	if (ModuleName=="Paint")
	{
		//if (core->glmode==1)
		core->setFixedSize( QSize( X_RESOLUTION, Y_RESOLUTION ) );
	}
	else
	{
		layerlist->at(0)->thelayer->updateResolution(JahResolutionValue);
		titlesafe->updateResolution(JahResolutionValue);
	}

	updateGL();


}

void GLWorld::updateResMenu(int val)
{
	if(hasResMenu)
	{
		JahresCombo->setCurrentItem(val);
	}
}

void GLWorld::changeZoom(int val)
{
    //val is -10 to 10 we need to compute this
    val = val+10;           //this maps it to 0 to 20
    GLfloat value = val*10; //now its from 0 to 200

    if (value==0) value= 1; //trap divide by zero error

    GLfloat theval = value/100;

    core->makeCurrent();
    core->setZoom(theval);

    updateGL();

}

void GLWorld::resetZoom(void)
{
    makeCurrent();
    core->setZoom(1);

    updateGL();
}



//////////////////////////////////////////////
// Slider related

//not really slider but the main controller slider
void GLWorld::updateSlider(int value)
{

    controllerslider->setValue(value);
    currentFrame = value;
}

//main controller slider selected
void GLWorld::sliderselected()
{ 
    animsliderselected = TRUE;
}

void GLWorld::sliderreleased()
{
    animsliderselected = FALSE;
    currentFrame = controllerslider->value();

    //for fx caching buffers
    forceupdatefxbuffer = true;

    updateanimframe(currentFrame);

    updateSliderValues();

    //if videohead do a softrender here
    if (videohead)
        SoftRender();
}

void GLWorld::sliderValueChanged(int)
{
//and not rendering
    if (!animsliderselected && !animation && !renderStatus)
    {
        //debug("world slider value changed");
        int theframe;
        theframe = controllerslider->value();
        updateanimframe(theframe);
        
    }
}


////////////////////////////////////////////////
// Player related

void GLWorld::setForcePlay(bool forceit)  
{ 
    forceplay = forceit; 
    forcedplay->setChecked(forceplay);
}

void GLWorld::toggleForcePlay()  
{ 
    forceplay = !forceplay;
    setForcePlay(forceplay); //make forcedplay call its not a endless loop
}


void GLWorld::firstframeanimation()  
{
	currentFrame=1;
	updateSlider(currentFrame);
	updateanimframe(currentFrame);
	updateSliders();

}

void GLWorld::previousframeanimation()  
{

	if ( animation ) 
	{ 
		toggleAnimation(FALSE); 
	}

	currentFrame-=1;
	updateSlider(currentFrame);
	updateanimframe(currentFrame);
	updateSliders();

}

void GLWorld::nextframeanimation()  
{

	if ( animation ) 
	{ 
		toggleAnimation(FALSE); 
	}

	currentFrame+=1;
	updateSlider(currentFrame);
	updateanimframe(currentFrame);
	updateSliders();

}

void GLWorld::lastframeanimation()
{

	if ( animation ) 
	{ 
		toggleAnimation(FALSE); 
	}

	currentFrame=maxFrame;
	updateSlider(currentFrame);
	updateanimframe(currentFrame);
	updateSliders();

}




////////////////////////////////////////
// Font control routines

void GLWorld::setText(const QString& wassup) 
{ 
    layerlist->at(activeLayer)->thelayer->text->text = wassup;
    layerlist->at(activeLayer)->thelayer->makeText(); 
    updateGL();  
}  

void GLWorld::setFontColor() 
{ 
    setColor();
    layerlist->at(activeLayer)->thelayer->makeText(); 
    updateGL(); 
} 

void GLWorld::loadFont() 
{
    //need to use a loader here... give the user choice
    QString format = "ttf";

    QString filename = QFileDialog::getOpenFileName( JahBasePath+"fonts",  QString( "*.%1" ).arg( format.lower() ), this );
    if ( !filename.isEmpty() ) 
    {
        layerlist->at(activeLayer)->thelayer->text->font = filename;
        layerlist->at(activeLayer)->thelayer->loadFontText();
        layerlist->at(activeLayer)->thelayer->makeText();
    }
}

/////////////////////////////////////////
// 3D model control here

void GLWorld::changeParticleColors() 
{

	layerlist->at(activeLayer)->thelayer->ParticleColors = !layerlist->at(activeLayer)->thelayer->ParticleColors;

}

//////////////////////////////////////////
// Clip and Key control

void GLWorld::setKeyStatus()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::","called setKeyStatus()");

    if (activeLayer>0) 
    {
        layerlist->at(activeLayer)->thelayer->drawtheKey =  !layerlist->at(activeLayer)->thelayer->drawtheKey;
    }

    if (layerlist->at(activeLayer)->thelayer->drawtheKey) 
    { 
        layerlist->at(activeLayer)->thelayer->depth = false;
    }

    updateGL();
}

void GLWorld::setClipStatus()         
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","called setClipStatus()");

    if (activeLayer>0)
    {
        layerlist->at(activeLayer)->thelayer->drawtheLayer =  !layerlist->at(activeLayer)->thelayer->drawtheLayer;
    }

    //if (layerlist->at(activeLayer)->thelayer->drawtheKey) layerlist->at(activeLayer)->thelayer->depth = false;

    updateGL();
}

void GLWorld::InvertKeyData()  
{ 
    layerlist->at(activeLayer)->thelayer->changeKeyStatus(); 
    updateGL();
}   


///////////////////////////////////////////////////////////
// why not just pass videodata to the layer?
// would be much cleaner

void GLWorld::addClip(videoData clip) 
{

	layerlist->at(activeLayer)->thelayer->assignAsset(clip, IMAGE);

	layerlist->at(activeLayer)->thelayer->LoadLayerAsset(IMAGE);
	layerlist->at(activeLayer)->thelayer->makeTexture();

	updateGL();
	updateSliders();

}

void GLWorld::addClipKey(videoData clip)
{
	//  layerlist->at(activeLayer)->thelayer->keyasset = clip;
	layerlist->at(activeLayer)->thelayer->assignAsset(clip, KEY);

	layerlist->at(activeLayer)->thelayer->LoadLayerAsset(KEY);
	layerlist->at(activeLayer)->thelayer->makeTexture();

	updateGL();
	updateSliders();

}


/////////////////////////////////////////////////
// Reset and clear layers

void GLWorld::ResetLayer() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","called ResetLayer()");

    if (activeLayer==0) { camera->reset(); } else { layerlist->at(activeLayer)->thelayer->reset(); }
    animation = FALSE;
    updateGL();

}

void GLWorld::ResetAll() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","called ResetAll()");

    camera->reset();

    for ( uint i = 0; i <= layerlist->count(); ++i )
        if ( layerlist->at(i) ) 
        {
            layerlist->at(i)->thelayer->reset(); 
        }   

    animation = FALSE;
    updateGL();
}

void GLWorld::ClearAll() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","called ClearAll()");

    //first clear the modules listview
    lv->clear();

    //then clear the modules layers out

    //see void QPtrCollection::setAutoDelete ( bool enable )
    //autodelete isnt working and needs to be cleaned up
    for ( uint i = 0; i <= layerlist->count(); ++i )
        if ( layerlist->at(i) ) delete layerlist->at(i)->thelayer;

    layerlist->clear();
    delete layerlist;

    //the world order corresponds to the layers and must be cleard as well
    //for ( uint ii = 0; ii <= newworldorder->count(); ++ii )
    //    if ( newworldorder->at(ii) ) delete newworldorder->at(ii);            

    newworldorder->clear();
    delete newworldorder;

    delete camera;     
    delete titlesafe;

    if (ModuleName=="Effects")
    {
        for ( uint i = 0; i <= effects->count(); ++i )
            if ( effects->at(i) ) delete effects->at(i)->fxlayer;

        effects->clear();
        delete effects;

        initialized = false;
    }


    //recreate the variables
    layerlist = new QPtrList<NewLayerList>;
    layerlist->setAutoDelete( TRUE );     // delete items when they are removed

    newworldorder = new QPtrList<worldSort>;
    newworldorder->setAutoDelete( TRUE );     // delete items when they are removed  

    camera = new JahLayer;
    titlesafe = new JahLayer;

    numberLayers = 0;

    //initialize them now
    initializeWorld();
    initializeObjects();

	setResolution(JahResolutionValue);

}

void GLWorld::loadClearAll()
{
     //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","called LoadClearAll()");

    //first clear the modules listview
	#ifndef JAHPLAYER
    lv->clear();
	#endif
    //then clear the modules layers out

    //see void QPtrCollection::setAutoDelete ( bool enable )
    //autodelete isnt working and needs to be cleaned up
    for ( uint i = 0; i <= layerlist->count(); ++i )
        if ( layerlist->at(i) ) delete layerlist->at(i)->thelayer;            

    layerlist->clear();
    delete layerlist;

    //the world order corresponds to the layers and must be cleard as well
    //for ( uint ii = 0; ii <= newworldorder->count(); ++ii )
    //    if ( newworldorder->at(ii) ) delete newworldorder->at(ii);            

    newworldorder->clear();
    delete newworldorder;

    delete camera;
    delete titlesafe;

	#ifndef JAHPLAYER
    if (ModuleName=="Effects")
    {
        for ( uint i = 0; i <= effects->count(); ++i )
            if ( effects->at(i) ) delete effects->at(i)->fxlayer;

        effects->clear();
        delete effects;

        //add the defaults back
        effects = new QPtrList<FxLayerList>;
        effects->setAutoDelete( TRUE );     // delete items when they are removed

        //effects start at 2 since they match up with layers 0 and 1 that are used below...
        effects->append( new FxLayerList(0) );
        effects->at(0)->fxlayer->objtype = NOFX;

        effects->append( new FxLayerList(1) );
        effects->at(1)->fxlayer->objtype = NOFX;

    }
	#endif

    //recreate the variables
    layerlist = new QPtrList<NewLayerList>;
    layerlist->setAutoDelete( TRUE );     // delete items when they are removed

    newworldorder = new QPtrList<worldSort>;
    newworldorder->setAutoDelete( TRUE );     // delete items when they are removed

    camera = new JahLayer;
    titlesafe = new JahLayer;

    numberLayers = 0;

    //initialize them now
    initializeWorld();
    //initializeObjects();

    //initializeWorld();

}


void GLWorld::connectKeyframer( void ) 
{
	#ifndef JAHPLAYER
	haskeyframer = true; //let the world know its there

	connect ( thekeyframer, SIGNAL(AddKeyframe(void)),    this, SLOT(addKeyframe(void)) );
	connect ( thekeyframer, SIGNAL(RemoveKeyframe(void)), this, SLOT(removeKeyframe(void)) );
	connect ( thekeyframer, SIGNAL(PrevKeyframe(void)),   this, SLOT(prevKeyframe(void)) );
	connect ( thekeyframer, SIGNAL(NextKeyframe(void)),   this, SLOT(nextKeyframe(void)) );
	#endif
}

















