/*******************************************************************************
**
** The header file for the Jahshaka glworld module 
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef GLWORLD_H
#define GLWORLD_H

#include "interfaceheaders.h"
#include "coreobject.h"
#include "glworldeffects.h"
#include "widget.h"
#include "videodata.h"
#include "projectdata.h"

#include <ctype.h>
#include <qtextstream.h>

#include "InputLCD.h"
#include "dialogs.h"

#include <qdial.h>
#include <jahrender.h>
#include <glworldheaders.h>
#include <qlistview.h>
#include <qcursor.h>

#include "jahtracer.h"
#include "jahpreferences.h"
#include "jahtranslate.h"
#include "glworldlists.h"

//the jahshaka OpenGL Core
#include "glcore.h"

//used to nullify these objects when not used in jahplayer
#ifndef JAHPLAYER
#include "keyframer.h"
#include "jahnodes.h"
#endif

#ifdef JAHPLAYER
typedef struct { int null; } JahKeyframer; 
typedef struct { int null; } JahNodes; 
#endif

//for mesh layers
/////////////////////////////////////////////////////////////
//some globals for the fx routines here
#define JAHIMAGE_SCALE                     1.00
#define JAHMESH_X_DIMENSION                72
#define JAHMESH_Y_DIMENSION                48
#define JAHMESH_X_DIMENSION_DIV_2          (JAHMESH_X_DIMENSION / 2)
#define JAHMESH_Y_DIMENSION_DIV_2          (JAHMESH_Y_DIMENSION / 2)

///////////////////////////////////////////
// main class

class GLWorld : public QWidget {
    Q_OBJECT

public:

    GLWorld( GLCore* jahcore, const char* name, QHBox* controller=0, int* globalclipnumber=0 );

    ~GLWorld();

    GLCore * core;
    
    //these used to be globals shared exten
    int X_RESOLUTION;
    int Y_RESOLUTION;
    QString JahBasePath;
    QString JahMediaPath;
    
//////////////////////////////////////////
//gl commands to be sroted out later
    void makeCurrent(void) { core->makeCurrent(); }
    void updateGL   (void) { core->updateGL();   }

    //build the modules interface
    void buildController( QHBox* controller);
public slots:
    //opengl routine
    virtual void paintGL();


public:
        
    void initializeWorld();
    virtual void initializeObjects();

    //for the linked list for dynamic object layers
    QPtrList<NewLayerList> *layerlist;
    
    //for the linked list for dynamic effects layers
    QPtrList<FxLayerList> *effects;
    
    ///////////////////////////////////////////////////////////////
    //need to see which of these is faster on the sorting
    
        //new linked list for layer sorting
        QPtrList<worldSort> *newworldorder;

        //old way used to sort the layers before drawing the scene
       // worldSort  *worldorder;
        


    //default module variables
    QString    ModuleName;
    JahLayer  *thegrid;
    JahLayer  *titlesafe;
    JahLayer  *textsafe;
    JahLayer  *camera;

    ActiveLayerList  *activeLayers;

    bool multiSelect;
    bool slidersHaveChanged;
    bool forceupdatefxbuffer;
    bool useBuffering;

    //controls the rendering of objects in a scene
    JahRender* renderSpace;

    //the timer controls screen redraw
    QTimer* timer;
    
    //set up defalt variales here
    QString renderpath;
    QString scenepath;
    
    int numberLayers;
    int gridval;
    bool polySmooth, select;

    int  *clipnumber;
    int activeLayer, currentFrame, lastLayer;
    int minFrame, maxFrame;
    bool animation, animsliderselected;
    int  lastx,lasty,redrawWait;
    
    bool renderStatus;
    QString renderext; 
    QString renderformat; 
    int renderquality;
	int JahRenderer;
	
	int JahResolutionValue;
	QLabel			*jahreslabel;
	QComboBox		*JahresCombo;
	bool			hasResMenu;

	//for ui options
	int numUiOptions;

    bool HWFOG;
    bool HWALIASING;

    bool forceplay;
    
    //for hardware anti aliasing
    bool hwaliasing;
    
    //for video head suport
    bool videohead;

    //for effects
    bool updateEffect;
    bool initialized;

	//for file loading
	bool oldJahLoadfile;

    void raiseCore();

	//initialize a mash layer
	void initializeMeshLayer(int layer);

    //main create layer function
    bool buildLayer ( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype );
    
    //used for file loading layer creation
    bool loadLayer ( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype, 
                     bool isasset, videoData asset, bool haskey, videoData keyasset);

    bool loadLayer ( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype, 
                     bool isasset, videoData asset);

    //virtual functions are meant to be overridden in calling modules
    virtual void updateVideoTexture(int, int);
    QImage  screenShot(int Frame,int xloffset,int xroffset,int ytoffset,int yboffset,
                        int xres, int yres);

    //////////////////////////////////////
    //void  setVideoHead(bool thevideohead);

    //////////////////////////////////////////////////////////////////
    //mouse actions - should these be virtual ?
    bool	mousePressed;

public slots:
    void	mousePressEvent( QMouseEvent *e);
    void	mouseReleaseEvent( QMouseEvent *e );
    void	mouseMoveEvent( QMouseEvent  *e );

public:
    void    mouseMoveCamera( int x,int y );
    void    mouseMoveLayer ( int thelayer, int x, int y );
    void    processSelection(GLuint *pSelectBuff);
    MouseMode mmode;
    
    void contextMenuEvent(QContextMenuEvent * e);

    void keyPressEvent( QKeyEvent *k );

    //interface builders
    void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                            int w, int h, QString icon );

    void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                            int w, int h, QString iconOff, QString iconOn );

    void addButton( QPushButton * button, int x, int y, int w, int h, QString text );
    void addCheckButton( QCheckBox * button, int x, int y, int w, int h, QString text );
    void addToolButton( QToolButton * button, int x, int y, int w, int h, QString icon );
    void addToolButton( QToolButton * button, int x, int y, int w, int h  );
    void addIconButton( QPushButton * button, int x, int y, int w, int h , QString icon );
    void addIconButton( QPushButton * button, int x, int y, int w, int h  );

    void addSlider( QSlider * slider, int x, int y, int w, int h, int range);
    void addRangeSlider( QSlider * slider, int x, int y, int w, int h, int rangemin, int rangemax);

    void addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range);
    void addLabel( QLabel * label, int x, int y, int w, int h, QString text, int size);

    void addLcdLabel      ( QFrame * frame,
                            InputLCD * lcd, int x, int y, int w, int h, int range,
                            QString labeltext, int size );

    void addLabelLcd      ( QFrame * frame,
                            InputLCD * lcd, int x, int y, int w, int h, int range,
                            QString labeltext, int size );

    void addSliderAndLcd( QSlider * slider, int x, int y, int w, int h, int range,
                            InputLCD * lcd );

    void addSliderAndLcd( QSlider * slider, int x, int y, int w, int h, int rangemin, int rangemax,
                            InputLCD * lcd );

    void addSliderLcdLabel( QFrame * frame,
                            QSlider  * slider, int x, int y, int w, int h, int range,
                            InputLCD * lcd,
                            QString labeltext, int size );

    void addSliderLcdLabel( QFrame * frame,
                            QSlider  * slider, int x, int y, int w, int h,
                            int rangemin, int rangemax,
                            InputLCD * lcd,
                            QString labeltext, int size );

    void addSliderLcdLabel( QFrame * frame,
                            QSlider  * slider, int x, int y, int w, int h,
                            int rangemin, int rangemax, int preset,
                            InputLCD * lcd,
                            QString labeltext, int size );

    void addDialLcdLabel  ( QFrame * frame,
                            QDial  * slider, int x, int y, int w, int h, int range,
                            InputLCD * lcd,
                            QString labeltext, int size );

    //new routines
    void addJahSliderGroup( QFrame * frame,
                            QSlider  * slider, int x, int y, int w, int h, int range,
                            InputLCD * lcd,
                            QLabel * label,QString labeltext, int size );

    ///////////////////////////////////////////////////////////////////

    void swap(worldSort &x, worldSort &y);
    void newswap(int x, int y);



    ///////////////////////////////////////////////////////////////////
    // ui variables and such
    
    QFrame* maincontroller;
    QWidgetStack* MainController;
    QWidget* page;
    QFrame* forceplayframe;
    QCheckBox* forceplaycheckbox;
    QFrame* controllerframe;
    QPushButton* controllerpreviousbutton;
    QPushButton* controllerstopbutton;
    QPushButton* controllerplaybutton;
    QPushButton* controllerffworardbutton;
    QPushButton* controllernextbutton;
    QPushButton* controllerrewindbutton;
    QSlider* controllerslider;
    QFrame* timecodeframe;
    QPushButton* keyframebutton;
    QLCDNumber* timecodedisplay;
    QFrame* imagingframe;
    QPushButton* imagingphotobutton;
    QPushButton* imagingrenderbutton;

    //protected:
    QHBoxLayout* Form1Layout;
    QHBoxLayout* maincontrollerLayout;
    QHBoxLayout* pageLayout;

	//////////////////////////////////////
	//for the new dynamic label groups
	bool hasLabels;
    QLabel      * JahModuleLabel[5];
    QString       JahModuleLabelText[5];

    bool hasSliders;
	QSlider     * JahSliders[10];
	InputLCD    * JahSlidersLCD[10];
    QLabel      * JahSlidersLabel[10];

    QString       JahSlidersLabelText[10];

    bool hasButtons;
    QCheckBox   * JahModuleOption[5];



    //////////////////////////////////////////////////////////////////////
    //controller variables
    int    Astartframe;  int  Aendframe;  int  Aanimframe; int  MaxFrames;

    QListView*     lv;

    QHBox*       MiddleFrame;		QSlider *     Slider2;
    InputLCD*    animframe;			QLabel *      aframetext;  //was QLCDNumber
    QLabel*      sframetext;		InputLCD *    lcdstartframe;
    QLabel*      eframetext;		InputLCD *    lcdendframe;

    QCheckBox    * forcedplay;

    JahToolButton  * scrubfirst, * scrubprevious,  * scrubstop, * scrubplay, * scrubnext, * scrublast;

    QPushButton  * scrubkey, * scrubrender, * scrubrenderAll, * scrubrenderSlides;

    //buttons that control the listbox
    QPushButton *  AddButton, *  DelButton, * SceneButton;
    QPushButton*  NameButton;
    FancyPopup* namepopup;

    //FancyPopup* namepopup;

    //scene control variables
    QWidgetStack* controlpanel;
    QFrame* scene_controls;
    QFrame* object_controls;

    //interface stacks
    //need to switch to an array
    QWidgetStack*  objectControlStack;
	QHBox* objectControl[10];

	//interface widgets
	bool haskeyframer;
    JahKeyframer*           thekeyframer; 
    JahNodes*               thenodeframer; 

	//for global fog
    GLfloat hfog, vfog;

public slots:
    void connectKeyframer( void ) ;

    void setResolution(int renval);
    void updateResMenu(int renval);

    // layer controls
    void setXRotation( int degrees );    void	setYRotation( int degrees );    void	setZRotation( int degrees );
    
    float setScaleValue ( int   );
    int   getScaleValue ( float );
    
    void setXScale   ( int degrees );    void	setYScale   ( int degrees );    void	setZScale   ( int degrees );
    void setXTrans   ( int degrees );    void	setYTrans   ( int degrees );    void	setZTrans   ( int degrees );
    void setAlpha    ( int degrees );    void	setZoom     ( int degrees );    void	setExtrude  ( int degrees );

    void setColor(void); 

    void setKeyframeDrawStatus(void);

    //for spaceball
    void spaceEvent  ( double x, double y, double z, double a, double b, double c );

	//for 3d objects
    void loadObjObject(void);
    void loadMaxObject(void); 

    // for video clips
    void setinFrames      ( int frame );
    void setoutFrames     ( int frame );
    void setinKeyFrames   ( int frame );
    void setoutKeyFrames  ( int frame );

    //for editing
    void noTool( void );    
    void toolTranslate( void );    
	void toolScale( void );     
	void toolRotate( void );
    void changeZoom( int val );
    void resetZoom( void );


    //changed by tony back to void
    //void buildLayerEffect(void); //replace void with effect to build?
    void buildLayerEffect(int currentlayer); //replace void with effect to build?
    
    void callLayerEffect( FxCategory, int, QImage& );
    void doLayerGaussianBlur(QImage& pix, int layer);
    void setHBlur(int hb);
    void setVBlur(int vb);
    void setHFog(int hf);
    void setVFog(int vf);
    void updateFog(void);

    void doLayerSwapRGB(QImage& theimage, int thelayer);
    void doLayerMirror(QImage& theimage, int thelayer);
    void doLayerNegative(QImage& theimage, int thelayer);

    void updateToolDisplay(void);

    void setKeyStatus(void);
    void setClipStatus(void);
    
    void  toggleForegroundStatus();
    void  toggleDepthStatus();

    void  setText(const QString& wassup);    void  setFontColor( void );    void  loadFont( void );
    void  setTextSafe( void );

    void  setGrid( void );
    void  setSmooth( void );
    void  setBlur( void );
    void  setNegative(void);
    void  setRGB(void);
    void  setMirror(void);
    void  setFog(void);

    void  InvertKeyData(void);

    ///////////////////////////////////////////////////////////////////
    //slider control
    virtual void  updateSliders();//to be overridden...

    virtual void  updateSliderValues();//to be overridden...

    virtual void  updateUiOptions();//to be overridden...
    virtual void  updateUiSettings();//to be overridden...


	void hideAllHeadings( );
    void showAllHeadings( );
    void showHeadings(int ns );
    void setHeadingValue(int heading, QString value);
    void setDefaultHeadings(void);

    void hideAllSliders( );
    void showAllSliders( );
    void showSliders(int ns );

    void setSliderValue(int slider, int value);
    void setSliderLabelValue(int slider, QString value);
    void setDefaultSliderText(void);

    void hideAllOptions( );
    void showAllOptions( );
    void showOptions(int ns );
    void hideOptions(int ns );

    void setOptionValue(int option, bool value);


    void  sliderselected();
    void  sliderreleased();
    void  updateSlider(int value);
    void  sliderValueChanged(int value);

    void  updatesliderStartframe(int);
    void  updatesliderEndframe(int);

    void toggleForcePlay(void);
    void setForcePlay(bool status);

    void firstframeanimation(void);
    void previousframeanimation(void);
    void nextframeanimation(void);
    void lastframeanimation(void);

    //for when you hit the play and stop buttons
    void  startanimation();
    void  stopanimation();
    void  toggleAnimation(bool value);

    void  updateanimframe(int Frame);
    void  updateanimation(int Frame);

	//key frame routines
    void  addKeyframe();
    void  removeKeyframe();
    void  prevKeyframe();
    void  nextKeyframe();
    void  updateKeyframeDisplay();


    bool isLayerVisible(int i);

    //used to render from module out to desktop
    QImage  Photo(videoData);
    QImage  RenderAll(videoData,int,int);
    
    void  renderGL(void);
    void  Render(void);
    void  RenderScene(void);
    QString checkandcleandir(QString destDir, int clipnum);

    void  checkRenderQuality(void);
    void  checkJahRenderer(void);
    void  setRenderQuality(int quality);
    
    //used for vga emulated video head
    void  SetVideoRender(void);
    void  SoftRender(void);
    QImage  SoftPhoto();

    //used to grap asset from desktop and add to layer
    void  addClip(videoData);
    void  addClipKey(videoData);

    void  LoadAll(QString loadfilename);      void SceneLoad(void);
    void  LoadAllData(QFile &, int laynum);

    void  LoadImageData( QFile & ,int);       void LoadImageKeyData(QFile & ,int);
    void  LoadNodes(QFile & ,int);            void LoadColor(QFile & ,int);
    void  LoadText(QFile & ,int);

    videoData  LoadAsset(QFile & ,int);
    videoData  LoadKeyAsset(QFile & ,int);

    void LoadObjectData(QFile & ,int);
    void  LoadEffect(QFile & ,int);
    void  LoadParticleData(QFile & ,int);

    void SceneSave(void);
    void PackageSave(void);

    void  SaveAll(QString savefilename,bool flag=0);
    void  SaveAllData( QFile & );
    void  SaveAllAssets(QString destination);

    void  SaveImageData( QFile & ,int);       void SaveImageKeyData(QFile & ,int);
    void  SaveNodes(QFile & ,int);            void SaveColor(QFile & ,int);
    void  SaveText(QFile & ,int);
    void  SaveAsset(QFile & ,int);
    void  SaveKeyAsset(QFile & ,int);
    void  SaveEffect(QFile & ,int);

    void SaveObjectData(QFile & ,int);
    void  SaveParticleData(QFile & ,int);

    void nameLayer(void);

    void  ResetAll(void);
    void  ResetLayer(void);
    void  ClearAll(void);
    void  loadClearAll(void);

    //used to sort the layers for rendering proper z depth
    void sortLayers(void);
    void newsortLayers(void);

    int   ParseDataInt(QString data);
    QString ParseDataString(QString data);
    QString ParseWinDataString(QString data);
    QString ParseDataFilename(QString data);
    float ParseDataFloat(QString data);

    void  setlayerName(QString);
    void  toggleLight(void);
    void  toggleMesh(void);

    void  toggleExtendHeadTail(void);
	void  toggleLoop(void);
	void  togglePingPong(void);


	void  toggleShader1(void);
    void  toggleShader2(void);
    void  toggleReflect(void);
    void  toggleSmooth(void);
    void  changeParticleColors();

    void  addSelection(bool status = TRUE, bool headtail = FALSE);
    void  changeSelection(int layer);

    void  setLayerVisible( int, bool  );  //this can be overridden if necessary

    //change layer type
    void ChangeObjectLayer(void);
    void ChangeObjectCube(void);
    void ChangeObjectCylinder(void);
    void ChangeObjectSphere(void);
    void ChangeObjectBezier(void);
    void ChangeObjectMesh(void);

    void  objectMenu(int ); // this must be overridden in the calling object

    void  delLayer(void);

    // file io parseing routines
    QString JahParse(QString message);
    QString JahParse(QString message, int value);
    QString JahParse(QString message, float value);
    QString JahParse(QString message, QString value);
    QString JahParse(QString message, ObjectCategory cat );
    QString JahParse(QString message, LayerCategory cat );
    QString JahParse(QString debugmessage, VideoCategory cat);
    QString JahParse(QString debugmessage, VideoType cat);

signals:
    void    updateDesktop( videoData );
    
    //this was added for Siggraph dualhead demo
    //since our video card couldnt do live opengl video refresh
    void    updatePlayer ( QImage );
    void    updateBrowser( QString debugmessage );

private:

    QString jahLoadfilename;
};

#endif // GLWORLD_H




















































