/*******************************************************************************
**
** The source file for the Jahshaka glworld.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"


////////////////////////////////////////////////
// Animation related

void GLWorld::updateVideoTexture(int thelayer, int theframe) 
{
    //if theres a effect on the layer then update effects as necessary
    if (layerlist->at(thelayer)->thelayer->layerEffect) updateEffect = true;

    layerlist->at(thelayer)->thelayer->updateVidTex(theframe);
}

void GLWorld::toggleAnimation(bool value) 
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    animation = value;

    if ( animation ) 
    { 
        updateGL(); 
        //jtrace.debug( "GLWorld::","toggleAnimation() starting animation");
    }
    else
    {
        //jtrace.debug( "GLWorld::","toggleAnimation() starting animation");
        timer->stop(); 
    }

}

void GLWorld::startanimation() 
{    
    if (animation)
        toggleAnimation(FALSE);
    else
        toggleAnimation(TRUE);
}

void GLWorld::stopanimation()
{    
    toggleAnimation(FALSE);

    if (renderStatus) renderStatus = false;

    updateSliderValues();

}

///////////////////////////////////////////////////////////
//this is a core routine that draws a frame in the scene

void GLWorld::updateanimframe(int Frame)
{ 
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "GLWorld::","updateanimframe() updating scene");

    for (int i=0; i<=numberLayers; i++) 
    {
        //we check for the camera here
        if (i==0)
        { 
            if(camera->layernodes->numnodes>0) { camera->updateNodes(Frame); } 
        }
        //we only compute the nodes if the layer is visible to save processing
        if ( layerlist->at(i) ) 
        {
            if (layerlist->at(i)->thelayer->layerStatus)
            {
                if (i>0)
                {
                    if(layerlist->at(i)->thelayer->layernodes->numnodes>0)
                    {
                        layerlist->at(i)->thelayer->updateNodes(Frame);
                    }
                }
                //how to clean this mess up?
                if ( (layerlist->at(i)->thelayer->asset.videotex == TRUE) && (animsliderselected==TRUE) && (layerlist->at(i)->thelayer->layerStatus) && (forceplay) )
                {
                    updateVideoTexture(i,Frame);
                }
                else if ( (layerlist->at(i)->thelayer->asset.videotex == TRUE) && (animsliderselected==FALSE) && (layerlist->at(i)->thelayer->layerStatus) )
                {
                    updateVideoTexture(i,Frame);
                }
                else if ( (layerlist->at(i)->thelayer->keyasset.videotex == TRUE) && (animsliderselected==FALSE) && (layerlist->at(i)->thelayer->layerStatus) )
                {
                    updateVideoTexture(i,Frame);
                }
            }
        }
}
    //jtrace.debug( "GLWorld::","updateanimframe() updated scene");

    updateGL();

    //jtrace.debug( "GLWorld::","updateanimframe() update gl");

}

void GLWorld::updateanimation(int Frame) 
{ 
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "GLWorld::","updateanimation() updating scene");

    for (int i=0; i<=numberLayers; i++) 
    {
        //we check the camera here
        if (i==0) 
        { 
            if  (camera->layernodes->numnodes>0) { camera->updateNodes(Frame); }            
        }

        if ( layerlist->at(i) ) 
        {
        //we only compute the nodes if the layer is visible to save processing
        if (layerlist->at(i)->thelayer->layerStatus) 
        {
            if (i>0)  
            { 
                if  (layerlist->at(i)->thelayer->layernodes->numnodes>0) 
                { 
                    layerlist->at(i)->thelayer->updateNodes(Frame); 
                }  
            }
        }

        //we update the textures here
        if ( (layerlist->at(i)->thelayer->asset.videotex == TRUE) && (animsliderselected==FALSE) && (layerlist->at(i)->thelayer->layerStatus) ) 
        {
            updateVideoTexture(i,Frame); 
        } 
        else 
        {
            if ( (layerlist->at(i)->thelayer->keyasset.videotex == TRUE) && (animsliderselected==FALSE) && (layerlist->at(i)->thelayer->layerStatus) ) 
            {
                updateVideoTexture(i,Frame); 
            }
        }
        }
    }

    //update the sliders as well for visual feedback
    updateSlider(Frame);
}


void GLWorld::addKeyframe() 
{ 
    // theres a workaround here for the camera keyframes which are in layer 0
    currentFrame = controllerslider->value();

    if (activeLayer==0) 
    { 
        camera->addNodes(currentFrame,camera->node); 
    }       
    else 
    {

        if (multiSelect)
        {
            for (int x=0; x< activeLayers->getNumLayers(); x++)
            {
                int proclayer = activeLayers->getLayers(x);

                layerlist->at(proclayer)->thelayer->addNodes(
                currentFrame,layerlist->at(proclayer)->thelayer->node);
            }
        }
        else
        {
            layerlist->at(activeLayer)->thelayer->addNodes(
            currentFrame,layerlist->at(activeLayer)->thelayer->node);
        }

    }

    updateKeyframeDisplay();
}

void GLWorld::removeKeyframe() 
{ 
    // theres a workaround here for the camera keyframes which are in layer 0
    currentFrame = controllerslider->value();

    if (activeLayer==0) 
    { 
        camera->removeNodes(currentFrame); 
    }       
    else 
    {
        if (multiSelect)
        {
            for (int x=0; x< activeLayers->getNumLayers(); x++)
            {
                int proclayer = activeLayers->getLayers(x);

                layerlist->at(proclayer)->thelayer->removeNodes(currentFrame);
            }
        }
        else
        {
            layerlist->at(activeLayer)->thelayer->removeNodes(currentFrame);
        }
    }

    updateKeyframeDisplay();
}

void GLWorld::prevKeyframe() 
{ 
    // theres a workaround here for the camera keyframes which are in layer 0
	int newFrame;

    currentFrame = controllerslider->value();

    if (activeLayer==0) 
    { 
        newFrame = camera->layernodes->getPrevFrame(currentFrame); 
    }       
    else 
    {
        newFrame = layerlist->at(activeLayer)->thelayer->layernodes->getPrevFrame(currentFrame);
    }

	if (newFrame) 
	{
		currentFrame = newFrame;
		updateanimation(currentFrame);
        updateGL(); 
		updateKeyframeDisplay();
	}
}

void GLWorld::nextKeyframe() 
{ 
    // theres a workaround here for the camera keyframes which are in layer 0
	int newFrame;

    currentFrame = controllerslider->value();

    if (activeLayer==0) 
    { 
        newFrame = camera->layernodes->getNextFrame(currentFrame); 
    }       
    else 
    {
        newFrame = layerlist->at(activeLayer)->thelayer->layernodes->getNextFrame(currentFrame);
    }

	if (newFrame) 
	{
		currentFrame = newFrame;
		updateanimation(currentFrame);
        updateGL(); 
		updateKeyframeDisplay();
	}
}


void GLWorld::updateKeyframeDisplay() 
{ 
	#ifndef JAHPLAYER
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::","updateKeyframeDisplay() updating keyframes");

    if (haskeyframer==true)
    {

        if (layerlist->at(activeLayer))
        {
            if (layerlist->at(activeLayer)->thelayer->layernodes->numnodes > 0)
            {

            thekeyframer->updateKeyframes(layerlist->at(activeLayer)->thelayer->layernodes->numnodes, 
                                          layerlist->at(activeLayer)->thelayer->layernodes->nodelist );
                                      
            }
            else
            {
                thekeyframer->updateKeyframes( );
            }
        }
    }
	#endif
}


///////////////////////////////////////////////////////////////
//routine to check if layers are visible and should be drawn

bool GLWorld::isLayerVisible(int i) 
{
    int theFrame = controllerslider->value();
    bool returnval = false;

    //is it visible?
    if (layerlist->at(i)->thelayer->layerStatus) 
    {
        //its visible
        returnval = true;

        //now check if bleeding is on...
        if (!layerlist->at(i)->thelayer->showEdges) 
        {
            //if its not check that edges are in bounds
            if (theFrame < layerlist->at(i)->thelayer->inframe ) { returnval = false; } //startframe inframe
            if (theFrame > layerlist->at(i)->thelayer->outframe   ) { returnval = false; } //endframe outframe
        }
    }

    return returnval;

}















