/*******************************************************************************
**
** The source file for the Jahshaka glworld dataio code
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "glworld.h"
#include <qfile.h>
#include <qfiledialog.h>

#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>

#include <qpainter.h>
#include <qpaintdevicemetrics.h>

///////////////////////////////////////////////////////////////
// file load routine

void GLWorld::SceneLoad()
{
    QString fn = QFileDialog::getOpenFileName( scenepath, "*.jsf", this );
    if ( !fn.isEmpty() )
    {
        LoadAll(fn);
    } 
    else 
    {
        //QMessageBox::information(0, "Loading aborted", fn);
    }
}

void GLWorld::LoadAll(QString loadfilename) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "GLWorld::","Initializing Scene LoadAll()");

    QString text;

    QFile f( loadfilename );
    if ( !f.open( IO_ReadOnly ) ) 
    {
        QMessageBox::information(0, "Could not read file  ", loadfilename);
        return;
    }

	//set up the text stream object
    QTextStream t( &f );

    ////////////////////////////////////////////////////
    //read pasth the header and parse the file
    text = t.readLine(); jtrace.debug(text);                   //line
    text = t.readLine(); jtrace.debug(text);                   //gets app version here


    ////////////////////////////////////////////////////
    //if jahversion is 1.8 not 1.9
    //then enable old file read support

    //lets see whats there...
    text = t.readLine(); jtrace.debug(text);

    if (text.startsWith("Scene File"))
        oldJahLoadfile = true;
    else
        oldJahLoadfile = false;

	//some error checking here
	if (!oldJahLoadfile)
	{
		if (!text.contains("1.9"))
		{
			//this means there was a problem close the file and let the user know
			f.close();
			QMessageBox::information(0, "Error loading Scene file","Sorry, we cant read this Scene file");
			return;
		}
	}


    ////////////////////////////////////////////////////

    if (!oldJahLoadfile)
    {
        QString jahversion = ParseDataString( text );   //module name is here
        jtrace.debug("The version is:",jahversion);

        //now read the scene file name
        text = ParseDataString( t.readLine() );   //scene name is here
    }

    ///////////////////////////////////////////////////

    QString modtext = ParseDataString( t.readLine() );   //module name is here
    jtrace.debug("The module is:",modtext);

    int frames = ParseDataInt( t.readLine() );    //read in top keyframe
    Aanimframe = frames;

    updatesliderEndframe(Aanimframe);                 //this sets maxframe for us

    //lcdendframe is defined in the module so we need to check if it was before stting it
    
	#ifndef JAHPLAYER
	if (lcdendframe)
    {
        lcdendframe->setValue(Aanimframe);
    }
	#endif

    ///////////////////////////////////////////////////

    if (!oldJahLoadfile)
    {

        QString projectDataType = ParseDataString( t.readLine() );    //read in top keyframe

        JahResolutionValue = ParseDataInt( t.readLine() );    //read in top keyframe
        //setResolution (JahResolutionValue);
    }


    //we can only load a scene that was generated by a module into the same module
    //module sharing has not yet been implemented
    if (modtext==ModuleName) 
    {
        useBuffering = false;

        //first clear the module and reset variables
        loadClearAll();

        //check how many layers are stored
        int laynum = ParseDataInt( t.readLine() );    //numberLayers is here
        jtrace.debug("The numberLayers is:",laynum);    
        numberLayers = laynum;

        //then read in the scene info and tell the user
        text = t.readLine(); JahParse(text);

        ///////////////////////////////////////////////////////////
        //this is the main routine here in glworlddatalod.cpp
        jtrace.debug(">>loading scene in now");

        //a class global so we can access this in LoadAllData vs passing it around
        jahLoadfilename = loadfilename;

        LoadAllData(f, laynum);
        jtrace.debug(">>finished loading scene in");

        //now close out the file
        f.close();

        //clear the selections 
		#ifndef JAHPLAYER
        if (ModuleName=="Animation") lv->clearSelection();
		#endif

        //update the screen and let the user know everything was ok
        //need to set start frame to 0 here first..

        updateanimframe(0);

        updateGL();

        if (ModuleName=="Effects")
        {
            useBuffering = true;
            forceupdatefxbuffer = true;
            updateGL();
            initialized = true;
        }

		//now update the resolution
        setResolution (JahResolutionValue);
		updateResMenu(JahResolutionValue);

    } 
    else 
    {
        //this means there was a problem close the file and let the user know
        f.close();
        QMessageBox::information(0, "Error loading Scene file  ", "Scene file was written for the "+modtext+" module");

    }

	#ifdef JAHPLAYER
	activeLayer=0;
	#endif
}


///////////////////////////////////////////////////////////////
// file save routine

void GLWorld::SceneSave()
{
    //QString fn = QFileDialog::getSaveFileName( JahBasePath+"scenes", "*.jsf",  this );
    QString fn = QFileDialog::getSaveFileName( scenepath, "*.jsf",  this );

    if ( !fn.isEmpty() ) 
    {

        //first see if it ends in .jsf
        if (fn.endsWith(".jsf"))
        {
            SaveAll(fn,true);
        }
        else
        {
            fn = fn + ".jsf";
            SaveAll(fn,true);
        }
    }
    else
    {
        QMessageBox::information(0, "Saving aborted", fn);
    }
}

void GLWorld::SaveAll(QString savefilename, bool status)
{
    QString text;
    QString JahVersion;

    //initalize the global vars from the prefs
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahVersion  = jprefs.getJahVersion();

    QFile f( savefilename );

    //if we cant open the file for writing let the user know
    if ( !f.open( IO_WriteOnly ) )
    {
        QMessageBox::information(0, "Could not write to file  ", savefilename);
        return;
    }

    //set up a textstream to write data to the file
    QTextStream t( &f );

    //write out the file header
    text = JahParse("---------------------------------");   text += "\n"; t << text;
    text = JahParse("Jahshaka Animation Scene File");       text += "\n"; t << text;

    text = JahParse("Jahshaka Version:",JahVersion);        text += "\n"; t << text;

    text = JahParse("Scene File Name:"+ParseDataFilename(savefilename) );      text += "\n"; t << text;
    text = JahParse("Module:",ModuleName);                  text += "\n"; t << text;
    text = JahParse("Number of frames:",maxFrame);          text += "\n"; t << text;

	projectData thedata;
	QString projectResVal = thedata.getTitle(JahResolutionValue);

    text = JahParse("Project Resolution:",projectResVal);   text += "\n"; t << text;
    text = JahParse("Resolution Code:",JahResolutionValue); text += "\n"; t << text;

    //now we must compute actual number of layers
    //less layers deleted
    int numLay=0;
    for ( uint i = 0; i <= layerlist->count(); ++i )
    {
        if ( layerlist->at(i) )
        {
            numLay++;
        }
    }
    numLay -=1; //we start at layer number 0 ?

    text = JahParse("Number Of Layers:",numLay);           text += "\n"; t << text;
    text = JahParse("---------------------------------");  text += "\n"; t << text;

    ////////////////////////////////////////////////////////////////////////////
    //this is the main routine that writes out the scene in glworlddatasave.cpp
    SaveAllData(f);

    ///////////////////////////////////////////////////////////////////////////
    //now close the file and let the user know
    f.close();

    if (status)
    {
        QMessageBox::information(0, "File saved successfully ", savefilename);
    }
}


//////////////////////////////////////////////////////////////
// routines to export a package to the desktop
// need to add the ability to 'tar' the finished folder

void GLWorld::PackageSave()
{
    QString fn = QFileDialog::getSaveFileName( JahBasePath+"scenes", "*.*",  this );

    //mkdir the folder to save in
    if (fn.endsWith(".jsf"))
    {
        fn = fn.left(fn.length()-4);
    }

    int i = fn.findRev("/");
    QString package = fn.right(fn.length()-(i+1));

    QString destinationDir = fn ;
    QString destination = fn + "/" + package + ".jsf";

    QDir d( destinationDir );

    bool everythingOk = false;

    if ( !d.exists() )
    {
        if (d.mkdir( destinationDir ))
        {
            everythingOk = true;
        }
    }

    if (everythingOk)
    {
        if ( !destination.isEmpty() )
        {
            SaveAll(destination,false);

            SaveAllAssets(destinationDir);
        }
    }
    else
    {
        QMessageBox::information(0, "Error Packaging", fn);
    }

}

void GLWorld::SaveAllAssets(QString destination)
{
    QString assetname;
    assetExchange exchange;

    for (int i=0; i<=numberLayers; i++) 
    {
        if ( layerlist->at(i) )
        {
            //now write the objects information out
            if (layerlist->at(i)->thelayer->layerAssetStatus)
            {
                //bool copyit = false;
                if (exchange.isMediaValid(layerlist->at(i)->thelayer->asset))
                {
                    exchange.getMediaFilename(layerlist->at(i)->thelayer->asset, 0, assetname);
                    exchange.ExportAsset( layerlist->at(i)->thelayer->asset, destination);
                }
            }

            if (layerlist->at(i)->thelayer->keyAssetStatus)
            {
                //bool copyit = false;
                if (exchange.isMediaValid(layerlist->at(i)->thelayer->keyasset))
                {
                    exchange.getMediaFilename(layerlist->at(i)->thelayer->keyasset, 0, assetname);
                    exchange.ExportAsset( layerlist->at(i)->thelayer->keyasset, destination);
                }
            }
        }
    }

}  // save scene files to disk



