/*******************************************************************************
**
** The source file for the Jahshaka glworld dataioutils files
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "glworld.h"

//////////////////////////////////////////////////////////////////
// Paresing utilities used in the save and load routines

QString GLWorld::JahParse(QString debugmessage)
{

    //debug(debugmessage);

    return debugmessage;

}

QString GLWorld::JahParse(QString debugmessage, int value)
{

    QString stringval; debugmessage += stringval.setNum(value);
    //debug(debugmessage);

    return debugmessage;

}

QString GLWorld::JahParse(QString debugmessage, LayerCategory cat)
{

    QString LayType;      // based on enum LayerCategory or layertype

    switch ( cat ) {

        case LAYER        :     {     LayType = "LAYER";         break; }
        case PARTICLE     :     {     LayType = "PARTICLE";      break; }
        case OBJECTS      :     {     LayType = "OBJECTS";       break; }
        case FRAME        :     {     LayType = "FRAME";         break; }
        case TEXT         :     {     LayType = "TEXT";          break; }
        case GRID         :     {     LayType = "GRID";          break; }
        case AXIS         :     {     LayType = "AXIS";          break; }
        case LIGHT        :     {     LayType = "LIGHT";         break; }

        default           :     {     LayType = "*NULL* LayerCategory in GlWorld JahParse";
                                    break; }
    }

    debugmessage += LayType;
    //debug(debugmessage);

    return debugmessage;

}

QString GLWorld::JahParse(QString debugmessage, ObjectCategory cat)
{

    QString ObjType;      // based on enum ObjectCategory or objtype

    switch ( cat ) {

        case JAHLAYER     :     {     ObjType = "JAHLAYER";      break; }
        case MESHLAYER    :     {     ObjType = "MESHLAYER";     break; }
        case BACKDROP     :     {     ObjType = "BACKDROP";      break; }
        case MAINBACK     :     {     ObjType = "MAINBACK";      break; }
        case BEZIERS      :     {     ObjType = "BEZIERS";       break; }
        case CUBE         :     {     ObjType = "CUBE";          break; }
        case SPHERE       :     {     ObjType = "SPHERE";        break; }
        case CYLINDER     :     {     ObjType = "CYLINDER";      break; }

        case JAHTEXT      :     {     ObjType = "JAHTEXT";       break; }
        case JAHPARTICLE  :     {     ObjType = "JAHPARTICLE";   break; }

        case OBJOBJECTS   :     {     ObjType = "OBJOBJECTS";    break; }
        case MAXOBJECTS   :     {     ObjType = "MAXOBJECTS";    break; }

        case JAHAXIS      :     {     ObjType = "JAHAXIS";       break; }
        case JAHLIGHT     :     {     ObjType = "JAHLIGHT";      break; }
        case TITLEGRID    :     {     ObjType = "TITLEGRID";     break; }
        case JAHFOG       :     {     ObjType = "JAHFOG";        break; }

        case EFFECT       :     {     ObjType = "EFFECT";    break; }

        default           :     {     ObjType = "*NULL* ObjectCategory in GlWorld JahParse";
                                    break; }
    }

    debugmessage += ObjType;
    //debug(debugmessage);

    return debugmessage;

}



QString GLWorld::JahParse(QString debugmessage, VideoCategory cat)
{

    QString ObjType;      // based on enum ObjectCategory or objtype

    switch ( cat ) {

        case IMAGE      :     {     ObjType = "IMAGE";      break; }
        case KEY        :     {     ObjType = "KEY";        break; }
        case CLIP       :     {     ObjType = "CLIP";       break; }
        case CLIPKEY    :     {     ObjType = "CLIPKEY";    break; }
        case AUDIO      :     {     ObjType = "AUDIO";      break; }
        case MEDIA      :     {     ObjType = "MEDIA";      break; }

        case NOCATEGORY :     {     ObjType = "NOCATEGORY"; break; }

        default         :     {     ObjType = "*NULL* VideoCategory in GlWorld JahPAres";
                                    break;
                            }
    }

    debugmessage += ObjType;
    //debug(debugmessage);

    return debugmessage;

}


QString GLWorld::JahParse(QString debugmessage, VideoType cat)
{

    QString ObjType;      // based on enum ObjectCategory or objtype

    switch ( cat ) {

        case STILL      :     {     ObjType = "STILL";      break; }
        case SEQUENCE   :     {     ObjType = "SEQUENCE";   break; }
        case MOVIE      :     {     ObjType = "MOVIE";      break; }
        case AVI        :     {     ObjType = "AVI";        break; }
        case MPEG       :     {     ObjType = "MPEG";       break; }
        case MP3        :     {     ObjType = "MP3";        break; }
        case AIFF       :     {     ObjType = "AIFF";       break; }

        case NOTYPE     :     {     ObjType = "NOTYPE";     break; }

        default         :     {     ObjType = "*NULL* VideoType in GlWorld JahParse";
                                    break;
                            }
    }

    debugmessage += ObjType;
    //debug(debugmessage);

    return debugmessage;

}


QString GLWorld::JahParse(QString debugmessage, float value)
{

    QString stringval; debugmessage += stringval.setNum(value);
    //debug(debugmessage);

    return debugmessage;

}

QString GLWorld::JahParse(QString debugmessage, QString value)
{

    debugmessage += value;
    //debug(debugmessage);

    return debugmessage;

}

//////////////////////////////////////////////////////////////////
// Conversion utilities used in the save and load routines


/* these are the conversions
short toShort  (bool*ok=0, intbase=10)const
ushorttoUShort (bool*ok=0, intbase=10)const
int   toInt    (bool*ok=0, intbase=10)const
uint  toUInt   (bool*ok=0, intbase=10)const
long  toLong   (bool*ok=0, intbase=10)const
ulong toULong  (bool*ok=0, intbase=10)const
float toFloat  (bool*ok=0)const
doubletoDouble (bool*ok=0)const
*/

int GLWorld::ParseDataInt(QString data)
{

	int index,jitlen;

	index	= data.findRev(":");   index +=1;     //find the colon and skip over it
	jitlen = data.length();    data = data.right(jitlen - index) ;
	int integer = data.toInt();
	return integer;
}

float GLWorld::ParseDataFloat(QString data)
{

	int index,jitlen;

	index	= data.findRev(":");   index +=1;     //find the colon and skip over it
	jitlen = data.length();    data = data.right(jitlen - index) ;
	float integer = data.toFloat();
	return integer;
}

QString GLWorld::ParseDataString(QString data)
{

	int index,jitlen;

	index	= data.findRev(":");   index +=1;     //find the colon and skip over it
	jitlen = data.length();    data = data.right(jitlen - index) ;
	return data;
}

//used for windows paths where the drive letter messes up our search as it has a :
QString GLWorld::ParseWinDataString(QString data)
{

	int index,jitlen;

	index	= data.find(":");   index +=1;     //find the colon and skip over it
	jitlen = data.length();    data = data.right(jitlen - index) ;
	return data;
}

QString GLWorld::ParseDataFilename(QString data)
{ //caution, on unix this may be a  \ backslash

	int index,jitlen;

	index	= data.findRev("/");   index +=1;     //find the slash and skip over it
	jitlen = data.length();    data = data.right(jitlen - index) ;
	jitlen = data.length();    data = data.left(jitlen - 4) ; //file extension is four characters including the dot .jsf
	return data;
}





