/*******************************************************************************
**
** The source file for the Jahshaka glworld dataio code
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "glworld.h"
#include "qfile.h"
#include <qfiledialog.h>

#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>

#include <qpainter.h>
#include <qpaintdevicemetrics.h>


void GLWorld::LoadAllData(QFile & f, int laynum) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    //the file text stream to load data from
    QTextStream t( &f );

    QString text, thelayername;
    bool backdropstatus = false;

    videoData loadasset;
    bool isasset;

    //need to prebuffer for the effects layers
    if (ModuleName=="Effects" || ModuleName=="Colorize")
    {
        effects->append( new FxLayerList(0) );
        effects->append( new FxLayerList(1) );
    }

    jtrace.debug("GLWorld::"," LoadAllData() starting loop");
    for (int i=0; i<=laynum; i++) 
    {
        activeLayer = i; numberLayers = i;

        text = t.readLine(); //JahParse(text);  "Info for Layer:",i

        //create the layer in the list first - name is a placeholder for now
        thelayername   = ParseDataString( t.readLine() );  

        layerlist->append( new NewLayerList(i, thelayername) );
        layerlist->at(i)->thelayer->layername   = thelayername;

        layerlist->at(i)->thelayer->blend       = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->depth       = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->cull        = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->lighting    = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->foreground  = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->smooth  = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->reflect  = ParseDataInt( t.readLine() );

        layerlist->at(i)->thelayer->layerEffect  = ParseDataInt( t.readLine() );     


        if (layerlist->at(i)->thelayer->layerEffect)
        {
            layerlist->at(i)->thelayer->negative  = ParseDataInt( t.readLine() );  //
            layerlist->at(i)->thelayer->swaprgb  = ParseDataInt( t.readLine() );
            layerlist->at(i)->thelayer->mirror  = ParseDataInt( t.readLine() );

            layerlist->at(i)->thelayer->blur  = ParseDataInt( t.readLine() );

            //we need to get rid of these values 
            //but maybe we can keep them as placeholders we can use later on
            layerlist->at(i)->thelayer->blur  = ParseDataInt( t.readLine() ); //blurx
            layerlist->at(i)->thelayer->blur  = ParseDataInt( t.readLine() ); //blury

            updateEffect = true;
        }


        layerlist->at(i)->thelayer->invertKey  = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->drawtheKey  = ParseDataInt( t.readLine() );
        layerlist->at(i)->thelayer->keyAssetStatus  = ParseDataInt( t.readLine() );

        //this is weather the layer is on or off
        bool layerstat = ParseDataInt( t.readLine() );  //get the layerstatus

        layerlist->at(i)->thelayer->layerStatus = layerstat;

        text = t.readLine();

        ////////////////////////////////////////////////////////////////
        //this is were we decode the layer type and object category data
        text = ParseDataString( t.readLine() ); 

        int LayerCode = ParseDataInt( t.readLine() );
        LayerCategory layerCat = (LayerCategory)LayerCode;

        text = ParseDataString( t.readLine() );

        int ObjCode   = ParseDataInt( t.readLine() );
        ObjectCategory objectCat = (ObjectCategory)ObjCode;

        isasset = false;

        ////////////////////////////////////////////////////////////////
        //now we read the objects information in and set it up
        switch ( LayerCode ) 
        {

        case LAYER :     {

                            if (ObjCode==BACKDROP)
                            {
                                gridval    = ParseDataInt( t.readLine() ); //gridval
                                polySmooth = ParseDataInt( t.readLine() ); //polySmooth
                                textsafe->layerStatus = ParseDataInt( t.readLine() ); //textsafe layerStatus;

                                text = t.readLine();
                                if (text.contains(":")) {   layerlist->at(i)->thelayer->layerAssetStatus=true;
                                                            isasset = true;
                                                            loadasset = LoadAsset(f,i);
                                                            loadLayer(i,thelayername, layerCat, objectCat, isasset, loadasset );
                                                        }

                                LoadNodes(f,i); 

                                //this is a effects hack for layer 0 for the effects module
                                //to be removed later on
                                if (ModuleName=="Effects" || ModuleName=="Colorize" )
                                {
                                }
                                else
                                {
                                    addSelection(layerstat);
                                }
                            }
                            else
                            {
                                ////////////////////////////////////////////////
                                //effects module has special needs
                                if (ModuleName=="Effects" || ModuleName=="Colorize" ) 
                                {

                                    if (i==0) 
                                    {
                                        jtrace.debug("its fxworld do nothing");
                                    }
                                    else if (i==1) 
                                    {
                                    jtrace.debug(">>loading main fx layer");
                                        layerlist->at(i)->thelayer->layerAssetStatus=true;
                                        isasset = true;

                                        text = t.readLine(); 
                                        loadasset = LoadAsset(f,i);

                                        text = t.readLine(); 
                                        LoadNodes(f,i);

                                        jtrace.debug(">>building main effect layer");
                                        loadLayer(i,thelayername, layerCat, objectCat, isasset, loadasset );

                                        //set renderspace imagesize here...
                                        core->imagewidth  = layerlist->at(1)->thelayer->getImageWidth();
                                        core->imageheight = layerlist->at(1)->thelayer->getImageHeight();

                                        //addSelection(layerstat);
                                    }
                                    else
                                    {
                                        layerlist->at(i)->thelayer->layerAssetStatus=true;
                                        text = t.readLine(); 
                                        LoadNodes(f,i);
                                        text = t.readLine(); 
                                        LoadEffect(f,i);  
                                        loadLayer(i,thelayername, layerCat, objectCat, false, loadasset );

                                        //addSelection(layerstat);
                                    }
                                }
                                else  //its not effects do the regular thang
                                {         
                                    bool haskey = false;
                                    videoData keyasset;

                                    text = t.readLine();
                                    if (text.contains(":")) 
                                    {   
                                        isasset = true;
                                        loadasset = LoadAsset(f,i);
                                    }

                                    //used to add a key
                                    text = t.readLine();

                                    if (text.contains(":")) 
                                    {	
                                        haskey = true;
                                        keyasset = LoadKeyAsset(f,i);                        
                                    }

                                    loadLayer(i,thelayername, layerCat, objectCat, isasset, loadasset, 
                                                                                haskey,  keyasset  );

                                    LoadNodes(f,i);
                                    addSelection(layerstat);
                                }
                            }

                            break;
                        }


        case PARTICLE : {
                            text = t.readLine();
                            if (text.contains(":")) {   isasset = true; 
                                                        layerlist->at(i)->thelayer->layerAssetStatus=true;   
                                                        loadasset = LoadAsset(f,i); 
                                                    }

                            LoadParticleData(f,i);

                            loadLayer(numberLayers,thelayername, layerCat, objectCat, isasset, loadasset);

                            LoadNodes(f,i);
                            addSelection(layerstat);

                            break;
                        }

        case OBJECTS :  {
                            text = t.readLine();
                            if (text.contains(":")) { isasset = true;
                                                    layerlist->at(i)->thelayer->layerAssetStatus=true;   
                                                    loadasset = LoadAsset(f,i); }

                            LoadObjectData(f,i);

                            loadLayer(numberLayers,thelayername, layerCat, objectCat, isasset, loadasset);

                            LoadNodes(f,i);
                            addSelection(layerstat);

                            break;
                        }

        case FRAME        :     {   break; }

        case TEXT         :     {

                                    text = t.readLine();
                                    if (text.contains(":")) { isasset = true;
                                                            layerlist->at(i)->thelayer->layerAssetStatus=true;    
                                                            loadasset = LoadAsset(f,i); 
                                    }

                                    layerlist->at(i)->thelayer->addTextObj();

                                    LoadColor(f,i);
                                    LoadText(f,i);


                                    loadLayer(numberLayers,thelayername, layerCat, objectCat, isasset, loadasset);

                                    LoadNodes(f,i);
                                    addSelection(layerstat);


                                    break;
                                }

        case GRID         :     {   break; }

        default           :     {   break; }

    }


        //need to reset this as adding the lyaer overrides it
        //if (layerstat==1) setLayerVisible(i,TRUE); else setLayerVisible(i,FALSE);
        if (ModuleName!="Effects")   
        {
            setLayerVisible(i,layerstat); 
            layerlist->at(i)->thelayer->selected = false;
        }
        else
        { //for effects module
            if (i==1) 
            {
                setLayerVisible(i,layerstat); 
                layerlist->at(i)->thelayer->selected = false;
                layerlist->at(1)->thelayer->objtype = MESHLAYER;
                addSelection(layerstat);
            }
            else if (i>1)
            {
                setLayerVisible(i,layerstat);
                layerlist->at(1)->thelayer->objtype = MESHLAYER;
                addSelection(layerstat);
                //thenodeframer->addNodes(numberLayers,title);
            }
        }
        jtrace.debug("finished reading layer");
        text = t.readLine(); 
    } 


}


///////////////////////////////////////////////////////////////
// file save routine
//needs to be working!

void GLWorld::LoadObjectData(QFile & file, int i)
{
        QString text;  QTextStream t( &file );

        text = t.readLine(); JahParse(text);
        text = ParseDataString( t.readLine() ); layerlist->at(i)->thelayer->objfilename = text;
        text = ParseDataString( t.readLine() ); layerlist->at(i)->thelayer->objextension = text;

}  // save scene files to disk

void GLWorld::LoadEffect(QFile & file, int i)
{
    QString text;
    QTextStream t( &file );
    int datavar;

    effects->append( new FxLayerList(i) );

    datavar   = ParseDataInt   ( t.readLine() ); 
    effects->at(i)->fxlayer->objtype = FxCategory(datavar);

    if (datavar==0)
    {
        int plnum   = ParseDataInt   ( t.readLine() ); effects->at(i)->fxlayer->pluginnumber = plnum;
    }

    datavar = ParseDataInt   ( t.readLine() ); effects->at(i)->fxlayer->effectnumber = datavar;
    text	= ParseDataString( t.readLine() ); layerlist->at(i)->thelayer->layername = text;

    datavar = ParseDataInt   ( t.readLine() ); effects->at(i)->fxlayer->selected = datavar;
    datavar = ParseDataInt   ( t.readLine() ); effects->at(i)->fxlayer->layerStatus = datavar;
    datavar = ParseDataInt   ( t.readLine() ); effects->at(i)->fxlayer->isOn = datavar;

    datavar	= ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->option1 = datavar;
    datavar	= ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->option2 = datavar;
    datavar	= ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->option3 = datavar;
    datavar	= ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->option4 = datavar;

    //currently a global on layer 1 for effects but soon to be a per effect option
    datavar	= ParseDataInt( t.readLine() ); layerlist->at(1)->thelayer->usedagpu = datavar;


}  // save scene files to disk

void GLWorld::LoadParticleData(QFile & file, int i)
{

    QString text;  QTextStream t( &file );

    text = t.readLine(); JahParse(text);  //JahParse("Info for Layer:",i);

    int ParticleStyle   = ParseDataInt( t.readLine() ); JahParse("Stored Particle Style is:",ParticleStyle);
    ParticleCategory particleStyle = (ParticleCategory)ParticleStyle;
    layerlist->at(i)->thelayer->ParticleStyle = particleStyle;

    int ParticleDraw   = ParseDataInt( t.readLine() ); JahParse("Stored Particle Draw is:",ParticleDraw);
    ParticleDrawstyle particleDraw = (ParticleDrawstyle)ParticleDraw;
    layerlist->at(i)->thelayer->ParticleDraw = particleDraw;

    int ParticleColor   = ParseDataInt( t.readLine() ); JahParse("Stored Particle color is:",ParticleColor);
    bool particleColor = (bool)ParticleColor;
    layerlist->at(i)->thelayer->ParticleColors = particleColor;


}  // save scene files to disk

void GLWorld::LoadImageData( QFile & file, int i) {

}

//add video support here mon
void GLWorld::LoadImageKeyData(QFile & file, int i) 
{

    QString text;  QTextStream t( &file ); bool status;

    status = ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->drawtheKey = status;
    status = ParseDataInt( t.readLine() ); layerlist->at(i)->thelayer->invertKey = status;

}


//i think this is fixed now as we cleaned up the keyframe engine so we no
//longer need 2 routines... though maybe we do as the 2nd is for when no keyframe
//has been set but the object has been positioned

void GLWorld::LoadNodes(QFile & file,int i) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::","LoadNodes() Loading node data");

    QString text;  QTextStream t( &file );

    int numnodes = ParseDataInt( t.readLine() );
    jtrace.debug("Number of keyframes",numnodes);

    if (numnodes>0)
    {

        for (int x=0; x<numnodes; x++)
        {
            int keyframe = ParseDataInt( t.readLine() );
            jtrace.debug("Adding Current keyframe",keyframe);

            if (i>0)
            {
                layerlist->at(i)->thelayer->node->sx = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->sy = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->sz = ParseDataFloat( t.readLine() );

                layerlist->at(i)->thelayer->node->tx = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->ty = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->tz = ParseDataFloat( t.readLine() );

                layerlist->at(i)->thelayer->node->rx = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->ry = ParseDataFloat( t.readLine() );
                layerlist->at(i)->thelayer->node->rz = ParseDataFloat( t.readLine() );

                layerlist->at(i)->thelayer->node->Alpha = ParseDataFloat( t.readLine() );

                //need to add this for the blurs...
                //if (!oldJahLoadfile)
                //{
                //layerlist->at(i)->thelayer->node->bx = ParseDataFloat( t.readLine() );
                //layerlist->at(i)->thelayer->node->by = ParseDataFloat( t.readLine() );
                //}

                //add the node
                layerlist->at(i)->thelayer->addNodes(keyframe,layerlist->at(i)->thelayer->node);
            }
            else
            //its the camera
            {
                camera->node->sx = ParseDataFloat( t.readLine() );
                camera->node->sy = ParseDataFloat( t.readLine() );
                camera->node->sz = ParseDataFloat( t.readLine() );

                camera->node->tx = ParseDataFloat( t.readLine() );
                camera->node->ty = ParseDataFloat( t.readLine() );
                camera->node->tz = ParseDataFloat( t.readLine() );

                camera->node->rx = ParseDataFloat( t.readLine() );
                camera->node->ry = ParseDataFloat( t.readLine() );
                camera->node->rz = ParseDataFloat( t.readLine() );

                camera->node->Alpha = ParseDataFloat( t.readLine() );

                //add the node
                camera->addNodes(keyframe,camera->node);
            }

        }
    }
    else
    {

        int keyframe = ParseDataInt( t.readLine() );
        jtrace.debug("Current keyframe",keyframe);

        if (i>0) 
        {
            layerlist->at(i)->thelayer->node->sx = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->sy = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->sz = ParseDataFloat( t.readLine() );

            layerlist->at(i)->thelayer->node->tx = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->ty = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->tz = ParseDataFloat( t.readLine() );

            layerlist->at(i)->thelayer->node->rx = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->ry = ParseDataFloat( t.readLine() );
            layerlist->at(i)->thelayer->node->rz = ParseDataFloat( t.readLine() );

            layerlist->at(i)->thelayer->node->Alpha = ParseDataFloat( t.readLine() );

            //need to add this for the blurs...
            //if (!oldJahLoadfile)
            //{
            //layerlist->at(i)->thelayer->node->bx = ParseDataFloat( t.readLine() );
            //layerlist->at(i)->thelayer->node->by = ParseDataFloat( t.readLine() );
            //}

        }
        else
        {
            camera->node->sx = ParseDataFloat( t.readLine() );
            camera->node->sy = ParseDataFloat( t.readLine() );
            camera->node->sz = ParseDataFloat( t.readLine() );

            camera->node->tx = ParseDataFloat( t.readLine() );
            camera->node->ty = ParseDataFloat( t.readLine() );
            camera->node->tz = ParseDataFloat( t.readLine() );

            camera->node->rx = ParseDataFloat( t.readLine() );
            camera->node->ry = ParseDataFloat( t.readLine() );
            camera->node->rz = ParseDataFloat( t.readLine() );

            camera->node->Alpha = ParseDataFloat( t.readLine() );

        }
    }

}  // save scene files to disk


void GLWorld::LoadColor(QFile & file,int i) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString text;  QTextStream t( &file );
    jtrace.debug("Loading color data");
        layerlist->at(i)->thelayer->Red = ParseDataFloat( t.readLine() );
        layerlist->at(i)->thelayer->Green = ParseDataFloat( t.readLine() );
        layerlist->at(i)->thelayer->Blue = ParseDataFloat( t.readLine() );

}  // save scene files to disk


void GLWorld::LoadText(QFile & file,int i) {
    QString text;  QTextStream t( &file );

    layerlist->at(i)->thelayer->text->text = ParseDataString( t.readLine() );  //JahParse("Stored lighting settings:",layerlist->at(i)->thelayer->lighting);
    layerlist->at(i)->thelayer->text->font = ParseDataString( t.readLine() );  //JahParse("Stored lighting settings:",layerlist->at(i)->thelayer->lighting);

    int FontStyle   = ParseDataInt( t.readLine() );
    FontDrawstyle fontStyle = (FontDrawstyle)FontStyle;
    layerlist->at(i)->thelayer->text->fontstyle = fontStyle;

    layerlist->at(i)->thelayer->text->extrude = ParseDataFloat( t.readLine() );

}  // save scene files to disk

videoData GLWorld::LoadAsset(QFile & file,int i) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::","Calling LoadAsset()");

    QString text;  QTextStream t( &file );
    int ObjCode;

    videoData theloadasset;

    //read past commment
    text = t.readLine(); // JahParse(text);

    //read in asset category
    text = ParseDataString( t.readLine() );       
    jtrace.debug("Setting asset category to",text);
    ObjCode   = ParseDataInt( t.readLine() );

    VideoCategory videoCat = (VideoCategory)ObjCode;
    theloadasset.theCategory = videoCat;

    //read in asset type
    text = ParseDataString( t.readLine() );       
    jtrace.debug("Setting asset type to",text);
    ObjCode   = ParseDataInt( t.readLine() );    

    VideoType videoTy = (VideoType)ObjCode;
    theloadasset.theType     = videoTy;

    //read in asset clip data
    theloadasset.clipname    = ParseDataString( t.readLine() );
    theloadasset.clipnumber  = ParseDataInt( t.readLine() );
    theloadasset.location    = ParseDataString( t.readLine() );
    theloadasset.filename    = ParseDataString( t.readLine() );
    theloadasset.extension   = ParseDataString( t.readLine() );

    theloadasset.startframe  = ParseDataInt( t.readLine() );
    theloadasset.endframe    = ParseDataInt( t.readLine() );

    theloadasset.videoext    = ParseDataString( t.readLine() );
    //theloadasset.videoextkey = ParseDataString( t.readLine() );

    theloadasset.videotex    = ParseDataInt( t.readLine() );

    //this is not asset data here but is used for video slip and shold be stored in the asset
    if (theloadasset.videotex) 
    { 
        theloadasset.clipinframe   = ParseDataInt( t.readLine() );
        theloadasset.clipoutframe   = ParseDataInt( t.readLine() );

        //should be done later on?
        layerlist->at(i)->thelayer->inframe  = theloadasset.clipinframe;
        layerlist->at(i)->thelayer->outframe = theloadasset.clipoutframe;
        layerlist->at(i)->thelayer->showEdges = ParseDataInt( t.readLine() );

        //this is not related to the asset itself
        //should be stored in the asset
        if (!oldJahLoadfile)
        {
            layerlist->at(i)->thelayer->loop = ParseDataInt( t.readLine() );
            layerlist->at(i)->thelayer->ping = ParseDataInt( t.readLine() );
        }
}

    theloadasset.xpos        = ParseDataInt( t.readLine() );
    theloadasset.ypos        = ParseDataInt( t.readLine() );

    ///////////////////////////////////////////////////////////////////////////////////
    //need to do some testing here mon we will start it simple and move it out later
    //so we can use it in the load key asset routine below :)

	jtrace.debug(">>>>>>>Loading asset:"+theloadasset.location+theloadasset.filename+theloadasset.extension);

        assetExchange exchange;
        if (exchange.isMediaValid(theloadasset)) 
        {
            jtrace.debug("MEDIA IS VALID");
        }
        else
        {
			//and for exported scenes

            jtrace.debug("MEDIA IS NOT VALID");

			///////////////////////////////////////////////////////////////
			//first place we should check is : jahbasepath + assetname 
			//this only works for the demo files in the base path
			//////////////////////////////////////////////////////////////

            jtrace.debug("trying to locate media");

            QString assetname;
            exchange.getMediaFilename(theloadasset, 0, assetname);
	        jtrace.debug(">> asset name is:"+assetname);

			//if it starts with ./ its relative and we need to get rid of the ./
			if (assetname.startsWith("./"))
				assetname = assetname.right( assetname.length()-2 );   

			QString relativeFile = JahBasePath + assetname;
			jtrace.debug("first we are trying:"+relativeFile);

            if (QFile::exists(relativeFile))
            {
                jtrace.debug("Found the media at",JahMediaPath);
                theloadasset.location = JahBasePath+"media/images/";
            }
			else
			{
				//////////////////////////////////////////////////////////////
				//next we look within the folder that the jsf file was in
				//used for exported scene files where its all in the same place
				debug("jahloadfilename:"+jahLoadfilename);

				int i = jahLoadfilename.findRev("/");
				QString newlocation;

				newlocation = jahLoadfilename.left(i+1);

				i = assetname.findRev("/");
				QString newname = assetname.right(assetname.length()-(i+1));

				QString thaname;
				thaname = newlocation + newname;

				debug("thaname is:"+thaname);

				if (QFile::exists(thaname))
				{
					jtrace.debug("Found media at",thaname);
					theloadasset.location = newlocation;
				}
				else
				{
					//this means its in a folder (ie sequence)
					QString foldername;
					foldername = theloadasset.location;

					int ii = -1;
					if (foldername.endsWith("/")) ii = -2;
					i = foldername.findRev("/",ii);

					QString thefoldername;
					thefoldername = foldername.right(foldername.length()-(i+1));

					thaname = newlocation + thefoldername + newname;

					jtrace.debug("trying:::",thaname);

					if (QFile::exists(thaname))
					{
						jtrace.debug("Found media at",thaname);
						theloadasset.location = newlocation + thefoldername;
					}

				}
			}
        }

    return theloadasset;

}

videoData GLWorld::LoadKeyAsset(QFile & file,int i) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::","Calling LoadKeyAsset()");

    QString text;  QTextStream t( &file );
    int ObjCode;

    videoData theloadasset;

    //read past commment
    //bet we dont save it in the key
    //text = t.readLine(); JahParse(text);

    //read in asset category
    text = ParseDataString( t.readLine() );       
    jtrace.debug("Setting asset category to",text);
    ObjCode   = ParseDataInt( t.readLine() );

    VideoCategory videoCat = (VideoCategory)ObjCode;
    theloadasset.theCategory = videoCat;

    //read in asset type
    text = ParseDataString( t.readLine() );       
    jtrace.debug("Setting asset type to",text);
    ObjCode   = ParseDataInt( t.readLine() );    

    VideoType videoTy = (VideoType)ObjCode;
    theloadasset.theType     = videoTy;


    //read in asset clip data
    theloadasset.clipname    = ParseDataString( t.readLine() );
    theloadasset.clipnumber  = ParseDataInt( t.readLine() );
    theloadasset.location    = ParseDataString( t.readLine() );
    theloadasset.filename    = ParseDataString( t.readLine() );
    theloadasset.extension   = ParseDataString( t.readLine() );

    theloadasset.startframe  = ParseDataInt( t.readLine() );
    theloadasset.endframe    = ParseDataInt( t.readLine() );

    theloadasset.videoext    = ParseDataString( t.readLine() );
    //theloadasset.videoextkey = ParseDataString( t.readLine() );

    theloadasset.videotex    = ParseDataInt( t.readLine() );

    //this is not asset data here but is used for video slip and shold be stored in the asset
    if (theloadasset.videotex) 
    { 
        theloadasset.clipinframe   = ParseDataInt( t.readLine() );
        theloadasset.clipoutframe   = ParseDataInt( t.readLine() );
        //layerlist->at(i)->thelayer->currentframe = 1;
}

    theloadasset.xpos        = ParseDataInt( t.readLine() );
    theloadasset.ypos        = ParseDataInt( t.readLine() );

    return theloadasset;

}
