/*******************************************************************************
**
** The source code for the Jahshaka glworldeffect.cpp file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"
#include "blurlib.h"

//used to build cumulitive effects on a layer in our case its the blur
//need switch to pointers here on images for speeeeed!
void GLWorld::buildLayerEffect(int currentlayer)
{

    if (numberLayers>0) 
    {
    //updateEffect=true; //need to debug this now as its needed for videotextures

        //flag used so as to not have to recalculate effects unless necessary
        if (updateEffect==true)
        {            
            //grab the source image into buffer so we can restore it later
            QImage sourceimage = layerlist->at(currentlayer)->thelayer->textureData.objectImage.copy();

            //grab the source image into buffer to apply effects to
            QImage renderimage = layerlist->at(currentlayer)->thelayer->textureData.objectImage.copy();

            //we reset the mesh every time if itrs in use - used for when we turn of geometry effects
            //need to fix this later on we'll start with effects being mesh-only
            if (layerlist->at(currentlayer)->thelayer->objtype == MESHLAYER) 
                layerlist->at(currentlayer)->thelayer->initializeMeshLayer();

            //now we build the effects stacked on top of each other using renderimage
            if (layerlist->at(currentlayer)->thelayer->layerStatus )
            {
                //hardcoded here to do blurs
                if (layerlist->at(currentlayer)->thelayer->negative)    
                    callLayerEffect(NEGATIVE,currentlayer,renderimage); //was renderimage not sourceimage

                if (layerlist->at(currentlayer)->thelayer->swaprgb)     
                    callLayerEffect(SWAPRGB,currentlayer,renderimage);

                if (layerlist->at(currentlayer)->thelayer->mirror)      
                    callLayerEffect(MIRROR,currentlayer,renderimage);

                if (layerlist->at(currentlayer)->thelayer->blur)        
                    callLayerEffect(GAUSSIANBLUR,currentlayer,renderimage);

            }

            //draw the updated texture using the effects generated image
            //there is a memory leak here!!!!!
            layerlist->at(currentlayer)->thelayer->rebuildTextureData(renderimage);

            //restore the source image
            layerlist->at(currentlayer)->thelayer->textureData.objectImage = sourceimage;

            updateEffect = false;
        }
    }

}


void GLWorld::callLayerEffect( FxCategory thefx,  int i, QImage& newimage ) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("Building the Effect for layer ",i);

    switch ( thefx ) 
    {
    //qt image processing
    case NEGATIVE           : {  doLayerNegative(newimage,i);      break; }
    case SWAPRGB            : {  doLayerSwapRGB(newimage,i);       break; }
    case MIRROR             : {  doLayerMirror(newimage,i);        break; }
    case GAUSSIANBLUR       : {  doLayerGaussianBlur(newimage,i);  break; }

    default                 : {  break; }

    }
}

void GLWorld::doLayerSwapRGB(QImage& theimage, int thelayer)	{

    theimage = theimage.swapRGB();

}

//need to add controls here
void GLWorld::doLayerMirror(QImage& theimage, int thelayer)	{

    bool VERT = TRUE;  
    bool HORIZ = TRUE;

    theimage = theimage.mirror(VERT,HORIZ);

}

void GLWorld::doLayerNegative(QImage& theimage, int thelayer) {

    theimage.invertPixels();

}

void GLWorld::doLayerGaussianBlur(QImage& pix, int thelayer) {
    // GimpPixelRgn src_rgn, dest_rgn;
    int                     width, height;
    int                     bytes;
    int                     has_alpha;
    uchar*                  dst; 
    uchar*                  dst_ptr;
    uchar*                  src;
    uchar*                  src_ptr;
    int*                    buf; 
    int*                    bb;
    int                     pixels;
    int                     total = 1;
    //int                     x1, y1, x2, y2;
    int                     i, row, column, b;
    int                     start, end;
    //double                  progress, max_progress;
    int*                    curve;
    int*                    sum = NULL;
    int                     val;
    int                     length;
    int                     initial_p, initial_m;
    double                  std_dev;
    double                  horz;
    double                  vert;
    double                  temp;
    Gaussian_blur_c         blur;
    QImage                  newimage; 
    uint                    pixel_data;

    newimage = pix.copy();

    // Values returned from user input sliders
    horz = (double)abs((int)layerlist->at(thelayer)->thelayer->node->bx) / 2.0;
    vert = (double)abs((int)layerlist->at(thelayer)->thelayer->node->by) / 2.0;

    if (horz <= 0.0 && vert <= 0.0)
    {
        return;
    }

    width = (int)layerlist->at(thelayer)->thelayer->textureData.width;
    height = (int)layerlist->at(thelayer)->thelayer->textureData.height;

    if (width < 1 || height < 1) { return; }

    bytes = 4;  // Always with QT (including alpha)

    has_alpha = TRUE;

    buf = new int [MAX(width, height) * 2];

    src = new uchar [MAX(width, height) * bytes];
    dst = new uchar [MAX(width, height) * bytes];

    /*  First the vertical pass  */
    if (vert > 0.0)
    {
        vert = fabs(vert) + 1.0;
        std_dev = sqrt (-(vert * vert) / (2 * log(1.0 / 255.0)));

        curve = blur.makeCurve(std_dev, &length);
        //sum = g_new (int, 2 * length + 1);
        //sum = (int*)calloc((2 * length) + 1, sizeof(int));
        sum = new int [(2 * length) + 1];

        sum[0] = 0;

        for (i = 1; i <= length*2; i++)
        {
            sum[i] = curve[i - length - 1] + sum[i-1];
        }

        sum += length;

        total = sum[length] - sum[-length];

        for (column = 0; column < width; column++)
        {

            for (row = 0; row < height; row++)
            {
                pixel_data = (uint)newimage.pixel(column, row);
                src[row * bytes + 0] = (pixel_data >> 16) & 0xff;  // Red
                src[row * bytes + 1] = (pixel_data >>  8) & 0xff;  // Green
                src[row * bytes + 2] = (pixel_data >>  0) & 0xff;  // Blue
                src[row * bytes + 3] = (pixel_data >> 24) & 0xff;  // Alpha
            } 

            if (has_alpha)
            {
                blur.multiplyAlpha(src, height, bytes);
            }

            src_ptr = src;
            dst_ptr = dst;

            for (b = 0; b < bytes; b++)
            {
                initial_p = src_ptr[b];
                initial_m = src_ptr[(height-1) * bytes + b];

                /*  Determine a run-length encoded version of the row  */
                blur.runLengthEncode(src_ptr + b, buf, bytes, height);

                for (row = 0; row < height; row++)
                {
                    start = (row < length) ? -row : -length;
                    end = (height <= (row + length) ? (height - row - 1) : length);

                    val = 0;
                    i = start;
                    bb = buf + ((row + i) * 2);

                    if (start != -length)
                    {
                        val += initial_p * (sum[start] - sum[-length]);
                    }

                    while (i < end)
                    {
                        pixels = bb[0];
                        i += pixels;

                        if (i > end)
                        {
                            i = end;
                        }

                        val += bb[1] * (sum[i] - sum[start]);
                        bb += (pixels * 2);
                        start = i;
                    }

                    if (end != length)
                    {
                        val += initial_m * (sum[length] - sum[end]);
                    }

                    dst_ptr[row * bytes + b] = val / total;
                }
            }

            if (has_alpha)
            {
                blur.separateAlpha(dst, height, bytes);
            }

            //gimp_pixel_rgn_set_col (&dest_rgn, dst, column + x1, y1, (y2 - y1));
            for (row = 0; row < height; row++)
            {
                pixel_data = dst[row * bytes + 3] << 24;
                pixel_data |= dst[row * bytes + 0] << 16;
                pixel_data |= dst[row * bytes + 1] <<  8;
                pixel_data |= dst[row * bytes + 2] <<  0;
                newimage.setPixel(column, row, pixel_data);
            } 
        }
    }   

    /*  Now the horizontal pass  */
    if (horz > 0.0)
    {
        horz = fabs(horz) + 1.0;

        if (horz != vert)
        {
            temp = -(horz * horz) / (2 * log(1.0 / 255.0));
            std_dev = sqrt(temp);

        curve = blur.makeCurve(std_dev, &length);

        if (sum != NULL)
        {
            //delete [] sum;
        }

        sum = new int[(2 * length) + 1];

        sum[0] = 0;

        for (i = 1; i <= length*2; i++)
        {
            sum[i] = curve[i-length-1] + sum[i-1];
        }

        sum += length;

        total = sum[length] - sum[-length];
    }

    for (row = 0; row < height; row++)
    {
        //gimp_pixel_rgn_get_row (&src_rgn, src, x1, row + y1, (x2 - x1));
            for (column = 0; column < width; column++)
            {
                pixel_data = (uint)newimage.pixel(column, row);
                src[column * bytes + 0] = (pixel_data >> 16) & 0xff;  // Red
                src[column * bytes + 1] = (pixel_data >>  8) & 0xff;  // Green
                src[column * bytes + 2] = (pixel_data >>  0) & 0xff;  // Blue
                src[column * bytes + 3] = (pixel_data >> 24) & 0xff;  // Alpha
            } 

            if (has_alpha)
            {
                blur.multiplyAlpha(src, width, bytes);
            }

            src_ptr = src;
            dst_ptr = dst;

            for (b = 0; b < bytes; b++)
            {
                initial_p = src_ptr[b];
                initial_m = src_ptr[(width-1) * bytes + b];

                // Determine a run-length encoded version of the row
                blur.runLengthEncode(src_ptr + b, buf, bytes, width);

                for (column = 0; column < width; column++)
                {
                    start = (column < length) ? -column : -length;
                    end = (width <= (column + length)) ? (width - column - 1) : length;

                    val = 0;
                    i = start;
                    bb = buf + (column + i) * 2;

                    if (start != -length)
                    {
                        val += initial_p * (sum[start] - sum[-length]);
                    }

                    while (i < end)
                    {
                        pixels = bb[0];
                        i += pixels;

                        if (i > end)
                        {
                            i = end;
                        }

                        val += bb[1] * (sum[i] - sum[start]);
                        bb += (pixels * 2);
                        start = i;
                    }

                    if (end != length)
                    {
                        val += initial_m * (sum[length] - sum[end]);
                    }

                    dst_ptr[column * bytes + b] = val / total;
                }
            }

            if (has_alpha)
            {
                blur.separateAlpha(dst, width, bytes);
            }

            for (column = 0; column < width; column++)
            {
                pixel_data = dst[column * bytes + 3] << 24;
                pixel_data |= dst[column * bytes + 0] << 16;
                pixel_data |= dst[column * bytes + 1] <<  8;
                pixel_data |= dst[column * bytes + 2] <<  0;
                newimage.setPixel(column, row, pixel_data);

            } 

        }
    }

    pix = newimage;
}

////////////////////////////////////////////////////////////
//world and layer fog controls
void GLWorld::setFog() {
    layerlist->at(activeLayer)->thelayer->layerfog = !layerlist->at(activeLayer)->thelayer->layerfog;

}

void GLWorld::setHFog(int hb)
{
    hfog=  (GLfloat)hb;
    updateFog();
}

void GLWorld::setVFog(int vb)
{
    vfog = (GLfloat)vb;
    updateFog();

}

void GLWorld::updateFog(void)
{
    //glFogi(GL_FOG_MODE, fogMode[fogfilter]);       // Fog Mode
    //glFogfv(GL_FOG_COLOR, fogColor);               // Set Fog Color
    //glFogf(GL_FOG_DENSITY, 0.45f);                 // How Dense Will The Fog Be
    //glHint(GL_FOG_HINT, GL_NICEST);                // Fog Hint Value

    glFogf(GL_FOG_START, hfog);                    // Fog Start Depth : 1000.0f
    glFogf(GL_FOG_END, vfog);                      // Fog End Depth   : 0.0 f

    updateGL();
}
