#include "glworld.h"


// Subroutine to determine which object is selected in
// On Screen selections
void GLWorld::contextMenuEvent(QContextMenuEvent * e)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("GLWorld","QContextMenuEvent()");

    e->accept();		// put this at the start or end?

    QPopupMenu* contextMenu = new QPopupMenu( this );
    Q_CHECK_PTR( contextMenu );
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
        "Context Menu</b></u></font>", this );
    caption->setAlignment( Qt::AlignCenter );
    contextMenu->insertItem( caption );
    contextMenu->insertItem( "Add layer",  this, SLOT(addLayer())  );

    QPopupMenu *submenu = new QPopupMenu( this );
    Q_CHECK_PTR( submenu );
    submenu->insertItem( "Translate",   this, SLOT(toolTranslate()) );
    submenu->insertItem( "Rotate",      this, SLOT(toolRotate()) );
    submenu->insertItem( "Scale",       this, SLOT(toolScale()) );

    contextMenu->insertItem( "Tools", submenu );


    contextMenu->exec( QCursor::pos() );

    delete contextMenu;
}

void GLWorld::mousePressEvent( QMouseEvent *e ) {

    /////////// check for object selection

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("GLWorld::","mousePressEvent() on screen selection");

    int xpos,ypos; xpos = e->x(); ypos = e->y();
    lastx = xpos; lasty = ypos;   // used for tracking motion

    GLuint selectBuff[64];
    GLint hits, viewport[4];

    // Setup selection buffer
    glSelectBuffer(64, selectBuff);

    // Get the viewport
    glGetIntegerv(GL_VIEWPORT, viewport);

    // Switch to projection and save the matrix
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glRenderMode(GL_SELECT);
    glLoadIdentity();
    gluPickMatrix(xpos, viewport[3] - ypos, 2,2, viewport);

    // Apply perspective matrix
    gluPerspective(45.0f, core->fAspect, 1.0, 425.0);

    glTranslatef( 0.0, 0.0, -75.0 );

        // Draw the scene without the background on

        //messes up the opengl cue so it has been disabled for now
        //select = TRUE;	paintGL();  select = FALSE;

        // Collect the hits
        hits = glRenderMode(GL_RENDER);

        if(hits == 1) processSelection(selectBuff);

        glMatrixMode(GL_PROJECTION);
        glPopMatrix();
        glMatrixMode(GL_MODELVIEW);

//      updateScene();                  //whats this do?
    updateGL();

    mousePressed = TRUE;

}

void GLWorld::processSelection(GLuint *pSelectBuff)
{
    JahTrace& jtrace = JahTrace::getInstance();

    int id,count;
    // How many names on the name stack
    count = pSelectBuff[0];
    // Bottom of the name stack
    id = pSelectBuff[3];

    jtrace.debug("GLWorld::processSelection ...you clicked",id);

    //QString clicked;  qDebug("you clicked"+clicked.setNum(id) );

//we are getting the wrong id and it crashes


    if (id<=numberLayers)
    {
        layerlist->at(activeLayer)->thelayer->selected=FALSE;
        activeLayer = id;
        layerlist->at(activeLayer)->thelayer->selected=TRUE;
    }
    else
    {
        jtrace.debug("GLWorld::processSelection error, selection out of range",id);
    }
    //  updateGL();
    //we need a switch layer routine that updates sliders according to layers information!
//switchLayers(activeLayer,newlayer);


}

void GLWorld::mouseReleaseEvent( QMouseEvent * ) 
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug("GLWorld::","mouseReleaseEvent() on screen selection");

    mousePressed = FALSE; 
}

//need to update this routine for camera moves
void GLWorld::mouseMoveEvent( QMouseEvent *e ) 
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug("GLWorld::","mouseMoveEvent() on screen selection");

    if ( mmode == MOFF || mousePressed==FALSE) return;

    int x = e->x();
    int y = e->y();

    if  (activeLayer == 0) { mouseMoveCamera(x,y);            }
    else                   { mouseMoveLayer (activeLayer,x,y); }

    updateGL();
    updateSliders();

    lastx = x; lasty = y;

}

void GLWorld::keyPressEvent( QKeyEvent *k )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLWorld::", "pressed key in module "+ModuleName);

}


//onscreen select tools and widgets
void GLWorld::noTool( void ) 
{ 
    mmode = MOFF;  
    updateToolDisplay(); 
    updateGL(); 
}
void GLWorld::toolTranslate( void ) 
{ 
    mmode = MTRANSLATE;  
    updateToolDisplay(); 
    updateGL(); 
}


void GLWorld::toolScale( void )     
{ 
    mmode = MSCALE;      
    updateToolDisplay(); 
    updateGL(); 
}

void GLWorld::toolRotate( void )    
{
    mmode = MROTATE;     
    updateToolDisplay(); 
    updateGL(); 
}



void GLWorld::updateToolDisplay(void)
{
    renderSpace->setToolMode(mmode);
}

void GLWorld::mouseMoveLayer ( int thelayer, int x, int y )
{

    //this will be used to apply zdepth to movements for sensitivity
    GLfloat coef;  coef = layerlist->at(thelayer)->thelayer->node->tz;             //        (GLfloat)(degrees )

    switch ( mmode ) {

        case MROTATE    : { if ( x>lastx)      { layerlist->at(thelayer)->thelayer->node->ry +=10.0; }
                            else if ( x<lastx) { layerlist->at(thelayer)->thelayer->node->ry -=10.0; }
                            if ( y>lasty)      { layerlist->at(thelayer)->thelayer->node->rx +=10.0; }
                            else if ( y<lasty) { layerlist->at(thelayer)->thelayer->node->rx -=10.0; }
                            break; }

        case MTRANSLATE : { if ( x>lastx)      { layerlist->at(thelayer)->thelayer->node->tx +=5.0; }
                            else if ( x<lastx) { layerlist->at(thelayer)->thelayer->node->tx -=5.0; }
                            if ( y>lasty)      { layerlist->at(thelayer)->thelayer->node->ty -=5.0; }
                            else if ( y<lasty) { layerlist->at(thelayer)->thelayer->node->ty +=5.0; }
                            break; }

        case MSCALE     : { if ( x>lastx)      { layerlist->at(thelayer)->thelayer->node->sx +=0.1; }
                            else if ( x<lastx) { layerlist->at(thelayer)->thelayer->node->sx -=0.1; }
                            if ( y>lasty)      { layerlist->at(thelayer)->thelayer->node->sy -=0.1; }
                            else if ( y<lasty) { layerlist->at(thelayer)->thelayer->node->sy +=0.1; }
                            break; }

        case MOFF       : {  break; }

        default         : {  break; }

    }

}

void GLWorld::mouseMoveCamera( int x,int y )
{

    //this will be used to apply zdepth to movements for sensitivity
    GLfloat coef;  coef = camera->node->tz;             //        (GLfloat)(degrees )

    switch ( mmode ) {

        case MROTATE    : { if ( x>lastx)         { camera->node->ry +=10.0; }
                            else if ( x<lastx)    { camera->node->ry -=10.0; }
                            if ( y>lasty)         { camera->node->rx +=10.0; }
                            else if ( y<lasty)    { camera->node->rx -=10.0; }
                            break; }

        case MTRANSLATE : { if ( x>lastx)         { camera->node->tx +=5.0; }
                            else if ( x<lastx)    { camera->node->tx -=5.0; }
                            if ( y>lasty)         { camera->node->ty -=5.0; }
                            else if ( y<lasty)    { camera->node->ty +=5.0; }
                            break; }

        case MSCALE     : { if ( x>lastx)         { camera->node->sx +=0.1; }
                            else if ( x<lastx)    { camera->node->sx -=0.1; }
                            if ( y>lasty)         { camera->node->sy -=0.1; }
                            else if ( y<lasty)    { camera->node->sy +=0.1; }
                            break; }

        case MOFF       : { break; }

        default         : { break; }

    }

}








