/*******************************************************************************
**
** The source file for the Jahshaka glworldglcore.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"


///////////////////////////////////////////////////////////////////
// the main opengl routine that iterates through all objects
// and paints the scene for us
void GLWorld::paintGL()
{
    //checks to see if the play button is on and increments frames
    if ( animation )
    {  
        updateanimation(currentFrame); currentFrame++;
        if ( currentFrame > maxFrame ) currentFrame=1;
    }

    //turn on hardware aliasing for the scene if it is enabled

    if (hwaliasing)
    {
        glEnable(GL_MULTISAMPLE_ARB);
    }

    //clear the scene and set up matrices
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    //glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();


    //draw black video safe area if no layer present
    if (!select) 
    { 
        if ( isLayerVisible(0) )     
        {  
            renderSpace->drawLayer(*(layerlist->at(0)->thelayer));  
        }
        else 
        {   
            renderSpace->drawLayer(*titlesafe); 
        }
    }

    //we position the camera here
    renderSpace->positionObject(camera->node);

    //draw the grid if its visible
    if (gridval) {  renderSpace->drawLayer(*thegrid); }

    //set up opengl layer selection
    glInitNames(); 
    glPushName(0);                      // initialize the selection stack

    /////////////////////////////////////////////////
    //the main loop where we draw all visible layers
    int dlayer;

    //sort the layers before drawing them
    newsortLayers();

    //now we iterate the list...
    for ( uint i = 1; i <= layerlist->count(); ++i )
    {
        if ( layerlist->at(i) )
        {
            if (numberLayers>1 && newworldorder->at(i-1))
            {
                dlayer = newworldorder->at(i-1)->laynum;
            }
            else
            {
                dlayer = i;
            }

            if ( (isLayerVisible(dlayer)) && (layerlist->at(dlayer)->thelayer->foreground !=1 ))
            {
                //build layer effects first  
                if (layerlist->at(dlayer)->thelayer->layerEffect)
                {
                    buildLayerEffect(dlayer);
                }

                if (layerlist->at(dlayer)->thelayer->objtype==MESHLAYER)
                {
					initializeMeshLayer(dlayer);
                    //buildLayerEffect(dlayer);
                }

                //then draw the layer

                renderSpace->drawLayer(*(layerlist->at(dlayer)->thelayer));

                //then draw the keyframes if visible
                if (layerlist->at(dlayer)->thelayer->drawKeyframes)
                {
                    renderSpace->drawNodes(*(layerlist->at(dlayer)->thelayer));
                }
            }
        }
    }


    glPopMatrix();

    /////////////////////////////////////////////////////////////////////////////
    //draw the forground layers ie objects not affeceted by worldspace or sorting
    
    if (numberLayers>0) 
    {
        for ( uint i = 1; i <= layerlist->count(); ++i )
        //for (int i=1; i<=numberLayers; i++) 
        {
        glPushMatrix();
        if ( layerlist->at(i) ) 
        {
            if (isLayerVisible(i) && layerlist->at(i)->thelayer->foreground==1) 
            {
                renderSpace->drawLayer(*(layerlist->at(i)->thelayer));
                if(layerlist->at(i)->thelayer->drawKeyframes) 
                { 
                    renderSpace->drawNodes(*(layerlist->at(i)->thelayer)); 
                }
            }
        }
        glPopMatrix();
        }
    }

    //if textsafe is on draw it as well
    if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

    if (hwaliasing)
    {
        glDisable(GL_MULTISAMPLE_ARB);
    }

    //flush render que
    glFlush(); 

    /////////////////////////////////////////////////
    // finished the scene now Wait this many msecs before redraw
    if ( animation ) 
    {
        timer->start( redrawWait, TRUE );
    } 

}

void GLWorld::initializeWorld() 
{
    //set up some default variables and objects
    polySmooth=0;
    activeLayer = 0; currentFrame=1;
    lastLayer = 0; select = FALSE;  // lastLayer is used to unselect...

    animation = FALSE; animsliderselected=FALSE;  renderStatus=FALSE;
    lastx = lasty = 0;  currentFrame =0;

    // set up the world first   - is this necessary? what are the defaults
    camera->node->sx = camera->node->sy = camera->node->sz = camera->node->sc = 1.0;
    //camera->layerStatus = TRUE;
    //camera->drawtheLayer = TRUE;

    ////////////////////////////////////////////////
    // note these three should be shared display lists 
    // and not layers

    // set up the black title safe area
    //should be called rendersafe
    titlesafe->layertype = LAYER; 
    titlesafe->objtype = MAINBACK;

    titlesafe->blend = TRUE; 
    titlesafe->depth = FALSE;
    titlesafe->layerStatus = TRUE;
    titlesafe->drawtheLayer = TRUE;

    // this is the green textsafe overlay
    textsafe->node->sx = 0.8; textsafe->node->sy = 0.8; //it is scaled to fit here...
    textsafe->layertype = FRAME; textsafe->objtype = TITLEGRID;
    textsafe->Red=0.0; textsafe->Green=1.0; textsafe->Blue=0.0;
    textsafe->blend = FALSE; textsafe->depth = FALSE;
    textsafe->foreground = TRUE; 
    textsafe->layerStatus = FALSE;
    textsafe->drawtheLayer = TRUE;

    // this is the grid
    thegrid->layertype = GRID;
    thegrid->Red=0.75; thegrid->Green=0.75; thegrid->Blue=0.75;
    thegrid->blend = TRUE; thegrid->depth = TRUE;
    thegrid->layerStatus = TRUE;

}

void GLWorld::initializeObjects() {   /* this is overridden */    }

void GLWorld::raiseCore(void)
{
    core->makeCurrent();
    core->updateGL();
    core->setFocus();
    
    core->setZoom(1);

    updateGL();
}



///////////////////////////////////////////////////////
// initialize mesh layer
void GLWorld::initializeMeshLayer(int layer)
{

    int                     i, j;
    GLfloat                 xx, yy;

    GLfloat                    x_unit_size;
    GLfloat                    y_unit_size;
    float                    image_width;
    float                    image_height;

    image_width = layerlist->at(layer)->thelayer->getImageWidth();
    image_height = layerlist->at(layer)->thelayer->getImageHeight();

    x_unit_size = image_width  / (float)JAHMESH_X_DIMENSION;
    y_unit_size = image_height / (float)JAHMESH_Y_DIMENSION;

    for (i = 0; i <= JAHMESH_X_DIMENSION; i++)
    {
            xx = (float)(i - JAHMESH_X_DIMENSION_DIV_2) * x_unit_size;

            for (j = 0; j <= (JAHMESH_Y_DIMENSION); j++)
            {
            yy = (float)(j - JAHMESH_Y_DIMENSION_DIV_2) * y_unit_size;

                    layerlist->at(layer)->thelayer->setMeshCoord(i, j, xx, yy, 0.0);
            }
    }

}













