/*******************************************************************************
**
** The source file for the Jahshaka glworldlayer.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"

//////////////////////////////////////////////
// function thats used to create a new layer
// when you add a layer in a module

bool GLWorld::buildLayer( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype ) 
{

    //first we set up default variables
    layerlist->at(nl)->thelayer->asset.clipnumber   = nl;
    layerlist->at(nl)->thelayer->layertype          = layertype;
    layerlist->at(nl)->thelayer->objtype            = objtype;

    QString layNum; layNum.setNum(nl);
    //layerlist->at(nl)->thelayer->asset.clipname     = layerName;
    layerlist->at(nl)->thelayer->layername          = layerName;

    bool itworked = TRUE;

    ////////////////////////////////////////
    //now we create the layer

    //first we add it to the worldsort list
    newworldorder->append( new worldSort );


    switch (layertype) {

        case LAYER    :  {    
                                //we should check to see if no path is set...
                                layerlist->at(nl)->thelayer->drawtheLayer = TRUE;

                                //we only auto-create single frame layers
                                //but the .jah loader uses this so we override for clips
                                if (!layerlist->at(nl)->thelayer->asset.videotex)
                                {
                                    layerlist->at(nl)->thelayer->inframe    = 1;
                                    layerlist->at(nl)->thelayer->outframe   = 1;
                                }

                                //load the image into the layer
								//debug("adding layer");
                                layerlist->at(nl)->thelayer->LoadLayerAsset(NOCATEGORY);

                                //now we make the texture
                                layerlist->at(nl)->thelayer->makeTexture();
                                
								//debug("added layer");

								break; 
                            }

        case PARTICLE :     {   
                                layerlist->at(nl)->thelayer->addParticleObj();   
                                break; 
                            }	

        case OBJECTS  :     {   
                                if (!layerlist->at(nl)->thelayer->asset.location) 
                                { 
                                    layerlist->at(nl)->thelayer->asset.location  = JahMediaPath;
                                }

                                //this adds a obj type object            
                                if ( objtype==OBJOBJECTS) 
                                {
                                    layerlist->at(nl)->thelayer->loadOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                                }

                                //this adds a 3dsmax type object
                                if ( objtype==MAXOBJECTS) 
                                {
                            
                                    layerlist->at(nl)->thelayer->loadMaxOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                
                                }

                                break; 
                            }

        case FRAME    :     {   break; }

        case TEXT     :     {   
                                if ( layerlist->at(nl)->thelayer->loadFontText()) 
                                { 
                                    layerlist->at(nl)->thelayer->drawText(); 
                                }

                                break; 
                            }

        case GRID     :     {   break; }

        case LIGHT    :     {   //need to keep track of gloabl number of lights
                                //and pas the light number over
                                layerlist->at(nl)->thelayer->makeLight();  break; }

        default       :     {   
                                itworked = FALSE;
                                //debug("object type not supported"); 

                                break; 
                            }

    }

    return itworked;

}

//////////////////////////////////////////////
// function thats used to create a layer
// when you load it in from disk ie from a scene file

bool GLWorld::loadLayer( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype, 
                        bool isasset, videoData asset ) 
{

    ////////////////////////////////////////
    //first we set up default variables
    layerlist->at(nl)->thelayer->layertype          = layertype;
    layerlist->at(nl)->thelayer->objtype            = objtype;
    layerlist->at(nl)->thelayer->layername          = layerName;

    bool itworked = TRUE;

    ////////////////////////////////////////
    //now we create the layer

    //first we add it to the worldsort list
    newworldorder->append( new worldSort );


    switch (layertype) {

        case LAYER    :  {    
                                //we assume that all loaded LAYERS have a asset assigned

                                //first we process the IMAGE layer
                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case PARTICLE :     {   
                                layerlist->at(nl)->thelayer->addParticleObj(); 

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }             

                                break; 
                            }	

        case OBJECTS  :     {   
                                if (!layerlist->at(nl)->thelayer->asset.location) 
                                { 
                                    layerlist->at(nl)->thelayer->asset.location  = JahMediaPath;
                                }

                                //this adds a obj type object            
                                if ( objtype==OBJOBJECTS) 
                                {
                                    layerlist->at(nl)->thelayer->loadOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                                }

                                //this adds a 3dsmax type object
                                if ( objtype==MAXOBJECTS) 
                                {
                                    layerlist->at(nl)->thelayer->loadMaxOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                                }

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case TEXT     :     {   
                                if ( layerlist->at(nl)->thelayer->loadFontText()) 
                                { 
                                    layerlist->at(nl)->thelayer->drawText(); 
                                }

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case GRID     :     {   break; }
        case FRAME    :     {   break; }

        default       :     {   
                                itworked = FALSE;
                                //debug("object type not supported"); 

                                break; 
                            }

    }

    return itworked;

}

//we are overloading this call for now later on we need to set defaults
//in the function header ie loadkey=0
//but we need to switch all variables to pointers in order
//to do this
bool GLWorld::loadLayer( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype, 
                        bool isasset, videoData asset, bool haskey, videoData keyasset ) 
{

    ////////////////////////////////////////
    //first we set up default variables
    layerlist->at(nl)->thelayer->layertype          = layertype;
    layerlist->at(nl)->thelayer->objtype            = objtype;
    layerlist->at(nl)->thelayer->layername          = layerName;

    bool itworked = TRUE;

    ////////////////////////////////////////
    //now we create the layer

    //first we add it to the worldsort list
    newworldorder->append( new worldSort );


    switch (layertype) {

        case LAYER    :  {    
                                //we assume that all loaded LAYERS have a asset assigned

                                //first we process the IMAGE layer
                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                //first we process the IMAGE layer
                                if (haskey)
                                {
                                    //layerlist->at(nl)->thelayer->drawtheKey = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(keyasset, KEY);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(KEY);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case PARTICLE :     {   
                                layerlist->at(nl)->thelayer->addParticleObj(); 

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }             

                                break; 
                            }	

        case OBJECTS  :     {   
                                if (!layerlist->at(nl)->thelayer->asset.location) 
                                { 
                                    layerlist->at(nl)->thelayer->asset.location  = JahMediaPath;
                                }

                                //this adds a obj type object            
                                if ( objtype==OBJOBJECTS) 
                                {
                                    layerlist->at(nl)->thelayer->loadOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                                }

                                //this adds a 3dsmax type object
                                if ( objtype==MAXOBJECTS) 
                                {
                                    layerlist->at(nl)->thelayer->loadMaxOBJ(layerlist->at(nl)->thelayer->objfilename+layerlist->at(nl)->thelayer->objextension);   
                                }

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case TEXT     :     {   
                                if ( layerlist->at(nl)->thelayer->loadFontText()) 
                                { 
                                    layerlist->at(nl)->thelayer->drawText(); 
                                }

                                if (isasset)
                                {
                                    layerlist->at(nl)->thelayer->drawtheLayer = TRUE;
                                    layerlist->at(nl)->thelayer->assignAsset(asset, IMAGE);
                                    layerlist->at(nl)->thelayer->LoadLayerAsset(IMAGE);
                                    layerlist->at(nl)->thelayer->makeTexture();
                                }

                                break; 
                            }

        case GRID     :     {   break; }
        case FRAME    :     {   break; }

        default       :     {   
                                itworked = FALSE;
                                //debug("object type not supported"); 

                                break; 
                            }

    }

    return itworked;

}
/////////////////////////////////////////////////////////////
//switch the layer object type but we dont mess with layer 0
//as its the world object

void GLWorld::ChangeObjectLayer(void)    
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = JAHLAYER;  
        updateGL(); 
    } 
}

void GLWorld::ChangeObjectCube(void)     
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = CUBE;      
        updateGL(); 
    } 
}

void GLWorld::ChangeObjectCylinder(void) 
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = CYLINDER;  
        updateGL(); 
    } 
}

void GLWorld::ChangeObjectSphere(void)   
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = SPHERE;    
        updateGL(); 
    } 
}

void GLWorld::ChangeObjectBezier(void)   
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = BEZIERS;   
        updateGL(); 
    } 
}


/////////////////////////////////////////////////////////////////
//we need to initialize the mesh if we plan to use it...

void GLWorld::ChangeObjectMesh(void)     
{ 
    if (activeLayer>0) 
    { 
        layerlist->at(activeLayer)->thelayer->objtype = MESHLAYER; 
        updateGL(); 
    } 
}
