/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "anime.h"


void GLAnime::initializeObjects()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLAnime Class");

    //initialize the keyframe editor
    thekeyframer = new JahKeyframer( this, moduleOptionsUI[5], "keyframer" );
    thekeyframer->setFixedSize( QSize( 980, 306 ) ); 
    connectKeyframer();
    haskeyframer = true; //let the world know its there

    //make the current opengl context current so the textures can be set up properly
    makeCurrent();

    ///////////////////////////////////////////////
    //this creates the default world layer using the linked list
    layerlist->append( new NewLayerList(0, "World") );

    //setup and build the default layer these should all be defaults
    layerlist->at(0)->thelayer->blend = TRUE;
    layerlist->at(0)->thelayer->depth = FALSE;
    layerlist->at(0)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(0)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer";
    layerlist->at(0)->thelayer->asset.extension = ".bmp";
    //layerlist->at(0)->thelayer->layerfog = TRUE;
    buildLayer(0,"World", LAYER, BACKDROP );

    //adds default layer as 'off' uses default numberLayers = 1
    addSelection(FALSE);  

    //hook up menu and sliders and default tool
    objectMenu(numberLayers);
    updateSliders();

    //set the ui tool
    noTool();

    //turn forceplay on here
    setForcePlay(true);


}

//all modules are responsable for their own interface and their own sliders
//we need to find a different way of doing the updates
//ie update only the current settings for the current tabbox

//both uioptions and ui settings need to be local

void GLAnime::updateUiOptions()
{
    ////////////////////////////////////////////////////////////////////////////////////
    // all other ui settings
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLAnime","updating ui options");


	//need a counter to track numoptions...
	//numUiOptions - needs to be a array, 1 per layer... ?

	if (numUiOptions >0)
	{

		hideOptions(numUiOptions-1);


		//basic values here

		if (activeLayer>0)
		{
			showOptions(numUiOptions-1);
		}

		//for (int i=0l i<=numUiOptions; i++)
		//{
			setOptionValue(numUiOptions-1, layerlist->at(activeLayer)->thelayer->drawKeyframes );
		//}
	}

}

void GLAnime::updateUiSettings()
{
    ////////////////////////////////////////////////////////////////////////////////////
    // all other ui settings
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLAnime","updating ui settings");

    //for transparency
    Trans_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );
    Translcd_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );


    //update video frames here
    FramesLcd->setValue   ( layerlist->at(activeLayer)->thelayer->asset.getNumFrames() );
    inFramesLcd->setValue ( layerlist->at(activeLayer)->thelayer->inframe );
    outFramesLcd->setValue( layerlist->at(activeLayer)->thelayer->outframe );

    KeyFramesLcd->setValue   ( layerlist->at(activeLayer)->thelayer->keyasset.getNumFrames() );
    inKeyFramesLcd->setValue ( layerlist->at(activeLayer)->thelayer->keyinframe );
    outKeyFramesLcd->setValue( layerlist->at(activeLayer)->thelayer->keyoutframe );

    extendHeadTail->setChecked ( layerlist->at(activeLayer)->thelayer->showEdges );
    toggleTheLoop->setChecked ( layerlist->at(activeLayer)->thelayer->loop );
    togglePing->setChecked ( layerlist->at(activeLayer)->thelayer->ping );

    //extended values here need to add everything new
    GridSelect->setChecked(gridval);
    TextSelect->setChecked(textsafe->layerStatus);

    light1->setChecked(layerlist->at(activeLayer)->thelayer->lighting);
    ClipSelect->setChecked(layerlist->at(activeLayer)->thelayer->drawtheLayer);
    KeySelect->setChecked(layerlist->at(activeLayer)->thelayer->drawtheKey);
    ForegroundSelect->setChecked(layerlist->at(activeLayer)->thelayer->foreground);

    //for the effects panel
    BlurSelect->setChecked(layerlist->at(activeLayer)->thelayer->blur);
    NegativeSelect->setChecked(layerlist->at(activeLayer)->thelayer->negative);
    SwapRGBSelect->setChecked(layerlist->at(activeLayer)->thelayer->swaprgb);
    MirrorSelect->setChecked(layerlist->at(activeLayer)->thelayer->mirror);

    reflect1->setChecked(layerlist->at(activeLayer)->thelayer->reflect);
    smooth1->setChecked(layerlist->at(activeLayer)->thelayer->smooth);
    shader1->setChecked(layerlist->at(activeLayer)->thelayer->pixelshader);
    shader2->setChecked(layerlist->at(activeLayer)->thelayer->pixelshader);
}



///////////////////////////////////////////////////////////
//this is a fricking mess
//and needs to be cleaned up!!!

//can we move this routine into the World object?
//seems like it needs to be local
//in order to call the local updatesliders routine

void GLAnime::clicked( QListViewItem* items )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "**Clicked on layer in GLAnime ");
    

    //nothing added to listview yet
    if (!items) return;

	//clear the selection list first
	activeLayers->clearList();
   
    //this is the number of items in the listview
    //problems will arise here when we add children ie grouping    
    activeLayers->setTotalLayers(lv->childCount());

    //iterate the list now to see whats selected
    //this is slow but its a easy way to
    //go through all layers to find the ones selected

    int numSelections=0;

    QListViewItemIterator it( lv );
    while ( it.current() ) 
    {
        QListViewItem *item = it.current();
        ++it;

        if ( item->isSelected() )  activeLayers->addLayer(numSelections);

        numSelections++; //re becomes total layers...
    }

    //if we got more than one selected its a global to let the core know
    if (activeLayers->getNumLayers()>1) multiSelect = TRUE; else multiSelect = FALSE;

    //checks if the item is checked or not
    int itemstatus;
    if (   ( ( QCheckListItem* )items )->isOn()==1   ) { itemstatus=1; }
    if (   ( ( QCheckListItem* )items )->isOn()==0   ) { itemstatus=0; }

    int pp = activeLayers->getFinalSelection();

    QString ras;
//    debug("clicked layer:"+ras.setNum(pp));
    
    //brings up the menu for the selected layer according to layer number
    objectMenu(pp);

    //we waste a interupt here, always checking the clicked item
    //would be better with a signal slot implementation
    int layerStat = layerlist->at(activeLayer)->thelayer->layerStatus;

    if      ( itemstatus==1  && layerStat==0 ) 
    { 
        setLayerVisible(pp, TRUE); 
    }
    else if ( itemstatus==0  && layerStat==1 ) 
    { 
        setLayerVisible(pp, FALSE); 
    }
    else
    {      
        updateGL();
    }  
    
    //clicked on layer, update keyframes
    //should only do this if the layer has changed...
    updateKeyframeDisplay();
     
    //now update the sliders
    //do we need a activeLayer = pp here?
    updateSliders();

}





















