/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "colorize.h"


void GLColorize::initializeObjects()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLEffect Class");

    //initialize the keyframe editor
    //we can always move this into the effectinterface.cpp obj
	
    thekeyframer = new JahKeyframer( this, moduleOptionsUI[4], "keyframer" );
    thekeyframer->setFixedSize( QSize( 980, 600 ) );    //was 955,486
    connectKeyframer();
    haskeyframer = true;

    //make the current opengl context current so the textures can be set up properly
    makeCurrent();

    ///////////////////////////////////////////////
    // the linked list is here
    effects = new QPtrList<FxLayerList>;
    effects->setAutoDelete( TRUE );     // delete items when they are removed

    //effects start at 2 since they match up with layers 0 and 1 that are used below...
    effects->append( new FxLayerList(0) );
    effects->at(0)->fxlayer->objtype = NOFX;

    effects->append( new FxLayerList(1) );
    effects->at(1)->fxlayer->objtype = NOFX;

    ///////////////////////////////////////////////
    //this creates the default world layer using the linked list

    //for effects there is no world control at the moment
    //so we need to build it and blank it out...
    layerlist->append( new NewLayerList(0, "World") );
    layerlist->at(0)->thelayer->layerStatus = false;	//set the status to false since we dont useit
    layerlist->at(0)->thelayer->blend = TRUE;
    layerlist->at(0)->thelayer->depth = FALSE;
    layerlist->at(0)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(0)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer";
    layerlist->at(0)->thelayer->asset.extension = ".bmp";
    buildLayer(0,"World", LAYER, BACKDROP );
    numberLayers+=1;

    //this creates layer 1 all the effects are grouped to it
    //in prepapartion for layer effect grouping  
    layerlist->append( new NewLayerList(1, "World") );

    //setup and build the default layer these should all be defaults
    layerlist->at(1)->thelayer->blend = TRUE;
    layerlist->at(1)->thelayer->depth = TRUE;
    layerlist->at(1)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(1)->thelayer->node->tz = 0.0;
    layerlist->at(1)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer"; 
    layerlist->at(1)->thelayer->asset.extension = ".bmp";
    //layerlist->at(1)->thelayer->objtype = MESHLAYER;
    buildLayer(1,"FxLayer", LAYER, MESHLAYER );  //was JAHLAYER

    addSelection(FALSE);   // adds effects layer as 'off' uses default numberLayers = 1

    objectMenu(numberLayers);
    updateSliders();


    //used to prevent people adding fx till they get a clip
    initialized = false;

    //turn forceplay on here
    setForcePlay(true);

    //////////////////////////////////////////////////////
    // added for custom variables
    static bool initVariables = true;
    if (initVariables)
        initializeVariables();
    initVariables = false;

    //set the ui tool
    noTool();

	//render using the source clip res
	SetVideoRender();
	VideoRenderSelect->setChecked(true);

}

void GLColorize::initializeVariables(void)
{
    //set up prefs and check for gpu support
    JahPrefs& jprefs = JahPrefs::getInstance();

    //for gpu effects
    gpusupport = false;
    nv_gpusupport = false;
    gpuactive = false;

    gpusupport = jprefs.getGpusupport();
    nv_gpusupport = jprefs.getNVGpusupport();



    //use source size renders
    SetVideoRender();

    //initialize timer object
    //used to time the effects
    fxtimer = new JahTimer;

    //this is used for the gpu effects
    // The camera distance calculation should be passed in from the GL resize
    // routine.  This is a HACK which should be fixed FIXME

    //#define Y_RESOLUTION 		486
    #define PI                  3.14159265358979323846
    #define VIEW_ANGLE			45.0
    #define VIEW_ANGLE_RADIANS 	((VIEW_ANGLE / 180.0) * PI)

    camera_distance = ((float)Y_RESOLUTION / 2.0) / tan(VIEW_ANGLE_RADIANS / 2.0);

    noiseMatrixInitialized = FALSE;
    fireTextureInitialized = FALSE;
    randomTextureInitialized = FALSE;
    contrastTextureInitialized = FALSE;

    #ifdef NVIDIA_GPGPU
    shader_display = 0;
    shader_pbuffer = 0;
    pixel_buffer = NULL;
    #endif

    ////////////////////////////////////////////////////////////////////////////
    // routine to load plugins in

//    loadPlugins();

}

void GLColorize::buildEffect() 
{
	    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QImage renderimage;

    //a hack to keep the smooth settings the way they are?
    GLint               current_shade_model;

    glGetIntegerv(GL_SHADE_MODEL, &current_shade_model);
    glShadeModel(GL_SMOOTH);

    //////////////////////////////////////////////////////////
    //the core effect loop

        jtrace.debug("GLEffect::buildEffect()","updating effect");

        if (numberLayers>0)
        {
            //grab the source image into buffer to apply effects to
            renderimage = layerlist->at(1)->thelayer->textureData.objectImage.copy();

            if (gpuactive)
                layerlist->at(1)->thelayer->rebuildTextureDataFx(renderimage,true);

            // now we build the effects stacked on top of each other.
            // Using the mesh, we always have to reset the base layer
            // to flat before adding effects on top of eachother
			initializeMeshLayer(1);
            //doBaseLayer();

            //now we build the effects stacked on top of each other using renderimage
            for (int i=2; i<=numberLayers; i++)
            {

                if (layerlist->at(i)->thelayer->layerStatus ) // && effects(i).recalculate to be implemented later
                {
                        callEffect(effects->at(i)->fxlayer->objtype,i,renderimage);
                }
            }

            if (!gpuactive)
                layerlist->at(1)->thelayer->rebuildTextureDataFx(renderimage,true);

        }


    glShadeModel(current_shade_model);

}

void GLColorize::paintGL() 
{
    if ( animation )
    {  
        updateanimation(currentFrame);
        currentFrame++;

        if ( currentFrame > maxFrame )
        currentFrame = 1;
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    if (!select)
    { 
        if ( isLayerVisible(0) )
        {
            renderSpace->drawLayer(*(layerlist->at(0)->thelayer));
        }
        else
        {
            if (renderStatus)
                titlesafe->renderStatus = true;

            renderSpace->drawLayer(*titlesafe);
        }
    }

    //we position the camera here
    renderSpace->positionObject(camera->node);

    //draw the grid if its visible
    if (gridval) 
    {  
            renderSpace->drawLayer(*thegrid); 
    }

    if ( layerlist->at(1) ) 
    {
        if (layerlist->at(1)->thelayer->layerStatus)
        {

            buildEffect();

            renderSpace->drawLayer(*(layerlist->at(1)->thelayer));

            if (gpuactive)
            {
                check_gl();
            }
        }

    }
    glPopMatrix();

    //this would be titlesafe...
    if (textsafe->layerStatus) 
    {  
            renderSpace->drawLayer(*textsafe);  
    }

    glFlush();

    if ( animation ) 
    {
            timer->start( redrawWait, TRUE );                // Wait this many msecs before redraw
    }

    slidersHaveChanged = false;

}


void GLColorize::updateSliders()
{
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLColorize","updating slider display");
    jtrace.debug( "active layer",activeLayer);


    //check to see if activelayer is a plugin here so we can name sliders accordingly...

    //step 1. hide all sliders
    //loop through num of sliders to use
    //  turn them on
    //use this for all effects ie plugins and not plugins
    //or just make all effects plugins :)

    if (effects->at(activeLayer)->fxlayer->objtype == JAHPLUGIN)
    {
    }
    else
    {
        showAllSliders();
    }

        /////////////////////////////////////////////////////////////////////////////
        //basic values here

        setSliderValue( 0, (int)layerlist->at(activeLayer)->thelayer->node->rx );
        setSliderValue( 1, (int)layerlist->at(activeLayer)->thelayer->node->ry );
        setSliderValue( 2, (int)layerlist->at(activeLayer)->thelayer->node->rz );

        setSliderValue( 3, (int)layerlist->at(activeLayer)->thelayer->node->tx );
        setSliderValue( 4, (int)layerlist->at(activeLayer)->thelayer->node->ty );
        setSliderValue( 5, (int)layerlist->at(activeLayer)->thelayer->node->tz );

        setSliderValue( 6, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sx)  );
        setSliderValue( 7, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sy)  );
        setSliderValue( 8, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sz)  );

        updateUiSettings();




}

void GLColorize::updateUiSettings()
{
    //Trans_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );
    //Translcd_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );

    //light1->setChecked(layerlist->at(activeLayer)->thelayer->lighting);
    VideoRenderSelect->setChecked(core->useimagesize);

}


//tis routine is always the same but needs to be local in order to call
//the local routines such as updateSliders etc
void GLColorize::clicked( QListViewItem* items )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "**Clicked on layer in GLEffect ");


    //nothing added to listview yet
    if (!items) return;

    //clear the selection list first
    activeLayers->clearList();

    //this is the number of items in the listview
    //problems will arise here when we add children ie grouping
    activeLayers->setTotalLayers(lv->childCount());

    //iterate the list now to see whats selected
    //this is slow but its a easy way to
    //go through all layers to find the ones selected

    int numSelections=0;

    QListViewItemIterator it( lv );
    while ( it.current() )
    {
            QListViewItem *item = it.current();
            ++it;

            if ( item->isSelected() )  activeLayers->addLayer(numSelections);

            numSelections++; //re becomes total layers...
    }

    //if we got more than one selected its a global to let the core know
    if (activeLayers->getNumLayers()>1) multiSelect = TRUE; else multiSelect = FALSE;

    //checks if the item is checked or not
    int itemstatus=0;
    if (   ( ( QCheckListItem* )items )->isOn()==1   ) { itemstatus=1; }
    if (   ( ( QCheckListItem* )items )->isOn()==0   ) { itemstatus=0; }

    //in effects we start with the layer and hide the world so we have to increment this
    int pp = activeLayers->getFinalSelection()+1;

    //QString ras;
    //debug("clicked layer:"+ras.setNum(pp));

    //brings up the menu for the selected layer according to layer number
    objectMenu(pp);

    //we waste a interupt here, always checking the clicked item
    //would be better with a signal slot implementation
    int layerStat = layerlist->at(activeLayer)->thelayer->layerStatus;


    if      ( itemstatus==1  && layerStat==0 )
    { 
        forceupdatefxbuffer = true;
        setLayerVisible(pp, TRUE);
    }
    else if ( itemstatus==0  && layerStat==1 )
    {
        forceupdatefxbuffer = true;
        setLayerVisible(pp, FALSE);
    }
    else
    {
        forceupdatefxbuffer = true;
        updateGL();
    }

    //clicked on layer, update keyframes
    //should only do this if the layer has changed...

    //updateKeyframeDisplay();

    //now update the sliders
    //do we need a activeLayer = pp here?
    
	updateSliders();

}


//we override the object menu routine here since we have non standard
//menu selections
void GLColorize::objectMenu(int menu) 
{      
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "raising layer",menu);

    if (menu==1)
    {
        //the default layer
        //fxoptionals->raiseWidget(3);
    }
    else
    {
        //the effects layers
        //fxoptionals->raiseWidget(0);
    }

    changeSelection(menu);

}

//we override the add clip routine here to always add to layer 1
//since the effects are on the other layers
void GLColorize::addClip(videoData clip) 
{
    JahTrace& jtrace = JahTrace::getInstance();

    initialized = true;

    layerlist->at(1)->thelayer->assignAsset(clip, IMAGE);

    layerlist->at(1)->thelayer->LoadLayerAsset(IMAGE);

    layerlist->at(1)->thelayer->makeTexture();

    //set renderspace imagesize here...
    core->imagewidth  = layerlist->at(1)->thelayer->getImageWidth();
    core->imageheight = layerlist->at(1)->thelayer->getImageHeight();

    forceupdatefxbuffer = true;

    //initialize the buffer
    //if (useBuffering)
    //    initBuffer(&layerlist->at(1)->thelayer->textureData.objectImage);

    updateGL();
    updateSliders();
}






















