/*******************************************************************************
**
** The header file for the Jahshaka Colorize module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef COLORIZE_H
#define COLORIZE_H

#include <glworld.h>

#include "qcheckbox.h"
#include "qtabwidget.h"

#include <stdlib.h>

#include "jahnodes.h"

#include "jahtimer.h"

////////////////////////////////////////////////////////////
//for gpu support
#include "gpumathlib.h"

extern unsigned char*       loadshaderfile(const char* name);
extern void                 find_shader_program_error(unsigned char* source, char* program_name);
extern void                 check_gl(void);
extern int                  ImageLoad(char* filename, Image* image);


class GLColorize :  public  GLWorld {
  Q_OBJECT

public:

  GLColorize( GLCore* core, const char* name,
              QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
              int* globalclipnumber, const QGLWidget* shareWidget=0  )
              :  GLWorld(core,name,controller,globalclipnumber )
    
{
    buildInterface(controls);
 
	buildInterfaceMenus(leftcontroller,rightcontroller);
    
	initializeObjects();
};

GLColorize::~GLColorize() {  }

    void buildInterface( QHBox* qtinterface);
    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

    void createAxisPanel ( QFrame* theuiframe);
    void createColorPanel ( QFrame* theuiframe);

    void initializeObjects();
    
    //used to prevent people adding fx till they get a clip 
    bool initialized;

	void initializeVariables(void);

public slots:
    void addClip(videoData clip);

    void addLayer(void);
	
    void clicked( QListViewItem* item );

    void updateSliders();
    void updateUiSettings();

    void grabDesktop(void) { emit GrabClip(); };

	//these routines can also be shared
	void  callEffect( FxCategory, int, QImage& );

    void doColorMatrix(int layer);
    void doRtColors(int layer);
    void doNegative(QImage& theimage);
    void doSwapRGB(QImage& theimage); 
    
    //void doBaseLayer(void);


signals:
    void    GrabClip(void);    //linked to main jahcontrol to talk to desktop

protected:

    void objectMenu(int menu);
    void paintGL();
	void buildEffect();

private:
	
	////////////////////////////////////////////////////////////////////////////
	// effect based variables


    QImage      sourceimage,savedimage;

    bool        rebuildStatus;
    int         effectobj;
    bool        gpusupport,nv_gpusupport,gpuactive;

    bool        noiseMatrixInitialized;
    int*        permutation_table;

    float4*     gradient_table;

    bool        fireTextureInitialized;
    bool	    randomTextureInitialized;
    bool	    contrastTextureInitialized;

    GLuint      gradient_texture;
    GLuint      permutation_texture;
    GLuint      turbulence_texture;
    GLuint      random_texture;
    GLuint      contrast_texture;

#ifdef NVIDIA_GPGPU 
    GLXFBConfig*                        glxConfig;
    Display*                            shader_display;
    GLXPbuffer                          shader_pbuffer;
    GLXContext                          shader_context;
    float4*								pixel_buffer;
    GLXContext                          jahshaka_glx_context;
    GLXDrawable                         jahshaka_glx_drawable;
    Display*                            jahshaka_display;
#endif

    int         iScreen;

    //The effects timer object
    JahTimer    *fxtimer;

    float       z_offset;
    float       camera_distance;

    ////////////////////////////////////////////////////////////////////////////
    //interface variables
	///////////////////////////////////////////////////////////////////////
	//interface variables for the left and right world space menus
	///////////////////////////////////////////////////////////////////////
	
	QFrame			*glworldLeft;
	JahToolButton	*tooltranslate,	*toolscale,		*toolrotate;
	JahToolButton	*playerphoto,	*switchtheview;

	QFrame			*glworldRight;
	JahToolButton	*toolzoomin,	*toolzoomout;
	QSlider			*zoom;

	///////////////////////////////////////////////////////////////////////
	//animation module ui options
	///////////////////////////////////////////////////////////////////////

	//declare all controllers that we are using here
	enum tabUI {SCENE, OBJECT, AXIS, CLIP, EFFECTS, KEYFRAMES, numTabs};
  
	///////////////////////////////////////////////////////////////////////
	//now the main widget variables

	//the first, main frame
	QFrame*			LayersFrame;
	QFrame*			EffectsFrame;

    //for the control panel
    QPushButton*    AddRTButton;
    QPushButton*    AddGpuButton;
    QPushButton*    AddPluginButton;

	///////////////////////////////////////////////////////////////////////
	//the tab frame to hold the tabs

	QTabWidget		*tabframe;

	//widegts for each of the tabs
	QWidget			*moduleOptionsUI[numTabs];

	//each module option ui can have up to 10 embedded ui's
    QFrame			*ControlFrameUI[numTabs][10];

	///////////////////////////////////////////////////////////////////////
	// variables for the different tabs

	///////////////////////////////////////////////////////////////////////
	//set up the scene controls

	QPushButton*	ResetButton;             
	QPushButton*	SaveButton;    
	QPushButton*	LoadButton;              
	QPushButton*	ClearButton;
	QPushButton*	PackageButton;
    QCheckBox*      VideoRenderSelect;
    QCheckBox*      GpuSelect;

	//lcdstartframe and endframe are initialized in world

	QCheckBox*		FogSelect;
	QSlider			*HFogSlider,		*VFogSlider;
	InputLCD		*HFogSlider_lcd,	*VFogSlider_lcd;

	//////////////////////////////////////////////////////////////////////
	//set up the different object controls, multiple layers are embedded

	//the world
	QCheckBox		*SmoothSelect;            
	QCheckBox		*TextSelect;
	QCheckBox		*GridSelect;
	QPushButton		*ResetWorldButton;

	//the layer types
	QPushButton		*ObjectLayer;
	QPushButton		*ObjectCube;
	QPushButton		*ObjectCylinder;
	QPushButton		*ObjectSphere;
	QPushButton		*ObjectBezier;
	QPushButton		*ObjectMesh;
	QPushButton		*ResetLayerButton;

	//the font options
	//the particle options

	//the object options obj's
	QPushButton		*loadobject_1,	*objectstyle;
	QPushButton		*ResetObjectButton1;

	//the object options 3dsmax
	QPushButton		*loadmaxobject_1;
	QPushButton		*ResetObjectButton2;

	///////////////////////////////////////////////////////////////////////
	//set up the axis control panel
	//uses the createAxisPanel() subroutine

	///////////////////////////////////////////////////////////////////////
	//set up the texture and clip control panel

	//clip options
	QCheckBox		*ClipSelect;
	QPushButton		*grabdesktopClip; 
	QPushButton		*keyimage_2;       
	QCheckBox		*KeySelect;
	QPushButton		*grabdesktopKey;
	QPushButton		*keyimage_3;
	QPushButton		*keyStatusBox;

	//video options
	QLabel			*VideoLabel;
	InputLCD		*FramesLcd;		
	InputLCD		*inFramesLcd;	
	InputLCD		*outFramesLcd;
	InputLCD		*KeyFramesLcd;	
	InputLCD		*inKeyFramesLcd;	
	InputLCD		*outKeyFramesLcd;
	QToolButton		*lockicon;

	QCheckBox		*extendHeadTail;
	QCheckBox		*toggleTheLoop;
	QCheckBox		*togglePing;

	///////////////////////////////////////////////////////////////////////
	//set up the effects control panel

	QCheckBox		*ForegroundSelect;    
	QCheckBox		*DepthSelect;
	QCheckBox		*BlurSelect;
	QSlider			*HBlurSlider, *VBlurSlider;
	InputLCD		*HBlurSlider_lcd, *VBlurSlider_lcd;

	QCheckBox		*NegativeSelect, *SwapRGBSelect, *MirrorSelect; 
	JahToolButton	*lighticon;
	QCheckBox		*light1, *reflect1, *shader1, *shader2, *smooth1, *mesh1;
	QLabel			*Transparency_1;     
	QSlider			*Trans_1;         
	InputLCD		*Translcd_1;

	//for the color qdials
    QLabel*        TranslateText_1;
  
    QDial*         TranX_1;       InputLCD*  TranXlcd_1;
    QDial*         TranY_1;       InputLCD*  TranYlcd_1;
    QDial*         TranZ_1;       InputLCD*  TranZlcd_1;

    QLabel*        TranslateText_2;

    QDial*       RotX_1;        InputLCD*  RotXlcd_1;   
    QDial*       RotY_1;        InputLCD*  RotYlcd_1;   
    QDial*       RotZ_1;        InputLCD*  RotZlcd_1;   

    QLabel*        TranslateText_3;
	  
    QDial*       ScaleX_1;      InputLCD*  ScaleXlcd_1;
    QDial*       ScaleY_1;      InputLCD*  ScaleYlcd_1;
    QDial*       ScaleZ_1;      InputLCD*  ScaleZlcd_1;


};



#endif // COLORIZE_H





