/*******************************************************************************
**
** The source file for the Jahshaka animation interface module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "colorize.h"
#include <qstylefactory.h>

void GLColorize::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) 
{

//////////////////////////////////////////////////////////////
// This is to be overridden...
    //make sure platinum is loaded
    QStyle *platinum=QStyleFactory::create("platinum");

    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");

    tooltranslate = new JahToolButton( glworldLeft, "thumbview" );
    addJahPlayerButton( tooltranslate, 0, 205, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/transtool.png", JahBasePath+"Pixmaps/desktop/transtool.png" );

    toolrotate =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( toolrotate,   0, 233, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/rotatetool.png", JahBasePath+"Pixmaps/desktop/rotatetool.png" );

    toolscale =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( toolscale,   0, 263, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/scaletool.png", JahBasePath+"Pixmaps/desktop/scaletool.png" );

    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );

    toolzoomin =   new JahToolButton( glworldRight, "zoominw" );
    addJahPlayerButton( toolzoomin,   0, 115, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomin.png", JahBasePath+"Pixmaps/desktop/zoomin.png" );

    zoom = new QSlider( glworldRight, "scrubslider" );
    if (platinum) zoom->setStyle( platinum );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( -10 );    zoom->setMaxValue( 10 );    zoom->setValue   ( 0 );

    toolzoomout =   new JahToolButton( glworldRight, "zoomout" );
    addJahPlayerButton( toolzoomout,   0, 355, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomout.png", JahBasePath+"Pixmaps/desktop/zoomout.png" );


    //connect tools to object and add object to display widget
    //note that zoom isnt hooked up yet...
    connect( tooltranslate,  SIGNAL(clicked()), this, SLOT(toolTranslate() )  );
    connect( toolscale,      SIGNAL(clicked()), this, SLOT(toolScale() )  );
    connect( toolrotate,     SIGNAL(clicked()), this, SLOT(toolRotate() )  );

    connect( zoom,  SIGNAL(sliderMoved(int)), this, SLOT(changeZoom(int) )  );

}

void GLColorize::buildInterface( QHBox* qtinterface ) 
{
	//set up translator
    JahTranslate& jt = JahTranslate::getInstance();
    JahPrefs& jprefs = JahPrefs::getInstance();

	QStyle *platinum=QStyleFactory::create("platinum");

    ///////////////////////////////////////////////////////////
    //The layers interface
    LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
    LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
    LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 140
    LayersFrame->setMaximumSize( QSize( 60, 195 ) );

    ///////////////////////////////////////////////////////////
    //The layer options
    //these are defined in the world object
    AddButton = new QPushButton( LayersFrame, "AddButton" );
    addButton( AddButton,  0, 5, 55, 25,  jt.tr("AddCFX") );
    connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

    AddGpuButton = new QPushButton( LayersFrame, "AddButton" );
    addButton( AddGpuButton,  0, 5, 55, 25,  jt.tr("AddGPU") );
    //connect  ( AddGpuButton,  SIGNAL(clicked()), this, SLOT( addGpuLayer())  );
    AddGpuButton->hide();


    NameButton = new QPushButton( LayersFrame, "NameButton" );
    addButton( NameButton, 0, 90, 55, 25 , "Name" );
    connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

    namepopup = new FancyPopup( this, jt.tr("Enter Layer Name"),250,150 );  //send size and position as well
    connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

    DelButton = new QPushButton( LayersFrame, "Delete Button" );
    addButton( DelButton, 0, 120, 55, 25 , jt.tr("Del") );
    connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

    //////////////////////////////////////////////////////////////
    // set up the scene options, or prefernces

    EffectsFrame = new QFrame(qtinterface);
    EffectsFrame->setGeometry( QRect( 0, 0, 855, 195) );

    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

    connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

    lv->setGeometry( QRect( 2, 2, 146, 180 ) );
    lv->addColumn( "LAYERS          ", -1 );
    lv->setSorting( -1,1 );   // disables the autosort

    /////////////////////////////////////////////////////////
    //Set up tabbed interface

    tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );	
    tabframe->setGeometry( QRect( 148, 1, 700, 180 ) );

    tabframe->setTabShape( QTabWidget::Triangular );
    tabframe->setAutoMask( FALSE );
    if (platinum) tabframe->setStyle( platinum );


    //this has become the settings panel need to update variable names
    moduleOptionsUI[0] = new QWidget( EffectsFrame, "encoderUI0" ); //scenecontrols
    tabframe->insertTab( moduleOptionsUI[0], tr( "  "+jt.tr("SCENE")+"  " ) );

    moduleOptionsUI[1] = new QWidget( EffectsFrame, "axiscontrols" ); //axiscontrols
    tabframe->insertTab( moduleOptionsUI[1], tr( "  "+jt.tr("AXIS")+"  " ) );

    moduleOptionsUI[2] = new QWidget( EffectsFrame, "texturecontrols" ); //texturecontrols
    tabframe->insertTab( moduleOptionsUI[2], tr( "    "+jt.tr("CLIP")+"    " ) );

    //this has become the settings panel need to update variable names
    moduleOptionsUI[3] = new QWidget( EffectsFrame, "keysettings" ); //keycontrols
    tabframe->insertTab( moduleOptionsUI[3], tr( "  "+jt.tr("COLOR")+"  " ) );

    //this has become the settings panel need to update variable names
    moduleOptionsUI[4] = new QWidget( EffectsFrame, "keysettings" ); //keycontrols
    tabframe->insertTab( moduleOptionsUI[4], tr( "  "+jt.tr("KEYFRAMES")+"  " ) );

    //this raises the timeline above all other objects
    tabframe->setCurrentPage( 3 );

    /////////////////////////////////////////////////////////////////////
    // the layout boxes for different types of layers
    // the object control tab bar
    // this is the object control panel


    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //mover this into the layers...
    ControlFrameUI[0][0] = new QFrame( moduleOptionsUI[0], "ControlFrame3" ); //scenecontrols
    ControlFrameUI[0][0]->setGeometry( QRect( 0, 0, 660, 140 ) );

    LoadButton = new QPushButton( ControlFrameUI[0][0], "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , jt.tr("Load") );
    connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( ControlFrameUI[0][0], "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, jt.tr("Save") );
    connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( ControlFrameUI[0][0], "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, jt.tr("Reset All") );
    connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( ControlFrameUI[0][0], "ClearButton" );
    addButton( ClearButton, 10, 115, 86, 25, jt.tr("Clear Scene") );
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  );

    VideoRenderSelect = new QCheckBox( ControlFrameUI[0][0], "SmoothSelect" );
    addCheckButton( VideoRenderSelect, 200, 10, 210, 21, jt.tr("render using source res") );
    connect( VideoRenderSelect,  SIGNAL(clicked()), this, SLOT(SetVideoRender())  );

    //GpuSelect = new QCheckBox( ControlFrameUI[0][0], "GpuSelect" );
    //addCheckButton( GpuSelect, 200, 40, 210, 21, jt.tr("activate gpu effects") );
    //connect( GpuSelect,  SIGNAL(clicked()), this, SLOT(SetGpuSelect())  );

    //mesh1 = new QCheckBox( ControlFrameUI[0][0], "light1" );
    //mesh1->setGeometry( QRect( 200, 90, 210, 21 ) );
    //mesh1->setText( jt.tr("toggle mesh") );
    //connect( mesh1,  SIGNAL(clicked()), this, SLOT(toggleMesh())  );


    jahreslabel = new QLabel( ControlFrameUI[0][0], "languagelabel" );
    jahreslabel->setGeometry( QRect( 450, 75, 120, 21 ) );
    jahreslabel->setAlignment( int( QLabel::AlignCenter ) );
    jahreslabel->setText( jt.tr( "Resolution" ) );

    JahresCombo = new QComboBox( FALSE, ControlFrameUI[0][0], "JahresCombo" );
    JahresCombo->setGeometry( QRect( 450, 100, 170, 21 ) );
	hasResMenu = true;

	projectData thedata;
	thedata.buildComboBox(JahresCombo);
    int defaultRes = jprefs.getJahResolution();
	JahresCombo->setCurrentItem(defaultRes); //should be set to appdefault
    connect( JahresCombo,  SIGNAL(activated(int)), this, SLOT(setResolution(int))  );

    //need to move this into the module
    //this is mandatory for all modules...

    sframetext = new QLabel( ControlFrameUI[0][0], "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( jt.tr("STARTFRAME") ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( ControlFrameUI[0][0], "startframe",9 );
    addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
    lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
    connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( ControlFrameUI[0][0], "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( jt.tr("ENDFRAME")  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

    lcdendframe = new InputLCD( ControlFrameUI[0][0], "endframe",9 );
    addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
    lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
    connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );


    ///////////////////////////////////////////////////////////////
    // object layer options sub control set

    ControlFrameUI[1][0] = new QFrame( moduleOptionsUI[1], "ControlFrame1" ); //ControlFrame1
    ControlFrameUI[1][0]->setGeometry( QRect( 0, 0, 760, 120 ) );    // was 510

    createAxisPanel ( ControlFrameUI[1][0] );

    //add the placeholders for plugins
    JahModuleOption[0] = new QCheckBox( ControlFrameUI[1][0], "Placeholder" );
    addCheckButton( JahModuleOption[0], 520, 30, 145, 25, "Placeholder1" );
    //connect( JahModuleOption[0],  SIGNAL(clicked()), this, SLOT(updateEffects())  ); //for clip

    //increment the ui uption counter...
    numUiOptions +=1;

    //add the placeholders for plugins
    JahModuleOption[1] = new QCheckBox( ControlFrameUI[1][0], "Placeholder" );
    addCheckButton( JahModuleOption[1], 520, 60, 145, 25, "Placeholder2" );
    //connect( JahModuleOption[1],  SIGNAL(clicked()), this, SLOT(updateEffects())  ); //for clip

    //increment the ui uption counter...
    numUiOptions +=1;

    //add the placeholders for plugins
    JahModuleOption[2] = new QCheckBox( ControlFrameUI[1][0], "Placeholder" );
    addCheckButton( JahModuleOption[2], 520, 90, 145, 25, "Placeholder3" );
    //connect( JahModuleOption[2],  SIGNAL(clicked()), this, SLOT(updateEffects())  ); //for clip

    //increment the ui uption counter...
    numUiOptions +=1;

    //add the placeholders for plugins
    JahModuleOption[3] = new QCheckBox( ControlFrameUI[1][0], "Placeholder" );
    addCheckButton( JahModuleOption[3], 520, 120, 145, 25, "Placeholder4" );
    //connect( JahModuleOption[3],  SIGNAL(clicked()), this, SLOT(updateEffects())  ); //for clip

    //increment the ui uption counter...
    numUiOptions +=1;

    hasButtons = true;


    ///////////////////////////////////////////////////////////////
    // texture layer options
    // need to add video-texture options here
    //image options

    ControlFrameUI[2][0] = new QFrame( moduleOptionsUI[2], "ControlFrame2" ); //was ControlFrame2
    ControlFrameUI[2][0]->setGeometry( QRect( 0, 0, 660, 150 ) );

    ClipSelect = new QCheckBox( ControlFrameUI[2][0], "ClipSelect" );
    addCheckButton( ClipSelect, 0, 30, 45, 25, jt.tr("Clip") );
    connect( ClipSelect,  SIGNAL(clicked()), this, SLOT(setClipStatus())  ); //for clip

    grabdesktopClip = new QPushButton( ControlFrameUI[2][0], "grabdesktop" );
    addButton( grabdesktopClip, 45, 30, 110, 25, jt.tr("Grab Desktop") );
    connect( grabdesktopClip,  SIGNAL(clicked()), this, SLOT(grabDesktop())  );

    keyimage_2 = new QPushButton( ControlFrameUI[2][0], "clipmapping" );
    addButton( keyimage_2, 165, 30, 35, 25, "IM" );
    //connect( keyimage_2,  SIGNAL(clicked()), this, SLOT(keyCliplayer())  );


    KeySelect = new QCheckBox( ControlFrameUI[2][0], "KeySelect" );
    addCheckButton( KeySelect, 0, 75, 45, 25, "Key" );
    connect( KeySelect,  SIGNAL(clicked()), this, SLOT(setKeyStatus())  );

    grabdesktopKey = new QPushButton( ControlFrameUI[2][0], "grabdesktop" );
    addButton( grabdesktopKey, 45, 75, 110, 25, jt.tr("Grab Desktop") );
    //connect( grabdesktopKey,  SIGNAL(clicked()), this, SLOT(grabDesktopKey())  ); //for key...

    keyimage_3 = new QPushButton( ControlFrameUI[2][0], "keyOne_3" );
    addButton( keyimage_3, 165, 75, 35, 25, "KM" );
    //connect( keyimage_3,  SIGNAL(clicked()), this, SLOT(keyClipkey())  );

    keyStatusBox = new QPushButton( ControlFrameUI[2][0], "Select1" );
    addButton( keyStatusBox, 220, 75, 35, 25, jt.tr("Invert") );
    connect( keyStatusBox,  SIGNAL(clicked()), this, SLOT(InvertKeyData())  );

    // video options
    // need to hook this up now...

    VideoLabel = new QLabel( ControlFrameUI[2][0], "TranslateText_1" );
    addLabel( VideoLabel, 380, 10, 110, 21, "VIDEO OPTIONS", 9);


    FramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLabelLcd( ControlFrameUI[2][0], FramesLcd,       380, 30, 60, 21,  0,  "Frames", 8);
    FramesLcd->setActive	(false);

    inFramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLcdLabel( ControlFrameUI[2][0], inFramesLcd,     460, 30, 30, 21,  0,  "In", 8);
    connect(inFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinFrames(int)) );

    outFramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLcdLabel( ControlFrameUI[2][0], outFramesLcd,    530, 30, 30, 21,  0,  "Out", 8);
    connect(outFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutFrames(int)) );

    KeyFramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLabelLcd( ControlFrameUI[2][0], KeyFramesLcd,    380, 75, 60, 21,  0,  "Frames", 8);

    inKeyFramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLcdLabel( ControlFrameUI[2][0], inKeyFramesLcd,  460, 75, 30, 21,  0,  "In", 8);
    connect(inKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinKeyFrames(int)) );

    outKeyFramesLcd = new InputLCD( ControlFrameUI[2][0] );
    addLcdLabel( ControlFrameUI[2][0], outKeyFramesLcd, 530, 75, 30, 21,  0,  "Out", 8);
    connect(outKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutKeyFrames(int)) );

    extendHeadTail = new QCheckBox( ControlFrameUI[2][0], "extendHeadTail" );
    addCheckButton( extendHeadTail, 380, 95, 100, 25, "Extend Head-Tail" );
    connect( extendHeadTail,  SIGNAL(clicked()), this, SLOT(toggleExtendHeadTail())  );

    toggleTheLoop = new QCheckBox( ControlFrameUI[2][0], "toggleTheLoop" );
    addCheckButton( toggleTheLoop, 480, 95, 100, 25, "Loop" );
    connect( toggleTheLoop,  SIGNAL(clicked()), this, SLOT(toggleLoop())  );

    togglePing = new QCheckBox( ControlFrameUI[2][0], "togglePing" );
    addCheckButton( togglePing, 480, 130, 100, 25, "Ping Pong" );
    connect( togglePing,  SIGNAL(clicked()), this, SLOT(togglePingPong())  );

    //what is this used for?
    //need to make the icon a lock... 3x current height as well for links
    lockicon	= new QToolButton(ControlFrameUI[2][0],"lockicon");
    lockicon->setGeometry( QRect( 600, 55, 18, 26 ) );
    lockicon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
    lockicon->setAutoRaise(true);

    ///////////////////////////////////////////////////////////////
    // texture layer options
    // need to add video-texture options here
    //image options

    ControlFrameUI[3][0] = new QFrame( moduleOptionsUI[3], "ControlFrame2" ); //was ControlFrame2
    ControlFrameUI[3][0]->setGeometry( QRect( 0, 0, 660, 150 ) );

    createColorPanel ( ControlFrameUI[3][0] );

}






void GLColorize::createColorPanel ( QFrame* theuiframe)
{
    JahTranslate& jt = JahTranslate::getInstance();

    // set up color translation sliders
    TranslateText_1 = new QLabel( theuiframe, "TranslateText_1" );
    addLabel( TranslateText_1, 100, 10, 110, 21, jt.tr("LUMINANCE"), 9);

    TranX_1 = new QDial( theuiframe );   TranXlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, TranX_1, 0, 30, 70, 70,  100, TranXlcd_1,  "R", 9);
    TranX_1->setWrapping(true);

    connect( TranX_1,   SIGNAL(dialMoved(int)), this, SLOT(setXTrans(int)) );
    connect( TranX_1,   SIGNAL(dialMoved(int)), TranXlcd_1, SLOT(display(int)) );
    connect( TranXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    TranY_1 = new QDial( theuiframe );   TranYlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, TranY_1, 90, 30, 70, 70,  100, TranYlcd_1,  "G", 9);
    TranY_1->setWrapping(true);

    connect( TranY_1, SIGNAL(dialMoved(int)), this, SLOT(setYTrans(int)) );
    connect( TranY_1, SIGNAL(dialMoved(int)), TranYlcd_1, SLOT(display(int)) );
    connect( TranYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    TranZ_1 = new QDial( theuiframe );   TranZlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, TranZ_1, 180, 30, 70, 70,  100, TranZlcd_1,  "B", 9);
    TranZ_1->setWrapping(true);

    connect( TranZ_1,   SIGNAL(dialMoved(int)), this, SLOT(setZTrans(int)) );
    connect( TranZ_1,   SIGNAL(dialMoved(int)), TranZlcd_1, SLOT(display(int)) );
    connect(TranZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );

    //set up object control set and sliders

    TranslateText_2 = new QLabel( theuiframe, "TranslateText_1" );
    addLabel( TranslateText_2, 390, 10, 110, 21, jt.tr("BRIGHTNESS"), 9);

    RotX_1 = new QDial( theuiframe );   RotXlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, RotX_1, 290, 30, 70, 70,  100, RotXlcd_1,  "R", 9);

    connect( RotX_1,   SIGNAL(dialMoved(int)), this, SLOT(setXRotation(int)) );
    connect( RotX_1,   SIGNAL(dialMoved(int)), RotXlcd_1, SLOT(display(int)) );
    connect( RotXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );

    RotY_1 = new QDial( theuiframe );   RotYlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, RotY_1, 380, 30, 70, 70,  100, RotYlcd_1,  "G", 9);

    connect( RotY_1, SIGNAL(dialMoved(int)), this, SLOT(setYRotation(int)) );
    connect( RotY_1, SIGNAL(dialMoved(int)), RotYlcd_1, SLOT(display(int)) );
    connect( RotYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );


    RotZ_1 = new QDial( theuiframe );   RotZlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, RotZ_1, 470, 30, 70, 70,  100, RotZlcd_1,  "B", 9);

    connect( RotZ_1,    SIGNAL(dialMoved(int)), this, SLOT(setZRotation(int)) );
    connect( RotZ_1,    SIGNAL(dialMoved(int)), RotZlcd_1, SLOT(display(int)) );
    connect( RotZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );

    TranslateText_3 = new QLabel( theuiframe, "TranslateText_1" );
    addLabel( TranslateText_3, 560, 10, 110, 21, jt.tr("SATURATION"), 9);

    ScaleX_1 = new QDial( theuiframe );   ScaleXlcd_1 = new InputLCD( theuiframe );
    addDialLcdLabel( theuiframe, ScaleX_1, 560, 30, 70, 70,  100, ScaleXlcd_1,  "S", 9);

    connect( ScaleX_1,    SIGNAL(dialMoved(int)), this, SLOT(setXScale(int)) );
    connect( ScaleX_1,    SIGNAL(dialMoved(int)), ScaleXlcd_1, SLOT(display(int)) );
    connect( ScaleXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );

}

void GLColorize::createAxisPanel ( QFrame* theuiframe)
{
    //set some flags
    hasLabels = true;
    hasSliders = true;
    JahTranslate& jt = JahTranslate::getInstance();

    //////////////////////////////////////////////////////////////////////////////////////////
    // set up rotate sliders

    JahModuleLabel[0] = new QLabel( theuiframe, "RotateText_1" );
    addLabel( JahModuleLabel[0], 10, 10, 110, 21, jt.tr("ROTATE"), 9);

    JahSliders[0] = new QSlider( theuiframe );
    JahSlidersLCD[0] = new InputLCD( theuiframe );
    JahSlidersLabel[0] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[0], 0, 30, 10, 21,  500, JahSlidersLCD[0], JahSlidersLabel[0], "X", 9);
    connect( JahSliders[0], SIGNAL(sliderMoved(int)), this, SLOT(setXRotation(int)) );
    connect( JahSliders[0], SIGNAL(sliderMoved(int)), JahSlidersLCD[0], SLOT(display(int)) );
    connect(JahSlidersLCD[0], SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );

    JahSliders[1] = new QSlider( theuiframe );
    JahSlidersLCD[1] = new InputLCD( theuiframe );
    JahSlidersLabel[1] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[1], 0, 60, 10, 21,  500, JahSlidersLCD[1], JahSlidersLabel[1], "Y", 9);
    connect( JahSliders[1], SIGNAL(sliderMoved(int)), this, SLOT(setYRotation(int)) );
    connect( JahSliders[1], SIGNAL(sliderMoved(int)), JahSlidersLCD[1], SLOT(display(int)) );
    connect(JahSlidersLCD[1], SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );

    JahSliders[2] = new QSlider( theuiframe );
    JahSlidersLCD[2] = new InputLCD( theuiframe );
    JahSlidersLabel[2] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[2], 0, 90, 10, 21,  500, JahSlidersLCD[2], JahSlidersLabel[2], "Z", 9);
    connect( JahSliders[2], SIGNAL(sliderMoved(int)), this, SLOT(setZRotation(int)) );
    connect( JahSliders[2], SIGNAL(sliderMoved(int)), JahSlidersLCD[2], SLOT(display(int)) );
    connect(JahSlidersLCD[2], SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );


    //////////////////////////////////////////////////////////////////////////////////////////
    // set up translate sliders

    JahModuleLabel[1] = new QLabel( theuiframe, "TranslateText_1" );
    addLabel( JahModuleLabel[1], 180, 10, 110, 21, jt.tr("TRANSLATE"), 9);

    JahSliders[3] = new QSlider( theuiframe );
    JahSlidersLCD[3] = new InputLCD( theuiframe );
    JahSlidersLabel[3] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[3], 180, 30, 10, 21,  500, JahSlidersLCD[3], JahSlidersLabel[3], "X", 9);
    connect( JahSliders[3],   SIGNAL(sliderMoved(int)), this, SLOT(setXTrans(int)) );
    connect( JahSliders[3],   SIGNAL(sliderMoved(int)), JahSlidersLCD[3], SLOT(display(int)) );
    connect(JahSlidersLCD[3], SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    JahSliders[4] = new QSlider( theuiframe );
    JahSlidersLCD[4] = new InputLCD( theuiframe );
    JahSlidersLabel[4] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[4], 180, 60, 10, 21,  500, JahSlidersLCD[4], JahSlidersLabel[4], "Y", 9);
    connect( JahSliders[4], SIGNAL(sliderMoved(int)), this, SLOT(setYTrans(int)) );
    connect( JahSliders[4], SIGNAL(sliderMoved(int)), JahSlidersLCD[4], SLOT(display(int)) );
    connect(JahSlidersLCD[4], SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    JahSliders[5] = new QSlider( theuiframe );
    JahSlidersLCD[5] = new InputLCD( theuiframe );
    JahSlidersLabel[5] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[5], 180, 90, 10, 21,  500, JahSlidersLCD[5],  JahSlidersLabel[5], "Z", 9);
    connect( JahSliders[5],   SIGNAL(sliderMoved(int)), this, SLOT(setZTrans(int)) );
    connect( JahSliders[5],   SIGNAL(sliderMoved(int)), JahSlidersLCD[5], SLOT(display(int)) );
    connect(JahSlidersLCD[5], SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );


    //////////////////////////////////////////////////////////////////////////////////////////
    // set up scale sliders

    JahModuleLabel[2] = new QLabel( theuiframe, "ScaleText_1" );
    addLabel( JahModuleLabel[2], 350, 10, 110, 21, jt.tr("SCALE"), 9);

    JahSliders[6] = new QSlider( theuiframe );
    JahSlidersLCD[6] = new InputLCD( theuiframe );
    JahSlidersLabel[6] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[6], 350, 30, 10, 21,  500, JahSlidersLCD[6], JahSlidersLabel[6],  "X", 9);
    connect( JahSliders[6], SIGNAL(sliderMoved(int)), this, SLOT(setXScale(int)) );
    connect( JahSliders[6], SIGNAL(sliderMoved(int)), JahSlidersLCD[6], SLOT(display(int)) );
    connect(JahSlidersLCD[6], SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );

    JahSliders[7] = new QSlider( theuiframe );
    JahSlidersLCD[7] = new InputLCD( theuiframe );
    JahSlidersLabel[7] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[7], 350, 60, 10, 21,  500, JahSlidersLCD[7], JahSlidersLabel[7], "Y", 9);
    connect( JahSliders[7], SIGNAL(sliderMoved(int)), this, SLOT(setYScale(int)) );
    connect( JahSliders[7], SIGNAL(sliderMoved(int)), JahSlidersLCD[7], SLOT(display(int)) );
    connect(JahSlidersLCD[7], SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );

    JahSliders[8] = new QSlider( theuiframe );
    JahSlidersLCD[8] = new InputLCD( theuiframe );
    JahSlidersLabel[8] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[8], 350, 90, 10, 21,  500, JahSlidersLCD[8], JahSlidersLabel[8], "Z", 9);
    connect( JahSliders[8], SIGNAL(sliderMoved(int)), this, SLOT(setZScale(int)) );
    connect( JahSliders[8], SIGNAL(sliderMoved(int)), JahSlidersLCD[8], SLOT(display(int)) );
    connect(JahSlidersLCD[8], SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );

}




















