/*******************************************************************************
**
** The source file for the Jahshaka effects objects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "colorize.h"

void GLColorize::doRtColors(int layer) 
{

    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "doing rt colors");
    
    //raise the color matrix
    glMatrixMode(GL_COLOR);
    glLoadIdentity();

    // Transform the luminance colors:
    GLfloat lumx = ((GLfloat)layerlist->at(layer)->thelayer->node->tx+100) / 100;
    GLfloat lumy = ((GLfloat)layerlist->at(layer)->thelayer->node->ty+100) / 100;
    GLfloat lumz = ((GLfloat)layerlist->at(layer)->thelayer->node->tz+100) / 100;

    glScalef(lumx,lumy,lumz);   // Scale r*=0.5, g*=2.0, b*=1.0

    // Transform the brightness colors:
    GLfloat brightx = ((GLfloat)layerlist->at(layer)->thelayer->node->rx) / 100;
    GLfloat brighty = ((GLfloat)layerlist->at(layer)->thelayer->node->ry) / 100;
    GLfloat brightz = ((GLfloat)layerlist->at(layer)->thelayer->node->rz) / 100;

    glTranslatef(brightx,brighty,brightz);   // Scale r*=0.5, g*=2.0, b*=1.0
     

    // Transform the saturation colors:
    
    GLfloat saturation = ((GLfloat)layerlist->at(layer)->thelayer->node->sx);
    
    GLfloat matrix[16];

    matrix[0]  = (1.0 - saturation ) * 0.3086 + saturation;
    matrix[1]  = (1.0 - saturation ) * 0.3086 ;
    matrix[2]  = (1.0 - saturation ) * 0.3086 ;
    matrix[3]  = 0.0 ;
    matrix[4]  = (1.0 - saturation ) * 0.6094 ;
    matrix[5]  = (1.0 - saturation ) * 0.6094 + saturation;
    matrix[6]  = (1.0 - saturation ) * 0.6094 ;
    matrix[7]  = 0.0 ;
    matrix[8]  = (1.0 - saturation ) * 0.0820 ;
    matrix[9]  = (1.0 - saturation ) * 0.0820 ;
    matrix[10] = (1.0 - saturation ) * 0.0820 + saturation;
    matrix[12] = 0.0 ;
    matrix[13] = 0.0 ;
    matrix[14] = 0.0 ;
    matrix[15] = 1.0 ;

    glMultMatrixf(matrix);
    
    glMatrixMode(GL_MODELVIEW);

}

void GLColorize::doColorMatrix(int layer) 
{

    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "doing luminance matrix");
    
    glMatrixMode(GL_COLOR);
    glLoadIdentity();
    
  
  GLfloat lumx = ((GLfloat)layerlist->at(layer)->thelayer->node->tx+100) / 100;
  GLfloat lumy = ((GLfloat)layerlist->at(layer)->thelayer->node->ty+100) / 100;
  GLfloat lumz = ((GLfloat)layerlist->at(layer)->thelayer->node->tz+100) / 100;
  
  GLfloat rgb2luminance[16] =
  {
    lumx, lumx, lumx, lumx,  // 30% red.
    lumy, lumy, lumy, lumy,  // 30% red.
    lumz, lumz, lumz, lumz,  // 30% red.
    0.00, 0.00, 0.00, 0.00  // 0% alpha. 
  };
  

  /* RGB to NTSC luminance color conversion matrix. */ 
  /*
  static GLfloat rgb2luminance[16] =
  {
    0.30, 0.30, 0.30, 0.30,  // 30% red.
    0.59, 0.59, 0.59, 0.59,  // 59% green. 
    0.11, 0.11, 0.11, 0.11,  // 11% blue. 
    0.00, 0.00, 0.00, 0.00  // 0% alpha. 
  };
*/
    
  glLoadMatrixf(rgb2luminance);

  glMatrixMode(GL_MODELVIEW);

}

void GLColorize::doSwapRGB(QImage& theimage)	
{

    theimage = theimage.swapRGB();
    
}



void GLColorize::doNegative(QImage& theimage) 
{

    theimage.invertPixels();

}