/*******************************************************************************
**
** The source file for the Jahshaka editing module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "edit.h"
//#include <jahtimeline.h>

void GLEdit::initializeObjects() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLEdit Class");
    
	//initialize the timeline
    theeditdesktop = new EditDesktop( this, moduleOptionsUI[5], "editdesk" );
    theeditdesktop->setFixedSize( QSize( 980, 600 ) );    //was 955,486

    //make the current opengl context current so the textures can be set up properly
    this->makeCurrent();

    //this creates the default world layer using the linked list
    layerlist->append( new NewLayerList(0, "World") );

    //setup and build the default layer these should all be defaults
    layerlist->at(0)->thelayer->blend = FALSE;
    layerlist->at(0)->thelayer->depth = FALSE;
    layerlist->at(0)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(0)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer"; 
    layerlist->at(0)->thelayer->asset.extension = ".bmp";
    buildLayer(0,"Background", LAYER, BACKDROP );
  
    addSelection(FALSE);   // adds backgroundimage as 'off' uses default numberLayers = 1

    objectMenu(numberLayers); updateSliders();

    //turn forceplay on here
    setForcePlay(true);

}

///////////////////////////////////////////////////////////////////
// the main opengl routine that iterates through all objects
// edit module has its own paintGL so we can override the behaviour

void GLEdit::paintGL()
{
    //checks to see if the play button is on and increments frames
    if ( animation )
    {  
        updateanimation(currentFrame); currentFrame++;
        if ( currentFrame > maxFrame ) currentFrame=1;
    }

    //turn on hardware aliasing for the scene if it is enabled

        if (hwaliasing)
        {
            glEnable(GL_MULTISAMPLE_ARB);
        }

    //clear the scene and set up matrices
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();


    //draw black video safe area if no layer present
    if (!select) 
    { 
        if ( isLayerVisible(0) )     
        {  
            renderSpace->drawLayer(*(layerlist->at(0)->thelayer));  
        }
        else 
        {   
            renderSpace->drawLayer(*titlesafe); 
        }
    }

    //we position the camera here
    renderSpace->positionObject(camera->node);

    //draw the grid if its visible
    if (gridval) {  renderSpace->drawLayer(*thegrid); }

    //set up opengl layer selection
    glInitNames(); 
    glPushName(0);                      // initialize the selection stack

    /////////////////////////////////////////////////
    //the main loop where we draw all visible layers
    //int dlayer;

    //sort the layers before drawing them
	//probably not necessary for editing mode since the layers
	//are painted in order
	//but maybe for when we have 2 layers on the same timeline?
    newsortLayers();

    //now we iterate the list... but in reverese
    //for ( uint i = 1; i <= layerlist->count(); ++i )
	/*
    for ( uint i = layerlist->count(); i >= 1; --i )
    {
        if ( layerlist->at(i) )
        {
            if (numberLayers>1 && newworldorder->at(i-1))
            {
                dlayer = newworldorder->at(i-1)->laynum;
            }
            else
            {
                dlayer = i;
            }

            if ( (isLayerVisible(dlayer)) && (layerlist->at(dlayer)->thelayer->foreground !=1 ))
            {
                //build layer effects first  
                if (layerlist->at(dlayer)->thelayer->layerEffect)
                {
                    buildLayerEffect(dlayer);
                }
				debug("de ya 1");

                //then draw the layer
                renderSpace->drawLayer(*(layerlist->at(dlayer)->thelayer));

                //then draw the keyframes if visible
                if (layerlist->at(dlayer)->thelayer->drawKeyframes)
                {
                    renderSpace->drawNodes(*(layerlist->at(dlayer)->thelayer));
                }
            }
        }
    }

*/
    glPopMatrix();

    /////////////////////////////////////////////////////////////////////////////
    //draw the forground layers ie objects not affeceted by worldspace or sorting
    
    if (numberLayers>0) 
    {

        //for ( uint i = 1; i <= layerlist->count(); ++i )
		for ( uint i = layerlist->count(); i >= 1; --i )
        //for (int i=1; i<=numberLayers; i++) 
        {
        glPushMatrix();
        if ( layerlist->at(i) ) 
        {
            if (isLayerVisible(i) && layerlist->at(i)->thelayer->foreground==1) 
            {
				debug("de ya 2");
                //build layer effects first  
                //if (layerlist->at(dlayer)->thelayer->layerEffect)
                //{
                //    buildLayerEffect(dlayer);
                //}

                renderSpace->drawLayer(*(layerlist->at(i)->thelayer));
                if(layerlist->at(i)->thelayer->drawKeyframes) 
                { 
                    renderSpace->drawNodes(*(layerlist->at(i)->thelayer)); 
                }
            }
        }
        glPopMatrix();
        }
    }

    //if textsafe is on draw it as well
    if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

    if (hwaliasing)
    {
        glDisable(GL_MULTISAMPLE_ARB);
    }

    //flush render que
    glFlush(); 

    /////////////////////////////////////////////////
    // finished the scene now Wait this many msecs before redraw
    if ( animation ) 
    {
        timer->start( redrawWait, TRUE );
    } 

}


void GLEdit::updateUiSettings()
{
    ////////////////////////////////////////////////////////////////////////////////////
    // all other ui settings
    //for transparency
    Trans_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );
    Translcd_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );


    //update video frames here
    FramesLcd->setValue   ( layerlist->at(activeLayer)->thelayer->asset.getNumFrames() );
    inFramesLcd->setValue ( layerlist->at(activeLayer)->thelayer->inframe );
    outFramesLcd->setValue( layerlist->at(activeLayer)->thelayer->outframe );

    KeyFramesLcd->setValue   ( layerlist->at(activeLayer)->thelayer->keyasset.getNumFrames() );
    inKeyFramesLcd->setValue ( layerlist->at(activeLayer)->thelayer->keyinframe );
    outKeyFramesLcd->setValue( layerlist->at(activeLayer)->thelayer->keyoutframe );

    extendHeadTail->setChecked ( layerlist->at(activeLayer)->thelayer->showEdges );
    toggleTheLoop->setChecked ( layerlist->at(activeLayer)->thelayer->loop );
    togglePing->setChecked ( layerlist->at(activeLayer)->thelayer->ping );

    //extended values here need to add everything new
    GridSelect->setChecked(gridval);
    TextSelect->setChecked(textsafe->layerStatus);

    light1->setChecked(layerlist->at(activeLayer)->thelayer->lighting);
    ClipSelect->setChecked(layerlist->at(activeLayer)->thelayer->drawtheLayer);
    KeySelect->setChecked(layerlist->at(activeLayer)->thelayer->drawtheKey);
    ForegroundSelect->setChecked(layerlist->at(activeLayer)->thelayer->foreground);

    //for the effects panel
    BlurSelect->setChecked(layerlist->at(activeLayer)->thelayer->blur);
    NegativeSelect->setChecked(layerlist->at(activeLayer)->thelayer->negative);
    SwapRGBSelect->setChecked(layerlist->at(activeLayer)->thelayer->swaprgb);
    MirrorSelect->setChecked(layerlist->at(activeLayer)->thelayer->mirror);

    reflect1->setChecked(layerlist->at(activeLayer)->thelayer->reflect);
    smooth1->setChecked(layerlist->at(activeLayer)->thelayer->smooth);
    shader1->setChecked(layerlist->at(activeLayer)->thelayer->pixelshader);
    shader2->setChecked(layerlist->at(activeLayer)->thelayer->pixelshader);

}



void GLEdit::clicked( QListViewItem* items )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLAnime Class");

    //nothing added yet
    if (!items) return;

    QString traceout,rasta;
    int re=0;
    int ra=0;

    //this is slow but its a easy way to
    //go through all layers to find this one
    QListViewItemIterator it( lv );
    while ( it.current() )
    {
        QListViewItem *item = it.current();
        ++it;

        if ( item->isSelected() ) 
        {
            jtrace.debug("selected"+rasta.setNum(re) );
            ra = re;
        }
        re++;
    }
       
    //re is total layers
    //ra is selection
    int finallayer;

	//usually we reverese the selections
	//for editing its flipped...

    //finallayer = (re - ra)-1;
    finallayer = ra;
    jtrace.debug("selected layer"+traceout.setNum(finallayer) );
    
    //this raises the control panel when you click on the layer
    //if ( controlpanel->id (controlpanel->visibleWidget() )==0 )
    //    controlpanel->raiseWidget(1);

    QString laynum =  items->text( 1 );

    //checks if the item is checked or not
    int itemstatus;
    if (   ( ( QCheckListItem* )items )->isOn()==1   ) { itemstatus=1; }
    if (   ( ( QCheckListItem* )items )->isOn()==0   ) { itemstatus=0; }

    //int pp = laynum.toInt();
    int pp = finallayer;

    //QString ras;
    //debug("clicked layer:"+ras.setNum(pp));
    
    //brings up the menu for the selected layer according to layer number
    objectMenu(pp);

    //we waste a interupt here, always checking the clicked item
    //would be better with a signal slot implementation
    int layerStat = layerlist->at(activeLayer)->thelayer->layerStatus;
    
    if      ( itemstatus==1  && layerStat==0 ) 
    { 
        setLayerVisible(pp, TRUE); 
        //layerlist->at(activeLayer)->thelayer->printInfo(); 
    }
    else if ( itemstatus==0  && layerStat==1 ) 
    { 
        setLayerVisible(pp, FALSE); 
    }
    else
    {      
        updateGL();
    }  
    
    //clicked on layer, update keyframes
    //should only do this if the layer has changed...
    //updateNodeDisplay(); //???
     
    //now update the sliders
    //do we need a activeLayer = pp here?
    updateSliders();
    
}


















