/*******************************************************************************
**
** The source file for the Jahshaka editing interface module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "edit.h"
#include <qstylefactory.h>

void GLEdit::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) 
{
    QStyle *platinum=QStyleFactory::create("platinum");

    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");

    tooltranslate = new JahToolButton( glworldLeft, "thumbview" );
    addJahPlayerButton( tooltranslate, 0, 205, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/transtool.png", JahBasePath+"Pixmaps/desktop/transtool.png" );

    toolrotate =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( toolrotate,   0, 233, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/rotatetool.png", JahBasePath+"Pixmaps/desktop/rotatetool.png" );

    toolscale =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( toolscale,   0, 263, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/scaletool.png", JahBasePath+"Pixmaps/desktop/scaletool.png" );

    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );

    toolzoomin =   new JahToolButton( glworldRight, "zoomin" );
    addJahPlayerButton( toolzoomin,   0, 115, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomin.png", JahBasePath+"Pixmaps/desktop/zoomin.png" );

    zoom = new QSlider( glworldRight, "scrubslider" );
    if (platinum) zoom->setStyle( platinum );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( -10 );    zoom->setMaxValue( 10 );    zoom->setValue   ( 0 );

    toolzoomout =   new JahToolButton( glworldRight, "zoomout" );
    addJahPlayerButton( toolzoomout,   0, 355, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomout.png", JahBasePath+"Pixmaps/desktop/zoomout.png" );

    //connect tools to object and add object to display widget
    connect( tooltranslate,  SIGNAL(clicked()), this, SLOT(toolTranslate() )  );
    connect( toolscale,      SIGNAL(clicked()), this, SLOT(toolScale() )  );
    connect( toolrotate,     SIGNAL(clicked()), this, SLOT(toolRotate() )  );

    connect( toolzoomin,    SIGNAL(clicked()), this, SLOT(resetZoom() )  );
    connect( toolzoomout,    SIGNAL(clicked()), this, SLOT(resetZoom() )  );

	//need to ifdef the videohead for this to be set up
    //added for videohead
    playerphoto =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( playerphoto,   0, 301, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/phototool.png", JahBasePath+"Pixmaps/desktop/phototool.png" );
    connect( playerphoto,    SIGNAL(clicked()), this, SLOT(SoftRender() )  );

    connect( zoom,  SIGNAL(sliderMoved(int)), this, SLOT(changeZoom(int) )  );

}


void GLEdit::buildInterface( QHBox* qtinterface ) 
{
	//set up translator
    JahTranslate& jt = JahTranslate::getInstance();
    JahPrefs& jprefs = JahPrefs::getInstance();

	QStyle *platinum=QStyleFactory::create("platinum");

	///////////////////////////////////////////////////////////
	//The layers interface
	LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
	LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
	LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 140
	LayersFrame->setMaximumSize( QSize( 60, 195 ) );

	///////////////////////////////////////////////////////////
	//The layer options
	//these are defined in the world object
	AddButton = new QPushButton( LayersFrame, "AddButton" );
	addButton( AddButton,  0, 5, 55, 25,  jt.tr("Add") );
	connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

	NameButton = new QPushButton( LayersFrame, "NameButton" );
	addButton( NameButton, 0, 30, 55, 25, jt.tr("Name") );
	connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

	namepopup = new FancyPopup( this, "Enter Layer Name",250,150 );  //send size and position as well
	connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

	DelButton = new QPushButton( LayersFrame, "Delete Button" );
	addButton( DelButton, 0, 60, 55, 25 , jt.tr("Del") );
	connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

	//////////////////////////////////////////////////////////////
	// set up the scene options, or prefernces
	
	EffectsFrame = new QFrame(qtinterface);
	EffectsFrame->setGeometry( QRect( 0, 0, 855, 195) );

    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

    connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

    lv->setGeometry( QRect( 2, 2, 146, 180 ) );
    lv->addColumn( "LAYERS          ", -1 );
    
	lv->setSorting( -1,1 );   // disables the autosort

	/////////////////////////////////////////////////////////
	//Set up tabbed interface

	tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );	
	tabframe->setGeometry( QRect( 148, 1, 700, 180 ) );

	//tabframe = new QTabWidget( object_controls, "axiscontrols" );	
	//tabframe->setGeometry( QRect( 1, 2, 700, 180 ) );
	
	tabframe->setTabShape( QTabWidget::Triangular );
	tabframe->setAutoMask( FALSE );
	if (platinum) tabframe->setStyle( platinum );


	//this has become the settings panel need to update variable names
    moduleOptionsUI[0] = new QWidget( EffectsFrame, "encoderUI0" ); //scenecontrols
	tabframe->insertTab( moduleOptionsUI[0], tr( "  "+jt.tr("SCENE")+"  " ) );

	moduleOptionsUI[1] = new QWidget( EffectsFrame, "objectcontrols" ); //objectcontrols
	tabframe->insertTab( moduleOptionsUI[1], tr( "  "+jt.tr("OBJECT")+"  " ) );

	moduleOptionsUI[2] = new QWidget( EffectsFrame, "axiscontrols" ); //axiscontrols
	tabframe->insertTab( moduleOptionsUI[2], tr( "  "+jt.tr("AXIS")+"  " ) );

	moduleOptionsUI[3] = new QWidget( EffectsFrame, "texturecontrols" ); //texturecontrols
	tabframe->insertTab( moduleOptionsUI[3], tr( "    "+jt.tr("CLIP")+"    " ) );

	moduleOptionsUI[4] = new QWidget( EffectsFrame, "effectscontrols" ); //effectscontrols
	tabframe->insertTab( moduleOptionsUI[4], tr( "  "+jt.tr("EFFECTS")+"  " ) );

	//this has become the settings panel need to update variable names
    moduleOptionsUI[5] = new QWidget( EffectsFrame, "keysettings" ); //keycontrols
    tabframe->insertTab( moduleOptionsUI[5], tr( "  "+jt.tr("TIMELINE")+"  " ) );

	//this raises the timeline above all other objects
	tabframe->setCurrentPage( 5 );

	/////////////////////////////////////////////////////////////////////
	// the layout boxes for different types of layers
    // the object control tab bar
	// this is the object control panel

	objectControlStack = new QWidgetStack(moduleOptionsUI[1]); //objectcontrols

	//initialize the stack for object switching
	for (int i=0; i<=6; i++)
	{
     objectControl[i]  = new QHBox(objectControlStack);          
	 objectControlStack->addWidget(objectControl[i], i); //fxobject_clip
    }

	 objectControlStack->setGeometry( QRect( 0, 0, 700, 180));   // was 635


    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //mover this into the layers...
    ControlFrameUI[0][0] = new QFrame( moduleOptionsUI[0], "ControlFrame3" ); //scenecontrols
    ControlFrameUI[0][0]->setGeometry( QRect( 0, 0, 660, 140 ) );

    LoadButton = new QPushButton( ControlFrameUI[0][0], "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , jt.tr("Load") );
    connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( ControlFrameUI[0][0], "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, jt.tr("Save") );
    connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( ControlFrameUI[0][0], "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, jt.tr("Reset All") );
    connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( ControlFrameUI[0][0], "ClearButton" );
    addButton( ClearButton, 10, 115, 86, 25, jt.tr("Clear Scene") );
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  );
    
    //PackageButton = new QPushButton( ControlFrameUI[0][0], "ClearButton" );
    //addButton( PackageButton, 110, 80, 86, 25, "Export Scene" );
    //connect( PackageButton,  SIGNAL(clicked()), this, SLOT( PackageSave())  );

  //need to move this into the module
    //this is mandatory for all modules...

    sframetext = new QLabel( ControlFrameUI[0][0], "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( jt.tr("STARTFRAME") ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( ControlFrameUI[0][0], "startframe",9 );
    addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
    lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
    connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( ControlFrameUI[0][0], "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( jt.tr("ENDFRAME")  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

    lcdendframe = new InputLCD( ControlFrameUI[0][0], "endframe",9 );
    addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
    lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
    connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );

    jahreslabel = new QLabel( ControlFrameUI[0][0], "languagelabel" );
    jahreslabel->setGeometry( QRect( 450, 75, 120, 21 ) );
    jahreslabel->setAlignment( int( QLabel::AlignCenter ) );
    jahreslabel->setText( jt.tr( "Resolution" ) );

    JahresCombo = new QComboBox( FALSE, ControlFrameUI[0][0], "JahresCombo" );
    JahresCombo->setGeometry( QRect( 450, 100, 170, 21 ) );
	hasResMenu = true;

	projectData thedata;
	thedata.buildComboBox(JahresCombo);
    int defaultRes = jprefs.getJahResolution();
	JahresCombo->setCurrentItem(defaultRes); //should be set to appdefault
    connect( JahresCombo,  SIGNAL(activated(int)), this, SLOT(setResolution(int))  );


    ///////////////////////////////////////////////////////////////////////
    //set up world sub control set
	//these are all in ControlFrameUI[1]
	//since fxobject_world is in ControlFrameUI[1]

    //ControlFrameUI[1][0] = new QFrame( fxobject_world, "ControlFrame3" ); //was ControlFrame3
    
	ControlFrameUI[1][0] = new QFrame( objectControl[2], "ControlFrame3" ); //was ControlFrame3
    ControlFrameUI[1][0]->setGeometry( QRect( 0, 0, 660, 120 ) );


    TextSelect = new QCheckBox( ControlFrameUI[1][0], "TitleSafe" );
    addCheckButton( TextSelect, 300, 80, 110, 21, jt.tr("title safe") );
    connect( TextSelect,  SIGNAL(clicked()), this, SLOT(setTextSafe())  );

    GridSelect = new QCheckBox( ControlFrameUI[1][0], "Grid" );
    addCheckButton( GridSelect, 300, 40, 110, 21, jt.tr("grid") );
    connect( GridSelect,  SIGNAL(clicked()), this, SLOT(setGrid())  );

    SmoothSelect = new QCheckBox( ControlFrameUI[1][0], "SmoothSelect" );
    addCheckButton( SmoothSelect, 300, 10, 110, 21, jt.tr("aliasing") );
    connect( SmoothSelect,  SIGNAL(clicked()), this, SLOT(setSmooth())  );

    ResetWorldButton = new QPushButton( ControlFrameUI[1][0], "ResetLayerButton" );
    addButton( ResetWorldButton, 410, 10, 86, 25, jt.tr("Reset") );
    connect  ( ResetWorldButton,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );

    //initalize the global vars from the prefs
    bool HWFOG = jprefs.getHwFog();

    if (HWFOG)
    {
        //toggle layer blur - need to add sliders here
        FogSelect = new QCheckBox( ControlFrameUI[1][0], "fog" );
        addCheckButton( FogSelect, 500, 70, 110, 21, jt.tr("fog") );
        connect( FogSelect,  SIGNAL(clicked()), this, SLOT(setFog())  );

        //QSlider       *HFogSlider, *VFogSlider;
        HFogSlider = new QSlider( ControlFrameUI[1][0] );    HFogSlider_lcd = new InputLCD( ControlFrameUI[1][0] );
        addSliderLcdLabel( ControlFrameUI[1][0], HFogSlider, 500, 90, 10, 21,  100, HFogSlider_lcd,  "H", 9);
        connect( HFogSlider,     SIGNAL(sliderMoved(int)), this, SLOT(setHFog(int)) );
        connect( HFogSlider,     SIGNAL(sliderMoved(int)), HFogSlider_lcd, SLOT(display(int)) );
        connect( HFogSlider_lcd, SIGNAL(valueChanged(int)), this, SLOT(setHFog(int)) );

        VFogSlider = new QSlider( ControlFrameUI[1][0] );    VFogSlider_lcd = new InputLCD( ControlFrameUI[1][0] );
        addSliderLcdLabel( ControlFrameUI[1][0], VFogSlider, 500, 120, 10, 21,  100, VFogSlider_lcd,  "V", 9);
        connect( VFogSlider,     SIGNAL(sliderMoved(int)), this, SLOT(setVFog(int)) );
        connect( VFogSlider,     SIGNAL(sliderMoved(int)), VFogSlider_lcd, SLOT(display(int)) );
        connect( VFogSlider_lcd, SIGNAL(valueChanged(int)), this, SLOT(setVFog(int)) );
    }

    ///////////////////////////////////////////////////////////////
    // object layer options sub control set
	//these are all in ControlFrameUI[1]
	//since fxobject_clip is in ControlFrameUI[1]

    //ControlFrameUI[1][1] = new QFrame( fxobject_clip, "ControlFrame5" ); //ControlFrame5
    
	ControlFrameUI[1][1] = new QFrame( objectControl[0], "ControlFrame5" ); //ControlFrame5
    ControlFrameUI[1][1]->setGeometry( QRect( 0, 0, 660, 120 ) );

	//the different object types

    ObjectLayer = new QPushButton( ControlFrameUI[1][1], "Layer" );
    addButton( ObjectLayer, 10, 10, 86, 25 , jt.tr("Layer") );
    connect  ( ObjectLayer,  SIGNAL(clicked()), this, SLOT( ChangeObjectLayer())  );

    ObjectCube = new QPushButton( ControlFrameUI[1][1], "Cube" );
    addButton( ObjectCube, 10, 45, 86, 25, jt.tr("Cube") );
    connect  ( ObjectCube,  SIGNAL(clicked()), this, SLOT( ChangeObjectCube())  );

    ObjectCylinder = new QPushButton( ControlFrameUI[1][1], "Cylinder" );
    addButton( ObjectCylinder, 10, 80, 86, 25, jt.tr("Cylinder") );
    connect  ( ObjectCylinder,  SIGNAL(clicked()), this, SLOT( ChangeObjectCylinder())  );

    ObjectSphere = new QPushButton( ControlFrameUI[1][1], "Sphere" );
    addButton( ObjectSphere, 110, 10, 86, 25, jt.tr("Sphere") );
    connect( ObjectSphere,  SIGNAL(clicked()), this, SLOT( ChangeObjectSphere())  );

    ObjectBezier = new QPushButton( ControlFrameUI[1][1], "Bezier" );
    addButton( ObjectBezier, 110, 45, 86, 25, jt.tr("Bezier") );
    connect( ObjectBezier,  SIGNAL(clicked()), this, SLOT( ChangeObjectBezier())  );

    ObjectMesh = new QPushButton( ControlFrameUI[1][1], "Mesh" );
    addButton( ObjectMesh, 110, 80, 86, 25, jt.tr("Mesh") );
    connect( ObjectMesh,  SIGNAL(clicked()), this, SLOT( ChangeObjectMesh())  );

    ResetLayerButton = new QPushButton( ControlFrameUI[1][1], "ResetLayerButton" );
    addButton( ResetLayerButton, 410, 10, 86, 25, jt.tr("Reset") );
    connect  ( ResetLayerButton,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );


    /////////////////font options
	//these are all in ControlFrameUI[1]
    //ControlFrameUI[1][2] = new QFrame( fxobject_text, "ControlFrame6" ); //was ControlFrame6
    
	ControlFrameUI[1][2] = new QFrame( objectControl[1], "ControlFrame6" ); //was ControlFrame6
    ControlFrameUI[1][2]->setGeometry( QRect( 0, 0, 40, 120 ) );

    ScaleText_5 = new QLabel( ControlFrameUI[1][2], "ScaleText_4" );
    addLabel( ScaleText_5, 0, 10, 110, 21, "EXTRUDE", 9);

    ScaleX_5 = new QSlider( ControlFrameUI[1][2] ); ScaleXlcd_5 = new InputLCD( ControlFrameUI[1][2] );
    addSliderLcdLabel( ControlFrameUI[1][2], ScaleX_5,   300, 30, 10, 21,  1, 200,  30,  ScaleXlcd_5,  "E", 9);

    connect( ScaleX_5,    SIGNAL(sliderMoved(int)), ScaleXlcd_5, SLOT(display(int)) );
    connect( ScaleX_5,    SIGNAL(sliderMoved(int)), this, SLOT(setExtrude(int)) );
    connect( ScaleXlcd_5, SIGNAL(valueChanged(int)), this, SLOT(setExtrude(int)) );

    textDraw = new QPushButton( ControlFrameUI[1][2], "draw" );
    addButton( textDraw, 115, 30, 36, 21, "Draw" );
    connect( textDraw,  SIGNAL(clicked()), this, SLOT(changeFontDraw())  );

    textEd	= new QLineEdit( ControlFrameUI[1][2], "text" );
    textEd->setGeometry( QRect( 0, 60, 150, 21 ) );
    textEd->setText( "jahshaka" );
    connect( textEd, SIGNAL(textChanged(const QString&)), this, SLOT(setText(const QString&)) );

    textFont = new QPushButton( ControlFrameUI[1][2], "font" );
    addButton( textFont, 0, 90, 55, 21, "Font" );
    connect( textFont,  SIGNAL(clicked()), this, SLOT(loadFont())  );

    textColor = new QPushButton( ControlFrameUI[1][2], "color" );
    addButton( textColor, 60, 90, 55, 21, "Color" );
    connect( textColor,  SIGNAL(clicked()), this, SLOT(setFontColor())  );

    ///////////////////////////////////////////////////////
    //particle options

    //ControlFrameUI[1][3] = new QFrame( fxobject_particle, "ControlFrame7" ); //was ControlFrame7
    
	ControlFrameUI[1][3] = new QFrame( objectControl[4], "ControlFrame7" ); //was ControlFrame7
    ControlFrameUI[1][3]->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadimage_3 = new QPushButton( ControlFrameUI[1][3], "emitter" );
    addButton( loadimage_3, 0, 20, 111, 20,  "Emitter" );
    connect( loadimage_3,  SIGNAL(clicked()), this, SLOT(changeParticle())  );

    loadimage_4 = new QPushButton( ControlFrameUI[1][3], "style" );
    addButton( loadimage_4, 0, 60, 111, 20,  "Style" );
    //connect( loadimage_4,  SIGNAL(clicked()), this, SLOT(changeParticleDraw())  );

    particlestyle = new QPushButton( ControlFrameUI[1][3], "tex" );
    addButton( particlestyle, 115, 20, 35, 20, "Color" );
    connect( particlestyle,  SIGNAL(clicked()), this, SLOT(changeParticleColors())  );

    ////////////////////////////////////////////////////////
    //object options 5

    ControlFrameUI[1][4] = new QFrame( objectControl[5], "ControlFrame9" ); //ControlFrame9
    ControlFrameUI[1][4]->setGeometry( QRect( 0, 0, 40, 120 ) );


    loadobject_1 = new QPushButton( ControlFrameUI[1][4], "load" );
    addButton( loadobject_1, 0, 20, 111, 30, "Load Object" );
    connect( loadobject_1,  SIGNAL(clicked()), this, SLOT(loadObjObject())  );

    objectstyle = new QPushButton( ControlFrameUI[1][4], "draw" );
    addButton( objectstyle, 115, 20, 35, 30, "fill" );
    connect( objectstyle,  SIGNAL(clicked()), this, SLOT(changeObjectDraw())  );

    ResetObjectButton1 = new QPushButton( ControlFrameUI[1][4], "ResetLayerButton" );
    addButton( ResetObjectButton1, 410, 10, 86, 25, "Reset" );
    connect  ( ResetObjectButton1,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );

    ////////////////////////////////////////////////////////
    //max object options - should be merged in with the object code above
	//as we can determine what we are loading when we are loading it no?
	//but then we need to monitor the 'object style' above... 6

    ControlFrameUI[1][4] = new QFrame( objectControl[6], "ControlFrame10" ); // was ControlFrame10
    ControlFrameUI[1][4]->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadmaxobject_1 = new QPushButton( ControlFrameUI[1][4], "load" );
    addButton( loadmaxobject_1, 0, 20, 111, 30, "Load MObject" );
    connect( loadmaxobject_1,  SIGNAL(clicked()), this, SLOT(loadMaxObject())  );

    ResetObjectButton2 = new QPushButton( ControlFrameUI[1][4], "ResetLayerButton" );
    addButton( ResetObjectButton2, 410, 10, 86, 25, "Reset" );
    connect  ( ResetObjectButton2,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );


    ///////////////////////////////////////////////////////////////////////
    //set up object control set and sliders
    //set up world sub control set
	//these are all in ControlFrameUI[2]
	//since fxaxis_clipone is in ControlFrameUI[2]

    ControlFrameUI[2][0] = new QFrame( moduleOptionsUI[2], "ControlFrame1" ); //ControlFrame1
    ControlFrameUI[2][0]->setGeometry( QRect( 0, 0, 760, 120 ) );    // was 510

    createAxisPanel ( ControlFrameUI[2][0] );

    //add the checkbox
    JahModuleOption[0] = new QCheckBox( ControlFrameUI[2][0], "PathSelect" );
    addCheckButton( JahModuleOption[0], 520, 30, 145, 25, "Show Path" );
    connect( JahModuleOption[0],  SIGNAL(clicked()), this, SLOT(setKeyframeDrawStatus())  ); //for clip

	//increment the ui uption counter...
	//we can add this for sliders as well
	//as we use them in the plugins
	
	numUiOptions +=1;

	hasButtons = true;

    ///////////////////////////////////////////////////////////////
    // texture layer options
	// need to add video-texture options here

	//image options

    ControlFrameUI[3][0] = new QFrame( moduleOptionsUI[3], "ControlFrame2" ); //was ControlFrame2
    ControlFrameUI[3][0]->setGeometry( QRect( 0, 0, 660, 150 ) );

    ClipSelect = new QCheckBox( ControlFrameUI[3][0], "ClipSelect" );
    addCheckButton( ClipSelect, 0, 30, 45, 25, "Clip" );
    connect( ClipSelect,  SIGNAL(clicked()), this, SLOT(setClipStatus())  ); //for clip

    grabdesktopClip = new QPushButton( ControlFrameUI[3][0], "grabdesktop" );
    addButton( grabdesktopClip, 45, 30, 110, 25, "Grab Desktop" );
    connect( grabdesktopClip,  SIGNAL(clicked()), this, SLOT(grabDesktop())  );

    keyimage_2 = new QPushButton( ControlFrameUI[3][0], "clipmapping" );
    addButton( keyimage_2, 165, 30, 35, 25, "IM" );
    connect( keyimage_2,  SIGNAL(clicked()), this, SLOT(keyCliplayer())  );


    KeySelect = new QCheckBox( ControlFrameUI[3][0], "KeySelect" );
    addCheckButton( KeySelect, 0, 75, 45, 25, "Key" );
    connect( KeySelect,  SIGNAL(clicked()), this, SLOT(setKeyStatus())  );

    grabdesktopKey = new QPushButton( ControlFrameUI[3][0], "grabdesktop" );
    addButton( grabdesktopKey, 45, 75, 110, 25, "Grab Desktop" );
    //connect( grabdesktopKey,  SIGNAL(clicked()), this, SLOT(grabDesktopKey())  ); //for key...

    keyimage_3 = new QPushButton( ControlFrameUI[3][0], "keyOne_3" );
    addButton( keyimage_3, 165, 75, 35, 25, "KM" );
    connect( keyimage_3,  SIGNAL(clicked()), this, SLOT(keyClipkey())  );

    keyStatusBox = new QPushButton( ControlFrameUI[3][0], "Select1" );
    addButton( keyStatusBox, 220, 75, 35, 25, "Invert" );
    connect( keyStatusBox,  SIGNAL(clicked()), this, SLOT(InvertKeyData())  );

	// video options
	// need to hook this up now...

	VideoLabel = new QLabel( ControlFrameUI[3][0], "TranslateText_1" );
	addLabel( VideoLabel, 380, 10, 110, 21, "VIDEO OPTIONS", 9);


	FramesLcd = new InputLCD( ControlFrameUI[3][0] );
	addLabelLcd( ControlFrameUI[3][0], FramesLcd,       380, 30, 60, 21,  0,  "Frames", 8);
	FramesLcd->setActive	(false);

    inFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel( ControlFrameUI[3][0], inFramesLcd,     460, 30, 30, 21,  0,  "In", 8);
    connect(inFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinFrames(int)) );

    outFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel( ControlFrameUI[3][0], outFramesLcd,    530, 30, 30, 21,  0,  "Out", 8);
    connect(outFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutFrames(int)) );

    KeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLabelLcd( ControlFrameUI[3][0], KeyFramesLcd,    380, 75, 60, 21,  0,  "Frames", 8);

    inKeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel( ControlFrameUI[3][0], inKeyFramesLcd,  460, 75, 30, 21,  0,  "In", 8);
    connect(inKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinKeyFrames(int)) );

    outKeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel( ControlFrameUI[3][0], outKeyFramesLcd, 530, 75, 30, 21,  0,  "Out", 8);
    connect(outKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutKeyFrames(int)) );
    
    extendHeadTail = new QCheckBox( ControlFrameUI[3][0], "extendHeadTail" );
    addCheckButton( extendHeadTail, 380, 95, 100, 25, "Extend Head-Tail" );
    connect( extendHeadTail,  SIGNAL(clicked()), this, SLOT(toggleExtendHeadTail())  );

    toggleTheLoop = new QCheckBox( ControlFrameUI[3][0], "toggleTheLoop" );
    addCheckButton( toggleTheLoop, 480, 95, 100, 25, "Loop" );
    connect( toggleTheLoop,  SIGNAL(clicked()), this, SLOT(toggleLoop())  );
    
	togglePing = new QCheckBox( ControlFrameUI[3][0], "togglePing" );
    addCheckButton( togglePing, 480, 130, 100, 25, "Ping Pong" );
    connect( togglePing,  SIGNAL(clicked()), this, SLOT(togglePingPong())  );

    //what is this used for?
    //need to make the icon a lock... 3x current height as well for links
  	lockicon	= new QToolButton(ControlFrameUI[3][0],"lockicon");
    lockicon->setGeometry( QRect( 600, 55, 18, 26 ) );
  	lockicon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lockicon->setAutoRaise(true);






    ////////////////////////////////////////////////////////
    //effects options

    ControlFrameUI[4][0] = new QFrame( moduleOptionsUI[4], "ControlFrame8" ); //was ControlFrame8
    ControlFrameUI[4][0]->setGeometry( QRect( 0, 0, 660, 120 ) );

	//toggle foreground background status
    ForegroundSelect = new QCheckBox( ControlFrameUI[4][0], "foreground" );
    addCheckButton( ForegroundSelect, 10, 10, 110, 21, "foreground" );
    connect( ForegroundSelect,  SIGNAL(clicked()), this, SLOT(toggleForegroundStatus())  );
    
	//toggle foreground background status
    DepthSelect = new QCheckBox( ControlFrameUI[4][0], "depth" );
    addCheckButton( DepthSelect, 120, 10, 110, 21, "depth" );
    connect( DepthSelect,  SIGNAL(clicked()), this, SLOT(toggleDepthStatus())  );

    //toggle layer blur - need to add sliders here
    BlurSelect = new QCheckBox( ControlFrameUI[4][0], "blur" );
    addCheckButton( BlurSelect, 10, 40, 110, 21, "blur" );
    connect( BlurSelect,  SIGNAL(clicked()), this, SLOT(setBlur())  );

    //QSlider       *HBlurSlider, *VBlurSlider;
    HBlurSlider = new QSlider( ControlFrameUI[4][0] );    HBlurSlider_lcd = new InputLCD( ControlFrameUI[4][0] );
    addSliderLcdLabel( ControlFrameUI[4][0], HBlurSlider, 10, 60, 10, 21,  100, HBlurSlider_lcd,  "H", 9);
    connect( HBlurSlider,     SIGNAL(sliderMoved(int)), this, SLOT(setHBlur(int)) );
    connect( HBlurSlider,     SIGNAL(sliderMoved(int)), HBlurSlider_lcd, SLOT(display(int)) );
    connect( HBlurSlider_lcd, SIGNAL(valueChanged(int)), this, SLOT(setHBlur(int)) );

    VBlurSlider = new QSlider( ControlFrameUI[4][0] );    VBlurSlider_lcd = new InputLCD( ControlFrameUI[4][0] );
    addSliderLcdLabel( ControlFrameUI[4][0], VBlurSlider, 10, 90, 10, 21,  100, VBlurSlider_lcd,  "V", 9);
    connect( VBlurSlider,     SIGNAL(sliderMoved(int)), this, SLOT(setVBlur(int)) );
    connect( VBlurSlider,     SIGNAL(sliderMoved(int)), VBlurSlider_lcd, SLOT(display(int)) );
    connect( VBlurSlider_lcd, SIGNAL(valueChanged(int)), this, SLOT(setVBlur(int)) );

    //toggle layer blur - need to add sliders here
    NegativeSelect = new QCheckBox( ControlFrameUI[4][0], "negative" );
    addCheckButton( NegativeSelect, 130, 40, 110, 21, "negative" );
    connect( NegativeSelect,  SIGNAL(clicked()), this, SLOT(setNegative())  );

    //toggle layer blur - need to add sliders here
    SwapRGBSelect = new QCheckBox( ControlFrameUI[4][0], "swap" );
    addCheckButton( SwapRGBSelect, 130, 60, 110, 21, "swap rgb" );
    connect( SwapRGBSelect,  SIGNAL(clicked()), this, SLOT(setRGB())  );
    
    //toggle layer blur - need to add sliders here
    MirrorSelect = new QCheckBox( ControlFrameUI[4][0], "mirror" );
    addCheckButton( MirrorSelect, 130, 90, 110, 21, "mirror" );
    connect( MirrorSelect,  SIGNAL(clicked()), this, SLOT(setMirror())  );
    
    
    // creates layer light option
    lighticon =   new JahToolButton( ControlFrameUI[4][0], "lighticon" );
    addJahPlayerButton( lighticon,   600, 10, 18, 26 ,    18, 26, JahBasePath+"Pixmaps/modules/light.png", JahBasePath+"Pixmaps/modules/light.png" );

    light1 = new QCheckBox( ControlFrameUI[4][0], "light1" );
    light1->setGeometry( QRect( 600, 36, 75, 21 ) );
    //light1->setText( "light" );
    connect( light1,  SIGNAL(clicked()), this, SLOT(toggleLight())  );
    
    reflect1 = new QCheckBox( ControlFrameUI[4][0], "reflect1" );
    reflect1->setGeometry( QRect( 400, 76, 105, 21 ) );
    reflect1->setText( "reflectionmap" );
    connect( reflect1,  SIGNAL(clicked()), this, SLOT(toggleReflect())  );
    
    smooth1 = new QCheckBox( ControlFrameUI[4][0], "reflect1" );
    smooth1->setGeometry( QRect( 525, 76, 105, 21 ) );
    smooth1->setText( "smoothshading" );
    connect( smooth1,  SIGNAL(clicked()), this, SLOT(toggleSmooth())  );

    shader1 = new QCheckBox( ControlFrameUI[4][0], "shader1" );
    shader1->setGeometry( QRect( 400, 100, 105, 21 ) );
    shader1->setText( "pixelshader1" );
    connect( shader1,  SIGNAL(clicked()), this, SLOT(toggleShader1())  );

    shader2 = new QCheckBox( ControlFrameUI[4][0], "shader1" );
    shader2->setGeometry( QRect( 525, 100, 105, 21 ) );
    shader2->setText( "pixelshader2" );
    connect( shader2,  SIGNAL(clicked()), this, SLOT(toggleShader2())  );

    mesh1 = new QCheckBox( ControlFrameUI[4][0], "light1" );
    mesh1->setGeometry( QRect( 400, 125, 115, 21 ) );
    mesh1->setText( "toggle mesh" );
    connect( mesh1,  SIGNAL(clicked()), this, SLOT(toggleMesh())  );
 
    //trans slider
    Transparency_1 = new QLabel( ControlFrameUI[4][0], "ScaleText_2" );
    addLabel( Transparency_1, 400, 5, 110, 21, "TRANSPARENCY", 9);
    Trans_1 = new QSlider( ControlFrameUI[4][0] );  Translcd_1 = new InputLCD( ControlFrameUI[4][0] );
    addSliderLcdLabel( ControlFrameUI[4][0], Trans_1,   400, 25, 10, 21,  1, 100,    Translcd_1,  "T", 9);
    connect( Trans_1,    SIGNAL(sliderMoved(int)), Translcd_1, SLOT(display(int)) );
    connect( Trans_1,    SIGNAL(sliderMoved(int)), this, SLOT(setAlpha(int)) );
    connect( Translcd_1, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );

    ////////////////////////////////////////////////////////
    // raise the control bar and raise backgound as default

    objectControlStack->raiseWidget(0);

}


void GLEdit::createAxisPanel ( QFrame* theuiframe)
{
	//set some flags
	hasLabels = true;
    hasSliders = true;
    JahTranslate& jt = JahTranslate::getInstance();

    //////////////////////////////////////////////////////////////////////////////////////////
    // set up rotate sliders

    JahModuleLabel[0] = new QLabel( theuiframe, "RotateText_1" );
    addLabel( JahModuleLabel[0], 10, 10, 110, 21, jt.tr("ROTATE"), 9);

    JahSliders[0] = new QSlider( theuiframe );
    JahSlidersLCD[0] = new InputLCD( theuiframe );
    JahSlidersLabel[0] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[0], 0, 30, 10, 21,  500, JahSlidersLCD[0], JahSlidersLabel[0], "X", 9);
    connect( JahSliders[0], SIGNAL(sliderMoved(int)), this, SLOT(setXRotation(int)) );
    connect( JahSliders[0], SIGNAL(sliderMoved(int)), JahSlidersLCD[0], SLOT(display(int)) );
    connect(JahSlidersLCD[0], SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );

    JahSliders[1] = new QSlider( theuiframe );
    JahSlidersLCD[1] = new InputLCD( theuiframe );
    JahSlidersLabel[1] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[1], 0, 60, 10, 21,  500, JahSlidersLCD[1], JahSlidersLabel[1], "Y", 9);
    connect( JahSliders[1], SIGNAL(sliderMoved(int)), this, SLOT(setYRotation(int)) );
    connect( JahSliders[1], SIGNAL(sliderMoved(int)), JahSlidersLCD[1], SLOT(display(int)) );
    connect(JahSlidersLCD[1], SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );

    JahSliders[2] = new QSlider( theuiframe );
    JahSlidersLCD[2] = new InputLCD( theuiframe );
    JahSlidersLabel[2] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[2], 0, 90, 10, 21,  500, JahSlidersLCD[2], JahSlidersLabel[2], "Z", 9);
    connect( JahSliders[2], SIGNAL(sliderMoved(int)), this, SLOT(setZRotation(int)) );
    connect( JahSliders[2], SIGNAL(sliderMoved(int)), JahSlidersLCD[2], SLOT(display(int)) );
    connect(JahSlidersLCD[2], SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );


    //////////////////////////////////////////////////////////////////////////////////////////
    // set up translate sliders

    JahModuleLabel[1] = new QLabel( theuiframe, "TranslateText_1" );
    addLabel( JahModuleLabel[1], 180, 10, 110, 21, jt.tr("TRANSLATE"), 9);

    JahSliders[3] = new QSlider( theuiframe );
    JahSlidersLCD[3] = new InputLCD( theuiframe );
    JahSlidersLabel[3] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[3], 180, 30, 10, 21,  500, JahSlidersLCD[3], JahSlidersLabel[3], "X", 9);
    connect( JahSliders[3],   SIGNAL(sliderMoved(int)), this, SLOT(setXTrans(int)) );
    connect( JahSliders[3],   SIGNAL(sliderMoved(int)), JahSlidersLCD[3], SLOT(display(int)) );
    connect(JahSlidersLCD[3], SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    JahSliders[4] = new QSlider( theuiframe );
    JahSlidersLCD[4] = new InputLCD( theuiframe );
    JahSlidersLabel[4] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[4], 180, 60, 10, 21,  500, JahSlidersLCD[4], JahSlidersLabel[4], "Y", 9);
    connect( JahSliders[4], SIGNAL(sliderMoved(int)), this, SLOT(setYTrans(int)) );
    connect( JahSliders[4], SIGNAL(sliderMoved(int)), JahSlidersLCD[4], SLOT(display(int)) );
    connect(JahSlidersLCD[4], SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    JahSliders[5] = new QSlider( theuiframe );
    JahSlidersLCD[5] = new InputLCD( theuiframe );
    JahSlidersLabel[5] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[5], 180, 90, 10, 21,  500, JahSlidersLCD[5],  JahSlidersLabel[5], "Z", 9);
    connect( JahSliders[5],   SIGNAL(sliderMoved(int)), this, SLOT(setZTrans(int)) );
    connect( JahSliders[5],   SIGNAL(sliderMoved(int)), JahSlidersLCD[5], SLOT(display(int)) );
    connect(JahSlidersLCD[5], SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );


    //////////////////////////////////////////////////////////////////////////////////////////
    // set up scale sliders

    JahModuleLabel[2] = new QLabel( theuiframe, "ScaleText_1" );
    addLabel( JahModuleLabel[2], 350, 10, 110, 21, jt.tr("SCALE"), 9);

    JahSliders[6] = new QSlider( theuiframe );
    JahSlidersLCD[6] = new InputLCD( theuiframe );
    JahSlidersLabel[6] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[6], 350, 30, 10, 21,  500, JahSlidersLCD[6], JahSlidersLabel[6],  "X", 9);
    connect( JahSliders[6], SIGNAL(sliderMoved(int)), this, SLOT(setXScale(int)) );
    connect( JahSliders[6], SIGNAL(sliderMoved(int)), JahSlidersLCD[6], SLOT(display(int)) );
    connect(JahSlidersLCD[6], SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );

    JahSliders[7] = new QSlider( theuiframe );
    JahSlidersLCD[7] = new InputLCD( theuiframe );
    JahSlidersLabel[7] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[7], 350, 60, 10, 21,  500, JahSlidersLCD[7], JahSlidersLabel[7], "Y", 9);
    connect( JahSliders[7], SIGNAL(sliderMoved(int)), this, SLOT(setYScale(int)) );
    connect( JahSliders[7], SIGNAL(sliderMoved(int)), JahSlidersLCD[7], SLOT(display(int)) );
    connect(JahSlidersLCD[7], SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );

    JahSliders[8] = new QSlider( theuiframe );
    JahSlidersLCD[8] = new InputLCD( theuiframe );
    JahSlidersLabel[8] = new QLabel ( theuiframe);

    addJahSliderGroup( theuiframe, JahSliders[8], 350, 90, 10, 21,  500, JahSlidersLCD[8], JahSlidersLabel[8], "Z", 9);
    connect( JahSliders[8], SIGNAL(sliderMoved(int)), this, SLOT(setZScale(int)) );
    connect( JahSliders[8], SIGNAL(sliderMoved(int)), JahSlidersLCD[8], SLOT(display(int)) );
    connect(JahSlidersLCD[8], SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );

}













