/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"

void GLEffect::initializeObjects()
{    
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLEffect Class");

    //initialize the keyframe editor
    //we can always move this into the effectinterface.cpp obj

    thekeyframer = new JahKeyframer( this, moduleOptionsUI[3], "keyframer" );
    thekeyframer->setFixedSize( QSize( 980, 600 ) );    //was 955,486
    connectKeyframer();
    haskeyframer = true;

    //make the current opengl context current so the textures can be set up properly
    makeCurrent();

    ///////////////////////////////////////////////
    // the linked list is here
    effects = new QPtrList<FxLayerList>;
    effects->setAutoDelete( TRUE );     // delete items when they are removed

    //effects start at 2 since they match up with layers 0 and 1 that are used below...
    effects->append( new FxLayerList(0) );
    effects->at(0)->fxlayer->objtype = NOFX;

    effects->append( new FxLayerList(1) );
    effects->at(1)->fxlayer->objtype = NOFX;

    ///////////////////////////////////////////////
    //this creates the default world layer using the linked list

    //for effects there is no world control at the moment
    //so we need to build it and blank it out...
    layerlist->append( new NewLayerList(0, "World") );
    layerlist->at(0)->thelayer->layerStatus = false;	//set the status to false since we dont useit
    layerlist->at(0)->thelayer->blend = TRUE;
    layerlist->at(0)->thelayer->depth = FALSE;
    layerlist->at(0)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(0)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer";
    layerlist->at(0)->thelayer->asset.extension = ".bmp";
    buildLayer(0,"World", LAYER, BACKDROP );
    numberLayers+=1;

    //this creates layer 1 all the effects are grouped to it
    //in prepapartion for layer effect grouping
    layerlist->append( new NewLayerList(1, "World") );

    //setup and build the default layer these should all be defaults
    layerlist->at(1)->thelayer->blend = TRUE;
    layerlist->at(1)->thelayer->depth = TRUE;
    layerlist->at(1)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(1)->thelayer->node->tz = 0.0;
    layerlist->at(1)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer"; 
    layerlist->at(1)->thelayer->asset.extension = ".bmp";
    //layerlist->at(1)->thelayer->objtype = MESHLAYER;
    buildLayer(1,"FxLayer", LAYER, MESHLAYER );  //was JAHLAYER

    addSelection(FALSE);   // adds effects layer as 'off' uses default numberLayers = 1

    objectMenu(numberLayers);
    updateSliders();

    //initialize the buffer
    //for buffering
    fxbufferexists = false;
    useBuffering = true;

    initBuffer(&layerlist->at(1)->thelayer->textureData.objectImage);
    forceupdatefxbuffer = true;

    //used to prevent people adding fx till they get a clip
    initialized = false;

    //turn forceplay on here
    setForcePlay(true);

    //////////////////////////////////////////////////////
    // added for custom variables
    static bool initVariables = true;
    if (initVariables)
        initializeVariables();
    initVariables = false;

    //set the ui tool
    noTool();

    //render using the source clip res
    //SetVideoRender();
    VideoRenderSelect->setChecked(true);

    //this is where we get the global instance of the plugin class
    jplugin = JahPluginLib::getInstance();

}


void GLEffect::initializeVariables(void)
{
    //set up prefs and check for gpu support
    JahPrefs& jprefs = JahPrefs::getInstance();

    //for gpu effects
    gpusupport = false;
    nv_gpusupport = false;
    gpuactive = false;

    gpusupport = jprefs.getGpusupport();
    nv_gpusupport = jprefs.getNVGpusupport();

    //use source size renders
    SetVideoRender();

    //initialize timer object
    //used to time the effects
    fxtimer = new JahTimer;

    //this is used for the gpu effects
    // The camera distance calculation should be passed in from the GL resize
    // routine.  This is a HACK which should be fixed FIXME

    //#define Y_RESOLUTION 		486
    #define PI                  3.14159265358979323846
    #define VIEW_ANGLE			45.0
    #define VIEW_ANGLE_RADIANS 	((VIEW_ANGLE / 180.0) * PI)

    camera_distance = ((float)Y_RESOLUTION / 2.0) / tan(VIEW_ANGLE_RADIANS / 2.0);

    noiseMatrixInitialized = FALSE;
    fireTextureInitialized = FALSE;
    randomTextureInitialized = FALSE;
    contrastTextureInitialized = FALSE;

    #ifdef NVIDIA_GPGPU
    shader_display = 0;
    shader_pbuffer = 0;
    pixel_buffer = NULL;
    #endif

}

//this initializes the node list view

void GLEffect::initializeNodes(GLCore* core, QHBox* parent2)
{
    thenodeframer = new JahNodes( parent2, "keyframer" );
    //thenodeframer->setFixedSize( QSize( 280, 600 ) );    //was 955,486
    thenodeframer->setFixedSize( QSize( 980, 600 ) );    //was 955,486

}


void GLEffect::toggleMesh(void)
{
    layerlist->at(activeLayer)->thelayer->ObjectDisplayMode =
        !layerlist->at(activeLayer)->thelayer->ObjectDisplayMode;

    updateGL();
}

void GLEffect::SwitchView(void)    
{ 
    //debug("emiting a switch");
    emit switchview(); 
}

void GLEffect::updateSliders()
{   
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLEffect","updating slider display");
    jtrace.debug( "active layer",activeLayer);


    if (effects->at(activeLayer)->fxlayer->objtype == JAHPLUGIN)
    {
        //plugins currently have a lot of ui control

        if (activeLayer>1)
        {
            hideAllSliders();
            hideAllHeadings();

            //get the number of the plugin we are dealing with
            int plugFx = effects->at(activeLayer)->fxlayer->pluginnumber;

            ///////////////////////////////////////////////////////////////
            //first we do the slider labels!
            bool us  = jplugin->vfxPlugins->at(plugFx)->getUseSliders(); //

            if (us)
            {
                int  ns  = jplugin->vfxPlugins->at(plugFx)->getNumberSliders(); //      3;

                for (int is=0; is<ns; is++)
                {
                    setSliderLabelValue(is, jplugin->vfxPlugins->at(plugFx)->getSliderLabel(is));
                }

                showSliders(ns);
            }

            ///////////////////////////////////////////////////////////////
            //first we do the group labels!
            bool ul  = jplugin->vfxPlugins->at(plugFx)->getUseLabels(); //

            if (ul)
            {
                int  nl  = jplugin->vfxPlugins->at(plugFx)->getNumberLabels();

                for (int il=0; il<nl; il++)
                {
                    setHeadingValue(il, jplugin->vfxPlugins->at(plugFx)->getLabel(il));
                }

                showHeadings(nl);
            }


        }


    }
    else
    {
        setDefaultSliderText();
        showAllSliders();

        setDefaultHeadings();
        showAllHeadings();
    }

    //update the rest of the slider/ui data
    updateSliderValues();

    updateUiOptions();

    updateUiSettings();



}

//we override this here since the 'world' is now layer 1

void GLEffect::updateSliderValues()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLEffect","updating slider value");
    //jtrace.debug( "activelayer",activeLayer);

    if (!multiSelect)
    {
        /////////////////////////////////////////////////////////////////////////////
        //basic values here
        if (activeLayer>=1 )
        {
            setSliderValue( 0, (int)layerlist->at(activeLayer)->thelayer->node->rx );
            setSliderValue( 1, (int)layerlist->at(activeLayer)->thelayer->node->ry );
            setSliderValue( 2, (int)layerlist->at(activeLayer)->thelayer->node->rz );

            setSliderValue( 3, (int)layerlist->at(activeLayer)->thelayer->node->tx );
            setSliderValue( 4, (int)layerlist->at(activeLayer)->thelayer->node->ty );
            setSliderValue( 5, (int)layerlist->at(activeLayer)->thelayer->node->tz );

            setSliderValue( 6, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sx)  );
            setSliderValue( 7, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sy)  );
            setSliderValue( 8, getScaleValue(layerlist->at(activeLayer)->thelayer->node->sz)  );

        }

    }
}

void GLEffect::updateUiOptions()
{
    ////////////////////////////////////////////////////////////////////////////////////
    // all other ui settings
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLEffect","updating ui options");

    //JahPluginLib* jplugin = JahPluginLib::getInstance();

    //first hide em all
    hideOptions(numUiOptions-1);

    if (effects->at(activeLayer)->fxlayer->objtype == JAHPLUGIN)
    {
        //move me to updateUiOptions()
        //rename to getNumberOptions
        //also need array for option names and modes...
        int plugFx = effects->at(activeLayer)->fxlayer->pluginnumber;

        int  nb  = jplugin->vfxPlugins->at(plugFx)->getNumberButtons(); //      3;

        //QString rere; rere.setNum(nb);
        //jtrace.debug( rere,"buttons");

        for (int i=0; i<nb; i++)
        {
            JahModuleOption[i]->setText( jplugin->vfxPlugins->at(plugFx)->getButton(i) );
        }

        showOptions(nb-1);

    }

}

void GLEffect::updateUiSettings()
{

    //these cause the app to crash?
    //Trans_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );
    //Translcd_1->setValue ( (int)(layerlist->at(activeLayer)->thelayer->node->Alpha)  );
    //light1->setChecked(layerlist->at(activeLayer)->thelayer->lighting);

    VideoRenderSelect->setChecked(core->useimagesize);

    JahModuleOption[0]->setChecked(layerlist->at(activeLayer)->thelayer->option1);
    JahModuleOption[1]->setChecked(layerlist->at(activeLayer)->thelayer->option2);
    JahModuleOption[2]->setChecked(layerlist->at(activeLayer)->thelayer->option3);
    JahModuleOption[3]->setChecked(layerlist->at(activeLayer)->thelayer->option4);

}


void GLEffect::setOption1(void)
{
    layerlist->at(activeLayer)->thelayer->option1 = JahModuleOption[0]->isChecked();
    updateEffects();
}

void GLEffect::setOption2(void)
{
    layerlist->at(activeLayer)->thelayer->option2 = JahModuleOption[1]->isChecked();
    updateEffects();
}

void GLEffect::setOption3(void)
{
    layerlist->at(activeLayer)->thelayer->option3 = JahModuleOption[2]->isChecked();
    updateEffects();
}

void GLEffect::setOption4(void)
{
    layerlist->at(activeLayer)->thelayer->option4 = JahModuleOption[3]->isChecked();
    updateEffects();
}


void GLEffect::clicked( QListViewItem* items )
{

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "**Clicked on layer in GLEffect ");


    //nothing added to listview yet
    if (!items) return;

    //clear the selection list first
    activeLayers->clearList();

    //this is the number of items in the listview
    //problems will arise here when we add children ie grouping
    activeLayers->setTotalLayers(lv->childCount());

    //iterate the list now to see whats selected
    //this is slow but its a easy way to
    //go through all layers to find the ones selected

    int numSelections=0;

    QListViewItemIterator it( lv );
    while ( it.current() )
    {
            QListViewItem *item = it.current();
            ++it;

            if ( item->isSelected() )  activeLayers->addLayer(numSelections);

            numSelections++; //re becomes total layers...
    }

    //if we got more than one selected its a global to let the core know
    if (activeLayers->getNumLayers()>1) multiSelect = TRUE; else multiSelect = FALSE;

    //checks if the item is checked or not
    int itemstatus=0;
    if (   ( ( QCheckListItem* )items )->isOn()==1   ) { itemstatus=1; }
    if (   ( ( QCheckListItem* )items )->isOn()==0   ) { itemstatus=0; }

    //in effects we start with the layer and hide the world so we have to increment this
    int pp = activeLayers->getFinalSelection()+1;

    //QString ras;
    //debug("clicked layer:"+ras.setNum(pp));

    //brings up the menu for the selected layer according to layer number
    objectMenu(pp);

    //we waste a interupt here, always checking the clicked item
    //would be better with a signal slot implementation
    int layerStat = layerlist->at(activeLayer)->thelayer->layerStatus;


    if      ( itemstatus==1  && layerStat==0 )
    { 
        forceupdatefxbuffer = true;
        setLayerVisible(pp, TRUE);
    }
    else if ( itemstatus==0  && layerStat==1 )
    {
        forceupdatefxbuffer = true;
        setLayerVisible(pp, FALSE);
    }
    else
    {
        forceupdatefxbuffer = true;
        updateGL();
    }

    //clicked on layer, update keyframes
    //should only do this if the layer has changed...

    updateKeyframeDisplay();

    //now update the sliders
    //do we need a activeLayer = pp here?

    updateSliders();

}

void GLEffect::objectMenu(int menu)
{

    changeSelection(menu);
}

//////////////////////////////////////////
// need to fix missing command below makeTexture
//////////////////////////////////////////
///////////////////////////////////////////
///////////////////////////////////////////

void GLEffect::addClip(videoData clip)
{
    JahTrace& jtrace = JahTrace::getInstance();

    initialized = true;

    layerlist->at(1)->thelayer->assignAsset(clip, IMAGE);

    layerlist->at(1)->thelayer->LoadLayerAsset(IMAGE);

    layerlist->at(1)->thelayer->makeTexture();

    //set renderspace imagesize here...
    core->imagewidth  = layerlist->at(1)->thelayer->getImageWidth();
    core->imageheight = layerlist->at(1)->thelayer->getImageHeight();

    forceupdatefxbuffer = true;

    //initialize the buffer
    if (useBuffering)
        initBuffer(&layerlist->at(1)->thelayer->textureData.objectImage);

    updateGL();
    updateSliders();
}

void GLEffect::SetGpuSelect(void)
{
    if (gpusupport || nv_gpusupport)
    {
        gpuactive = GpuSelect->isChecked();
    }

    if (gpuactive)
    {
        layerlist->at(1)->thelayer->usedagpu = true;
    }
    else
    {
        layerlist->at(1)->thelayer->usedagpu = false;
    }

    updateGL();
}


