/*******************************************************************************
**
** The header file for the Jahshaka effects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef EFFECT_H
#define EFFECT_H

#include <glworld.h>

#include "qcheckbox.h"
#include "qtabwidget.h"

//#include "fftlib.h" //this header includes blurlib.h
#include "blurlib.h" //this header includes blurlib.h
#include <stdlib.h>

#include "jahnodes.h"
#include "jahtimer.h"

//#include "glcore.h"

//for dynamic plugin support
#include "jahpluginlib.h"

////////////////////////////////////////////////////////////
//for gpu support
#include "gpumathlib.h"

extern unsigned char*       loadshaderfile(const char* name);
extern void                 find_shader_program_error(unsigned char* source, char* program_name);
extern void                 check_gl(void);
extern int                  ImageLoad(char* filename, Image* image);


class GLEffect :  public  GLWorld {
Q_OBJECT

public:

GLEffect( GLCore* core, QHBox* parent2, const char* name, QHBox* controls,
            QHBox* controller,QHBox* leftcontroller, QHBox* rightcontroller,
            int* globalclipnumber )
            :  GLWorld(core,name,controller,globalclipnumber )

{
    buildInterface(controls);

    buildInterfaceMenus(leftcontroller,rightcontroller);

    initializeNodes(core,parent2);

    initializeObjects();

	//jpluginl = JahPluginLib::getInstance();
};

GLEffect::~GLEffect() {  }

    //object and variable initialization routines
    void initializeVariables(void);
    void initializeObjects();

    //ui initialization routines
    void createAxisPanel ( QFrame* theuiframe);
    void buildInterface( QHBox* qtinterface);
    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );
    void initializeNodes( GLCore* core, QHBox* parent2 );


    //used to load the plugins in
    JahPluginLib * jplugin;

    //used for the node view
    JahNodes*               thenodeframer;

    ///////////////////////////////////////////////
    //for the onscreen buffer
    QImage * fxbuffer;
    bool fxbufferexists;

    void initBuffer(QImage * bufimage);
    void updateBuffer(QImage * bufimage);
    bool updateTheEffect();

    //these arent slots are they? they should just be public
    void updateSliders();
    void updateSliderValues();
    void updateUiSettings();
    void updateUiOptions();

public slots:
    //used for frame switching
    void SwitchView(void);
    
    void addClip(videoData clip);
    void addLayer(void);
    void addRTLayer(void);
    void addGpuLayer(void);
    //void addPluginLayer(void);
    int  addPluginsToMenu(int usingclass, int existingitems, QPopupMenu *menu);
    void toggleMesh(void);
    void SetGpuSelect(void);

    void grabDesktop(void) { emit GrabClip(); };

    void clicked( QListViewItem* item );

    // plugin based routines
    void doPlugin(QImage& pix, int i);
    void udpatePluginSettings(int pl);

    //slot for the onscreen buffer
    void  updateEffects(void);

    //option selection routines
    void setOption1(void);
    void setOption2(void);
    void setOption3(void);
    void setOption4(void);

    ////////////////////////////////////////////
    // routines to draw effects

    //main calling routine here
    void  callEffect( FxCategory, int, QImage& );

    ////////////////////////////////////////////
    //qt based routines
    void doNegative (QImage& theimage);
    void doSwapRGB (QImage& theimage);
    void doMirror (QImage& theimage);

    //math based functions
    void doGrey(QImage& pix);
    void doContrast(QImage& pix );
    void doBrightness(QImage& pix );

    void doMatrixSharpen(QImage& pix);
    void doMatrixBlur(QImage& pix);
    void doMatrixEmboss(QImage& pix);

    void doEdge(QImage& pix);
    void doColorSelect(QImage& pix);
    //void doBlur(QImage& pix);
    void doGaussianBlur(QImage& pix);

    //opengl accelerated
    void doColorMatrixRt(void);
    void doBlurRt(void);
    void doSharpenRt(void);
    void doContrastRt(void);

    //opengl mesh accelerated functions
    void doFlagRt(void);
    void doRippleRt(void);
    //void doBaseLayer(void);

    //opengl geometry based mesh effects
    void doSwirlRt(void);
    void doFisheyeRt(void);

    //////////////////////////////////////////////////////////////
    //gpu based effects

    //embeded routines
    void doNV30Fire3DGPU(void);
    void doNV30CharcoalGPU(void);
    void doNV30LightingGPU(void);
    void doNV30EarthquakeGPU(void);

    // GPGPU effects
    void doNV30FisheyeGPGPU(QImage& pix);
    void doNV30SwirlGPGPU(QImage& pix);

    ///////////////////////////////////////////////////////////////
    // gpu support routines

    void            get_mesh_normal(float4& normal, int mesh_x_coordinate, int mesh_y_coordinate,
                                    int mesh_x_dimension, int mesh_y_dimension);

signals:
    void    GrabClip(void);    //linked to main jahcontrol to talk to desktop
    void    switchview(void);    //linked to main jahcontrol to do ui switching

protected:

    void objectMenu(int menu);
    void paintGL();
    void buildEffect();

private:

    ////////////////////////////////////////////////////////////////////////////
    // effect based variables

    QImage      sourceimage,savedimage;

    bool        rebuildStatus;
    int         effectobj;
    bool        gpusupport,nv_gpusupport,gpuactive;

    bool        noiseMatrixInitialized;
    int*        permutation_table;

    float4*     gradient_table;

    bool        fireTextureInitialized;
    bool	    randomTextureInitialized;
    bool	    contrastTextureInitialized;

    GLuint      gradient_texture;
    GLuint      permutation_texture;
    GLuint      turbulence_texture;
    GLuint      random_texture;
    GLuint      contrast_texture;

#ifdef NVIDIA_GPGPU 
    GLXFBConfig*                        glxConfig;
    Display*                            shader_display;
    GLXPbuffer                          shader_pbuffer;
    GLXContext                          shader_context;
    float4*								pixel_buffer;
    GLXContext                          jahshaka_glx_context;
    GLXDrawable                         jahshaka_glx_drawable;
    Display*                            jahshaka_display;
#endif

    int         iScreen;

    //The effects timer object
    JahTimer    *fxtimer;

    float       z_offset;
    float       camera_distance;

    ////////////////////////////////////////////////////////////////////////////
    //interface variables
	///////////////////////////////////////////////////////////////////////
	//interface variables for the left and right world space menus
	///////////////////////////////////////////////////////////////////////
	
	QFrame			*glworldLeft;
	JahToolButton	*tooltranslate,	*toolscale,		*toolrotate;
	JahToolButton	*playerphoto,	*switchtheview;

	QFrame			*glworldRight;
	JahToolButton	*toolzoomin,	*toolzoomout;
	QSlider			*zoom;

	///////////////////////////////////////////////////////////////////////
	//animation module ui options
	///////////////////////////////////////////////////////////////////////

	//declare all controllers that we are using here
	enum tabUI {SCENE, OBJECT, AXIS, CLIP, EFFECTS, KEYFRAMES, numTabs};
  
	///////////////////////////////////////////////////////////////////////
	//now the main widget variables

	//the first, main frame
	QFrame*			LayersFrame;
	QFrame*			EffectsFrame;

    //for the control panel
    QPushButton*    AddRTButton;
    QPushButton*    AddGpuButton;
//    QPushButton*    AddPluginButton;

	///////////////////////////////////////////////////////////////////////
	//the tab frame to hold the tabs

	QTabWidget		*tabframe;

	//widegts for each of the tabs
	QWidget			*moduleOptionsUI[numTabs];

	//each module option ui can have up to 10 embedded ui's
    QFrame			*ControlFrameUI[numTabs][10];

	///////////////////////////////////////////////////////////////////////
	// variables for the different tabs

	///////////////////////////////////////////////////////////////////////
	//set up the scene controls

	QPushButton*	ResetButton;             
	QPushButton*	SaveButton;    
	QPushButton*	LoadButton;              
	QPushButton*	ClearButton;
	QPushButton*	PackageButton;
    QCheckBox*      VideoRenderSelect;
    QCheckBox*      GpuSelect;

	//lcdstartframe and endframe are initialized in world

	QCheckBox*		FogSelect;
	QSlider			*HFogSlider,		*VFogSlider;
	InputLCD		*HFogSlider_lcd,	*VFogSlider_lcd;

	//////////////////////////////////////////////////////////////////////
	//set up the different object controls, multiple layers are embedded

	//the world
	QCheckBox		*SmoothSelect;            
	QCheckBox		*TextSelect;
	QCheckBox		*GridSelect;
	QPushButton		*ResetWorldButton;

	//the layer types
	QPushButton		*ObjectLayer;
	QPushButton		*ObjectCube;
	QPushButton		*ObjectCylinder;
	QPushButton		*ObjectSphere;
	QPushButton		*ObjectBezier;
	QPushButton		*ObjectMesh;
	QPushButton		*ResetLayerButton;

	//the font options
	//the particle options

	//the object options obj's
	QPushButton		*loadobject_1,	*objectstyle;
	QPushButton		*ResetObjectButton1;

	//the object options 3dsmax
	QPushButton		*loadmaxobject_1;
	QPushButton		*ResetObjectButton2;

	///////////////////////////////////////////////////////////////////////
	//set up the axis control panel
	//uses the createAxisPanel() subroutine

	///////////////////////////////////////////////////////////////////////
	//set up the texture and clip control panel

	//clip options
	QCheckBox		*ClipSelect;
	QPushButton		*grabdesktopClip; 
	QPushButton		*keyimage_2;       
	QCheckBox		*KeySelect;
	QPushButton		*grabdesktopKey;
	QPushButton		*keyimage_3;
	QPushButton		*keyStatusBox;

	//video options
	QLabel			*VideoLabel;
	InputLCD		*FramesLcd;		
	InputLCD		*inFramesLcd;	
	InputLCD		*outFramesLcd;
	InputLCD		*KeyFramesLcd;	
	InputLCD		*inKeyFramesLcd;	
	InputLCD		*outKeyFramesLcd;
	QToolButton		*lockicon;

	QCheckBox		*extendHeadTail;
	QCheckBox		*toggleTheLoop;
	QCheckBox		*togglePing;

	///////////////////////////////////////////////////////////////////////
	//set up the effects control panel

	QCheckBox		*ForegroundSelect;    
	QCheckBox		*DepthSelect;
	QCheckBox		*BlurSelect;
	QSlider			*HBlurSlider, *VBlurSlider;
	InputLCD		*HBlurSlider_lcd, *VBlurSlider_lcd;

	QCheckBox		*NegativeSelect, *SwapRGBSelect, *MirrorSelect; 
	JahToolButton	*lighticon;
	QCheckBox		*light1, *reflect1, *shader1, *shader2, *smooth1, *mesh1;
	QLabel			*Transparency_1;     
	QSlider			*Trans_1;         
	InputLCD		*Translcd_1;

};



#endif // ANIMATION_H





