/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"


void GLEffect::buildEffect()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QImage renderimage;

    //a hack to keep the smooth settings the way they are?
    GLint               current_shade_model;

    glGetIntegerv(GL_SHADE_MODEL, &current_shade_model);
    glShadeModel(GL_SMOOTH);

    //////////////////////////////////////////////////////////
    //the core effect loop

    if (updateTheEffect())
    {
        //jtrace.debug("GLEffect::buildEffect()","updating effect");

        if (numberLayers>0)
        {
            //grab the source image into buffer to apply effects to
            renderimage = layerlist->at(1)->thelayer->textureData.objectImage.copy();

            if (gpuactive)
                layerlist->at(1)->thelayer->rebuildTextureDataFx(renderimage,true);

            // now we build the effects stacked on top of each other.
            // Using the mesh, we always have to reset the base layer
            // to flat before adding effects on top of eachother
			initializeMeshLayer(1);
            //doBaseLayer();

            //now we build the effects stacked on top of each other using renderimage
            for (int i=2; i<=numberLayers; i++)
            {

                if (layerlist->at(i)->thelayer->layerStatus ) // && effects(i).recalculate to be implemented later
                {
                        callEffect(effects->at(i)->fxlayer->objtype,i,renderimage);
                }
            }

            if (!gpuactive)
                layerlist->at(1)->thelayer->rebuildTextureDataFx(renderimage,true);

            //if (not) scrubbing or playing or rendering update the buffer?
            if (!animation && !animsliderselected && !gpuactive && !renderStatus && useBuffering)
            {
                //debug("updating buffer");
                updateBuffer(&renderimage);
            }

            forceupdatefxbuffer = false;
        }

    }
    else
    {
        //jtrace.debug("GLEffect::buildEffect()","not updating effect");

        if (!gpuactive)
            layerlist->at(1)->thelayer->rebuildTextureDataFx(*fxbuffer,false);
    }

    glShadeModel(current_shade_model);

}



void GLEffect::paintGL()
{
    if ( animation )
    {  
        updateanimation(currentFrame);
        currentFrame++;

        if ( currentFrame > maxFrame )
        currentFrame = 1;
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    if (!select)
    { 
        if ( isLayerVisible(0) )
        {
            renderSpace->drawLayer(*(layerlist->at(0)->thelayer));
        }
        else
        {
            if (renderStatus)
                titlesafe->renderStatus = true;

            renderSpace->drawLayer(*titlesafe);
        }
    }

    //we position the camera here
    renderSpace->positionObject(camera->node);

    //draw the grid if its visible
    if (gridval) 
    {  
            renderSpace->drawLayer(*thegrid); 
    }

    if ( layerlist->at(1) ) 
    {
        if (layerlist->at(1)->thelayer->layerStatus)
        {

            buildEffect();

            renderSpace->drawLayer(*(layerlist->at(1)->thelayer));

            if (gpuactive)
            {
                check_gl();
            }
        }

    }
    glPopMatrix();

    //this would be titlesafe...
    if (textsafe->layerStatus) 
    {  
            renderSpace->drawLayer(*textsafe);  
    }

    glFlush();

    if ( animation ) 
    {
            timer->start( redrawWait, TRUE );                // Wait this many msecs before redraw
    }

    slidersHaveChanged = false;
}

/////////////////////////////////////////////////////////////
// routines to maintain the buffer
// buffer is used to cache effects on a any single frame

void GLEffect::initBuffer(QImage * bufimage)
{
    //debug("initializing buffer");

    int w = bufimage->width();
    int h = bufimage->height();

    if (fxbufferexists)
        delete fxbuffer;

    fxbuffer= new QImage(w,h,32);
    fxbufferexists = true;

    *fxbuffer = bufimage->copy();

}


void GLEffect::updateBuffer(QImage * bufimage)
{
    //debug("called updateBuffer");

    *fxbuffer = bufimage->copy();
}

bool GLEffect::updateTheEffect()
{
    //debug("called updateTheEffect");

    bool updateTheEffect;

    if (useBuffering && !gpuactive)
    {
        updateTheEffect = ( (slidersHaveChanged==true) && (activeLayer!=1)  );

        if (forceupdatefxbuffer)
            updateTheEffect = true;

        //if (scrubbing or playing or rendering)

        if (animation)
            updateTheEffect = true;

        if (animsliderselected)
            updateTheEffect = true;

        if (titlesafe->renderStatus=false)
            updateTheEffect = true;

        if (renderStatus)
            updateTheEffect = true;


    }
    else
    {
        updateTheEffect = true;
    }

    return updateTheEffect;
}

void  GLEffect::updateEffects(void)
{
    //debug("called updateEffects");

    //need to set this option in the layer itself so we can remember it
    //JahModuleOption[0] to [3] -> 4 options

    forceupdatefxbuffer = true;
    updateGL();
}





void GLEffect::get_mesh_normal(float4& normal, int mesh_x_coordinate, int mesh_y_coordinate, 
                                            int mesh_x_dimension, int mesh_y_dimension)
{
    bool						on_the_mesh_left_edge = FALSE;
    bool						on_the_mesh_bottom_edge = FALSE;
    bool						on_the_mesh_right_edge = FALSE;
    bool						on_the_mesh_top_edge = FALSE;
    bool						on_the_mesh_edge;
    float4						vector1;
    float4						vector2;
    float4						point1;
    float4						point2;
    float4						point3;
    float4						normal1;
    float4						normal2;
    float4						normal3;
    float4						normal4;

    point1.w = 1.0;
    point2.w = 1.0;
    point3.w = 1.0;

    if (mesh_x_coordinate == 0)
    {
        on_the_mesh_left_edge = TRUE;
    }

    if (mesh_y_coordinate == 0)
    {
        on_the_mesh_bottom_edge = TRUE;
    }

    if (mesh_x_coordinate == (mesh_x_dimension))
    {
        on_the_mesh_right_edge = TRUE;
    }

    if (mesh_y_coordinate == (mesh_y_dimension))
    {
        on_the_mesh_top_edge = TRUE;
    }

    on_the_mesh_edge = on_the_mesh_left_edge || on_the_mesh_bottom_edge || on_the_mesh_right_edge || on_the_mesh_top_edge;

    if (on_the_mesh_edge)
    {
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate, 
                                                point1.x, point1.y, point1.z);

        if (on_the_mesh_right_edge)
        {
            layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate - 1, mesh_y_coordinate, 
                                                point2.x, point2.y, point2.z);
            get_vector_from_points(vector1, point2, point1);
        }
        else
        {
            layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate + 1, mesh_y_coordinate,
                                                point2.x, point2.y, point2.z);
            get_vector_from_points(vector1, point1, point2);
        }

        if (on_the_mesh_top_edge)
        {
            layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate - 1,
                                                point3.x, point3.y, point3.z);
            get_vector_from_points(vector2, point3, point1);
        }
        else
        {
            layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate + 1,
                                                point3.x, point3.y, point3.z);
            get_vector_from_points(vector2, point1, point3);
        }


        get_cross_product_4(normal, vector1, vector2);
    }
    else
    {
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate,
                                                point1.x, point1.y, point1.z);

        // Upper right quad
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate + 1, mesh_y_coordinate, 
                                                point2.x, point2.y, point2.z);

        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate + 1,
                                                point3.x, point3.y, point3.z);

        get_vector_from_points(vector1, point1, point2);
        get_vector_from_points(vector2, point1, point3);
        get_cross_product_4(normal1, vector1, vector2);

        // Upper left quad
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate + 1,
                                                point2.x, point2.y, point2.z);

        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate - 1, mesh_y_coordinate,
                                                point3.x, point3.y, point3.z);

        get_vector_from_points(vector1, point1, point2);
        get_vector_from_points(vector2, point1, point3);
        get_cross_product_4(normal2, vector1, vector2);

        // Lower left quad
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate - 1, mesh_y_coordinate,
                                                point2.x, point2.y, point2.z);

        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate - 1,
                                                point3.x, point3.y, point3.z);

        get_vector_from_points(vector1, point1, point2);
        get_vector_from_points(vector2, point1, point3);
        get_cross_product_4(normal3, vector1, vector2);

        // Lower right quad
        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate, mesh_y_coordinate - 1,
                                                point2.x, point2.y, point2.z);

        layerlist->at(1)->thelayer->getMeshCoord(mesh_x_coordinate + 1, mesh_y_coordinate,
                                                point3.x, point3.y, point3.z);

        get_vector_from_points(vector1, point1, point2);
        get_vector_from_points(vector2, point1, point3);
        get_cross_product_4(normal4, vector1, vector2);

        normal.x = (normal1.x + normal2.x + normal3.x + normal4.x) / 4;
        normal.y = (normal1.y + normal2.y + normal3.y + normal4.y) / 4;
        normal.z = (normal1.z + normal2.z + normal3.z + normal4.z) / 4;
        normal.w = 1.0;
// 		normal.w = (normal1.w + normal2.w + normal3.w + normal4.w) / 4;
    }

    normalize_vector4(normal);
}






