/*******************************************************************************
**
** The source file for the Jahshaka effects objects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"

//we need to update the add-effect menu
//break menu into rt and processor based

void GLEffect::callEffect( FxCategory thefx,  int i, QImage& newimage ) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "GLEffect::","building effect#",i);

    effectobj = i;

    switch ( thefx ) {

     //plugin image processing
      case JAHPLUGIN                  : {  doPlugin(newimage,i); break; }

      //qt image processing
      case NEGATIVE                   : {  doNegative(newimage);                  break; }
      case SWAPRGB                    : {  doSwapRGB(newimage);                   break; }
      case MIRROR                     : {  doMirror(newimage);                    break; }
      case GREY                       : {  doGrey(newimage);                      break; }
      case CONTRAST                   : {  doContrast(newimage);                  break; }
      case BRIGHTNESS                 : {  doBrightness(newimage);                break; }
      case MATRIXSHARPEN              : {  doMatrixSharpen(newimage);             break; }
      case MATRIXBLUR                 : {  doMatrixBlur(newimage);                break; }
      case MATRIXEMBOSS               : {  doMatrixEmboss(newimage);              break; }
      case EDGE                       : {  doEdge(newimage);                      break; }
      case SELECTCOLORS               : {  doColorSelect(newimage);               break; }
      //case BLUR                       : {  doBlur(newimage);                      break; }
      case GAUSSIANBLUR               : {  doGaussianBlur(newimage);              break; }

      //for the gpu based effects
      case FIRE3D_NV30_GPU            : {  doNV30Fire3DGPU();                     break; }
      case CHARCOAL_NV30_GPU          : {  doNV30CharcoalGPU();                   break; }
      case LIGHTING_NV30_GPU          : {  doNV30LightingGPU();                   break; }
      case EARTHQUAKE_NV30_GPU        : {  doNV30EarthquakeGPU();                 break; }

      //opengl based realtimefx
      case COLORMATRIX                : {  doColorMatrixRt();                     break; }
      case SHARPEN                    : {  break; }
      case CONTRASTRT                 : {  break; }

      //opengl geometry effects here using meshlayer
      case SWIRLRT                    : {  doSwirlRt();                           break; }
      case FISHEYERT                  : {  doFisheyeRt();                         break; }
      case FLAGRT                     : {  doFlagRt();                            break; }
      case RIPPLERT                   : {  doRippleRt();                          break; }

      #ifdef NVIDIA_GPGPU
      case FISHEYE_NV30_GPGPU         : {  doNV30FisheyeGPGPU(newimage);          break; }
      case SWIRL_NV30_GPGPU           : {  doNV30SwirlGPGPU(newimage);            break; }
      #endif // NVIDIA_GPGPU

      default                         : {  break; }

    }

}

//////////////////////////////////////////////////////////
//these are Qt placeholders for GPGPU if its not in use

#ifndef NVIDIA_GPGPU // !NVIDIA_GPGPU
void GLEffect::doNV30FisheyeGPGPU(QImage& pix)
{
// Placeholder to keep Qt happy
//    int* dummy = (int*)&pix;
//    dummy = dummy;
}
void GLEffect::doNV30SwirlGPGPU(QImage& pix)
{
// Placeholder to keep Qt happy
//    int* dummy = (int*)&pix;
//    dummy = dummy;
}

#endif // NVIDIA_GPGPU


// this is for reference
/*
class effectLayer {
public:
    effectLayer(void);
    ~effectLayer(void);
    int         effectnumber;
    QString     effectname;
    FxCategory  objtype;
    bool        selected;
    bool        layerStatus;
    bool        isOn;
    GLfloat     x,y,z;
    GLfloat     r;
    bool        active;
};
*/


