/*******************************************************************************
**
** The source file for the Jahshaka effects gpgpu module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"

#ifdef NVIDIA_GPGPU

void GLEffect::doNV30FisheyeGPGPU(QImage& pix) {

    static int                                  count = 0;

    GLuint                                      texture_id[10];
    GLuint                                      program_id[10];
    GLint                                       width;
    GLint                                       half_width;
    GLint                                       height;
    unsigned char*                              source;

    float                                       fish_eye_radius;
    float                                       xcenter;
    float                                       ycenter;
    float                                       gamma;

    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    half_width = width / 2;

    float float_width = (float)width;
    float float_height = (float)height;

    xcenter = layerlist->at(effectobj)->thelayer->node->tx + (float_width / 2.0);
    ycenter = layerlist->at(effectobj)->thelayer->node->ty + (float_height / 2.0);
    fish_eye_radius = (layerlist->at(effectobj)->thelayer->node->rx * 2.0) + 2.0;
    gamma  = (layerlist->at(effectobj)->thelayer->node->ry * 0.01) + 2.0;

    QImage newimage = pix.copy();   

    count++;

    jahshaka_display = XOpenDisplay(NULL);
    jahshaka_glx_context = glXGetCurrentContext();
    jahshaka_glx_drawable = glXGetCurrentDrawable();

    if (pixel_buffer == NULL)
    {
        pixel_buffer = (float4*)new float[height * half_width * 4];
    }

    if (shader_pbuffer == 0)
    {
        create_pbuffer(width, height);
    }

    check_gl();


    glXMakeCurrent(shader_display, shader_pbuffer, shader_context);

    glDrawBuffer(GL_FRONT);
    glReadBuffer(GL_FRONT);

    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glEnable(GL_TEXTURE_RECTANGLE_NV);

    glGenTextures(1, &texture_id[0]);
    glActiveTextureARB(GL_TEXTURE0_ARB);

    glBindTexture (GL_TEXTURE_RECTANGLE_NV, texture_id[0]);

    glTexImage2D(GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGBA32_NV, 
        half_width, height, 0, GL_RGBA, GL_FLOAT, NULL);

    source = loadshaderfile(JahBasePath+"jah/libraries/shader/fisheye_frag_gpgpu.fp");
    glGenProgramsNV(1, &program_id[0]);

    check_gl();

    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, program_id[0], strlen((char*)source), (const GLubyte*)source);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(source, "fisheye_frag_gpgpu.fp");
    }


    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, program_id[0]);
    glViewport(0, 0, half_width, height);

    check_gl();

    glProgramNamedParameter4fNV(program_id[0], strlen("fish_eye_radius"), 
        (const GLubyte*)"fish_eye_radius", fish_eye_radius, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("xcenter"), 
        (const GLubyte*)"xcenter", xcenter, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("ycenter"), 
        (const GLubyte*)"ycenter", ycenter, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("gamma"), 
        (const GLubyte*)"gamma", gamma, 0.0f, 0.0f, 0.0f);

    check_gl();

    float                    vff = 0.49952;    // Vertical fudge factor

    glBegin(GL_QUADS);
    {
        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, -0.5f, (float)height - vff, 0.0f, 1.0f);
        glVertex2f(-1.0f, 1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, -0.5f, -vff, 0.0f, 1.0f);
        glVertex2f(-1.0f, -1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, (float)half_width - 0.5, -vff, 0.0f, 1.0f);
        glVertex2f(1.0f, -1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, (float)half_width - 0.5, (float)height - vff, 0.0f, 1.0f);
        glVertex2f(1.0f, 1.0f);
    }
    glEnd();

    check_gl();
    glCopyTexSubImage2D(GL_TEXTURE_RECTANGLE_NV, 0, 0, 0, 0, 0, half_width, height);

    glGetTexImage(GL_TEXTURE_RECTANGLE_NV, 0, GL_RGBA, GL_FLOAT, (void*)pixel_buffer);

    uint*                           pnew;
    uint*                           pold;
    int                             successfully_restored;
    int                             offset;

    for (int x = 0; x < half_width; x++)
    {
        for (int y = 0; y < height; y++)
        {
            offset = (y * half_width) + x;

            if (pixel_buffer[offset].x >= 0.000000f && pixel_buffer[offset].x < width && 
                pixel_buffer[offset].y >= 0.000000f && pixel_buffer[offset].y < height)
            {
                pnew = (uint *)newimage.scanLine(y) + (2 * x);
                pold = (uint *)pix.scanLine((int)pixel_buffer[offset].y) + (int)pixel_buffer[offset].x;
                *pnew = *pold;
            }

            if (pixel_buffer[offset].z >= 0.000000f && pixel_buffer[offset].z < width && 
                pixel_buffer[offset].w >= 0.000000f && pixel_buffer[offset].w < height)
            {
                pnew = (uint *)newimage.scanLine(y) + (2 * x) + 1;
                pold = (uint *)pix.scanLine((int)pixel_buffer[offset].w) + (int)pixel_buffer[offset].z;
                *pnew = *pold;
            }
        }
    }

    glDeleteTextures(1, &texture_id[0]);

    successfully_restored = glXMakeCurrent(jahshaka_display, jahshaka_glx_drawable, jahshaka_glx_context);

    pix = newimage;

    fxtimer->endtime();
    fxtimer->printtime("Time for GPGPU FishEye: ");

}

void GLEffect::doNV30SwirlGPGPU(QImage& pix) {

    static int                                  count = 0;

    GLuint                                      texture_id[10];
    GLuint                                      program_id[10];
    GLint                                       width;
    GLint                                       half_width;
    GLint                                       height;
    unsigned char*                              source;

    float                                       swirl_radius;
    float                                       xcenter;
    float                                       ycenter;
    float                                       twist_effect;

    check_gl();

    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    half_width = width / 2;

    float float_width = (float)width;
    float float_height = (float)height;

    xcenter = layerlist->at(effectobj)->thelayer->node->tx + (float_width / 2.0);
    ycenter = layerlist->at(effectobj)->thelayer->node->ty + (float_height / 2.0);
    swirl_radius = (layerlist->at(effectobj)->thelayer->node->rx * 3.0) + (float_width / 10);
    twist_effect  = (layerlist->at(effectobj)->thelayer->node->ry / 10.0) + 1.0;

    QImage newimage = pix.copy();   

    count++;
    printf("In GLEffect::doNV30Swirl  Count = %d\n", count);

    jahshaka_display = XOpenDisplay(NULL);
    jahshaka_glx_context = glXGetCurrentContext();
    jahshaka_glx_drawable = glXGetCurrentDrawable();

    if (pixel_buffer == NULL)
    {
        pixel_buffer = (float4*)new float[PIXEL_BUFFER_WIDTH * PIXEL_BUFFER_HEIGHT * 4];
    }

    if (shader_pbuffer == 0)
    {
        create_pbuffer(PIXEL_BUFFER_WIDTH, PIXEL_BUFFER_HEIGHT);
    }

    check_gl();


    glXMakeCurrent(shader_display, shader_pbuffer, shader_context);

    glDrawBuffer(GL_FRONT);
    glReadBuffer(GL_FRONT);

    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glEnable(GL_TEXTURE_RECTANGLE_NV);

    glGenTextures(1, &texture_id[0]);

    glActiveTextureARB(GL_TEXTURE0_ARB);

    glBindTexture (GL_TEXTURE_RECTANGLE_NV, texture_id[0]);

    glTexImage2D(GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGBA32_NV, half_width, height, 0, GL_RGBA, GL_FLOAT, NULL);

    source = loadshaderfile(JahBasePath+"jah/libraries/shader/swirl_frag_gpgpu.fp");
    glGenProgramsNV(1, &program_id[0]);

    check_gl();

    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, program_id[0], strlen((char*)source), (const GLubyte*)source);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(source, "swirl_frag_gpgpu.fp");
    }


    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, program_id[0]);
    glViewport(0, 0, half_width, height);

    check_gl();

    glProgramNamedParameter4fNV(program_id[0], strlen("swirl_radius"), 
        (const GLubyte*)"swirl_radius", swirl_radius, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("xcenter"), 
        (const GLubyte*)"xcenter", xcenter, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("ycenter"), 
        (const GLubyte*)"ycenter", ycenter, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(program_id[0], strlen("twist_effect"), 
        (const GLubyte*)"twist_effect", twist_effect, 0.0f, 0.0f, 0.0f);

    check_gl();

    float                    vff = 0.49952;    // Vertical fudge factor

    glBegin(GL_QUADS);
    {
        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, -0.5f, (float)height - vff, 0.0f, 1.0f);
        glVertex2f(-1.0f, 1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, -0.5f, -vff, 0.0f, 1.0f);
        glVertex2f(-1.0f, -1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, (float)half_width - 0.5, -vff, 0.0f, 1.0f);
        glVertex2f(1.0f, -1.0f);

        glMultiTexCoord4fARB(GL_TEXTURE0_ARB, (float)half_width - 0.5, (float)height - vff, 0.0f, 1.0f);
        glVertex2f(1.0f, 1.0f);
    }

    glEnd();

    check_gl();


    //glCopyTexImage2D(GL_TEXTURE_RECTANGLE_NV, 0, GL_FLOAT_RGBA32_NV, 0, 0, half_width, height, 0);
    glCopyTexSubImage2D(GL_TEXTURE_RECTANGLE_NV, 0, 0, 0, 0, 0, half_width, height);

    glGetTexImage(GL_TEXTURE_RECTANGLE_NV, 0, GL_RGBA, GL_FLOAT, (void*)pixel_buffer);

    uint*                           pnew;
    uint*                           pold;
    int                             successfully_restored;
    int                             offset;

    for (int x = 0; x < half_width; x++)
    {
        for (int y = 0; y < height; y++)
        {
            offset = (y * half_width) + x;

//             printf("x = %d  y = %d  (%6.2f %6.2f)  (%6.2f %6.2f)\n", x, y, 
//                     pixel_buffer[offset].x, pixel_buffer[offset].y,
//                     pixel_buffer[offset].z, pixel_buffer[offset].w);

            if (pixel_buffer[offset].y >= 0.0 && pixel_buffer[offset].y < height && 
                pixel_buffer[offset].x >= 0.0 && pixel_buffer[offset].x < width)
            {
                pnew = (uint *)newimage.scanLine(y) + (2 * x);
                pold = (uint *)pix.scanLine((int)pixel_buffer[offset].y) + (int)pixel_buffer[offset].x;
                *pnew = *pold;
            }

            if (pixel_buffer[offset].w >= 0.0 && pixel_buffer[offset].w < height && 
                pixel_buffer[offset].z >= 0.0 && pixel_buffer[offset].z < width)
            {
                pnew = (uint *)newimage.scanLine(y) + (2 * x) + 1;
                pold = (uint *)pix.scanLine((int)pixel_buffer[offset].w) + (int)pixel_buffer[offset].z;
                *pnew = *pold;
            }
        }
    }

    glDeleteTextures(1, &texture_id[0]);

    successfully_restored = glXMakeCurrent(jahshaka_display, jahshaka_glx_drawable, jahshaka_glx_context);

    pix = newimage;

    fxtimer->endtime();
    fxtimer->printtime("Time for NV30 GPGPU Swirl: ");

}

#endif
