/*******************************************************************************
**
** The source file for the Jahshaka effects objects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"


////////////////////////////  NV30 Effects ////////////////////////////////


////////////////////////////////////////////////////////
// experimental work using offscreen rendering here
// for the gpgpu effects

void GLEffect::doNV30EarthquakeGPU(void) {

    GLint                                       width;
    GLint                                       height;

    int                                         horizontal_mesh_size;
    int                                         vertical_mesh_size;
    float                                       float_horizontal_mesh_size;
    float                                       float_vertical_mesh_size;

    float                                       no_distortion_radius;

    float4                                      Displacement;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    static int                                  count = 0;

    static float4                               noise_scale;
    static float4                               noise_translate;

    float2                                      movement;

    int                                         ii, jj;
    GLfloat                                     i, j;

    int                                         num_distortion_layers;
    int                                         speed;

    //////////////////////////////////////////////////////////////////////

    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    float float_width = (float)width;
    float float_height = (float)height;

    ////////////////////////////////////////////////////////////////////////

    Displacement.x = -(layerlist->at(effectobj)->thelayer->node->tx / 1.0) + 27.0;
    Displacement.y = -(layerlist->at(effectobj)->thelayer->node->ty / 1.0) + 27.0;
    Displacement.z = (layerlist->at(effectobj)->thelayer->node->tz) + 80.0;

    num_distortion_layers = (int)(1.0 + (layerlist->at(effectobj)->thelayer->node->rx / 10.0));
    speed = 10 + (int)(layerlist->at(effectobj)->thelayer->node->ry / 10.0);
    no_distortion_radius = layerlist->at(effectobj)->thelayer->node->rz;


#define USE_OFFSCREEN_RENDERING         FALSE

#if USE_OFFSCREEN_RENDERING
    jahshaka_display = XOpenDisplay(NULL);
    jahshaka_glx_context = glXGetCurrentContext();
    jahshaka_glx_drawable = glXGetCurrentDrawable();

    if (shader_pbuffer == 0)
    {
        create_pbuffer(PIXEL_BUFFER_WIDTH, PIXEL_BUFFER_HEIGHT);
    }

    glXMakeCurrent(shader_display, shader_pbuffer, shader_context);

    glViewport(0, 0, width, height);

    glDrawBuffer(GL_FRONT);
    glReadBuffer(GL_FRONT);
#endif  // USE_OFFSCREEN_RENDERING

    horizontal_mesh_size = 3;
    vertical_mesh_size = 2;

    movement.x = (layerlist->at(effectobj)->thelayer->node->sx * 50.0) + 11.0;
    movement.y = movement.x;

    float_horizontal_mesh_size = (float)horizontal_mesh_size;
    float_vertical_mesh_size = (float)vertical_mesh_size;

    noise_scale.x = 1.3;
    noise_scale.y = 1.3;
    noise_scale.z = 1.3;

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);

    if (!noiseMatrixInitialized)
    {
        initVertexNoiseConstants(DISTORTION_TABLE_SIZE, permutation_table, gradient_table,GRADIENT_TABLE_SIZE);
//        initializeMesh(float_width, float_height);
        noiseMatrixInitialized = true;
    }

    count++;

    ////////////////////////////////////////////////////////////////////

    vertex_program = loadshaderfile(JahBasePath+"jah/libraries/shader/earthquake_vert_gpu.vp");
    glGenProgramsNV(1, &vertex_program_handle);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);
    glLoadProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle, strlen((char*)vertex_program), vertex_program);

    ////////////////////////////////////////////////////////////////////

    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW_PROJECTION_NV, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_TEXTURE, GL_IDENTITY_NV);

    glEnable(GL_VERTEX_PROGRAM_NV);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);

    glProgramParameters4fvNV(GL_VERTEX_PROGRAM_NV, 8, DISTORTION_TABLE_SIZE * 2, (float*)gradient_table);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 8 + (DISTORTION_TABLE_SIZE * 2), (float*)&gradient_table[0]);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 8 + (DISTORTION_TABLE_SIZE * 2) + 1, (float*)&gradient_table[1]);

    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 74, float_height, 0.0, 0.0, 0.0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 75, float_height / 2.0, 0.0, 0.0, 0.0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 76, no_distortion_radius, 0.0, 0.0, 0.0);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 77, (float*)&Displacement.x);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 78, 10000.0, 10000.0, 10000.0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 79, 0.37, 0.5, 1.0, 2.0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 80, 0.03125, 32.0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 81, 1.0, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 82, 3.0, 3.0, 3.0, 0);

    ////////////////////////////////////////////////////////////////////

    fragment_program = loadshaderfile(JahBasePath+"jah/libraries/shader/earthquake_frag_gpu.fp");
    glGenProgramsNV(1, &fragment_program_handle);
    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle,
                    strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(fragment_program, "earthquake_frag_gpu.fp");
    }

    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle);


    /////////////////////////////////////////////////////////////////////

    int           current_matrix_mode;

    glGetIntegerv(GL_MATRIX_MODE, &current_matrix_mode);
    glMatrixMode( GL_TEXTURE );
    glLoadIdentity();

    //draws texturized flame layers and blends them
    for (int layer = 0; layer < num_distortion_layers; ++layer)
    {
        glPushMatrix();

        noise_translate.x = (float)count / movement.x;
        noise_translate.y = (float)count / movement.y;
        noise_translate.z = (float)count / 300.0 / movement.y;


            //animate noise by scaling and translating
        glScalef(noise_scale.x, noise_scale.y, noise_scale.z);
        glTranslatef(noise_translate.x, noise_translate.y, noise_translate.z);

        GLfloat                 x_unit_size = width / (float)horizontal_mesh_size;
        GLfloat                 y_unit_size = height / (float)vertical_mesh_size;

        glBegin(GL_TRIANGLES);

        for (jj = 0; jj < vertical_mesh_size; jj++) 
        {
            j = (float)jj;

            for (ii = 0; ii < horizontal_mesh_size; ii++) 
            {
                i = (float)ii;

                // First Triangle
                glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                    ((i + 0) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055,  
                    ((j + 0) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0);
                glVertex3f( (-float_width / 2.0) + (i * x_unit_size), (-float_height / 2.0) + (j * y_unit_size), 0.0);

                glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                    ((i + 1) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055, 
                    ((j + 0) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0); 
                glVertex3f( (-float_width / 2.0) + ((i + 1) * x_unit_size), (-float_height / 2.0) + (j * y_unit_size), 0.0);

                glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                    ((i + 0) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055, 
                    ((j + 1) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0);
                glVertex3f( (-float_width / 2.0) + (i * x_unit_size), (-float_height / 2.0) + ((j + 1) * y_unit_size), 0.0);


                // Second Triangle
                glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                    ((i + 1) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055, 
                    ((j + 1) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0);
                glVertex3f( (-float_width / 2.0) + ((i + 1) * x_unit_size), (-float_height / 2.0) + ((j + 1) * y_unit_size), 0.0);

                glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                    ((i + 1) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055,
                    ((j + 0) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0);
                glVertex3f( (-float_width / 2.0) + ((i + 1) * x_unit_size), (-float_height / 2.0) + (j * y_unit_size), 0.0);

                glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                    ((i + 0) / float_horizontal_mesh_size) * layerlist->at(1)->thelayer->textureData.texwidthratio + 0.00055,
                    ((j + 1) / float_vertical_mesh_size) * layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0);
                glVertex3f( (-float_width / 2.0) + (i * x_unit_size), (-float_height / 2.0) + ((j + 1) * y_unit_size), 0.0);

            }
        }

        glEnd(); //GL_QUADS

        glPopMatrix();   // GL_TEXTURE

        glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (core->renderwidth - width) / 2,
                        (core->renderheight - height) / 2, width, height);

    }

    ////////////////////////////////////////////////////////////////////////
    
    glDisable(GL_VERTEX_PROGRAM_NV);
    glDisable(GL_FRAGMENT_PROGRAM_NV);
    delete vertex_program;
    delete fragment_program;

    glDeleteProgramsNV(1, &vertex_program_handle);
    glDeleteProgramsNV(1, &fragment_program_handle);

#if USE_OFFSCREEN_RENDERING
    glXMakeCurrent(jahshaka_display, jahshaka_glx_drawable, jahshaka_glx_context);
#endif  // USE_OFFSCREEN_RENDERING

    glMatrixMode(current_matrix_mode);

    fxtimer->endtime();
    fxtimer->printtime("Time for NV30 Earthquake: ");
}


//////////////////////////////////////////////////////////////////////////
//cant us this since we moved the turbulence generator out
//but cant move this out yet since we cant do meshfx in plugins yet

void GLEffect::doNV30Fire3DGPU(void) {
/*

    GLint                                       width;
    GLint                                       height;

    int                                         number_of_layers;

    int                                         horizontal_mesh_size;
    int                                         vertical_mesh_size;
    static int                                  previous_horizontal_mesh_size = 0;
    static int                                  previous_vertical_mesh_size = 0;
    float                                       float_horizontal_mesh_size;
    float                                       float_vertical_mesh_size;

    float2                                      distortion;
    float                                       gradient;
    float                                       cosine_factor;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    static int                                  count = 0;

    float                                       intensity;
    float                                       timescale;


    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    float float_width = (float)width;
    float float_height = (float)height;

    timescale = (layerlist->at(effectobj)->thelayer->node->tx / 5.0) + 1.0;
    gradient = layerlist->at(effectobj)->thelayer->node->ty / 100.0;
    cosine_factor = -layerlist->at(effectobj)->thelayer->node->tz / 100.0;

    number_of_layers = 5;
//     number_of_layers = (int)(layerlist->at(effectobj)->thelayer->node->tz / 10.0) + 5;
//  number_of_layers = (number_of_layers > 5) ? 5 : number_of_layers;

    distortion.x = ((layerlist->at(effectobj)->thelayer->node->rx) / 100.0) + 0.1;
    distortion.y = ((layerlist->at(effectobj)->thelayer->node->ry) / 100.0) + 0.1;
    intensity = (layerlist->at(effectobj)->thelayer->node->rz / 10.0) + 3.0;

    horizontal_mesh_size = (int)((layerlist->at(effectobj)->thelayer->node->sx * 10.0) + 5.0);
    horizontal_mesh_size = (horizontal_mesh_size > width) ? width : horizontal_mesh_size;

    vertical_mesh_size = ((int)(layerlist->at(effectobj)->thelayer->node->sy * 10));
    vertical_mesh_size = (vertical_mesh_size > height) ? height : vertical_mesh_size;

    float_horizontal_mesh_size = (float)horizontal_mesh_size;
    float_vertical_mesh_size = (float)vertical_mesh_size;

    createTurbulenceTextureGPU(width, height, intensity, horizontal_mesh_size, vertical_mesh_size, number_of_layers);

    vertex_program = loadshaderfile(JahBasePath+"jah/libraries/shader/fire3D_vert_gpu.vp");
    glGenProgramsNV(1, &vertex_program_handle);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);
    glLoadProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle, strlen((char*)vertex_program), vertex_program);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW_PROJECTION_NV, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_MODELVIEW, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 8, GL_MODELVIEW, GL_INVERSE_TRANSPOSE_NV);
    glEnable(GL_VERTEX_PROGRAM_NV);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);

    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 12, width, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 13, height, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 14, camera_distance, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 15, 2.0, 0, 0, 0);

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, turbulence_texture);

    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);

    fragment_program = loadshaderfile(JahBasePath+"jah/libraries/shader/fire3D_frag_gpu.fp");
    glGenProgramsNV(1, &fragment_program_handle);
    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle, 
                    strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(fragment_program, "fire3D_frag_gpu.fp");
    }


    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle);

    float scale_factor_x = (float)width / (float)TURBULENCE_TEXTURE_SIZE;
    float scale_factor_y = (float)height / (float)TURBULENCE_TEXTURE_SIZE;

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("scale_factor"), 
        (const GLubyte*)"scale_factor", scale_factor_x, scale_factor_y, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("distortion"), 
        (const GLubyte*)"distortion", distortion.x, distortion.y, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("time"), 
        (const GLubyte*)"time", (float)count / 100.0, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("timescale"), 
        (const GLubyte*)"timescale", timescale, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("gradient"), 
        (const GLubyte*)"gradient", gradient, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("cosine_factor"), 
        (const GLubyte*)"cosine_factor", cosine_factor, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("base_scale_factor"), 
                            (const GLubyte*)"base_scale_factor", 
                            layerlist->at(1)->thelayer->textureData.texwidthratio, 
                            layerlist->at(1)->thelayer->textureData.texheightratio, 0.0f, 0.0f);



    glBegin(GL_QUADS);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, 0.0, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, 0.0, 0.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                                layerlist->at(1)->thelayer->textureData.texwidthratio, 0.0, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, 1.0, 0.0, 0.0, 1.0);
            glVertex2f( float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                                layerlist->at(1)->thelayer->textureData.texwidthratio, 
                                layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, 1.0, 1.0, 0.0, 1.0);
            glVertex2f( float_width / 2.0, float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, layerlist->at(1)->thelayer->textureData.texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, 0.0, 1.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, float_height / 2.0);

    glEnd();


    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (core->renderwidth - width) / 2, 
                        (core->renderheight - height) / 2, width, height);

    glDisable(GL_VERTEX_PROGRAM_NV);
    glDisable(GL_FRAGMENT_PROGRAM_NV);
    delete vertex_program;
    delete fragment_program;

    glDeleteProgramsNV(1, &vertex_program_handle);
    glDeleteProgramsNV(1, &fragment_program_handle);

    float_vertical_mesh_size = (float)MESH_Y_DIMENSION;
    float_horizontal_mesh_size = (float)MESH_X_DIMENSION;

    float2                          base_scale_factor;
    int                             ii, jj;
    float                           i, j;
    float                           y_unit_size = height / float_vertical_mesh_size;

    float4                          position;
    float2                          radians;
    float4                          current_position;
;

    base_scale_factor.x = layerlist->at(1)->thelayer->textureData.texwidthratio;
    base_scale_factor.y = layerlist->at(1)->thelayer->textureData.texheightratio;

    for (jj = 0; jj <= MESH_Y_DIMENSION; jj++) 
    {
        j = (float)jj;

        for (ii = 0; ii <= MESH_X_DIMENSION; ii++) 
        {   
            i = (float)ii;

            layerlist->at(1)->thelayer->getMeshCoord(ii, jj, current_position.x, current_position.y, current_position.z);

            radians.x = (i + 0) * 2 * PI / float_horizontal_mesh_size;
            radians.y = (j + 0) * 2 * PI / float_vertical_mesh_size;

            position.x = ((float_width / 2.0 * -sin(radians.x)) + (20.0 * cos(radians.y + (count / 10.0)))) * 
//                       ((2 * PI) - radians.y) / (4 * PI);
                            cos(radians.y - PI) / 2.0;
            position.y = (-float_height / 2.0) + ((j + 0) * y_unit_size);
            position.z = current_position.z + ((-cos(radians.x) * float_width / 2.0) * ((2 * PI) - radians.y) / (4 * PI));

            layerlist->at(1)->thelayer->setMeshCoord(ii, jj, position.x, position.y, position.z);
        }
    }



    count++;

    previous_horizontal_mesh_size = horizontal_mesh_size;
    previous_vertical_mesh_size = vertical_mesh_size;

    fxtimer->endtime();
    fxtimer->printtime("Time for NV30 3D Fire: ");
    */
}


void GLEffect::doNV30CharcoalGPU(void) {

    GLint                                       width;
    GLint                                       height;

    float4                                      light1_position;
    float4                                      light2_position;

    float                                       transparency;
    float                                       blend;
    float                                       random_factor;

    double                                      density;
    static double                               previous_density = 0.0;
    double                                      exponent;
    static double                               previous_exponent = 0.0;
    double                                      darker;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    int                                         row;
    int                                         column;

    float                                       f_row;
    float                                       f_column;

    float2                                      lower_left_tex_coords;
    float2                                      lower_right_tex_coords;
    float2                                      upper_left_tex_coords;
    float2                                      upper_right_tex_coords;

    float4                                      lower_left_vertex_coords;
    float4                                      lower_right_vertex_coords;
    float4                                      upper_left_vertex_coords;
    float4                                      upper_right_vertex_coords;

    float4                                      lower_left_normal_coords;
    float4                                      lower_right_normal_coords;
    float4                                      upper_left_normal_coords;
    float4                                      upper_right_normal_coords;

    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    light1_position.x = (layerlist->at(effectobj)->thelayer->node->rx * 5) + 200.0;
    light1_position.y = (layerlist->at(effectobj)->thelayer->node->ry * 5);
    light1_position.z = (layerlist->at(effectobj)->thelayer->node->rz * 5) + 200.0;
    light1_position.w = 1.0;

    transparency = -(layerlist->at(effectobj)->thelayer->node->tx / 100.0) + 1.5;
    blend = (layerlist->at(effectobj)->thelayer->node->ty / 100.0) + 1.0;
    random_factor = (layerlist->at(effectobj)->thelayer->node->tz / 100.0) + 1.0;

    light2_position.x = light1_position.x;
    light2_position.y = light1_position.y;
    light2_position.z = light1_position.z;
    light2_position.w = light1_position.w;

    density = (double)(((layerlist->at(effectobj)->thelayer->node->sx)) + 2.0);
    exponent = (double)(((layerlist->at(effectobj)->thelayer->node->sy - 1.0)) + 3.5);
    darker = (double)(((layerlist->at(effectobj)->thelayer->node->sz - 1.0) / 10.0) + 0.1);
    //printf("darker = %f\n", darker);

    if (!randomTextureInitialized)
    {
        create_random_texture(512, 512, random_texture);
        randomTextureInitialized = TRUE;
    }

    if (!contrastTextureInitialized || (exponent != previous_exponent))
    {
        if (contrastTextureInitialized)
        {
            glDeleteTextures(1, &contrast_texture);
        }

        create_contrast_texture(1024, 512, density, exponent, contrast_texture);
        contrastTextureInitialized = TRUE;
    }

    vertex_program = loadshaderfile(JahBasePath+"jah/libraries/shader/charcoal_vert_gpu.vp");
    glGenProgramsNV(1, &vertex_program_handle);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);
    glLoadProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle, strlen((char*)vertex_program), vertex_program);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW_PROJECTION_NV, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_MODELVIEW, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 8, GL_MODELVIEW, GL_INVERSE_TRANSPOSE_NV);
    glEnable(GL_VERTEX_PROGRAM_NV);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);


    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 12, 0.0, 1.0, 1.0, 1.0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 13, 0.0, 1.0, 1.0, 1.0);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 14, (float*)&light1_position);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 15, (float*)&light2_position);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 16, width, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 17, height, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 18, camera_distance, 0, 0, 0);

    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 19, 2, 0, 1, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 20, 0.5, 0.5, 0, 0);


    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);

    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D, contrast_texture);

    glActiveTextureARB(GL_TEXTURE2_ARB);
    glBindTexture(GL_TEXTURE_2D, random_texture);

    fragment_program = loadshaderfile(JahBasePath+"jah/libraries/shader/charcoal_frag_gpu.fp");
    glGenProgramsNV(1, &fragment_program_handle);
    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle, 
                    strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(fragment_program, "charcoal_frag_gpu.fp");
    }

    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("darker"), 
        (const GLubyte*)"darker", darker, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("transparency"), 
        (const GLubyte*)"transparency", transparency, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("blend"), 
        (const GLubyte*)"blend", blend, 0.0f, 0.0f, 0.0f);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("random_factor"), 
        (const GLubyte*)"random_factor", random_factor, 0.0f, 0.0f, 0.0f);


    float texwidthratio = layerlist->at(1)->thelayer->textureData.texwidthratio;
    float texheightratio = layerlist->at(1)->thelayer->textureData.texheightratio;



    glBegin(GL_QUADS);

    for (row = 0; row < JAHMESH_Y_DIMENSION; row++)
    {
        f_row = (float)row;

        for (column = 0; column < JAHMESH_X_DIMENSION; column++)
        {
            f_column = (float)column;

            lower_left_tex_coords.x = (f_column + 0.0) / JAHMESH_X_DIMENSION;
            lower_left_tex_coords.y = (f_row + 0.0) / JAHMESH_Y_DIMENSION;

            lower_right_tex_coords.x = (f_column + 1.0) / JAHMESH_X_DIMENSION;
            lower_right_tex_coords.y = (f_row + 0.0) / JAHMESH_Y_DIMENSION;

            upper_right_tex_coords.x = (f_column + 1.0) / JAHMESH_X_DIMENSION;
            upper_right_tex_coords.y = (f_row + 1.0) / JAHMESH_Y_DIMENSION;

            upper_left_tex_coords.x = (f_column + 0.0) / JAHMESH_X_DIMENSION;
            upper_left_tex_coords.y = (f_row + 1.0) / JAHMESH_Y_DIMENSION;

            get_mesh_normal(lower_left_normal_coords, column, row, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(lower_right_normal_coords, column + 1, row, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(upper_right_normal_coords, column + 1, row + 1, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(upper_left_normal_coords, column, row + 1, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);

            layerlist->at(1)->thelayer->getMeshCoord(column, row,
                                                    lower_left_vertex_coords.x,
                                                    lower_left_vertex_coords.y,
                                                    lower_left_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column + 1, row,
                                                    lower_right_vertex_coords.x,
                                                    lower_right_vertex_coords.y,
                                                    lower_right_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column + 1, row + 1,
                                                    upper_right_vertex_coords.x,
                                                    upper_right_vertex_coords.y,
                                                    upper_right_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column, row + 1,
                                                    upper_left_vertex_coords.x,
                                                    upper_left_vertex_coords.y,
                                                    upper_left_vertex_coords.z);



            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, lower_left_tex_coords.x * texwidthratio, 
                                                lower_left_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, lower_left_tex_coords.x, lower_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, lower_left_tex_coords.x / density, lower_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(lower_left_normal_coords.x, lower_left_normal_coords.y, lower_left_normal_coords.z);
//             glVertex3f(lower_left_vertex_coords.x, lower_left_vertex_coords.y, lower_left_vertex_coords.z);
            glVertex2f(lower_left_vertex_coords.x, lower_left_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, lower_right_tex_coords.x * texwidthratio, 
                                                lower_right_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, lower_right_tex_coords.x, lower_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, lower_right_tex_coords.x / density, lower_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(lower_right_normal_coords.x, lower_right_normal_coords.y, lower_right_normal_coords.z);
//             glVertex3f(lower_right_vertex_coords.x, lower_right_vertex_coords.y, lower_right_vertex_coords.z);
            glVertex2f(lower_right_vertex_coords.x, lower_right_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, upper_right_tex_coords.x * texwidthratio, 
                                                upper_right_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, upper_right_tex_coords.x, upper_right_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, upper_right_tex_coords.x / density, upper_right_tex_coords.y, 0.0, 1.0); 
            glNormal3f(upper_right_normal_coords.x, upper_right_normal_coords.y, upper_right_normal_coords.z);
//             glVertex3f(upper_right_vertex_coords.x, upper_right_vertex_coords.y, upper_right_vertex_coords.z);
            glVertex2f(upper_right_vertex_coords.x, upper_right_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, upper_left_tex_coords.x * texwidthratio, 
                                                upper_left_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, upper_left_tex_coords.x, upper_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, upper_left_tex_coords.x / density, upper_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(upper_left_normal_coords.x, upper_left_normal_coords.y, upper_left_normal_coords.z);
//             glVertex3f(upper_left_vertex_coords.x, upper_left_vertex_coords.y, upper_left_vertex_coords.z);
            glVertex2f(upper_left_vertex_coords.x, upper_left_vertex_coords.y);
        }
    }

    glEnd();


    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (core->renderwidth - width) / 2, 
                        (core->renderheight - height) / 2, width, height);

    glDisable(GL_VERTEX_PROGRAM_NV);
    glDisable(GL_FRAGMENT_PROGRAM_NV);
    delete vertex_program;
    delete fragment_program;

    glDeleteProgramsNV(1, &vertex_program_handle);
    glDeleteProgramsNV(1, &fragment_program_handle);

    previous_density = density;
    previous_exponent = exponent;

    fxtimer->endtime();
    fxtimer->printtime("Time for NV30 Charcoal: ");
}

void GLEffect::doNV30LightingGPU(void) {

    GLint                                       width;
    GLint                                       height;

    float4                                      light_position;
    float4                                      color;

    double                                      density;
    static double                               previous_density = 0.0;
    double                                      exponent;
    static double                               previous_exponent = 0.0;
    double                                      darker;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    int                                         row;
    int                                         column;

    float                                       f_row;
    float                                       f_column;

    float2                                      lower_left_tex_coords;
    float2                                      lower_right_tex_coords;
    float2                                      upper_left_tex_coords;
    float2                                      upper_right_tex_coords;

    float4                                      lower_left_vertex_coords;
    float4                                      lower_right_vertex_coords;
    float4                                      upper_left_vertex_coords;
    float4                                      upper_right_vertex_coords;

    float4                                      lower_left_normal_coords;
    float4                                      lower_right_normal_coords;
    float4                                      upper_left_normal_coords;
    float4                                      upper_right_normal_coords;

    fxtimer->starttime();

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    light_position.x = (layerlist->at(effectobj)->thelayer->node->rx * 5) + 200.0;
    light_position.y = (layerlist->at(effectobj)->thelayer->node->ry * 5);
    light_position.z = (layerlist->at(effectobj)->thelayer->node->rz * 5) + 500.0;
    light_position.w = 1.0;

    color.x = (layerlist->at(effectobj)->thelayer->node->tx / 1000.0) + 1.0;
    color.y = (layerlist->at(effectobj)->thelayer->node->ty / 1000.0) + 1.0;
    color.z = (layerlist->at(effectobj)->thelayer->node->tz / 1000.0) + 1.0;
    color.w = 1.0;

    density = (double)(((layerlist->at(effectobj)->thelayer->node->sx)) + 2.0);
    exponent = (double)(((layerlist->at(effectobj)->thelayer->node->sy - 1.0)) + 3.5);
    darker = (double)(((layerlist->at(effectobj)->thelayer->node->sz - 1.0) / 10.0) + 0.1);
    //printf("darker = %f\n", darker);

    vertex_program = loadshaderfile(JahBasePath+"jah/libraries/shader/lighting_vert_gpu.vp");
    glGenProgramsNV(1, &vertex_program_handle);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);
    glLoadProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle, strlen((char*)vertex_program), vertex_program);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 0, GL_MODELVIEW_PROJECTION_NV, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 4, GL_MODELVIEW, GL_IDENTITY_NV);
    glTrackMatrixNV(GL_VERTEX_PROGRAM_NV, 8, GL_MODELVIEW, GL_INVERSE_TRANSPOSE_NV);
    glEnable(GL_VERTEX_PROGRAM_NV);
    glBindProgramNV(GL_VERTEX_PROGRAM_NV, vertex_program_handle);


    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 12, 0.0, 1.0, 1.0, 1.0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 13, 0.0, 1.0, 1.0, 1.0);
    glProgramParameter4fvNV(GL_VERTEX_PROGRAM_NV, 14, (float*)&light_position);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 15, width, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 16, height, 0, 0, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 17, camera_distance, 0, 0, 0);

    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 18, 2, 0, 1, 0);
    glProgramParameter4fNV(GL_VERTEX_PROGRAM_NV, 19, 0.5, 0.5, 0, 0);


    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);

    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D, contrast_texture);

    glActiveTextureARB(GL_TEXTURE2_ARB);
    glBindTexture(GL_TEXTURE_2D, random_texture);

    fragment_program = loadshaderfile(JahBasePath+"jah/libraries/shader/lighting_frag_gpu.fp");
    glGenProgramsNV(1, &fragment_program_handle);
    glLoadProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle, 
                    strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION) 
    {
        find_shader_program_error(fragment_program, "lighting_frag_gpu.fp");
    }

    glEnable(GL_FRAGMENT_PROGRAM_NV);
    glBindProgramNV(GL_FRAGMENT_PROGRAM_NV, fragment_program_handle);

    glProgramNamedParameter4fNV(fragment_program_handle, strlen("light_color"), 
        (const GLubyte*)"light_color", color.x, color.y, color.z, color.w);

    float texwidthratio = layerlist->at(1)->thelayer->textureData.texwidthratio;
    float texheightratio = layerlist->at(1)->thelayer->textureData.texheightratio;



    glBegin(GL_QUADS);

    for (row = 0; row < JAHMESH_Y_DIMENSION; row++)
    {
        f_row = (float)row;

        for (column = 0; column < JAHMESH_X_DIMENSION; column++)
        {
            f_column = (float)column;

            lower_left_tex_coords.x = (f_column + 0.0) / JAHMESH_X_DIMENSION;
            lower_left_tex_coords.y = (f_row + 0.0) / JAHMESH_Y_DIMENSION;

            lower_right_tex_coords.x = (f_column + 1.0) / JAHMESH_X_DIMENSION;
            lower_right_tex_coords.y = (f_row + 0.0) / JAHMESH_Y_DIMENSION;

            upper_right_tex_coords.x = (f_column + 1.0) / JAHMESH_X_DIMENSION;
            upper_right_tex_coords.y = (f_row + 1.0) / JAHMESH_Y_DIMENSION;

            upper_left_tex_coords.x = (f_column + 0.0) / JAHMESH_X_DIMENSION;
            upper_left_tex_coords.y = (f_row + 1.0) / JAHMESH_Y_DIMENSION;

            get_mesh_normal(lower_left_normal_coords, column, row, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(lower_right_normal_coords, column + 1, row, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(upper_right_normal_coords, column + 1, row + 1, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);
            get_mesh_normal(upper_left_normal_coords, column, row + 1, JAHMESH_X_DIMENSION, JAHMESH_Y_DIMENSION);

            layerlist->at(1)->thelayer->getMeshCoord(column, row,
                                                    lower_left_vertex_coords.x,
                                                    lower_left_vertex_coords.y,
                                                    lower_left_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column + 1, row,
                                                    lower_right_vertex_coords.x,
                                                    lower_right_vertex_coords.y,
                                                    lower_right_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column + 1, row + 1,
                                                    upper_right_vertex_coords.x,
                                                    upper_right_vertex_coords.y,
                                                    upper_right_vertex_coords.z);

            layerlist->at(1)->thelayer->getMeshCoord(column, row + 1,
                                                    upper_left_vertex_coords.x,
                                                    upper_left_vertex_coords.y,
                                                    upper_left_vertex_coords.z);



            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, lower_left_tex_coords.x * texwidthratio, 
                                                lower_left_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, lower_left_tex_coords.x, lower_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, lower_left_tex_coords.x / density, lower_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(lower_left_normal_coords.x, lower_left_normal_coords.y, lower_left_normal_coords.z);
//             glVertex3f(lower_left_vertex_coords.x, lower_left_vertex_coords.y, lower_left_vertex_coords.z);
            glVertex2f(lower_left_vertex_coords.x, lower_left_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, lower_right_tex_coords.x * texwidthratio, 
                                                lower_right_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, lower_right_tex_coords.x, lower_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, lower_right_tex_coords.x / density, lower_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(lower_right_normal_coords.x, lower_right_normal_coords.y, lower_right_normal_coords.z);
//             glVertex3f(lower_right_vertex_coords.x, lower_right_vertex_coords.y, lower_right_vertex_coords.z);
            glVertex2f(lower_right_vertex_coords.x, lower_right_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, upper_right_tex_coords.x * texwidthratio, 
                                                upper_right_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, upper_right_tex_coords.x, upper_right_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, upper_right_tex_coords.x / density, upper_right_tex_coords.y, 0.0, 1.0); 
            glNormal3f(upper_right_normal_coords.x, upper_right_normal_coords.y, upper_right_normal_coords.z);
//             glVertex3f(upper_right_vertex_coords.x, upper_right_vertex_coords.y, upper_right_vertex_coords.z);
            glVertex2f(upper_right_vertex_coords.x, upper_right_vertex_coords.y);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, upper_left_tex_coords.x * texwidthratio, 
                                                upper_left_tex_coords.y * texheightratio, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE1_ARB, upper_left_tex_coords.x, upper_left_tex_coords.y, 0.0, 1.0); 
            glMultiTexCoord4fARB(GL_TEXTURE2_ARB, upper_left_tex_coords.x / density, upper_left_tex_coords.y, 0.0, 1.0); 
            glNormal3f(upper_left_normal_coords.x, upper_left_normal_coords.y, upper_left_normal_coords.z);
//             glVertex3f(upper_left_vertex_coords.x, upper_left_vertex_coords.y, upper_left_vertex_coords.z);
            glVertex2f(upper_left_vertex_coords.x, upper_left_vertex_coords.y);
        }
    }

    glEnd();


    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, layerlist->at(1)->thelayer->textureData.texID);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (core->renderwidth - width) / 2, 
                        (core->renderheight - height) / 2, width, height);

    glDisable(GL_VERTEX_PROGRAM_NV);
    glDisable(GL_FRAGMENT_PROGRAM_NV);
    delete vertex_program;
    delete fragment_program;

    glDeleteProgramsNV(1, &vertex_program_handle);
    glDeleteProgramsNV(1, &fragment_program_handle);

    previous_density = density;
    previous_exponent = exponent;

    fxtimer->endtime();
    fxtimer->printtime("Time for NV30 Lighting: ");
}
