/*******************************************************************************
 **
 ** The source file for the Jahshaka effects objects module
 ** The Jahshaka Project
 ** Copyright (C) 2000-2004 The Jahshaka Project
 ** Released under the GNU General Public License
 **
 *******************************************************************************/
//#define FFT_MAX

#include "effect.h"

//#define PIXEL_BUFFER_WIDTH				2048
//#define PIXEL_BUFFER_HEIGHT				2048


////////////////////////////////////////////////
//qt image processing based functions

void GLEffect::doSwapRGB(QImage& theimage)
{

    theimage = theimage.swapRGB();

}

//need to add controls here
void GLEffect::doMirror(QImage& theimage)
{
    //need to add ui controls for this
    bool VERT = TRUE;
    bool HORIZ = TRUE;

    theimage = theimage.mirror(VERT,HORIZ);

}

void GLEffect::doNegative(QImage& theimage)
{

    theimage.invertPixels();

}

////////////////////////////////////////////////
//math and processor based functions here

void GLEffect::doGrey(QImage& pix)
{
    int r,g,b,grey;
    QString conv,text;

    //note that this crashes when x or y are over the scope
    //for some reason this happens with the default image

    //int yy = (int)layers[1].textureData.height;
    //int xx = (int)layers[1].textureData.width;
    int yy = (int)layerlist->at(1)->thelayer->textureData.height;
    int xx = (int)layerlist->at(1)->thelayer->textureData.width;

    for ( int y=0; y<yy; y++ )
    {
        for ( int x=0; x<xx; x++ )
        {
            //if we can do this one SCANLINE at a time it would be faster
            //uchar *pe = pix.scanLine(y);

            QRgb *p = (QRgb *)pix.scanLine(y) + x; //debug("here1");

            r=qRed(*p);
            g=qGreen(*p);
            b=qBlue(*p); //debug("here2");

            grey=( (r+g+b)/3 ); //debug("here3");
            *p = qRgb(grey,grey,grey);
        }
    }

}

void GLEffect::doContrast(QImage& pix) //was double power
{
    //define variables
    int yy,xx;              //source buffer width and height

    float contrast;
    bool contrastmode;
    int cval;

    int r,g,b;              //pixel variables

    //get source buffer size
    yy = (int)layerlist->at(1)->thelayer->textureData.height;
    xx = (int)layerlist->at(1)->thelayer->textureData.width;

    //get contrast value
    contrast = (layerlist->at(effectobj)->thelayer->node->rx)/255;

    //catch default startup value
    if (contrast==0) contrast=1;

    //figure out cmode using slider need a checkbox!
    cval = (int)(layerlist->at(effectobj)->thelayer->node->sx);
    if (cval>=0) contrastmode = true; else contrastmode=false;

    //start main loop
    if (contrastmode)
    {
        //faster matrix controlled contrast
        int Contrast_transform[256];

        for(int i=0;i<256;i++)
        {
        if(i<(int)(128.0f+128.0f*tan(contrast))&&i>(int)(128.0f-128.0f*tan(contrast)))
            Contrast_transform[i]=(int)((i-128)/tan(contrast)+128);
        else if(i>=(int)(128.0f+128.0f*tan(contrast)))
            Contrast_transform[i]=255;
        else
            Contrast_transform[i]=0;
        }

        for ( int y=0; y<yy; y++ )
        {
            for ( int x=0; x<xx; x++ )
            {
                //this is the source and destination pixel
                QRgb *p = (QRgb *)pix.scanLine(y) + x;
                r=qRed(*p);
                g=qGreen(*p);
                b=qBlue(*p);

                //now write out the destination pixel
                *p = qRgb(Contrast_transform[r],Contrast_transform[g],Contrast_transform[b]);

            }
        }
    }
    else
    {
        //slower math controlled contrast
        for ( int y=0; y<yy; y++ )
        {
            for ( int x=0; x<xx; x++ )
            {
                QRgb *p = (QRgb *)pix.scanLine(y) + x;
                r=qRed(*p);
                g=qGreen(*p);
                b=qBlue(*p);

                int r1 = (int)( 255 * pow(  (r/255.0) ,contrast ));
                int g1 = (int)( 255 * pow(  (g/255.0) ,contrast ));
                int b1 = (int)( 255 * pow(  (b/255.0) ,contrast ));

                //some saftey tests here to make sure colors dont go out of range
                if (r1>255) r1=255;   if (g1>255) g1=255;   if (b1>255) b1=255;
                if (r1<0)   r1=0;     if (g1<0)   g1=0;     if (b1<0)   b1=0;

                *p = qRgb(r1,g1,b1);
            }
        }
    }
}

void GLEffect::doBrightness(QImage& pix) //was double power
{
    //define variables
    int yy,xx;              //source buffer width and height

    float brightness;

    int r,g,b;              //pixel variables

    //get source buffer size
    yy = (int)layerlist->at(1)->thelayer->textureData.height;
    xx = (int)layerlist->at(1)->thelayer->textureData.width;

    //get contrast value
    brightness = (layerlist->at(effectobj)->thelayer->node->rx);

    //catch default startup value
    //if (brightness==0) brightness=1;

    int Light_transform[256];

    for(int i=0;i<256;i++)
    {
        Light_transform[i]=(int)(i+brightness);
        if(Light_transform[i]>255)
            Light_transform[i]=255;
        if(Light_transform[i]<0)
            Light_transform[i]=0;
    }

    for ( int y=0; y<yy; y++ )
    {
        for ( int x=0; x<xx; x++ )
        {
            //this is the source and destination pixel
            QRgb *p = (QRgb *)pix.scanLine(y) + x;
            r=qRed(*p);
            g=qGreen(*p);
            b=qBlue(*p);

            //now write out the destination pixel
            *p = qRgb(Light_transform[r],Light_transform[g], Light_transform[b]);
        }
    }

}

void GLEffect::doMatrixSharpen(QImage& pix) //was double power
{

    //debug("sharpen");
       //define variables
    int yy,xx;              //source buffer width and height

    int r,g,b;              //pixel variables

    //get source buffer size
    yy = (int)layerlist->at(1)->thelayer->textureData.height;
    xx = (int)layerlist->at(1)->thelayer->textureData.width;

    const int sharp_w = 3;
    const int sharp_h = 3;

    int sumr=0;
    int sumg=0;
    int sumb=0;

   // int sharpen_filter[sharp_w][sharp_h]={{0,-1,0},{-1,5,-1},{0,-1,0}};
    //the correct filter
    int sharpen_filter[sharp_w][sharp_h]={{-1,-1,-1},{-1,9,-1},{-1,-1,-1}};


    //try this
    //int sharpen_filter[sharp_w][sharp_h]={{-1,0, 1},{-2,0,2},{-1,0,1}};

    //get sharp value
    int sharp_sum = (int)(layerlist->at(effectobj)->thelayer->node->rx+1);
    if( sharp_sum <1) sharp_sum = 1;

    //problem with matrix effects is we have to build a new image with the data
    //since sampling is done across 3 rows at a time
    QImage* buffer = new QImage( xx, yy, 32 );

    for ( int y=1; y<yy-1; y++ )
    {
        for ( int x=1; x<xx-1; x++ )
        {
            sumr = 0;

            for(int k=0;k<sharp_w;k++)
            {
                for(int l=0;l<sharp_h;l++)
                {
                    int xval = x- ((sharp_w-1)>>1)+k;
                    int yval = y- ((sharp_h-1)>>1)+l;

                    if (xval>=(xx-1)) xval = xx-2;
                    if (yval>=(yy-1)) yval = yy-2;

                    if (yval<1) yval = 1;
                    if (xval<1) xval = 1;

                    QRgb *sp = (QRgb *)pix.scanLine(yval) + xval;
                    r=qRed(*sp);
                    g=qGreen(*sp);
                    b=qBlue(*sp);

                    sumr+=r*sharpen_filter[k][l];
                    sumg+=g*sharpen_filter[k][l];
                    sumb+=b*sharpen_filter[k][l];

                }
            }

            QRgb *p = (QRgb *)buffer->scanLine(y) + x;

            sumr/=sharp_sum;
            sumb/=sharp_sum;
            sumg/=sharp_sum;

            //now write out the destination pixel
            *p = qRgb(sumr,sumg,sumb);

        }
    }

    //now we must copy buffer into pix
    pix = buffer->copy();

}

void GLEffect::doMatrixBlur(QImage& pix) //was double power
{
   //define variables
    int yy,xx;              //source buffer width and height
    int r,g,b;              //pixel variables

    //get source buffer size
    yy = (int)layerlist->at(1)->thelayer->textureData.height;
    xx = (int)layerlist->at(1)->thelayer->textureData.width;

    const int gauss_width = 7;

    int gauss_fact[gauss_width]={1,6,15,20,15,6,1};
    //int gauss_sum=64;
    //get contrast value
    int gauss_sum = (int)(layerlist->at(effectobj)->thelayer->node->rx+64);

    int sumr=0;
    int sumg=0;
    int sumb=0;

    int x,y,k;

    //problem with matrix effects is we have to build a new image with the data
    //since sampling is done across 3 rows at a time
    QImage* buffer = new QImage( pix.width(), pix.height(), pix.depth() );

    //first we do a horizontal blur
    for (  y=1; y<yy-1; y++ )
    {
        for (  x=1; x<xx-1; x++ )
        {
            sumr=0; sumg=0; sumb=0;

            for( k=0;k<gauss_width;k++)
            {
                int xval = x-((gauss_width-1)>>1)+k;
                int yval = y;

                if (xval>=(xx-1)) xval = xx-2;
                if (yval>=(yy-1)) yval = yy-2;

                if (yval<1) yval = 1;
                if (xval<1) xval = 1;

                QRgb *sp = (QRgb *)pix.scanLine(yval) + xval;
                r=qRed(*sp);
                g=qGreen(*sp);
                b=qBlue(*sp);

                sumr+=r*gauss_fact[k];
                sumg+=g*gauss_fact[k];
                sumb+=b*gauss_fact[k];
            }


            QRgb *p = (QRgb *)buffer->scanLine(y) + x;

            //now write out the destination pixel
            *p = qRgb(sumr/gauss_sum,sumg/gauss_sum,sumb/gauss_sum);
        }
    }

    //now we do a vertical blur
    for (  y=1; y<yy-1; y++ )
    {
        for (  x=1; x<xx-1; x++ )
        {
            sumr=0; sumg=0; sumb=0;

            for( k=0;k<gauss_width;k++)
            {
                int xval = x;
                int yval = y-((gauss_width-1)>>1)+k;

                if (xval>=(xx-1)) xval = xx-2;
                if (yval>=(yy-1)) yval = yy-2;

                if (yval<1) yval = 1;
                if (xval<1) xval = 1;

                QRgb *sp = (QRgb *)buffer->scanLine(yval) + xval;

                r=qRed(*sp);
                g=qGreen(*sp);
                b=qBlue(*sp);

                sumr+=r*gauss_fact[k];
                sumg+=g*gauss_fact[k];
                sumb+=b*gauss_fact[k];
            }


            QRgb *p = (QRgb *)pix.scanLine(y) + x;

            //now write out the destination pixel
            *p = qRgb(sumr/gauss_sum,sumg/gauss_sum,sumb/gauss_sum);
        }
    }
    //now we must copy buffer into pix
   // pix = buffer->copy();

}



void GLEffect::doMatrixEmboss(QImage& pix) //was double power
{
    //define variables
    int yy,xx;              //source buffer width and height

    int r,g,b;              //pixel variables

    //get source buffer size
    yy = (int)layerlist->at(1)->thelayer->textureData.height;
    xx = (int)layerlist->at(1)->thelayer->textureData.width;

    const int emboss_w = 3;
    const int emboss_h = 3;

    int sumr=0;
    int sumg=0;
    int sumb=0;
    //int grey;

    //original emboss
    int emboss_filter[emboss_w][emboss_h]={{2,0,0},{0,-1,0},{0,0,-1}};

    //int emboss_filter[emboss_w][emboss_h]={{6,12,6},{12,25,12},{6,12,6}};
    //int emboss_filter[emboss_w][emboss_h]={{2,0,0},{0,-1,0},{0,0,-1}};
    //int emboss_filter[emboss_w][emboss_h]={{2,0,0},{0,-1,0},{0,0,-1}};
    //int emboss_filter[emboss_w][emboss_h]={{2,0,0},{0,-1,0},{0,0,-1}};
    //int emboss_filter[emboss_w][emboss_h]={{2,0,0},{0,-1,0},{0,0,-1}};

    //int emboss_sum=1;
    int emboss_sum = (int)(layerlist->at(effectobj)->thelayer->node->rx+1);

    if (emboss_sum<1) emboss_sum = 1;

    //first we convert to greysacle...
    /*
    for ( int y1=0; y1<yy; y1++ )
    {
        for ( int x1=0; x1<xx; x1++ )
        {
            //if we can do this one SCANLINE at a time it would be faster
            //uchar *pe = pix.scanLine(y);

            QRgb *p = (QRgb *)pix.scanLine(y1) + x1; //debug("here1");

            r=qRed(*p);
            g=qGreen(*p);
            b=qBlue(*p); //debug("here2");

            grey=( (r+g+b)/3 ); //debug("here3");
            *p = qRgb(grey,grey,grey);
        }
    }
*/
    //then we emboss
    //problem with matrix effects is we have to build a new image with the data
    //since sampling is done across 3 rows at a time
    QImage* buffer = new QImage( xx, yy, 32 );

    for ( int y=1; y<yy-1; y++ )
    {
        for ( int x=1; x<xx-1; x++ )
        {
            sumr = 0;

            //added these
            sumg = 0;
            sumb = 0;

            for(int k=0;k<emboss_w;k++)
            {
                for(int l=0;l<emboss_h;l++)
                {
                    int xval = x-( (emboss_w-1)>>1)+k;
                    int yval = y-( (emboss_h-1)>>1)+l;

                    QRgb *sp = (QRgb *)pix.scanLine(yval) + xval;

                    r=qRed(*sp);
                    g=qGreen(*sp);
                    b=qBlue(*sp);

                    sumr+=r*emboss_filter[k][l];
                    sumg+=g*emboss_filter[k][l];
                    sumb+=b*emboss_filter[k][l];

                }
            }

            QRgb *p = (QRgb *)buffer->scanLine(y) + x;

            sumr/=emboss_sum; sumr+=128;
            if(sumr>255) sumr=255; if(sumr<0)   sumr=0;

            //added these
            sumg/=emboss_sum; sumg+=128;
            if(sumg>255) sumg=255; if(sumg<0)   sumg=0;

            sumb/=emboss_sum; sumb+=128;
            if(sumb>255) sumb=255; if(sumb<0)   sumb=0;

            //now write out the destination pixel
            //*p = qRgb(sumr,sumr,sumr);

            *p = qRgb(sumr,sumg,sumb);
        }
    }

    //now we must copy buffer into pix
    pix = buffer->copy();

}

void GLEffect::doEdge(QImage& pix)
{
    int r,g,b;
	//int grey;
    QString conv,text;
    bool overlay;

    overlay = true; //false is black background, true adds effect to image

    int yy = (int)layerlist->at(1)->thelayer->textureData.height;
    int xx = (int)layerlist->at(1)->thelayer->textureData.width;

    int r1, g1, b1;
    int r2, g2, b2;
    int addvalx,addvaly;

    int sensitivity = (int)layerlist->at(effectobj)->thelayer->node->tx+1;
    int edgecolor   = (int)layerlist->at(effectobj)->thelayer->node->ty+50;
    int backcolor   = (int)layerlist->at(effectobj)->thelayer->node->tz;

    for ( int y=0; y<yy; y++ )
    {
        for ( int x=0; x<xx; x++ )
        {
            //this is the source and destination pixel
            QRgb *p = (QRgb *)pix.scanLine(y) + x;
            r=qRed(*p);
            g=qGreen(*p);
            b=qBlue(*p);

            //bounds checking here for algorithim
            if (y==yy-1) addvaly = 0; else addvaly = 1;
            if (x==xx-1) addvalx = 0; else addvalx = 1;

            //these are neighbouring pixels for the algorithim
            QRgb *p1 = (QRgb *)pix.scanLine(y) + x+addvalx;
            r1=qRed(*p1);
            g1=qGreen(*p1);
            b1=qBlue(*p1);

            QRgb *p2 = (QRgb *)pix.scanLine(y+addvaly) + x;
            r2=qRed(*p2);
            g2=qGreen(*p2);
            b2=qBlue(*p2);

            //this is the main algorithim
            if((sqrt((r-r1)*(r-r1)+(g-g1)*(g-g1)+(b-b1)*(b-b1))>=sensitivity)||
            (sqrt((r-r2)*(r-r2)+(g-g2)*(g-g2)+(b-b2)*(b-b2))>=sensitivity))
            {
                r = r+edgecolor; if (r>255) r=255;
                g = g+edgecolor; if (g>255) g=255;
                b = b+edgecolor; if (b>255) b=255;
            }
            else
            {
                r = r-backcolor; if (r<0) r=0;
                g = g-backcolor; if (g<0) g=0;
                b = b-backcolor; if (b<0) b=0;
            }

            //now write out the destination pixel
            *p = qRgb(r,g,b);

        }
    }

}

void GLEffect::doColorSelect(QImage& pix)
{
    int r,g,b;
	//int grey;
    QString conv,text;
    bool overlay;

    overlay = true; //false is black background, true adds effect to image

    int yy = (int)layerlist->at(1)->thelayer->textureData.height;
    int xx = (int)layerlist->at(1)->thelayer->textureData.width;

    //int r1, g1, b1;
    //int r2, g2, b2;
    //int addvalx,addvaly;

    int sensitivity = (int)layerlist->at(effectobj)->thelayer->node->rx;

    int r0   = (int)layerlist->at(effectobj)->thelayer->node->tx;
    int g0   = (int)layerlist->at(effectobj)->thelayer->node->ty;
    int b0   = (int)layerlist->at(effectobj)->thelayer->node->tz;

    for ( int y=0; y<yy; y++ )
    {
        for ( int x=0; x<xx; x++ )
        {
            //this is the source and destination pixel
            QRgb *p = (QRgb *)pix.scanLine(y) + x;
            r=qRed(*p);
            g=qGreen(*p);
            b=qBlue(*p);

            //this is the main algorithim
            if(sqrt((r-r0)*(r-r0)+(g-g0)*(g-g0)+(b-b0)*(b-b0))<=sensitivity)
            {
                r=0; //r = r+edgecolor; if (r>255) r=255;
                g=0; //g = g+edgecolor; if (g>255) g=255;
                b=0; //b = b+edgecolor; if (b>255) b=255;
            }
            else
            {
                r=255; //r = r-backcolor; if (r<0) r=0;
                g=255; //g = g-backcolor; if (g<0) g=0;
                b=255; //b = b-backcolor; if (b<0) b=0;
            }

            //now write out the destination pixel
            *p = qRgb(r,g,b);

        }
    }

}
/*
void GLEffect::doBlur(QImage& pix) {

    unsigned char            horizontal_blur;
    unsigned char            vertical_blur;
    COMPLEX_IMAGE             fft1; 
    COMPLEX_IMAGE            fft2;
    float                    real_1, imaginary_1;
    float                    real_2, imaginary_2;
    int                     row, column;
    Image_c                    pattern_image;
    Image_c                    picture_image;
    FFT_c                    fft_transform;
    int                     full_image_size;
    int                     original_width;
    int                     original_height;
    uint                    current_pixel;
    uint                    red;
    uint                    green;
    uint                    blue;
    QImage                  newimage; 
    int                     max_dimension;
    int                     use_row;
    int                     use_column;  

    newimage = pix.copy();

    //original_width =  (int)layers[1].textureData.width;
    //original_height = (int)layers[1].textureData.height;
    
    original_width =  (int)layerlist->at(1)->thelayer->textureData.width;
    original_height = (int)layerlist->at(1)->thelayer->textureData.height;
    
    // Force things to behave properly 
    max_dimension = (original_height > original_width) ? original_height : original_width;

    // Make the image square and round up to smallest  width 2^n and <= 4096   
    if (max_dimension > 2 && max_dimension < 4) { full_image_size = 4; }  
    else if (max_dimension > 4 && max_dimension < 8) { full_image_size = 8; }
    else if (max_dimension > 8 && max_dimension < 16) { full_image_size = 16; }
    else if (max_dimension > 16 && max_dimension < 32) { full_image_size =32; }
    else if (max_dimension > 32 && max_dimension < 64) { full_image_size = 64; }
    else if (max_dimension > 64 && max_dimension < 128) { full_image_size = 128; }
    else if (max_dimension > 128 && max_dimension < 256) { full_image_size = 256; }
    else if (max_dimension > 256 && max_dimension < 512) { full_image_size = 512; }
    else if (max_dimension > 512 && max_dimension < 1024) { full_image_size = 1024; }
    else if (max_dimension > 1024 && max_dimension < 2048) { full_image_size = 2048; }
    else if (max_dimension > 2048 && max_dimension < 4096) { full_image_size = 4096; }
    else if (max_dimension > 4096) { full_image_size = 4096; }
    else { full_image_size = max_dimension; }   // Don't change good values    
    
    // Values returned from user input sliders
    //horizontal_blur = abs((int)layers[effectobj].node->tx);
    horizontal_blur = abs((int)layerlist->at(1)->thelayer->node->tx);
    
    //vertical_blur = abs((int)layers[effectobj].node->ty);
    vertical_blur = abs((int)layerlist->at(1)->thelayer->node->ty);

    // Create a blur map based on the specified blur values
    pattern_image.createBlurMap(full_image_size, full_image_size, horizontal_blur, vertical_blur);

    // Create the picture image data structure
    picture_image.image_data = picture_image.allocateImage(full_image_size, full_image_size);
    picture_image.image_data->info->num_rows = full_image_size;
    picture_image.image_data->info->num_columns = full_image_size;
    picture_image.image_data->info->origin_x = 0;
    picture_image.image_data->info->origin_y = 0;
    
    printf("Bytes per scanline = %d\n", newimage.bytesPerLine());


    // Load the picture image data structure
    for (row = 0; row < full_image_size; row++) 
    {
        for (column = 0; column < full_image_size; column++) 
        {
            if ((row < original_height) && (column < original_width))
            {
                // Convert image to grey scale
                current_pixel = (uint)newimage.pixel(column, row);

                if (newimage.depth() > 8)
                {
                    red = (current_pixel >> 16) & 0xff;
                    green = (current_pixel >> 8) & 0xff;
                    blue = (current_pixel >> 0) & 0xff;
                    current_pixel =( (red + green + blue) / 3 );
                }

                picture_image.image_data->pixel_data[row][column] = (uchar)current_pixel;
            }
            else 
            {
                // Pad the edges with the last real pixel
                if (row >= original_height) 
                { 
                    use_row = original_height - 1; 
                }
                else
                {
                    use_row = row;
                }

                if (column >= original_width) 
                { 
                    use_column = original_width - 1; 
                }
                else
                {
                    use_column = column;
                }

                picture_image.image_data->pixel_data[row][column] = 
                    255;
                //picture_image.image_data->pixel_data[use_row][use_column];
            }
        }
    }

    if ((horizontal_blur == 0) && (vertical_blur == 0)) { goto math_done; }

    // Do the math
    fft_transform.image_fftoc(pattern_image.image_data, &fft1);
    fft_transform.normalize_set();
    fft_transform.image_fftoc(picture_image.image_data, &fft2);
    fft_transform.normalize_clear();

    for (row = 0; row < full_image_size; row++)
    {
        for (column = 0; column < full_image_size; column++)
        {
            real_1 = fft1[row][column];    
            imaginary_1 = fft1[row][column + full_image_size];
            real_2 = fft2[row][column];
            imaginary_2 = fft2[row][column + full_image_size];

            fft_transform.cprod(real_2, imaginary_2, &real_1, &imaginary_1);
            fft1[row][column] = real_1; 
            fft1[row][column + full_image_size] = imaginary_1;
        }
    }

    fft_transform.freecomplex(fft2); 
    fft2 = (COMPLEX_IMAGE)0;
    
    fft_transform.image_fftinvoc(fft1, &fft2);
    fft_transform.freecomplex(fft1); 
    fft1 = (COMPLEX_IMAGE)0;

    fft_transform.filt_toint(fft2, picture_image.image_data, 0);
    fft_transform.hack_reorder_quadrants(picture_image.image_data);

  math_done:
    // Write the data back
    for (row = 0; row < original_height; row++) 
    {
        for (column = 0; column < original_width; column++) 
        {
            
            // *((uchar*)newimage.scanLine(row) + column) = 
            // picture_image.image_data->pixel_data[row][column];
            
            current_pixel = (uint)picture_image.image_data->pixel_data[row][column];

            if (newimage.depth() > 8)
            {
                current_pixel = current_pixel | (current_pixel << 8);         // Copy to gree  
                current_pixel = current_pixel | (current_pixel << 8);         // Copy to blue
                current_pixel = current_pixel | 0xff000000;                   // Set alpha
            }

            newimage.setPixel(column, row, current_pixel);
        }
    }

    // Free up the allocated memory for the images
    pattern_image.freeImage(pattern_image.image_data); 
    picture_image.freeImage(picture_image.image_data);

    pix = newimage;

}
*/

void GLEffect::doGaussianBlur(QImage& pix) {

    // GimpPixelRgn src_rgn, dest_rgn;
    int                     width, height;
    int                     bytes;
    int                     has_alpha;
    uchar*                  dst; 
    uchar*                  dst_ptr;
    uchar*                  src;
    uchar*                  src_ptr;
    int*                    buf; 
    int*                    bb;
    int                     pixels;
    int                     total = 1;
    int                     i, row, column, b;
    int                     start, end;
    int*                    curve;
    int*                    sum = NULL;
    int                     val;
    int                     length;
    int                     initial_p, initial_m;
    double                  std_dev;
    double                  horz;
    double                  vert;
    double                  temp;
    Gaussian_blur_c         blur;
    QImage                  newimage; 
    uint                    pixel_data;

    if (pix.numColors() != 0)
    {
        return;
    }

    horz =  (double)abs((int)layerlist->at(effectobj)->thelayer->node->tx) / 2.0;
    vert =  (double)abs((int)layerlist->at(effectobj)->thelayer->node->ty) / 2.0;

    if (horz <= 0.0 && vert <= 0.0)
    {
        return;
    }

    //gimp_drawable_mask_bounds (drawable->drawable_id, &x1, &y1, &x2, &y2);

    width = (int)layerlist->at(1)->thelayer->textureData.width;
    height = (int)layerlist->at(1)->thelayer->textureData.height;

    if (width < 1 || height < 1) { return; }

    bytes = 4;  // Always with QT (including alpha)
    has_alpha = TRUE;

    buf = new int [MAX(width, height) * 2];
    src = new uchar [MAX(width, height) * bytes];
    dst = new uchar [MAX(width, height) * bytes];

    //  First the vertical pass  
    if (vert > 0.0)
    {
        vert = fabs(vert) + 1.0;
        std_dev = sqrt (-(vert * vert) / (2 * log(1.0 / 255.0)));

        curve = blur.makeCurve(std_dev, &length);
        //sum = g_new (int, 2 * length + 1);
        //sum = (int*)calloc((2 * length) + 1, sizeof(int));
        sum = new int [(2 * length) + 1];

        sum[0] = 0;

        for (i = 1; i <= length*2; i++)
        {
            sum[i] = curve[i - length - 1] + sum[i-1];
        }

        sum += length;

        total = sum[length] - sum[-length];

        for (column = 0; column < width; column++)
        {
            //gimp_pixel_rgn_get_col (&src_rgn, src, column + x1, y1, (y2 - y1));

            // Gimp uses red, green, blue, alpha and Qt uses alpha, red, green,
            // blue, so the ordering of the bytes needs to be changed for gimp
            // ported routines.
            for (row = 0; row < height; row++)
            {
                (*(uint*)&src[row * bytes]) = (uint)pix.pixel(column, row);;
            }

            if (has_alpha)
            {
                blur.multiplyAlpha(src, height, bytes);
            }

            src_ptr = src;
            dst_ptr = dst;

            for (b = 0; b < bytes; b++)
            {
                initial_p = src_ptr[b];
                initial_m = src_ptr[(height-1) * bytes + b];

                //  Determine a run-length encoded version of the row  
                blur.runLengthEncode(src_ptr + b, buf, bytes, height);

                for (row = 0; row < height; row++)
                {
                    start = (row < length) ? -row : -length;
                    end = (height <= (row + length) ? (height - row - 1) : length);

                    val = 0;
                    i = start;
                    bb = buf + ((row + i) * 2);

                    if (start != -length)
                    {
                        val += initial_p * (sum[start] - sum[-length]);
                    }

                    while (i < end)
                    {
                        pixels = bb[0];
                        i += pixels;

                        if (i > end)
                        {
                            i = end;
                        }

                        val += bb[1] * (sum[i] - sum[start]);
                        bb += (pixels * 2);
                        start = i;
                    }

                    if (end != length)
                    {
                        val += initial_m * (sum[length] - sum[end]);
                    }

                    dst_ptr[row * bytes + b] = val / total;
                }
            }

            if (has_alpha)
            {
                blur.separateAlpha(dst, height, bytes);
            }

            //gimp_pixel_rgn_set_col (&dest_rgn, dst, column + x1, y1, (y2 - y1));
            for (row = 0; row < height; row++)
            {
                pixel_data = *(uint*)&dst[row * bytes + 0];
                //newimage.setPixel(column, row, pixel_data);
                pix.setPixel(column, row, pixel_data);
            }
        }

    }

    //  Now the horizontal pass  
    if (horz > 0.0)
    {
        horz = fabs(horz) + 1.0;

        if (horz != vert)
        {
            temp = -(horz * horz) / (2 * log(1.0 / 255.0));
            std_dev = sqrt(temp);

            curve = blur.makeCurve(std_dev, &length);
            //sum = g_new (int, 2 * length + 1);
            //sum = (int*)calloc((2 * length) - 1, sizeof(int));
            if (sum != NULL)
            {
                //delete [] sum;
            }

            sum = new int[(2 * length) + 1];

            sum[0] = 0;

            for (i = 1; i <= length*2; i++)
            {
                sum[i] = curve[i-length-1] + sum[i-1];
            }

            sum += length;

            total = sum[length] - sum[-length];
        }

        for (row = 0; row < height; row++)
        {
            //gimp_pixel_rgn_get_row (&src_rgn, src, x1, row + y1, (x2 - x1));
            for (column = 0; column < width; column++)
            {
                (*(uint*)&src[column * bytes]) = (uint)pix.pixel(column, row);;

            } 

            if (has_alpha)
            {
                blur.multiplyAlpha(src, width, bytes);
            }

            src_ptr = src;
            dst_ptr = dst;

            for (b = 0; b < bytes; b++)
            {
                initial_p = src_ptr[b];
                initial_m = src_ptr[(width-1) * bytes + b];

                // Determine a run-length encoded version of the row
                blur.runLengthEncode(src_ptr + b, buf, bytes, width);

                for (column = 0; column < width; column++)
                {
                    start = (column < length) ? -column : -length;
                    end = (width <= (column + length)) ? (width - column - 1) : length;

                    val = 0;
                    i = start;
                    bb = buf + (column + i) * 2;

                    if (start != -length)
                    {
                        val += initial_p * (sum[start] - sum[-length]);
                    }

                    while (i < end)
                    {
                        pixels = bb[0];
                        i += pixels;

                        if (i > end)
                        {
                            i = end;
                        }

                        val += bb[1] * (sum[i] - sum[start]);
                        bb += (pixels * 2);
                        start = i;
                    }

                    if (end != length)
                    {
                        val += initial_m * (sum[length] - sum[end]);
                    }

                    dst_ptr[column * bytes + b] = val / total;
                }
            }

            if (has_alpha)
            {
                blur.separateAlpha(dst, width, bytes);
            }

            //gimp_pixel_rgn_set_row (&dest_rgn, dst, x1, row + y1, (x2 - x1));
            for (column = 0; column < width; column++)
            {
                pixel_data = *(uint*)&dst[column * bytes + 0];
                pix.setPixel(column, row, pixel_data);

            } 

        }
    }

    //  free buffers
    // For some reason delete crashes with a segmentation violation.  Maybe the
    // compiler is taking care of it for me?
    //if (sum != NULL)
    //{
    //delete [] sum;
    //}

    //delete [] buf;
    //delete [] src;
    //delete [] dst;

}


