/*******************************************************************************
**
** The source file for the Jahshaka effects objects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"

#define PI                   3.14159265358979323846
#define TWO_PI               (PI * 2.0)

/////////////////////////////////////////////////////////////////
// mesh based effects and opengl based effects

void GLEffect::doSwirlRt(void)
{

    GLfloat                 xx, yy, radius, original_angle, twisted_angle;
    GLfloat                 lensX, lensY;
    GLfloat                 delta_xx, delta_yy;
    GLfloat                 new_xx, new_yy;

    GLfloat                 TwistEffect = 2.4;                // The amount of Twist
    GLfloat                 LensRadius;                       // Radius of the lens
 
    lensX = layerlist->at(effectobj)->thelayer->node->tx;
    lensY = layerlist->at(effectobj)->thelayer->node->ty; 
 
    LensRadius = (layerlist->at(effectobj)->thelayer->node->rx * 3.0) + 100.0;
    
    TwistEffect = (layerlist->at(effectobj)->thelayer->node->ry / 10.0) + 1.0;

    for (int i = 0; i <= JAHMESH_X_DIMENSION; i++) 
    {
        for (int j = 0; j <= JAHMESH_Y_DIMENSION; j++) 
        {
            //layers[1].getMeshCoord(i, j, xx, yy, z_offset);
            layerlist->at(1)->thelayer->getMeshCoord(i, j, xx, yy, z_offset);

            delta_xx = xx - lensX;
            delta_yy = yy - lensY;
            radius = sqrt(((delta_xx) * (delta_xx)) + ((delta_yy) * (delta_yy)));  

            // If we're inside the unit circle
            if (radius < LensRadius) 
            { 
                // Get the radius of the current point    
                original_angle = atan2(-delta_yy, delta_xx);
                twisted_angle = original_angle + ((LensRadius - radius) / 100.0 * TwistEffect); 
                new_xx = lensX + (radius * ( cos(twisted_angle))); 
                new_yy = lensY + (radius * ( -sin(twisted_angle)));                  

                 //set x and y coords in mesh
                layerlist->at(1)->thelayer->setMeshCoord(i, j, new_xx, new_yy, z_offset);                   
            }
            else
            {
                layerlist->at(1)->thelayer->setMeshCoord(i, j, xx, yy, z_offset); 
            }
        }
    }
    
}

void GLEffect::doFlagRt(void) {

    GLfloat                 zero_offset;
    GLfloat                 magnitude;
    GLfloat                 num_waves;

    GLfloat                 radians;

    int                     i, j;
    GLfloat                 xx, yy, zz;

    float                    image_width;
    float                    image_height;



    image_width = layerlist->at(1)->thelayer->getImageWidth();
    image_height = layerlist->at(1)->thelayer->getImageHeight();
    
    //zero_offset =  layers[effectobj].node->tx / 100.0; 
    zero_offset = layerlist->at(effectobj)->thelayer->node->tx / 100.0; 
    
    //magnitude  =  (layers[effectobj].node->ty / 2.0) + 20.0;
    magnitude = (layerlist->at(effectobj)->thelayer->node->ty / 2.0) + 20.0;
    
    //num_waves =   (layers[effectobj].node->rx / 10.0) + 1; 
    num_waves = (layerlist->at(effectobj)->thelayer->node->rx / 10.0) + 1; 
    
    for (i = 0; i <= JAHMESH_X_DIMENSION; i++)
    {
        radians = (((float)i / (float)JAHMESH_X_DIMENSION) - zero_offset) * TWO_PI * num_waves;

        for (j = 0; j <= JAHMESH_Y_DIMENSION; j++) 
        {
            layerlist->at(1)->thelayer->getMeshCoord(i, j, xx, yy, z_offset);
            zz = z_offset - (magnitude * (float)sin(radians));

            //set x, y and z coords in mesh
            layerlist->at(1)->thelayer->setMeshCoord(i, j, xx, yy, zz);
        }
    }
}


void GLEffect::doRippleRt(void)
{

    GLfloat                 ripple_radius;
    GLfloat                 wave_height;
    GLfloat                 half_wave_width;

    GLfloat                 radians;
    GLfloat                 current_radius;
    GLfloat                 current_radius_offset;

    GLfloat                 inner_limit;
    GLfloat                 outer_limit;

    int                     i, j;
    GLfloat                 xx, yy, zz;

    ripple_radius = (layerlist->at(effectobj)->thelayer->node->tx * 2.0) + 100.0; 
    wave_height = (layerlist->at(effectobj)->thelayer->node->ty / 2.0) + 10.0;

    half_wave_width = (PI / 2.0) * wave_height;

    inner_limit = ripple_radius - half_wave_width;
    outer_limit = ripple_radius + half_wave_width;

    for (i = 0; i <= JAHMESH_X_DIMENSION; i++) 
    {
        for (j = 0; j <= (JAHMESH_Y_DIMENSION + 1); j++) 
        {
            layerlist->at(1)->thelayer->getMeshCoord(i, j, xx, yy, z_offset);
            //layers[1].getMeshCoord(i, j, xx, yy, z_offset);
            
            current_radius = sqrt((xx * xx) + (yy * yy));

            if ((current_radius < outer_limit) && (current_radius > inner_limit))
            {
                current_radius_offset = current_radius - ripple_radius;

                radians = current_radius_offset / half_wave_width;
                zz = z_offset + ((wave_height * cos(radians)) * 5.0);
            }
            else
            {
                zz = z_offset;
            }

            //set x and y coords in mesh
            layerlist->at(1)->thelayer->setMeshCoord(i, j, xx, yy, zz); 
        }
    }
}

void GLEffect::doFisheyeRt(void) 
{

    GLfloat                  xx, yy, radius, angle;
    GLfloat                  lensX, lensY;
    GLfloat                  delta_xx, delta_yy;

    GLfloat                  LensEffect;  LensEffect  = 0.4;   
    GLfloat                  LensRadius;                       

    lensX = layerlist->at(effectobj)->thelayer->node->tx;
    lensY = layerlist->at(effectobj)->thelayer->node->ty; 
 
    LensRadius = (layerlist->at(effectobj)->thelayer->node->rx * 3.0) + 80.0;
 
    for (int i = 0; i <= JAHMESH_X_DIMENSION; i++) 
    {
        for (int j = 0; j <= (JAHMESH_Y_DIMENSION + 2); j++) 
        {
            //layers[1].getMeshCoord(i, j, xx, yy, z_offset);
            layerlist->at(1)->thelayer->getMeshCoord(i, j, xx, yy, z_offset);
                        
            delta_xx = xx - lensX;
            delta_yy = yy - lensY;

            radius = sqrt(((delta_xx) * (delta_xx)) + ((delta_yy) * (delta_yy)));  

            // If we're inside the unit circle
            if (radius < LensRadius) 
            { 
                // Get the radius of the current point    
                angle = atan2((yy - lensY),(xx - lensX));                
                radius = LensRadius * exp ( log (radius / LensRadius) * LensEffect ); 
                    
                //set x and y coords in mesh
                layerlist->at(1)->thelayer->setMeshCoord(i, j, 
                                                        (lensX + (radius * ( cos(angle))  )), 
                                                        (lensY + (radius * ( sin(angle))  )),
                                                         z_offset);
            }
            else
            {
                layerlist->at(1)->thelayer->setMeshCoord(i, j, xx, yy, z_offset);
            }
        }
    }

}


void GLEffect::doColorMatrixRt(void) 
{
    JahTrace& jtrace = JahTrace::getInstance();

    //raise the color matrix
    glMatrixMode(GL_COLOR);
    glLoadIdentity();

    // Transform the luminance colors:
    GLfloat lumx = ((GLfloat)layerlist->at(effectobj)->thelayer->node->tx+100) / 100;
    GLfloat lumy = ((GLfloat)layerlist->at(effectobj)->thelayer->node->ty+100) / 100;
    GLfloat lumz = ((GLfloat)layerlist->at(effectobj)->thelayer->node->tz+100) / 100;

    glScalef(lumx,lumy,lumz);   // Scale r*=0.5, g*=2.0, b*=1.0

    // Transform the brightness colors:
    GLfloat brightx = ((GLfloat)layerlist->at(effectobj)->thelayer->node->rx) / 100;
    GLfloat brighty = ((GLfloat)layerlist->at(effectobj)->thelayer->node->ry) / 100;
    GLfloat brightz = ((GLfloat)layerlist->at(effectobj)->thelayer->node->rz) / 100;

    glTranslatef(brightx,brighty,brightz);   // Scale r*=0.5, g*=2.0, b*=1.0

    // Transform the saturation colors:
    GLfloat saturation = ((GLfloat)layerlist->at(effectobj)->thelayer->node->sx);
    
    GLfloat matrix[16];

    matrix[0]  = (1.0 - saturation ) * 0.3086 + saturation;
    matrix[1]  = (1.0 - saturation ) * 0.3086 ;
    matrix[2]  = (1.0 - saturation ) * 0.3086 ;
    matrix[3]  = 0.0 ;
    matrix[4]  = (1.0 - saturation ) * 0.6094 ;
    matrix[5]  = (1.0 - saturation ) * 0.6094 + saturation;
    matrix[6]  = (1.0 - saturation ) * 0.6094 ;
    matrix[7]  = 0.0 ;
    matrix[8]  = (1.0 - saturation ) * 0.0820 ;
    matrix[9]  = (1.0 - saturation ) * 0.0820 ;
    matrix[10] = (1.0 - saturation ) * 0.0820 + saturation;
    matrix[12] = 0.0 ;
    matrix[13] = 0.0 ;
    matrix[14] = 0.0 ;
    matrix[15] = 1.0 ;

    glMultMatrixf(matrix);

    glMatrixMode(GL_MODELVIEW);
}


//////////////////////////////////////////////////////////
// none of these functions work yet
// we need to clean these up

void GLEffect::doSharpenRt(void)
{
    /*
    //we need to toggle this when ever its turned off...
    //int width = 9;

    //GLfloat convolveMat[3][3] = { -ColorMR, -ColorMR,     -ColorMR,
    //                              -ColorMR, 1+8*ColorMR,  -ColorMR,
    //                              -ColorMB ,-ColorMR,     -ColorMR };

    //glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE, 3,3, GL_LUMINANCE,
    //                        GL_FLOAT, convolveMat[0]);

    glMatrixMode(GL_COLOR);
    glPushMatrix();

    GLfloat convolve_2d[300][200];
    GLfloat convolve_1d[500];
    glMatrixMode(GL_COLOR);
    glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE, 300,200, GL_LUMINANCE,
                        GL_FLOAT, convolve_2d[]);

    glConvolutionFilter1D(GL_CONVOLUTION_1D, GL_LUMINANCE, 500, GL_LUMINANCE,
                        GL_FLOAT, convolve_1d);

    //glMatrixMode(GL_COLOR);
    //glPushMatrix();
    glLoadMatrixf(convolveMat);
    */

}

void GLEffect::doContrastRt()    
{

}


void GLEffect::doBlurRt(void) 
{
    /*
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glClearColor(0.0, 0.0, 0.0, 0.0);

    glPushMatrix();
    // glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE,
    //               3, 3, GL_LUMINANCE, GL_FLOAT, horizontal);
    //glEnable(GL_CONVOLUTION_2D);
    //glDisable(GL_CONVOLUTION_2D);
    glPopMatrix();
    */

}

