/*******************************************************************************
**
** The header file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

/* based in part on code from flip 
* Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
*/

#include "MainEncoder.h"

double MainEncoder::fpsVal[NumFPS];


//==============================================================================
MainEncoder::MainEncoder( QWidget *parent, const char* name, QHBox* control, QHBox* controler,
                        QHBox* leftcontroller, QHBox* rightcontroller) : QWidget (parent, name)

{
    //initalize the global vars from the prefs
    JahPrefs& jprefs = JahPrefs::getInstance();

    JahBasePath  = jprefs.getBasePath();
    JahMediaPath = jprefs.getMediaPath();

    buildInterfaceMenus(leftcontroller,rightcontroller);
    buildController(controler);

    //restore the preferences note that its not a dialog
    InitializeVariables();

    /////////////////////////////////////////////////////////
    //create the ui for the player

    mainworldLayout = new QHBoxLayout( parent, 11, 6, "mainworldLayout");
    player = new QFrame( parent, "player" );
    mainworldLayout->addWidget( player );

    //this is where we initialize  the disk player
    jahplayer = new JahPlayer(player, 0);
    jahplayer->hide();

    //connect player drag n drop to player load signal
    connect(jahplayer, SIGNAL(draggedAImage()), this, SLOT(draggedAImage()) );
    connect(jahplayer, SIGNAL(resizePlayer(mediaSize)), this, SLOT(ResizeDisplay(mediaSize)) );
    //connect(jahplayer, SIGNAL(LoadingFrame(int)), this, SLOT(UpdateLoadStatus(int)) );

    //////////////////////////////////////////////////////////
    //this creates the interface of the module for us
    buildInterface( control );

    //this builds the interfaces for the modules that are used
    buildEncoderInterfaces();

    ///////////////////////////////////////////////////////////////////////
    //set up extra options for mpeg encoder
    buildEncoderInterfacesMpeg();
    //buildEncoderInterfacesAvi();

    //////////////////////////////////////////////////////////
    //create our timer for the player
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(Action()) );

    //////////////////////////////////////////////////////////
    //create config object to be passed to the encoder
    configData = new RenderConfig;

    //////////////////////////////////////////////////////////
    //set up the encoders here
    encodingStatus = false;
    theEncoder = NONE;

    addEncoders();

    //default encoder
    #ifdef JAHMPEGENCODER
    theEncoder = JAHMPEG;
    #endif

}


MainEncoder::~MainEncoder()
{

}

void MainEncoder::InitializeVariables()
{

    //different frame rates
    fpsVal[F60] = 60.0;
    fpsVal[F48] = 48.0;
    fpsVal[F30] = 30.0;
    fpsVal[F24] = 24.0;
    fpsVal[F20] = 20.0;
    fpsVal[F15] = 15.0;
    fpsVal[F12] = 12.0;
    fpsVal[F10] = 10.0;

    //default to 30fps
    currentSpeed = fpsVal[30];

    //default values
    curFrame	=  0;
    pingpong	=  1;
    frameStep   =  1;

}


///////////////////////////////////////////////////////////////
// this is the main calling routine

void MainEncoder::LoadMySequence(videoData desktopclip)
{


    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing the encoder");

    //the local instance of the asset
    assetData = desktopclip;

    //cant play audio if theres no audio support
    if (assetData.theCategory == AUDIO) return;

    //need to update ui settings for resolution changes, etc
    updateEncoderSettings();

    //NEED TO ALSO RESET OR UPDATE THIS OBJECT
    //configData = new RenderConfig;

    /////////////////////////////////////////////////////////////
    //hook up jahplayer
    jahplayer->show();
    Clear();

    controllerslider->setRange	(0, 0);
    animframe->setValue( 1 );
    controllerslider->setRange	(assetData.startframe, assetData.endframe);

    //initialize MaxFrames for RamPlayer
    //int MaxFrames = assetData.getMaxFrames();
    startFrame = assetData.startframe;

    //this sets up the interface values
    LoadSequenceCompleted(startFrame, false);

    //initializes the disk player
    startFrame = jahplayer->LoadImages(assetData);

    //SetFrameNumber(curFrame);
    //a messy hack for still image loading loads using assetexchange default
    if (assetData.theCategory==IMAGE || assetData.theCategory==AUDIO || assetData.theCategory==CLIP )
    {
        startFrame = 1;
    }

    SetFrameNumber(startFrame);

    //default to 30fps but we should check speed for streams...
    switchFps(30);

    //updating asset display data
    UpdateAssetData(desktopclip);

}

//-----we can erase this----------------------------------------
void	MainEncoder::draggedAImage()
{
    emit playerDraggedAImage();
}




//--------------------------------------------------------------------
void	MainEncoder::SetFrameNumber(int i)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    // This slot tells player to draw a certain frame
    QString rere;
    //jtrace.debug("MainEncoder::", "SetFrameNumber() setting framenumber to:",i);
    curFrame = i;

    frameNumString.setNum(curFrame-minFrame+1);
    //jtrace.debug("MainEncoder::","framenumber is now:"+frameNumString);

    animframe->setValue( curFrame-minFrame+1 );

            if (assetData.theType == SEQUENCE)
            {
                //debug("its a sequence... hack the framenumber");
                int skip = assetData.videoext.toInt();
                int theframe = (curFrame-startFrame)+skip;
                //debug(">>Mainwindow Setframenumber startFrame:"+rere.setNum(theframe));

                jahplayer->LoadTheFrame(theframe);
            }
            else
            {
            jahplayer->LoadTheFrame(curFrame-startFrame); //+imagestartframe if its a sequence
            }




}

//--------------------------------------------------------------------
void	MainEncoder::Clear()
{
    Stop();

    jahplayer->Reset();

    //zero the sliders and settings
    curFrame	=  0;
    pingpong	=  0;
    frameStep   =  1;

}

//--------------------------------------------------------------------
//used for external access to plyaer
void	MainEncoder::ClearOut()
{
    Clear();
}

//--------------------------------------------------------------------
void	MainEncoder::Stop()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Stopping player");

    if (timer->isActive())
        timer->stop();

    pingpong = 0;
}




//--------------------------------------------------------------------
void	MainEncoder::LoadSequenceCompleted(int sf, bool aborted)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString rere;

        jtrace.debug("LoadSequenceCompleted() in Diskplayer");


        startFrame = assetData.startframe;
        minFrame = startFrame;
        maxFrame = assetData.endframe;

        controllerslider->setRange	(minFrame, maxFrame);
        jahplayer->setCurMinFrame(minFrame);
        jahplayer->setCurMaxFrame(maxFrame);

        curFrame = 0;
        controllerslider->setValue(curFrame);

        //default to 30fps but we should check speed for streams...
        currentSpeed = fpsVal[30];


}

//actually we should use this routine for single frames as well
//for less confusion...
//and we can streamline the loader... in tiffflipp
//since all the necessary data is already in the desktopclip
//--------------------------------------------------------------------
void	MainEncoder::UpdateAssetData(videoData &desktopclip)
{

}

//--------------------------------------------------------------------
void	MainEncoder::SetMinFrame(int val)
{

    //maxFrameInput->setMinInt( (minFrame = val) + 1 );
    controllerslider->setRange(minFrame, maxFrame);
    jahplayer->setCurMinFrame(minFrame);
    jahplayer->setCurMaxFrame(maxFrame);

}
void	MainEncoder::SetMaxFrame(int val)
{
    //minFrameInput->setMaxInt( (maxFrame = val) - 1 );
    controllerslider->setRange(minFrame, maxFrame);
    jahplayer->setCurMinFrame(minFrame);

}

//--------------------------------------------------------------------
void	MainEncoder::LoopForward()
{
    if (timer->isActive())  timer->stop();

    bool loopstatus = forcedplay->isChecked(); // status...

    playMode = loopstatus;
    pingpong = loopstatus;

    //timer speeds here
    timer->start((int)(1000.0/currentSpeed));

}
//--------------------------------------------------------------------
void	MainEncoder::PingPong()
{
    bool isact = false;

    if (timer->isActive())
    {
        timer->stop(); isact=true;
    }

    bool loopstatus = forcedplay->isChecked(); // status...

    playMode = loopstatus;
    pingpong = loopstatus;

    if (isact)
    {
        timer->start((int)(1000.0/currentSpeed));
    }
}






//--------------------------------------------------------------------
void	MainEncoder::FastForward()
{
    controllerslider->setValue( curFrame = maxFrame );
    SetFrameNumber(curFrame);
}

void	MainEncoder::FrameForward()
{
    int value = controllerslider->value();
    value+=1;

    if (value>= maxFrame) value = maxFrame;

    controllerslider->setValue( value );
    SetFrameNumber(value);
}

void	MainEncoder::FrameBackward()
{
    int value = controllerslider->value();
    value-=1;

    if (value<0) value = 0;

    controllerslider->setValue( value );
    SetFrameNumber(value);
}
//--------------------------------------------------------------------
void	MainEncoder::Rewind()
{
    controllerslider->setValue( curFrame = minFrame );
    SetFrameNumber(curFrame);
}

//--------------------------------------------------------------------
void	MainEncoder::Action()
{

    static int		counter = 1;
    static int		elapsed;
    static int		accumulated;
    static double	framesPerSec;
    static char		labelText[64];


    switch(playMode)
    {
        case LOOP_FORWARD:
            ActionLoopForward();
            break;
        case PING_PONG:
            ActionPingPong();
            break;
        default:
            warning ("MainEncoder::playMode is set incorrectly.\nplayMode = %d", playMode);
            timer->stop();
    }

    #ifndef _WIN32
    gettimeofday(&curTime, 0);
    #endif

    elapsed = (int)curTime.tv_usec - (int)oldTime.tv_usec;
    oldTime.tv_usec = curTime.tv_usec;
    if(elapsed < 0)
        elapsed += 1000000;

    accumulated += elapsed;
    if (counter >= 10)
    {
        framesPerSec = 1000*1.0/((double)accumulated*0.0001);
        counter = 0;
        accumulated = 0;
        sprintf(labelText, "%3.2f", framesPerSec);
    }
    counter++;
}
//- - - - - - - - - - - - - - - - - - - - - - -
void	MainEncoder::ActionLoopForward()
{
        curFrame += frameStep;

        if (curFrame > maxFrame)
            curFrame = minFrame + 1;

        controllerslider->setValue(curFrame);
        SetFrameNumber(curFrame);

        qApp->processEvents();
}

void	MainEncoder::ActionPingPong()
{
    curFrame += pingpong;
    if (curFrame > maxFrame)
    {
        curFrame = maxFrame;
        pingpong = -frameStep;
    }
    else if (curFrame < minFrame)
    {
        curFrame = minFrame;
        pingpong = frameStep;
    }

    controllerslider->setValue(curFrame);
    SetFrameNumber(curFrame);
    qApp->processEvents();
}


//----------------------------------------------------------

//--------------------------------------------------------------------
void	MainEncoder::switchFps(int fpsval)
{

    switch(fpsval) {
        case 60  : { SetFPS(F60); break; }
        case 48  : { SetFPS(F48); break; }
        case 30  : { SetFPS(F30); break; }
        case 24  : { SetFPS(F24); break; }
        case 20  : { SetFPS(F20); break; }
        case 15  : { SetFPS(F15); break; }
        case 12  : { SetFPS(F12); break; }
        case 10  : { SetFPS(F10); break; }
        default  : { SetFPS(F30); break; }
    }


}
//--------------------------------------------------------------------
void	MainEncoder::SetFPS(FPS index)
{
    for(int i = 0; i < 7; i++)

    currentSpeed = fpsVal[index];

    if (timer->isActive())
    {
        timer->changeInterval((int)(1000.0/currentSpeed));
    }
}

//--------------------------------------------------------------------

void	MainEncoder::SetStep(int index)
{

    switch(index)
    {
        case 1:	frameStep = 1;
            break;
        case 2:	frameStep = 2;
            break;
        case 3:	frameStep = 3;
            break;
        case 4:	frameStep = 4;
            break;
        case 5:	frameStep = 5;
            break;
        case 6:	frameStep = 6;
            break;
    }
}

//--------------------------------------------------------------------

void	MainEncoder::SetZoom(int index)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Calling setzoom");

    //rplace with a switch
    float thezoom;
    if (index == 1) thezoom = 0.5;
    if (index == 2) thezoom = 1.0;
    if (index == 3) thezoom = 2.0;
    if (index <1 || index >3) thezoom = 1.0;

    jahplayer->SetDisplayZoom(thezoom);
}

void	MainEncoder::ResizeDisplay(mediaSize thesize)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "Calling Resize display");


    player->setMinimumSize( QSize( thesize.width, thesize.height ) );
    player->setMaximumSize( QSize( thesize.width, thesize.height ) );
}
//----------------------------------------------------------



