/*******************************************************************************
**
** The header file for the Jahshaka Encoder Module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef MainEncoder_h
#define MainEncoder_h

#ifndef WIN32
#include <unistd.h>
#include <sys/time.h>
#else
#include <sys/timeb.h>
#include <windows.h>
#endif

#include <stdio.h>

#include <interfaceheaders.h>

#include <qsettings.h>
#include <qtimer.h>
#include <qmultilineedit.h>
#include <qkeycode.h>
#include <qtooltip.h>
#include <qiconset.h>
#include <qmessagebox.h>
#include <qtabwidget.h>

////////////////////////////////////
//header from the ol ui based encoder
//needs to be cleaned up
//and duplicates removed

#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <sstream>
#include <stdio.h>

#include <qlabel.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtextstream.h>
#include <qprogressbar.h>
#include "InputLCD.h"
#include <qmessagebox.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qprogressdialog.h>    
#include <qlistview.h>     


/////////////////////////////////

//used for videodata support
#include "videodata.h"

//custom ui objects
#include "widget.h"
#include "InputLCD.h"

//included from libraries/jahplayer
#include "diskplayer.h"

//for the tracer
#include "jahtracer.h"

//for the render configuratio object
#include "RenderConfig.h"

//the mpeg renderer
#include "RenderDialog.h"

//for preferences global
#include "jahpreferences.h"

#define SIZE_50         0
#define SIZE_100        1
#define SIZE_200        2
#define LOOP_FORWARD    0
#define PING_PONG       1

//====================================================================
class MainEncoder : public QWidget
{
        Q_OBJECT

public:
    MainEncoder(QWidget *parent, const char* name,
    QHBox* control, QHBox* controler, QHBox* leftcontroller, QHBox* rightcontroller );
    ~MainEncoder();

    //the asset to be loaded in the player
    videoData assetData;

    //create config object to be passed to the encoder
    RenderConfig *configData;

    enum FPS { F60, F48, F30, F24, F20, F15, F12, F10,NumFPS };

	JahEncoder theEncoder;

    static double fpsVal[NumFPS];

    ///////////////////////////////////////////////
    //added for the encoder
    void	LoadMySequence(videoData desktopclip); //was just videoData
    void	updateEncoderSettings(void); //was just videoData
    void	loadEncoderSettings(void); //was just videoData
    bool	setupConfigForEncoding(void);

    void buildEncoderInterfaces( void );
    void buildEncoderInterfacesMpeg( void );
    void buildEncoderInterfacesAvi( void );

    void browseFilename();

	void addEncoders();
	void addEncoder(QString encodername);
	void selectEncoder(QString encodername);
	QString	parseDestinationFilename(QString filename);

    ////////////////////////////////////////////
    //added for default to play from disk
    void	ClearOut();

    void	LoadDiskSequence();
    void	PlayDiskSequence(videoData &desktopclip);

	QListView*     lv;


    //preferences variables
    bool videohead;
    int videoheadx, videoheady;

    //builds the controler
    void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                            int w, int h, QString iconOff, QString iconOn );

    void    addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range);
    void    buildController( QHBox* controller);
    void    buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller);
    void    buildInterface( QHBox* qtinterface );

    void	UpdateAssetData(videoData &desktopclip);

public slots:

    void clicked( QListViewItem* item );

protected slots:

    //////////////////////////////////////////////////////////
    // for the encoder

    void	runEncoder(void);

    //////////////////////////////////////////////////////////
    // for the player

    void	LoadSequenceCompleted(int sf, bool aborted);

    void	SetFrameNumber(int);
    void    switchModeFps(void);
    void	SetFPS(FPS);
	void	switchFps(int fpsval);
    void    switchModeStep(void);
    void	SetStep(int);
    void	SetZoom(int);
    void	ResizeDisplay(mediaSize);
    void	InitializeVariables();

    void	SetMinFrame(int);
    void	SetMaxFrame(int);

    void	Clear();
    void	LoopForward();
    void	PingPong();
    void	Stop();
    void	FastForward();
    void	FrameForward();
    void	FrameBackward();
    void	Rewind();
    void	Action();
    void	ActionLoopForward();
    void	ActionPingPong();

    void draggedAImage();

    //for use when built as player
signals:

    void playerDraggedAImage();

private:

    QString JahBasePath,JahMediaPath;

    JahPlayer		*jahplayer;

    bool encodingStatus;

    int			fpsID[7];

    int			step1_ID;
    int			step2_ID;
    int			step3_ID;
    int			step4_ID;
    int			step5_ID;
    int			step6_ID;

    int			zoom1_ID;
    int			zoom2_ID;


    unsigned long	curFrame; // in audio: current file pointer (in samples)
    unsigned long	minFrame; // in audio: 0
    unsigned long	maxFrame; // in audio: max samples
    int             startFrame;
    int             pingpong;
    double          currentSpeed;
    unsigned long	frameStep;// in audio: buffer size in samples (1 channel)


    struct	timeval	oldTime;
    struct	timeval	curTime;

    bool _is_audio;		// object to play is audio

    //////////////////////////////////////////////////////////////////////
    //player interface objects

    QFrame* maincontroller;
    QWidgetStack* MainController;
    QWidget* page;
    QFrame* forceplayframe;
    QCheckBox* forceplaycheckbox;
    QFrame* controllerframe;
    QFrame* TimecodeFrame;

    QPushButton* controllerpreviousbutton;
    QPushButton* controllerstopbutton;
    QPushButton* controllerplaybutton;
    QPushButton* controllerffworardbutton;
    QPushButton* controllernextbutton;
    QPushButton* controllerrewindbutton;

    QSlider* controllerslider;
    QFrame* timecodeframe;
    QPushButton* keyframebutton;
    QLCDNumber* timecodedisplay;
    QFrame* imagingframe;
    QPushButton* imagingphotobutton;
    QPushButton* imagingrenderbutton;

    //protected:
    QHBoxLayout* Form1Layout;
    QHBoxLayout* maincontrollerLayout;
    QHBoxLayout* pageLayout;

    //////////////////////////////////////////////////////////////////////
    //controller variables
    int    Astartframe;  int  Aendframe;  int  Aanimframe; int  MaxFrames;

    //for the left and right menus
    QFrame         *glworldLeft;
    JahToolButton   *tooltranslate, *toolscale, *toolrotate;
    JahToolButton  *toolthumbview, *toolbinview;

    QFrame         *glworldRight;
    JahToolButton  *toolzoomin, *toolzoomout;
    QSlider        *zoom;

	//other stuff
    QHBoxLayout * mainplayerLayout;
    QFrame * playerframe;
    QFrame * mainplayerframe;
    QWidget * playerClip;

    QGridLayout * jahshakaLayout;
    QFrame * mainworld;
    QHBoxLayout * mainworldLayout;
    QFrame * player;

    QHBox * playerh;
    QVBox * playerv;

    //scene control variables
    QWidgetStack* controlpanel;
    QFrame* scene_controls;
    QFrame* object_controls;

    //for the player
    QVBoxLayout	*topLayout;
    QHBoxLayout	*middle;
    QVBoxLayout	*midR;
    QVBoxLayout	*midM;
    QVBoxLayout	*midL;
    QHBoxLayout	*buttons;
    QHBoxLayout	*status;

    int			playMode;
    QScrollBar	*frameScroll;
    QTimer		*timer;
    QString		frameNumString;
    QLabel		*frameNum;
    QFrame* controls;
    QHBox* controlers;


    QPushButton* rewButton;
    QPushButton* pingButton;
    QPushButton* stopButton;

    QMenuBar	*menuBar;
    QPopupMenu	*filePopup;
    QPopupMenu	*utilPopup;

    QCheckBox    * forcedplay;

    JahToolButton  * scrubfirst, * scrubprevious, * scrubstop, * scrubplay, * scrubnext, * scrublast;
    QPushButton  * scrubkey, * scrubrender, * scrubrenderAll;

    //for the rest of the interface
    QHBox*       MiddleFrame;       QSlider *     Slider2;
    InputLCD*    animframe;         QLabel *      aframetext;  //was QLCDNumber
    QLabel*      sframetext;        InputLCD *    lcdstartframe;
    QLabel*      eframetext;        InputLCD *    lcdendframe;

    QLabel* maxtext, *mintext, *mpegtext;

    //for the new interface
    QFrame*      LayersFrame;

    QWidgetStack*  EffectsMain;
    QFrame*        SceneFrame;
    QFrame*        EffectsFrame;

    QTabWidget*    tabframe;


    QFrame*        ControlFrame[NumEncoders];
    QWidget*       encoderUI[NumEncoders];
    QFrame*        sceneframe;


    /////////////////////////////////////////////////////////////////
    //for the mpeg encoder
    QTabWidget* jahencoder;
    QWidget* tab;
    QLabel* OutputDirLabel;
    QLineEdit* OutputDirEdit;
    QLabel* FileNameLabel;
    QPushButton* OutputDirSearchButton;
    QLabel* FrameSizeLabel;
    QLabel* BitrateLabel;
    QLineEdit* FileNameEdit;
    QLabel* RangeInLabel;
    QSpinBox* RangeInSpinbox;
    QLabel* RangeOutLabel;
    QSpinBox* RangeOutSpinbox;
    QComboBox* FrameRateComboBox;
    QComboBox* FrameSizeComboBox;
    InputLCD* BitRateLCD;
    QSlider* BitRateSlider;
    QLabel* FrameRateLabel;
    QLabel* OptionLabel_2;
    QLabel* OptionLabel_1;
    QComboBox* OptionComboBox_1;
    QComboBox* OptionComboBox_2;
    QComboBox* OptionComboBox_3;
    QPushButton* EncodeButton;
    QLabel* OptionLabel_3;
    QWidget* tab_2;
    QWidget* tab_3;
    QWidget* tab_4;
    QWidget* tab_5;


};

#endif

