/*******************************************************************************
**
** The source file for the Jahshaka core file file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "MainEncoder.h"
#include <qstylefactory.h>
#include "InputLCD.h"

void MainEncoder::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) {


    QStyle *platinum=QStyleFactory::create("platinum");

    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");


    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );

    toolzoomin =   new JahToolButton( glworldRight, "zoomin" );
    addJahPlayerButton( toolzoomin,   0, 355, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomin.png", JahBasePath+"Pixmaps/desktop/zoomin.png" );

    zoom = new QSlider( glworldRight, "scrubslider" );
    if (platinum) zoom->setStyle( platinum );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( 1 );    zoom->setMaxValue( 3 );    zoom->setValue   ( 2 );

    toolzoomout =   new JahToolButton( glworldRight, "zoomout" );
    addJahPlayerButton( toolzoomout,   0, 115, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomout.png", JahBasePath+"Pixmaps/desktop/zoomout.png" );


    connect( zoom,         SIGNAL(sliderMoved(int)), this, SLOT(SetZoom(int) )  );


}


void MainEncoder::buildController( QHBox* controller) {

   //////////////////////////////////////////////////////////////
   // This is the control bar
   // please clean me up and make me look cool
   // also get rid of constraints on startframe and endframe
   // and use the resizeable lcds in timecode format
   // for these variables display

   Astartframe=1;  Aendframe=100;  Aanimframe=1; MaxFrames = 100000;

    Form1Layout = new QHBoxLayout( controller, 11, 6, "Form1Layout");

    maincontroller = new QFrame( controller, "maincontroller" );
    maincontroller->setMinimumSize( QSize( 0, 35 ) );
    maincontroller->setMaximumSize( QSize( 32767, 35 ) );
    maincontroller->setPaletteBackgroundColor( QColor( 100, 100, 100 ) );
    maincontroller->setFrameShape( QFrame::Box );
    maincontroller->setFrameShadow( QFrame::Plain );
    maincontrollerLayout = new QHBoxLayout( maincontroller, 0, 0, "maincontrollerLayout");

    MainController = new QWidgetStack( maincontroller, "MainController" );

    page = new QWidget( MainController, "page" );
    pageLayout = new QHBoxLayout( page, 0, 0, "pageLayout");
    QSpacerItem* spacer = new QSpacerItem( 30, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer );

    forceplayframe = new QFrame( page, "forceplayframe" );
    forceplayframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, forceplayframe->sizePolicy().hasHeightForWidth() ) );
    forceplayframe->setMinimumSize( QSize( 90, 20 ) );
    forceplayframe->setMaximumSize( QSize( 90, 20 ) );
    forceplayframe->setFrameShape( QFrame::StyledPanel );
    forceplayframe->setFrameShadow( QFrame::Raised );

    forcedplay = new QCheckBox( forceplayframe, "forceplaycheckbox" );
    forcedplay->setGeometry( QRect( 0, 0, 91, 21 ) );
    forcedplay->setText("Loop" );
    pageLayout->addWidget( forceplayframe );
    connect( forcedplay,  SIGNAL(clicked()), this, SLOT(PingPong())  );

    QSpacerItem* spacer_2 = new QSpacerItem( 16, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_2 );
    QSpacerItem* spacer_3 = new QSpacerItem( 16, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_3 );

    controllerframe = new QFrame( page, "controllerframe" );
    controllerframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, controllerframe->sizePolicy().hasHeightForWidth() ) );
    controllerframe->setMinimumSize( QSize( 170, 30 ) );
    controllerframe->setMaximumSize( QSize( 170, 30 ) );
    controllerframe->setFrameShape( QFrame::NoFrame );
    controllerframe->setFrameShadow( QFrame::Plain );

    //////////////////////////////////////////////////////////////////
    // set up the main playbar buttons

    scrubfirst = new JahToolButton( controllerframe, "controllerrewindbutton" );
    addJahPlayerButton( scrubfirst, 0, 1, 27, 29,   28, 30, JahBasePath+"Pixmaps/player/rewind_off.png", JahBasePath+"Pixmaps/player/rewind_on.png" );
    connect( scrubfirst,  SIGNAL(clicked()), this, SLOT(Rewind())  );

    scrubprevious = new JahToolButton( controllerframe, "controllerpreviousbutton" );
    addJahPlayerButton( scrubprevious, 27, 1, 27, 29,  24, 28, JahBasePath+"Pixmaps/player/frewind_off.png", JahBasePath+"Pixmaps/player/frewind_on.png" );
    connect( scrubprevious,  SIGNAL(clicked()), SLOT(FrameBackward())  );

    scrubstop = new JahToolButton( controllerframe, "controllerstopbutton" );
    addJahPlayerButton( scrubstop, 54, 1, 27, 29,    24, 28, JahBasePath+"Pixmaps/player/stop_off.png", JahBasePath+"Pixmaps/player/stop_on.png" );
    connect( scrubstop,  SIGNAL(clicked()), this, SLOT(Stop())  );

    scrubplay = new JahToolButton( controllerframe, "play" );
    addJahPlayerButton( scrubplay, 81, 1, 27, 29 ,    24, 28, JahBasePath+"Pixmaps/player/play_off.png", JahBasePath+"Pixmaps/player/play_on.png" );
    connect( scrubplay,  SIGNAL(clicked()), this, SLOT(LoopForward())  );

    scrubnext = new JahToolButton( controllerframe, "controllernextbutton" );
    addJahPlayerButton( scrubnext, 108, 1, 27, 29,    24, 28, JahBasePath+"Pixmaps/player/ffoward_off.png", JahBasePath+"Pixmaps/player/ffoward_on.png" );
    connect( scrubnext,  SIGNAL(clicked()), SLOT(FrameForward())  );

    scrublast = new JahToolButton( controllerframe, "controllerffworardbutton" );
    addJahPlayerButton( scrublast,  135, 1, 27, 29,   24, 28, JahBasePath+"Pixmaps/player/foward_off.png",  JahBasePath+"Pixmaps/player/foward_on.png" );
    connect( scrublast,  SIGNAL(clicked()), this, SLOT(FastForward())  );

    pageLayout->addWidget( controllerframe );

    controllerslider = new QSlider( page, "controllerslider" );
    //make sure platinum is loaded
    QStyle *platinum=QStyleFactory::create("platinum");
    //if(platinum) qApp->setStyle(platinum);
    if(platinum) controllerslider->setStyle( platinum );
    controllerslider->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, controllerslider->sizePolicy().hasHeightForWidth() ) );


    controllerslider->setMinimumSize( QSize( 490, 0 ) );

    controllerslider->setMaximumSize( QSize( 500, 32767 ) );
    controllerslider->setOrientation( QSlider::Horizontal );
    controllerslider->setTickmarks( QSlider::Right );

	//QObject::connect(controllerslider, SIGNAL(valueChanged(int)), this, SLOT(SetFrameNumber(int)) );
	QObject::connect(controllerslider, SIGNAL(sliderMoved(int)), this, SLOT(SetFrameNumber(int)) );


    pageLayout->addWidget( controllerslider );

    timecodeframe = new QFrame( page, "timecodeframe" );
    timecodeframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0,
                                  timecodeframe->sizePolicy().hasHeightForWidth() ) );
    timecodeframe->setMinimumSize( QSize( 125, 30 ) );
    timecodeframe->setMaximumSize( QSize( 125, 30 ) );
    timecodeframe->setFrameShape( QFrame::NoFrame );
    timecodeframe->setFrameShadow( QFrame::Raised );

    animframe = new InputLCD( timecodeframe, "timecodedisplay",9 );
    addSliderLcd(animframe, 5, 0, 90, 31, 100);
    animframe->setValue( 1 );
    connect( controllerslider,   SIGNAL(valueChanged(int)),   animframe, SLOT(setValue(int)) );
    //connect( animframe, SIGNAL(valueChanged(int)),   SLOT(updateanimframe(int)) );

    pageLayout->addWidget( timecodeframe );

    QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_4 );

    imagingframe = new QFrame( page, "imagingframe" );
    imagingframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0,
                                 imagingframe->sizePolicy().hasHeightForWidth() ) );
    imagingframe->setMinimumSize( QSize( 105, 20 ) );
    imagingframe->setMaximumSize( QSize( 105, 20 ) );
    imagingframe->setFrameShape( QFrame::NoFrame );
    imagingframe->setFrameShadow( QFrame::Raised );

    scrubrender = new QPushButton( imagingframe, "imagingphotobutton" );
    scrubrender->setGeometry( QRect( 0, 0, 50, 20 ) );
    scrubrender->setMinimumSize( QSize( 50, 20 ) );
    scrubrender->setMaximumSize( QSize( 50, 20 ) );
    scrubrender->setText( tr( "FPS"  ) );
    //connect( scrubrender, SIGNAL(clicked()), SLOT( Render()) );   //change this mon
    connect( scrubrender,  SIGNAL(clicked()), SLOT( switchModeFps())  );

    scrubrenderAll = new QPushButton( imagingframe, "imagingrenderbutton" );
    scrubrenderAll->setGeometry( QRect( 55, 0, 50, 20 ) );
    scrubrenderAll->setMinimumSize( QSize( 50, 20 ) );
    scrubrenderAll->setMaximumSize( QSize( 50, 20 ) );
    scrubrenderAll->setText( tr( "Step"  ) );
    //connect( scrubrenderAll, SIGNAL(clicked()), SLOT( RenderScene()) );   //change this mon
    connect( scrubrenderAll,  SIGNAL(clicked()), SLOT( switchModeStep())  );

    pageLayout->addWidget( imagingframe );

    QSpacerItem* spacer_5 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_5 );

    MainController->addWidget( page, 0 );
    maincontrollerLayout->addWidget( MainController );
    Form1Layout->addWidget( maincontroller );
    MainController->raiseWidget(0);
}

void MainEncoder::buildInterface( QHBox* qtinterface )
{

    QStyle *platinum=QStyleFactory::create("platinum");

    ///////////////////////////////////////////////////////////
    //The layers interface
    //LayersFrame = new QFrame( qtinterface, "EffectsFrame" );

    //////////////////////////////////////////////////////////////
    // set up the scene options, or prefernces

    EffectsFrame = new QFrame(qtinterface,"interface");
    EffectsFrame->setGeometry( QRect( 0, 0, 855, 195) );

    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

    connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

    lv->setGeometry( QRect( 2, 2, 146, 155 ) );
    lv->addColumn( "ENCODERS         ", -1 );
    lv->setSorting( -1,1 );   // disables the autosort
	
    /////////////////////////////////////////////////////////////////////
    //set up the optionbar

    encoderUI[0] = new QWidget( EffectsFrame, "encoderUI0" );
    encoderUI[0]->setGeometry( QRect( 148,1, 700, 190 ) ); // 1, 2, 700, 190

    ControlFrame[0] = new QFrame( encoderUI[0], "ControlFrame0" );
    ControlFrame[0]->setGeometry( QRect( 0, 0, 660, 170 ) );


}


void MainEncoder::buildEncoderInterfaces( void )
{

    //initialize platnum stlye here
    QStyle *platinum=QStyleFactory::create("platinum");


    ///////////////////////////////////////////////////////////////////////
    //set up mpeg encoder in ControlFrame[0]

    OutputDirLabel = new QLabel( ControlFrame[0], "OutputDirLabel" );
    OutputDirLabel->setGeometry( QRect( 10, 10, 60, 21 ) );

    OutputDirEdit = new QLineEdit( ControlFrame[0], "OutputDirEdit" );
    OutputDirEdit->setGeometry( QRect( 71, 10, 250, 21 ) );

    OutputDirSearchButton = new QPushButton( ControlFrame[0], "OutputDirSearchButton" );
    OutputDirSearchButton->setGeometry( QRect( 329, 8, 30, 21 ) );
    //QObject::connect(OutputDirSearchButton, SIGNAL(clicked()), this, SLOT(browseFilename()) );

    FileNameLabel = new QLabel( ControlFrame[0], "FileNameLabel" );
    FileNameLabel->setGeometry( QRect( 10, 40, 60, 21 ) );

    FileNameEdit = new QLineEdit( ControlFrame[0], "FileNameEdit" );
    FileNameEdit->setGeometry( QRect( 71, 40, 290, 21 ) );

    ///////////////////////////////////////////////////////////////////
    //frame range info

    RangeInLabel = new QLabel( ControlFrame[0], "RangeInLabel" );
    RangeInLabel->setGeometry( QRect( 10, 70, 50, 21 ) );
    RangeInLabel->setAlignment( int( QLabel::AlignVCenter ) );

    RangeInSpinbox = new QSpinBox( ControlFrame[0], "RangeInSpinbox" );
    RangeInSpinbox->setGeometry( QRect( 70, 70, 110, 21 ) );
    RangeInSpinbox->setMinValue(1);

    RangeOutLabel = new QLabel( ControlFrame[0], "RangeOutLabel" );
    RangeOutLabel->setGeometry( QRect( 190, 70, 50, 21 ) );
    RangeOutLabel->setAlignment( int( QLabel::AlignCenter ) );

    RangeOutSpinbox = new QSpinBox( ControlFrame[0], "RangeOutSpinbox" );
    RangeOutSpinbox->setGeometry( QRect( 250, 70, 110, 21 ) );
    RangeOutSpinbox->setMinValue(1);
    RangeOutSpinbox->setMaxValue(99999);

    ////////////////////////////////////////////////////////////////////
    // the frame size options

    FrameSizeLabel = new QLabel( ControlFrame[0], "FrameSizeLabel" );
    FrameSizeLabel->setGeometry( QRect( 10, 100, 60, 21 ) );

    FrameSizeComboBox = new QComboBox( FALSE, ControlFrame[0], "FrameSizeComboBox" );
    FrameSizeComboBox->setGeometry( QRect( 70, 100, 110, 21 ) );

    //get rid of these later
    int szX = 720; int szY =486;
    QString szStr; //need to get this from videodata not tf

    for(int ii = 1; ii < 5; ii++)
    {
        szStr.sprintf("           %d x %d           ", szX/ii, szY/ii);
        FrameSizeComboBox->insertItem(szStr);
    }

    FrameRateLabel = new QLabel( ControlFrame[0], "FrameRateLabel" );
    FrameRateLabel->setGeometry( QRect( 190, 100, 50, 21 ) );
    FrameRateLabel->setAlignment( int( QLabel::AlignCenter ) );

    FrameRateComboBox = new QComboBox( FALSE, ControlFrame[0], "FrameRateComboBox" );
    FrameRateComboBox->setGeometry( QRect( 250, 100, 110, 21 ) );

    FrameRateComboBox->insertItem("            23.976         ");
    FrameRateComboBox->insertItem("            24             ");
    FrameRateComboBox->insertItem("            25             ");
    FrameRateComboBox->insertItem("            29.97             ");
    FrameRateComboBox->insertItem("            30             ");


    /////////////////////////////////////////////////////////////////
    // the bit rate options

    BitrateLabel = new QLabel( ControlFrame[0], "BitrateLabel" );
    BitrateLabel->setGeometry( QRect( 10, 130, 60, 21 ) );

    BitRateSlider = new QSlider( ControlFrame[0], "BitRateSlider" );
    BitRateSlider->setGeometry( QRect( 70, 130, 170, 21 ) );
    BitRateSlider->setOrientation( QSlider::Horizontal );

    if (platinum) BitRateSlider->setStyle(  platinum );
    BitRateSlider->setTickInterval ( 10 );
    BitRateSlider->setTickmarks ( QSlider::Below );


    BitRateLCD = new InputLCD( ControlFrame[0], "BitRateLCD" );
    BitRateLCD->setGeometry( QRect( 250, 130, 110, 21 ) );

    BitRateLCD->setLineWidth( 1 );
    BitRateLCD->setNumDigits( 8 );
    BitRateLCD->setSegmentStyle( QLCDNumber::Flat );
    BitRateLCD->setActive(true);

    connect( BitRateSlider,     SIGNAL(valueChanged(int)), BitRateLCD, SLOT(setValue(int)) );
    connect( BitRateLCD,        SIGNAL(incrementValueChanged(int)), BitRateSlider, SLOT(setValue(int)) );


    //languageChange()
    OutputDirLabel->setText( tr( "Output Dir" ) );
    FileNameLabel->setText( tr( "Filename" ) );
    OutputDirSearchButton->setText( tr( "..." ) );
    FrameSizeLabel->setText( tr( "FrameSize" ) );
    BitrateLabel->setText( tr( "Bitrate" ) );
    RangeInLabel->setText( tr( "In Frame" ) );
    RangeOutLabel->setText( tr( "Out" ) );
    FrameRateLabel->setText( tr( "Rate" ) );



    EncodeButton = new QPushButton( ControlFrame[0], "EncodeButton" );
    EncodeButton->setGeometry( QRect( 500, 130, 160, 21 ) );
    EncodeButton->setText( tr( "ENCODE CLIP" ) );
    connect(EncodeButton, SIGNAL(clicked()), this, SLOT(runEncoder()) );


}

void MainEncoder::buildEncoderInterfacesMpeg( void )
{
    //initialize platnum stlye here
    QStyle *platinum=QStyleFactory::create("platinum");

    ////////////////////////////////////////////////////////
    // the options

    OptionLabel_1 = new QLabel( ControlFrame[0], "chroma" );
    OptionLabel_1->setGeometry( QRect( 400, 10, 81, 21 ) );

    OptionComboBox_1 = new QComboBox( FALSE, ControlFrame[0], "chromacombobox" );
    OptionComboBox_1->setGeometry( QRect( 500, 10, 160, 21 ) );

    OptionComboBox_1->insertItem("    4:2:0    ");
    OptionComboBox_1->insertItem("    4:2:2    ");
    OptionComboBox_1->insertItem("    4:4:4    ");

    OptionLabel_2 = new QLabel( ControlFrame[0], "precision" );
    OptionLabel_2->setGeometry( QRect( 400, 40, 81, 21 ) );

    OptionComboBox_2 = new QComboBox( FALSE, ControlFrame[0], "precisioncombobox" );
    OptionComboBox_2->setGeometry( QRect( 500, 40, 160, 21 ) );

    OptionComboBox_2->insertItem("    8 bit    ");
    OptionComboBox_2->insertItem("    9 bit    ");
    OptionComboBox_2->insertItem("    10 bit    ");
    OptionComboBox_2->insertItem("    11 bit    ");

    OptionLabel_3 = new QLabel( ControlFrame[0], "formula" );
    OptionLabel_3->setGeometry( QRect( 400, 70, 81, 21 ) );

    OptionComboBox_3 = new QComboBox( FALSE, ControlFrame[0], "formulacombobox" );
    OptionComboBox_3->setGeometry( QRect( 500, 70, 160, 21 ) );

    OptionComboBox_3->insertItem("    High       ");
    OptionComboBox_3->insertItem("    Spatial    ");
    OptionComboBox_3->insertItem("    SNR        ");
    OptionComboBox_3->insertItem("    Main       ");
    OptionComboBox_3->insertItem("    Simple     ");



    OptionLabel_2->setText( tr( "Precision" ) );
    OptionLabel_1->setText( tr( "Chroma" ) );
    OptionLabel_3->setText( tr( "Formula" ) );
}



void MainEncoder::buildEncoderInterfacesAvi( void )
{

    ///////////////////////////////////////////////////////////////////////
    //set up avi
    //ControlFrame[1] = new QFrame( encoderUI[1], "ControlFrame1" );



}



//this was taken from World/glworldinterface since we dont share code with it
//we could break out all interface building code into its own library
//and put it in libraries folder!

void MainEncoder::addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range) {

    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    lcd->setGeometry( QRect( x, y, w, h ) );
    lcd->setPalette( p, TRUE );
    lcd->setLineWidth( 1 );
    lcd->setSegmentStyle( QLCDNumber::Flat );  lcd->setActive	( true);
    lcd->setValue( 0 );

}

void MainEncoder::addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                                  int w, int h, QString iconOff, QString iconOn ) {

    button->setToggleButton(false);
    //button->setFixedSize(w,h);
    button->setUsesBigPixmap(true);
    button->setGeometry( QRect( x1, y1, x2, y2 ) );

    QIconSet *icoSet;
    icoSet = new QIconSet( QPixmap(iconOff) );
    icoSet->setPixmap    ( QPixmap(iconOn), QIconSet::Large, QIconSet::Active);
    icoSet->setPixmap    ( QPixmap(iconOff), QIconSet::Large, QIconSet::Disabled);
    icoSet->setPixmap    ( QPixmap(iconOn), QIconSet::Small, QIconSet::Active);
    icoSet->setPixmap    ( QPixmap(iconOff), QIconSet::Small, QIconSet::Disabled);
    button->setIconSet(*icoSet);
    //delete icoSet;

}

void MainEncoder::switchModeFps(void)
{

	int fpsval;

   QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( "15",4,0 );
    menu->insertItem( "24",3,0 );
    menu->insertItem( "30",2,0 );
    menu->insertItem( "60",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( scrubrender->mapToGlobal( QPoint( 0, scrubrender->height() + 1 ) ) );

    if ( id != -1 ) {

      switch(id) {
         case 1  : { fpsval = 60; break; }
         case 2  : { fpsval = 30; break; }
         case 3  : { fpsval = 24; break; }
         case 4  : { fpsval = 15; break; }
         default : { fpsval = 30; break; }
      }
    }

    delete menu;

	switchFps(fpsval);

}

void MainEncoder::switchModeStep(void)
{
	int stepval;

    QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( "1",4,0 );
    menu->insertItem( "2",3,0 );
    menu->insertItem( "3",2,0 );
    menu->insertItem( "5",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( scrubrenderAll->mapToGlobal( QPoint( 0, scrubrenderAll->height() + 1 ) ) );

    if ( id != -1 ) {
      //if (numberLayers==NUMBER_LAYERS) return;
      //numberLayers+=1; layNum.setNum(numberLayers);

      switch(id) {
         case 1  : { stepval = 5; break; }
         case 2  : { stepval = 3; break; }
         case 3  : { stepval = 2; break; }
         case 4  : { stepval = 1; break; }
         default : { stepval = 1; break; }
      }
    }

    delete menu;

	SetStep (stepval);
}
