/*******************************************************************************
**
** The header file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "MainEncoder.h"

//enum JahEncoder { NONE, JAHMPEG, LINAVI, WINAVI, THEORA, QUICKTIME, DIVX, NumEncoders };

void MainEncoder::addEncoders()
{
    //this adds the supported encoders to the list of encoders
    //this needs a lot of clenaing up to work properly
    //as each encoder should really be a plugin 
    #ifdef WIN32
    addEncoder("AVI");
    #endif

    #ifdef JAHMPEGENCODER
    addEncoder("MPEG");
    #endif
}

void MainEncoder::addEncoder(QString encodername)
{
    //bool status = true;

    //clear all current selections
    lv->clearSelection();

    (void) new QListViewItem( lv, encodername );
}

void MainEncoder::selectEncoder(QString encodername)
{
    debug("selecting encoder");

    if (encodername=="MPEG")
            theEncoder = JAHMPEG;

    if (encodername=="AVI")
            theEncoder = WINAVI;


}

void MainEncoder::clicked( QListViewItem* items )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "**Clicked on layer in MainEncoder ");


    //nothing added to listview yet
    if (!items) return;

    //iterate the list now to see whats selected

    //store the reuslts here
    QString encoder;

    QListViewItemIterator it( lv );
    while ( it.current() ) 
    {
        QListViewItem *item = it.current();

        if ( item->isSelected() ) 
        { 
            encoder = item->text(0); 
        }
        ++it;
    }

    //QString ras;
    debug("selected:"+encoder);

    //brings up the menu for the selected layer according to layer number
    //and sets the selected encoder to what was selected
    selectEncoder(encoder);

}

/////////////////////////////////////////////////////////////////
// encoder starts here

void	MainEncoder::runEncoder()
{
    bool runval;

    runval = setupConfigForEncoding();

    if (runval)
    {
        if (encodingStatus==false)
        {
            encodingStatus = true;

            debug("launching encoder now...");

            RenderDialog *  thedialog = new RenderDialog( "jahstartup", configData, assetData, theEncoder );
            thedialog->show();     //this is modeless :)
            thedialog->doEncode();     //this is modeless :)

            debug("finished using encoder");
            encodingStatus = false;
        }
        //launch encoder
        //pas it the config object
    }

}

bool	MainEncoder::setupConfigForEncoding()
{

    /////////////////////////////////////////////////////////////////
    //set up config object for encoding

    bool retVal = true;

    //int sz;

    int conversionmode = FrameSizeComboBox->currentItem();
    //should be 0 to 4... need to map to 1 to 5
    conversionmode +=1;

    assetExchange exchange;
    QImage assetimage = exchange.getImage(assetData);
    int szX = assetimage.width();
    int szY = assetimage.height();

    int x = (int)(szX/conversionmode);
    int y = (int)(szY/conversionmode);

    QString rere;
    debug("the x val is:"+rere.setNum(x));
    debug("the y val is:"+rere.setNum(y));
    debug("mode     is:"+rere.setNum(conversionmode));


    configData->conversionratio =      conversionmode;
    configData->mpegXres=              x; //720;
    configData->mpegYres=              y; //486;
    configData->mpegDisplayX=          x; //352;
    configData->mpegDisplayY=          y; //288;


    //set up destination filename defaults... need to be careful here!
    QString outdir = OutputDirEdit->text();
    char * slot1 = new char [outdir.length()];
    strcpy (slot1,outdir);
    configData->DestDirectory = slot1;

    //set up destination filename defaults... need to be careful here!
    QString outfile = parseDestinationFilename(FileNameEdit->text());
    char * slot2 = new char [outfile.length()];
    strcpy (slot2,outfile);
    configData->DestFilename = slot2;

    //debug(configData->DestDirectory);
    //debug(configData->DestFilename);

    int bFrame   = RangeInSpinbox->value();
    int eFrame   = RangeOutSpinbox->value();
    int numFrames = eFrame - bFrame;

    //defaults
    configData->startFrame = bFrame; //startframe
    configData->endFrame   = eFrame; //startframe

    //mpeg specific
    configData->mpegFirstFrame = bFrame; //startframe
    configData->mpegNumberFrames =  numFrames; //sendframe

    configData->mpegBitRate = BitRateLCD->intValue();

    configData->mpegChromaFormat        =  OptionComboBox_1->currentItem()+1;//    1;
    configData->mpegProfileID           =  OptionComboBox_3->currentItem()+1;//    4;
    configData->mpegIntraDcPrecision    =  OptionComboBox_2->currentItem();//    0;

    return retVal;
}

/////////////////////////////////////////////////////////////////
// very basic destination parsing...

QString	MainEncoder::parseDestinationFilename(QString filename)
{
    QString returnname;

    ////////////////////////////////////////////////////////////
    // for avi encoder

    if (theEncoder==WINAVI)
    {
        if (! ( (filename.endsWith(".avi")) || (filename.endsWith(".AVI")) ) )
        {
            returnname = filename + ".avi";
        }
        else
        {
            returnname = filename;
        }
    }

    ////////////////////////////////////////////////////////////
    // for mpeg encoder

    if (theEncoder==JAHMPEG)
    {
        if (! ( (filename.endsWith(".mpg")) || (filename.endsWith(".MPG")) ) )
        {
            returnname = filename + ".mpg";
        }
        else
        {
            returnname = filename;
        }
    }

    return returnname;
}

void MainEncoder::updateEncoderSettings()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Updating Encode Settings");


    ////////////////////////////////////////////////////////
//set the defaults up

    assetExchange exchange;
    QImage assetimage = exchange.getImage(assetData);
    int szX = assetimage.width();
    int szY = assetimage.height();

    //before updating settings
    //we need to remove all the current settings
    QString szStr; //need to get this from videodata not tf
    FrameSizeComboBox->clear();

    for(int ii = 1; ii < 5; ii++)
    {
        szStr.sprintf("           %d x %d           ", szX/ii, szY/ii);
        FrameSizeComboBox->insertItem(szStr);
        //FrameSizeComboBox->changeItem(szStr,ii);
    }


    OutputDirEdit->setText(JahMediaPath+"media/encoded/");

    //filename is stored in patternEdit for now
    QString fileString;
    fileString	= assetData.clipname;
    FileNameEdit->setText(fileString);


    /////////////////////////////////////////////////////////
    //first we load the presets into the configoption
    loadEncoderSettings();


    ///////////////////////////////////////////////////////////////////////////
    // now we set up all the details...

    //set default options
    RangeInSpinbox->setValue(configData->mpegFirstFrame);       //startframe
    RangeOutSpinbox->setValue(configData->mpegNumberFrames);    //endframe

    FrameRateComboBox->setCurrentItem(configData->mpegFrameRate);

    //what options can we put here?
    OptionComboBox_1->setCurrentItem(0); //4:2:0
    OptionComboBox_2->setCurrentItem(0); //8 bit
    OptionComboBox_3->setCurrentItem(0); //Simple

    BitRateSlider->setMinValue( 1 );
    BitRateSlider->setMaxValue( (int)(configData->mpegBitRate/10000) ); //use default as max ? 200000/100
    BitRateSlider->setValue   ( (int)(configData->mpegBitRate/10000)/2 );

    BitRateLCD->setIncrement( 10000 );
    BitRateLCD->setValue( (int)(configData->mpegBitRate/10000)/2 );



}


void	MainEncoder::loadEncoderSettings() //
{

    /////////////////////////////////////////////////////
    //these are global
    QString sfname = assetData.filename;
    char * slot1 = new char [sfname.length()];
    strcpy (slot1,sfname);
    configData->SourceFilename = slot1;

    QString sfloc = assetData.location;
    char * slot2 = new char [sfloc.length()];
    strcpy (slot2,sfloc);
    configData->SourceDirectory = slot2;

    QString sfext = assetData.extension;
    char * slot3 = new char [sfext.length()];
    strcpy (slot3,sfext);
    configData->SourceExtension = slot3;

    if (assetData.videoext)
    {
        //thought videotex was the flag here?
        //debug(">>it is videotex");

        QString sfvext = assetData.videoext;
        char * slot4 = new char [sfvext.length()];
        strcpy (slot4,sfvext);
        configData->FramePadding = slot4;

        //configData->FramePadding    =   assetData->videoext;
        //configData->FramePadding    =   "0001";
    }

    //set up destination filename defaults... need to be careful here!
    QString destname = assetData.clipname+".mpg";
    char * rere = new char [destname.length()];
    strcpy (rere,destname);
    configData->DestFilename = rere;

    QString destdir = JahMediaPath+"media/encoded/";
    char * rara = new char [destdir.length()];
    strcpy (rara,destdir);
    configData->DestDirectory = rara;

    //////////////////////////////////////////////////////
    //these are unique to the mpeg2 encoder
    configData->mpeg_tplref="-";
    configData->mpeg_iqname="-";
    configData->mpeg_niqname="-";
    configData->mpeg_statname="-";

    //3 means we use the new QImage type!!!
    configData->mpegPicFormat=         3;

    //dont think this is really accurate - we may be underadjusting here
    configData->mpegNumberFrames=      assetData.endframe - assetData.startframe;
    configData->mpegFirstFrame=        assetData.startframe;

    configData->mpegStartTimecode=     "00:00:00:00";
    configData->mpegGopFrames=         12;
    configData->mpegIPframeDist=       3;
    configData->mpegISOstream=         1;
    configData->mpegFieldFrameSwitch=  0;

    configData->mpegXres=              720; //720;
    configData->mpegYres=              486; //486;
    configData->conversionratio =      1;

    configData->mpegAspectRatio=       8;

    /* frame_rate_code 1=23.976, 2=24, 3=25, 4=29.97, 5=30 frames/sec. */
    configData->mpegFrameRate=         3;

    configData->mpegBitRate=           2000000.0;
    configData->mpegVdvBufferSize=     20;
    configData->mpegLowDelay=          0;
    configData->mpegConstParamFlag=    1;
    configData->mpegProfileID=         4;
    configData->mpegLevelID=           8;
    configData->mpegProgessiveSeq=     1;
    configData->mpegChromaFormat=      1;
    configData->mpegVideoFormat=       1;
    configData->mpegColorPrimaries=    5;
    configData->mpegTransferCharac=    5;
    configData->mpegMatricCoeff=       5;
    configData->mpegDisplayX=          720; //352;
    configData->mpegDisplayY=          486; //288;
    configData->mpegIntraDcPrecision=  0;
    configData->mpegTopFieldFirst=     0;


    //we can make these 3 value arrays
    configData->mpeg_frame_pred_dct_tab[0]=1;
    configData->mpeg_frame_pred_dct_tab[1]=1;
    configData->mpeg_frame_pred_dct_tab[2]=1;

    configData->mpeg_conceal_tab[0]=0;
    configData->mpeg_conceal_tab[1]=0;
    configData->mpeg_conceal_tab[2]=0;

    configData->mpeg_qscale_tab[0]=0;
    configData->mpeg_qscale_tab[1]=0;
    configData->mpeg_qscale_tab[2]=0;

    configData->mpeg_intravlc_tab[0]=0;
    configData->mpeg_intravlc_tab[1]=0;
    configData->mpeg_intravlc_tab[2]=0;

    configData->mpeg_altscan_tab[0]=0;
    configData->mpeg_altscan_tab[1]=0;
    configData->mpeg_altscan_tab[2]=0;


    configData->mpegRepeatFirstField=   0;
    configData->mpegProgressiveFrame=   1;
    configData->mpegPDistanceRefresh=   0;
    configData->mpegRRateControl=       0;
    configData->mpegAvgRateControl=     0;
    configData->mpegXiRateControl=      0;
    configData->mpegXpRateControl=      0;
    configData->mpegXbRateControl=      0;
    configData->mpegdOiRateControl=     0;
    configData->mpegdOpRateControl=     0;
    configData->mpegdObRateControl=     0;

    //we can make these 4 value arrays
    configData->motion_data[0]=         2;
    configData->motion_data[1]=         2;
    configData->motion_data[2]=         11;
    configData->motion_data[3]=         11;

    configData->mpegB1FHorizCode=           1;
    configData->mpegB1FVertCode=            1;
    configData->mpegB1FSearchWidth=         3;
    configData->mpegB1FSearchHeight=        3;

    configData->mpegB1BHorizCode=           1;
    configData->mpegB1BVertCode=            1;
    configData->mpegB1BSearchWidth=         7;
    configData->mpegB1BSearchHeight=        7;

    configData->mpegB2FHorizCode=           1;
    configData->mpegB2FVertCode=            1;
    configData->mpegB2FSearchWidth=         7;
    configData->mpegB2FSearchHeight=        7;

    configData->mpegB2BHorizCode=           1;
    configData->mpegB2BVertCode=            1;
    configData->mpegB2BSearchWidth=         3;
    configData->mpegB2BSearchHeight=        3;

}


void	MainEncoder::browseFilename()
{
    QString fileName;
    fileName = QFileDialog::getSaveFileName(OutputDirEdit->text(), "*.mpg", this);

    if(!fileName.isEmpty())
        OutputDirEdit->setText(fileName);

}


