#include "RenderDialog.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <qstyle.h>
#include <qstylefactory.h>


RenderDialog::RenderDialog(const char *name, RenderConfig * thedata, videoData assetData, JahEncoder encoder)
    : QDialog(0, "JahEncoder", false)
{
    //initialize JahBasePath
    //JahPrefs& jprefs = JahPrefs::getInstance();
    //JahMediaPath = jprefs.getMediaPath();

    //create asset object and store videodata in it
    configData = new RenderConfig;
    configData = thedata;

	assetObject = assetData;

	theEncoder = encoder;

	//determine encoder
	QString encName;

	if (theEncoder == JAHMPEG)
		encName = "The MPEG Encoder";

	if (theEncoder == WINAVI)
		encName = "The AVI Encoder";

    /////////////////////////////////////////////////////////
    //start building the interface for the dialog box
    TitleLabel = new QLabel( this, "Title" );
    TitleLabel->setGeometry( QRect( 5, 0, 395, 21 ) );
    TitleLabel->setText( encName );
    TitleLabel->setAlignment( int( QLabel::AlignCenter ) );


    encodeOutput = new QMultiLineEdit(this);
    encodeOutput->setFont(QFont("system", 8));
    encodeOutput->setText("Encoder Output:\n-------------------");
    encodeOutput->setEnabled(false);
    encodeOutput->setGeometry( QRect( 5, 40, 395, 300 ) );

    //my progress bar
    progBar = new QProgressBar(this);
    progBar->setGeometry( QRect( 5, 350, 340, 30 ) );


    cancelBut	= new QPushButton("Cancel", this);
    cancelBut->setGeometry( QRect( 340, 350, 45, 30 ) );
    cancelBut->setText("Cancel");


    //-- Connect -------------
    connect(cancelBut, SIGNAL(clicked()), this, SLOT(cancelPressed()) );

    resize(400, 400);


}

RenderDialog::~RenderDialog()
{
}


//----------------------------------------------------------
void	RenderDialog::cancelPressed()
{
    reject();
}

//----------------------------------------------------------
void	RenderDialog::doEncode()
{

    //////////////////////////////////////////////////////////////
    //setup misc variables and launch encoder
    debug(">>starting mpeg encode");

    QString translate;

    //update the user as to whats going on
    encodeOutput->append("Encoding MPEG File");

    int bFrame = configData->startFrame; //startframe
    int eFrame = configData->endFrame;   //endframe

    int numFrames = eFrame - bFrame;

    encodeOutput->append("Start frame :"+translate.setNum(bFrame));
    encodeOutput->append("End Frame   :"+translate.setNum(eFrame));
    encodeOutput->append("Total frames:"+translate.setNum(numFrames));

    //int da = configData->mpegChromaFormat;
    //int db= configData->mpegProfileID;
    //int dc= configData->mpegIntraDcPrecision;
    //encodeOutput->append("formatting:"+translate.setNum(da)+" "+translate.setNum(db)+" "+translate.setNum(dc)     ); //1 4 0

    ////////////////////////////////////////////////////////////
    //this command launches the encoder... in a thread
    //we send over the videodata and the asset data objects
    //need more flags and variables in the configData object
    //so it knows what to do
    //mythread = new EncodeThread(this,*assetData, *configData);
    //videoData mydata;

    ////////////////////////////////////////////////////////
    //mpeg encoder needs these values here sent to it
    //and they are not being set for some reason

    debug(configData->SourceFilename); //"AnimationClip4_"
    debug(configData->SourceDirectory);
    debug(configData->SourceExtension);
    debug(configData->FramePadding);   //"0001"

    debug(configData->DestDirectory);
    debug(configData->DestFilename);

    mythread = new EncodeThread(this, assetObject, *configData, theEncoder);
    mythread->start();

    encodeOutput->append("Starting Encoder...");


    //set up the progress bar
    progBar->setTotalSteps(numFrames);
    progBar->setPercentageVisible (true);

    QString result,lastresult;
    int curFrame;
    bool wascancelled = false;

    //////////////////////////////////////////////////////////////
    //this is the main loop...
    //this is where we can poll the encoder to see whats going on
    while (mythread->running())
    {
		/////////////////////////////////////////////////////////
		//actually not a win32 but are we encoding for avi on windows
		//as mpeg on windows uses checkmessage as well
		mythread->checkmessage(result);

        if (result!=lastresult)
        {
            encodeOutput->append(">> "+result);

            translate = result.section( ':', -1 );
            curFrame = translate.toInt();
            progBar->setProgress(curFrame);
        }

        //implement a cancel button here but it needs to clean up
        //behind it ie erase dest file, release memory, etc
        //if(progress.wasCancelled()) 
		//{ 
		//	mythread->stop(); wascancelled = true; break; 
		//}

        lastresult = result;

        qApp->processEvents();

    };

    if (!wascancelled)
    {
        progBar->setProgress(eFrame);
    }

    //encodeBut->setEnabled(true);
    cancelBut->setText("Close");
    progBar->setPercentageVisible (false);
    encodeOutput->append("Finished Encoding...");

    //leave and clean up
    //debug("finished encoding");

    //the cleanup isnt releasing the encoder as calling it a 2nd time
    //it dont work right...
    mythread->cleanup();
    delete mythread;

}


