/*******************************************************************************
**
** The header file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef ENCODE_THREAD_h
#define ENCODE_THREAD_h


/////////////////////////////////////////////////
// RenderConfig class stores configuration settings
// and is used to send them to the encoder...
#include "RenderConfig.h"

//////////////////////////////////////////////////
// asset/media is sent as a videodata object
#include "videodata.h"

#include <qthread.h>
#include <qsemaphore.h>
#include <qmutex.h>

/////////////////////////////////////////////////
// used if the mpeg encoder is initialized
extern "C" int mpeglauncher (RenderConfig theconfig);
extern "C" char *mpegenc_feedback;

//for avi support
#ifdef WIN32
#include <RenderAvi.h>
#include <avi_utils.h>
#endif

#if defined(QT_NO_THREAD)
#  error Thread support not enabled.
#endif


class EncodeThread : public QThread
{
public:
    EncodeThread(QWidget *o, videoData thedata, RenderConfig theconfig, JahEncoder theEncoder) 
        : receiver(o), stopped(FALSE)
    { 
        frame = 0; 

        assetconfig = theconfig;
        assetdata   = thedata;
        encoder		= theEncoder;
    }

    ~EncodeThread(void);

    void run();
    void stop();
    void cleanup();

    void checkmessage(QString& data);
    void checkmessageString(QString& data);


private:

    QWidget *receiver;
    QMutex mutex;
    bool stopped;

    QString feedbackString;
    int frame;

    RenderConfig assetconfig;
    videoData    assetdata;

    JahEncoder encoder;

};



#endif // RenderMPEGDialog_h
