/*******************************************************************************
**
** The source file for the Jahshaka 3d player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "glplayer.h"


void GLPlayer::initializeObjects()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLAnime Class");

    //initialize the keyframe editor
    //thekeyframer = new JahKeyframer( this, moduleOptionsUI[5], "keyframer" );
    //thekeyframer->setFixedSize( QSize( 980, 306 ) ); 
    //connectKeyframer();
    //haskeyframer = true; //let the world know its there

    //make the current opengl context current so the textures can be set up properly
    makeCurrent();

    ///////////////////////////////////////////////
    //this creates the default world layer using the linked list
    layerlist->append( new NewLayerList(0, "World") );

    //setup and build the default layer these should all be defaults
    layerlist->at(0)->thelayer->blend = TRUE;
    layerlist->at(0)->thelayer->depth = FALSE;
    layerlist->at(0)->thelayer->layerAssetStatus = TRUE;
    layerlist->at(0)->thelayer->asset.filename = JahBasePath+"Pixmaps/jahlayer";
    layerlist->at(0)->thelayer->asset.extension = ".bmp";
    //layerlist->at(0)->thelayer->layerfog = TRUE;
    buildLayer(0,"World", LAYER, BACKDROP );

    //adds default layer as 'off' uses default numberLayers = 1
    addSelection(FALSE);  

    //hook up menu and sliders and default tool
    //objectMenu(numberLayers);
    //updateSliders();

    //set the ui tool
    noTool();

    //turn forceplay on here
    //setForcePlay(true);

	//ignore the foreground lock
	ignoreForegroundLock = true;


}

// the main opengl routine that iterates through all objects
// and paints the scene for us
void GLPlayer::paintGL()
{
    //checks to see if the play button is on and increments frames
    if ( animation )
    {  
        updateanimation(currentFrame); currentFrame++;
        if ( currentFrame > maxFrame ) currentFrame=1;
    }

    //turn on hardware aliasing for the scene if it is enabled

    if (hwaliasing)
    {
        glEnable(GL_MULTISAMPLE_ARB);
    }

    //clear the scene and set up matrices
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    //glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();


    //draw black video safe area if no layer present
    if (!select) 
    { 
        if ( isLayerVisible(0) )     
        {  
            renderSpace->drawLayer(*(layerlist->at(0)->thelayer));  
        }
        else 
        {   
            renderSpace->drawLayer(*titlesafe); 
        }
    }

    //we position the camera here
    renderSpace->positionObject(camera->node);

    //draw the grid if its visible
    if (gridval) {  renderSpace->drawLayer(*thegrid); }

    //set up opengl layer selection
    glInitNames(); 
    glPushName(0);                      // initialize the selection stack

    /////////////////////////////////////////////////
    //the main loop where we draw all visible layers
    int dlayer;

    //sort the layers before drawing them
    newsortLayers();

    //now we iterate the list...
    for ( uint i = 1; i <= layerlist->count(); ++i )
    {
        if ( layerlist->at(i) )
        {
            if (numberLayers>1 && newworldorder->at(i-1))
            {
                dlayer = newworldorder->at(i-1)->laynum;
            }
            else
            {
                dlayer = i;
            }

			//we ignore lock to foreground in the player
			//so we can get a true world rotate...
			//should make this a option...
			//ignoreForegroundLock = true;

            //if ( (isLayerVisible(dlayer)) && (layerlist->at(dlayer)->thelayer->foreground !=1 ))
			bool foreLock;
			if (ignoreForegroundLock)
				foreLock = true;
			else
				foreLock = (layerlist->at(dlayer)->thelayer->foreground !=1);


			
            if ( (isLayerVisible(dlayer)) && foreLock)
            {
                //build layer effects first  
                if (layerlist->at(dlayer)->thelayer->layerEffect)
                {
                    buildLayerEffect(dlayer);
                }

                if (layerlist->at(dlayer)->thelayer->objtype==MESHLAYER)
                {
					initializeMeshLayer(dlayer);
                    //buildLayerEffect(dlayer);
                }

                //then draw the layer

                renderSpace->drawLayer(*(layerlist->at(dlayer)->thelayer));

                //then draw the keyframes if visible
                if (layerlist->at(dlayer)->thelayer->drawKeyframes)
                {
                    renderSpace->drawNodes(*(layerlist->at(dlayer)->thelayer));
                }
            }
        }
    }


    glPopMatrix();

    /////////////////////////////////////////////////////////////////////////////
    //draw the forground layers ie objects not affeceted by worldspace or sorting
    
    if (numberLayers>0 && !ignoreForegroundLock) 
    {
        for ( uint i = 1; i <= layerlist->count(); ++i )
        //for (int i=1; i<=numberLayers; i++) 
        {
        glPushMatrix();
        if ( layerlist->at(i) ) 
        {
            if (isLayerVisible(i) && layerlist->at(i)->thelayer->foreground==1) 
            {
                renderSpace->drawLayer(*(layerlist->at(i)->thelayer));
                if(layerlist->at(i)->thelayer->drawKeyframes) 
                { 
                    renderSpace->drawNodes(*(layerlist->at(i)->thelayer)); 
                }
            }
        }
        glPopMatrix();
        }
    }

    //if textsafe is on draw it as well
    if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

    if (hwaliasing)
    {
        glDisable(GL_MULTISAMPLE_ARB);
    }

    //flush render que
    glFlush(); 

    /////////////////////////////////////////////////
    // finished the scene now Wait this many msecs before redraw
    if ( animation ) 
    {
        timer->start( redrawWait, TRUE );
    } 

}

//all modules are responsable for their own interface and their own sliders
//we need to find a different way of doing the updates
//ie update only the current settings for the current tabbox

//both uioptions and ui settings need to be local

void GLPlayer::updateUiOptions()
{
}

void GLPlayer::updateUiSettings()
{
    ////////////////////////////////////////////////////////////////////////////////////
    // all other ui settings
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "GLAnime","updating ui settings");

    //extended values here need to add everything new
    //GridSelect->setChecked(gridval);
    //TextSelect->setChecked(textsafe->layerStatus);
}






















