/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "paint.h"


//first thing to do is make buffer a array so we can store multiple clips
//allowing us to paint on video or sequences
//alpha support as well would be cool
//then we need to add layer support
//i think a new paint engine is necessary really...

void GLPaint::initializeObjects() 
{    
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing GLPaint Class");
	    
	//make the current opengl context current so the textures can be set up properly
    makeCurrent();

    myHistoryStartLoc = myHistoryEndLoc = myHistoryLoc = 0;

    myPenColor        = new QColor(0,58,140);
    myFillColor       = new QColor(0,97,140);
    myBackgroundColor = new QColor(0,0,0);

    myBrushSize = 3;
    myGradientDegree = 76;
    myFadeDegree = 128;
    myActiveTool = none;

    glPointSize(1);
    glClear(GL_COLOR_BUFFER_BIT);

    buffer = core->grabFrameBuffer(true);

    clear();

	//minFrame=1; 
	//maxFrame=1;
}



 /*
 | Clear the canvas with the background color
*/
void GLPaint::clear() 
{
  buffer.fill( myBackgroundColor->pixel() );
  openPic=true;
  updateGL();
  historyStore();
}

void GLPaint::addClip(videoData clip) 
{

    assetExchange exchange;
    QImage theimage; 
    theimage = exchange.getImage(clip);

	buffer = theimage;

    X_RESOLUTION  = theimage.width();
    Y_RESOLUTION = theimage.height();

	core->setFixedSize( QSize( X_RESOLUTION, Y_RESOLUTION ) );

    openPic=true;
    historyStore();
    updateGL();

    qApp->processEvents();

}


 /*
 | Receive a mouse press event.
 | Store the location at which the mouse was pressed for drawing purposes.
*/
void GLPaint::mousePressEvent( QMouseEvent *e ) {

  mousePressed = true;
  x1 = e->x();
  y1 = core->height() - e->y();
  x2 = x1;
  y2 = y1;
}

 /*
 | Receive a mouse release event.
 | With the active tool, clear any XOR drawings and make a regular drawing.
*/
void GLPaint::mouseReleaseEvent( QMouseEvent * ) {

   // rubber-banding tools only:
  if ( myActiveTool != pen ) 
  {
     //draw over previous XOR drawing, cancelling it out
    glLogicOp(GL_XOR);              // draw in XOR mode
    drawWithActiveTool();
     // draw the final drawing, for the current mouse location
    glLogicOp(GL_COPY);  // switch back to drawing in COPY mode, the default
    drawWithActiveTool();
  }

  mousePressed = false;
  updateGL();

  buffer=core->grabFrameBuffer();	// save image for resizing

  historyStore();

}

 /*
 | Receive a mouse motion event
 | Store the new mouse location as the destination drawing point.
 | With the active tool, remove any previous XOR drawing and draw a new one.
*/
void GLPaint::mouseMoveEvent( QMouseEvent *e ) 
{

    if ( mousePressed ) {  // drawing is currently occurring
    if ( myActiveTool == pen ) 
    {
      x1 = x2;
      y1 = y2;
      x2 = e->x();
      y2 = core->height() - e->y();
      drawWithActiveTool();
    } 
    else 
    {
      glLogicOp(GL_XOR);      // draw in XOR mode
       //draw over previous XOR drawing, cancelling it out
      drawWithActiveTool();
       //draw new XOR drawing, for the current mouse location
      x2 = e->x();
      y2 = core->height() - e->y();
      drawWithActiveTool();
      glLogicOp(GL_COPY);    // switch back to drawing in COPY mode, the default
    }
    updateGL();
  }
}

 /*
 | Resize this canvas to fit the current window size.
*/

void GLPaint::resizePaintGL( void ) 
{

    int w = core->renderwidth;
    int h = core->renderheight;

    buffer = buffer.scale(w, h);	//stretch or shrink image
    openPic = true;
    updateGL();
}

 /*
 | If the buffer has been changed, by opening a picture from a file or by
 | a manipulation function, draw the pixels from the buffer into the screen.
*/
void GLPaint::paintGL( ) 
{

  if (openPic) {
    QImage gl_buffer = core->convertToGLFormat(buffer);
    //QImage gl_buffer = QGLWidget::convertToGLFormat(buffer);
    glRasterPos2i(0,0);
    glDrawPixels(buffer.width(), buffer.height(), GL_RGBA,
      GL_UNSIGNED_BYTE, gl_buffer.bits());
    glFlush();
    openPic = false;

  }
}

void GLPaint::refreshGL( ) {
  qApp->processEvents();

  //debug("refreshing");
  makeCurrent();
  historyRetrieve(0); glFlush();
  paintGL( );
  //updateGL();
  qApp->processEvents();
}

 /*
 | Draw with the active tool using this canvas's two points, x1,y1 and x2,y2.
*/
void GLPaint::drawWithActiveTool() {

  GLint   minPointSize = 1;
  color3 penColor,  penColorDark,  penColorLight,
         fillColor, fillColorDark, fillColorLight;

  //switch from QT colors to openGL colors, 0...255  to  0.0...1.0
  GLfloat gradientDegree = myGradientDegree / 255.0;
  penColor[0] = myPenColor->red()   / 255.0;
  penColor[1] = myPenColor->green() / 255.0;
  penColor[2] = myPenColor->blue()  / 255.0;

  penColorLight[0] = penColor[0];
  penColorLight[1] = penColor[1];
  penColorLight[2] = penColor[2];

  //penColorDark  = penColor;     need to write a copy constructor to do this for me
  penColorDark[0]  = penColor[0];
  penColorDark[1]  = penColor[1];
  penColorDark[2]  = penColor[2];

  for (int i = 0; i < 3; i++) {
    if  (penColorLight[i]  + gradientDegree > 1) penColorLight[i] = 1;
    else penColorLight[i] += gradientDegree;
    if  (penColorDark[i]   - gradientDegree < 0) penColorDark[i]  = 0;
    else penColorDark[i]  -= gradientDegree;
  }
  fillColor[0] = myFillColor->red()   / 255.0;
  fillColor[1] = myFillColor->green() / 255.0;
  fillColor[2] = myFillColor->blue()  / 255.0;

  fillColorLight[0] = fillColor[0];
  fillColorLight[1] = fillColor[1];
  fillColorLight[2] = fillColor[2];

  fillColorDark[0]  = fillColor[0];
  fillColorDark[1]  = fillColor[1];
  fillColorDark[2]  = fillColor[2];

  for (int ii = 0; ii < 3; ii++) {
    if  (fillColorLight[ii]  + gradientDegree > 1) fillColorLight[ii] = 1;
    else fillColorLight[ii] += gradientDegree;
    if  (fillColorDark[ii]   - gradientDegree < 0) fillColorDark[ii]  = 0;
    else fillColorDark[ii]  -= gradientDegree;
  }

  // use my brush size for points and lines
  glPointSize(myBrushSize);
  glLineWidth(myBrushSize);

  switch(myActiveTool) {
    case none    :
         break;

    case pen     :

         drawToolPen( myBrushSize, penColor, minPointSize,
                      x1, y1, x2, y2 );
         break;

    case line      :

         drawToolLine( myBrushSize, penColorLight, penColorDark, minPointSize,
                       x1, y1, x2, y2 );
         break;

    case rectangle :

         drawToolRect( myBrushSize, penColor, penColorLight, penColorDark, minPointSize,
                       x1, y1, x2, y2  );
         break;

    case rectangleFilled :

         drawToolRectFilled( myBrushSize, minPointSize,
                             penColor,  penColorLight,  penColorDark, fillColor, fillColorLight, fillColorDark,
                             x1,  y1,  x2,  y2  );
         break;

    case circle :

         drawToolCircle( penColor, gradientDegree, x1, y1, x2, y2 );
         break;

    case circleFilled :

         drawToolCircleFilled( penColor, fillColor, gradientDegree, x1, y1, x2, y2  );
         break;

    case triangle :

         drawToolTri ( penColor, penColorLight, penColorDark,
                       x1, y1, x2, y2  );
         break;

    case triangleFilled :

         drawToolTriFilled( penColor,  penColorLight,  penColorDark,
                            fillColor, fillColorLight, fillColorDark,
                            x1, y1, x2, y2  );
         break;

  }
}

//Jah Object code to be implemented
void GLPaint::updateSliders() {   debug("GLPaint we are updating the sliders"); }
void GLPaint::clicked( QListViewItem* items ) { debug("we are in paint clicked"); }


void GLPaint::slotHistoryPrevious() { historyRetrieve(-1); }
void GLPaint::slotHistoryNext()     { historyRetrieve(+1); }

void GLPaint::slotInvert()       { invert(); }
void GLPaint::slotFade()         { fade(); }
void GLPaint::slotIntensify()    { intensify(); }
void GLPaint::slotBlur()         { convolute(blur); }
void GLPaint::slotSharpen()      { convolute(sharpen); }
void GLPaint::slotEdgeDetectX()  { convolute(edgeDetectX); }
void GLPaint::slotEdgeDetectY()  { convolute(edgeDetectY); }
void GLPaint::slotSobel()        { convolute(sobel); }
void GLPaint::slotLaplacian()    { convolute(laplacian); }
void GLPaint::slotLaplacian2()   { convolute(laplacian2); }
void GLPaint::slotLapOfGauss()   { convolute(lapOfGauss); }
void GLPaint::slotClear()        { clear(); }

void GLPaint::slotPen()          { activateTool(pen); }
void GLPaint::slotLine()         { activateTool(line); }
void GLPaint::slotRectangle()    { activateTool(rectangle); }
void GLPaint::slotRectangleFilled()  { activateTool(rectangleFilled); }
void GLPaint::slotCircle()       { activateTool(circle); }
void GLPaint::slotCircleFilled()  { activateTool(circleFilled); }
void GLPaint::slotTriangle()     { activateTool(triangle); }
void GLPaint::slotTriangleFilled()   { activateTool(triangleFilled); }


void GLPaint::slotPenColor()  {
	
  QColor pickColor = QColorDialog::getColor( penColor(), this );
  if ( pickColor.isValid() ) {
    setPenColor( pickColor );
    ControlFrame4B2->setPaletteBackgroundColor( pickColor ); repaint();

  }
}

void GLPaint::slotFillColor() {
  QColor pickColor = QColorDialog::getColor( fillColor(), this );
  if ( pickColor.isValid() ) {
    setFillColor( pickColor );
    ControlFrame4B3->setPaletteBackgroundColor( pickColor ); repaint();

  }
}

void GLPaint::slotBackgroundColor() {
  QColor pickColor = QColorDialog::getColor( backgroundColor(), this );
  if ( pickColor.isValid() ) {
    setBackgroundColor( pickColor );
    ControlFrame4B1->setPaletteBackgroundColor( pickColor ); repaint();

  }
}


void GLPaint::slotBrushSize(int value)  { setBrushSize(value); }
void GLPaint::slotGradientDegree(int value) {  setGradientDegree(value); }










