/*******************************************************************************
**
** The header file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef PAINT_H
#define PAINT_H

#include <qcheckbox.h>
#include <qtabwidget.h>

#include "glworld.h"
#include "paintdefs.h"

//#include "glcore.h"

class GLPaint :  public  GLWorld {
  Q_OBJECT

public:

GLPaint( GLCore* core, const char* name,
         QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
         int* globalclipnumber, const QGLWidget* shareWidget=0  )
        :  GLWorld(core,name,controller,globalclipnumber ), mousePressed( FALSE ) {
		buildInterface(controls);
		buildInterfaceMenus(leftcontroller,rightcontroller);
		initializeObjects();
	};

	GLPaint::~GLPaint() {  }


	void buildInterface( QHBox* qtinterface);
	void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

	void initializeObjects();

	void refreshGL();

	// History functions.
	void setHistoryWidgets(QWidget *wPrevious, QWidget *wNext);
	void historyStore();
	void historyRetrieve(int relativeLoc);


	// Image Manipulation functions.
	void fade();
	void intensify();
	void invert();
	void convolute(const convolutionType type);
	void clear();



	// Return the given integer value limited within the range 0 to 255.
	int limit0_255(const int & val) {
	if      (val < 0  ) return 0;
	else if (val > 255) return 255;
	else return val;
	}

	// Activate the given tool.
	void activateTool(CanvasTool toolNum) { myActiveTool = toolNum; }
	void makeCheckImage(void);

	// Accessor functions.
	QColor penColor()        { return *myPenColor; }
	QColor fillColor()       { return *myFillColor; }
	QColor backgroundColor() { return *myBackgroundColor; }
	int    brushSize()       { return myBrushSize; }
	int    gradientDegree()  { return myGradientDegree; }

	// Modifier functions.
	void setPenColor (QColor newColor)       { *myPenColor        = newColor; }
	void setFillColor(QColor newColor)       { *myFillColor       = newColor; }
	void setBackgroundColor(QColor newColor) { *myBackgroundColor = newColor; }
	void setBrushSize(int newSize)           { myBrushSize       = newSize;  }
	void setGradientDegree(int newVal)       { myGradientDegree  = newVal; }

	//the paint tools
	void drawToolPen( int myBrushSize,color3 penColor, GLint minPointSize,
				  int x1, int y1, int x2, int y2  );

	void drawToolLine( int myBrushSize,color3 penColorLight,color3 penColorDark, GLint minPointSize,
				   int x1, int y1, int x2, int y2  );

	void drawToolRect( int myBrushSize,
				   color3 penColor, color3 penColorLight,color3 penColorDark, GLint minPointSize,
				   int x1, int y1, int x2, int y2  );

	void drawToolRectFilled( int myBrushSize,  GLint minPointSize,
						 color3 penColor, color3 penColorLight,color3 penColorDark,
						 color3 fillColor, color3 fillColorLight,color3 fillColorDark,
						 int x1, int y1, int x2, int y2  );


	void drawToolCircle( color3 penColor, GLfloat gradientDegree, int x1, int y1, int x2, int y2  );

	void drawToolCircleFilled( color3 penColor, color3 fillColor, GLfloat gradientDegree, int x1, int y1, int x2, int y2  );

	void drawToolTri ( color3 penColor, color3 penColorLight,color3 penColorDark,
				   int x1, int y1, int x2, int y2  );

	void drawToolTriFilled( color3 penColor, color3 penColorLight,color3 penColorDark,
						color3 fillColor, color3 fillColorLight,color3 fillColorDark,
						int x1, int y1, int x2, int y2  );
protected:

	void   drawWithActiveTool();
	//void   initializeGL();
	void   paintGL();

	// Overloaded QT functions.
	virtual void mousePressEvent  ( QMouseEvent* event);
	virtual void mouseReleaseEvent( QMouseEvent* event);
	virtual void mouseMoveEvent   ( QMouseEvent* event);

	QWidget *wHistoryPrevious, *wHistoryNext;
	QImage myHistory[myHistorySize];                  // Used as a circular array.
	int    myHistoryStartLoc, myHistoryEndLoc, myHistoryLoc;

	QImage buffer, workingBuffer;
	QColor *myPenColor, *myFillColor, *myBackgroundColor;
	int    myBrushSize, myActiveTool, myGradientDegree, myFadeDegree;
	int    x1, y1, x2, y2;
	bool   mousePressed, openPic;

	QButtonGroup  *bgDrawingTools;

	QString       myProgramDirectory; // Directory where this program resides.
	QString       myWorkingPath;      // Path in which to look for files.
	QString       myAuthorText;

protected slots:

	void slotHistoryPrevious();
	void slotHistoryNext();

	// Image Manipulation Slots.
	void slotInvert();
	void slotFade();
	void slotIntensify();
	void slotBlur();
	void slotSharpen();
	void slotEdgeDetectX();
	void slotEdgeDetectY();
	void slotSobel();
	void slotLaplacian();
	void slotLaplacian2();
	void slotLapOfGauss();
	void slotClear();

	// Tool slots.
	void slotPen();
	void slotLine();
	void slotRectangle();
	void slotRectangleFilled();
	void slotCircle();
	void slotCircleFilled();
	void slotTriangle();
	void slotTriangleFilled();

	// Tool configuration slots.
	void slotPenColor();
	void slotFillColor();
	void slotBackgroundColor();
	void slotBrushSize(int value);
	void slotGradientDegree(int value);


public slots:

	void addLayer(void);
	void addClip(videoData clip);

	void clicked( QListViewItem* item );

	void changeParticle();
	void changeFontDraw();
	void changeObjectDraw();
	void keyCliplayer();
	void keyClipkey();
	void updateSliders();

	void   resizePaintGL (void);

signals:
	void    GrabClip(void);    //linked to main jahcontrol to talk to desktop
	void    GrabClipKey(void);    //linked to main jahcontrol to talk to desktop

private:

	///////////////////////////////////////////////////////////////////////
	//paint mouse tracking
    GLfloat mousex, mousey;

	///////////////////////////////////////////////////////////////////////
	//interface variables for the left and right world space menus
	///////////////////////////////////////////////////////////////////////
	
	QFrame			*glworldLeft;
	JahToolButton	*tooltranslate,	*toolscale,		*toolrotate;
	JahToolButton	*playerphoto;

	QFrame			*glworldRight;
	JahToolButton	*toolzoomin,	*toolzoomout;
	QSlider			*zoom;

	///////////////////////////////////////////////////////////////////////
	//animation module ui options
	///////////////////////////////////////////////////////////////////////

	//declare all controllers that we are using here
	enum tabUI {SCENE, OBJECT, AXIS, CLIP, EFFECTS, KEYFRAMES, numTabs};
  
	///////////////////////////////////////////////////////////////////////
	//now the main widget variables

	//the first, main frame
	QFrame*			LayersFrame;
	QFrame*			EffectsFrame;

	///////////////////////////////////////////////////////////////////////
	//the tab frame to hold the tabs

	QTabWidget		*tabframe;

	//widegts for each of the tabs
	QWidget			*moduleOptionsUI[numTabs];

	//each module option ui can have up to 10 embedded ui's
    QFrame			*ControlFrameUI[numTabs][10];

    ////////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////
	// variables for the different tabs

	///////////////////////////////////////////////////////////////////////
	//set up the scene controls

	QPushButton*	ResetButton;             
	QPushButton*	SaveButton;    
	QPushButton*	LoadButton;              
	QPushButton*	ClearButton;
	QPushButton*	PackageButton;

	//lcdstartframe and endframe are initialized in world

	QCheckBox*		FogSelect;
	QSlider			*HFogSlider,		*VFogSlider;
	InputLCD		*HFogSlider_lcd,	*VFogSlider_lcd;

	//////////////////////////////////////////////////////////////////////
	//set up the different object controls, multiple layers are embedded

	//the world
	QCheckBox		*SmoothSelect;            
	QCheckBox		*TextSelect;
	QCheckBox		*GridSelect;
	QPushButton		*ResetWorldButton;

	//the layer types
	QPushButton		*ObjectLayer;
	QPushButton		*ObjectCube;
	QPushButton		*ObjectCylinder;
	QPushButton		*ObjectSphere;
	QPushButton		*ObjectBezier;
	QPushButton		*ObjectMesh;
	QPushButton		*ResetLayerButton;

	//the font options
	QLabel			*ScaleText_5;        
	QSlider			*ScaleX_5;         
	InputLCD		*ScaleXlcd_5;
	QLineEdit		*textEd;
	QPushButton		*textFont;           
	QPushButton		*textColor;        
	QPushButton		*textDraw;

	//the particle options
	QPushButton		*loadimage_3;
	QPushButton		*loadimage_4;
	QPushButton		*particlestyle;

	//the object options obj's
	QPushButton		*loadobject_1,	*objectstyle;
	QPushButton		*ResetObjectButton1;

	//the object options 3dsmax
	QPushButton		*loadmaxobject_1;
	QPushButton		*ResetObjectButton2;

	///////////////////////////////////////////////////////////////////////
	//set up the axis control panel
	//uses the createAxisPanel() subroutine

	///////////////////////////////////////////////////////////////////////
	//set up the texture and clip control panel

	//clip options
	QCheckBox		*ClipSelect;
	QPushButton		*grabdesktopClip; 
	QPushButton		*keyimage_2;       
	QCheckBox		*KeySelect;
	QPushButton		*grabdesktopKey;
	QPushButton		*keyimage_3;
	QPushButton		*keyStatusBox;

	//video options
	QLabel			*VideoLabel;
	InputLCD		*FramesLcd;		
	InputLCD		*inFramesLcd;	
	InputLCD		*outFramesLcd;
	InputLCD		*KeyFramesLcd;	
	InputLCD		*inKeyFramesLcd;	
	InputLCD		*outKeyFramesLcd;
	QToolButton		*lockicon;

	QCheckBox		*extendHeadTail;
	QCheckBox		*toggleTheLoop;
	QCheckBox		*togglePing;

	///////////////////////////////////////////////////////////////////////
	//set up the effects control panel

	QCheckBox		*ForegroundSelect;    
	QCheckBox		*DepthSelect;
	QCheckBox		*BlurSelect;
	QSlider			*HBlurSlider, *VBlurSlider;
	InputLCD		*HBlurSlider_lcd, *VBlurSlider_lcd;

	QCheckBox		*NegativeSelect, *SwapRGBSelect, *MirrorSelect; 
	JahToolButton	*lighticon;
	QCheckBox		*light1, *reflect1, *shader1, *shader2, *smooth1, *mesh1;
	QLabel			*Transparency_1;     
	QSlider			*Trans_1;         
	InputLCD		*Translcd_1;

	///////////////////////////////////////////////////////////////////////
	// Paint Module Specific need to weed out depreciated variables
	// and clean up the wacky names...

	//bool           EffectsStatus;

	//paint brushes
	QPushButton*	paintbrush1;
	QPushButton*	paintbrush2;
	QPushButton*	paintclip1;
	QPushButton*	paintclip1grab;
	QPushButton*	paintclip2;
	QPushButton*	paintclip2grab;
	QPushButton*	paintcolor;

	QPushButton		*ControlFrame1B1, *ControlFrame1B2, *ControlFrame1B3, *ControlFrame1B4,
  			  		*ControlFrame1B5, *ControlFrame1B6, *ControlFrame1B7, *ControlFrame1B8,
  					*ControlFrame1B9, *ControlFrame1B10;

	QToolButton		*ControlFrame1B20, *ControlFrame1B21, *ControlFrame1B22, *ControlFrame1B23,
  					*ControlFrame1B24, *ControlFrame1B25, *ControlFrame1B26, *ControlFrame1B27;

	QSlider			*ControlFrame1S1,    *ControlFrame1S2;
	InputLCD	    *ControlFrame1S1Lcd, *ControlFrame1S2Lcd;
	QLabel			*ControlFrame1L1,    *ControlFrame1L2,    *ControlFrame1L3;

	QPushButton		*ControlFrame4B1,  *ControlFrame4B2,  *ControlFrame4B3;
	QToolButton		*ControlFrame4B4, *ControlFrame4B5,  *ControlFrame4B6,  *ControlFrame4B7,  *ControlFrame4B8;
  					
	QSlider			*ControlFrame4S1, *ControlFrame4S2;
	InputLCD		*ControlFrame4S1Lcd, *ControlFrame4S2Lcd;

	QLabel			*ControlFrame4L1, *ControlFrame4L2, *ControlFrame4L3;

	QPushButton		*ControlFrame5B1;
	QToolButton		*ControlFrame5B2,  *ControlFrame5B3,  *ControlFrame5B4;

	QToolButton		*ControlFrame7B1, *ControlFrame7B2, *ControlFrame7B3, *ControlFrame7B4,
  					*ControlFrame7B5, *ControlFrame7B6, *ControlFrame7B7, *ControlFrame7B8,
  					*ControlFrame7B9, *ControlFrame7B10;

	QPushButton		*effectInvert, *effectFade,  *effectIntensify, *effectBlur,     *effectSharpen,
					*effectEdgeX,  *effectEdgeY, *effectSobel,     *effectLapacian, *effectLapacian2, *effectGauss;


};



#endif // ANIMATION_H





