/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "paint.h"


void GLPaint::setHistoryWidgets(QWidget *wPrevious, QWidget *wNext) {
    wHistoryPrevious = wPrevious;
    wHistoryNext = wNext;
    wHistoryPrevious->setDisabled(true);
    wHistoryNext->setDisabled(true);
}

 /*
 | Store the current buffer into the next location in this canvas' history.
*/
void GLPaint::historyStore() {

  myHistoryLoc = (myHistoryLoc+1) % myHistorySize;  // To next history location. 
  myHistory[myHistoryLoc] = buffer;        // Store a copy of the active buffer.
  myHistoryEndLoc = myHistoryLoc;       // Set this as the end history location.

  ControlFrame1B9->setDisabled(false);   // Enable the history previous button.

   // If history end location is now one location behind history start location,
   // move history start location forward one location.
  if ( ((myHistoryEndLoc+1) % myHistorySize) == myHistoryStartLoc ) {
    myHistoryStartLoc = (myHistoryStartLoc+1) % myHistorySize;
    ControlFrame1B10->setDisabled(true);         // Disable the history next button.
  }

}

 /*
 | If the given location is within between the start and end history locations,
 | move this canvas' history location to the given location and copy the buffer
 | at that location into the active buffer.
*/
void GLPaint::historyRetrieve(int relativeLoc) {

   // Make sure the given location is within the history size.
  relativeLoc += myHistoryLoc;
  while ( relativeLoc < 0 ) relativeLoc += myHistorySize;
  relativeLoc %= myHistorySize;  

   // If necessary, translate new and end locations forward by length of 
   // history, so that the order will be: [ ... start ... new ... end ... ]
  int testLoc = relativeLoc;
  int endLoc  = myHistoryEndLoc;
  if ( endLoc < myHistoryStartLoc ) { 
    endLoc += myHistorySize;
    if ( testLoc < myHistoryStartLoc ) testLoc += myHistorySize;
  }

   // Make sure the given location is between the history start and end 
   // locations, but is not wrapping around.
  if ( myHistoryStartLoc <= testLoc && testLoc <= endLoc ) {
    myHistoryLoc = relativeLoc;
    buffer = myHistory[myHistoryLoc];
    openPic=true;
    updateGL();
  }

  // Set whether the history previous and next widgets are disabled.
  ControlFrame1B9->setDisabled( testLoc <= myHistoryStartLoc );
  ControlFrame1B10->setDisabled(     testLoc >= endLoc            );

}


















