/*******************************************************************************
**
** The source file for the Jahshaka paint interface module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "paint.h"
#include <qstylefactory.h>

void GLPaint::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) 
{
    //make sure platinum is loaded
    QStyle *platinum=QStyleFactory::create("platinum");

    //////////////////////////////////////////////////////////////
    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");

    tooltranslate = new JahToolButton( glworldLeft, "pen" );
    addJahPlayerButton( tooltranslate, 0, 205, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/paint/buttonPen.png", JahBasePath+"Pixmaps/paint/buttonPen.png" );

    toolrotate =   new JahToolButton( glworldLeft, "line" );
    addJahPlayerButton( toolrotate,   0, 233, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/paint/buttonLine.png", JahBasePath+"Pixmaps/paint/buttonLine.png" );

    toolscale =   new JahToolButton( glworldLeft, "erase" );
    addJahPlayerButton( toolscale,   0, 263, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/paint/buttonErase.png", JahBasePath+"Pixmaps/paint/buttonErase.png" );

    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );

    toolzoomin =   new JahToolButton( glworldRight, "zoominw" );
    addJahPlayerButton( toolzoomin,   0, 115, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomin.png", JahBasePath+"Pixmaps/desktop/zoomin.png" );

    zoom = new QSlider( glworldRight, "scrubslider" );
    if (platinum) zoom->setStyle( platinum );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( -10 );    zoom->setMaxValue( 10 );    zoom->setValue   ( 0 );

    toolzoomout =   new JahToolButton( glworldRight, "zoomout" );
    addJahPlayerButton( toolzoomout,   0, 355, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomout.png", JahBasePath+"Pixmaps/desktop/zoomout.png" );

	//connect tools to object and add object to display widget
    //note that zoom isnt hooked up yet...
    connect( tooltranslate,  SIGNAL(clicked()), this, SLOT(slotPen() )  );
    connect( toolrotate,      SIGNAL(clicked()), this, SLOT(slotLine() )  );
}

void GLPaint::buildInterface( QHBox* qtinterface ) 
{
	//set up translator
    JahTranslate& jt = JahTranslate::getInstance();
    JahPrefs& jprefs = JahPrefs::getInstance();

	QStyle *platinum=QStyleFactory::create("platinum");

	///////////////////////////////////////////////////////////
	//The layers interface
	LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
	LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
	LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 140
	LayersFrame->setMaximumSize( QSize( 60, 195 ) );

	///////////////////////////////////////////////////////////
	//The layer options
	//these are defined in the world object
	AddButton = new QPushButton( LayersFrame, "AddButton" );
	addButton( AddButton,  0, 5, 55, 25,  jt.tr("Add") );
	//connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

	NameButton = new QPushButton( LayersFrame, "NameButton" );
	addButton( NameButton, 0, 30, 55, 25, jt.tr("Name") );
	//connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

	namepopup = new FancyPopup( this, jt.tr("Enter Layer Name"),250,150 );  //send size and position as well
	//connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

	DelButton = new QPushButton( LayersFrame, "Delete Button" );
	addButton( DelButton, 0, 60, 55, 25 , jt.tr("Del") );
	//connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

	//////////////////////////////////////////////////////////////
	// set up the scene options, or prefernces
	
	EffectsFrame = new QFrame(qtinterface);
	EffectsFrame->setGeometry( QRect( 0, 0, 855, 195) );


    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

    connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

    lv->setGeometry( QRect( 2, 2, 146, 180 ) );
    lv->addColumn( "LAYERS          ", -1 );
	lv->setSorting( -1,1 );   // disables the autosort

	//allow for multiple selections
    lv->setSelectionMode (QListView::Extended );


	/////////////////////////////////////////////////////////
	//Set up tabbed interface

	tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );	
	tabframe->setGeometry( QRect( 148, 1, 700, 180 ) );
	
	tabframe->setTabShape( QTabWidget::Triangular );
	tabframe->setAutoMask( FALSE );
	if (platinum) tabframe->setStyle( platinum );

	/////////////////////////////////////////////////////////
	//Set up tabbed interface
	//this has become the settings panel need to update variable names
    moduleOptionsUI[0] = new QWidget( EffectsFrame, "encoderUI0" ); //scenecontrols
	tabframe->insertTab( moduleOptionsUI[0], tr("  "+jt.tr("SCENE")+"  ") );

	moduleOptionsUI[1] = new QWidget( EffectsFrame, "objectcontrols" ); //objectcontrols
	tabframe->insertTab( moduleOptionsUI[1], tr( "  "+jt.tr("TOOLS")+"  " ) );

	moduleOptionsUI[2] = new QWidget( EffectsFrame, "axiscontrols" ); //axiscontrols
	tabframe->insertTab( moduleOptionsUI[2], tr( "  "+jt.tr("COLOR")+"  " ) );

	moduleOptionsUI[3] = new QWidget( EffectsFrame, "texturecontrols" ); //texturecontrols
	tabframe->insertTab( moduleOptionsUI[3], tr( "    "+jt.tr("EFFECTS")+"    " ) );

	//moduleOptionsUI[4] = new QWidget( EffectsFrame, "effectscontrols" ); //effectscontrols
	//tabframe->insertTab( moduleOptionsUI[4], tr( "  "+jt.tr("CONTROLS")+"  " ) );

	//this has become the settings panel need to update variable names
    //moduleOptionsUI[5] = new QWidget( EffectsFrame, "keysettings" ); //keycontrols
    //tabframe->insertTab( moduleOptionsUI[5], tr( "  "+jt.tr("KEYFRAMES")+"  " ) );

	///////////////////////////////////////////////////////////////////
	// the object control tab bar
	// this is the object control panel
	QPalette p( QColor( 0, 0, 0 ) );
	p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //mover this into the layers...
    ControlFrameUI[0][0] = new QFrame( moduleOptionsUI[0], "ControlFrame3" ); //scenecontrols
    ControlFrameUI[0][0]->setGeometry( QRect( 0, 0, 660, 140 ) );

    LoadButton = new QPushButton( ControlFrameUI[0][0], "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , jt.tr("Load") );
    //connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( ControlFrameUI[0][0], "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, jt.tr("Save") );
    //connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( ControlFrameUI[0][0], "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, jt.tr("Reset All") );
    //connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( ControlFrameUI[0][0], "ClearButton" );
    addButton( ClearButton, 10, 115, 86, 25, jt.tr("Clear Scene") );
    //connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  ); 
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( slotClear())  ); 
    
    //need to move this into the module
    //this is mandatory for all modules...

    sframetext = new QLabel( ControlFrameUI[0][0], "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( jt.tr("STARTFRAME") ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( ControlFrameUI[0][0], "startframe",9 );
    addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
    lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
    //connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( ControlFrameUI[0][0], "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( jt.tr("ENDFRAME")  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

    lcdendframe = new InputLCD( ControlFrameUI[0][0], "endframe",9 );
    addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
    lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
    //connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );


    jahreslabel = new QLabel( ControlFrameUI[0][0], "languagelabel" );
    jahreslabel->setGeometry( QRect( 450, 75, 120, 21 ) );
    jahreslabel->setAlignment( int( QLabel::AlignCenter ) );
    jahreslabel->setText( jt.tr( "Resolution" ) );

    JahresCombo = new QComboBox( FALSE, ControlFrameUI[0][0], "JahresCombo" );
    JahresCombo->setGeometry( QRect( 450, 100, 170, 21 ) );
	hasResMenu = true;

	projectData thedata;
	thedata.buildComboBox(JahresCombo);
    int defaultRes = jprefs.getJahResolution();
	JahresCombo->setCurrentItem(defaultRes); //should be set to appdefault
    connect( JahresCombo,  SIGNAL(activated(int)), this, SLOT(setResolution(int))  );

	///////////////////////////////////////////////////////////
	//Tool control panel

	ControlFrameUI[1][0] = new QFrame( moduleOptionsUI[1], "ControlFrame1" );
	ControlFrameUI[1][0]->setGeometry( QRect( 0, 0, 660, 120 ) );    // was 510

	//standard tools are here
	ControlFrame1L1 = new QLabel( ControlFrameUI[1][0], "brushslider" );
	addLabel( ControlFrame1L1, 10, 1, 150, 10, "DRAW OBJECTS", 9);

	//these were toolbuttons not pushbuttons so we could see what was selected
	ControlFrame1B1	= new QPushButton(ControlFrameUI[1][0],"brush");
	addIconButton(ControlFrame1B1, 10, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonPen.png");
	connect( ControlFrame1B1,  SIGNAL(clicked()), this, SLOT(slotPen())  );

	ControlFrame1B2	= new QPushButton(ControlFrameUI[1][0],"line");
	addIconButton(ControlFrame1B2, 35, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonLine.png");
	connect( ControlFrame1B2,  SIGNAL(clicked()), this, SLOT(slotLine())  );

	ControlFrame1B3	= new QPushButton(ControlFrameUI[1][0],"square");
	addIconButton(ControlFrame1B3, 70, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonRectangle.png");
	connect( ControlFrame1B3,  SIGNAL(clicked()), this, SLOT(slotRectangle())  );

	ControlFrame1B4	= new QPushButton(ControlFrameUI[1][0],"square");
	addIconButton(ControlFrame1B4, 95, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonRectangleFilled.png");
	connect( ControlFrame1B4,  SIGNAL(clicked()), this, SLOT(slotRectangleFilled())  );

	ControlFrame1B5	= new QPushButton(ControlFrameUI[1][0],"triangle");
	addIconButton(ControlFrame1B5, 130, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonTriangle.png");
	connect( ControlFrame1B5,  SIGNAL(clicked()), this, SLOT(slotTriangle())  );

	ControlFrame1B6	= new QPushButton(ControlFrameUI[1][0],"triangle");
	addIconButton(ControlFrame1B6, 155, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonTriangleFilled.png");
    connect( ControlFrame1B6,  SIGNAL(clicked()), this, SLOT(slotTriangleFilled())  );

	ControlFrame1B7	= new QPushButton(ControlFrameUI[1][0],"circle");
	addIconButton(ControlFrame1B7, 190, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonCircle.png");
    connect( ControlFrame1B7,  SIGNAL(clicked()), this, SLOT(slotCircle())  );

	ControlFrame1B8	= new QPushButton(ControlFrameUI[1][0],"circle");
	addIconButton(ControlFrame1B8, 215, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonCircleFilled.png");
    connect( ControlFrame1B8,  SIGNAL(clicked()), this, SLOT(slotCircleFilled())  );

	// the undo and redo buttons are here
	ControlFrame1B9	= new QPushButton(ControlFrameUI[1][0],"undo");
	addIconButton(ControlFrame1B9, 400, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonHistoryPrevious.png");
	connect( ControlFrame1B9,  SIGNAL(clicked()), this, SLOT(slotHistoryPrevious())  );

	ControlFrame1B10	= new QPushButton(ControlFrameUI[1][0],"redo");
	addIconButton(ControlFrame1B10, 425, 20, 22, 22, JahBasePath+"Pixmaps/paint/buttonHistoryNext.png");
	connect( ControlFrame1B10,  SIGNAL(clicked()), this, SLOT(slotHistoryNext())  );

	// Brushes

	ControlFrame1L2 = new QLabel( ControlFrameUI[1][0], "brushslider" );
	addLabel( ControlFrame1L2, 10, 55, 150, 10, "BRUSHES", 9);

	//dynamic brush size slider
	ControlFrame1L3 = new QLabel( ControlFrameUI[1][0], "brushslider" );
	addLabel( ControlFrame1L3, 400, 70, 150, 22, "BRUSH SIZE", 9);

	ControlFrame1S1 = new QSlider( ControlFrameUI[1][0] );   ControlFrame1S1Lcd = new InputLCD( ControlFrameUI[1][0] );
	addSliderAndLcd(  ControlFrame1S1, 400, 90, 150, 21,  1, 50, ControlFrame1S1Lcd );
	ControlFrame1S1->setTickInterval(2); ControlFrame1S1->setTickmarks(QSlider::Below);
	connect( ControlFrame1S1, SIGNAL(valueChanged(int)), this, SLOT(slotBrushSize(int)) );
	connect( ControlFrame1S1, SIGNAL(valueChanged(int)), ControlFrame1S1Lcd, SLOT(display(int)) );
	connect(ControlFrame1S1Lcd, SIGNAL(valueChanged(int)), this, SLOT(slotBrushSize(int)) );

	ControlFrame5B1	= new QPushButton(ControlFrameUI[1][0],"fillcolor");
	addButton(ControlFrame5B1, 10, 90, 45, 32, "Clear" );
	connect( ControlFrame5B1,  SIGNAL(clicked()), this, SLOT(slotClear())  );


	///////////////////////////////////////////////////////////
	//color control panel

    ControlFrameUI[2][0] = new QFrame( moduleOptionsUI[2], "ControlFrame4" );
    ControlFrameUI[2][0]->setGeometry( QRect( 0, 0, 660, 120 ) );    // was 510


    ControlFrame4L1 = new QLabel( ControlFrameUI[2][0], "gradientslider" );
    addLabel( ControlFrame4L1, 10, 10, 90, 22, "BACKGROUND", 9);

	ControlFrame4B1	= new QPushButton(ControlFrameUI[2][0],"backgroundcolor");
	addIconButton(ControlFrame4B1, 100, 10, 40, 22 );
	ControlFrame4B1->setPaletteBackgroundColor( QColor(5,38,10) );
	connect( ControlFrame4B1,  SIGNAL(clicked()), this, SLOT(slotBackgroundColor())  );

	ControlFrame4L2 = new QLabel( ControlFrameUI[2][0], "gradientslider" );
	addLabel( ControlFrame4L2, 10, 40, 90, 22, "PEN", 9);

	ControlFrame4B2	= new QPushButton(ControlFrameUI[2][0],"pencolor");
	addIconButton(ControlFrame4B2, 100, 40, 40, 22 );
	ControlFrame4B2->setPaletteBackgroundColor( QColor(0,58,140) );
	connect( ControlFrame4B2,  SIGNAL(clicked()), this, SLOT(slotPenColor())  );

	ControlFrame4L3 = new QLabel( ControlFrameUI[2][0], "gradientslider" );
	addLabel( ControlFrame4L3, 10, 70, 90, 22, "FILL", 9);

	ControlFrame4B3	= new QPushButton(ControlFrameUI[2][0],"fillcolor");
	addIconButton(ControlFrame4B3, 100, 70, 40, 22 );
	ControlFrame4B3->setPaletteBackgroundColor( QColor(0,97,140) );
	connect( ControlFrame4B3,  SIGNAL(clicked()), this, SLOT(slotFillColor())  );

	Transparency_1 = new QLabel( ControlFrameUI[2][0], "ScaleText_2" );
	addLabel( Transparency_1, 400, 5, 110, 21, "TRANSPARENCY", 9);
	Trans_1 = new QSlider( ControlFrameUI[2][0] );  Translcd_1 = new InputLCD( ControlFrameUI[2][0] );
	addSliderLcdLabel( ControlFrameUI[2][0], Trans_1,   400, 25, 10, 21,  1, 100,    Translcd_1,  "T", 9);
	connect( Trans_1,    SIGNAL(valueChanged(int)), Translcd_1, SLOT(display(int)) );
	connect( Trans_1,    SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );
	connect( Translcd_1, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );


	//dynamic brush size slider change to a dial...
	ControlFrame4L2 = new QLabel( ControlFrameUI[2][0], "gradientslider" );
	addLabel( ControlFrame4L2, 400, 70, 150, 22, "GRADIENT ANGLE", 9);

	ControlFrame4S2 = new QSlider( ControlFrameUI[2][0] );   ControlFrame4S2Lcd = new InputLCD( ControlFrameUI[2][0] );
	addSliderAndLcd(  ControlFrame4S2, 400, 90, 150, 21,  -255, 255, ControlFrame4S2Lcd );
	ControlFrame4S2->setTickInterval(25); ControlFrame4S2->setTickmarks(QSlider::Below);
	connect( ControlFrame4S2, SIGNAL(valueChanged(int)), this, SLOT(slotGradientDegree(int)) );
	connect( ControlFrame4S2, SIGNAL(valueChanged(int)), ControlFrame4S2Lcd, SLOT(display(int)) );
	connect(ControlFrame4S2Lcd, SIGNAL(valueChanged(int)), this, SLOT(slotGradientDegree(int)) );

	/////////////////////////////////////////////////////////////////////////
	//

	//add the layer reset button here as well
	ControlFrameUI[3][0] = new QFrame( moduleOptionsUI[3], "ControlFrame4" );
	ControlFrameUI[3][0]->setGeometry( QRect( 0, 0, 660, 180 ) );

	//column one
	effectInvert = new QPushButton( ControlFrameUI[3][0], "invert" );
	addButton( effectInvert, 10, 10, 100, 21, "Invert" );
	connect( effectInvert,  SIGNAL(clicked()), this, SLOT(slotInvert())  );

	effectBlur = new QPushButton( ControlFrameUI[3][0], "blur" );
	addButton( effectBlur, 10, 40, 100, 21, "Blur" );
	connect( effectBlur,  SIGNAL(clicked()), this, SLOT(slotBlur())  );

	effectSharpen = new QPushButton( ControlFrameUI[3][0], "sharpen" );
	addButton( effectSharpen, 10, 70, 100, 21, "Sharpen" );
	connect( effectSharpen,  SIGNAL(clicked()), this, SLOT(slotSharpen())  );

	//column 2
	effectFade = new QPushButton( ControlFrameUI[3][0], "fade" );         //these need a slider as well
	addButton( effectFade, 150, 10, 100, 21, "Fade" );
	//connect( effectFade,  SIGNAL(clicked()), this, SLOT(fade())  );

	effectIntensify = new QPushButton( ControlFrameUI[3][0], "intensify" );     //these need a slider as well
	addButton( effectIntensify, 150, 40, 100, 21, "Intensify" );
	//connect( effectIntensify,  SIGNAL(clicked()), this, SLOT(intensify())  );

	//add slider here

	//column three
	effectEdgeX = new QPushButton( ControlFrameUI[3][0], "edgex" );
	addButton( effectEdgeX, 280, 10, 100, 21, "Edge detectX" );
	connect( effectEdgeX,  SIGNAL(clicked()), this, SLOT(slotEdgeDetectX())  );

	effectEdgeY = new QPushButton( ControlFrameUI[3][0], "edgey" );
	addButton( effectEdgeY, 280, 40, 100, 21, "Edge detectY" );
	connect( effectEdgeY,  SIGNAL(clicked()), this, SLOT(slotEdgeDetectY())  );

	//column four
	effectSobel = new QPushButton( ControlFrameUI[3][0], "sobel" );
	addButton( effectSobel, 500, 10, 100, 21, "Sobel" );
	connect( effectSobel,  SIGNAL(clicked()), this, SLOT(slotSobel())  );

	effectLapacian = new QPushButton( ControlFrameUI[3][0], "lapacian" );
	addButton( effectLapacian, 500, 40, 100, 21, "Lapacian" );
	connect( effectLapacian,  SIGNAL(clicked()), this, SLOT(slotLaplacian())  );

	effectLapacian2 = new QPushButton( ControlFrameUI[3][0], "lapacian2" );
	addButton( effectLapacian2, 500, 70, 100, 21, "Lapacian2" );
	connect( effectLapacian2,  SIGNAL(clicked()), this, SLOT(slotLaplacian2())  );

	effectGauss = new QPushButton( ControlFrameUI[3][0], "gaussian" );
	addButton( effectGauss, 500, 70, 100, 21, "Gaussian" );
	connect( effectGauss,  SIGNAL(clicked()), this, SLOT(slotLapOfGauss())  );

    ///////////////////////////////////////////////////////////////
    // texture layer options need to add video-texture options here
/*
    ControlFrameUI[3][0] = new QFrame( moduleOptionsUI[3], "ControlFrame5" );
    ControlFrameUI[3][0]->setGeometry( QRect( 0, 0, 660, 120 ) );

    VideoLabel = new QLabel( ControlFrameUI[3][0], "TranslateText_1" );
    addLabel( VideoLabel, 380, 10, 110, 21, "VIDEO OPTIONS", 9);

    FramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLabelLcd(  ControlFrameUI[3][0], FramesLcd,       380, 30, 60, 21,  0,  "Frames", 8);

    inFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel(  ControlFrameUI[3][0], inFramesLcd,     460, 30, 30, 21,  0,  "In", 8);
	connect(inFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinFrames(int)) );

	outFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel(  ControlFrameUI[3][0], outFramesLcd,    530, 30, 30, 21,  0,  "Out", 8);
	connect(outFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutFrames(int)) );

    KeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLabelLcd(  ControlFrameUI[3][0], KeyFramesLcd,    380, 75, 60, 21,  0,  "Frames", 8);

    inKeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel(  ControlFrameUI[3][0], inKeyFramesLcd,  460, 75, 30, 21,  0,  "In", 8);
	connect(inKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinKeyFrames(int)) );

	outKeyFramesLcd = new InputLCD( ControlFrameUI[3][0] );
    addLcdLabel(  ControlFrameUI[3][0], outKeyFramesLcd, 530, 75, 30, 21,  0,  "Out", 8);
	connect(outKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutKeyFrames(int)) );

    //need to make the icon a lock... 3x current height as well for links
  	lockicon	= new QToolButton(ControlFrameUI[3][0],"lockicon");
    lockicon->setGeometry( QRect( 600, 55, 18, 26 ) );
  	lockicon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lockicon->setAutoRaise(true);

*/
}













