/*******************************************************************************
**
** The source file for the Jahshaka animation selections module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/



#include "paint.h"

 /*
 | Apply the given convolution matrix to the pixel at the given location, and
 | set the rgb color to the color for the pixel at that location.
*/
void GLPaint::convolute(const convolutionType type) {
  color3f255 rgb;
  unsigned int pix;

  buffer = core->grabFrameBuffer(true);
  workingBuffer = buffer;

  for (int x=1; x<(buffer.width()-2); x++)
    for (int y=1; y<(buffer.height()-2); y++) {

      rgb[0] = rgb[1] = rgb[2] = 0.0;
      for( int row = -1;  row <= 1;  row++ ) {
        for( int col = -1;  col <= 1;  col++ ) {
          pix = buffer.pixel( x+row, y+col );
          rgb[0] += (double)qRed(pix)   * convolutionMatrix[type][col+1][row+1];
          rgb[1] += (double)qGreen(pix) * convolutionMatrix[type][col+1][row+1];
          rgb[2] += (double)qBlue(pix)  * convolutionMatrix[type][col+1][row+1];
        }
      }

      workingBuffer.setPixel( x, y,
       qRgb(limit0_255((int)rgb[0]),
            limit0_255((int)rgb[1]),
            limit0_255((int)rgb[2])) );

    }

  buffer = workingBuffer;
  openPic=true;
  historyStore();
  updateGL();
}


 /*
 | Invert the colors in the image.
*/
void GLPaint::invert() {
  buffer.invertPixels();
  openPic=true;
  updateGL();
}


 /*
 | Fade the colors in the image towards white.
 | Approximately the opposite of intensify().
*/
void GLPaint::fade() {
  unsigned int pix;

  buffer = core->grabFrameBuffer(true);

  for (int y=0; y<buffer.height(); y++)
    for (int x=0; x<buffer.width(); x++) {
      pix = buffer.pixel(x,y);
      buffer.setPixel(x, y,
       qRgb(limit0_255( qRed(pix)/2   + myFadeDegree ),
            limit0_255( qGreen(pix)/2 + myFadeDegree ),
            limit0_255( qBlue(pix)/2  + myFadeDegree )) );
    }

  openPic=true;
  historyStore();
  updateGL();
}


 /*
 | Intensify the colors in the image towards black.
 | Approximately the opposite of fade().
*/
void GLPaint::intensify() {
  unsigned int pix;

  buffer = core->grabFrameBuffer(true);

  for (int y=0; y<buffer.height(); y++)
    for (int x=0; x<buffer.width(); x++) {
      pix = buffer.pixel(x,y);
      buffer.setPixel(x, y,
       qRgb(limit0_255( (qRed(pix)   - myFadeDegree) * 2),
            limit0_255( (qGreen(pix) - myFadeDegree) * 2),
            limit0_255( (qBlue(pix)  - myFadeDegree) * 2)) );
    }

  openPic=true;
  historyStore();
  updateGL();
}


void GLPaint::drawToolPen( int myBrushSize,color3 penColor, GLint minPointSize,
                           int x1, int y1, int x2, int y2  ) {

     // A point on point1 and point2, with a line between, without any gradient.
      glPointSize(myBrushSize*0.5);
      glColor3fv(penColor);
      if (myBrushSize > minPointSize) {
        glBegin(GL_POINTS);
          glVertex2f(x1,y1); glVertex2f(x2,y2);
        glEnd();
      }
      glBegin(GL_LINES);
        glVertex2f(x1,y1);   glVertex2f(x2,y2);
      glEnd();
      glPointSize(myBrushSize);

}

void GLPaint::drawToolLine( int myBrushSize,color3 penColorLight,color3 penColorDark, GLint minPointSize,
                            int x1, int y1, int x2, int y2  ) {

     // A point on point1 and point2, with a line between.
      glPointSize(myBrushSize*0.6);
      if (myBrushSize > minPointSize) {
        glBegin(GL_POINTS);
          glColor3fv(penColorLight);
          glVertex2f(x1,y1);
        glEnd();
      }
      glBegin(GL_LINES);
        glVertex2f(x1,y1);
        glColor3fv(penColorDark);
        glVertex2f(x2,y2);
      glEnd();
      if (myBrushSize > minPointSize) {
        glBegin(GL_POINTS);
          glVertex2f(x2,y2);
        glEnd();
      }
      glPointSize(myBrushSize);
}

void GLPaint::drawToolRect( int myBrushSize,
                            color3 penColor, color3 penColorLight,color3 penColorDark, GLint minPointSize,
                            int x1, int y1, int x2, int y2  ) {

      glDisable(GL_POINT_SMOOTH);       //use square points
      if (myBrushSize > minPointSize) {
        glBegin(GL_POINTS);
          glColor3fv(penColorLight);  glVertex2f(x1,y1);
          glColor3fv(penColor);       glVertex2f(x2,y1);
          glColor3fv(penColorDark);   glVertex2f(x2,y2);
          glColor3fv(penColor);       glVertex2f(x1,y2);
        glEnd();
      }
      glBegin(GL_LINE_STRIP);
        glColor3fv(penColorLight); glVertex2f(x1,y1);
        glColor3fv(penColor);      glVertex2f(x2,y1);
        glColor3fv(penColorDark);  glVertex2f(x2,y2);
        glColor3fv(penColor);      glVertex2f(x1,y2);
        glColor3fv(penColorLight); glVertex2f(x1,y1);
      glEnd();
      glEnable(GL_POINT_SMOOTH);       //use smooth, rounded points

}

void GLPaint::drawToolRectFilled( int myBrushSize,  GLint minPointSize,
                                 color3 penColor, color3 penColorLight,color3 penColorDark,
                                 color3 fillColor, color3 fillColorLight,color3 fillColorDark,
                                 int x1, int y1, int x2, int y2  ) {

      glDisable(GL_POINT_SMOOTH);       //use square points
      glBegin(GL_POLYGON);
        glColor3fv(fillColorDark);  glVertex2f(x1,y1);
        glColor3fv(fillColor);      glVertex2f(x2,y1);
        glColor3fv(fillColorLight); glVertex2f(x2,y2);
        glColor3fv(fillColor);      glVertex2f(x1,y2);
      glEnd();
      glColor3fv(penColor);
      if (myBrushSize > minPointSize) {
        glBegin(GL_POINTS);
          glColor3fv(penColorLight); glVertex2f(x1,y1);
          glColor3fv(penColor);      glVertex2f(x2,y1);
          glColor3fv(penColorDark);  glVertex2f(x2,y2);
          glColor3fv(penColor);      glVertex2f(x1,y2);
        glEnd();
      }
      glBegin(GL_LINE_STRIP);
        glColor3fv(penColorLight);   glVertex2f(x1,y1);
        glColor3fv(penColor);        glVertex2f(x2,y1);
        glColor3fv(penColorDark);    glVertex2f(x2,y2);
        glColor3fv(penColor);        glVertex2f(x1,y2);
        glColor3fv(penColorLight);   glVertex2f(x1,y1);
      glEnd();
      glEnable(GL_POINT_SMOOTH);       //use smooth, rounded points

}



void GLPaint::drawToolCircle(  color3 penColor, GLfloat gradientDegree,
                               int x1, int y1, int x2, int y2  ) {

      GLfloat angle,drawingAngle,xLen,yLen,radius;

 // A circle with its centre on point1, and its radius out to point2
      xLen = fabs(fabs((double)x1)-fabs((double)x2));
      yLen = fabs(fabs((double)y1)-fabs((double)y2));

      radius = sqrt( xLen*xLen + yLen*yLen );
      if (xLen == 0) xLen = 0.0001;          // avoid dividing by zero
      drawingAngle = atan ( yLen / xLen );  // angle from point1 to point2
      if ( y2 > y1 ) {
        if ( x2 < x1 ) drawingAngle  = PI - drawingAngle;    //flip   into quad2
      } else {
        if ( x2 > x1 ) drawingAngle  = twoPI - drawingAngle; //flip   into quad4
        else           drawingAngle += PI;                   //rotate into quad3
      }
      glBegin(GL_LINE_STRIP);
        for ( angle  = 0; angle  < twoPI + PI / toolCirclePointsPerPI;
              angle += PI / toolCirclePointsPerPI ) {
          glColor3f( penColor[0] - gradientDegree * cos(angle),
                     penColor[1] - gradientDegree * cos(angle),
                     penColor[2] - gradientDegree * cos(angle) );
          glVertex2f( x1 + radius * cos(angle+drawingAngle),
                      y1 + radius * sin(angle+drawingAngle));
        }
      glEnd();


}

void GLPaint::drawToolCircleFilled(  color3 penColor, color3 fillColor, GLfloat gradientDegree,
                                 int x1, int y1, int x2, int y2  ) {

      GLfloat angle,drawingAngle,xLen,yLen,radius;

     // A filled circle with its centre on point1, and its radius out to point2,
     // and a pen circle with its centre on point1 and its radius out to point2.
      xLen = fabs(fabs((double)x1)-fabs((double)x2));
      yLen = fabs(fabs((double)y1)-fabs((double)y2));
      radius = sqrt( xLen*xLen + yLen*yLen );
      if (xLen == 0) xLen = 0.0001;          // avoid dividing by zero
      drawingAngle = atan ( yLen / xLen );  // angle from point1 to point2
      if ( y2 > y1 ) {
        if ( x2 < x1 ) drawingAngle  = PI - drawingAngle;    //flip   into quad2
      } else {
        if ( x2 > x1 ) drawingAngle  = twoPI - drawingAngle; //flip   into quad4
        else           drawingAngle += PI;                   //rotate into quad3
      }
      glBegin(GL_POLYGON);
        for ( angle  = 0; angle  < twoPI + PI / toolCirclePointsPerPI;
              angle += PI / toolCirclePointsPerPI ) {
          glColor3f( fillColor[0] + gradientDegree * cos(angle),
                     fillColor[1] + gradientDegree * cos(angle),
                     fillColor[2] + gradientDegree * cos(angle) );
          glVertex2f( x1 + radius * cos(angle+drawingAngle),
                      y1 + radius * sin(angle+drawingAngle));
        }
      glEnd();
      glBegin(GL_LINE_STRIP);
        for ( angle  = 0; angle  < twoPI + PI / toolCirclePointsPerPI;
              angle += PI / toolCirclePointsPerPI ) {
          glColor3f( penColor[0] - gradientDegree * cos(angle),
                     penColor[1] - gradientDegree * cos(angle),
                     penColor[2] - gradientDegree * cos(angle) );
          glVertex2f( x1 + radius * cos(angle+drawingAngle),
                      y1 + radius * sin(angle+drawingAngle));
        }
      glEnd();


}

void GLPaint::drawToolTri ( color3 penColor, color3 penColorLight,color3 penColorDark,
                            int x1, int y1, int x2, int y2  ) {

      GLfloat angle,xLen,yLen,radius;

     // A pen triangle from point1 to point2, attempting to be equilateral,
     // with a point on each of its corners.
      xLen   = fabs(fabs((double)x2)-fabs((double)x1));
      yLen   = fabs(fabs((double)y1)-fabs((double)y2));
      radius = sqrt( xLen*xLen + yLen*yLen ) * atanPI;
      if (radius != 0) {                  // be careful not to divide by zero
        angle  = asin( yLen / radius );
        xLen   = radius * sin(angle) * 0.77;
        yLen   = radius * cos(angle) * 0.77;
        glColor3fv(penColor);
        glBegin(GL_LINE_STRIP);
          glColor3fv(penColorLight);
          glVertex2f( x2, y2 );
          glColor3fv(penColorDark);
          if        (y2<=y1 && x2>=x1) {
            glVertex2f( x1-xLen, y1-yLen ); glVertex2f( x1+xLen, y1+yLen );
          } else if (y2<=y1 && x2< x1) {
            glVertex2f( x1-xLen, y1+yLen ); glVertex2f( x1+xLen, y1-yLen );
          } else if (y2> y1 && x2>=x1) {
            glVertex2f( x1+xLen, y1-yLen ); glVertex2f( x1-xLen, y1+yLen );
          } else                       {
            glVertex2f( x1+xLen, y1+yLen ); glVertex2f( x1-xLen, y1-yLen );
          }
          glColor3fv(penColorLight);
          glVertex2f( x2, y2 );
        glEnd();
      }


}

void GLPaint::drawToolTriFilled( color3 penColor, color3 penColorLight,color3 penColorDark,
                                 color3 fillColor, color3 fillColorLight,color3 fillColorDark,
                                 int x1, int y1, int x2, int y2  ) {

      GLfloat angle,xLen,yLen,radius;


     // A filled triangle from point1 to point2, attempting to be equilateral,
     // a pen triangle from point1 to point2, attempting to be equilateral,
     // and a point on each of the corners.
      xLen   = fabs(fabs((double)x2)-fabs((double)x1));
      yLen   = fabs(fabs((double)y1)-fabs((double)y2));
      radius = sqrt( xLen*xLen + yLen*yLen ) * atanPI;
      if (radius != 0) {                  // be careful not to divide by zero
        angle  = asin( yLen / radius );
        xLen   = radius * sin(angle) * 0.77;
        yLen   = radius * cos(angle) * 0.77;
        glColor3fv(fillColor);
        glBegin(GL_POLYGON);
          glColor3fv(fillColorLight);
          glVertex2f( x2, y2 );
          glColor3fv(fillColorDark);
          if        (y2<=y1 && x2>=x1) {        // quadrant 4
            glVertex2f( x1-xLen, y1-yLen ); glVertex2f( x1+xLen, y1+yLen );
          } else if (y2<=y1 && x2< x1) {        // quadrant 3
            glVertex2f( x1-xLen, y1+yLen ); glVertex2f( x1+xLen, y1-yLen );
          } else if (y2> y1 && x2>=x1) {        // quadrant 1
            glVertex2f( x1+xLen, y1-yLen ); glVertex2f( x1-xLen, y1+yLen );
          } else                       {        // quadrant 2
            glVertex2f( x1+xLen, y1+yLen ); glVertex2f( x1-xLen, y1-yLen );
          }
        glEnd();
        glColor3fv(penColor);
        glBegin(GL_LINE_STRIP);
          glColor3fv(penColorDark);
          glVertex2f( x2, y2 );
          glColor3fv(penColorLight);
          if        (y2<=y1 && x2>=x1) {        // quadrant 4
            glVertex2f( x1-xLen, y1-yLen ); glVertex2f( x1+xLen, y1+yLen );
          } else if (y2<=y1 && x2< x1) {        // quadrant 3
            glVertex2f( x1-xLen, y1+yLen ); glVertex2f( x1+xLen, y1-yLen );
          } else if (y2> y1 && x2>=x1) {        // quadrant 1
            glVertex2f( x1+xLen, y1-yLen ); glVertex2f( x1-xLen, y1+yLen );
          } else                       {        // quadrant 2
            glVertex2f( x1+xLen, y1+yLen ); glVertex2f( x1-xLen, y1-yLen );
          }
          glColor3fv(penColorDark);
          glVertex2f( x2, y2 );
        glEnd();
      }

}


void GLPaint::addLayer(void)
{
   //add a new layer


}


void GLPaint::changeParticle() {

}

void GLPaint::changeFontDraw() {

   
}

void GLPaint::changeObjectDraw() {

    
}


void GLPaint::keyCliplayer() {

}

void GLPaint::keyClipkey() {

}

















