/*******************************************************************************
**
** The source file for the Jahshaka desktop module file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

////////////////////////////////////////////////////////////////
// This is the file that sets up all the individual modules
// and connects them to the desktop

#include "jahcreate.h"

////////////////////////////////////////////////////////////////
//Code to set up the Animation module


void JahControl::CreateDesktopModule(QHBox* frame,QHBox* frame2)
{
    if(dualheadprefs==true)
    {

        MainDesktopControler    = new QHBox(D2MainControler);
        D2MainControler->addWidget(MainDesktopControler,0);

        MainDesktopLeftControler  = new QHBox(D2MainLeftControler);
        D2MainLeftControler->addWidget(MainDesktopLeftControler,0); //animation

        MainDesktopRightControler  = new QHBox(D2MainRightControler);
        D2MainRightControler->addWidget(MainDesktopRightControler,0); //animation

        //module user controles
        optionals_desktop = new QVBox(D2optionals);
        D2optionals->addWidget(optionals_desktop,0);

        // Create Desktop Interface Preferences Here
        optionals_prefs   = new QVBox(D2optionals);
        D2optionals->addWidget(optionals_prefs,8);  

    }
    else
    {
        MainDesktopControler    = new QHBox(MainControler);
        MainControler->addWidget(MainDesktopControler,0);

        MainDesktopLeftControler  = new QHBox(MainLeftControler);
        MainLeftControler->addWidget(MainDesktopLeftControler,0); //animation

        MainDesktopRightControler  = new QHBox(MainRightControler);
        MainRightControler->addWidget(MainDesktopRightControler,0); //animation

        //module user controles
        optionals_desktop = new QVBox(optionals);
        optionals->addWidget(optionals_desktop,0);

        // Create Desktop Interface Preferences Here
        optionals_prefs   = new QVBox(optionals);
        optionals->addWidget(optionals_prefs,8);   

    }

    ////////////////////////////////////////////////////////////////////////
    // create the desktop module/object

    thedesktop = new JahDesktop( frame, frame2, MainDesktopLeftControler, 
                                MainDesktopRightControler,
                                "desktop", &globalclipnumber );

    ///////////////////////////////////////////////////////////////
    //now we set up the desktop interface
    //all other modules do this themselves ie effects,animation
    //but since desktop controls all of them its easier to do it here

    // setup the desktop middle frame first ie load, delete, play, etc
    CreateDesktopMiddleFrame(MainDesktopControler);

    //Set up Desktop Interface Options and buttons here
    desktopframe  = new QHBox(optionals_desktop);
    CreateDesktopOptions(desktopframe);

    //Set up Desktop Preferences Options and buttons here
    preferencesframe  = new QHBox(optionals_prefs);
    CreateDesktopPreferences(preferencesframe);

}

void JahControl::CreatePlayerModule(QFrame* frame)
{
    if (dualheadprefs==true)
    {
        MainPlayerControler  = new QHBox(D2MainControler);
        D2MainControler->addWidget(MainPlayerControler,1);

        MainPlayerLeftControler  = new QHBox(D2MainLeftControler);
        D2MainLeftControler->addWidget(MainPlayerLeftControler,1); //animation

        MainPlayerRightControler  = new QHBox(D2MainRightControler);
        D2MainRightControler->addWidget(MainPlayerRightControler,1); //animation

        optionals_player   = new QHBox(D2optionals);
        D2optionals->addWidget(optionals_player,1);

    }
    else
    {

        MainPlayerControler  = new QHBox(MainControler);
        MainControler->addWidget(MainPlayerControler,1);

        MainPlayerLeftControler  = new QHBox(MainLeftControler);
        MainLeftControler->addWidget(MainPlayerLeftControler,1); //animation

        MainPlayerRightControler  = new QHBox(MainRightControler);
        MainRightControler->addWidget(MainPlayerRightControler,1); //animation

        optionals_player   = new QHBox(optionals);
        optionals->addWidget(optionals_player,1);


    }

    ////////////////////////////////////////////////////////////////////////
    // create the player module/object

    vidplayer = new MainPlayer( frame, "playerlayout", optionals_player, MainPlayerControler,
                                MainPlayerLeftControler,MainPlayerRightControler );

    //////////////////////////////////////////////////////
    //display startup image.in video player.
    //gives trouble under linux... due to timer in player?
    displayVideo2startupImage();

}

void JahControl::CreateNetworkModule(QFrame* frame)
{

    //this module only uses the bottom half of the desktop for now...
    if (dualheadprefs==true)
    {
        MainNetworkControler  = new QHBox(D2MainControler);
        D2MainControler->addWidget(MainNetworkControler,2); //network

        optionals_network = new QHBox(D2optionals);
        D2optionals->addWidget(optionals_network,2);
    }
    else
    {
        MainNetworkControler  = new QHBox(MainControler);
        MainControler->addWidget(MainNetworkControler,2); //network

        optionals_network = new QHBox(optionals);
        optionals->addWidget(optionals_network,2);
    }

    ////////////////////////////////////////////////////////////////////////
    // create the network module/object

    jahnetwork = new JahNet( frame, theeffectsdisplay2, "networklayout", optionals_network,
                            MainNetworkControler, &globalclipnumber);

    connect ( jahnetwork,    SIGNAL(GrabClip()), SLOT( GrabClipNetwork()) );
    connect ( jahnetwork,    SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );

    connect ( jahnetwork,    SIGNAL(updateDesktopString(QString)), SLOT( addStringToDesktop(QString)) );

    //for ui switching
    connect ( jahnetwork,    SIGNAL(switchToNetP2P()), SLOT( changeModeNetworkP2P()) );
    connect ( jahnetwork,    SIGNAL(switchToNet()), SLOT( changeModeNetwork()) );

}

void JahControl::CreateAnimationModule() 
{

    MainAnimationControler  = new QHBox(MainControler);
    MainControler->addWidget(MainAnimationControler,3); //animation

    MainAnimationLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainAnimationLeftControler,3); //animation

    MainAnimationRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainAnimationRightControler,3); //animation

    //set up module control space
    optionals_animation = new QHBox(optionals);
    optionals->addWidget(optionals_animation,3);

    ////////////////////////////////////////////////////////////////////////
    // create the animation module/object

    animation = new GLAnime( core, "Animation" , optionals_animation,
                            MainAnimationControler,
                            MainAnimationLeftControler, MainAnimationRightControler,
                            &globalclipnumber  );

    //without this windows boxes loose tracking in the anime module (only?)
    animation->setMouseTracking( false );

    //connect module to desktop framework
    connect ( animation,   SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( animation,   SIGNAL(GrabClip()),    SLOT( GrabClipAnim()) );
    connect ( animation,   SIGNAL(GrabClipKey()), SLOT( GrabClipAnimKey()) );

    //added for video head
    connect ( animation,   SIGNAL(updatePlayer(QImage)), SLOT( displayClipInPlayer(QImage)) );


}

////////////////////////////////////////////////////////////////
//Code to set up the Effects module

void JahControl::CreateEffectsModule()
{

    MainEffectsControler  = new QHBox(MainControler);
    MainControler->addWidget(MainEffectsControler,4);

    MainEffectsLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainEffectsLeftControler,4); //animation

    MainEffectsRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainEffectsRightControler,4); //animation

    //set up module control space
    optionals_effects = new QHBox(optionals);
    optionals->addWidget(optionals_effects,4);

    ////////////////////////////////////////////////////////////////////////
    // create the effects module/object

    effects = new GLEffect( core, theeffectsdisplay2, "Effects" , optionals_effects,
                        MainEffectsControler,
                        MainEffectsLeftControler, MainEffectsRightControler,
                        &globalclipnumber  );
                        //&globalclipnumber, animation  );

    //connect module to desktop framework
    connect ( effects,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( effects,     SIGNAL(GrabClip()), SLOT( GrabClipEffects()) );

    //added for video head support
    connect ( effects,   SIGNAL(updatePlayer(QImage)), SLOT( displayClipInPlayer(QImage)) );

    //for node switching
    connect ( effects,   SIGNAL(switchview()), SLOT( switchViewEffects()) );

}



void JahControl::CreateEditingModule() 
{

    MainEditingControler  = new QHBox(MainControler);
    MainControler->addWidget(MainEditingControler,5); //animation

    MainEditingLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainEditingLeftControler,5); //animation

    MainEditingRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainEditingRightControler,5); //animation

    //set up module control space
    optionals_editing = new QHBox(optionals);
    optionals->addWidget(optionals_editing,5);

    ////////////////////////////////////////////////////////////////////////
    // create the editing module/object

    editing = new GLEdit( core, "Editing" , optionals_editing,
                            MainEditingControler,
                            MainEditingLeftControler, MainEditingRightControler,
                            &globalclipnumber  );
                            //&globalclipnumber, animation  );

    //connect module to desktop framework
    connect ( editing,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( editing,     SIGNAL(GrabClip()), SLOT( GrabClipEdit()) );       //not yet implemented
    connect ( editing,     SIGNAL(AddClip()),  SLOT( AddClipEdit()) );       //not yet implemented

    //added for video head support
    connect ( editing,   SIGNAL(updatePlayer(QImage)), SLOT( displayClipInPlayer(QImage)) );

}

void JahControl::CreatePaintModule() 
{

    MainPaintControler  = new QHBox(MainControler);
    MainControler->addWidget(MainPaintControler,6); //animation

    MainPaintLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainPaintLeftControler,6); //animation

    MainPaintRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainPaintRightControler,6); //animation

    //set up module control space
    optionals_paint = new QHBox(optionals);
    optionals->addWidget(optionals_paint,6);

    ////////////////////////////////////////////////////////////////////////
    // create the paint module/object

    paint = new GLPaint( paintcore, "Paint" , optionals_paint,
                            MainPaintControler,
                            MainPaintLeftControler, MainPaintRightControler,
                            &globalclipnumber  );

    //paint core has a fixed size
    paintcore->setFixedSize( QSize( 720, 486 ) );

    //connect module to desktop framework
    connect ( paint,  SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( paint,  SIGNAL(GrabClip()), SLOT( GrabClipPaint()) );

    //a hack here for the paint buffer updates
    connect ( paintcore,  SIGNAL(resizeit()), paint, SLOT( resizePaintGL()) );

}

void JahControl::CreateTextModule() 
{

    MainTextControler  = new QHBox(MainControler);
    MainControler->addWidget(MainTextControler,7); //animation

    MainTextLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainTextLeftControler,7); //animation

    MainTextRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainTextRightControler,7); //animation

    //set up module control space
    optionals_text = new QHBox(optionals);
    optionals->addWidget(optionals_text,7);

    ////////////////////////////////////////////////////////////////////////
    // create the text module/object

    text = new GLChar( core, "Text" , optionals_text,
                    MainTextControler,
                    MainTextLeftControler, MainTextRightControler,
                    &globalclipnumber  );

    //connect module to desktop framework
    connect ( text,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( text,     SIGNAL(GrabClip()), SLOT( GrabClipCG()) );

    //added for video head support
    connect ( text,   SIGNAL(updatePlayer(QImage)), SLOT( displayClipInPlayer(QImage)) );

}


void JahControl::CreateColorModule() 
{

    MainColorControler  = new QHBox(MainControler);
    MainControler->addWidget(MainColorControler,10); //animation

    MainColorLeftControler  = new QHBox(MainLeftControler);
    MainLeftControler->addWidget(MainColorLeftControler,10); //animation

    MainColorRightControler  = new QHBox(MainRightControler);
    MainRightControler->addWidget(MainColorRightControler,10); //animation

    //set up module control space
    optionals_color = new QHBox(optionals);
    optionals->addWidget(optionals_color,10);

    ////////////////////////////////////////////////////////////////////////
    // create the color module/object

    colorize = new GLColorize( core, "Color" , optionals_color,
                            MainColorControler,
                            MainColorLeftControler, MainColorRightControler,
                            &globalclipnumber  );

    //connect module to desktop framework
    connect ( colorize,    SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( colorize,    SIGNAL(GrabClip()), SLOT( GrabClipColorize()) );

    //added for video head support
    connect ( colorize,   SIGNAL(updatePlayer(QImage)), SLOT( displayClipInPlayer(QImage)) );

}


void JahControl::CreateEncodingModule(QFrame* frame)
{
    if (dualheadprefs==true)
    {
        MainEncodingControler  = new QHBox(D2MainControler);
        D2MainControler->addWidget(MainEncodingControler,11);

        MainEncodingLeftControler  = new QHBox(D2MainLeftControler);
        D2MainLeftControler->addWidget(MainEncodingLeftControler,11); //animation

        MainEncodingRightControler  = new QHBox(D2MainRightControler);
        D2MainRightControler->addWidget(MainEncodingRightControler,11); //animation

        optionals_encoding   = new QHBox(D2optionals);
        D2optionals->addWidget(optionals_encoding,11);

    }
    else
    {

        MainEncodingControler  = new QHBox(MainControler);
        MainControler->addWidget(MainEncodingControler,11);

        MainEncodingLeftControler  = new QHBox(MainLeftControler);
        MainLeftControler->addWidget(MainEncodingLeftControler,11); //animation

        MainEncodingRightControler  = new QHBox(MainRightControler);
        MainRightControler->addWidget(MainEncodingRightControler,11); //animation

        optionals_encoding   = new QHBox(optionals);
        optionals->addWidget(optionals_encoding,11);


    }

    ////////////////////////////////////////////////////////////////////////
    // create the player module/object

    jahencoder = new MainEncoder( frame, "encodinglayout", optionals_encoding, MainEncodingControler,
                                MainEncodingLeftControler,MainEncodingRightControler );


}

void JahControl::CreateLibraryModule(QHBox* frame)
{

    ////////////////////////////////////////////////////////////////////////
    // create the player module/object

    theLibrary = new FileMainFrame ( frame,"shareview");

	//this is where we set the share folder
    theLibrary->fileView()->setDirectory( JahMediaPath+"media/" );
    
	connect( theLibrary, SIGNAL(loadButton( )), this, SLOT(getClipFromLibrary( )) );

}

