/*******************************************************************************
**
** The header file for the Jahshaka Jah module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef JAH_H
#define JAH_H

#include <qstring.h>
#include <qdir.h>
#include <qsettings.h>
#include <qpopupmenu.h>
#include <qsizepolicy.h>

#include <stdlib.h>
#include <stdio.h>

#include "jahplayercreate.h"

#include "jahtracer.h"
#include "jahpreferences.h"

#include "glew.h"

//qt imaging plugins
#include "tga.h"
#include "rgb.h"


//  The Main Application Constructor
class Jah : public QWidget {

    Q_OBJECT

public:
  Jah( QWidget *parent=0, const char *name=0 );
  ~Jah();

  bool setBasePath(void);

  void checkJahExtensions(void);
  void enableJahExtensions(void);

  void exitError(void);

  void keyPressEvent( QKeyEvent *k );
  int  moderator;

  void setupMyCustomColors();

  //refresh video window...
  void updatev2vid(void);


protected:
  //initailizes the main interface
  void setupInterface( );

  //the main interface frame variables
  QGridLayout* jahshakaLayout;
  QHBoxLayout* moduleinterfaceLayout;
  QGridLayout* modulesLayout;

  QFrame* menucontroller;

  QFrame* mainleftmenu;
  QFrame* mainworld;
  QFrame* mainrightmenu;
  QFrame* maincontroller;
  QFrame* moduleinterface;
  QFrame* moduleselection;
  QFrame* modules;
  QFrame* feedbackbar;
  QFrame* modulesettings;
  QWidget * centralWidget;


public slots:

  void JahchangeModeText(const QString & text) { emit sendchangeModeText(text); };
  void JahsetProgressValue(int progress) { emit sendsetProgressValue(progress*10);  };
  void setStyleColor(QPalette pal);

  void storeGlobalPositionSize(void);
  void restoreGlobalPositionSize(void);

signals:

  void sendchangeModeText(const QString & text);
  void sendsetProgressValue(int progress);


private:
  void contextMenuEvent ( QContextMenuEvent * );

  //the startup screen
  QTimer     *timer;

  //this is the core jahshaka class
  JahControl *jahapp;

  //graphics card extension
  QString * vendor, * renderer, * version, * extensions;

  bool dualheadOption;

};


#endif // JAH_H






