/*******************************************************************************
**
** The source file for the Jahshaka jahcontrol module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

////////////////////////////////////////////////////////////////
// This is the main class that creates and launches the jahshaka interface
// and sets up and connects all the modules and interface objects

#include "jahplayercreate.h"

////////////////////////////////////////////////////////////////
// a reaaaaaly messy hack for dual head using a DEFINE vs a global variable
//will be cleaned out once we get it working
//since its also the easiest way :)


JahControl::JahControl( QWidget *parent,
                        QWidget *menucontroller,
                        QFrame  *mainleftmenu, QFrame *mainworld, QFrame *mainrightmenu,
                        QFrame  *maincontroller,
                        QFrame  *moduleselection, QFrame *modulesettings,QFrame *feedbackbar
                        )

{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    //set up translator
    JahTranslate& jt = JahTranslate::getInstance();

    //initialize JahBasePath & Mediapath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath   = jprefs.getBasePath();
    JahMediaPath  = jprefs.getMediaPath();
    JahDefaultRes = jprefs.getJahResolution();

    //this is a global variable used by all modules
    globalclipnumber = 0;

    //retreive preferences
    getGlobalPreferences();

    //setup menubar if needed
    //if (menuprefs)
    buildMenuBar(parent);

    //////////////////////////////////////////////////////////////
    //create the main frame object controllers
    //used to switch the framesets out according to
    //what module we are currently in
    jtrace.info( "Setting up Interface and loading modules" );

    //////////////////////////////////////////////////////////////
    //create the main controler for the world space object

//////////////////////////////////////////////////////////////
//create the main frame object controllers
//used to switch the framesets out according to
//what module we are currently in

//////////////////////////////////////////////////////////////
//create the main controler for the world space object
mainworldLayout = new QHBoxLayout( mainworld, 0, 0, "mainworldLayout");
WorldLayout = new QWidgetStack(mainworld);
mainworldLayout->addWidget( WorldLayout );

//////////////////////////////////////////////////////////////
//create the main left menu controler
mainworldLeftLayout = new QHBoxLayout( mainleftmenu, 0, 0, "mainworldLeftLayout");
MainLeftControler = new QWidgetStack(mainleftmenu);
mainworldLeftLayout->addWidget( MainLeftControler );

//////////////////////////////////////////////////////////////
//create the main right menu controler
mainworldRightLayout = new QHBoxLayout( mainrightmenu, 0, 0, "mainworldRightLayout");
MainRightControler = new QWidgetStack(mainrightmenu);
mainworldRightLayout->addWidget( MainRightControler );

//////////////////////////////////////////////////////////////
//create the middle frame menu/option controler
maincontrollerLayout = new QHBoxLayout( maincontroller, 0, 0, "maincontrollerLayout");
MainControler = new QWidgetStack(maincontroller);
maincontrollerLayout->addWidget( MainControler );

//////////////////////////////////////////////////////////////
//Create Interface frame and switching controllers for modules
optionals = new QWidgetStack(modulesettings);
optionals->setGeometry( QRect( 5, 1, 860, 200) );

    /////////////////////////////////////////////////////////////////
    //Create Desktop interface and buttons
    //we should be doing this in desktopinterface.cpp so it can create
    //its own interface like all the other modules...
    jtrace.info( ">>Loading Desktop Modules" );

    desktopdisplay = new QHBox (WorldLayout, "desktop");
    CreateDesktopModule(desktopdisplay);
    WorldLayout->addWidget(desktopdisplay,0);


    //connect the desktop preferences to parent object
    connect( this,  SIGNAL(emitStyleColor(QPalette)), parent, SLOT(setStyleColor(QPalette))  );
    connect( thedesktop,  SIGNAL(resetGlobalClipNumber()),  this, SLOT(resetGlobalClipNumber())  );

    //desktop scene file loader callback
    connect( thedesktop,  SIGNAL(loadJahSceneFile(QString)),  this, SLOT(loadJahSceneFile(QString))  );


    /////////////////////////////////////////////////////////////////
    //Now we Create and setup the desktop related modules
    jtrace.info( ">>Loading Main Modules" );

    /////////////////////////////////////////////////////////////////
    //we set up the player application here
    jtrace.info( "Loading Player Module" );

    playerdisplay = new QFrame (WorldLayout, "player");
    CreatePlayerModule(playerdisplay);
    WorldLayout->addWidget(playerdisplay,1);

    /////////////////////////////////////////////////////////////////
    //Now we can Create and setup the different jahshaka modules

    //we will break the main object into
    //2 widgets for user mode
    //and optional view modes ie nodeview in effects

    theCoreLayout = new QWidgetStack(mainworld);

    corebox = new QHBox(theCoreLayout,"glcore");

    theCoreLayout->addWidget(corebox,0);

    theeffectsdisplay2 = new QHBox (theCoreLayout, "effectsnode");
    theCoreLayout->addWidget(theeffectsdisplay2,1);


    //raise the gl interface as default
    theCoreLayout->raiseWidget(0);

    /////////////////////////////////////////////////////////////////
    // the main jahshaka 3d core

    //create core in widget 0
    core = new GLCore (corebox,"glcore");
    connect( core,  SIGNAL(draggedAImage()), SLOT(draggedAImage())  );

    ////////////////////////////////////////////////////////////////
    //we set up the animation module here
    jtrace.info( "Loading Animation Module" );

    emit creatingModule(jt.tr("Animation"));

    CreateAnimationModule();

    WorldLayout->addWidget(theCoreLayout,3);


    ///////////////////////////////////////////////////////////////////

    //finished loading modules
    jtrace.info( "Finished loading modules" );

    //configure modules for user
    configSystem();

}

void JahControl::configSystem()
{
    /////////////////////////////////////////////////////////////////
    //Raise and activate the default modules

    //set the main module to desktop
    module = DESKTOP;
    changeModeDesktop();
}


void JahControl::displayVideo2startupImage()
{
    //////////////////////////////////////////////////////
    //display startup image.in video player
    //but only if video player is active...
    if (videohead)
    {
        QImage image;
        QString ldfilename = JahBasePath + "/Pixmaps/jahstartupimage.png";

        //debug("Dualhead pic:"+ldfilename);

        QImageIO iio; iio.setFileName( ldfilename );

        if ( iio.read() )
        {
            image = iio.image();
            displayClipInPlayer(image);
        }
    }
}

void JahControl::resetGlobalClipNumber()
{
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahControl::resetGlobalClipNumber()","resetting global clipnumber" );

    globalclipnumber=0;
}


//this is to pass data down to the modules
void JahControl::loadJahSceneFile(QString filename)
{
    //debug(filename);
    animation->LoadAll(filename);
    changeModeAnimation();
}

//we detect keypresses in Jah above so we have to process them here
void JahControl::processKeyPress(QKeyEvent *k)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    switch ( tolower(k->ascii()) )
    {
    case 'd':  {	changeModeDesktop();
                    break; }

    case 'p':  {	changeModePlayer();
                    break; }

    case 'l':  {	thedesktop->slotLoadIconview();
                    break; }

    case 'a':  {     //animation->SceneLoad();
                    changeModeAnimation();
                    break; }

    case ' ':  {	//should only play if something was loaded...
                    static bool playmode = false;

                    if (module == GLPLAYER)
                        animation->startanimation();

                    if (module == PLAYER)
                    {
                        playmode = !playmode;

                        if (playmode)
                            vidplayer->LoopForward();
                        else
                            vidplayer->Stop();

                    }

                    if (module == DESKTOP)
                    {
                        changeModePlayer();
                    }
                    //debug("space");

                    break; }

        default :  jtrace.debug("pressed unmapped key"); break;
    }
}

////////////////////////////////////////////////////////
//Jahshaka Destructor
//cleans up as we leave and releases memory...
JahControl::~JahControl() 
{
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahControl Destructor","Closing out the core modules" );

    delete thedesktop;   jtrace.debug( ">> Closed thedesktop");
    delete animation;    jtrace.debug( ">> Closed animation");
    delete vidplayer;    jtrace.debug( ">> Closed vidplayer");

    jtrace.debug( "JahControl Destructor","Closed out the core modules" );


}


void JahControl::buildMenuBar(QWidget *  parent)
{

    /* XPM */
    static const char * p1_xpm[] = {
    "16 16 3 1",
    "       c None",
    ".      c #000000000000",
    "X      c #FFFFFFFF0000",
    "                ",
    "                ",
    "         ....   ",
    "        .XXXX.  ",
    " .............. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .XXXXXXXXXXXX. ",
    " .............. ",
    "                "};

    /* XPM */
    static const char * p2_xpm[] = {
    "16 16 3 1",
    "       c None",
    ".      c #000000000000",
    "X      c #FFFFFFFFFFFF",
    "                ",
    "   ......       ",
    "   .XXX.X.      ",
    "   .XXX.XX.     ",
    "   .XXX.XXX.    ",
    "   .XXX.....    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .XXXXXXX.    ",
    "   .........    ",
    "                ",
    "                "};

    /* XPM */
    static const char * p3_xpm[] = {
    "16 16 3 1",
    "       c None",
    ".      c #000000000000",
    "X      c #FFFFFFFFFFFF",
    "                ",
    "                ",
    "   .........    ",
    "  ...........   ",
    "  ........ ..   ",
    "  ...........   ",
    "  ...........   ",
    "  ...........   ",
    "  ...........   ",
    "  ...XXXXX...   ",
    "  ...XXXXX...   ",
    "  ...XXXXX...   ",
    "  ...XXXXX...   ",
    "   .........    ",
    "                ",
    "                "};

    //for the menus if they are activated
    //need to make this conditional
    //and need to add as a subroutine
    QPixmap p1( p1_xpm );
    QPixmap p2( p2_xpm );
    QPixmap p3( p3_xpm );

    //set up translator
    JahTranslate& jt = JahTranslate::getInstance();

    QMenuBar* menu = new QMenuBar ( parent );//menuBar();

    QPopupMenu* file = new QPopupMenu( menu );
    file->insertItem(p1,"&"+jt.tr("Load"), this, SLOT(slotLoadIconview()), CTRL+Key_L );
    //file->insertItem(p3,"&"+jt.tr("Save"), this, SLOT(slotSave()), CTRL+Key_S );
    file->insertSeparator();
    file->insertItem("&"+jt.tr("Import"), this, SLOT(slotLoadIconview()), CTRL+Key_I);
    file->insertItem("&"+jt.tr("Export"), this, SLOT(slotExport()), CTRL+Key_E);
    file->insertSeparator();
    file->insertItem(""+jt.tr("Preferences"), this, SLOT(changeModePref()), CTRL+Key_P);
    file->insertSeparator();
    file->insertItem("E&xit", qApp, SLOT(quit()), CTRL+Key_X);
    menu->insertItem("&"+jt.tr("File"), file);

    //QPopupMenu *viewmode = new QPopupMenu( parent );
    //Q_CHECK_PTR( viewmode );
    //viewmode->insertItem( jt.tr("Media"), this, SLOT(printer()) );
    //viewmode->insertItem( jt.tr("Table"), this, SLOT(file()) );

    QPopupMenu* desk = new QPopupMenu( menu );
    desk->insertItem("&"+jt.tr("Load clip"), this, SLOT(slotLoadIconview()) );
    //desk->insertItem("&"+jt.tr("Save clip"), this, SLOT(slotSave()) );
    desk->insertSeparator();
    desk->insertItem(jt.tr("Play clip"), this, SLOT(changeModePlayer()) );
    desk->insertSeparator();
    desk->insertItem(jt.tr("Delete clip"), this, SLOT(clearItem()) );
    //desk->insertSeparator();
    //desk->insertItem("&"+jt.tr("View mode"), viewmode );
    desk->insertSeparator();
    desk->insertItem("&"+jt.tr("Clear desktop"), this, SLOT(clearAll()) );
    //desk->insertItem("&Add desktop", this, SLOT(notImplemented()) );
    //desk->insertItem("&Switch desktop", this, SLOT(notImplemented()) );
    menu->insertItem("&"+jt.tr("Desktop"), desk);

    QPopupMenu* view = new QPopupMenu( menu );
    view->insertItem("&"+jt.tr("Desktop"), this, SLOT(changeModeDesktop()) );
    view->insertItem("&"+jt.tr("2D Player"), this, SLOT(changeModePlayer()) );
    view->insertItem("&"+jt.tr("3D Player"), this, SLOT(changeModeAnimation()) );
    menu->insertItem("&"+jt.tr("Mode"), view);

    //QPopupMenu* net = new QPopupMenu( menu );
    //net->insertItem("&"+jt.tr("Network"), this, SLOT(changeModeNetwork()) );
    //net->insertItem("&"+jt.tr("Broadcast"), this, SLOT(changeModeNetwork()) );
    //menu->insertItem("&"+jt.tr("JahNet"), net);

}


/////////////////////////////////////////////////////
//some slots for the drop down menus
//since 'thedesktop' doesnt seem to work for them

void JahControl::slotLoadIconview(void)
{
    thedesktop->slotLoadIconview();
}

void JahControl::slotExport(void)
{
    thedesktop->slotExport();
}

void JahControl::clearItem(void)
{
    thedesktop->clearItem();
}

void JahControl::clearAll(void)
{
    thedesktop->clearAll();
}

