/*******************************************************************************
**
** The header file for the Jahshaka Source jahcreate file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef JAHCONTROL_H
#define JAHCONTROL_H

#include <stdlib.h>
#include <qsettings.h>
#include <qsplitter.h>
#include <qlineedit.h>
#include <qstylefactory.h>
#include <qcursor.h>
#include <qmenubar.h>
#include <qcombobox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlabel.h>

#include "sqlite.h"

#include "interfaceheaders.h"

#include "desktop.h"
#include "MainPlayer.h"
#include "dialogs.h"
#include "glworld.h"
#include "glplayer.h"


#include "jahtracer.h"
#include "jahtranslate.h"

extern int xresolution;
extern int yresolution;

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QFrame;
class QPushButton;
class QTextView;


////////////////////////////////////////////////////////////////////
//these are the modules that are supported in this build of jahshaka
//but dont necessarily have to be here

enum Module { DESKTOP, PLAYER, NETWORK, ANIMATION, EFFECTS, EDITING, 
            TEXTCG, PAINT, COLORIZE, ENCODING, BROADCASTING, GLPLAYER };

//change interface look and feel
enum StyleTheme { JPLATINUM,JWINDOWS, JSGI, JCHARCOAL, JMETAL };
enum StyleColor { JGREY, JGREEN, JWHITE, JBLUE };

////////////////////////////////////////////////////////////////////
// This is the main Controling class that sets up all the modules

class JahControl : public QObject {
    Q_OBJECT

public:

JahControl( QWidget *parent=0,
            QWidget *menucontroller=0,
            QFrame  *mainleftmenu=0, QFrame *mainworld=0, QFrame *mainrightmenu=0,
            QFrame  *maincontroller=0,
            QFrame  *moduleselection=0, QFrame *modulesettings=0,QFrame *feedbackbar=0
            );


~JahControl();

    ////////////////////////////////////////
    // main module classes
    JahDesktop*     thedesktop;
    MainPlayer*     vidplayer;
    GLPlayer*        animation;

    ////////////////////////////////////////
    //OpenGL Core
    QHBox * corebox;
    GLCore * core;

    ///////////////////////////////////////
    //Paint Core
    QHBox * paintbox;
    GLCore * paintcore;

    ////////////////////////////////////////////////////////////////////
    // base routines

    //used to configure system on startup after loading all modules
    void configSystem();

    void buildMenuBar(QWidget *  parent);

    //ui feedback routines
    void processKeyPress(QKeyEvent *k);

    //routines to get style and app preferences
    bool getPrefs(void)      { return rememberprefs; }

    //used to track edit returns on singlehead system
    int globalclipnumber;

    bool editReturnClip;
    bool buildmenubars;
    int jahrenderquality;
    int renderer;

    void restoreStyle();
    void restoreColor(int color);


signals:
    void creatingModule(const QString &);
    void finishedModule(int);
    void emitStyleColor(QPalette);

public slots:

    void switchMode(void);

    void switchDesktop(void);
    void GetDesktop();
    void ReturnDesktop();
    void addImageToDesktop( videoData );
    void addStringToDesktop( QString );

    void GrabClipEffects();
    void GrabClipEdit();
    void AddClipEdit();
    void ReturnClipEdit();// used for grab on add...
    void GrabClipAnim();
    void GrabClipAnimKey();
    void GrabClipCG();
    void GrabClipColorize();
    void GrabClipPaint();
    void GrabClipNetwork();

    void printnameClip(QString item);
    void resetGlobalClipNumber();

    void displayVideo2startupImage(void);
    void switchViewEffects(void);

    void draggedAImage(void);

    //passthru slots for filemenu
    void slotLoadIconview(void);
    void slotExport(void);
    void clearItem(void);
    void clearAll(void);

    //load slot for the player
    void loadJahSceneFile(QString filename);

private slots:

    //change mode for desktop and subsections
    void changeModeDesktop();

    void changeModePlayer();
    void notifyPlayerOfChange();

    //void displayClipInPlayer(videoData thedata ); //added for siggraph demo
    void displayClipInPlayer(QImage thedata ); //added for siggraph demo

    void changeModeNetwork();
    void changeModeNetworkP2P();
    void changeModeEncoding();
    void changeModeVideoIO();
    void changeModeBroadcasting();
    void changeModePref();
    void changeModeColorize();

    //change mode for modules
    void changeModeEffects();
    void changeModeAnimation();
    void changeModeEditing();
    void changeModePaint();
    void changeModeText();

    //jahroutine routines
    void checkClearPlayer();
    void checkDesktopButton();
    void stopAnimation();
    void nameClip();

    void stylePlatinum();
    void styleWindows();
    void styleSgi();
    void styleCharcoal();
    void styleMetal();

    void styleGrey();
    void styleBlue();
    void styleGreen();
    void styleWhite();
    void styleFont(int size, QString font);

    //store preferences
    void setStoragePath();
    //Get the storage loaction fron user through a file Dialog
    void getStoragePath();

    void setGlobalRenderQuality(void);
    void getGlobalPreferences(void);
    void setModulesRenderQuality(void);

    void setGlobalPositionSize(void);
    void setVideoHeadOption(void);
    void setStyleOption(void);
    void setWidescreenOption(void);
    void setDualHeadOption(void);
    void setMenuOption(void);
    void setRenderer(int renval);
    void setLanguage(int renval);
    void setResolution(int renval);

protected:

    void CreateDesktopModule(QHBox* frame=0,QHBox* frame2=0);
    void CreateDesktopMiddleFrame(QHBox*);
    void CreateDesktopFeedbackBox(QFrame*);
    void CreateDesktopOptions(QHBox*);

    void CreateDesktopPreferences(QHBox*);
    void PreferenceslanguageChange();

    void CreateAnimationModule();
    void CreateEffectsModule();
    void CreateEditingModule();
    void CreatePaintModule();
    void CreateTextModule();

    void CreateColorModule();
    void CreatePlayerModule(QFrame* frame);
    void CreateEncodingModule(QFrame* frame);
    void CreateNetworkModule(QFrame* frame);

private:

    //////////////////////////////////////////
    // All variable Initializations here
    QString JahBasePath;
    QString JahMediaPath;
    int     JahDefaultRes;

    bool desktopButtonStatus;
    QFont          appFont;
    Module         module;

    //style prefs
    bool rememberprefs;
    bool videohead;
    bool dualhead;
    bool dualheadprefs;
    bool menuprefs;
    bool widescreen;
    bool widescreenprefs;
    bool jahStyleprefs;

    StyleTheme jahStyleTheme;
    StyleColor jahStyleColor;

    // Main Interface layout variables
    QHBoxLayout* mainworldLayout;       QWidgetStack*  WorldLayout;

    QHBoxLayout * mainworldLeftLayout;  QWidgetStack * MainLeftControler;
    QHBoxLayout * mainworldRightLayout; QWidgetStack * MainRightControler;

    //main window layout variables
    QHBoxLayout*   maincontrollerLayout;
    QWidgetStack*  MainControler;
    QWidgetStack*  optionals;

    // Dual head Main Interface layout variables
    QHBoxLayout* D2mainworldLayout;       QWidgetStack*  D2WorldLayout;

    QHBoxLayout * D2mainworldLeftLayout;  QWidgetStack * D2MainLeftControler;
    QHBoxLayout * D2mainworldRightLayout; QWidgetStack * D2MainRightControler;

    // Dual head main window layout variables
    QHBoxLayout*   D2maincontrollerLayout;
    QWidgetStack*  D2MainControler;
    QWidgetStack*  D2optionals;

    //the main desktop module
    //desktop controllers
    QHBox*         MainDesktopControler;
    QHBox*         MainDesktopLeftControler, *MainDesktopRightControler;
    QHBox*         desktopdisplay;
    QHBox*         desktopdisplay2;
    QHBox*         optionals_desktop;
    QPushButton*   Desktop;
    QPushButton*   DualDesktop;

    //mandatory modules in the desktop
    //network controllers
    QHBox*         networkdisplay;
    QHBox*         optionals_network;
    QHBox*         MainNetworkControler;
    QPushButton*   Network;
    QPushButton*   DualNetwork;

    //the main encoding module
    QPushButton*   Share;
    QPushButton*   Encoding;
    QPushButton*   DualEncoding;

    //encoding controllers
    QFrame*        encodingdisplay;
    QHBox*         MainEncodingLeftControler, *MainEncodingRightControler;
    QHBox*         optionals_encoding;
    QHBox*         MainEncodingControler;

    //the main broadcast module
    QPushButton*   Broadcasting;

    //player controllers
    QFrame*        playerdisplay;
    QHBox*         MainPlayerLeftControler, *MainPlayerRightControler;
    QHBox*         optionals_player;
    QHBox*         MainPlayerControler;

    //main modules located in the left option panel
    //animation controllers
    QHBox*         MainAnimationControler;
    QHBox*         MainAnimationLeftControler, *MainAnimationRightControler;
    QHBox*         animationdisplay;
    QHBox*         optionals_animation;
    QPushButton*   AnimationModule;

    //effects controllers
    QHBox*         MainEffectsControler;
    QHBox*         MainEffectsLeftControler, *MainEffectsRightControler;
    QHBox*         effectsdisplay;
    QHBox*         optionals_effects;
    QPushButton*   EffectsModule;

    //treying a ting
    QHBoxLayout  * themaineffectsLayout;
    QWidgetStack * theCoreLayout ;
    QHBox * theeffectsdisplay1;
    QHBox * theeffectsdisplay2;

    QFrame * thenetworkdisplay;

    /////////////////////////////////////////////////////////////
//for jahplayer
JahToolButton *playername, *playerplay, *playerdelete, *playerload;

void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                                int w, int h, QString iconOff, QString iconOn );

    /////////////////////////////////////////////////////////////

    //editing controllers
    QHBox*         MainEditingControler;
    QHBox*         MainEditingLeftControler, *MainEditingRightControler;
    QHBox*         editingdisplay;
    QHBox*         optionals_editing;
    QPushButton*   EditingModule;

    //The Text Module
    QHBox*         MainTextControler;
    QHBox*         MainTextLeftControler, *MainTextRightControler;
    QHBox*         textdisplay;
    QHBox*         optionals_text;
    QPushButton*   TextModule;

    //The Paint Module
    QHBox*         MainPaintControler;
    QHBox*         MainPaintLeftControler, *MainPaintRightControler;
    QHBox*         paintdisplay;
    QHBox*         optionals_paint;
    QPushButton*   PaintModule;

    //sub modules located on the desktop
    //The Color Module
    QHBox*         MainColorControler;
    QHBox*         MainColorLeftControler, *MainColorRightControler;
    QHBox*         colordisplay;
    QHBox*         optionals_color;
    QPushButton*   Colors;


    // for the desktop controlbar interface
    QWidget* page;
    QFrame*      modeselectframe;
    QPushButton* modeselectbutton;
    QFrame*      buttonframe;
    QPushButton* DesktopName;
    QPushButton* DesktopLoad;
    QPushButton* DesktopPlay;
    QPushButton* DeleteButton;
    QFrame*      desktopselectframe;
    QPushButton* desktopselectbutton;
    QHBoxLayout* pageLayout;

    //for the middleframe
    QFrame*        MiddleFrame;
    FancyPopup*    popup2;
    QPushButton*   clipname;
    QPushButton*   clipplay;
    QPushButton*   clipdelete;
    QPushButton*   clipload;
    QPushButton*   SwitchDesk;



    //these need to be sorted and remove the inactive ones

    QFrame*         Display;
    QFrame*        FrameMain;

    QHBox * desktopcontrolerbar;

    QTextView*     TextView1_2;
    QPushButton*   GetButton;
    QPushButton*   SwitchButton;
    QPushButton*   ReturnButton;

    QFrame*        FrameSubmenu;

    //QVBox*         optionals_prefs;
    QGridLayout*	prefsLayout;

    QFrame*         optionals_prefs;
    QWidget*       prefsdialog;


    // QButtonGroup*  bsub;
    QHBox*         desktopframe;
    QFrame*        desktopframe1;
    QFrame*        desktopframe2;
    QFrame*        desktopframe3;
    QFrame*        desktopframe4;
    QFrame*        desktopframe5;

    QPushButton*   SubLoad;
    QPushButton*   SubSave;
    QPushButton*   SubPref;
    QPushButton*   SubImport;
    QPushButton*   SubExport;
    QPushButton*   SubComp;

    QPushButton*   SubColor;
    //QPushButton*   SubPaint;
    QPushButton*   SubVideo;
    QPushButton*   SubLoadImg;
    QPushButton*   SubSaveImg;
    QPushButton*   SubQuit;

    //preferences stuff starts here
    QHBox* preferencesframe;

    QFrame* prefframe1;
    QPushButton* Pref4;
    QPushButton* Pref2;
    QPushButton* Pref3;
    QPushButton* Pref5;
    QPushButton* Pref6;
    QPushButton* Pref7;
    QPushButton* Pref8;
    QCheckBox*   Pref10;
    QLineEdit* StorageBox;
    QToolButton* StorageLocation;
    QPushButton* StorageButton;
    QSlider* renderqualityslider;
    QComboBox* renderercombo;
    QCheckBox* MenuOption;
    QCheckBox* DualHead;
    QCheckBox* VideoHead;
    QCheckBox* WideScreenBox;
    QCheckBox* PositionPrefs;
    QPushButton* PrefExit;
    QComboBox* LanguageCombo;
    QComboBox* JahresCombo;
    QLabel* interfacelabel;
    QLabel* storagelabel;
    QLabel* renderlabel;
    QLabel* uisettingslabel;
    QLabel* languagelabel;
    QLabel* jahreslabel;
    QPushButton* Pref1;
    QLabel* rendererlabel;
    QLabel* renderqualitylabel;
    QFrame* uiframe1;
    QFrame* uiframe2;
    QFrame* uiframe3;
    QFrame* uiframe4;
    QFrame* uiframe6;

    //QFrame* prefframe1;
    QFrame* prefframe2;
    QFrame* prefframe3;

    //trying a ting for the widescreen mode
    QGridLayout    *wideframeLayout;

    //the splitter
    QSplitter *splitter2;

    QHBox* wideframeLeft;
    QHBox *wideframeRight;

        //this will be for the desktop alone
    QHBoxLayout*    mainworldLayout2;
    QWidgetStack*    WorldLayout2;

};


#endif




























