/*******************************************************************************
**
** The source file for the Jahshaka jahcontrol module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "jahcreate.h"


////////////////////////////////////////////////////////
//Jahshaka Prefernces Routines
//need to break these out into a separate file...


void JahControl::setStoragePath(void) 
{
    JahTrace& jtrace = JahTrace::getInstance();

    int status = QMessageBox::critical( 0, "Setting the Storage Path",
                                        "You are about to change the Storage Path!\n\n"
                                        "do so with caution, and only if you have\n"
                                        "copied your media folder to the new location!\n\n"

                                        "You will need to restart for changes to take effect.\n\n"
                                        "Do you still want to Proceed?\n",
                                        QMessageBox::Yes | QMessageBox::Default,
                                        QMessageBox::No,
                                        QMessageBox::Cancel | QMessageBox::Escape );


    switch (status) {

    case QMessageBox::Yes    : {

                                        QString thepath = StorageBox->text();

                                        QDir d( thepath+"media" );                        // "./example"

                                        if ( d.exists() )
                                        {
                                                JahMediaPath = thepath;

                                                JahPrefs& jprefs = JahPrefs::getInstance();
                                                jprefs.setMediaPath(JahMediaPath);
                                        }
                                        else
                                        {
                                                //let them know whats up
                                                QMessageBox::information(0, "Error",
                                                "No media folder found at\n\n"+ thepath);

                                                //restore settings
                                                JahPrefs& jprefs = JahPrefs::getInstance();
                                                StorageBox->setText(jprefs.getMediaPath());

                                        }


                                break; }

    case QMessageBox::No     : { break; }

    case QMessageBox::Cancel : { break; }

    default :                  { break; }

}

}

void JahControl::getStoragePath(void)
{
        QString s = QFileDialog::getExistingDirectory(
        QDir::homeDirPath (),
        0,
        "Set Jahshaka Storage Space",
        "Choose a Directory for Media & Scene Storage" );


        QDir d( s+"media" );                        // "./example"

        if ( d.exists() )
        {
                StorageBox->setText(s);
                QString thepath = StorageBox->text();

                JahPrefs& jprefs = JahPrefs::getInstance();
                jprefs.setMediaPath(JahMediaPath);
        }
        else
        {
                QMessageBox::information(0, "Error",
                "No media folder found at\n\n"+ s);
        }
}

void JahControl::setGlobalRenderQuality(void)
{

    //read the slider setting
    int quality = renderqualityslider->value();

    if ( quality<0) { jahrenderquality=2; }
    else if ( quality>2) { jahrenderquality=2; }
    else { jahrenderquality = quality; }

    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    jprefs.setRenderQuality(jahrenderquality);


    //update all the modules now
    setModulesRenderQuality();


}

//change the name here to set stored settings for prefs
void JahControl::setModulesRenderQuality(void) 
{
    //update all the modules with render quality settings and videohead settings
    animation->setRenderQuality(jahrenderquality);
    effects->setRenderQuality(jahrenderquality);
    editing->setRenderQuality(jahrenderquality);
    text->setRenderQuality(jahrenderquality);
    colorize->setRenderQuality(jahrenderquality);
    paint->setRenderQuality(jahrenderquality);

}

//change the name here to get stored settings for prefs
void JahControl::getGlobalPreferences(void)
{
    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    jahrenderquality  = jprefs.getRenderQuality();

    rememberprefs     = jprefs.getRememberGeometry();

    dualheadprefs     = jprefs.getDualhead();

    videohead         = jprefs.getVideohead();

    menuprefs         = jprefs.getMenubars();

    renderer          = jprefs.getRenderer();

    widescreen        = jprefs.getWidescreen();

    jahStyleprefs     = jprefs.getStyle();

}


void JahControl::setStyleOption(void)
{
    debug("setting style");

    JahPrefs& jprefs  = JahPrefs::getInstance();

    jahStyleprefs = Pref10->isChecked();

    jprefs.setStyle(jahStyleprefs);
}

void JahControl::setGlobalPositionSize(void)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    rememberprefs = PositionPrefs->isChecked();

    jprefs.setRememberGeometry(rememberprefs);
}

void JahControl::setVideoHeadOption(void)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    videohead = VideoHead->isChecked();

    jprefs.setVideohead(videohead);

    if (videohead)
    {
        vidplayer->videohead = true;
        vidplayer->launchVideoHead();
        vidplayer->activateVideoHead();

        displayVideo2startupImage();

        //vidplayer->playerdialog->show();
    }
    else
    {
        vidplayer->deactivateVideoHead();
    }

}

void JahControl::setWidescreenOption(void)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    widescreenprefs = WideScreenBox->isChecked();

    jprefs.setWidescreen(widescreenprefs);
}


void JahControl::setDualHeadOption(void)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    dualheadprefs = DualHead->isChecked();

    jprefs.setDualhead(dualheadprefs);
}

void JahControl::setMenuOption(void)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    menuprefs = MenuOption->isChecked();

    jprefs.setMenubars(menuprefs);
}

void JahControl::setRenderer(int renval)
{
    JahPrefs& jprefs  = JahPrefs::getInstance();

    jprefs.setRenderer(renval);
}

void JahControl::setLanguage(int renval)
{
    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    switch (renval) {

    case 0    : { jprefs.setLanguage("English");		break; }
    case 1    : { jprefs.setLanguage("Spanish");		break; }
    case 2    : { jprefs.setLanguage("French");		break; }
    case 3    : { jprefs.setLanguage("Portuguese");	break; }
    case 4    : { jprefs.setLanguage("German");		break; }

    default :   { jprefs.setLanguage("English");		break; }

}

}

void JahControl::setResolution(int renval)
{
    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    jprefs.setJahResolution(renval);	
}



