/*******************************************************************************
**
** The source file for the Jahshaka Jahroutines module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

// These are all the routines used by the jahcontrol class
//this file can be broken into module switching and utility routines

#include "jahcreate.h"


void JahControl::addImageToDesktop( videoData thedata )
{

    //need to call the table function when we load clips in the desktop module
    thedesktop->addItemToDesktopTable( globalclipnumber, thedata );
    thedesktop->addImage2(thedata);
    thedesktop->doRefresh();

    thedesktop->addAssetTodatabase(thedata);

}

////////////////////////////////////////////////////////////////////////////
// used in the network p2p module to grab clips from the p2p share
// thedata is the path+filename of the clip
// and also by the desktop library module

void JahControl::addStringToDesktop( QString thedata )
{
    //need to call the table function when we load clips in the desktop module
    //thedesktop->addItemToDesktopTable( globalclipnumber, thedata );
    //thedesktop->addImage2(thedata);
    //debug("adding string to desktop");

    thedesktop->loadJahAsset(thedata);

    //thedesktop->addImage2database(thedata);

}

void JahControl::getClipFromLibrary()
{
    QString result = theLibrary->returnFilename;

    if (result)
    {
        addStringToDesktop(result);
		changeModeDesktop();
    }

}

void JahControl::switchModule(QString modname)
{
	modname = modname.lower();

	if (modname=="animation") changeModeAnimation();
	if (modname=="editing")   changeModeEditing();
	if (modname=="effects")   changeModeEffects();
	if (modname=="text")      changeModeText();
	if (modname=="colorize")  changeModeColorize();
	if (modname=="desktop")   changeModeDesktop();
	if (modname=="paint")     changeModePaint();
	if (modname=="network")   changeModeNetwork();
}

void JahControl::loadModuleFile(QString filename)
{
    //first see if a layer is selected in the calling module
    JahTrace& jtrace = JahTrace::getInstance();

	switch(module) {

	    case ANIMATION    : {   animation->LoadAll(filename);
                                break; }

        case EDITING      : {   editing->LoadAll(filename);
                                break; }

        case EFFECTS      : {   effects->LoadAll(filename);
                                break; }

        case TEXTCG       : {   text->LoadAll(filename);
                                break; }

        case PAINT        : {   paint->LoadAll(filename);
                                break; }

        case COLORIZE     : {   colorize->LoadAll(filename);
                                break; }

        default           : {
                              jtrace.debug("loadModuleFile()","unknown module");
                              break;
                            }
	}


}


////////////////////////////////////////////////////////////////////////////
// used for drag and drop

void JahControl::draggedAImage( )
{
    //first see if a layer is selected in the calling module
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("grabbing asset now that it dropped");

     switch(module) {

        case ANIMATION    : { //animation->stopanimation();
                                jtrace.debug("animation");
                                int pp = animation->activeLayers->getFinalSelection();
                                jtrace.debug("acitve layer:",pp);
                                break; }

        case EDITING      : {   break; }
        case EFFECTS      : {   break; }
        case TEXTCG       : {   break; }
        case PAINT        : {   break; }
        case COLORIZE     : {   break; }
		case PLAYER       : {   break; }

        default           : {
                              jtrace.debug("drag and drop::","unknown module");
                              break;
                            }

      }

    ReturnDesktop();


}

void JahControl::checkClearPlayer() 
{
    if (  (module == PLAYER) )
    { 
        vidplayer->ClearOut(); 
    }
}


void JahControl::notifyPlayerOfChange()
{
    bool clipstatus = false;

    videoData theclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
    {
        if (module == PLAYER)
        {
            vidplayer->ClearOut();

            vidplayer->LoadMySequence(theclip);
            vidplayer->setFocus();

        }
    }
}

/////////////////////////////////////////////////////////////////////////////
//this was added  to blast a image to the player on demand from a module
//but its used for videohead support from modules
//and so it messes up the player in dual head mode

void JahControl::displayClipInPlayer(QImage thedata )
{
    debug("JahControl >>doing display clip in player");

    vidplayer->ClearOut();
    vidplayer->BlastImage(thedata);
}

void JahControl::stopAnimation() 
{    //stop playback in other modules

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

     switch(module) {

        case ANIMATION    : { animation->stopanimation();
                                QObject::disconnect(core, SIGNAL(paintit()), animation, SLOT(paintGL()) );
                                QObject::disconnect(core, SIGNAL(mousePress(QMouseEvent*)),   animation, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseRelease(QMouseEvent*)), animation, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseMove(QMouseEvent*)),    animation, SLOT(mouseMoveEvent(QMouseEvent*)) );
                              break; }
        case EDITING      : { editing->stopanimation();     
                                QObject::disconnect(core, SIGNAL(paintit()), editing, SLOT(paintGL()) );
                                QObject::disconnect(core, SIGNAL(mousePress(QMouseEvent*)),   editing, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseRelease(QMouseEvent*)), editing, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseMove(QMouseEvent*)),    editing, SLOT(mouseMoveEvent(QMouseEvent*)) );
                               break; }
        case EFFECTS      : { effects->stopanimation();
                                QObject::disconnect(core, SIGNAL(paintit()), effects, SLOT(paintGL()) );
                                QObject::disconnect(core, SIGNAL(mousePress(QMouseEvent*)),   effects, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseRelease(QMouseEvent*)), effects, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseMove(QMouseEvent*)),    effects, SLOT(mouseMoveEvent(QMouseEvent*)) );
                                break; }
        case TEXTCG       : { text->stopanimation();
                                QObject::disconnect(core, SIGNAL(paintit()), text, SLOT(paintGL()) );
                                QObject::disconnect(core, SIGNAL(mousePress(QMouseEvent*)),   text, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseRelease(QMouseEvent*)), text, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseMove(QMouseEvent*)),    text, SLOT(mouseMoveEvent(QMouseEvent*)) );
                                break; }
        case PAINT        : { paint->stopanimation();        
                                QObject::disconnect(paintcore, SIGNAL(paintit()), paint, SLOT(paintGL()) );
                                QObject::disconnect(paintcore, SIGNAL(mousePress(QMouseEvent*)),   paint, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(paintcore, SIGNAL(mouseRelease(QMouseEvent*)), paint, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(paintcore, SIGNAL(mouseMove(QMouseEvent*)),    paint, SLOT(mouseMoveEvent(QMouseEvent*)) );
                                break; }
        case COLORIZE     : { colorize->stopanimation();
                                QObject::disconnect(core, SIGNAL(paintit()), colorize, SLOT(paintGL()) );
                                QObject::disconnect(core, SIGNAL(mousePress(QMouseEvent*)),   colorize, SLOT(mousePressEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseRelease(QMouseEvent*)), colorize, SLOT(mouseReleaseEvent(QMouseEvent*)) );
                                QObject::disconnect(core, SIGNAL(mouseMove(QMouseEvent*)),    colorize, SLOT(mouseMoveEvent(QMouseEvent*)) );
                                break; }


	    case DESKTOP      : { break; }  //was StopPlayer

        case PLAYER       : { vidplayer->ClearOut();        break; }  //was StopPlayer

        default           : {
                              jtrace.debug("JahControl::","unknown module in stopAnimation()");
                              break;
                            }

      }

 }




//checks the get/return button on the desktops status
void JahControl::checkDesktopButton() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "JahControl::","checkDesktopButton() checking button status");

    if ( desktopButtonStatus )
    {
        ReturnButton->hide();
        GetButton->show();
    }
    else 
    {
        GetButton->show();
    }

}


////////////////////////////////////////////////////////////
// routines to send clips from desktop to modules

void JahControl::GetDesktop() 
{
    if (module==DESKTOP) 
    {
        return;
    }

    if (!widescreen && !dualheadprefs)
    {

    desktopButtonStatus = TRUE;
    GetButton->hide();
    ReturnButton->show();

    //in widescreen mode they have to grab from the left bar

        if (dualheadprefs==true)
        {
            D2WorldLayout->raiseWidget(0);
            D2MainControler->raiseWidget(0);
        }
        else
        {
            WorldLayout->raiseWidget(0);
            MainControler->raiseWidget(0);
        }

    }
    else
    {
        desktopButtonStatus = FALSE;
        ReturnDesktop();
    }
}

void JahControl::ReturnDesktop()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    
    desktopButtonStatus = TRUE; 
    checkDesktopButton();

    //switch on module for the return    are we using mode or module?
      switch(module) {

         case ANIMATION :  { jtrace.debug( "JahControl","returnDesktop ANIMATION");
         
                             GrabClipAnim();
                             
                             WorldLayout->raiseWidget(10); 
                             optionals->raiseWidget(3); 
                             MainControler->raiseWidget(3);
                             
                             break; }

         case EFFECTS   :  { jtrace.debug( "JahControl","returnDesktop EFFECT");
                             
                             GrabClipEffects();
                             
                             WorldLayout->raiseWidget(10); 
                             optionals->raiseWidget(4); 
                             MainControler->raiseWidget(4);
                             
                             break; }

         case EDITING   :  { jtrace.debug( "JahControl","returnDesktop EDITING");
                             
                             if (editReturnClip) 
                             {
								 debug("returning");
                                ReturnClipEdit(); 
                             }
                             else 
                             {
								 debug("grabbing");
                                GrabClipEdit();
                             }
                             
                             WorldLayout->raiseWidget(10); 
                             optionals->raiseWidget(5); 
                             MainControler->raiseWidget(5);
                             
                             break; 
                           }


        case TEXTCG     :  { jtrace.debug( "JahControl","returnDesktop TEXT");
                             
                             GrabClipCG();
                             
                             WorldLayout->raiseWidget(10); 
                             optionals->raiseWidget(7); 
                             MainControler->raiseWidget(7);
                             break; 
                           }

        case PAINT      :  { jtrace.debug( "JahControl","returnDesktop PAINT");
                             
                             GrabClipPaint();
                             
                             WorldLayout->raiseWidget(6); 
                             optionals->raiseWidget(6); 
                             MainControler->raiseWidget(6);
                             paint->refreshGL(); break; 
                           }

        case COLORIZE   :  { jtrace.debug( "JahControl","returnDesktop COLORIZE");
                             GrabClipColorize();
                             WorldLayout->raiseWidget(10); 
                             optionals->raiseWidget(10); 
                             MainControler->raiseWidget(10);
                             break; 
                           }


        default  :         { break; }

      }

}

//these are used to grab the current active desktop clip...
//need error checking on this ie what if newclip is blank?
void JahControl::GrabClipAnim()    
{
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        animation->addClip(newclip);
}

void JahControl::GrabClipEffects() 
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if(clipstatus)
        effects->addClip(newclip);
}


//the editor needs to return a clip
//future drag and drop implementation here
void JahControl::AddClipEdit()     
{
    editReturnClip=TRUE;
    GetDesktop();
}

void JahControl::GrabClipEdit()    
{
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        editing->addClip(newclip);
}

void JahControl::ReturnClipEdit()  
{    

    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

	editReturnClip=FALSE;

    if (clipstatus)
    {
		editing->addClipEdit(newclip);
    }
}

void JahControl::GrabClipAnimKey() 
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        animation->addClipKey(newclip);
}

void JahControl::GrabClipCG()      
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        text->addClip(newclip);
}

void JahControl::GrabClipColorize()
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        colorize->addClip(newclip);
}

void JahControl::GrabClipPaint()   
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        paint->addClip(newclip);
}

void JahControl::GrabClipNetwork() 
{    
    bool clipstatus = false;
    videoData newclip = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
        jahnetwork->addClip(newclip,globalclipnumber);
}

void JahControl::nameClip() 
{  
    if (globalclipnumber>0) 
    {
        popup2->popup(DesktopName);
    }
}

//check to see if a clip is selected first or we crash!
void JahControl::printnameClip(QString item) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahControl::", "printnameClip() naming clip "+item);

    //grab the current desktop clip
    bool clipstatus = false;
    videoData data = thedesktop->getVideoData(clipstatus);

    if (clipstatus)
    {
        //find out the clip number
        int clipnum = data.clipnumber;

        //if its a good number ie >0 continue
        if (clipnum>0)
        {
            thedesktop->setClipname(item);

            //then update the database
            thedesktop->renameImageindatabase( clipnum, item);

            //force a full refresh
            thedesktop->doRefresh();
        }
    }

}


void JahControl::switchMode(void)
{
   //add a new layer
   //issues when adding over a deleted layer -> need to reset the layer before a add is done...
   bool added = TRUE;
   //need a global mode variable to track the main mode...

   //add on demand layers...
   QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( "Web",3,0 );
    menu->insertItem( "Audio",2,0 );
    menu->insertItem( "Video",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( modeselectbutton->mapToGlobal( QPoint( 0, modeselectbutton->height() + 1 ) ) );
    QString layNum,temp;

    if ( id != -1 ) 
    {
   
      switch(id) {
         case 1   :  { break; }
         case 2   :  { break; }
         case 3   :  { break; }
         default  :  {  break; }
      }
    } 
    else 
    { 
        added = FALSE;
    }

    delete menu;

}


void JahControl::switchDesktop(void)
{
   //add a new layer
   //issues when adding over a deleted layer -> need to reset the layer before a add is done...
   bool added = TRUE;
   //need a global mode variable to track the main mode...

   //add on demand layers...
   QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( ">> add new",2,0 );
    menu->insertItem( "Desktop 1",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( desktopselectbutton->mapToGlobal( QPoint( 0, desktopselectbutton->height() + 1 ) ) );
    QString layNum,temp;

    if ( id != -1 ) {
      //if (numberLayers==NUMBER_LAYERS) return;
      //numberLayers+=1; layNum.setNum(numberLayers);

      switch(id) {
         case 1  : { break; }
         case 2  : { break; }
         case 3  : { break; }
         default  :  {  break; }

      }
    } else { added = FALSE;}

    delete menu;

}













