/*******************************************************************************
**
** The header file for the Jahshaka Jah module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef JAH_H
#define JAH_H

#include "jahcreate.h"
#include "jahstartup.h"
#include "jahtracer.h"
#include "jahpreferences.h"

#include "glew.h"

#include <qstring.h>
#include <qdir.h>
#include <qsettings.h>
#include "qpopupmenu.h"

#include <stdlib.h>
#include <stdio.h>


//qt imaging plugins
#include "tga.h"
#include "rgb.h"

//for spaceball under linux
#ifdef SPACEBALL
#if defined(Q_WS_X11)
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <xdrvlib.h>
#endif
#endif


//  The Main Application Constructor
class Jah : public QWidget {

    Q_OBJECT

public:
  Jah( QWidget *parent=0, const char *name=0 );
  ~Jah();

  bool setBasePath(void);

  void checkJahExtensions(void);
  void enableJahExtensions(void);


  void exitError(void);

  //for the spaceball event
  void customEvent(QCustomEvent *);
  void getSpaceballData(QCustomEvent *event);
  void getSpaceballButton(QCustomEvent *event);

  void keyPressEvent( QKeyEvent *k );
  int  moderator;

  void setupMyCustomColors();

  //refresh video window...
  void updatev2vid(void);


protected:
  //initailizes the main interface
  void setupInterface( );

  //the main interface frame variables
  QGridLayout* jahshakaLayout;
  QHBoxLayout* moduleinterfaceLayout;
  QGridLayout* modulesLayout;

  QFrame* menucontroller;

  QFrame* mainleftmenu;
  QFrame* mainworld;
  QFrame* mainrightmenu;
  QFrame* maincontroller;
  QFrame* moduleinterface;
  QFrame* moduleselection;
  QFrame* modules;
  QFrame* feedbackbar;
  QFrame* modulesettings;
  QWidget * centralWidget;

  //initailizes the dual head interface
  void setupDualHeadInterface( );

  //the dual head interface frame variables
  QDialog* D2dialog;
  QWidget*  D2Widget;
  QGridLayout* D2jahshakaLayout;
  QHBoxLayout* D2moduleinterfaceLayout;
  QGridLayout* D2modulesLayout;
  QFrame* D2menucontroller;
  QFrame* D2mainleftmenu;
  QFrame* D2mainworld;
  QFrame* D2mainrightmenu;
  QFrame* D2maincontroller;
  QFrame* D2moduleinterface;
  QFrame* D2moduleselection;
  QFrame* D2modules;
  QFrame* D2feedbackbar;
  QFrame* D2modulesettings;
  QWidget * D2centralWidget;


public slots:

  void JahchangeModeText(const QString & text) { emit sendchangeModeText(text); };
  void JahsetProgressValue(int progress) { emit sendsetProgressValue(progress*10);  };
  void setStyleColor(QPalette pal);

  void storeGlobalPositionSize(void);
  void restoreGlobalPositionSize(void);
  bool checkDualHead(void);

signals:

  void sendchangeModeText(const QString & text);
  void sendsetProgressValue(int progress);


private:
  void contextMenuEvent ( QContextMenuEvent * );

  //the startup screen
  startup    *thestartupscreen;
  QTimer     *timer;

  //this is the core jahshaka class
  JahControl *jahapp;

  //graphics card extension
  QString * vendor, * renderer, * version, * extensions;

  bool dualheadOption;

};


#endif // JAH_H






