/*******************************************************************************
**
** The source file for the Jahshaka startup module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "jahstartup.h"

//This class creates and deletes the startup splash screen
//this will be the user-logon and profile manager later on

//the most important part is the getGraphicsInfo() routine
//as it gets GL info and sends back to the main app for startup
//so this is not just pretty - its functional

#include <qlayout.h>
#include <qlabel.h>
#include <qprogressbar.h>
#include <qapplication.h>

startup::startup( )
: QDialog( 0, "Splash", false, Qt::WStyle_NoBorderEx |Qt::WStyle_Customize | Qt::WStyle_StaysOnTop )
{

  //should be smaller was 300x200 for the gl intro alone
  //either resize images or resize this object to fiot...
  setGeometry ( 0, 0, 422, 312 );

  QFrame * startupframe = new QFrame (this);

  startupframe->setFrameStyle( QFrame::Box | QFrame::Plain );
  startupframe->setLineWidth( 1 );
  startupframe->setGeometry ( 0, 0, 422, 312 );       //42 extra

  QVBoxLayout *top = new QVBoxLayout( startupframe, 10 );
  QHBoxLayout *hb = new QHBoxLayout();
  top->addLayout( hb, 5 );
  hb->addSpacing( 10 );

  // Add gl frame
  QLabel *l = new QLabel( startupframe );
  l->setGeometry ( 1, 1, 420, 270 );

  // Create an OpenGL widget in splashscreen
  glsplash = new GLTexobj( l, "glbox1" );

  connect( glsplash,  SIGNAL(introFinished()), this, SLOT(introIsFinished())  );
  glsplash->setGeometry ( 1, 1, 420, 270 );

  // add progressbar and text info
  hb = new QHBoxLayout();
  top->addLayout( hb );

  QFrame * theframe = new QFrame (startupframe);
  theframe->setGeometry ( 1, 275, 420, 35 );

  // Add text label
  plabel = new QLabel( theframe );
  plabel->setAlignment( QLabel::AlignLeft );
  plabel->setFixedHeight( fontMetrics().height()+6 );
  plabel->setGeometry ( 5, 0, 65, fontMetrics().height()+6 );
  plabel->setText( "Loading" );
  hb->addWidget( plabel );

  // Add progress bar
  pbar = new QProgressBar( 100, theframe );
  pbar->setFixedHeight( fontMetrics().height()+6 );
  pbar->setGeometry ( 65, 0, 350, fontMetrics().height()+6 );
  pbar->setProgress( 0 );
  hb->addWidget( pbar );

  top->activate();

  //position it on the screen
  QRect qRect(QApplication::desktop()->screenGeometry());
  int iXpos=qRect.width()/2-this->width()/2;
  int iYpos=qRect.height()/2-this->height()/2;
  this->move(iXpos,iYpos);

  show(); raise();
  repaint( true );

  //wait until opengl intro is finished
  inIntro = true;
  while (inIntro) { qApp->processEvents(); }

  //get graphics card info and return
  getGraphicsInfo();

}

startup::~startup()
{
}

void startup::setProgressValue( int v ) { pbar->setProgress( v ); }

void startup::setProgressText( const QString &s )
{
  plabel->setText( s );
  qApp->processEvents();
}

void startup::getGraphicsInfo(void)
{
  //get the graphics card info so we can send it back to the core
  vendor     = new QString (*glsplash->vendor);
  renderer   = new QString (*glsplash->renderer);
  version    = new QString (*glsplash->version);
  extensions = new QString (*glsplash->extensions);

}
