/*******************************************************************************
**
** The source file for the Jahshaka OpenGL Intro from the startup
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "jahstartupgl.h"



const int redrawWait = 25;

// Create a GLTexobj widget


GLTexobj::GLTexobj( QWidget* parent, const char* name )
    : QGLWidget( parent, name )
{


    firsttime = true;

    xRot    = yRot = zRot = 0.0;		// default object rotation
    zPos    = 80;
    scale   = 5.0;			// default object scale
    object1 = 0;  
	object2 = 0;
    animation = TRUE;

    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), SLOT(update()) );
    timer->start( 20, TRUE );
    
    //initialize JahBasePath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath = jprefs.getBasePath();
    

}


// Release allocated resources


GLTexobj::~GLTexobj()
{
    makeCurrent();
    glDeleteLists( object1, 1 );
    glDeleteLists( object2, 1 );

}


// Paint the texobj. The actual openGL commands for drawing the texobj are
//  performed here.


void GLTexobj::paintGL()
{
    if (zPos>0.0) 
    {  
        zPos-=1.5; 
    }
    else 
    { 
        if (firsttime) 
        { 
            emit introFinished(); 
            firsttime=false;
        }
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT  );
	glMatrixMode(GL_MODELVIEW);
    
    glPushMatrix();
        glScalef( scale*2.2, scale*1.65, scale );   
        glCallList( object1 );
    glPopMatrix();

    glPushMatrix();
        glTranslatef( 0.0, 1.5, zPos);
        glScalef( scale*1.1, scale*0.825, scale );
        glCallList( object2 );
    glPopMatrix();

    if ( animation ) 
    {
	    glFlush(); // Make sure everything is drawn before restarting timer
	    timer->start( redrawWait, TRUE ); // Wait this many msecs before redraw
    }
}


// Set up the OpenGL rendering state, and define display list
void GLTexobj::initializeGL()
{
  // Set up the lights

  GLfloat whiteDir[4] = {2.0, 2.0, 2.0, 1.0};
  GLfloat whiteAmb[4] = {2.0, 2.0, 2.0, 1.0};
  GLfloat lightPos[4] = {30.0, 30.0, 30.0, 1.0};

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, whiteAmb);

  glMaterialfv(GL_FRONT, GL_DIFFUSE, whiteDir);
  glMaterialfv(GL_FRONT, GL_SPECULAR, whiteDir);
  glMaterialf(GL_FRONT, GL_SHININESS, 20.0);

  glLightfv(GL_LIGHT0, GL_DIFFUSE, whiteDir);		// enable diffuse
  glLightfv(GL_LIGHT0, GL_SPECULAR, whiteDir);	// enable specular
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos);

  // Set up the textures
  QImage tex1, tex2, buf;
  if ( !buf.load( JahBasePath+"Pixmaps/jahback.png" ) ) {	}
  tex1 = QGLWidget::convertToGLFormat( buf );  // flipped 32bit RGBA

  if ( !buf.load( JahBasePath+"Pixmaps/jahlogo.png" ) ) {	}
  tex2 = QGLWidget::convertToGLFormat( buf );  // flipped 32bit RGBA

  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glEnable( GL_TEXTURE_2D );

  // Set up various other stuff
  glClearColor( 0.0, 0.0, 0.0, 0.5 ); // Let OpenGL clear to black
  glEnable( GL_CULL_FACE );  	// don't need Z testing for convex objects
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

  // Make the object display list
  object1 = makeObjectLayer ( 1, tex1 );	// Generate an OpenGL display list
  object2 = makeObjectLayer ( 2, tex2 );	// Generate an OpenGL display list

  //this is where we get the rendering contect's and return them to the system
  //so it knows what to launch up with
  vendor     = new QString ((char *) glGetString(GL_VENDOR));
  renderer   = new QString ((char *) glGetString(GL_RENDERER));
  version    = new QString ((char *) glGetString(GL_VERSION));
  extensions = new QString ((char *) glGetString(GL_EXTENSIONS));

}

// Set up the OpenGL view port, matrix mode, etc.
void GLTexobj::resizeGL( int w, int h )
{
    glViewport( 0, 0, w, h );
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    glFrustum( -1.0, 1.0, -1.0, 1.0, 10.0, 100.0 );
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    glTranslatef( 0.0, 0.0, -70.0 );
}


// Generate an OpenGL display list for the object to be shown, i.e. the texobj
GLuint GLTexobj::makeObjectLayer( int laynum, const QImage& tex1)
{
  GLuint theObj = glGenLists(laynum);
  glNewList( theObj, GL_COMPILE );

   glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, tex1.width(), tex1.height(), 0,
		  GL_RGBA, GL_UNSIGNED_BYTE, tex1.bits() );

	 glBegin(GL_QUADS);					// Start Drawing Quads
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Top Left Of The Texture and Quad
		
	 glEnd();	
  glEndList();

  return theObj;

}

//Turns animation on or off


void GLTexobj::toggleAnimation()
{
    animation = !animation;
    if ( animation )
	updateGL();
    else
	timer->stop();
}

