/*******************************************************************************
**
** The source file for the Jahshaka Jahstyle module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifdef JAHPLAYER
#include "jahplayercreate.h"
#else
#include "jahcreate.h"
#endif

#ifdef JAHTHEMES
#include "jahmetalstyle.h"
#endif

void JahControl::stylePlatinum()
{
    jahStyleTheme = JPLATINUM;

    QStyle *platinum=QStyleFactory::create("platinum");
    if(platinum) qApp->setStyle(platinum);

    styleFont( 10, "Helvetica" );
    qApp->flushX();

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleTheme(jahStyleTheme);
}

void JahControl::styleWindows()
{
    jahStyleTheme = JWINDOWS;

    QStyle *QWindowsStyle=QStyleFactory::create("windows");
    if(QWindowsStyle) qApp->setStyle(QWindowsStyle);

    styleFont( 10, "Helvetica" );
    qApp->flushX();

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleTheme(jahStyleTheme);

}

void JahControl::styleSgi()
{
    jahStyleTheme = JSGI;

    QStyle *QSGIStyle=QStyleFactory::create("sgi");
    if(QSGIStyle) qApp->setStyle(QSGIStyle);

    styleFont( 10, "Helvetica" );
    qApp->flushX();

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleTheme(jahStyleTheme);

}

void JahControl::styleCharcoal()
{
#ifdef JAHTHEMES
debug("charcoal");
    jahStyleTheme = JCHARCOAL;
    qApp->setStyle( new CharcoalStyle );

    styleFont( 10, "Helvetica" );
    qApp->flushX();

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleTheme(jahStyleTheme);
#endif
}

//this one is building jahmetalstyle...
void JahControl::styleMetal()
{
#ifdef JAHTHEMES
debug("metal");
    jahStyleTheme = JMETAL;
    qApp->setStyle( new MetalStyle );

    styleFont( 10, "Helvetica" );
    qApp->flushX();

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleTheme(jahStyleTheme);
#endif
}


/////////////////////////////////////////////////////////
//Color styles are sent up to desktop object so as
//to color all widgets

void JahControl::styleGrey()
{
    jahStyleColor = JGREY;

    QPalette p( QColor( 80, 80, 80 ) );
    emit emitStyleColor(p);

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleColor(jahStyleColor);
}


void JahControl::styleGreen()
{
    jahStyleColor = JGREEN;

    QPalette p( QColor( 20, 150, 20 ) );
    emit emitStyleColor(p);

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleColor(jahStyleColor);
}

void JahControl::styleWhite()
{
    jahStyleColor = JWHITE;

    QPalette p( QColor( 180, 180, 180 ) );
    emit emitStyleColor(p);

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleColor(jahStyleColor);
}

void JahControl::styleBlue()
{
    jahStyleColor = JBLUE;

    QPalette p( QColor( 100, 100, 200 ) );
    emit emitStyleColor(p);

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setStyleColor(jahStyleColor);
}

////////////////////////////////////////////////////////
//used to set the global font and font size

void JahControl::styleFont(int size, QString font)
{

    QFont appFont;

    //dont know which ones work so we do them all!
    appFont = QApplication::font();
    appFont.setPointSize(size);
    appFont.setFamily(font);

    QFont::setDefaultFont(QFont(font, size, QFont::Normal, false));
    qApp->setFont( QFont( font, size ), TRUE );
}

//    enum StyleTheme { JPLATINUM,JWINDOWS, JSGI, JCHARCOAL, JMETAL };
//    enum StyleColor { JGREY, JGREEN, JWHITE, JBLUE };

void JahControl::restoreStyle()
{
    /////////////////////////////////////////////////////////////////
    //restore the style
    if (jahStyleprefs)
    {
        JahPrefs& jprefs  = JahPrefs::getInstance();

        jahStyleTheme = (StyleTheme)jprefs.getStyleTheme();
        jahStyleColor = (StyleColor)jprefs.getStyleColor();

        bool satisfied = false;

        switch(jahStyleTheme)   {

            case JPLATINUM  : { stylePlatinum();    satisfied = true; break;      }
            case JWINDOWS   : { styleWindows();     satisfied = true; break;      }
            case JSGI       : { styleSgi();         satisfied = true; break;      }
            default         : { break;                          }

        }

        //color can be applied after the themes above
        if (satisfied)
        {
            restoreColor(jahStyleColor);
            return;
        }


        if (!satisfied)
        {
            //color must be applied before the themes below
            restoreColor(jahStyleColor);

            switch(jahStyleTheme)   {

            case JCHARCOAL  : { styleCharcoal();    break;      }
            case JMETAL     : { styleMetal();       break;      }
            default         : { break;                          }

            }

        }
    }
}


void JahControl::restoreColor(int color)
{
    StyleColor theStyleColor = (StyleColor)color;

    switch(theStyleColor)   {

            case JGREY      : { styleGrey();        break;      }
            case JGREEN     : { styleGreen();       break;      }
            case JWHITE     : { styleWhite();       break;      }
            case JBLUE      : { styleBlue();        break;      }
            default   : {}

    }

}




