/*******************************************************************************
**
** The source file for the Jahshaka desktop database
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "desktop.h"

JahDesktop::JahDesktop(QHBox* parent, QHBox* wideframe, QHBox* leftcontroller, QHBox* rightcontroller,
                       const char* name, int * globalclipnumber, WFlags f)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("Setting up the Desktop Module");

   //initialize JahBasePath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath   = jprefs.getBasePath();
    JahMediaPath  = jprefs.getMediaPath();
    widescreen    = jprefs.getWidescreen();

    /////////////////////////////////////////////////////////////////
    // interface work

    //build the left and right menus
    buildInterfaceMenus(leftcontroller,rightcontroller);

    //hook up desktop switching between canvas and table
    themaindesktopLayout = new QHBoxLayout( parent, 0, 0, "maindesktopLayout");
    theDesktopLayout = new QWidgetStack(parent);
    themaindesktopLayout->addWidget( theDesktopLayout );

    thedesktopdisplay3 = new QHBox (theDesktopLayout, "desktopshare");
    theDesktopLayout->addWidget(thedesktopdisplay3,2);


    if (widescreen)
    {

    //hook up desktop switching between canvas and table
    themaindesktopWideLayout = new QHBoxLayout( wideframe, 0, 0, "maindesktopLayout");
    theDesktopLayoutWide = new QWidgetStack(wideframe);
    themaindesktopWideLayout->addWidget( theDesktopLayoutWide );

    thedesktopdisplaywide = new QHBox (theDesktopLayoutWide, "desktopshare");
    theDesktopLayoutWide->addWidget(thedesktopdisplaywide,2);

    }

    ////////////////////////////////////////////////////////////////
    // set up desktop display objects

    //initialize the desktop display
    initDesktopCanvas();

    //initialize the desktop table
    initDesktopTable();

    //initialize the desktop share display
    initDesktopSharedCanvas();

    //initialize the desktop broadcast display
    //initDesktopBroadcastCanvas();

    //initialize the shared desktop table
    //initSharedDesktopTable();

    //set some variables
    clipnumber = globalclipnumber;

    ////////////////////////////////////////////////////////////////
    //initialize the database and read in clips

    //this is where we would use our new database object
    theDatabase = new JahDatabase("JahDatabase");
    theDatabase->InitializeDatabase();

    RebuildDesktop();

    //QString stringval,debugmessage;
    //debugmessage="clipnumber is now set to:" + stringval.setNum(*clipnumber);
    //jtrace.debug(debugmessage);

    ////////////////////////////////////////////////////////////////
    //activate default ui selection the canvas
    theDesktopLayout->raiseWidget(0);


    if (widescreen)
    {
        theDesktopLayoutWide->raiseWidget(0);
    }

    desktopView = true;

    //for drag and drop
    setAcceptDrops(TRUE);

    //debug("finsihed desktop");

}

JahDesktop::~JahDesktop()
{

    ////////////////////////////////////////////////////////
    //JahDesktop Destructor
    //cleans up as we leave and releases memory...
    //should check to make sure database is open
    theDatabase->CloseDatabase(*clipnumber);

}

void JahDesktop::dragEnterEvent(QDragEnterEvent* event)
{
    debug("drag event");

    event->accept(
        QTextDrag::canDecode(event) ||
        QImageDrag::canDecode(event)
    );
}

void JahDesktop::dropEvent(QDropEvent* event)
{
    QImage image;
    QString text;

    debug("drop event");

    if ( QImageDrag::decode(event, image) )
    {
        debug("drop image event");
    }
    else if ( QTextDrag::decode(event, text) )
    {
       debug("drop text event");
    }
}

void JahDesktop::initDesktopCanvas() 
{

    thedesktopdisplay1 = new QHBox (theDesktopLayout, "desktopcanvas");
    theDesktopLayout->addWidget(thedesktopdisplay1,0);
    
    //build the main canvas object first
    canvas = new QCanvas(thedesktopdisplay1); // canvas = new QCanvas(this);
    QColor back;    back.setHsv(35,35,35);
    canvas->setBackgroundColor(back);
    canvas->resize( 1270, 700 );

    //build the node editor for the canvas
    editor = new FigureEditor(canvas,thedesktopdisplay1); 
    editor->setHScrollBarMode( FigureEditor::AlwaysOff );
    editor->setVScrollBarMode( FigureEditor::AlwaysOff );

    connect ( editor, SIGNAL(updateClipPosition(videoData)), this, SLOT( updateClipPosition(videoData)) );


    connect ( editor, SIGNAL(activeFlagDesktop()), this, SLOT( activeFlagDesktop()) );

}

void JahDesktop::initDesktopTable()
{
    if (!widescreen)
    {
        thedesktopdisplay2 = new QHBox (theDesktopLayout, "desktoptable");
        theDesktopLayout->addWidget(thedesktopdisplay2,1);
    }
    else
    {
        thedesktopdisplay2 = new QHBox (theDesktopLayoutWide, "desktoptable");
        theDesktopLayoutWide->addWidget(thedesktopdisplay2,0);
    }

    //this is hardcoded for now... need to fix this
    const int numRows = 40;
    const int numCols = 11;

    table = new JahDesktopTable( numRows, numCols, thedesktopdisplay2 );


    connect ( table, SIGNAL(currentChanged ( int, int )), this, SLOT( selectTableRow(int,int)) );

    table->show();

    globaltablenumber = 0;

    //means nothing is selected... in our table
    tableSelect = -1;
}

void JahDesktop::initDesktopSharedCanvas()
{

    thedesktopdisplay3 = new QHBox (theDesktopLayout, "desktopsharecanvas");
    theDesktopLayout->addWidget(thedesktopdisplay3,2);

    //build the main canvas object first
    canvasshare = new QCanvas(thedesktopdisplay3); // canvas = new QCanvas(this);
    QColor back;    back.setHsv(35,35,35);
    canvasshare->setBackgroundColor(back);
    canvasshare->resize( 1270, 700 );

    //build the node editor for the canvas
    shareeditor = new FigureEditor(canvasshare,thedesktopdisplay3); 
    shareeditor->setHScrollBarMode( FigureEditor::AlwaysOff );
    shareeditor->setVScrollBarMode( FigureEditor::AlwaysOff );
    //connect ( shareeditor, SIGNAL(updateClipDatabase(videoData)), this, SLOT( updateClipDatabase(videoData)) );

}

void JahDesktop::addItemToDesktopTable(int index, videoData thedata )
{
    //till we fix hardcoded rows
    if (globaltablenumber>40) return;

    table->addAsset(index, globaltablenumber,thedata);

    globaltablenumber += 1;
}


void JahDesktop::selectTableRow(int row, int col)
{
    QString text = table->text(row,10);

    tableSelect = text.toInt();

    //dont know why we have to do this...
    tableSelect+=1;

    //connect ( editor, SIGNAL(activeFlagDesktop()), this, SLOT( activeFlagDesktop()) );
    activeFlagTable();
}

  //used to flip desktop display modes
void JahDesktop::toggleDesktopImages()
{
    theDesktopLayout->raiseWidget(0);
    //desktopView = true;
}

void JahDesktop::toggleDesktopTable()
{
    theDesktopLayout->raiseWidget(1);
    //desktopView = false;
}


void JahDesktop::clearAll() 
{

    //create a popup window saying are you really sure?
    //do a qdialog...
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "JahDesktop::","called clear all...");

    int status = QMessageBox::critical( 0, "Clearing The Desktop",
                                           "You are about to Clear the Desktop!\n\n"
                                           "This will Erase all clips and images\n"
                                           "in your projects Renders directory!\n\n"
                                           "Export files you want to keep first!\n\n"
                                           "Do you still want to Clear All?\n",
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::No,
                                           QMessageBox::Cancel | QMessageBox::Escape );


    switch (status) {

     case QMessageBox::Yes    : { clearTheDesktop(); break; }

     case QMessageBox::No     : { break; }

     case QMessageBox::Cancel : { break; }

     default :                  { break; }

   }

}

//this is used when we rebuild the database as it doesnt append the clipnumber to the images
void JahDesktop::addImage( videoData thedata)
{

    //assethack
    assetExchange exchange;
    QImage theimage; 
    theimage = exchange.getImage(thedata);

    //we need to scale the proxy size here instead of using the default 0.25% reduction
    float wratio = (180/(float)theimage.width() );
    float hratio = (120/(float)theimage.height() );

    QString theclipnumber;
    theclipnumber.setNum(thedata.clipnumber);

    QString clipper1; clipper1= thedata.clipname;

    img = new QImage[2];
    //img[1] = theimage.smoothScale( int(theimage.width()*wratio), int(theimage.height()*hratio), QImage::ScaleFree );
    img[1] = theimage.smoothScale( int(theimage.width()*wratio), int(theimage.height()*hratio), QImage::ScaleMin );

    // we need to update this so that we dont have to send the thedata.Image just thedata...
    QCanvasPolygonalItem* i = new ImageItem(thedata, img[1],canvas, clipper1);
      i->setPen( QPen(QColor(200,0,0), 6) );
      i->setZ(1);
      i->setX(thedata.xpos);
      i->setY(thedata.ypos);
      i->show();

    delete [] img;      // is this necessary as img is not a pointer???
    editor->update();
}

//this is used to add a image to the desktop
void JahDesktop::addImage2( videoData thedata )
{


    //we need to grab the image here since its no longer in videoData
    assetExchange exchange;
    QImage theimage; 
    theimage = exchange.getImage(thedata);
    
    //we need to scale the proxy size here instead of using the default 0.25% reduction
    float wratio = (180/(float)theimage.width() );
    float hratio = (120/(float)theimage.height() );

    QString theclipnumber;
    theclipnumber.setNum(thedata.clipnumber);

    QString clipper1; clipper1= thedata.clipname;
    clipper1.append(theclipnumber);

    img = new QImage[2];
    img[1] = theimage.smoothScale( int(theimage.width()*wratio), int(theimage.height()*hratio), QImage::ScaleMin );

    // we need to update this so that we dont have to send the thedata.Image just thedata...
    QCanvasPolygonalItem* i = new ImageItem(thedata, img[1],canvas, clipper1);
      i->setPen( QPen(QColor(200,0,0), 6) );
      i->setZ(1);
      i->setX(thedata.xpos);
      i->setY(thedata.ypos);
      i->show();

    delete [] img;      // is this necessary as img is not a pointer???
    editor->update();

}



ImageItem* JahDesktop::getCurrentItem()
{

    return editor->globalitem;

}

void JahDesktop::addRectangle()
{
    QCanvasPolygonalItem *i = new QCanvasRectangle( 0,0, 180,120,canvas);
    i->setBrush( QColor(50,0,200) );
    i->setPen( QPen(QColor(0,50,200), 6) );
    i->setZ(1);
    i->show();
}

void JahDesktop::activeFlagDesktop()
{

    //debug("hit activeFlagDesktop");
    desktopView = true;
}

void JahDesktop::activeFlagTable()
{

    //debug("hit activeFlagTable");
    desktopView = false;

    //used to update player if player is active module
    //get rid of this
    //emit notifyPlayerOfChange();

}

////////////////////////////////////////////////////////////
//we used to return a pointer...
//return editor->selectedClip;
//now it returns the asset...

//this should return the last selected clip?
//weather its in the table or on the desktop...

videoData JahDesktop::getVideoData(bool & status)
{
    JahTrace& jtrace = JahTrace::getInstance();

    videoData assetData,nullAsset;
    int theAsset;

    //this is where the last selection was ie either desktop or table

    if (desktopView)
    {
        int desktopSelect = editor->getDesktopSelect();
        theAsset = desktopSelect;
    }
    else
    {
        theAsset = tableSelect;
    }

    QString rere;
    jtrace.debug("The asset is:"+rere.setNum(theAsset));

    //bool exists = doesAssetExistindatabase(theAsset);
    //if (exists) debug("it exists");
    //else
    //debug("doesnt exist");


    if (theAsset<0)
    {
        //debug("its false");
        status = false;
    }
    else
    {
        //debug("its true");
        status = true;
    }

    if (status)
        assetData = theDatabase->getImageFromDatabase(theAsset);

    int assetStatus;
    assetStatus = assetData.theCategory;
    QString rara;
    jtrace.debug("the asset status is:"+rara.setNum(assetStatus));

    if (assetStatus<0)
        status = false;


    return assetData;
}

void JahDesktop::setClipname(QString& newname)
{
     editor->setItemtext(newname);
	 editor->update();
}

void JahDesktop::doRefresh(void)
{
    canvas->setAllChanged();
    canvas->update();
    editor->update();

}

QString JahDesktop::getClipname()
{

    return editor->imagetext;
}

















