/*******************************************************************************
**
** The header file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef DESKTOP_H
#define DESKTOP_H

#include <qmessagebox.h>
#include <qwidgetstack.h>
#include <qtable.h>
#include <qdropsite.h>
#include <qfile.h>
#include <qfiledialog.h>

#include "desktopcanvas.h"
#include "desktoptable.h"
#include "videodata.h"
#include "mediaobject.h"
#include "jahtracer.h"
#include "jahdatabase.h"


class JahDesktop : public QWidget {
  Q_OBJECT

public:
    JahDesktop(QHBox* parent=0,QHBox* wideframe=0, QHBox* leftcontroller=0, QHBox* rightcontroller=0,
               const char* name=0, int * globalclipnumber=0, WFlags f=0);

    ~JahDesktop();

    void setClipname(QString&);
    void doRefresh(void);
    QString getClipname();
    int *clipnumber;
    int tableSelect;
    bool desktopView;
    bool widescreen;

    JahDatabase * theDatabase;

    videoData getVideoData(bool & status);

    ImageItem* getCurrentItem();	// audio hack

    void initDesktopCanvas();
    void initDesktopTable();
    void initDesktopSharedCanvas();

    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller);
    void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                                    int w, int h, QString iconOff, QString iconOn );

    int globaltablenumber;

    void loadJahAsset(QString);

	QString findModuleName      (QString loadfilename);
    void loadJahMediaAsset   (QString filename,QString shortName,QString pathString,QString extension);
    void loadJahAudioAsset   (QString filename,QString shortName,QString pathString,QString extension);
    void loadJahImageAsset   (QString filename,QString shortName,QString pathString,QString extension);
    void loadJahSequenceAsset(QString filename,QString shortName,QString pathString,QString extension,
                              QString theextension, int startFrame, int numberFrames);

protected:
    ///////////////////////////////////////////////////////////////
    void dragEnterEvent(QDragEnterEvent* event);
    void dropEvent(QDropEvent* event);

public slots:

    void addImage2( videoData thedata );
    void addImage( videoData thedata );
    void addItemToDesktopTable(int index, videoData thedata );

    //database calls
    void renameImageindatabase( int imagenumber, QString newname);
    void addAssetTodatabase(videoData thedata);
    void updateClipPosition( videoData data);

	//loaders
    void slotExport();
    void slotLoadIconview();
    void clearItem();
    void clearAll();

signals:
	void loadJahSceneFile(QString);

    void resetGlobalClipNumber();
    void notifyPlayerOfChange();

	void switchMainModule(QString);
	void loadMainModuleFile(QString);

private slots:

    bool clearTheDesktop();

    bool eraseTheAsset(videoData erase);
    bool exportTheAsset(QString exportto, videoData theasset);

    //void init();
    void addRectangle();
    void slotSave();
    void slotLoad();

    bool getExtension(QString & string);
    bool isanumber(QString & string);

    void RebuildDesktop();

    //used to flip desktop display modes
    void toggleDesktopImages();
    void toggleDesktopTable();
    void selectTableRow(int row, int col);

    void activeFlagDesktop();
    void activeFlagTable();

private:
    /////////////////////////////////////////////////
    //desktop interface variables

    //for the left and right menus
    QFrame         *glworldLeft;

    JahToolButton  *toolbinview, *toolthumbview;

    QFrame         *glworldRight;
    QPushButton    *toolzoomin, *toolzoomout;
    QSlider        *zoom;

    //for the canvas/table switching
    QHBoxLayout * themaindesktopLayout;
    QWidgetStack * theDesktopLayout;
    QHBoxLayout * themaindesktopLayout2;
    QWidgetStack * theDesktopLayout2;

    QHBox * thedesktopdisplay1;
    QHBox * thedesktopdisplay2;
    QHBox * thedesktopdisplay3;

    QHBoxLayout *themaindesktopWideLayout;
    QWidgetStack *theDesktopLayoutWide;

    QHBox *thedesktopdisplaywide;

    /////////////////////////////////////////////////
    // desktop core variables

    //QTable * table;
    JahDesktopTable * table;

    FigureEditor *editor, *shareeditor;
    QCanvas *canvas, *canvasshare;

    QImage* img;

    QString JahBasePath;
    QString JahMediaPath;
};

#endif




