/*******************************************************************************
**
** The source file for the Jahshaka desktop canvas file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "desktopcanvas.h"



//fix isse here with fullres - why are we storig this?
//seems like we can just store the img?

ImageItem::ImageItem( videoData full, QImage img, QCanvas *canvas, QString& clipname )
        : QCanvasRectangle( canvas ), image(img)
{
    setSize( image.width(), image.height() );
    _type = ITEM_IMAGE;

    #ifndef _WS_QWS_
    pixmap.convertFromImage(image, OrderedAlphaDither);
    #endif

    name     = clipname;
    itemdata = full;

    imagewidth  = image.width();
    imageheight = image.height();

    diewidth  = 180;
    dieheight = 120;
}


void ImageItem::drawShape( QPainter &p )
{
    //first figure out offset so we can center image on die
    int xoffset = 0;
    int yoffset = 0;

    if (imagewidth<diewidth)   { xoffset = ((diewidth  - imagewidth)/2);  }
    if (imageheight<dieheight) { yoffset = ((dieheight - imageheight)/2); }

    // On Qt/Embedded, we can paint a QImage as fast as a QPixmap,
    // but on other platforms, we need to use a QPixmap.
    #ifdef _WS_QWS_
    p.drawImage( int(x()), int(y()), image, 0, 0, -1, -1, OrderedAlphaDither );
    #else
    p.drawPixmap( int(x())+xoffset, int(y())+yoffset, pixmap );
    #endif
}

bool ImageItem::hit( const QPoint &p ) const 
{
    int ix = p.x()-int(x());
    int iy = p.y()-int(y());
    if ( !image.valid( ix , iy ) )
        return FALSE;
    QRgb pixel = image.pixel( ix, iy );
    return qAlpha( pixel ) != 0;
}

FigureEditor::FigureEditor( QCanvas* c, QWidget* parent,
 const char* name, WFlags f) :
 QCanvasView(c,parent,name,f) {

    height=1;   //what happens if this goes over the max value for a int? crash!!!
    //ImageItem globalitem;

    bordercreated = FALSE;
    move = FALSE;
}


void FigureEditor::contentsMousePressEvent(QMouseEvent* e)
{

    // creating border anywhere else ie in constructor crashes app...
    // we want to use a better method anyways so aint gonna waste no more time on this
    // ideas are just to draw selection boundary
    // moving->setSelected(1);  // QRect myframe = moving->boundingRect();

    QCanvasItemList l=canvas()->collisions(e->pos());
	videoData itemData;

    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
    {

        if ( (*it)->rtti() == imageRTTI )
        {
			//debug("clicked on item on desktop");
			emit activeFlagDesktop();

            ImageItem *item= (ImageItem*)(*it);

            if ( !item->hit( e->pos() ) )
            {
                continue;
            }
            move = 1;

            globalitem = (ImageItem*)(*it);

            imagetext = item->getText();

            selectedClip = item->getClip();
            selectedClip.xpos = item->x();
            selectedClip.ypos = item->y();

			itemData = item->getClip();
        }


	QColor bordercolor;
    QString thetype;

    if (!bordercreated)
    {

        border = new QCanvasRectangle( 0,0, 180,120,canvas());

        //its a messy hack and needs to be cleaned up
        switch ( itemData.theCategory )
        {
            case IMAGE    : { thetype="Image :"; bordercolor.setRgb(0,50,200); break; }
            case CLIP     : { thetype="Clip  :";bordercolor.setRgb(0,200,50); break; }
            case AUDIO    : { thetype="Audio :";bordercolor.setRgb(200,50,0); break; }
            case MEDIA    : { thetype="Media :";bordercolor.setRgb(200,0,50); break; }
            //case KEY      : { thetype="Key   :";bordercolor.setRgb(0,50,200); break; }
            //case CLIPKEY  : { thetype="ClipKey :";bordercolor.setRgb(0,200,50); break; }

            default       : { thetype="Unknown"; bordercolor.setRgb(200,0,50); break; }

        }
        border->setBrush( bordercolor );
        border->setPen( QPen( bordercolor, 6) );
        border->setZ(1);

        itemtext = new QCanvasText( canvas());
        itemtext->setColor( QColor(200,200,200) );
        itemtext->setText(thetype+imagetext);
        itemtext->setZ(1);
        bordercreated = TRUE;
    }



    //use a switch here
    //dont think we need to deal with keys at the desktop level
    switch ( itemData.theCategory )
    {
        case IMAGE    : { thetype="Image :"; bordercolor.setRgb(0,50,200); break; }
        case CLIP     : { thetype="Clip  :";bordercolor.setRgb(0,200,50); break; }
        case AUDIO    : { thetype="Audio :";bordercolor.setRgb(200,50,0); break; }
        case MEDIA    : { thetype="Media :";bordercolor.setRgb(200,0,50); break; }
        //case KEY      : { thetype="Key   :";bordercolor.setRgb(0,50,200); break; }
        //case CLIPKEY  : { thetype="ClipKey :";bordercolor.setRgb(0,200,50); break; }

        default       : { thetype="Unknown"; bordercolor.setRgb(200,0,50); break; }

    }

    itemtext->setText(thetype+imagetext);



        border->setBrush( bordercolor );
        border->setPen( QPen( bordercolor, 6) );

        //moving is set to canvas item referenced by *it
        moving = *it;

        moving_start = e->pos();


        //now we position border and bring selected item and
        //border to the front of others on the canvas
        border->setX( moving->x() );
        border->setY( moving->y() );
        height++; border->setZ(height);

        itemtext->setX( moving->x() );
        itemtext->setY( moving->y() );

        height++; moving->setZ(height);
        height++; itemtext->setZ(height);

        border->show();
        itemtext->show();
        canvas()->update();

        return;
    }
    moving = 0;


    //debug("clicked on desktop itself not a item");

}


void FigureEditor::clear()
{
    //debug("clearing the list");

    QCanvasItemList list = canvas()->allItems();
    QCanvasItemList::Iterator it = list.begin();

    for (; it != list.end(); ++it) 
	{
        if ( *it ) 
		{
            delete *it;
        }
    }

    canvas()->update();

    bordercreated = FALSE;
    move = FALSE;

    height=1;    //reset this variable when we clear the desktop
}


void FigureEditor::clearItem() 
{

    if (  moving ) {

		//delete selectedClip;
        delete moving;   
        //delete border;
        //delete itemtext;

        border->hide();   
		itemtext->hide();
        canvas()->update();
    }
}

bool FigureEditor::eraseItem(int clipnumber) 
{

    // creating border anywhere else ie in constructor crashes app...
    // we want to use a better method anyways so aint gonna waste no more time on this
    // ideas are just to draw selection boundary
    // moving->setSelected(1);  // QRect myframe = moving->boundingRect();
	//QString rere;
	//debug("removing item from desktop"+rere.setNum(clipnumber));

	bool foundclip = false;

	videoData itemData;
	QCanvasItem* theitem;
    QCanvasItemList l=canvas()->allItems();

    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
    {

		if ( (*it)->rtti() == imageRTTI )
		{

			ImageItem *item= (ImageItem*)(*it);

			itemData = item->getClip();

			if (itemData.clipnumber==clipnumber)
			{
				foundclip = true;
				theitem = *it;
			}
		}
    }

	//debug("looped");

	if (foundclip)
	{
		//debug("found clip");
        delete theitem;  
		//debug("deleted clip");

		if(border)
			border->hide();   
		if(itemtext)
			itemtext->hide();

        canvas()->update();
		//debug("erased and updated");
	}

	return foundclip;
	//debug("removed item from desktop");

}

void FigureEditor::setItemtext(QString& newname) 
{

//    debug("setting clipname");  debug(newname);
    globalitem->setText(newname);
    canvas()->update();
}



void FigureEditor::contentsMouseReleaseEvent(QMouseEvent* e) 
{

	if (moving) 
	{
		bool update = false;

		if  ( selectedClip.xpos != moving->x() ) { selectedClip.xpos = moving->x(); update=true; }
		if  ( selectedClip.ypos != moving->y() ) { selectedClip.ypos = moving->y(); update=true; }

		//update the clips position in the database
		if (update) { emit updateClipPosition(selectedClip); }

		//need to release move here?
		move = 0;
		//moving = 0;
		canvas()->update();
    }

}

void FigureEditor::contentsMouseMoveEvent(QMouseEvent* e) 
{
    if ( move ) 
	{   // was moving

        //move clip and the background border
        moving->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        border->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        //text moveby
        itemtext->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());

        moving_start = e->pos();
        canvas()->update();
    }
}

int FigureEditor::getDesktopSelect()
{
    return selectedClip.clipnumber;

}












