/*******************************************************************************
**
** The source file for the Jahshaka desktop io file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "desktop.h"

#include "jahfilewindow.h"
#include "jahfileiconview.h"


///////////////////////////////////////////////////////
// asset loader routine for the desktop
void JahDesktop::slotLoadIconview() 
{
    QString result;
    bool accepted;

    FileMainWindow *  theloader = new FileMainWindow (this,"theloader",TRUE);
    theloader->fileView()->setDirectory( JahMediaPath+"/media/" ); 

    if ( theloader->exec() == QDialog::Accepted ) 
    {
        accepted = true;
        result = theloader->returnFilename;
    }
    else
    {
        accepted = false;
    }

    delete theloader;


    //do some extra error checking here
    if (accepted) 
    {
            if (result)
            { 
                loadJahAsset(result);
            }
    }


}

//the old loader kept for backwards compatability
void JahDesktop::slotLoad() 
{

    //ask user for file now
    QString filename = QFileDialog::getOpenFileName( JahMediaPath+"media", "*.*" , this );
    filename=filename.stripWhiteSpace();

    //user didnt enter anything or cancelled
    if (filename) 
    { 
        loadJahAsset(filename); 
    }


}

/////////////////////////////////////////////////////////////////////////////////
//important routine here used to control supported extensions for the desktop

bool JahDesktop::getExtension(QString & string)
{
    bool windowssupport     = false;
    #ifdef WIN32
    windowssupport          = true;
    #endif

    bool audiosupport       = false;
    #ifdef AUDIOSUPPORT
    audiosupport            = true;
    #endif

    bool jahmpegdecoder     = false;
    #ifdef JAHMPEGDECODER
    jahmpegdecoder          = true;
    #endif

    bool linuxavisuuport    = false;
    #ifdef LINUXAVISUPPORT
    linuxavisuuport         = true;
    #endif

    //set up variables here
    bool ok = false;
    string      = string.right(4);
    QString ext = string.lower();

    //check against known supported extensions
    if (ext==".bmp")                     { ok = true; } else // image formats
    if (ext==".jpg")                     { ok = true; } else
    if (ext==".png")                     { ok = true; } else
    if (ext==".gif")                     { ok = true; } else
    if (ext==".tga")                     { ok = true; } else
    if (ext==".rgb")                     { ok = true; } else
    if (ext==".ppm")                     { ok = true; } else
	    
    if (audiosupport    && ext==".wav")  { ok = true; } else //audio formats
    if (audiosupport    && ext==".aif")  { ok = true; } else
    if (audiosupport    && ext==".mp3")  { ok = true; } else
    if (audiosupport    && ext==".ogg")  { ok = true; } else

    if (jahmpegdecoder  && ext==".mpg")  { ok = true; } else // media formats
    if (jahmpegdecoder  && ext=="mpeg")  { ok = true; } else
    if (linuxavisuuport && ext==".avi")  { ok = true; } else
    if (windowssupport  && ext==".avi")  { ok = true; } else

	//for scene files
	if (ext==".jsf")                     { ok = true; } else
	if (ext==".jah")                     { ok = true; } 

    //return and let caller know if extension is supported
    //if (ok) debug("its ok");
    //else debug("its not ok");

    return ok;

}

///////////////////////////////////////////////////////
// asset loader routine for the desktop
void JahDesktop::loadJahAsset(QString filename) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( ">> Loading Asset into Database:", filename );

    //user didnt enter anything or cancelled
    if (!filename)
        return;

    //clean filename
    filename=filename.stripWhiteSpace();

    //////////////////////////////////////////////////////////////////
    //figure out the extension here

    static char	errorMessage[256];
    bool validextension;
    QString extension(filename); //set to filename

    //get the extension and check to see if its supported
    validextension = getExtension(extension);

    if (!validextension)
    {
        QString errortext = "File type or sequence not supported:\n"+filename;
        errortext +="\n\nLooking for something like 'file.bmp', 'file_0000.BMP', etc.";
        jtrace.debug(errortext);

        QMessageBox::information(this, "Jahshaka", errortext);
        return;
    }

    //////////////////////////////////////////////////////////////////
	// This is a hack to load scene files in
	//currently they are not supported on the desktop...
	//and are only supported in JahPlayer

	#ifdef JAHPLAYER
	if (extension==".jsf" || extension==".jah")
	{
		emit loadJahSceneFile(filename);
		return;
	}
	#endif

	#ifndef JAHPLAYER
	if (extension==".jsf" || extension==".jah")
	{
		//before loading we need to switch to the respective module
		//but to do that we need to read the file header
		//and figure out what it is

		//1. find the module
		QString theModule = findModuleName(filename);

		if (theModule=="NULL")
			return;

		//2. switch to the module
		emit switchMainModule(theModule);

		//3. load the scene file
		emit loadMainModuleFile(filename);
		
		return;
	}
	#endif

    //////////////////////////////////////////////////////////////////

    //filestring is full name of file
    QString filestring(filename);
    int index = filestring.findRev(extension);
    filestring.truncate(index); //total path without .bmp extension
    //debug("filestring is:"+filestring);

    QString	shortName(filestring);
    int slash = shortName.findRev('/');
    if(slash > 0) shortName.remove(0, slash+1);
    //debug("shortname is:"+shortName);

    QString	pathString(filename);
    int ras = pathString.length() - (shortName.length()+extension.length() );
    pathString.truncate(ras);
    //debug("pathstring is:"+pathString);


    //////////////////////////////////////////////////
    // at this point, filestring and extension are ok
    //if its a media or audio clip all we need to do is
    //extract the clip name from filestring and send it over to the loader
    //debug("checking extensions");

    if (extension == ".wav" || extension==".aif" || extension==".mp3" || extension==".ogg")
    {
        jtrace.debug("its a audio file");
        loadJahAudioAsset(filename,shortName,pathString,extension);
        return;
    }

    if (extension == ".avi" || extension == ".mpg" || extension == ".mpeg")
    {
        jtrace.debug("its a media file");
        loadJahMediaAsset(filename,shortName,pathString,extension);
        return;
    }

    //////////////////////////////////////////////////////////////////
    // if its not a recognized media file and its a image file we need
    // to find out if its a still or a sequence

    /////////////////////////////////////////////////
    // prolem we are having is we see any filename that ends in a number
    //as a potential sequence...
    //////////////////////////////////////////////////////////////////


    //debug("Checking for padding for sequences");
    int		maxFrames = 99999; //maximum number of frames allowed in sequences due to current extension filename limitations
    int		i;
    bool	ok;

    //QImage  image;
    //QPixmap buffer;
    QString	stillName(filestring);
    slash = stillName.findRev('/');
    if(slash > 0) stillName.remove(0, slash+1);
    //debug("stillname is:"+stillName);

    QString	baseString(filestring);
    index = filestring.length();

    //////////////////////////////////////////////////
    //figure out how many zeros...
    int extcount=0;
    QString tempstring(filestring);
    QString extchar;

    for (int ii=index; ii>=0; ii--)
    {
        extchar = tempstring.right(1);

        if (isanumber(extchar))
        {
            tempstring.truncate( tempstring.length()-1 );
            extcount++;
            //debug(tempstring);
        }

    }

    index -= extcount;					//remove number of zeros in  sequence
    baseString.truncate(index);			//basestring is now total path without numbers or extension
    filestring.remove(0, index);

    QString theextension (filestring);	//set theextension value, later to be videoext
    //debug("videoext theextension is:"+theextension);
    //debug("baseString is:"+baseString);
    //debug("filestring is:"+filestring);


    jtrace.debug (">>> now we check to see if its a sequence");
    int	numFrames	= 0;
    int startFrame = filestring.toInt(&ok); //should this be theextension?

    if (!ok)
    {
        //we know for sure its not a image here
        //as the filename is clean ie no numbers
        //jtrace.debug("its a image");
        loadJahImageAsset(filename,shortName,pathString,extension);
        return;
    }
    else
        jtrace.debug("it could be a sequence");

    QString testnamestring;

    //we now assume the sequence data is correct?
    for (i = startFrame; i < (int)startFrame+maxFrames; i++)
    {
        //QString pad; pad = "%s%04d%s";
        QString pad; pad = "%s%"; pad += "0";
        QString extc; extc.setNum(extcount);
        pad+=extc; pad += "d%s";

        testnamestring.sprintf(pad, (const char*)baseString, i, (const char*)extension);

        //we dont open the file any more just make sure the sequence is there...
        //we can skip this for clips already loaded in the database
        if (!( QFile::exists(testnamestring) ))
        {
            if (i == 0)
            {
                warning ("File \'%s\' not found. Sequence aborted.", (const char*)testnamestring);
                return;
            }

            //we found the last frame
            maxFrames = i-startFrame;
            break;
        }
    }

    QString rere;

    int numberFrames =  startFrame + maxFrames;
    //jtrace.debug (">>> numberFrames",numberFrames);
    //jtrace.debug (">>> startFrame",startFrame);
    //debug ("num of frames is"+rere.setNum(numberFrames-startFrame));

    //debug("basestring:"+baseString);
    QString	newstillName(baseString);
    slash = newstillName.findRev('/');
    if(slash > 0) newstillName.remove(0, slash+1);
    //debug("newstillname is:"+newstillName);


    if ((numberFrames-startFrame) == 1 )
    {
        jtrace.debug("false alarm, its a image with numbers in the name");
        loadJahImageAsset(filename,shortName,pathString,extension);
        return;

    }
    else
    {
        jtrace.debug("its a sequence"); //filestring was
        loadJahSequenceAsset(filename,newstillName,pathString,extension,theextension, startFrame,numberFrames);
        return;

    }

}

///////////////////////////////////////////////////////////
// routine to clear item from database

void JahDesktop::clearItem()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Clearing Item");

    ///////////////////////////////////////////////////////////////
    //first we need to make sure that something is selected!
    //trying to erase a null clip causes a crash
    bool clipstatus = false;
    videoData erase = getVideoData(clipstatus);

    if (clipstatus)
    {
        //first we remove the clip from the desktop

        //in order to clear the item from the desktop
        //we need to make usre its selected first!
        //since it could be selected in the table

        bool theeraser;
        theeraser = editor->eraseItem(erase.clipnumber);

        if (theeraser)
        {
        //editor->clearItem(); //should return a true for debugging if successful before moving ahead

        //then we remove it from the table

        //then we grab the clips information for the erase procedure
        //erase =  editor->selectedClip;
        //videoData  erase =  editor->selectedClip;

        //we needed to know what its index was to remove it from the table
        //int tbitem = erase.clipnumber;
        table->removeItem(  erase.clipnumber );

        //then we remove the image from the database
        //currently its just de-activated as we will be doing purging later on and erasing at that time
        theDatabase->removeImagefromdatabase(erase.clipnumber);
        jtrace.debug("asset removed from database");

        //for safety we dont erase any stock images only images in the renders directory
        //we need to add the option to remove from database if the location is outside of the database
        bool EraseFile = false;

        QString jahrenderpath = JahMediaPath + "media/renders/"; //renders is renderpath as defined in glworld.cpp

        //we only check to make sure they are in the renders directory...
        //if (jahrenderpath.contains(erase.location))
        if (erase.location.contains(jahrenderpath))
        {
            EraseFile = true;
            jtrace.debug("nice location"+erase.location);
        }
        else
        {
            EraseFile = false; 
            jtrace.debug("bad location:"+erase.location);
        }


        //if we are clear to erase the clip we do so now
        if (EraseFile) 
        {
            if (eraseTheAsset(erase))
            {
                jtrace.debug("erased asset");
            }
        }

        }
    }
}


bool JahDesktop::clearTheDesktop() 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    ///////////////////////////////////////////////////////
    //caution using this routine! It wipes the datbase...

    //first we find out how many entries are in the database
    QString data;

    int counter = theDatabase->GetDatabaseClipNumber();
    jtrace.debug("Number of records in database:"+data.setNum(counter));

    //now we iterate through each one and erase the files
    //if they are in the renders directory
    //so we dont erase stuff we are not supposed to

    //we can also clear the desktop and wipe everything using the database...
    //good for corrupt databases
    videoData erase;
    bool EraseFile;

    QString jahrenderpath = JahMediaPath + "media/renders/"; //renders is renderpath as defined in glworld.cpp

    for (int loop=1; loop<=counter; loop++)
    {
        jtrace.debug(">>erasing clip"+data.setNum(loop));
        EraseFile = false;

        //get the clip from the database
        //jtrace.debug("getting clip from database");
        erase = theDatabase->getImageFromDatabase(loop);

        //we only check to make sure they are in the renders directory...
        //if (jahrenderpath.contains(erase.location)) 
        if (erase.location.contains(jahrenderpath)) 
        { 
            EraseFile = true; 
            jtrace.debug("nice location"+erase.location);
        }
        else
        {
            EraseFile = false; 
            jtrace.debug("bad location:"+erase.location);
        }

        //if we are clear to erase the clip we do so now
        if (EraseFile) 
        {
            if (eraseTheAsset(erase))
            {
                jtrace.debug("erased asset");
            }
        }//end of if

    } //finish looping thorugh files


    //jtrace.debug("finished erasing");
    //////////////////////////////////////////////////////////////////
    //now we wipe the database and set global clipnumber back to 0

    bool wipe = theDatabase->WipeDatabase();

    //after wiping all the clips this clears the desktop
    //need to iterate thru desktop objects and erase them that way
    //this crashes system later on
    editor->clear();

    //need to also clear the desktop table...
    table->clearAll();

    //set globalclipnumber to 0
    *clipnumber = 0;

    return wipe;
}

//why arent we using the assetexchange to get the clip names here?
bool JahDesktop::eraseTheAsset(videoData erase)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.info( "Clearing Item");

    bool eraseStatus = false;

    switch(erase.theCategory) {

        case  IMAGE:    { 
                            jtrace.debug("erasing image "+ erase.location + erase.filename + erase.extension);

                            QFile f(erase.location + erase.filename + erase.extension);
                            if ( f.remove() )  
                            { 
                                //jtrace.debug("erase sucessful");
                                eraseStatus = true; 
                            }
                            else 
                            {
                                //jtrace.debug("couldnt erase file");
                            }

                            break;
                        }

        case  CLIP :    { 
                            jtrace.debug("erasing clip " + erase.filename);

                            int length, blanklength;
                            QString currentframe, erasefile;

                            //flag that makes sure
                            bool lasterase = false;
                            bool cleanerase = true;

                            for (int i = erase.startframe; i<=erase.endframe; i++) 
                            {

                                // This is where we create the extension for the sequence

                                //first figure out length of current frame
                                currentframe.setNum(i);
                                length = currentframe.length();  //was theblankframe

                                //then remove it from the blank template for the clip
                                QString blank(erase.videoext);
                                blank = blank.remove(0, length);

                                //now we need to build the sequence template
                                blanklength = blank.length();
                                QString theblank;
                                for (int ii=0; ii<blanklength; ii++) 
                                {
                                    theblank +="0";
                                }

                                blank = theblank+currentframe;

                                //then we build the path to the frame and open the file
                                erasefile = erase.location + erase.filename + blank + erase.extension;
                                //jtrace.debug("erasing file:"+erasefile);

                                QFile f(erasefile);

                                //now some error checking on the erase procedure
                                if ( f.remove() )	
                                { 
                                    //jtrace.debug("erase sucessful"); 
                                    lasterase = true; 
                                }
                                else
                                { 
                                    //jtrace.debug("couldnt erase file"); 
                                    cleanerase = false; 
                                }
                            }

                            if (cleanerase == true && lasterase == true) 
                            { 
                                jtrace.info("erased sequence successfully"); 
                                eraseStatus = true;

                                //now we erase the directory if its a clip
                                QString destDir;
                                destDir             = erase.location;

                                QDir d( destDir );                        // "./example"
                                if ( d.exists() )
                                {
                                    //jtrace.debug( "found asset directory", destDir );
                                    if (d.rmdir( destDir ))
                                    {
                                        jtrace.debug("erased sequence directory",destDir );

                                    }
                                    else
                                    {
                                        jtrace.debug("couldnt erase directory",destDir );

                                    }
                                }

                            }
                            break;
                    }

        default :   { 
                        jtrace.debug("dont know what it is"); 
                        break; 
                    }

        } //end of switch here

    return eraseStatus;
}




void JahDesktop::slotExport()
{

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Exporting Item");

    //then we grab the clips information for the erase procedure
    videoData  theasset =  editor->selectedClip;


    int clipnum = theasset.clipnumber;

    //if its a good number ie >0 continue
    if (clipnum>0)
    {
        QString destination = QFileDialog::getExistingDirectory(
        QDir::homeDirPath (),
        //prefframe2,
        0,
        "Export Asset",
        "Choose a location to export asset to" );

        QDir d( destination );                        // "./example"

        if ( d.exists() )
        {
            assetExchange exchange;
            QString assetname;

            if (exchange.isMediaValid(theasset))
            {
                exchange.ExportAsset(theasset, destination);
            }
        }
        else
        {
            QMessageBox::information(0, "Error",
            "Invalid location:\n\n"+ destination);
        }
    }
}




/////////////////////////////////////////////////
// save doesnt do much right now
// later on it will export clips etc

void JahDesktop::slotSave()
{
    //we need to add conversion utilities!!!
    //ie export a jpg sequence as tga...

    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("saving or exporting asset");

    videoData  exporter =  editor->selectedClip;

    int clipnum = exporter.clipnumber;

    //if its a good number ie >0 continue
    if (clipnum>0) 
    {

        QString filename = QFileDialog::getSaveFileName( JahMediaPath+"/images",  
                                                        "*."+exporter.extension, 
                                                        this );
        if ( !filename.isEmpty() ) 
        {
            exportTheAsset(filename, exporter);
        }

    }

}

bool JahDesktop::exportTheAsset(QString exportto, videoData theasset)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Exporting Asset");
    jtrace.info( "Exporting Asset to"+exportto);

    bool exportStatus = false;

    // categories are IMAGE, KEY, CLIP, CLIPKEY, AUDIO, MEDIA
    switch(theasset.theCategory) {

        case  IMAGE:    { 
                            jtrace.debug("exporting image:"+ theasset.location + theasset.filename + theasset.extension);
                            break;
                        }

    case  MEDIA:    {
                            jtrace.debug("exporting media:"+ theasset.location + theasset.filename + theasset.extension);
                            break;
                        }

        case  CLIP :    { 
                            jtrace.debug("erasing clip " + theasset.location + theasset.filename + theasset.extension);
                            break;
                    }

        default :   { 
                        jtrace.debug("dont know what it is"); 
                        break; 
                    }

        } //end of switch here

    return exportStatus;
}


bool JahDesktop::isanumber(QString & string)
{

    bool ok;

    if (string=="0")	{ ok = true; } else
    if (string=="1")	{ ok = true; } else
    if (string=="2")	{ ok = true; } else
    if (string=="3")	{ ok = true; } else
    if (string=="4")	{ ok = true; } else
    if (string=="5")	{ ok = true; } else
    if (string=="6")	{ ok = true; } else
    if (string=="7")	{ ok = true; } else
    if (string=="8")	{ ok = true; } else
    if (string=="9")	{ ok = true; } else
                        { ok = false;}

    return ok;

}





