/*******************************************************************************
**
** The source file for the Jahshaka desktop io file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "desktop.h"

#include "jahfilewindow.h"
#include "jahfileiconview.h"
#include "jahfileiconview.h"



///////////////////////////////////////////////////////
// asset loader routines for the desktop

QString JahDesktop::findModuleName(QString loadfilename)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString text;

    QFile f( loadfilename );
    if ( !f.open( IO_ReadOnly ) ) 
    {
        QMessageBox::information(0, "Could not read file  ", loadfilename);
        return "NULL";
    }

    QTextStream t( &f );

    ////////////////////////////////////////////////////
    //read the header and parse the file
    text = t.readLine(); 
    text = t.readLine(); 
    text = t.readLine(); 
    text = t.readLine(); 

    ///////////////////////////////////////////////////
    text =  t.readLine() ;   //module name is here

	int jitindex,jitlen;
	jitindex	= text.findRev(":");   jitindex +=1;     //find the colon and skip over it
	jitlen = text.length();    
	text = text.right(jitlen - jitindex) ;


    jtrace.debug("The module is:",text);
    
	f.close();
	return text;
}

void JahDesktop::loadJahMediaAsset(QString filename,QString shortName,QString pathString,QString extension)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug ("Loading Media File:", filename);

    //necessary variables
    videoData thedata;

    //update desktop clip number
    int clipn; clipn = *clipnumber;
    clipn++;
    *clipnumber = clipn;

    QString temp;

    thedata.theCategory = MEDIA; //was clip
    thedata.theType     = MOVIE; // no?

    //figure out what this is and make it dynamic
    thedata.videoext    = "";

    thedata.clipname    = shortName;
    thedata.clipnumber  = clipn;
    thedata.location    = pathString;
    thedata.filename    = shortName;
    thedata.extension   = extension;
    thedata.videotex   = true;

    //thedata.keyStatus   = FALSE;
    thedata.xpos        = 10;
    thedata.ypos        = 10;


    //run as many checks as possible before using the media
    if ( !filename.isEmpty() )
    {
        if ( !no_writing )
        {
            assetExchange exchange;
            if (exchange.isMediaValid(thedata))
            {
                //number of frame sis based on file info being valid
                int startFrame = 0;
                int maxFrames;
                exchange.getMediaLength(thedata,maxFrames);    // The Last Frame Of The Stream

                thedata.startframe  = startFrame;
                int numberFrames    =  startFrame + maxFrames;
                thedata.endframe    = numberFrames; //minus startframe
                //debug("numberFrames  :" +data1.setNum(numberFrames) );

                //now add it to the system
                addItemToDesktopTable( clipn-1, thedata );
                addImage2(thedata);
                theDatabase->addImage2database( thedata );
            }
        }
    }

    canvas->update();

    return;

}

void JahDesktop::loadJahAudioAsset(QString filename,QString shortName,QString pathString,QString extension)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug ("Loading Audio File:", filename);

    //necessary variables
    videoData thedata;

    //update desktop clip number
    int clipn; clipn = *clipnumber;
    clipn++;
    *clipnumber = clipn;

    QString atemp;

    thedata.theCategory = AUDIO;
    thedata.theType     = AIFF; // no?

    //figure out what this is and make it dynamic
    thedata.videoext    = "";
    thedata.clipname    = shortName;
    thedata.clipnumber  = clipn;
    thedata.location    = pathString;
    thedata.filename    = shortName;
    thedata.extension   = extension;
    thedata.videotex    = false;

    //thedata.keyStatus   = FALSE;
    thedata.xpos        = 10;
    thedata.ypos        = 10;

    //run as many checks as possible before using the media
    if ( !filename.isEmpty() )
    {
        if ( !no_writing )
        {
            //assethack
            assetExchange exchange;
            if (exchange.isMediaValid(thedata))
            {
                //number of frame sis based on file info being valid
                int startFrame = 1;

                int maxFrames;

                //how do we figurew thisout
                //need to add it to the media exchange
                exchange.getMediaLength(thedata,maxFrames);    // The Last Frame Of The Stream
                //thedata.getMediaLength(maxFrames);    // The Last Frame Of The Stream
                maxFrames = 1000;

                thedata.startframe  = startFrame;
                int numberFrames    =  startFrame + maxFrames;
                thedata.endframe    = numberFrames; //minus startframe

                //debug("numberFrames  :" +data1.setNum(numberFrames) );

                //now add it to the system
                addItemToDesktopTable( clipn-1, thedata );
                addImage2(thedata);
                theDatabase->addImage2database( thedata );
            }
        }
    }

    canvas->update();

    return;

}


void JahDesktop::loadJahImageAsset(QString filename,QString shortName,QString pathString,QString extension)
{


    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug ("Loading Still File:", filename);

    //necessary variables
    videoData thedata;

    //increase desktop clipnumber
    int clipn; clipn = *clipnumber;
    clipn++;
    *clipnumber = clipn;

    QString temp;

    thedata.theCategory = IMAGE;
    thedata.theType     = STILL;

    thedata.clipname    = shortName;    //clipname is truncated filename
    thedata.clipnumber  = clipn;
    thedata.location    = pathString;    //the path without the filename...
    thedata.filename    = shortName;     //the filename on disk without extension
    thedata.extension   = extension; //we need to change this based on extension
    thedata.startframe  = 1;
    thedata.endframe    = 1;
    //thedata.keyStatus   = FALSE;
    thedata.xpos        = 10;
    thedata.ypos        = 10;
    thedata.videotex   = false;

    if ( !filename.isEmpty() )
{
        if ( !no_writing ) 
    {       //thedata.extension needs to be set properly here according to extension
            addItemToDesktopTable( clipn-1, thedata );
            addImage2(thedata);
            theDatabase->addImage2database( thedata );
        }
    }

    canvas->update();
    return;


}


void JahDesktop::loadJahSequenceAsset(QString filename,QString shortName,QString pathString,QString extension,
                                      QString theextension, int startFrame, int numberFrames)
{

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug ("Loading Sequence File:", filename);

    jtrace.debug ("Loading filename :", filename);
    jtrace.debug ("Loading shortName :", shortName);
     jtrace.debug ("Loading pathString :", pathString);
    jtrace.debug ("Loading extension :", extension);
    jtrace.debug ("Loading theextension :", theextension);


    //necessary variables
    videoData thedata;

    //update desktop clip number
    int clipn; clipn = *clipnumber;
    clipn++;
    *clipnumber = clipn;

    QString temp;

    thedata.theCategory = CLIP;
    thedata.theType     = SEQUENCE;

    //figure out what this is and make it dynamic
    thedata.videoext    = theextension;

    thedata.clipname    = shortName;
    thedata.clipnumber  = clipn;
    thedata.location    = pathString;
    thedata.filename    = shortName;
    thedata.extension   = extension;
    thedata.startframe  = startFrame;
    thedata.endframe    = numberFrames-1; //minus startframe
    //thedata.keyStatus   = FALSE;
    thedata.xpos        = 10;
    thedata.ypos        = 10;
    thedata.videotex   = true;

    if ( !filename.isEmpty() )
    {

        if ( !no_writing )
        {

            addItemToDesktopTable( clipn-1, thedata );
            addImage2(thedata);
            theDatabase->addImage2database( thedata );
        }
    }

    canvas->update();
    return;

}

