/*******************************************************************************
**
** The source file for the Jahshaka desktop canvas file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "desktoptable.h"

JahDesktopTable::JahDesktopTable( int rows, int cols, QWidget* parent, const char* name) :
 QTable(parent,name)
 {

    //table = new QTable( numRows, numCols, thedesktopdisplay2 );
    setFocusStyle(QTable::FollowStyle);
    setSelectionMode(QTable::SingleRow);

    QHeader *header = horizontalHeader();
        header->setLabel( 0,  QObject::tr( "Icon" ),      60  );
        header->setLabel( 1,  QObject::tr( "ClipName" ),  120 );
        header->setLabel( 2,  QObject::tr( "Videx" ),     40  );
        header->setLabel( 3,  QObject::tr( "Num" ),       40  );
        header->setLabel( 4,  QObject::tr( "Location" ),  360 );
        header->setLabel( 5,  QObject::tr( "Filename" ),  120 );
        header->setLabel( 6,  QObject::tr( "Ext" ),       40  );
        header->setLabel( 7,  QObject::tr( "SF" ),        40  );
        header->setLabel( 8,  QObject::tr( "EF" ),        40  );
        header->setLabel( 9, QObject::tr( "Key" ),       60  );
        header->setLabel( 10,  QObject::tr( "id" ),        40  );
    header->setMovingEnabled(FALSE);

    setLeftMargin(0);
    verticalHeader()->hide();

    jahnumRows = 0;
    
	//setNumRows(jahnumRows);
	setNumCols(cols);

    show();

    mySecret = 0;

    //setRowMovingEnabled(TRUE);
    setDragEnabled(TRUE);

}


void JahDesktopTable::addAsset(int index, int globaltablenumber, videoData thedata)
{
	//debug("adding asset to table");

	//////////////////////////////////////////
	//need to add the entireline here

	insertRows(jahnumRows,1);
    setRowHeight ( jahnumRows, 40 );
	
	setColumnWidth ( 0,    60  );
    setColumnWidth ( 1,    120 );
    setColumnWidth ( 2,    0  );
    setColumnWidth ( 3,    40  );
    setColumnWidth ( 4,    360 );
    setColumnWidth ( 5,    120 );
    setColumnWidth ( 6,    40  );
    setColumnWidth ( 7,    60  ); //SF
    setColumnWidth ( 8,    60  ); //EF
    setColumnWidth ( 9,    0  );
    setColumnWidth ( 10,   40  );


//    debug("added row");

	///////////////////////////////////////////

    assetExchange exchange;
    QImage theimage;
    theimage = exchange.getImage(thedata);

    //used to scale image down to size
    float wratio = (60/(float)theimage.width() );
    float hratio = (40/(float)theimage.height() );

    QPixmap pix = theimage.smoothScale( int(theimage.width()*wratio), int(theimage.height()*hratio) );


    //used for num to text conversion as table dopesnt seem to hold integers
    QString conv;

    setPixmap( jahnumRows, 0 , pix );
    setText  ( jahnumRows, 1 , thedata.clipname );
    setText  ( jahnumRows, 2 , thedata.videoext );
    setText  ( jahnumRows, 3 , conv.setNum(thedata.clipnumber) );
    setText  ( jahnumRows, 4 , thedata.location );
    setText  ( jahnumRows, 5 , thedata.filename );
    setText  ( jahnumRows, 6 , thedata.extension );
    setText  ( jahnumRows, 7 , getTimecode(thedata.startframe) );
    setText  ( jahnumRows, 8 , getTimecode(thedata.endframe) );
    setText  ( jahnumRows, 10 , conv.setNum(index) );

	setRowReadOnly(jahnumRows, true);

    update();
     
	jahnumRows +=1 ;

	//debug("added asset");
}

QString JahDesktopTable::getTimecode(int num)
{
	int hh=0; int mm=0; int ss=0; int ms=0;

	for (int x=1; x<num; x++) 
	{
		ms+=1;
		if (ms==30) { ms=0; ss+=1; }
		if (ss==60) { ss=0; mm+=1; }
		if (mm==60) { mm=0; hh+=1; }
	}

	QString s;
	s.sprintf( "%02d:%02d:%02d", mm, ss,ms );

	return s;
}

QDragObject * JahDesktopTable::dragObject()
{
    //debug("dragging");

    QImage image;
    QPixmap pix;

    pix = pixmap (currentRow(),0);
    image = pix;

    return new QImageDrag(image,this);

}


bool JahDesktopTable::removeItem(int theitem)
{
	//QString rere;
	//debug("--------------------------------");
	//debug("calling remove for item id"+rere.setNum(theitem));

    bool itworked = false;

    //theitem -=1;

    //we need to scan all rows
    //and find out which row contains theitem in col 10
    int therow;
    QString thetext;
    int theval;

    //should be start row to end row
    for (int x=0; x<=jahnumRows; x++)
    {
        //thetext = text(x,10);
        thetext = text(x,3);
        theval = thetext.toInt();

        if (theval == theitem)
        {
            therow = x;
            itworked = true;
        }
    }

    if (!itworked)
    {
        //debug("it didnt work");
    }
    else
    {
		removeRow(therow);
		jahnumRows -=1;
    }

    return itworked;
}

bool JahDesktopTable::clearAll(void)
{
	//debug("clearing all");

	for ( int row = 0; row < numRows(); row++ ) 
	{
		removeRow(row);
    }
	
	jahnumRows = 0;

	//for some reason removeRow doesnt remove the ccurrent selected row?
	//so we do it here

//	int sel = currentSelection (); 
	int sel = currentRow (); 
		removeRow(sel);

	show();

	return true;
}

