/*******************************************************************************
**
** The source file for the Jahshaka core avi file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include <RenderAvi.h>


CAVIGen::CAVIGen(videoData theasset, RenderConfig theconfig, QString &feedback)
{

    QString convert;
    DestDirectory=theconfig.DestDirectory;        //debug("Dest Dir   :"+DestDirectory); 
    DestFilename=theconfig.DestFilename;          //debug("Dest Name  :"+DestFilename);  
    SourceFilename=theconfig.SourceFilename;      //debug("Source Name:"+SourceFilename);  
    SourceDirectory=theconfig.SourceDirectory;    //debug("Source Dir :"+SourceDirectory);   
    SourceExtension=theconfig.SourceExtension;    //debug("Source Ext :"+SourceExtension);

    startframe = theconfig.mpegFirstFrame;        //debug("start      :"+convert.setNum(startframe)); 
    endframe=theconfig.mpegNumberFrames;          //debug("numframes  :"+convert.setNum(endframe)); 

	destwidth = theconfig.mpegXres;
	destheight = theconfig.mpegYres;
/*
    configData->conversionratio =      1;
    configData->mpegAspectRatio=       8;
    configData->mpegFrameRate=         3;
    configData->mpegBitRate=           2000000.0;
*/

    feedback =">>Starting the encoder";
    RenderAviSequence(feedback,theasset);
    feedback =">>Finished Encoding";

}

CAVIGen::~CAVIGen()
{
}

//problem is the encoder currently only handles bmp files
BOOL CAVIGen::RenderAviSequence(QString & feedback,videoData &theasset)
{
    QString returnstring, convert;
    assetExchange exchange;
    QImage theimage;

    QString renname = DestDirectory+DestFilename;
	
	debug("Encoding to:"+renname);

    HAVI avi = CreateAvi(renname,10,NULL); // 30ms is the period between frames
    
    QString tmpString;

    for (int i= startframe; i<= endframe; i++)
    {
		//get the file name into tmpstring
		exchange.getMediaFilename(theasset, i, tmpString);
		debug(tmpString);


		//convert to a bmp if its not a bmp
		//if we were to convert everything we could do image resizing as well...
		//and support cross media encoding ie avi to avi and mpeg to avi
		QString extension = theasset.extension.lower();
		QImage sourceimage = exchange.getImage(theasset, i);

		//we need to scale the image here
		QImage newimage = sourceimage.scale(destwidth,destheight,QImage::ScaleFree); //(ScaleMin and ScaleMax are options)

		//then we write it out to disk as a bmp for the encoder to use
		tmpString = tmpString.left(tmpString.length()-4);
		tmpString = tmpString + "TMPENC.bmp";

		QImageIO iio; 
		iio.setImage(newimage);
		iio.setFileName(tmpString);
		iio.setFormat("BMP");
		iio.write();

        //for this to work right we need to convert the frames on the fly to bmp's
		//would be great to just pass the QImage over
		//or convert the QImage to a hbm and pass it over instead of loading it
        HBITMAP hbm=(HBITMAP)LoadImage(NULL,tmpString,IMAGE_BITMAP,0,0,LR_LOADFROMFILE|LR_CREATEDIBSECTION);

		//////////////////////////////////////////////////////////////////
		//set the compression options here after hbm is initialized?
		
        if (i==0) // Set up compression just before the first frame
        {   
            AVICOMPRESSOPTIONS opts; 
            ZeroMemory(&opts,sizeof(opts));
            opts.fccHandler=mmioFOURCC('d','i','v','x');
            SetAviVideoCompression(avi,hbm,&opts,true,0);
        }
	
		/////////////////////////////////////////////////////////////////
		//feedback 
        returnstring = "PROCESSING frame number :" + convert.setNum(i);
        feedback     = returnstring;

		/////////////////////////////////////////////////////////////////
		//add the image to the avi
        AddAviFrame(avi,hbm);
        DeleteObject(hbm);

        //we need to erase the image file tmpString
        QFile f(tmpString);
        f.remove();	

    }

	/////////////////////////////////////////////////////////
	//wecan add audio the avi here if we want to...
    //AddAviWav(avi,"wav.wav",SND_FILENAME);

	/////////////////////////////////////////////////////////
	//close out and clean up
    CloseAvi(avi);

    return TRUE;
}


BOOL CAVIGen::AddFrame(BYTE *bmBits)
{
 char sError[2048];
 HRESULT hr;

 // compress bitmap
 hr = AVIStreamWrite(
  m_pStreamCompressed, // stream pointer
  m_lFrame,    // time of this frame
  1,      // number to write
  bmBits,     // image buffer
  m_bih.biSizeImage,  // size of this frame
  AVIIF_KEYFRAME,   // flags....
  NULL,
  NULL);

 // updating frame counter
 m_lFrame++;

 if (hr != AVIERR_OK)
 {
  strcpy( sError, "Unable to write data to stream.");
  return FALSE;
 }

 return TRUE;
}



BOOL CAVIGen::InitEngine()
{

 AVISTREAMINFO strHdr; // information for a single stream
 AVICOMPRESSOPTIONS opts;
 AVICOMPRESSOPTIONS FAR * aopts[1] = {&opts};

 char sError[2048];

 HRESULT hr;

 // Step 0 : Let's make sure we are running on 1.1
 DWORD wVer = HIWORD(VideoForWindowsVersion());
 if (wVer < 0x010a)
 {
   // oops, we are too old, blow out of here
  strcpy( sError, "Version of Video for Windows too old. Come on, join the 21th century!");
  return FALSE;
 }

 // Step 1 : initialize AVI engine
 AVIFileInit();

 // Step 2 : Open the movie file for writing....
 hr = AVIFileOpen(&m_pAVIFile,        // Address to contain the new file interface pointer
  (LPCSTR)m_sFile,    // Null-terminated string containing the name of the file to open
         OF_WRITE | OF_CREATE,        // Access mode to use when opening thefile.
         NULL);                       // use handler determined from file extension.
                                      // Name your file .avi -> very important

 if (hr != AVIERR_OK)
 {
  strcpy( sError, "AVI Engine failed to initialize.");
  return FALSE;
 }

 // Fill in the header for the video stream....
 memset(&strHdr, 0, sizeof(strHdr));
 strHdr.fccType                = streamtypeVIDEO; // video stream type
 strHdr.fccHandler             = 0;
 strHdr.dwScale                = 1;     // should be one for video
 strHdr.dwRate                 = m_dwRate;      // fps
 strHdr.dwSuggestedBufferSize  = m_bih.biSizeImage; // Recommended buffer size, in bytes, for the stream.
 SetRect(&strHdr.rcFrame, 0, 0,      // rectangle for stream
     (int) m_bih.biWidth,
     (int) m_bih.biHeight);

 // Step 3 : Create the stream;
 hr = AVIFileCreateStream(m_pAVIFile,      // file pointer
            &m_pStream,      // returned stream pointer
            &strHdr);     // stream header

 // Check it succeded.
 if (hr != AVIERR_OK)
 {
  strcpy( sError, "AVI Stream creation failed. Check Bitmap info.");
  return FALSE;
 }


 // Step 4: Get codec and infos about codec
 memset(&opts, 0, sizeof(opts));
 // Poping codec dialog
 if (!AVISaveOptions(NULL, 0, 1, &m_pStream, (LPAVICOMPRESSOPTIONS FAR *)
&aopts))
 {
  AVISaveOptionsFree(1,(LPAVICOMPRESSOPTIONS FAR *) &aopts);
  return FALSE;
 }

 // Step 5:  Create a compressed stream using codec options.
 hr = AVIMakeCompressedStream(&m_pStreamCompressed,
    m_pStream,
    &opts,
    NULL);

 if (hr != AVIERR_OK)
 {
  strcpy( sError, "AVI Compressed Stream creation failed.");
  return FALSE;
 }

 // releasing memory allocated by AVISaveOptionFree
 hr=AVISaveOptionsFree(1,(LPAVICOMPRESSOPTIONS FAR *) &aopts);
 if (hr!=AVIERR_OK)
 {
  strcpy( sError, "Error releasing memory");
  return FALSE;
 }

 // Step 6 : sets the format of a stream at the specified position
 hr = AVIStreamSetFormat(m_pStreamCompressed,
     0,   // position
     &m_bih,     // stream format
     m_bih.biSize +   // format size
     m_bih.biClrUsed * sizeof(RGBQUAD));

 if (hr != AVIERR_OK)
 {
  strcpy( sError, "AVI Compressed Stream format setting failed.");
  return FALSE;
 }

 // Step 6 : Initialize step counter
 m_lFrame=0;

 return TRUE;
}

BOOL CAVIGen::ReleaseEngine()
{
 if (m_pStream)
 {
  AVIStreamRelease(m_pStream);
  m_pStream=NULL;
 }

 if (m_pStreamCompressed)
 {
  AVIStreamRelease(m_pStreamCompressed);
  m_pStreamCompressed=NULL;
 }

 if (m_pAVIFile)
 {
  AVIFileRelease(m_pAVIFile);
  m_pAVIFile=NULL;
 }

 // Close engine
 AVIFileExit();

 return TRUE;
}



/*
To create an AVI from a bunch of files. The following code creates an AVI out of five bitmaps (1.bmp, 2.bmp, ... 5.bmp) and a WAV file (wav.wav).

  //http://www.wischik.com/lu/programmer/avi_utils.html

const char *fns[] = {"1.bmp","2.bmp","3.bmp","4.bmp","5.bmp"};
HAVI avi = CreateAvi("test.avi",1000,NULL); // 1000ms is the period between frames
for (int i=0; i<sizeof(fns)/sizeof(fns[0]); i++)
{ HBITMAP hbm=(HBITMAP)LoadImage(NULL,fns[i],IMAGE_BITMAP,0,0,LR_LOADFROMFILE|LR_CREATEDIBSECTION);
  AddAviFrame(avi,hbm);
  DeleteObject(hbm);
}
AddAviWav(avi,"wav.wav",SND_FILENAME);
CloseAvi(avi);

*/

  //for avi generator on wondows
  //CAVIGen* AviGen;
/*
BOOL MainWindow::renderAviScene()
{
    debug(">>initializing avi engine");
 InitRenderToAvi("C:/jahtest.avi");
 //timer->start( 20 );
 //m_flying = true;

 //RenderToAvi();

 //EndRenderToAvi();
     debug(">>rendering avi");
     AviGen->RenderAviSequence();
     debug(">>finished rendering avi");

 return TRUE;
}

BOOL MainWindow::InitRenderToAvi(const char * szFile )
{
 //double v_width  = width();
 //double v_height = height();
 double v_width  = 720;
 double v_height = 486;
 //AviGen = new CAVIGen( szFile, 40, v_width, v_height );
 AviGen = new CAVIGen( "C:/jahtest.avi", 40, v_width, v_height );
 AviGen->InitEngine();
 return TRUE;
}

BOOL MainWindow::RenderToAvi()
{
 // Alloc pixel bytes
 BYTE *pPixelData = new BYTE[AviGen->m_bih.biSizeImage];
 // Make GL Context current
 //makeCurrent();

 // Copy from OpenGL or grab image...
 //glReadPixels(0,0,AviGen->m_bih.biWidth,AviGen->m_bih.biHeight,GL_BGR_EXT,GL_
 //             UNSIGNED_BYTE,pPixelData);

 // send to avi engine
 AviGen->AddFrame(pPixelData);

 // Cleaning memory
 delete [] pPixelData;
 return TRUE;
}

BOOL MainWindow::EndRenderToAvi()
{
 // releasing engine and memory
 AviGen->ReleaseEngine();
 return TRUE;
}
*/


//dont know why this code is here?
/*
 strcpy( m_sFile, sFileName );
 m_dwRate=dwRate;

 double m_width  = win_width;
 double m_height = win_height;

 m_width=(m_width/4)*4;
 m_height=(m_height/4)*4;

 // initialize m_bih
 memset(&m_bih,0, sizeof(BITMAPINFOHEADER));
 // filling bitmap info structure.
 m_bih.biSize=sizeof(BITMAPINFOHEADER);
 m_bih.biWidth=(long)m_width;
 m_bih.biHeight=(long)m_height;
 m_bih.biPlanes=1;
 m_bih.biBitCount=24;
// m_bih.biSizeImage= 3* m_bih.biWidth * m_bih.biHeight;
 m_bih.biSizeImage= ((m_bih.biWidth*m_bih.biBitCount+31)/32 *4)*m_bih.biHeight;
 m_bih.biCompression=BI_RGB;  //BI_RGB means BRG in reality

 m_pAVIFile=NULL;
 m_pStream=NULL;
 m_pStreamCompressed=NULL;
 */
