/* mpeg2enc.c, main() and parameter file reading                            */

/* Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define GLOBAL /* used by global.h */
#include "config.h"
#include "global.h"

/* private prototypes */
static void init _ANSI_ARGS_((void));
static void readparmfile _ANSI_ARGS_((char *fname));
static void readquantmat _ANSI_ARGS_((void));
static void readconfigobject _ANSI_ARGS_((RenderConfig theconfig));



int mpeglauncher(RenderConfig theconfig)
{
    //declare all variables at the top of the routine
    char outputfile_name[256];
    
    mpegenc_feedback =">>Starting the encoder";

    /* read configuration object */ 
    readconfigobject(theconfig);

    /* read quantization matrices */
    readquantmat();

    /* ropen file for output */
    mpegenc_feedback = ">>opening output file";

    sprintf(outputfile_name, "%s%s", theconfig.DestDirectory, theconfig.DestFilename); 
 
    printf(outputfile_name);
    mpegenc_feedback = outputfile_name;
    printf(">>opening output file %s\n", outputfile_name);

    //we check for errors here and return if so...
    if (!(outfile = fopen(outputfile_name,"wb")))
    {
        sprintf(errortext,"Couldn't create output file %s", outputfile_name);
        error(errortext);
        return 1;
    }

    mpegenc_feedback=">>initializing";
    init();

    mpegenc_feedback = ">>start encoding sequence";
    putseq();
    mpegenc_feedback = ">>finished encoding sequence";
    
    fclose(outfile);

    if (statname[0] != '-')
    {
        fclose(statfile);
    }

    return 0;
}



static void init()
{
    int i, size;
    static int block_count_tab[3] = {6,8,12};

    initbits();
    init_fdct();
    init_idct();

    /* round picture dimensions to nearest multiple of 16 or 32 */
    mb_width = (horizontal_size+15)/16;
    mb_height = prog_seq ? (vertical_size+15)/16 : 2*((vertical_size+31)/32);
    mb_height2 = fieldpic ? mb_height>>1 : mb_height; /* for field pictures */
    width = 16*mb_width;
    height = 16*mb_height;

    chrom_width = (chroma_format==CHROMA444) ? width : width>>1;
    chrom_height = (chroma_format!=CHROMA420) ? height : height>>1;

    height2 = fieldpic ? height>>1 : height;
    width2 = fieldpic ? width<<1 : width;
    chrom_width2 = fieldpic ? chrom_width<<1 : chrom_width;
  
    block_count = block_count_tab[chroma_format-1];

    /* clip table */
    if (!(clp = (unsigned char *)malloc(1024)))
        error("malloc failed\n");
    clp+= 384;
    for (i=-384; i<640; i++)
        clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

   for (i=0; i<3; i++)
   {
      size = (i==0) ? width*height : chrom_width*chrom_height;

      if (!(newrefframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(oldrefframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(auxframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(neworgframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(oldorgframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(auxorgframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
      if (!(predframe[i] = (unsigned char *)malloc(size)))
         error("malloc failed\n");
   }

   mbinfo = (struct mbinfo *)malloc(mb_width*mb_height2*sizeof(struct mbinfo));

   if (!mbinfo)
      error("malloc failed\n");

   blocks =
      (short (*)[64])malloc(mb_width*mb_height2*block_count*sizeof(short [64]));

   if (!blocks)
      error("malloc failed\n");

   /* open statistics output file */
   if (statname[0]=='-')
      statfile = stdout;
   else if (!(statfile = fopen(statname,"w")))
   {
      sprintf(errortext,"Couldn't create statistics output file %s",statname);
      error(errortext);
   }
}

void error(text)
   char *text;
{
   fprintf(stderr,text);
   putc('\n',stderr);
   //exit(1);
}

static void readconfigobject( theconfig)
RenderConfig theconfig;
{
   
   int i;
   int h,m,s,f;
   static double ratetab[8]=
      {24000.0/1001.0,24.0,25.0,30000.0/1001.0,30.0,50.0,60000.0/1001.0,60.0};
   extern int r,Xi,Xb,Xp,d0i,d0p,d0b; /* rate control */
   extern double avg_act; /* rate control */
   int padlen; 
   
   ///////////////////////////////////////////////////////////////
   //now we start reading the object data in
   //here tplorg should be equal to the directory plus source filename
   //plus the %d
   //but we also need to send over the sequence padding flag called padding
   
   //EXTERN char id_string[256], tplorgdir[256], tplorgname[256], tplorg[256], tplref[256];

   //we send over filname and directory
   //maybe we should also send over file type extension for conversions?
   sscanf(theconfig.SourceFilename,"%s",&tplorgname); 
   sprintf(errortext,"tplorgname %s",tplorgname); 
   error(errortext);

   sprintf(tplorgdir, "%s", theconfig.SourceDirectory);
   //sscanf(theconfig.SourceDirectory,"%s",&tplorgdir); 
   sprintf(errortext,"tplorgdir %s",tplorgdir); 
   error(errortext);

   sscanf(theconfig.SourceExtension,"%s",&tplorgext); 
   sprintf(errortext,"tplorgext %s",tplorgext); 
   error(errortext);
   
   
   sscanf(theconfig.FramePadding,"%s",&frame_padding);     
   sprintf(errortext,"frame_padding %s",frame_padding); 
   error(errortext);
   
   padlen = strlen(frame_padding);
   if (padlen<2) padlen = 0;  
   framepad = padlen;              
   sprintf(errortext,"framepad %d",framepad); 
   error(errortext);

   //create this cuz its necessary
   sprintf(tplorg,"%s%s",tplorgdir,tplorgname);
   
   sscanf(theconfig.mpeg_tplref,"%s",&tplref);     
   sprintf(errortext,"tplref %s",tplref); 
   error(errortext);              
   sscanf(theconfig.mpeg_iqname,"%s",&iqname);     
   sprintf(errortext,"iqname %s",iqname); 
   error(errortext);        
   sscanf(theconfig.mpeg_niqname,"%s",&niqname);     
   sprintf(errortext,"niqname %s",niqname); 
   error(errortext);          
   sscanf(theconfig.mpeg_statname,"%s",&statname);     
   sprintf(errortext,"statname %s",statname); error(errortext);
   
   inputtype = theconfig.mpegPicFormat;            
   sprintf(errortext,"inputtype %d",inputtype); 
   error(errortext);
   nframes = theconfig.mpegNumberFrames;           
   sprintf(errortext,"nframes %d",nframes); 
   error(errortext);
   frame0 = theconfig.mpegFirstFrame;              
   sprintf(errortext,"frame0 %d",frame0); 
   error(errortext);
   conversionratio = (int)theconfig.conversionratio;    
   sprintf(errortext,"frame0 %d",frame0); 
   error(errortext);
   
   sscanf(theconfig.mpegStartTimecode,"%d:%d:%d:%d",&h,&m,&s,&f);
   sprintf(errortext,"timecode is %d:%d:%d:%d",h,m,s,f); error(errortext);
   
   N = theconfig.mpegGopFrames;                    
   sprintf(errortext,"N %d",N); 
   error(errortext);
   M = theconfig.mpegIPframeDist;                  
   sprintf(errortext,"M %d",M); 
   error(errortext);
   mpeg1 = theconfig.mpegISOstream;                
   sprintf(errortext,"mpeg1 %d",mpeg1); 
   error(errortext);
   fieldpic = theconfig.mpegFieldFrameSwitch;      
   sprintf(errortext,"fieldpic %d",fieldpic); 
   error(errortext);
   horizontal_size = theconfig.mpegXres;           
   sprintf(errortext,"horizontal_size %d",horizontal_size); 
   error(errortext);
   vertical_size = theconfig.mpegYres;             
   sprintf(errortext,"vertical_size %d",vertical_size); 
   error(errortext);
   aspectratio = theconfig.mpegAspectRatio;        
   sprintf(errortext,"aspectratio %d",aspectratio); 
   error(errortext);
   frame_rate_code = theconfig.mpegFrameRate;      
   sprintf(errortext,"frame_rate_code %d",frame_rate_code); 
   error(errortext);
   bit_rate = theconfig.mpegBitRate;               
   sprintf(errortext,"bit_rate %lf",bit_rate); 
   error(errortext);
   vbv_buffer_size = theconfig.mpegVdvBufferSize;  
   sprintf(errortext,"vbv_buffer_size %d",vbv_buffer_size); 
   error(errortext);
    
   low_delay = theconfig.mpegLowDelay;             
   sprintf(errortext,"low_delay %d",low_delay); 
   error(errortext);
   constrparms = theconfig.mpegConstParamFlag;     
   sprintf(errortext,"constrparms %d",constrparms); 
   error(errortext);
   profile = theconfig.mpegProfileID;              
   sprintf(errortext,"profile %d",profile); 
   error(errortext);
   level = theconfig.mpegLevelID;                  
   sprintf(errortext,"level %d",level); 
   error(errortext);
   prog_seq = theconfig.mpegProgessiveSeq;         
   sprintf(errortext,"prog_seq %d",prog_seq); 
   error(errortext);
   chroma_format = theconfig.mpegChromaFormat;     
   sprintf(errortext,"chroma_format %d",chroma_format); 
   error(errortext);
   video_format = theconfig.mpegVideoFormat;       
   sprintf(errortext,"video_format %d",video_format); 
   error(errortext);
   color_primaries = theconfig.mpegColorPrimaries;          
   sprintf(errortext,"color_primaries %d",color_primaries); 
   error(errortext);
   transfer_characteristics = theconfig.mpegTransferCharac; 
   sprintf(errortext,"transfer_characteristics %d",transfer_characteristics); 
   error(errortext);
   matrix_coefficients = theconfig.mpegMatricCoeff;   
   sprintf(errortext,"matrix_coefficients %d",matrix_coefficients); 
   error(errortext);
   display_horizontal_size = theconfig.mpegDisplayX;  
   sprintf(errortext,"display_horizontal_size %d",display_horizontal_size); 
   error(errortext);
   display_vertical_size = theconfig.mpegDisplayY;    
   sprintf(errortext,"display_vertical_size %d",display_vertical_size); 
   error(errortext);
   dc_prec = theconfig.mpegIntraDcPrecision;          
   sprintf(errortext,"dc_prec %d",dc_prec); 
   error(errortext);
   topfirst = theconfig.mpegTopFieldFirst;            
   sprintf(errortext,"topfirst %d",topfirst); 
   error(errortext);
   
   frame_pred_dct_tab[0]   = theconfig.mpeg_frame_pred_dct_tab[0];
   frame_pred_dct_tab[1]   = theconfig.mpeg_frame_pred_dct_tab[1];
   frame_pred_dct_tab[2]   = theconfig.mpeg_frame_pred_dct_tab[2];
 
   conceal_tab[0]  = theconfig.mpeg_conceal_tab[0];
   conceal_tab[1]  = theconfig.mpeg_conceal_tab[1];
   conceal_tab[2]  = theconfig.mpeg_conceal_tab[2];
  
   qscale_tab[0]  = theconfig.mpeg_qscale_tab[0];
   qscale_tab[1]  = theconfig.mpeg_qscale_tab[1];
   qscale_tab[2]  = theconfig.mpeg_qscale_tab[2];
   
   intravlc_tab[0]  = theconfig.mpeg_intravlc_tab[0];
   intravlc_tab[1]  = theconfig.mpeg_intravlc_tab[1];
   intravlc_tab[2]  = theconfig.mpeg_intravlc_tab[2];
   
   altscan_tab[0]  = theconfig.mpeg_altscan_tab[0];
   altscan_tab[1]  = theconfig.mpeg_altscan_tab[1];
   altscan_tab[2]  = theconfig.mpeg_altscan_tab[2];
   
   repeatfirst = theconfig.mpegRepeatFirstField;            sprintf(errortext,"repeatfirst %d",repeatfirst); error(errortext);
   prog_frame  = theconfig.mpegProgressiveFrame;            sprintf(errortext,"prog_frame %d",prog_frame); error(errortext);
   P           = theconfig.mpegPDistanceRefresh;            sprintf(errortext,"P %d",P); error(errortext);
   r           = theconfig.mpegRRateControl;                sprintf(errortext,"r %d",r); error(errortext);
   avg_act     = theconfig.mpegAvgRateControl;              sprintf(errortext,"avg_act %lf",avg_act); error(errortext);
   Xi          = theconfig.mpegXiRateControl;               sprintf(errortext,"Xi %d",Xi); error(errortext);
   Xp          = theconfig.mpegXpRateControl;               sprintf(errortext,"Xp %d",Xp); error(errortext);
   Xb          = theconfig.mpegXbRateControl;               sprintf(errortext,"Xb %d",Xb); error(errortext);
   d0i         = theconfig.mpegdOiRateControl;              sprintf(errortext,"d0i %d",d0i); error(errortext);
   d0p         = theconfig.mpegdOpRateControl;              sprintf(errortext,"d0p %d",d0p); error(errortext);
   d0b         = theconfig.mpegdObRateControl;              sprintf(errortext,"d0b %d",d0b); error(errortext);   

   //some error checking here for loops below
   if (N<1)
      { N=1; error("N must be positive"); }
   if (M<1)
      { M=1; error("M must be positive"); }
   if (N%M != 0)
      error("N must be an integer multiple of M");

   motion_data = (struct motion_data *)malloc(M*sizeof(struct motion_data));
   if (!motion_data)
      error("malloc failed\n");

      
   /////////////////////////////////////////////////
   //FIX ME ASAP!!!!!
   //this is the only part left to figure out  
   //here it would just keep reading objects in from file
   //looping on the read...
    
   for (i=0; i<M; i++)
   {
      //motion_data[i]  = theconfig.mpeg_altscan_tab[2];
      
      //fgets(line,254,fd);  sscanf(line,"%d %d %d %d",
         //&motion_data[i].forw_hor_f_code = theconfig.motion_data[0];
         motion_data[i].forw_hor_f_code = theconfig.motion_data[0];
     motion_data[i].forw_vert_f_code= theconfig.motion_data[1];
         motion_data[i].sxf             = theconfig.motion_data[2];
     motion_data[i].syf             = theconfig.motion_data[3];
     //);

      if (i!=0)
      {
        // fgets(line,254,fd);
        // sscanf(line,"%d %d %d %d",
            motion_data[i].back_hor_f_code= theconfig.motion_data[0];
        motion_data[i].back_vert_f_code= theconfig.motion_data[1];
            motion_data[i].sxb= theconfig.motion_data[2]; 
        motion_data[i].syb= theconfig.motion_data[3];
        //);
      }
   }

   //fclose(fd);

   /* make flags boolean (x!=0 -> x=1) */
   mpeg1 = !!mpeg1;
   fieldpic = !!fieldpic;
   low_delay = !!low_delay;
   constrparms = !!constrparms;
   prog_seq = !!prog_seq;
   topfirst = !!topfirst;

   for (i=0; i<3; i++)
   {
      frame_pred_dct_tab[i] = !!frame_pred_dct_tab[i];
      conceal_tab[i] = !!conceal_tab[i];
      qscale_tab[i] = !!qscale_tab[i];
      intravlc_tab[i] = !!intravlc_tab[i];
      altscan_tab[i] = !!altscan_tab[i];
   }
   repeatfirst = !!repeatfirst;
   prog_frame = !!prog_frame;

   /* make sure MPEG specific parameters are valid */
   range_checks();

   frame_rate = ratetab[frame_rate_code-1];

   /* timecode -> frame number */
   tc0 = h;
   tc0 = 60*tc0 + m;
   tc0 = 60*tc0 + s;
   tc0 = (int)(frame_rate+0.5)*tc0 + f;

   if (!mpeg1)
   {
      profile_and_level_checks();
   }
   else
   {
      /* MPEG-1 */
      if (constrparms)
      {
         if (horizontal_size>768
            || vertical_size>576
            || ((horizontal_size+15)/16)*((vertical_size+15)/16)>396
            || ((horizontal_size+15)/16)*((vertical_size+15)/16)*frame_rate>396*25.0
            || frame_rate>30.0)
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
            constrparms = 0;
         }
      }

      if (constrparms)
      {
         for (i=0; i<M; i++)
         {
            if (motion_data[i].forw_hor_f_code>4)
            {
               if (!quiet)
                  fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
               constrparms = 0;
               break;
            }

            if (motion_data[i].forw_vert_f_code>4)
            {
               if (!quiet)
                  fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
               constrparms = 0;
               break;
            }

            if (i!=0)
            {
               if (motion_data[i].back_hor_f_code>4)
               {
                  if (!quiet)
                     fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
                  constrparms = 0;
                  break;
               }

               if (motion_data[i].back_vert_f_code>4)
               {
                  if (!quiet)
                     fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
                  constrparms = 0;
                  break;
               }
            }
         }
      }
   }

   /* relational checks */

   if (mpeg1)
   {
      if (!prog_seq)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting progressive_sequence = 1\n");
         prog_seq = 1;
      }

      if (chroma_format!=CHROMA420)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting chroma_format = 1 (4:2:0)\n");
         chroma_format = CHROMA420;
      }

      if (dc_prec!=0)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting intra_dc_precision = 0\n");
         dc_prec = 0;
      }

      for (i=0; i<3; i++)
         if (qscale_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting qscale_tab[%d] = 0\n",i);
            qscale_tab[i] = 0;
         }

      for (i=0; i<3; i++)
         if (intravlc_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting intravlc_tab[%d] = 0\n",i);
            intravlc_tab[i] = 0;
         }

      for (i=0; i<3; i++)
         if (altscan_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting altscan_tab[%d] = 0\n",i);
            altscan_tab[i] = 0;
         }
   }

   if (!mpeg1 && constrparms)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
      constrparms = 0;
   }

   if (prog_seq && !prog_frame)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting progressive_frame = 1\n");
      prog_frame = 1;
   }

   if (prog_frame && fieldpic)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting field_pictures = 0\n");
      fieldpic = 0;
   }

   if (!prog_frame && repeatfirst)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting repeat_first_field = 0\n");
      repeatfirst = 0;
   }

   if (prog_frame)
   {
      for (i=0; i<3; i++)
         if (!frame_pred_dct_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting frame_pred_frame_dct[%d] = 1\n",i);
            frame_pred_dct_tab[i] = 1;
         }
   }

   if (prog_seq && !repeatfirst && topfirst)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting top_field_first = 0\n");
      topfirst = 0;
   }

   /* search windows */
   for (i=0; i<M; i++)
   {
      if (motion_data[i].sxf > (4<<motion_data[i].forw_hor_f_code)-1)
      {
         if (!quiet)
            fprintf(stderr,
               "Warning: reducing forward horizontal search width to %d\n",
               (4<<motion_data[i].forw_hor_f_code)-1);
         motion_data[i].sxf = (4<<motion_data[i].forw_hor_f_code)-1;
      }

      if (motion_data[i].syf > (4<<motion_data[i].forw_vert_f_code)-1)
      {
         if (!quiet)
            fprintf(stderr,
               "Warning: reducing forward vertical search width to %d\n",
               (4<<motion_data[i].forw_vert_f_code)-1);
         motion_data[i].syf = (4<<motion_data[i].forw_vert_f_code)-1;
      }

      if (i!=0)
      {
         if (motion_data[i].sxb > (4<<motion_data[i].back_hor_f_code)-1)
         {
            if (!quiet)
               fprintf(stderr,
                  "Warning: reducing backward horizontal search width to %d\n",
                  (4<<motion_data[i].back_hor_f_code)-1);
            motion_data[i].sxb = (4<<motion_data[i].back_hor_f_code)-1;
         }

         if (motion_data[i].syb > (4<<motion_data[i].back_vert_f_code)-1)
         {
            if (!quiet)
               fprintf(stderr,
                  "Warning: reducing backward vertical search width to %d\n",
                  (4<<motion_data[i].back_vert_f_code)-1);
            motion_data[i].syb = (4<<motion_data[i].back_vert_f_code)-1;
         }
      }
   }

}

//static void readparmfile(RenderConfig theconfig)
static void readparmfile(fname)
   char *fname;
{
  // printf();
  // error(">><< Reading Config File");
         
   int i;
   int h,m,s,f;
   FILE *fd;
   char line[256];
   static double ratetab[8]=
      {24000.0/1001.0,24.0,25.0,30000.0/1001.0,30.0,50.0,60000.0/1001.0,60.0};
   extern int r,Xi,Xb,Xp,d0i,d0p,d0b; /* rate control */
   extern double avg_act; /* rate control */

   if (!(fd = fopen(fname,"r")))
   {
      sprintf(errortext,"Couldn't open the ras parameter file %s",fname);
      error(errortext);
   }

   error(">><< Reading Config File");
   
   
   fgets(id_string,254,fd);
   fgets(line,254,fd); sscanf(line,"%s",tplorg);        sprintf(errortext,"tplorg %s",tplorg); error(errortext);
   fgets(line,254,fd); sscanf(line,"%s",tplref);        sprintf(errortext,"tplref %s",tplref); error(errortext);
   fgets(line,254,fd); sscanf(line,"%s",iqname);        sprintf(errortext,"iqname %s",iqname); error(errortext);
   fgets(line,254,fd); sscanf(line,"%s",niqname);       sprintf(errortext,"niqname %s",niqname); error(errortext);
   fgets(line,254,fd); sscanf(line,"%s",statname);      sprintf(errortext,"statname %s",statname); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&inputtype);    sprintf(errortext,"inputtype %d",inputtype); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&nframes);      sprintf(errortext,"nframes %d",nframes); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&frame0);       sprintf(errortext,"frame0 %d",frame0); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d:%d:%d:%d",&h,&m,&s,&f);
   fgets(line,254,fd); sscanf(line,"%d",&N);            sprintf(errortext,"N %d",N); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&M);            sprintf(errortext,"M %d",M); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&mpeg1);        sprintf(errortext,"mpeg1 %d",mpeg1); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&fieldpic);     sprintf(errortext,"fieldpic %d",fieldpic); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&horizontal_size); sprintf(errortext,"horizontal_size %d",horizontal_size); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&vertical_size);   sprintf(errortext,"vertical_size %d",vertical_size); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&aspectratio);  sprintf(errortext,"aspectratio %d",aspectratio); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&frame_rate_code); sprintf(errortext,"frame_rate_code %d",frame_rate_code); error(errortext);
   fgets(line,254,fd); sscanf(line,"%lf",&bit_rate);       sprintf(errortext,"bit_rate %lf",bit_rate); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&vbv_buffer_size);  sprintf(errortext,"vbv_buffer_size %d",vbv_buffer_size); error(errortext);
    
   fgets(line,254,fd); sscanf(line,"%d",&low_delay);     sprintf(errortext,"low_delay %d",low_delay); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&constrparms);   sprintf(errortext,"constrparms %d",constrparms); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&profile);       sprintf(errortext,"profile %d",profile); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&level);         sprintf(errortext,"level %d",level); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&prog_seq);      sprintf(errortext,"prog_seq %d",prog_seq); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&chroma_format); sprintf(errortext,"chroma_format %d",chroma_format); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&video_format);  sprintf(errortext,"video_format %d",video_format); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&color_primaries);sprintf(errortext,"color_primaries %d",color_primaries); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&transfer_characteristics); sprintf(errortext,"transfer_characteristics %d",transfer_characteristics); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&matrix_coefficients);      sprintf(errortext,"matrix_coefficients %d",matrix_coefficients); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&display_horizontal_size);  sprintf(errortext,"display_horizontal_size %d",display_horizontal_size); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&display_vertical_size);    sprintf(errortext,"display_vertical_size %d",display_vertical_size); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&dc_prec);                  sprintf(errortext,"dc_prec %d",dc_prec); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&topfirst);                 sprintf(errortext,"topfirst %d",topfirst); error(errortext);
   
   fgets(line,254,fd); sscanf(line,"%d %d %d",
      frame_pred_dct_tab,frame_pred_dct_tab+1,frame_pred_dct_tab+2);
  
   fgets(line,254,fd); sscanf(line,"%d %d %d",
      conceal_tab,conceal_tab+1,conceal_tab+2);
  
   fgets(line,254,fd); sscanf(line,"%d %d %d",
      qscale_tab,qscale_tab+1,qscale_tab+2);

   fgets(line,254,fd); sscanf(line,"%d %d %d",
      intravlc_tab,intravlc_tab+1,intravlc_tab+2);
      
   fgets(line,254,fd); sscanf(line,"%d %d %d",
      altscan_tab,altscan_tab+1,altscan_tab+2);
      
   fgets(line,254,fd); sscanf(line,"%d",&repeatfirst);          sprintf(errortext,"repeatfirst %d",repeatfirst); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&prog_frame);          sprintf(errortext,"prog_frame %d",prog_frame); error(errortext);
/* intra slice interval refresh period */  
   fgets(line,254,fd); sscanf(line,"%d",&P);          sprintf(errortext,"P %d",P); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&r);          sprintf(errortext,"r %d",r); error(errortext);
   fgets(line,254,fd); sscanf(line,"%lf",&avg_act);   sprintf(errortext,"avg_act %lf",avg_act); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&Xi);         sprintf(errortext,"Xi %d",Xi); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&Xp);         sprintf(errortext,"Xp %d",Xp); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&Xb);         sprintf(errortext,"Xb %d",Xb); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&d0i);        sprintf(errortext,"d0i %d",d0i); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&d0p);        sprintf(errortext,"d0p %d",d0p); error(errortext);
   fgets(line,254,fd); sscanf(line,"%d",&d0b);        sprintf(errortext,"d0b %d",d0b); error(errortext);

   if (N<1)
      error("N must be positive");
   if (M<1)
      error("M must be positive");
   if (N%M != 0)
      error("N must be an integer multiple of M");

   motion_data = (struct motion_data *)malloc(M*sizeof(struct motion_data));
   if (!motion_data)
      error("malloc failed\n");

   for (i=0; i<M; i++)
   {
      fgets(line,254,fd);
      sscanf(line,"%d %d %d %d",
         &motion_data[i].forw_hor_f_code, &motion_data[i].forw_vert_f_code,
         &motion_data[i].sxf, &motion_data[i].syf);

      if (i!=0)
      {
         fgets(line,254,fd);
         sscanf(line,"%d %d %d %d",
            &motion_data[i].back_hor_f_code, &motion_data[i].back_vert_f_code,
            &motion_data[i].sxb, &motion_data[i].syb);
      }
   }

   fclose(fd);

   /* make flags boolean (x!=0 -> x=1) */
   mpeg1 = !!mpeg1;
   fieldpic = !!fieldpic;
   low_delay = !!low_delay;
   constrparms = !!constrparms;
   prog_seq = !!prog_seq;
   topfirst = !!topfirst;

   for (i=0; i<3; i++)
   {
      frame_pred_dct_tab[i] = !!frame_pred_dct_tab[i];
      conceal_tab[i] = !!conceal_tab[i];
      qscale_tab[i] = !!qscale_tab[i];
      intravlc_tab[i] = !!intravlc_tab[i];
      altscan_tab[i] = !!altscan_tab[i];
   }
   repeatfirst = !!repeatfirst;
   prog_frame = !!prog_frame;

   /* make sure MPEG specific parameters are valid */
   range_checks();

   frame_rate = ratetab[frame_rate_code-1];

   /* timecode -> frame number */
   tc0 = h;
   tc0 = 60*tc0 + m;
   tc0 = 60*tc0 + s;
   tc0 = (int)(frame_rate+0.5)*tc0 + f;

   if (!mpeg1)
   {
      profile_and_level_checks();
   }
   else
   {
      /* MPEG-1 */
      if (constrparms)
      {
         if (horizontal_size>768
            || vertical_size>576
            || ((horizontal_size+15)/16)*((vertical_size+15)/16)>396
            || ((horizontal_size+15)/16)*((vertical_size+15)/16)*frame_rate>396*25.0
            || frame_rate>30.0)
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
            constrparms = 0;
         }
      }

      if (constrparms)
      {
         for (i=0; i<M; i++)
         {
            if (motion_data[i].forw_hor_f_code>4)
            {
               if (!quiet)
                  fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
               constrparms = 0;
               break;
            }

            if (motion_data[i].forw_vert_f_code>4)
            {
               if (!quiet)
                  fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
               constrparms = 0;
               break;
            }

            if (i!=0)
            {
               if (motion_data[i].back_hor_f_code>4)
               {
                  if (!quiet)
                     fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
                  constrparms = 0;
                  break;
               }

               if (motion_data[i].back_vert_f_code>4)
               {
                  if (!quiet)
                     fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
                  constrparms = 0;
                  break;
               }
            }
         }
      }
   }

   /* relational checks */

   if (mpeg1)
   {
      if (!prog_seq)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting progressive_sequence = 1\n");
         prog_seq = 1;
      }

      if (chroma_format!=CHROMA420)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting chroma_format = 1 (4:2:0)\n");
         chroma_format = CHROMA420;
      }

      if (dc_prec!=0)
      {
         if (!quiet)
            fprintf(stderr,"Warning: setting intra_dc_precision = 0\n");
         dc_prec = 0;
      }

      for (i=0; i<3; i++)
         if (qscale_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting qscale_tab[%d] = 0\n",i);
            qscale_tab[i] = 0;
         }

      for (i=0; i<3; i++)
         if (intravlc_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting intravlc_tab[%d] = 0\n",i);
            intravlc_tab[i] = 0;
         }

      for (i=0; i<3; i++)
         if (altscan_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting altscan_tab[%d] = 0\n",i);
            altscan_tab[i] = 0;
         }
   }

   if (!mpeg1 && constrparms)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting constrained_parameters_flag = 0\n");
      constrparms = 0;
   }

   if (prog_seq && !prog_frame)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting progressive_frame = 1\n");
      prog_frame = 1;
   }

   if (prog_frame && fieldpic)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting field_pictures = 0\n");
      fieldpic = 0;
   }

   if (!prog_frame && repeatfirst)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting repeat_first_field = 0\n");
      repeatfirst = 0;
   }

   if (prog_frame)
   {
      for (i=0; i<3; i++)
         if (!frame_pred_dct_tab[i])
         {
            if (!quiet)
               fprintf(stderr,"Warning: setting frame_pred_frame_dct[%d] = 1\n",i);
            frame_pred_dct_tab[i] = 1;
         }
   }

   if (prog_seq && !repeatfirst && topfirst)
   {
      if (!quiet)
         fprintf(stderr,"Warning: setting top_field_first = 0\n");
      topfirst = 0;
   }

   /* search windows */
   for (i=0; i<M; i++)
   {
      if (motion_data[i].sxf > (4<<motion_data[i].forw_hor_f_code)-1)
      {
         if (!quiet)
            fprintf(stderr,
               "Warning: reducing forward horizontal search width to %d\n",
               (4<<motion_data[i].forw_hor_f_code)-1);
         motion_data[i].sxf = (4<<motion_data[i].forw_hor_f_code)-1;
      }

      if (motion_data[i].syf > (4<<motion_data[i].forw_vert_f_code)-1)
      {
         if (!quiet)
            fprintf(stderr,
               "Warning: reducing forward vertical search width to %d\n",
               (4<<motion_data[i].forw_vert_f_code)-1);
         motion_data[i].syf = (4<<motion_data[i].forw_vert_f_code)-1;
      }

      if (i!=0)
      {
         if (motion_data[i].sxb > (4<<motion_data[i].back_hor_f_code)-1)
         {
            if (!quiet)
               fprintf(stderr,
                  "Warning: reducing backward horizontal search width to %d\n",
                  (4<<motion_data[i].back_hor_f_code)-1);
            motion_data[i].sxb = (4<<motion_data[i].back_hor_f_code)-1;
         }

         if (motion_data[i].syb > (4<<motion_data[i].back_vert_f_code)-1)
         {
            if (!quiet)
               fprintf(stderr,
                  "Warning: reducing backward vertical search width to %d\n",
                  (4<<motion_data[i].back_vert_f_code)-1);
            motion_data[i].syb = (4<<motion_data[i].back_vert_f_code)-1;
         }
      }
   }

}


static void readquantmat()
{
   int i,v;
   FILE *fd;

   if (iqname[0]=='-')
   {
      /* use default intra matrix */
      load_iquant = 0;
      for (i=0; i<64; i++)
         intra_q[i] = default_intra_quantizer_matrix[i];
   }
   else
   {
      /* read customized intra matrix */
      load_iquant = 1;
      if (!(fd = fopen(iqname,"r")))
      {
         sprintf(errortext,"Couldn't open quant matrix file %s",iqname);
         error(errortext);
      }

      for (i=0; i<64; i++)
      {
         fscanf(fd,"%d",&v);
         if (v<1 || v>255)
            error("invalid value in quant matrix");
         intra_q[i] = v;
      }

      fclose(fd);
   }

   if (niqname[0]=='-')
   {
      /* use default non-intra matrix */
      load_niquant = 0;
      for (i=0; i<64; i++)
         inter_q[i] = 16;
   }
   else
   {
      /* read customized non-intra matrix */
      load_niquant = 1;
      if (!(fd = fopen(niqname,"r")))
      {
         sprintf(errortext,"Couldn't open quant matrix file %s",niqname);
         error(errortext);
      }

      for (i=0; i<64; i++)
      {
         fscanf(fd,"%d",&v);
         if (v<1 || v>255)
            error("invalid value in quant matrix");
         inter_q[i] = v;
      }

      fclose(fd);
   }
}
