#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "config.h"
#include "global.h"

#include <qimage.h>
#include "qimagefilter.h"

//////////////////////////////////////////////////////////////
// the 'glue' between the c and c++ objects
EXTERN_C void read_qimage(char* file_name, unsigned char* frame[])
{
    Qimage_filter_c qimage_filter;

    qimage_filter.readQimage(file_name, frame);

}

////////////////////////////////////////////////////////////////////
// the c++ objects command to read using qimage and convert to char
void Qimage_filter_c::readQimage(char* file_name, unsigned char* frame[])
{
//#ifndef WIN32
    unsigned int                current_pixel;
    int                         i, j;
    int                         r, g, b;
    double                      y, u, v;
    double                      cr, cg, cb, cu, cv;
    char                        name[128];
    unsigned char               *yp, *up, *vp;
    static unsigned char        *u444, *v444, *u422, *v422;

    static double coef[7][3] = 
    {
        {0.2125,0.7154,0.0721}, /* ITU-R Rec. 709 (1990) */
        {0.299, 0.587, 0.114},  /* unspecified */
        {0.299, 0.587, 0.114},  /* reserved */
        {0.30,  0.59,  0.11},   /* FCC */
        {0.299, 0.587, 0.114},  /* ITU-R Rec. 624-4 System B, G */
        {0.299, 0.587, 0.114},  /* SMPTE 170M */
        {0.212, 0.701, 0.087}
    }; /* SMPTE 240M (1987) */

    i = matrix_coefficients;

    if (i > 8)
    {
        i = 3;
    }

    cr = coef[i-1][0];
    cg = coef[i-1][1];
    cb = coef[i-1][2];
    cu = 0.5 / (1.0 - cb);
    cv = 0.5 / (1.0 - cr);

    if (chroma_format == CHROMA444)
    {
        u444 = frame[1];
        v444 = frame[2];
    }
    else
    {
        if (!u444)
        {
            u444 = new unsigned char[width * height];
            v444 = new unsigned char[width * height];

            if (chroma_format == CHROMA420)
            {
                u422 = new unsigned char[(width >> 1) * height];	
                v422 = new unsigned char[(width >> 1) * height];
            }
        }
    }

    //was .ppm here nned to call this dynamically...
    //sprintf(name, "%s.jpg", file_name);
    sprintf(name, "%s", file_name);
    
    ///////////////////////////////////////////////////////////
    //this is the QImage load routine
    //if we are dealing with a tga we need to let
    //the image loader know this
    //image.load(name, "TGA");
    QImage loader;
    loader.load(name, 0);

    //here we build the image
    QImage image;
    
    //this is the resizing routine
    if (conversionratio > 1 )
    {
        float ratio = (1/(float)conversionratio );               
        
        image = loader.smoothScale( int(  loader.width()*ratio), 
                                    int(  loader.height()*ratio)  );
    } 
    else
    {
        image = loader;
    }
    
    
    horizontal_size = image.width();
    vertical_size = image.height();

    for (i = 0; i < vertical_size; i++)
    {
        yp = frame[0] + (i * width);
        up = u444 + (i * width);
        vp = v444 + (i * width);

        for (j = 0; j < horizontal_size; j++)
        {
            current_pixel = (unsigned int)image.pixel(j, i);

            r = (int)((current_pixel >> 16) & 0xff);
            g = (int)((current_pixel >>  8) & 0xff);
            b = (int)((current_pixel >>  0) & 0xff);

            /* convert to YUV */
            y = (cr * r) + (cg * g) + (cb * b);
            u = cu * (b - y);
            v = cv * (r - y);

            yp[j] = ((219.0 / 256.0) * y) + 16.5;  /* nominal range: 16..235 */
            up[j] = ((224.0 / 256.0) * u) + 128.5; /* 16..240 */
            vp[j] = ((224.0 / 256.0) * v) + 128.5; /* 16..240 */
        }
    }

    border_extend(frame[0], horizontal_size, vertical_size, width, height);
    border_extend(u444, horizontal_size, vertical_size, width, height);
    border_extend(v444, horizontal_size, vertical_size, width, height);

    if (chroma_format==CHROMA422)
    {
        conv444to422(u444, frame[1]);
        conv444to422(v444, frame[2]);
    }

    if (chroma_format==CHROMA420)
    {
        conv444to422(u444,u422);
        conv444to422(v444,v422);
        conv422to420(u422,frame[1]);
        conv422to420(v422,frame[2]);
    }
//#endif
}
