/*******************************************************************************
**
** The source file for the Jahshaka glworld.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glcore.h"

GLCore::GLCore( QWidget* parent, const char* name, int mode)
    : QGLWidget( parent, name ) 
{  

    //lets us know if its 3d perp or 2d ortho
    glmode = mode;

    //this just sets the format
    QGLFormat format;

    if (glmode == 1)
    {
        format.setAlpha(TRUE);
        format.setDoubleBuffer(FALSE);
        format.setDepth(FALSE);
    }
    else
    {
        format.setAlpha(TRUE);
        format.setDoubleBuffer(TRUE);
        format.setDepth(TRUE);
    }

    format.setRgba(TRUE);
    format.setAccum(FALSE);
    format.setStereo(FALSE);
    format.setStencil(FALSE); 
    format.setDirectRendering( TRUE ); 
    setFormat(format);

    JahPrefs& jprefs = JahPrefs::getInstance();
    //X_RESOLUTION = jprefs.getXRes();
    //Y_RESOLUTION = jprefs.getYRes();

    //set some variables
    useimagesize=false;
    imagewidth=0;
    imageheight=0;
    zoomVal=1;

    //for drag and drop
    setAcceptDrops(TRUE);

};

void GLCore::dragEnterEvent(QDragEnterEvent* event)
{
    //debug("glcore drag event");

    event->accept(
        QTextDrag::canDecode(event) ||
        QImageDrag::canDecode(event)
    );

}

void GLCore::dropEvent(QDropEvent* event)
{
    QImage image;
    QString text;

    //debug("glcore drop event");

    if ( QImageDrag::decode(event, image) )
    {
        //debug("its a image");
        //this is connected to the main app in Source/jahroutines.cpp
        emit draggedAImage();
    }
    else if ( QTextDrag::decode(event, text) )
    {
        //debug("its text");
    }

}

void GLCore::mousePressEvent( QMouseEvent *e )
{
    emit mousePress(e);
}

void GLCore::mouseReleaseEvent( QMouseEvent *e ) 
{
    emit mouseRelease(e);
}

void GLCore::mouseMoveEvent( QMouseEvent *e ) 
{
    emit mouseMove(e);
}

void GLCore::initializeGL() 
{       
    //initalize the global vars from the prefs
    JahPrefs& jprefs  = JahPrefs::getInstance();

    bool HWFOG = jprefs.getHwFog();

    if (glmode == 0)
    {
    //Set up the OpenGL rendering state, and define display list

    //set up environment and turn on depth testing
    glDepthFunc(GL_LESS); glEnable( GL_DEPTH_TEST ); glShadeModel(GL_SMOOTH);

    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glEnable( GL_TEXTURE_2D );

    GLfloat light_ambient[]  = {0.5, 0.5, 0.5, 1.0};     //    GLfloat light_ambient[] = {0.3, 0.3, 1.0, 1.0};
    GLfloat light_diffuse[]  = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_specular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_position[] = {0.0f, 0.0f, 1.0f, 0.0f};

    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glEnable(GL_LIGHT0);

    GLuint  fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR }; // Storage For Three Types Of Fog
    GLuint  fogfilter = 2;                         // Which Fog Mode To Use
    GLfloat fogColor[4] = {0.5f,0.5f,0.5f,1.0f};   // Fog Color

    //turns fog env on
    if (HWFOG)
    {
        glFogi(GL_FOG_MODE, fogMode[fogfilter]);       // Fog Mode
        glFogfv(GL_FOG_COLOR, fogColor);               // Set Fog Color
        glFogf(GL_FOG_DENSITY, 0.45f);                 // How Dense Will The Fog Be
        glHint(GL_FOG_HINT, GL_NICEST);                // Fog Hint Value
        glFogf(GL_FOG_START, 1000.0f);                    // Fog Start Depth
        glFogf(GL_FOG_END, 0.0f);                      // Fog End Depth
    }

    // Set up various other stuff
    glClearColor( 0.3, 0.3, 0.3, 0.0 ); // Let OpenGL clear to grey
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

    glEnable( GL_NORMALIZE );     //glEnable(GL_LINE_SMOOTH);

    //used to enable reflection mapping
    //what does this do exactly ie to speed and memory...
    //need to do this and rebuild textures if necessary
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For S To Sphere Mapping (NEW)
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For T To Sphere Mapping (NEW)

    }

    if (glmode==1)
    {

      glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

      glEnable(GL_POINT_SMOOTH);       //use smooth, rounded points
      glShadeModel(GL_SMOOTH);         //set default shading model
      glEnable(GL_COLOR_LOGIC_OP);     //enable color logic operations (ex: XOR)
      glPolygonMode(GL_FRONT,GL_FILL); //draw filled polygons, front only

      glClearColor( 0.0, 0.0, 0.0, 1.0 );
      //glClear(GL_COLOR_BUFFER_BIT);

    }
}

void GLCore::setZoom(GLfloat zoom) 
{

    zoomVal = zoom;

    //resizeGL( renderwidth, renderheight, fAspectval );
    resizeGL( renderwidth, renderheight );

}


void GLCore::resizeGL( int w, int h ) 
{


    if (glmode == 0)
    {

        //start the resizing
        GLfloat windowWidth  = (GLfloat)w;
        GLfloat windowHeight = (GLfloat)h;

        glViewport(0, 0, w, h);

        //store these for access in glworld renderer later on
        renderwidth  = w;
        renderheight = h;

        //this is the aspect ratio of the screen cords
        fAspect = (windowWidth/windowHeight);

        //Calculate aspect ratio we want to use
        //taken from these values in main.cpp
        GLfloat xres = 720;  //hardcode this to 720 was (GLfloat)X_RESOLUTION
        GLfloat yres = 486;  //hardcode tis to 486 was (GLfloat)Y_RESOLUTION

        fAspect = xres / yres;   

        fAspectval = fAspect;

        glMatrixMode(GL_PROJECTION);    // Set the perspective coordinate system
        glLoadIdentity();

#define PI                   3.14159265358979323846

        //GLfloat view_angle = 45.0f;
        GLfloat view_angle = 25.0f;
        GLfloat view_angle_radians = (view_angle / 180.0) * PI;

        gluPerspective(view_angle, fAspect, 1.0f, 10000.0f);   
        GLfloat camera_distance = (486.0 / 2.0) / tan(view_angle_radians / 2.0);
        //GLfloat camera_distance = ((float)Y_RESOLUTION / 2.0) / tan(view_angle_radians / 2.0);


        glMatrixMode(GL_MODELVIEW);    // Modelview matrix reset
        glLoadIdentity();

        //need to adjust the camera z depth here
        //this is a bug and shouldnt be here 
        GLfloat transval =  -camera_distance * (GLfloat)zoomVal; 
        glTranslatef( 0.0f, 0.0f, transval);

        //now scale the model viewport to keep the correct perspective!
        GLfloat scalew = xres / windowWidth;
        GLfloat scaleh = yres / windowHeight;

        glScalef ( scalew, scaleh, 1.0f );

    }

    if (glmode==1)
    {
          renderwidth  = w;
          renderheight = h;
         
          //glClear(GL_COLOR_BUFFER_BIT);
          glViewport(0, 0, (GLint) w, (GLint) h);
          glMatrixMode(GL_PROJECTION);
          glLoadIdentity();

          glOrtho(0, w, 0, h, -2, 2);

          glMatrixMode(GL_MODELVIEW);    // Modelview matrix reset
          glLoadIdentity();

          emit resizeit();

    }

}
