/*******************************************************************************
**
** The source file for the Jahshaka glworld.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glcore.h"


//JahGLCore*      GLCore::jahCore         = NULL;

GLCore::GLCore( QWidget* parent, const char* name, QWidget* module, const QGLWidget* shareWidget)
: QWidget( parent, name ) 
//: QGLWidget( parent, name, shareWidget ) 
{  

    //parent and name are issues
    //should be one object in jahcreate.cpp in source
    //then we can break and reconncect the slots as necessary
    //to use a static world...
    
    jahCore = new JahGLCore(parent, name);

    JahPrefs& jprefs = JahPrefs::getInstance();
    X_RESOLUTION = jprefs.getXRes();
    Y_RESOLUTION = jprefs.getYRes();

    //set some variables
    useimagesize=false;
    imagewidth=0;
    imageheight=0;
    zoomVal=1; 


    connect  ( jahCore,  SIGNAL(paint()), module, SLOT( paintGL())  );
    connect  ( jahCore,  SIGNAL(initialize()), this, SLOT( initializeGL())  );
    connect  ( jahCore,  SIGNAL(resize()), this, SLOT( resize())  );

};

void GLCore::initCore() 
{       
//    jahCore = new JahGLCore(this);

}

void GLCore::makeCurrent() 
{       
    //jahCore->makeCurrent();
    jahCore->makeCurrent();
}



void GLCore::updateGL() 
{       
    jahCore->updateGL();
}

//    QImage buffer = core->grabFrameBuffer(true);
QImage GLCore::grabFrameBuffer(bool val) 
{       
    QImage thebuffer = jahCore->grabFrameBuffer(val);
    return thebuffer;
}

void GLCore::initializeGL() 
{       

    //debug("init tha GL");
    //Set up the OpenGL rendering state, and define display list

    //set up environment and turn on depth testing
    glDepthFunc(GL_LESS); glEnable( GL_DEPTH_TEST ); glShadeModel(GL_SMOOTH);

    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glEnable( GL_TEXTURE_2D );

    GLfloat light_ambient[]  = {0.5, 0.5, 0.5, 1.0};     //    GLfloat light_ambient[] = {0.3, 0.3, 1.0, 1.0};
    GLfloat light_diffuse[]  = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_specular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_position[] = {0.0f, 0.0f, 1.0f, 0.0f};

    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glEnable(GL_LIGHT0);

    GLuint  fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR }; // Storage For Three Types Of Fog
    GLuint  fogfilter = 2;                         // Which Fog Mode To Use
    GLfloat fogColor[4] = {0.5f,0.5f,0.5f,1.0f};   // Fog Color

    //turns fog env on
    if (HWFOG)
    {
        glFogi(GL_FOG_MODE, fogMode[fogfilter]);       // Fog Mode
        glFogfv(GL_FOG_COLOR, fogColor);               // Set Fog Color
        glFogf(GL_FOG_DENSITY, 0.45f);                 // How Dense Will The Fog Be
        glHint(GL_FOG_HINT, GL_NICEST);                // Fog Hint Value
        glFogf(GL_FOG_START, 1000.0f);                    // Fog Start Depth
        glFogf(GL_FOG_END, 0.0f);                      // Fog End Depth
    }

    // Set up various other stuff
    glClearColor( 0.3, 0.3, 0.3, 0.0 ); // Let OpenGL clear to grey
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

    glEnable( GL_NORMALIZE );     //glEnable(GL_LINE_SMOOTH);

    //used to enable reflection mapping
    //what does this do exactly ie to speed and memory...
    //need to do this and rebuild textures if necessary
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For S To Sphere Mapping (NEW)
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); // Set The Texture Generation Mode For T To Sphere Mapping (NEW)

}

void GLCore::setZoom(GLfloat zoom) 
{

    zoomVal = zoom;

    //resizeGL( renderwidth, renderheight, fAspectval );
    resizeGL( renderwidth, renderheight );

}

void GLCore::resize() 
{       
    resizeGL( jahCore->gw, jahCore->gh ) ;
}

void GLCore::resizeGL( int w, int h ) 
{

//    debug("resize da gl");
    //start the resizing
    GLfloat windowWidth  = (GLfloat)w;
    GLfloat windowHeight = (GLfloat)h;

    glViewport(0, 0, windowWidth, windowHeight);

    //store these for access in glworld renderer later on
    renderwidth  = windowWidth;  
    renderheight = windowHeight;

    //this is the aspect ratio of the screen cords
    fAspect = (windowWidth/windowHeight);

    //Calculate aspect ratio we want to use
    //taken from these values in main.cpp
    //extern int  X_RESOLUTION = 720;
    //extern int  Y_RESOLUTION = 486;
    GLfloat xres = (GLfloat)X_RESOLUTION;
    GLfloat yres = (GLfloat)Y_RESOLUTION;

    fAspect = xres / yres;   

    fAspectval = fAspect;

    glMatrixMode(GL_PROJECTION);    // Set the perspective coordinate system
    glLoadIdentity();

#define PI                   3.14159265358979323846

    GLfloat view_angle = 45.0f;
    GLfloat view_angle_radians = (view_angle / 180.0) * PI;

    gluPerspective(view_angle, fAspect, 1.0f, 10000.0f);   
    GLfloat camera_distance = ((float)Y_RESOLUTION / 2.0) / tan(view_angle_radians / 2.0);


    glMatrixMode(GL_MODELVIEW);    // Modelview matrix reset
    glLoadIdentity();

    //need to adjust the camera z depth here
    //this is a bug and shouldnt be here 
    GLfloat transval =  -camera_distance * (GLfloat)zoomVal; 
    glTranslatef( 0.0f, 0.0f, transval);

    //now scale the model viewport to keep the correct perspective!
    GLfloat scalew = xres / windowWidth;
    GLfloat scaleh = yres / windowHeight;

    glScalef ( scalew, scaleh, 1.0f );

}

