/*******************************************************************************
**
** The source file for the Jahshaka assetexchange class
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "assetexchange.h"


assetExchange::assetExchange(void) 
{
    streamOpen = false;

//initialize JahBasePath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath  = jprefs.getBasePath();

}

assetExchange::~assetExchange(void) 
{
    if (streamOpen)
    {
        delete openStream;
    }
}

//////////////////////////////////////////////////////////////
//this routine figures out if the media object is present
//and functionable by testing the first frame

bool assetExchange::isMediaValid(videoData& theasset)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    bool returnval = false;
    QString loadfilename;

    //ok get the name of the asset
    returnval = getMediaFilename(theasset, 1, loadfilename);

    //if we can get a name thats a good sign...
    if (returnval)
    {
        //if its a movie file we can also test that it opens
        if (theasset.theType==MOVIE)
        {
            mediaData * mediaClip = new mediaData;

            if (mediaClip->openMedia(loadfilename))
            {
                returnval = true;
                mediaClip->closeMedia();
            }
            delete mediaClip;
        }
        else
        {
            //if its a image we just make sure the file is present
            if (QFile::exists(loadfilename) )
            {
                returnval=true;
            }
            else
            {
                returnval = false;
            }
        }
    }
    return returnval;
}

//figure out how many frames are in a media object...
bool assetExchange::getMediaLength(videoData& theasset, int &frame)
{
    bool returnval = false;;
    int numframes;
    QString loadfilename;

    //ok get the name of the asset
    returnval = getMediaFilename(theasset, 1, loadfilename);

    //if we can get a name thats a good sign...
    if (returnval)

    {
        //if its a movie file
        if (theasset.theType==MOVIE)
        {
            mediaData * mediaClip = new mediaData;

            if (mediaClip->openMedia(loadfilename))
            {   
                numframes = mediaClip->getVideoLength();
                mediaClip->closeMedia();	
                returnval = true;
            }
            delete mediaClip;
        }

    }

    if (numframes < 0) numframes = 1;

    frame = numframes;

    return returnval;

}

//this routine figures out the access filename based
//on the type of asset we are dealing with
bool assetExchange::getMediaFilename(videoData& theasset, int frame, QString &returnName)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString loadfilename;
    bool itworked = false;

    switch (theasset.theCategory) {

        case IMAGE  :   { //for still images
                            loadfilename = theasset.location + theasset.filename + theasset.extension;
                            itworked = true;
                            break;
                        }

        case AUDIO  :   { //for audio
                            loadfilename = theasset.location + theasset.filename + theasset.extension;
                            itworked = true;
                            break;
                        }

        case CLIP   :   { //for clips
                            if (theasset.theType==MOVIE) // for avi, mpeg, etc
                            {
                                loadfilename = theasset.location + theasset.filename + theasset.extension;
                                itworked = true;
                            }
                            else
                            {   //clip thats not a movie is a sequence
                                loadfilename = getsequencefilename(theasset,frame);
                                itworked = true;
                            }
                            break;
                        }

        case MEDIA   :   { //for clips
                            if (theasset.theType==MOVIE) // for avi, mpeg, etc
                            {
                                loadfilename = theasset.location + theasset.filename + theasset.extension;
                                itworked = true;
                            }
                            else
                            {   //clip thats not a movie is a sequence
                                loadfilename = getsequencefilename(theasset,frame);
                                itworked = true;
                            }
                            break;
                        }

        case KEY    :   {   //for key layers
                            if (theasset.theType==SEQUENCE)
                            {
                                loadfilename = getsequencefilename(theasset,frame);
                                itworked = true;
                            }
                            else
                            {
                                loadfilename = theasset.location + theasset.filename + theasset.extension;
                                itworked = true;
                            }

                            break;
                        }

        default     :   { //unknown media type
                            jtrace.debug("assetExchange::","getMediaFilename() unknown asset category");
                            itworked = true;
                            break;
                        }

    }

    returnName = loadfilename;
    return itworked;
}


QImage assetExchange::getImage(videoData& theasset, int frame)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QImage loadimage;
    bool itworked;

    QString loadfilename;

    itworked = getMediaFilename(theasset, frame, loadfilename);

    jtrace.debug("::::AssetExchange getting image:"+loadfilename);

    //movie is a media file
    if (theasset.theType==MOVIE) 
    {   
        //jtrace.debug("loading video asset");
        if ( loadMediaintoimage( loadfilename, loadimage, frame ) )
        {
            itworked = true;
        }
        else
        {
            itworked = false;
        }
    }
    else if (theasset.theCategory==AUDIO)
    {
        //jtrace.debug("loading audio asset");

        if ( !loadimage.load( JahBasePath+"Pixmaps/jahaudio.png" ) ) 
        {	// Load first image from file
            jtrace.error( "Could not read default image, using single-color instead." );
            QImage dummy( 256, 256, 32 );
            dummy.fill( Qt::blue.rgb() );
            loadimage = dummy;
            itworked = FALSE;
        }

    }
    //if its not movie or audio use this as it returns a default if we
    //dont find anything
    else
    {
        // this routine loads the image
        if ( loadFileintoimage( loadfilename, loadimage ) )
        {
            itworked = true;
        }
        else
        {
            itworked = false;
        }
    }

    return loadimage;
};


bool assetExchange::loadFileintoimage( const QString loadfilename, QImage & image)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    //jtrace.debug("assetExchange::loadFileintoimage");

    bool returnval = TRUE;

    QImageIO iio; iio.setFileName( loadfilename );
    if ( loadfilename.endsWith( ".tga" ) || loadfilename.endsWith( ".TGA" )) { iio.setFormat("TGA"); }
    if ( loadfilename.endsWith( ".rgb" ) || loadfilename.endsWith( ".RGB" )) { iio.setFormat("RGB"); }

    if ( iio.read() ) {	image = iio.image(); /*thepic = iio.image(); image = thepic;*/ } else
    {

        //jtrace.debug( "Unable to load file,"+loadfilename+" attempting default image" );
        //if ( !image.load( JahMediaPath+"/images/sky.bmp" ) ) {	// Load first image from file
        if ( !image.load( JahBasePath+"Pixmaps/missingmedia.png" ) ) {	// Load first image from file
            jtrace.error( "Could not read default image, using single-color instead." );
            QImage dummy( 256, 256, 32 );
            dummy.fill( Qt::blue.rgb() );
            image = dummy;
            returnval = FALSE;
        }

    }

    return returnval;

}



bool assetExchange::loadMediaintoimage( const QString loadfilename, QImage & image, int frame)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    //jtrace.debug( "assetExchange::","loadMediaintoimage() loading mediafile");
    bool returnval = TRUE;

    mediaData * mediaClip = new mediaData;

    if (mediaClip->openMedia(loadfilename))
    {   
        image = mediaClip->getVideoFrame(frame);
        mediaClip->closeMedia();	
    }
    else
    {
        //jtrace.debug( "assetX Unable to load mediafile,"+loadfilename+" attempting default image" );

        if ( !image.load( JahBasePath+"Pixmaps/missingmedia.png" ) )
        {
            //jtrace.error( "Could not read default image, using single-color instead." );
            QImage dummy( 256, 256, 32 );
            dummy.fill( Qt::blue.rgb() );
            image = dummy;
            returnval = FALSE;
        }

    }

    delete mediaClip;
    return returnval;
}





//////////////////////////////////////////////
// we really need to optimize this routine mon
// as its important and accessed all the time


//maybe add a initializing function so we dont have to send the
//videoData over for every frame?
QImage assetExchange::getStreamImage(videoData& theasset, int frame)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug("getting stream image in assetExchange::getStreamImage");

    QImage loadimage;
    bool itworked;


    QString loadfilename;

    //////////////////////////////////////////////////////////////
    //if its a media or movie file we open a stream
    //if its a image file we dont

    if (!streamOpen && theasset.theType==MOVIE)
    {
        //jtrace.debug("stream closed and asset is movie");

        openStream = new mediaData;
        //istreamOpen = true;

        //QString loadfilename;
        itworked = getMediaFilename(theasset, frame, loadfilename);

        //filename = loadfilename;
        //debug("AssetExchange Opening Stream >>:"+ loadfilename);
    }
    else
    {
        itworked = getMediaFilename(theasset, frame, loadfilename);
        //jtrace.debug("AssetExchange Opening image >>:"+ loadfilename);
    }

    //////////////////////////////////////////////////////////////
    //movie is a media file
    if (theasset.theType==MOVIE)
    {
        //jtrace.debug("its a movie getting picture");

        if ( streamMediaintoimage( loadfilename, loadimage, frame ) )
        {
            //debug("it worked");
            //jtrace.debug("width:",loadimage.width());
            //jtrace.debug("height:",loadimage.height());

            itworked = true;
        }
        else
        {
            //debug("it didnt work");
            itworked = false;
        }
    }
    else if (theasset.theCategory==AUDIO)
    {
        //hack to load audio image
        //if ( !image.load( JahMediaPath+"/images/sky.bmp" ) ) {	// Load first image from file
        if ( !loadimage.load( JahBasePath+"Pixmaps/jahaudio.png" ) ) {	// Load first image from file
            //qWarning( "Could not read default image, using single-color instead." );
            QImage dummy( 256, 256, 32 );
            dummy.fill( Qt::blue.rgb() );
            loadimage = dummy;
            itworked = FALSE;
        }

    }
    else
    {
        // this routine loads the image
        if ( loadFileintoimage( loadfilename, loadimage ) )
        {
            itworked = true;
        }
        else
        {
            itworked = false;
        }
    }


    return loadimage;
};

bool assetExchange::streamMediaintoimage( const QString loadfilename, QImage & image, int frame)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    bool returnval = TRUE;

    //mediaData * mediaClip = new mediaData;
    if (streamOpen != true)
    {
        //jtrace.debug("stream wasnt open, opening");
        if (openStream->openMedia(loadfilename))  //was filename
        {
            streamOpen = true;
            image = openStream->getVideoFrame(frame);	
        }
    }

    else 
    {   
        //jtrace.debug("stream was open");
        image = openStream->getVideoFrame(frame);	
    }

    return returnval;
}



QString assetExchange::getsequencefilename( videoData& theasset, int frame )
{


//    debug(">>>getting sequence filename");

    //////////////////////////////////////////////
    // we really need to optimize this routine mon
    // as its important and accessed all the time

    //keyFlag false means not the key and true means the key we can fix this later

    //calculate the sequence template for sequences
    //when support is added for avi and movie files need to update LoadLayerAsset as well

    //first we need to map the frame to the associated frame number in the sequence
    frame = (theasset.startframe + frame)-1;

    //make sure we are in range, if we are out of range use the head or tail
    //we can add a flag here to hide clip if we want
    if ( frame> theasset.endframe)         { frame= theasset.endframe; }
    if ( frame< theasset.startframe)       { frame= theasset.startframe; }

    //first figure out length of current frame
    QString currentframe;  currentframe.setNum(frame); //was theblankframe
    int     length = currentframe.length();

    //then remove it from the blank template for the clip
    QString blank(theasset.videoext);
    blank          = blank.remove(0, length);

    //now we need to build the blank from scratch to account for sequences begining with 0001 etc
    int blanklength = blank.length();
    QString theblank;
    for (int i=0; i<blanklength; i++) {
        theblank +="0";
    }

    blank = theblank;

    //then create the sequence extension
    QString temp; temp = blank + currentframe;

    //now we set the image name
    QString imagefilepath;

    imagefilepath = theasset.location+theasset.filename+temp+theasset.extension;

    return imagefilepath;

}

/////////////////////////////////////////////////////////////////////////
// used to export a asset to a predetermined location in the filesystem

void assetExchange::ExportAsset(videoData &asset, QString destination)
{
/*
enum VideoType         { STILL, SEQUENCE, MOVIE, AVI, MPEG, MP3, AIFF };
*/
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "assetExchange::","Calling ExportAsset()");

    VideoType LayerType = asset.theType;
    VideoCategory LayerCat= asset.theCategory;

    bool copyit = false;

    assetExchange exchange;
    QString assetname;

    QDir d( destination );                        // "./example"

    if ( d.exists() )
    {

        if (LayerType==STILL )
        {
            //debug("easy copy still");
            exchange.getMediaFilename(asset, 0, assetname);
            //debug("copy from:"+assetname);
        //copy assetname to destination

            QString destinationFile;
            destinationFile = destination + "/" + asset.filename + asset.extension;

            //debug("copy to:"+destinationFile);

            copyFile( assetname, destinationFile );


        }

        if (LayerType==MOVIE || LayerCat == MEDIA )
        { 
            //debug("easy copy still");
            exchange.getMediaFilename(asset, 0, assetname);
            //debug("copy from:"+assetname);
        //copy assetname to destination

            QString destinationFile;
            destinationFile = destination + "/" + asset.filename + asset.extension;

            //debug("copy to:"+destinationFile);

            copyFile( assetname, destinationFile );


        }

        if (LayerType==SEQUENCE)  
        {

        QString destDir,location,rere;

        destDir             = destination+"/"+asset.clipname+rere.setNum(asset.clipnumber);

        //debug("saving:"+destDir);

        //here we make the dir if it doesnt exist
        QDir d( destDir );                        // "./example"
        if ( !d.exists() )
        {
            //debug( "Cannot find the render directory - good" );
            if (d.mkdir( destDir ))
            {
                jtrace.debug("created directory",destDir );
                //location    = jahrenderpath+moduleinfo + temp.setNum(clipnum) + "/";
            }
            else
            {
                //debug("couldnt create directory using root:"+destDir );
                destDir             = destination;
            }
        }
        else
        {
            //the render folder already existed so just stick it in renders folder
            //debug("directory was already there using root"+destDir );
            destDir             = destination;

        }


            int startframe = asset.startframe;
            int endframe   = asset.endframe;

            for (int x = startframe; x<=endframe; x++)
            {
                    exchange.getMediaFilename(asset, x, assetname);

                    //cut the file name off the end and use it to copy to
                    //need a function to return it alone...

                    //debug("copy from:"+assetname);

                    QString destinationFile;

                    //int i = jahLoadfilename.findRev("/");
                    int i = assetname.findRev("/");
                    QString newname = assetname.right(assetname.length()-(i+1));
                    //debug("assetname:"+newname);


                    destinationFile = destDir + "/" + newname;//asset.filename + asset.extension;

                    //debug("copy to:"+destinationFile);
                    copyFile( assetname, destinationFile );
            }


        }

    }
}


bool assetExchange::copyFile( const QString oldFilePath, const QString newFilePath)
{

    //same file, no need to copy
    if(oldFilePath.compare(newFilePath) == 0)
        return true;

    //load both files
    QFile oldFile(oldFilePath);
    QFile newFile(newFilePath);
    bool openOld = oldFile.open( IO_ReadOnly );
    bool openNew = newFile.open( IO_WriteOnly );

    //if either file fails to open bail
    if(!openOld || !openNew) { return false; }

    //copy contents
    uint BUFFER_SIZE = 16000;
    char* buffer = new char[BUFFER_SIZE];
    while(!oldFile.atEnd())
    {
        Q_ULONG len = oldFile.readBlock( buffer, BUFFER_SIZE );
        newFile.writeBlock( buffer, len );
    }

    //deallocate buffer
    delete[] buffer;
    buffer = NULL;
    return true;


}
