/*******************************************************************************
**
** The source file for the Jahshaka desktop database file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "jahdatabase.h"


//is this callback ever used?
//probably for dbase error reporting?

static int callback(void *NotUsed, int argc, char **argv, char **azColName)
{
	int i;
	QString data, data1, data2;

	for(i=0; i<argc; i++)
	{
		data1 = azColName[i]; data2 = argv[i];
		data  = data1 + " = " +  data2;
		debug(data);
	}
	return 0;
}

JahDatabase::JahDatabase(const char* name)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("Setting up the Database Library");

   //initialize JahBasePath
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath   = jprefs.getBasePath();
    JahMediaPath  = jprefs.getMediaPath();



}

////////////////////////////////////////////////////////
//Initialize and Open the database

void JahDatabase::InitializeDatabase(void)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    char *zErrMsg = 0;

    QString databasename = "JahDesktopDB";
    QString jahdatabase  = JahBasePath+"database/"+ databasename;

    jtrace.info( "Initializing Jahshaka Database at:",jahdatabase );

    //execute the sql commnad to open the database
    db = sqlite_open(jahdatabase, 0, &zErrMsg);

    if( db==0 )
    {
        jtrace.error( "Unable to open the database");
        jtrace.error( "Error",zErrMsg);
        exit(1);
    }
    else
    {
        jtrace.debug("Initialized Database");
    }

}

int JahDatabase::TestDatabase(void)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info("Testing the database");

    int numclip = OpenDesktopTable();

    ///////////////////////////////////////////////////////////////////////
    // if there are one or more clips registered then test the database and
    // load all the clips into the desktop!

    //if (numclip>0)
    //{
        // test database first
        jtrace.info("Testing the database");

        //gets number of entries in database
        //int counter = 3;
        int counter = GetDatabaseClipNumber();

        //now we know clipnumber and number of entries
        if (counter==numclip)
        { 
            jtrace.debug("Database is ok");
        }
        else
        {
            jtrace.warn("database got issues");

            if (counter>numclip) 
            {
                jtrace.warn("Extra assets! Salvaging missing assets");
                numclip=counter;
            }

            if (counter<numclip)
            {
                jtrace.warn("Missing assets! Resetting asset clipnumber");
                numclip= counter;
            }
                        //rewrite desktop table data in case we crash again!
        }

        jtrace.debug("Finished Testing the database");

        return numclip;


    /*}
    else
    {
        jtrace.debug("Database is empty");
        return;
    }
*/

}

void JahDatabase::CloseDatabase(int clipnum)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info("Closing Jahshaka Database");

    // update up the desktop database both name and clipnumber...
    char *zErrMsg = 0;
    int  rc;

    //////////////////////////////////////////////////////
    // update desktop table

    //get the global clipnumber
    QString conversion;
    //int clipnum      = *clipnumber;
    QString numclips = conversion.setNum(clipnum);

    //build the sql command
    QString addtodbase = QString("update desktop set  numclips= %1").arg(numclips);
    jtrace.debug(addtodbase);

    //execute the sql command
    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

    //close the database
    sqlite_close(db);   

    jtrace.debug("Closed Jahshaka Database");

}


bool JahDatabase::WipeDatabase(void) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info("Wiping Jahshaka Database");

    // update up the desktop database both name and clipnumber...
    char *zErrMsg = 0;
    int  rc;
    QString sql;

    //build the sql command
    sql = QString("update desktop set  numclips= 0");
    rc = sqlite_exec(db, sql, callback, 0, &zErrMsg);

    if( rc!=SQLITE_OK ) { return false; }

    //build the sql command
    sql = QString("delete from videodata");
    rc = sqlite_exec(db, sql, callback, 0, &zErrMsg);

    if( rc!=SQLITE_OK ) { return false; }

    return true;

    jtrace.debug("Wiped Jahshaka Database");

}

//update the database with new information from
//videData on a clips position once it has been moved
//need some error checking done here

void JahDatabase::updateClipPosition(videoData data)
{
    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = data.clipnumber;
    double posx          = data.xpos;
    double posy          = data.ypos;

    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);
    QString posx_st = conversion.setNum(posx);
    QString posy_st = conversion.setNum(posy);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  xpos = %1 where MediaNumber = %2")
                        .arg(posx_st)
                        .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

    //build and execute the sql command to update ypos
    addtodbase = QString("update videodata set  ypos = %1 where MediaNumber = %2")
                        .arg(posy_st)
                            .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

}

//this command reads in the number of clips 
//from the desktop database table

int JahDatabase::OpenDesktopTable()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("Accessing Desktop Table");

    int numclip;

    char *zErrMsg = 0;
    int   rc,max_rows,max_cols;
    char  **desktopdata;

    //build the sql command here
    char   *sql; sql = "select * from desktop;"; //where desktop number = 1 for multiple desktops


    rc  = sqlite_get_table(db, sql, &desktopdata, &max_rows, &max_cols, &zErrMsg);

    if( rc!=SQLITE_OK )
    { 
        jtrace.error("SQL Error bad database"); 
        numclip = -1; 
    }
    else 
    { 
        jtrace.debug("Accessed desktop table");

        //object data
        QString desktopname    = desktopdata[2];  
        jtrace.debug("Desktop name:",desktopname);

        QString numberofclips  = desktopdata[3]; 
        jtrace.debug("Number clips:",numberofclips);

        numclip = numberofclips.toInt();
    }

    return numclip;

}



//this is the main startup routine that rebuilds the desktop
//from the database by reading in all assets
//break this out into separate routines to make it easier to deal with...
//make this return bool for success to trap errors

///////////////////////////////////////////////////////////
// gets the number of each desktop clip from the database
// not the desktop so we can verify in case there was a crash and recover

int JahDatabase::GetDatabaseClipNumber()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Retreiving number of clips stored in database clip" );

    int thecounter=0;

    int theloop=1; //how about bool status here outside the loop

    QString conversion;
    QString theclipid;
    QString sqlcom;
    QString theclipnum;

    char  **videodata;
    int rc,max_rows,max_cols;
    char *zErrMsg = 0;
    bool thestatus=true;

    sqlcom = "select count (*) from videodata";

    const char* fontchar =  sqlcom;
    char*  sqlcommand    =  new char[ sqlcom.length() ];
    strcpy(sqlcommand,fontchar);

    //execute the aql command here
    rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);


    //need to clean this up
    if( rc!=SQLITE_OK )
    {
        jtrace.error("GetDatabaseClipNumber","SQL Error in database"); 
        thestatus=false;
        return -1;
    }

    theclipid = videodata[1];
    //jtrace.debug(theclipid);

    thecounter = theclipid.toInt();

    QString temp;
    jtrace.debug("The number returned was"+temp.setNum(thecounter));

    return thecounter;

}


//need to clean this up and add new data types such as SEQUENCE and videoext, keyext, etc

void JahDatabase::addImage2database( videoData thedata )
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Adding Asset to database" );

    //we need to add this data to the database JahDB
    //but we need to make sure its open first...

    int rc;  char *zErrMsg = 0;

    //create add string
    QString addtodbase; QString conversion;

    QString category   = conversion.setNum(thedata.theCategory);  //enum
    QString cliptype   = conversion.setNum(thedata.theType);  //enum
    QString clipname   = thedata.clipname;
    QString clipnumber = conversion.setNum(thedata.clipnumber);   //int
    QString location   = thedata.location;
    QString filename   = thedata.filename;
    QString extension  = thedata.extension;
    QString startframe = conversion.setNum(thedata.startframe);   //int       .setNum(value)
    QString endframe   = conversion.setNum(thedata.endframe);     //int
    QString xypos      = conversion.setNum(thedata.xpos) +","+conversion.setNum(thedata.ypos);
    QString videoext   = thedata.videoext;

    QString thetable="videodata";

    addtodbase = QString("insert into videodata values( '%1','%2', '%3','%4','%5','%6','%7','%8', ")
                .arg(category)
                .arg(cliptype)
                .arg(videoext)
                .arg(clipname)
                .arg(clipnumber)
                .arg(location)
                .arg(filename)
                .arg(extension);

    addtodbase += startframe + ", " + endframe + ", " + xypos + ");";

    jtrace.debug( "Adding Asset",addtodbase );

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

    jtrace.debug("Added Asset to database");

}


void JahDatabase::removeImagefromdatabase( int imagenumber ) 
{

//we need to remove the data from the database
//for the case where clipnumber = imagenumber
// which is field MediaNumer in table videodata

    //for now we will just de-activate clips
    //until we can figure this out properly
    //we do this by setting category to 9

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahDatabase::","deleting asset from database" );

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  category = 9 where MediaNumber = %1")
                        .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);
    jtrace.debug( "JahDatabase::","deleted asset from database" );


}

void JahDatabase::deleteImagefromdatabase( int imagenumber ) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahDatabase::","erase asset from database" );

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("delete from videodata where MediaNumber = %1")
                        .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);


}

void JahDatabase::renameImageindatabase( int imagenumber, QString newname) 
{

//we need to remove the data from the database
//for the case where clipnumber = imagenumber
// which is field MediaNumer in table videodata

    //for now we will just de-activate clips
    //until we can figure this out properly
    //we do this by setting category to 9

    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahDatabase::","rename asset in database" );

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  MediaName = '%1' where MediaNumber = %2")
                        .arg(newname)
                        .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);


}

///////////////////////////////////////////////////////////
// gets the number of each desktop clip from the database

videoData JahDatabase::getImageFromDatabase(int clipnumber)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.info("getting asset from database");


    //this is the asset we return
    videoData thedata;

    //first see if it exists

    if (!doesAssetExistindatabase(clipnumber))
    {
            //jtrace.debug("bad asset, unknown category");
            int thecategory     = -1;
            thedata.theCategory = (VideoCategory)thecategory;

            return thedata;


    }

    //build the sql command
    QString conversion;
    QString theclipnum   = conversion.setNum(clipnumber);  //enum

    //workaround - if theres no name theres no clip
    QString sqlcom = "select * from videodata where MediaNumber="+theclipnum;

    //better way to do this ?
    const char* fontchar =  sqlcom;
    char*  sqlcommand    =  new char[ sqlcom.length()+3 ];
    strcpy(sqlcommand,fontchar);

    char  **videodata = NULL;
    int rc,max_rows,max_cols;
    char *zErrMsg = 0;

    rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);

        //we should set the clip name to something weird to trap errors
        if( rc!=SQLITE_OK )
        {
            jtrace.debug("ERROR locating clip in database");
            //fprintf(stderr, "SQL error: %s\n", zErrMsg);
        }
        else
        {
            //jtrace.debug("located clip in database");
        }


        // we start at 12 because there are 11 column headers that come first
        QString category    = videodata[12];
        QString cliptype    = videodata[13];
        QString videoext    = videodata[14];
        QString clipname    = videodata[15];
        QString clipnum     = videodata[16];
        QString location    = videodata[17];
        QString filename    = videodata[18];
        QString extension   = videodata[19];
        QString startframe  = videodata[20];
        QString endframe    = videodata[21];
        QString xpos        = videodata[22];
        QString ypos        = videodata[23];

        int thecat = category.toInt();

        if ( (thecat >=0) && (thecat <=5) )
        {
            int thecategory     = category.toInt();
            thedata.theCategory = (VideoCategory)thecategory;

            int thecliptype     = cliptype.toInt();
            thedata.theType     = (VideoType)thecliptype;

            thedata.clipname    = clipname;    //clipname is truncated filename
            thedata.videoext    = videoext;    //clipname is truncated filename
            thedata.clipnumber  = clipnum.toInt();
            thedata.location    = location;    //the path without the filename...
            thedata.filename    = filename;     //the filename on disk without extension
            thedata.extension   = extension; //we need to change this based on extension
            thedata.startframe  = startframe.toInt();
            thedata.endframe    = endframe.toInt();
            //thedata.keyStatus   = FALSE;
            thedata.xpos        = xpos.toInt();
            thedata.ypos        = ypos.toInt();

            switch (thedata.theCategory) {

                    case IMAGE  : { thedata.videotex = FALSE;  break; }
                    case CLIP   : { thedata.videotex = TRUE;  break;  }
                    case MEDIA  : { thedata.videotex = TRUE;  break;  }
                    case CLIPKEY: { thedata.videotex = TRUE;  break;  }

                    default     : { jtrace.warn("unknown asset category"); break; } //KEY, CLIPKEY and AUDIO not used
            }

        }
        else
        {
            //jtrace.debug("bad asset, unknown category");
            int thecategory     = -1;
            thedata.theCategory = (VideoCategory)thecategory;

        }

    return thedata;

}

bool JahDatabase::doesAssetExistindatabase(int clipnum)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "checking to see if asset exists",clipnum );

    int thecounter=0;

    int theloop=1; //how about bool status here outside the loop

    QString conversion;
    QString theclipid;
    QString sqlcom;
    QString theclipnum;

    char  **videodata;
    int rc,max_rows,max_cols;
    char *zErrMsg = 0;
    bool thestatus=true;



    sqlcom = "select count (*) from videodata where MediaNumber = "+conversion.setNum(clipnum);

    const char* fontchar =  sqlcom;
    char*  sqlcommand    =  new char[ sqlcom.length() ];
    strcpy(sqlcommand,fontchar);

    //execute the aql command here
    rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);


    //need to clean this up
    if( rc!=SQLITE_OK )
    {
        jtrace.error("GetDatabaseClipNumber","SQL Error in database"); 
        thestatus=false;
        return false;
    }

    theclipid = videodata[1];

    thecounter = theclipid.toInt();

    QString temp;
    jtrace.debug("The number returned was"+temp.setNum(thecounter));


    return thecounter;

}

///////////////////////////////////////////////////////////
// gets the number of each desktop clip from the database

videoData JahDatabase::restoreImageFromDatabase(int clipnumber)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.info("getting asset from database");


    //this is the asset we return
    videoData thedata;

    //first see if it exists

    //build the sql command
    QString conversion;
    QString theclipnum   = conversion.setNum(clipnumber);  //enum


    //we should use this on startup to restore the database...
    QString sqlcom = "select * from videodata where ROWID ="+theclipnum;

    //better way to do this ?
    const char* fontchar =  sqlcom;
    char*  sqlcommand    =  new char[ sqlcom.length()+3 ];
    strcpy(sqlcommand,fontchar);

    char  **videodata = NULL;
    int rc,max_rows,max_cols;
    char *zErrMsg = 0;

    rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);

        //we should set the clip name to something weird to trap errors
        if( rc!=SQLITE_OK )
        {
            jtrace.debug("ERROR locating clip in database");
            //fprintf(stderr, "SQL error: %s\n", zErrMsg);
        }
        else
        {
            //jtrace.debug("located clip in database");
        }


        // we start at 12 because there are 11 column headers that come first
        QString category    = videodata[12];
        QString cliptype    = videodata[13];
        QString videoext    = videodata[14];
        QString clipname    = videodata[15];
        QString clipnum     = videodata[16];
        QString location    = videodata[17];
        QString filename    = videodata[18];
        QString extension   = videodata[19];
        QString startframe  = videodata[20];
        QString endframe    = videodata[21];
        QString xpos        = videodata[22];
        QString ypos        = videodata[23];

        int thecat = category.toInt();

        if ( (thecat >=0) && (thecat <=5) )
        {
            int thecategory     = category.toInt();
            thedata.theCategory = (VideoCategory)thecategory;

            int thecliptype     = cliptype.toInt();
            thedata.theType     = (VideoType)thecliptype;

            thedata.clipname    = clipname;    //clipname is truncated filename
            thedata.videoext    = videoext;    //clipname is truncated filename
            thedata.clipnumber  = clipnum.toInt();
            thedata.location    = location;    //the path without the filename...
            thedata.filename    = filename;     //the filename on disk without extension
            thedata.extension   = extension; //we need to change this based on extension
            thedata.startframe  = startframe.toInt();
            thedata.endframe    = endframe.toInt();
            //thedata.keyStatus   = FALSE;
            thedata.xpos        = xpos.toInt();
            thedata.ypos        = ypos.toInt();

            switch (thedata.theCategory) {

                    case IMAGE  : { thedata.videotex = FALSE;  break; }
                    case CLIP   : { thedata.videotex = TRUE;  break;  }
                    case MEDIA  : { thedata.videotex = TRUE;  break;  }
                    case CLIPKEY: { thedata.videotex = TRUE;  break;  }

                    default     : { jtrace.warn("unknown asset category"); break; } //KEY, CLIPKEY and AUDIO not used
            }

        }
        else
        {
            //jtrace.debug("bad asset, unknown category");
            int thecategory     = -1;
            thedata.theCategory = (VideoCategory)thecategory;

        }

    return thedata;

}


/////////////////////////////////////////////////////
// called on exit in order to purge deactivated clips

//the cool part here is when erase doesnt actuall erase the clips... just hides them
//so you just purge the database when youre running out of storage
//instead of a index we can erase all hidden clips and renumber the database from start to finish
// ie set medianumber from 1 to xxx after removing all dead entries and deleting the files
//then we can set globalclipnumber back...

// old notes here

// but first we really need a index field since the clinumber is also part of
// the pathname for rendered clips
// but the general idea is to go through all clips
// erase the ones with a 9
// delete associated files
// and then reindex the clips

void JahDatabase::purgeDatabase( int imagenumber ) {
/*

    char *zErrMsg = 0;
    int   rc,max_rows,max_cols;
    char  **desktopdata;

    //build the sql command here
    char   *sql; sql = "select * from desktop;"; //where desktop number = 1 for multiple desktops

    rc  = sqlite_get_table(db, sql, &desktopdata, &max_rows, &max_cols, &zErrMsg);

    if( rc!=SQLITE_OK ){ fprintf(stderr, "SQL error: %s\n", zErrMsg); }
                        else { debug("Opened desktop table");  }

    //object data
    QString desktopname    = desktopdata[2];  debug(desktopname);
    QString numberofclips  = desktopdata[3];  debug(numberofclips);

    int numclip = numberofclips.toInt();

    //now we know howmany clips are in the database
    QString conversion;
    QString theclipnum       = conversion.setNum(numclip);

    int counter = GetDatabaseClipNumber();
    QString theclipcounter   = conversion.setNum(counter);

    debug("the clip number read is "+theclipnum);
    debug("the clip counter read   "+theclipcounter);




    int counter;

    int loop=1; //how about bool status here outside the loop

    while ( loop ) { //fix this to keep looping until it breaks

        //build the sql command
        QString theclipnum   = conversion.setNum(loop);  //enum

        //workaround - if theres no name theres no clip
        QString sqlcom = "select MediaName from videodata where MediaNumber="+theclipnum;

        //better way to do this ?
        const char* fontchar =  sqlcom;
        char*  sqlcommand    =  new char[ sqlcom.length() ];
        strcpy(sqlcommand,fontchar);

        char  **videodata;
        int rc,max_rows,max_cols;
    char *zErrMsg = 0;


        rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);

        QString clipnum;
        bool status;

        if( rc==SQLITE_OK )  { clipnum = videodata[0];
            if (clipnum == "MediaName") { status=true; } else { status=false; }	
        } else { status=false; }

        if (status) { debug("true");  counter = loop; loop++; } else { break; }
    }

    return counter;

*/

}
