/*******************************************************************************
**
** The header file for the Jahshaka jahpreferences object 
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "jahkeyframeobject.h"

////////////////////////////////////////////////////////////

JahKeyframeObject::JahKeyframeObject() 
{
    //set up the nodes for keyframes
    numnodes        = 0;
    node            = new motionNode;    // node is always equal to the current node
    
    ///////////////////////////////////////////////
    // the linked list is here
    nodelist = new QPtrList<motionNode>;
    nodelist->setAutoDelete( TRUE );     // delete items when they are removed!!!
    
    node->clear();
    //clear();

}

JahKeyframeObject::~JahKeyframeObject() 
{

}


//routine to reset and clear out the keyframes for a layer
void JahKeyframeObject::reset(void) 
{

    nodelist->clear();
    delete nodelist;

    nodelist = new QPtrList<motionNode>;
    nodelist->setAutoDelete( TRUE );     // delete items when they are removed!!!

    numnodes = 0;

    delete node; 
    node = new motionNode;   // node is always equal to the current node

    node->clear();

}

//routine to clear the setting for the current default node
void JahKeyframeObject::clear(void)
{

    node->clear();
}

//////////////////////////////////////////////////////////////////
//routine to add a keyframe 
//need errorchecking if  keyframe already has a node 
//or if this is the first keyframe                              
void JahKeyframeObject::addNodes(int currentFrame, motionNode * newNode) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahKeyframes::addNodes","adding a new node");

    //before adding a node we must check to see if a node exists for the frameNumber
    bool newnode = true;
    int  thenode = -1;


    if (doesKeyframeExist(currentFrame))
    {
        thenode = getKeyframe(currentFrame);
    }

    if (thenode== -1) 
    { 
        newnode = true; 
    } 
    else 
    {
        newnode = false; 
    }


    if (newnode)
    {
        //this is the new linked list
        //debug("updating new node");
        nodelist->append( new motionNode );    
        nodelist->at(numnodes)->copy(newNode);
        nodelist->at(numnodes)->nodenum = currentFrame;

        numnodes+= 1;

        //now we sort the nodes to keep keyframes in chronological order...    
        sortNodes();
    }
    else
    {
        //update node number thenode
        //with data in newNode
        //debug("updating old node");
        nodelist->at(thenode)->copy(newNode);
    }

}

void JahKeyframeObject::removeNodes(int theFrame) 
{
    //debug("removing node");

    //we go through the linked list searching for the node
    //and if we find it we delete it
    for ( uint x = 0; x <= nodelist->count(); ++x )    
    {
        if ( nodelist->at(x) ) 
        {
            if ( theFrame == nodelist->at(x)->nodenum ) 
            {
                //debug("removing it");
                nodelist->remove(x); 
                numnodes-= 1;
                sortNodes();
                break;
            } 
        }
    }

}

int JahKeyframeObject::getNextFrame(int theFrame) 
{
    int prevframe;
    bool status = false;

    //dont think we need this
    sortNodes();

    if (doesKeyframeExist(theFrame))
    {
        prevframe = getKeyframe(theFrame);
        status = true;
    }
    else
    {
        //to go to the next keyframe we need to get the one before...
        for ( uint x = 0; x <= nodelist->count(); x++ )    
        {	
            if ( nodelist->at(x) ) 
            {
                int thanode = nodelist->at(x)->nodenum;
                if ( theFrame > thanode ) 
                {
                    prevframe = (int)x;
                    status = true;
                }
            }
        }
    }

    //this is the next node
    bool foundframe = false;
    int newframe;

    if (status) 
    {
        //increment to next node
        prevframe+=1;

        if ( nodelist->at(prevframe) ) 
        {
            newframe = nodelist->at(prevframe)->nodenum;
            foundframe = true;
        }

    }

    if (foundframe) return newframe; else return -1;
}

int JahKeyframeObject::getPrevFrame(int theFrame) 
{
    int prevframe;
    bool status = false;
    bool existed = false;

    //dont think we need this
    sortNodes();

    if (doesKeyframeExist(theFrame))
    {
        prevframe = getKeyframe(theFrame);
        status = true;
        existed = true;
    }
    else
    {
        //to go to the next keyframe we need to get the one before...
        for ( uint x = 0; x <= nodelist->count(); x++ )    
        {	
            if ( nodelist->at(x) ) 
            {
                int thanode = nodelist->at(x)->nodenum;
                if ( theFrame > thanode ) 
                {
                    prevframe = (int)x;
                    status = true;
                }
            }
        }
    }

    //this is the next node
    bool foundframe = false;
    int newframe;

    if (status) 
    {
        //increment to next node
        if (existed)
        prevframe-=1;

        if ( nodelist->at(prevframe) ) 
        {
            newframe = nodelist->at(prevframe)->nodenum;
            foundframe = true;
        }

    }

    if (foundframe) return newframe; else return -1;
}



//returns -1 if frame is not found 
bool JahKeyframeObject::doesKeyframeExist(int theFrame) 
{
    JahTrace& jtrace = JahTrace::getInstance();

    bool newnode = false;
    int foundFrame;

    for ( uint x = 0; x <= nodelist->count(); ++x )    
    //for (int x=0; x<numnodes; x++) 
    { 
        if (nodelist->at(x))
        {
            if (theFrame == nodelist->at(x)->nodenum)
            {
                newnode = true;
                foundFrame = x;
                jtrace.debug(">>>>found old node");
                break;
            }
        }

    }

    if (newnode) return true;
    else         return false;


}

//returns -1 if frame is not found 
int JahKeyframeObject::getKeyframe(int theFrame) 
{

    bool newnode = false;
    int foundFrame;

    for ( uint x = 0; x <= nodelist->count(); ++x )    
    //for (int x=0; x<numnodes; x++) 
    { 
        if (nodelist->at(x))
        {
            if (theFrame == nodelist->at(x)->nodenum)
            {
                newnode = true;
                foundFrame = (int)x;
                //debug(">>>>found old node");
                break;
            }
        }

    }

    if (newnode) return foundFrame;
    else         return -1;


}
//used to sort the keyframes this needs a rewrite as its very messy
//the linked lists should make this real easy
//see void QPtrList::sort ()
void JahKeyframeObject::sortNodes(void) 
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("sorting layers");

    //need more than 2 nodes in order to sort 
    if (numnodes<2) return;

    //first we need to get all the keyframes and sort them...
    int * a = new int[numnodes];

    for (int x=0; x<numnodes; x++) { a[x] = nodelist->at(x)->nodenum; }

    // This is where we do the sorting
    for (int i = 0; i < (numnodes - 1); i++)
    {
        int minIndex = i;

        // Find the index of the minimum element
        for (int j = i + 1; j < numnodes; j++)
        {
            if (a[j] < a[minIndex]) { minIndex = j; }
        }

        // Swap if i-th element not already smallest
        if (minIndex > i) { swap(a[i], a[minIndex]); }
    }

    //now we copy the nodes in order to the sortnodes object
    motionNode * sortnodes = new motionNode[numnodes];

    for (int totalnodes =0; totalnodes<numnodes; totalnodes++) 
    {
    for (int aa=0; aa<=numnodes; aa++) 
    {
        if(nodelist->at(aa)->nodenum == a[totalnodes]) 
        {
            sortnodes[totalnodes].copy(nodelist->at(aa));
            break;
        }
    }
    }

    //the we copy the sorted nodes back into the main nodes object
    for (int xx =0; xx<numnodes; xx++) 
    { 
        nodelist->at(xx)->copy(&sortnodes[xx]);
    }

    delete [] sortnodes;

}

//used by the sort routine above
void JahKeyframeObject::swap(int &x, int &y)
{
    int temp;

    temp = x;
    x = y;
    y = temp;
}

//updates the current node with the interpolation
void JahKeyframeObject::updateNodes(int currentFrame) 
{
    //updates node to value at current keyframe
    getNode( currentFrame, node);
}


void JahKeyframeObject::getFirstLastFrame( int *firstFrame, int *lastFrame) 
{   
    //find frame before and frame after
    //set new frame in between

    *firstFrame = nodelist->at(0)->nodenum;  //nodenum is the keyframe that node was set at
    *lastFrame  = nodelist->at(numnodes-1)->nodenum;
}

///////////////////////////////////////////////////////////////////
//this is a linear interpreter for the keyframe values
//we need to add a bezier as well and clean this up
//to make it faster
int JahKeyframeObject::getNode(int frameNumber, motionNode *mn) 
{

    int   pn=0, nn=0, a = 0;
    int   firstFrame, lastFrame;

    ///////////////////////////////////////////////////
    //first we do all the error checking

    getFirstLastFrame(&firstFrame, &lastFrame);

    if(frameNumber < firstFrame) frameNumber = firstFrame;
    if(frameNumber > lastFrame)  frameNumber = lastFrame;

    for (int aa=0; aa<numnodes; aa++) 
    {
        if(nodelist->at(aa)->nodenum == frameNumber) 
        {
            mn->copy(nodelist->at(aa) );
            return 0;
        }
    }

    while(a < (numnodes -1 )) 
    {
        pn = nodelist->at(a)->nodenum;
        nn = nodelist->at(a+1)->nodenum;
        if(pn < frameNumber && nn > frameNumber) break;
        a++;
    }

    if (a == (numnodes-1)) 
    { 
        mn->clear();
        return -1;
    }

    //////////////////////////////////////////////////////
    // this is where we do the math for the interpolation
    getLinearInterpolation(mn, nn, pn, a, frameNumber);

    return 0;
}

void JahKeyframeObject::getLinearInterpolation(motionNode *mn,int nn, int pn, int a, int frameNumber) 
{
    //define variables
    float diff, mult, dist;

    //do the interpolation
    diff = nn - pn;
    mult = (float)(frameNumber - pn) / diff;

    dist      = nodelist->at(a + 1)->sx    - nodelist->at(a)->sx;    
    mn->sx    = nodelist->at(a)->sx + dist * mult;

    dist      = nodelist->at(a + 1)->sy    - nodelist->at(a)->sy;    
    mn->sy    = nodelist->at(a)->sy + dist * mult;

    dist      = nodelist->at(a + 1)->sz    - nodelist->at(a)->sz;    
    mn->sz    = nodelist->at(a)->sz + dist * mult;

    dist      = nodelist->at(a + 1)->rx    - nodelist->at(a)->rx;    
    mn->rx    = nodelist->at(a)->rx + dist * mult;

    dist      = nodelist->at(a + 1)->ry    - nodelist->at(a)->ry;    
    mn->ry    = nodelist->at(a)->ry + dist * mult;

    dist      = nodelist->at(a + 1)->rz    - nodelist->at(a)->rz;    
    mn->rz    = nodelist->at(a)->rz + dist * mult;

    dist      = nodelist->at(a + 1)->tx    - nodelist->at(a)->tx;    
    mn->tx    = nodelist->at(a)->tx + dist * mult;

    dist      = nodelist->at(a + 1)->ty    - nodelist->at(a)->ty;    
    mn->ty    = nodelist->at(a)->ty + dist * mult;

    dist      = nodelist->at(a + 1)->tz    - nodelist->at(a)->tz;    
    mn->tz    = nodelist->at(a)->tz + dist * mult;

    dist      = nodelist->at(a + 1)->Alpha - nodelist->at(a)->Alpha; 
    mn->Alpha = nodelist->at(a)->Alpha + dist * mult;

    dist      = nodelist->at(a + 1)->bx    - nodelist->at(a)->bx;    
    mn->bx    = nodelist->at(a)->bx + dist * mult;

    dist      = nodelist->at(a + 1)->by - nodelist->at(a)->by; 
    mn->by    = nodelist->at(a)->by + dist * mult;

    mn->nodenum = frameNumber;
}





