/* flip - pageflipper for TIFF animations
* Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
*
* "flip" is free software; you can redistribute it and/or use
* it and/or modify it under the terms of the "Artistic License"
*/

#include "JahPlayer.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qwmatrix.h>

#ifndef _WIN32
#include <unistd.h>
#endif

#define SWAP_THRESH 1.5f

static char	errorMessage[256];
#ifdef _WIN32
#define snprintf _snprintf
int getpid2() {return 1;};
#endif

//------------------------------------------------
JahPlayer::JahPlayer(QWidget *parent, const char* name)
    :QWidget (parent, name)
{
    //videohead preferences
    videohead = false;

    pixlist		= 0; 
    numFrames	= 0;
    maxFrames	= 0;
    frame		= 0;
    width		= 0;
    height		= 0;
    origWidth	= 0;
    origHeight	= 0;
    loadZoom	= 1;
    displayZoom	= 1;
    assetdefined = false;

    QFont	font("Helvetica", 10);

    memLabel 	= new QLabel(this);
    memLabel->setFixedSize(80, 15);
    memLabel->move(5, 5);
    memLabel->setFont(font);
    memLabel->setAlignment(AlignCenter);

    swapLabel	= new QLabel(this);
    swapLabel->setFixedSize(80, 15);
    swapLabel->move(5, 25);
    swapLabel->setFont(font);
    swapLabel->setAlignment(AlignCenter);

    swapThresh = SWAP_THRESH;

    //for drag and drop
    setAcceptDrops(TRUE);
}


//------------------------------------------------
JahPlayer::~JahPlayer()
{
    if (pixlist) delete pixlist;
}

void JahPlayer::dragEnterEvent(QDragEnterEvent* event)
{
    event->accept(
        QTextDrag::canDecode(event) ||
        QImageDrag::canDecode(event)
    );

}

void JahPlayer::dropEvent(QDropEvent* event)
{
    QImage image;
    QString text;

    if ( QImageDrag::decode(event, image) )
    {
        //debug("its a image");
        //this is connected to the main app in Source/jahroutines.cpp
        emit draggedAImage();
    }
    else if ( QTextDrag::decode(event, text) )
    {
        //debug("its text");
    }

}

//------------------------------------------------
void JahPlayer::showMemInfo(bool status)
{
    if(status)
    {
        memLabel->show();
        swapLabel->show();
    }
    else
    {
        memLabel->hide();
        swapLabel->hide();
    }
}

//------------------------------------------------
void JahPlayer::Reset()
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "Resetting player");

    if ( timer.isActive() )
        timer.stop();

    if (pixlist) delete pixlist;
    if (assetdefined) { delete exchange; assetdefined=false; }

    pixlist		= 0;
    numFrames	= 0;
    maxFrames	= 0;
    frame		= 0;
    width		= 0;
    height		= 0;
    origWidth	= 0;
    origHeight	= 0;
    showMemInfo(false);
}

//------------------------------------------------
void JahPlayer::SetDisplayZoom(double z)
{
    displayZoom = z;

    mediaSize thesize;

    int thew = (int)(width * displayZoom); //was double
    int theh = (int)(height * displayZoom);

    thesize.width = thew;
    thesize.height = theh;

    //thesize.width = width * displayZoom;
    //thesize.height = height * displayZoom;

    emit resizePlayer(thesize);

    setFixedSize(thew, theh);
    m.reset();

    //matrix not scaling properly
    m.scale(displayZoom, displayZoom);

}

//------------------------------------------------
//this preps the player when called for the first time
int JahPlayer::LoadImages(videoData &desktopclip)
{

    //the local instance of the asset
    //playerAssetData = new videoData;
    //debug(">>we only call this the first time<<");
    playerAssetData = desktopclip;

    //int max = playerAssetData.getMaxFrames();
    QString startfile = playerAssetData.getFullFileName();
    //int z=1;

    numFrames	= playerAssetData.endframe;
    maxFrames	= playerAssetData.endframe;	
    startFrame  = playerAssetData.startframe; 

    QString tmpString;

    //here we initialize the assetexchange
    //we can do this in the constructor as well to be safe
    if (!assetdefined) 
    {
        exchange = new assetExchange;
        assetdefined = true;
    }
    else
    {
        delete exchange;
        exchange = new assetExchange;
        assetdefined = true;
    }

    exchange->getMediaFilename(playerAssetData, 1, tmpString);

    //debug("LoadName :"+tmpString);
    baseString = tmpString;
    maxFrames = playerAssetData.endframe;

    //allocate memory for image
    if ( ! (pixlist = new QPixmap) )
    {
        OutOfMemory();
        return -1;
    }

    swapThresh = SWAP_THRESH;
    doMemCheck = getMemUsage(&begMem, &begSwap);
    //we can use this to tell the player what memory is looking like
    //fprintf(stderr, "begin: mem %3.1f%%, swap: %3.1f%%\n", begMem, begSwap);
    showMemInfo(false);

    //we dont need to do this every time it slows tings down
    //but it initializes the loadiner image pixlist
    //move it to initializing routine
    loadFrame = startFrame;
    QImage image;

    //stream doesnt return the image the first time its caled
    //for some reason - need to figure this out as its supposed to
    //set up streaming...
    //image = exchange->getStreamImage(playerAssetData, startFrame);
    image = exchange->getImage(playerAssetData, startFrame);

    origWidth = width = image.width();
    origHeight = height = image.height();

    mediaSize thesize;
    thesize.width  = (int)(width * displayZoom);
    thesize.height = (int)(height * displayZoom);
    emit resizePlayer(thesize);

    //setFixedSize((int)(width*displayZoom), (int)(height*displayZoom));
    setFixedSize(thesize.width, thesize.height);



    //load first frame
    //debug(">>loading the first frame");
    LoadTheFrame(startFrame);
    //debug(">>loaded the first frame");

    return startFrame;
}

void	JahPlayer::LoadTheFrame(int frame)
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "JahPlaer::","loading frame number",frame);

    if (frame<0) frame = 0;

    loadFrame = frame;

    LoadSingleImage();
    SetFrame(frame);

}


//--------------------------------------------------

void	JahPlayer::LoadSingleImage()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QImage	image;

    //this is where we grab the image
    image = exchange->getStreamImage(playerAssetData, loadFrame);

    //we dont need to do this every time it slows tings down
    //but it initializes the loadiner image pixlist
    //move it to initializing routine
    //origWidth = width = image.width();
    //origHeight = height = image.height();

    //make sure this is already initialized as well 
    //so we dont have to check things here
    if (!pixlist) 
    {
        jtrace.debug("JahPlayer::","LoadSingleImage() creating new pixlist");
        if ( ! (pixlist = new QPixmap) )
        { 
            OutOfMemory();
            return; 
        }
        pixlist->setOptimization(QPixmap::NoOptim);
    }


    //need a quicker way to do this as conversion is slow...
    //maybe have asset exchange do it? return a pixmap instead?
    //we could override the routine above to do this...
    if ( ! pixlist->convertFromImage(image) )
        { OutOfMemory(); return; }

    /////////////////////////////////////////////
    //used in preloadinng and timer increments
    //dont think we need all this crap as we send the frame number
    //to be loaded over but we need it for initializing routine
    // below
    //numFrames++;
    frame = loadFrame-startFrame;

}

//------------------------------------------------
void	JahPlayer::OutOfMemory()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    timer.stop();
    jtrace.debug (errorMessage, "Could not allocate enough memory at frame", loadFrame-startFrame);
    Reset();	// free memory
    QMessageBox::critical(this, "Selah", errorMessage);
    emit Done(-1, true);
}
//------------------------------------------------
void	JahPlayer::SetFrame(int in)
{

    frame = in;
    drawPixmap();
}

//------------------------------------------------
void	JahPlayer::drawPixmap()
{
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "JahPlayer::","Drawing the pixmap");

    //anything we can do to speeeeeed this up will be a good thing
    if (numFrames > 0 && frame >= 0 && frame < numFrames)
    {
        //these bitblt the image to the display
        if (displayZoom > 1 || displayZoom < 1)
        {
        //debug("drawing here");
            static	QPixmap tempPix;
            tempPix = pixlist->xForm(m);

            bitBlt(this, 0, 0, &tempPix, 0, 0, -1, -1);
        //debug("drawing videohead");
            //here we update video head
            if (videohead)
                emit updateplayer2(tempPix);
        }
        else
        {
            bitBlt(this, 0, 0, pixlist, 0, 0, -1, -1);
            //here we update video head
            if (videohead)
                emit updateplayer2(*pixlist);
        }

        frame ++;

        if (frame >= numFrames)
            frame = 0;
    }
}

//used to show a single image...
//need to get rid of this by having player show single frames as well
/*
void JahPlayer::GetImages( QImage SentImage, int max, int z)
{
    maxFrames	= 1;	numFrames	= 1; startFrame=0; loadFrame = 0;

    if ( ! (pixlist = new QPixmap) )
    {	 OutOfMemory(); return; }

swapThresh = SWAP_THRESH;
QImage  image;    image=SentImage;
origWidth = width = SentImage.width();   // = w
origHeight = height = SentImage.height();  // = h

    if ( ! (pixlist = new QPixmap) )
    { 
        OutOfMemory(); 
        return; 
    }
pixlist->setOptimization(QPixmap::MemoryOptim);

if ( ! pixlist->convertFromImage(image, Qt::ColorOnly | Qt::NoAlpha ) )
            { OutOfMemory(); return; }

mediaSize thesize;
thesize.width = width * displayZoom;
thesize.height = height * displayZoom;

//should be only if size has changed
emit resizePlayer(thesize);
setFixedSize(width*displayZoom, height*displayZoom);
drawPixmap();		
show();
loadFrame=1;
showMemInfo(false);
return;
}
*/

//------------------------------------------------
void	JahPlayer::paintEvent(QPaintEvent * )
{
    drawPixmap();
}

//------------------------------------------------
bool	JahPlayer::checkMem()
{
    bool retVal = getMemUsage(&curMem, &curSwap);
    if(retVal)
    {
        memString.sprintf("mem: %3.1f%%", curMem);
        swapString.sprintf("swap: %3.1f%%", curSwap);
        memLabel->setText(memString);
        swapLabel->setText(swapString);
    }

    if(curSwap > begSwap)
    {
        float diff = curSwap-begSwap;
        if(diff > swapThresh)
        {
            QString mssg;
            mssg.sprintf(	"WARNING:\nswap file useage has increased by %3.1f%%.\n", diff);
        }
    }
    return retVal;
}

/**
* get percentage of memory and swap used
*/
//------------------------------------------------
bool	JahPlayer::getMemUsage(float* mem, float* swap)
{
    int c = 0;
    FILE *meminfo;

    uint swap_total	= 16;
    uint swap_used	= 0;
    uint swap_free	= 16;

    uint mem_total	= 16;
    uint mem_used	= 0;
    uint mem_free	= 16;
    uint mem_share	= 0;
    uint mem_buff	= 0;
    uint mem_cache	= 0;

    meminfo = fopen("/proc/meminfo", "r");
    if(meminfo)
    {
        char buffer[512];
        while(fgets(buffer, 512, meminfo) != NULL)
        {
            if( strncmp(buffer, "Swap:", 5) == 0 )
            {
                sscanf(buffer, "Swap: %u %u %u", &swap_total, &swap_used, &swap_free);
                c++;
            }
            else if( strncmp(buffer, "Mem:", 4) == 0 )
            {
                sscanf(buffer, "Mem: %u %u %u %u %u %u",
                    &mem_total, &mem_used, &mem_free,
                    &mem_share, &mem_buff, &mem_cache);
                c++;
            }
        }
        fclose(meminfo);
    }
    float mb = 1024.0*1024.0;
    float mTotal = (float)mem_total/mb;
    float mUsed  = (float)(mem_used-(mem_share+mem_buff+mem_cache))/mb;
    *mem = 100.0f*(mUsed/mTotal);
    float sTotal = (float)swap_total/mb;
    float sUsed  = (float)swap_used/mb;
    *swap = 100.0f*(sUsed/sTotal);

    if(c == 2)
        return true;
    else
        return false;
}




