/* flip - pageflipper for TIFF animations
* Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
*
* "flip" is free software; you can redistribute it and/or use
* it and/or modify it under the terms of the "Artistic License"
*/

#include "TiffFlip.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qwmatrix.h>

#ifndef _WIN32
//#include <tiffio.h>
#include <unistd.h>
#endif

#define SWAP_THRESH 1.5f

static char	errorMessage[256];
#ifdef _WIN32
#define snprintf _snprintf
int getpid() {return 1;};
#endif

//------------------------------------------------
TiffFlip::TiffFlip(QWidget *parent, const char* name)
    :QWidget (parent, name)
{
    //videohead preferences
    videohead = false;

    pixlist		= 0; 
    numFrames	= 0;
    maxFrames	= 0;
    frame		= 0;
    width		= 0;
    height		= 0;
    origWidth	= 0;
    origHeight	= 0;
    loadZoom	= 1;
    displayZoom	= 1;
    QObject::connect(&timer, SIGNAL(timeout()), SLOT(LoadController()) );
    //flipPID		= (int)getpid();

    QFont	font("Helvetica", 10);

    memLabel 	= new QLabel(this);
    memLabel->setFixedSize(80, 15);
    memLabel->move(5, 5);
    memLabel->setFont(font);
    memLabel->setAlignment(AlignCenter);

    swapLabel	= new QLabel(this);
    swapLabel->setFixedSize(80, 15);
    swapLabel->move(5, 25);
    swapLabel->setFont(font);
    swapLabel->setAlignment(AlignCenter);

    swapThresh = SWAP_THRESH;
}


//------------------------------------------------
TiffFlip::~TiffFlip()
{
    if (pixlist)
    {
        for (int i = 0; i < numFrames; i++)
            delete pixlist[i];
        delete [] pixlist;
    }
}

//------------------------------------------------
void TiffFlip::showMemInfo(bool status)
{
    if(status)
    {
        memLabel->show();
        swapLabel->show();
    }
    else
    {
        memLabel->hide();
        swapLabel->hide();
    }
}

//------------------------------------------------
void TiffFlip::Reset()
{

    //if they press load ram 2x in a row this will crash
    //as pixlist hasnt been fully initialized...

    int	i;
    if ( timer.isActive() )
        timer.stop();
    //hide();

    //QString rere;
    //debug("mframe:"+rere.setNum(numFrames));

    if (pixlist)
    {
        for (i = 0; i < numFrames; i++)
        {
            delete pixlist[i];
        }
        delete [] pixlist;
    }
    pixlist		= 0;
    numFrames	= 0;
    maxFrames	= 0;
    frame		= 0;
    width		= 0;
    height		= 0;
    origWidth	= 0;
    origHeight	= 0;
    showMemInfo(true);
}

//------------------------------------------------
void TiffFlip::SetDisplayZoom(int z)
{
    displayZoom = z;

mediaSize thesize;
thesize.width = width * displayZoom;
thesize.height = height * displayZoom;
emit resizePlayer(thesize);

    setFixedSize(width*displayZoom, height*displayZoom);
    m.reset();
    m.scale((float)displayZoom, (float)displayZoom);

}

//------------------------------------------------
//int TiffFlip::LoadImages(const char *startfile, int max, int z)
int TiffFlip::LoadImages(QString startfile, int max, int z)
{

    int		i;
    int		index;
    bool	ok;
    QString tmpString(startfile);
    //FILE	*check;

    loadZoom		= z;
    maxFrames	= max;

    dotTif	= ".bmp"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".BMP"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".png"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".PNG"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".PPM"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".ppm"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".gif"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".GIF"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".jpg"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".JPG"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".tga"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
        dotTif = ".TGA"; index	= tmpString.findRev((const char*)dotTif);
    } } } } } } } } } } }

    if (index < 4)
    {
        return -1;
    }

    tmpString.truncate(index);
    baseString = tmpString;
    index = tmpString.length();

    //ok all we need to do for now is figure out how many zeros on our own...
    //later on look at the texture loading code which is much better
    //as it deals with a wider variety of options by building the sequences

    QChar ch; slen = 0;
    int sl=tmpString.length();  sl-=1;

    while(1) 
    {
        ch = tmpString.at( sl );
        if (ch.isNumber()) { slen++; sl--; } else { break; }
        if (slen>6) { break; }
    }

    QString rere;
    index -=slen;
    baseString.truncate(index);
    tmpString.remove(0, index);

    //QString	shortName((const char*)baseString);
    QString	shortName; shortName = baseString;
    int	slash = shortName.findRev('/');
    if(slash > 0) {shortName.remove(0, slash+1);}

    startFrame = tmpString.toInt(&ok);
    if (!ok)
    {
        return -1;
    }

    numFrames	= 0;
    // Check that the files exist, and figure out max frames to load
    // There has to be a better way to do this besides opening and closing
    // all these files.
    for (i = startFrame; i < (int)startFrame+maxFrames; i++)
    {
        //we will fix this later mon this is real lazy but we need to rewrite the enire routine to use videoData
        if (slen==2) { tmpString.sprintf("%s%02d%s", (const char*)baseString, i, (const char*)dotTif); }
        if (slen==3) { tmpString.sprintf("%s%03d%s", (const char*)baseString, i, (const char*)dotTif); }
        if (slen==4) { tmpString.sprintf("%s%04d%s", (const char*)baseString, i, (const char*)dotTif); }
        if (slen==5) { tmpString.sprintf("%s%05d%s", (const char*)baseString, i, (const char*)dotTif); }
        if (slen==6) { tmpString.sprintf("%s%06d%s", (const char*)baseString, i, (const char*)dotTif); }


        if (!( QFile::exists(tmpString) ))
        {
            if (i == 0)
            {
                warning ("File \'%s\' not found. Sequence aborted.", (const char*)tmpString);
                return -1;
            }
            maxFrames = i-startFrame;
            break;
        }
        else
        { /*fclose(check);*/ }
    }

    if ( ! (pixlist = new QPixmap*[maxFrames+1]) )
    {	
        OutOfMemory();
        return -1;
    }

    swapThresh = SWAP_THRESH;
    doMemCheck = getMemUsage(&begMem, &begSwap);
    fprintf(stderr, "begin: mem %3.1f%%, swap: %3.1f%%\n", begMem, begSwap);
    showMemInfo(true);

    loadFrame = startFrame;
    timer.start(0);
    return startFrame;
}


void TiffFlip::GetImages( QImage SentImage, int max, int z)
{
    maxFrames	= 1;	numFrames	= 1; startFrame=0; loadFrame = 0;

    if ( ! (pixlist = new QPixmap*[1]) )
    {	 
        OutOfMemory(); 
        return; 
    }

    swapThresh = SWAP_THRESH;

    QImage  image;    image=SentImage;

    origWidth = width = SentImage.width();   // = w
    origHeight = height = SentImage.height();  // = h

    if ( ! (pixlist[0] = new QPixmap) )
    { 
        OutOfMemory(); 
        return; 
    }

    pixlist[0]->setOptimization(QPixmap::MemoryOptim);

    if ( ! pixlist[0]->convertFromImage(image, Qt::ColorOnly | Qt::NoAlpha ) )
    { 
        OutOfMemory(); 
        return; 
    }

    mediaSize thesize;
    thesize.width = width * displayZoom;
    thesize.height = height * displayZoom;

    //should be only if size has changed
    emit resizePlayer(thesize);

    setFixedSize(width*displayZoom, height*displayZoom);

    drawPixmap();		

    loadFrame=1;
    showMemInfo(false);
    return;
}
//--------------------------------------------------
void	TiffFlip::LoadController()
{
    if (loadFrame >= startFrame+maxFrames)
    {
        LoadFinished();
    }
    else
    {
        int frame = loadFrame-startFrame;
        emit LoadingFrame(frame);
        LoadSingleImage();
        SetFrame(frame);
        if(frame%10 == 0)
            checkMem();
        loadFrame++;
    }
}

//--------------------------------------------------
void	TiffFlip::LoadFinished()
{
    showMemInfo(false);
    timer.stop();
    emit Done(startFrame, false);
}

//--------------------------------------------------
void	TiffFlip::abortLoad()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    jtrace.warn("load aborted");
    showMemInfo(false);
    timer.stop();
    numFrames = maxFrames = (loadFrame-startFrame);
    emit Done(startFrame, true);
}

//--------------------------------------------------
void	TiffFlip::LoadSingleImage()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString	tmpString;
    QImage	image;

    if (slen==2) { tmpString.sprintf("%s%02d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
    if (slen==3) { tmpString.sprintf("%s%03d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
    if (slen==4) { tmpString.sprintf("%s%04d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
    if (slen==5) { tmpString.sprintf("%s%05d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
    if (slen==6) { tmpString.sprintf("%s%06d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }

    //jtrace.info( "TiffFlip::","loading:"+tmpString);

    QString format = dotTif.right( (dotTif.length() -1 ) );  //remove the period

    QImage  myimage;
    QPixmap tif;
    bool gothere = 0;

    ////////////////////////////////////////////
    //add tga support here

    QImageIO iio; iio.setFileName( (const char*)tmpString );

    if ( tmpString.endsWith( ".tga" ) || tmpString.endsWith( ".TGA" )) 
    { 
        iio.setFormat("TGA"); 
    }//
    else 
    { 
        /*iio.setFormat( format.upper() );*/ 
    }

    if ( iio.read() ) 
    {	
        image = iio.image(); gothere=1; 
    } 
    else 
    { 
        //jtrace.debug("null image "+tmpString); 
    }


// tif.load( (const char*)tmpString );
// image = tif;

if( !(image.isNull() ) ) 
{  
        gothere=1; 
} 
else 
{ 
        //jtrace.debug("null image "+tmpString); 
}

    if (gothere)
    { 
        origWidth = width = image.width();
        origHeight = height = image.height();

        if ( ! (pixlist[loadFrame-startFrame] = new QPixmap) )
        { 
            OutOfMemory(); 
            return; 
        }
        //debug("1");
        pixlist[loadFrame-startFrame]->setOptimization(QPixmap::MemoryOptim);
        //pixlist[loadFrame-startFrame]->setOptimization(QPixmap::NoOptim);
        //debug("2");
        if ( ! pixlist[loadFrame-startFrame]->convertFromImage(image,Qt::ColorOnly | Qt::NoAlpha) )
        {
            OutOfMemory();
            return;
        }

        image.reset();
        numFrames++;
        frame = loadFrame-startFrame;
        drawPixmap();
        //debug("3");
        //--------------------------
        // if first time in loop, set the size of window
        // and show it
        //--------------------------
        if (loadFrame == startFrame)
        {
            mediaSize thesize;
            thesize.width = width * displayZoom;
            thesize.height = height * displayZoom;
            emit resizePlayer(thesize);

            setFixedSize(width*displayZoom, height*displayZoom);
            //show();
        }
    }
    else
    {
        jtrace.debug("Error loading file");
        abortLoad();
    }
}

//------------------------------------------------
void	TiffFlip::OutOfMemory()
{
    timer.stop();
    sprintf (errorMessage, "ERROR: Could not allocate enough memory.\n(memory exhausted at frame %d)\n\nLoad Aborted", loadFrame-startFrame);
    Reset();	// free memory
    QMessageBox::critical(this, "Selah", errorMessage);
    emit Done(-1, true);
}
//------------------------------------------------
void	TiffFlip::SetFrame(int in)
{
    frame = in;
    drawPixmap();
}

//------------------------------------------------
void	TiffFlip::drawPixmap()
{
    static	QPixmap		tempPix;

    if (numFrames > 0 && frame >= 0 && frame < numFrames)
    {
        if (displayZoom > 1)
        {
            tempPix = pixlist[frame]->xForm(m);
            bitBlt(this, 0, 0, &tempPix, 0, 0, -1, -1);

            //here we update player
            emit updateplayer2(tempPix);

            //one less copy for speed...
            //bitBlt(this, 0, 0, &(pixlist[frame]->xForm(m)), 0, 0, -1, -1);
        }
        else
        {
            bitBlt(this, 0, 0, pixlist[frame], 0, 0, -1, -1);
            //here we update player
            if (videohead) {
                emit updateplayer2(*pixlist[frame]);
            }
        }

        frame ++;
        if (frame >= numFrames)
            frame = 0;
    }

    //amsterdam
    //here we copy the imagfe back up to the player...
    //for simulated video preview
    //QPixmap secondimage = tempPix;
    //emit updateplayer2(tempPix);


}

//------------------------------------------------
void	TiffFlip::paintEvent(QPaintEvent * )
{
    drawPixmap();
}

//------------------------------------------------
bool	TiffFlip::checkMem()
{
    bool retVal = getMemUsage(&curMem, &curSwap);
    if(retVal)
    {
        memString.sprintf("mem: %3.1f%%", curMem);
        swapString.sprintf("swap: %3.1f%%", curSwap);
        memLabel->setText(memString);
        swapLabel->setText(swapString);
    }

    if(curSwap > begSwap)
    {
        float diff = curSwap-begSwap;
        if(diff > swapThresh)
        {
            timer.stop();
            QString mssg;
            mssg.sprintf(	"WARNING:\nswap file useage has increased by %3.1f%%.\n", diff);
            int but = QMessageBox::warning(this, "SELAH", mssg, QMessageBox::Abort, QMessageBox::Ignore);
            if(but == QMessageBox::Ignore)
            {
                swapThresh += SWAP_THRESH;
                timer.start(0);
            }
            else
            {
                abortLoad();
            }
        }
    }
    return retVal;
}

/**
* get percentage of memory and swap used
*/
//------------------------------------------------
bool	TiffFlip::getMemUsage(float* mem, float* swap)
{
    int c = 0;
    FILE *meminfo;

    uint swap_total	= 16;
    uint swap_used	= 0;
    uint swap_free	= 16;

    uint mem_total	= 16;
    uint mem_used	= 0;
    uint mem_free	= 16;
    uint mem_share	= 0;
    uint mem_buff	= 0;
    uint mem_cache	= 0;

    meminfo = fopen("/proc/meminfo", "r");
    if(meminfo)
    {
        char buffer[512];
        while(fgets(buffer, 512, meminfo) != NULL)
        {
            if( strncmp(buffer, "Swap:", 5) == 0 )
            {
                sscanf(buffer, "Swap: %u %u %u", &swap_total, &swap_used, &swap_free);
                c++;
            }
            else if( strncmp(buffer, "Mem:", 4) == 0 )
            {
                sscanf(buffer, "Mem: %u %u %u %u %u %u",
                    &mem_total, &mem_used, &mem_free,
                    &mem_share, &mem_buff, &mem_cache);
                c++;
            }
        }
        fclose(meminfo);
    }
    float mb = 1024.0*1024.0;
    float mTotal = (float)mem_total/mb;
    float mUsed  = (float)(mem_used-(mem_share+mem_buff+mem_cache))/mb;
    *mem = 100.0f*(mUsed/mTotal);
    float sTotal = (float)swap_total/mb;
    float sUsed  = (float)swap_used/mb;
    *swap = 100.0f*(sUsed/sTotal);

    if(c == 2)
        return true;
    else
        return false;
}
