/*******************************************************************************
**
** The header file for the Jahshaka effects module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/


#include "jahplugins.h" //jahplugin is temporarily in here

JahPluginList::JahPluginList( )
{
    thePlugin     = 0;
    jahPluginLib  = 0;

    pluginLoaded  = FALSE;

    useLocalUI    = 0;
    numbersliders = 0;
    numberbuttons = 0;

    //used to debug plugin class
    plDebug = true;

};

JahPluginList::~JahPluginList( )
{
	//debug("called jahpluginlist destructor");
	jahPluginLib->unload();
    delete jahPluginLib;
}

bool  JahPluginList::loadPlugin(QString loadname)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "JahPluginList","loadPlugin");

    JahPrefs& jprefs    = JahPrefs::getInstance();


    jahPluginLib = new QLibrary ( loadname);

    QString str = jahPluginLib->library(); //will set str to "mylib.dll" on Windows, and "libmylib.so" on Linux.

   //now load the plugin
    jahPluginLib->load();


    if ( !jahPluginLib->isLoaded() )
    {
        if (plDebug)
        {
            jtrace.debug(">>> error loading plugin:"+str);
        }
        return false;
    }

    //now we resolve the location and get a instance
    myfunction getThePlugin =  (myfunction) jahPluginLib->resolve("getPluginClass");

    if ( ! getThePlugin )
    {
        if (plDebug)
        {
            jtrace.debug(">>> error resolving plugin:"+str);
        }

        return false;
    }
    else
    {
        if (plDebug) 
        {
            jtrace.debug(">>> loaded plugin:"+str);
        }

        //first we grab the plugin here
        thePlugin = getThePlugin();

        //then we check to see if plugin is for the current version of jahshaka

        QString JahVersion    = jprefs.getJahVersion();
        QString PluginVersion = thePlugin->JahPluginVersion;

        if (JahVersion.contains(PluginVersion))
        {
            //if it is we set up important globals are set here
            pluginName     = thePlugin->getName();
            pluginFamily   = thePlugin->getFamily();
            pluginGroup    = thePlugin->getGroup();
            thepluginClass = thePlugin->getClass();
			
			pluginLoaded = true;
		}
        else
        {
            //if not dont load it - unload and erase plugin
            unloadPlugin();
            return false;
        }
    }

    return true;
}

bool JahPluginList::unloadPlugin()
{
    //debug("unloading plugin");

    //this is a manual unload
    if (pluginLoaded)
    {
            jahPluginLib->unload();
            pluginLoaded = false;
    }

	return true;

}


//we can get rid of this now and use the pluginLoaded bool variable...
bool JahPluginList::initPluginStatus()
{
    //checking isLoaded will crash if its not loaded
    //so we rely on the pluginLoaded value thats initialized above...

    //currently we only support effects plugins
    bool initializedOk = false;

    //this will be diffrent for ui plugins, etc
    if (pluginLoaded)
    {
        initializedOk = initializeEffectsPlugin();
    }

    return initializedOk;
    //return pluginLoaded;
}

//if it is an effects plugin then we run this...
//doesnt really initialize the plugin itself just what we need at startup time
bool JahPluginList::initializeEffectsPlugin()
{
	bool initOK = true;

    JahPrefs& jprefs    = JahPrefs::getInstance();

    numbersliders = thePlugin->getSliders();
    numberbuttons = thePlugin->getButtons();

	///////////////////////////////////////////////////////
	// custom routines for gpu plugins

	if (thepluginClass  == JAH_GPU_PLUGIN)
	{
		thePlugin->tempBasePath = jprefs.getBasePath();

		bool nvSupport		  = jprefs.getNVGpusupport();
	    bool arbSupport		  = jprefs.getGpusupport();

		thePlugin->nvsupport  = nvSupport;
		thePlugin->arbsupport = arbSupport;

		//if we have arb support lets send some data over
		if (arbSupport)
		{
			thePlugin->max_vp_instructions =  jprefs.getMaxVertexInstructionsARB();
			thePlugin->max_fp_instructions =  jprefs.getMaxFragmentInstructionsARB();
			
			//if the plugin supports arb lets test to see if it supports the hardware
			if (thePlugin->HASARB)
			{
				thePlugin->isComplexArbSupported();
			}
		}

		/////////////////////////////////////////////////////
		//this routine cheks to see if the plugin will be supported
		//under either arb or nv
		initOK = ( (thePlugin->HASNV && nvSupport ) || (thePlugin->HASARB && arbSupport  )) ? true : false;
        //fprintf(stderr, "thePlugin->HASARB = %d   arbSupport = %d\n", thePlugin->HASARB, arbSupport);

	}

	return initOK;

}

bool JahPluginList::isPluginLoaded()
{
    return pluginLoaded;
}

///////////////////////////////////////////////////////
// generic routines
QString JahPluginList::getJahPluginVersion()    { return thePlugin->JahPluginVersion;   }

int JahPluginList::getPluginNumber()            { return pluginNumber;                  }
JAH_PLUGIN_CLASS JahPluginList::getPluginClass(){ return thepluginClass;                }

QString JahPluginList::getPluginFamily()        { return thePlugin->pluginFamily;       }
QString JahPluginList::getPluginGroup()         { return thePlugin->pluginGroup;        }
QString JahPluginList::getPluginName()          { return thePlugin->pluginName;         }

int JahPluginList::getNumberSliders(void)       { return numbersliders;                 }
bool JahPluginList::getUseSliders(void)         { return thePlugin->usesliders;         }
QString JahPluginList::getSliderLabel(int i)    { return thePlugin->sliderlabel[i];     }

int JahPluginList::getNumberButtons(void)       { return numberbuttons;                 }
QString JahPluginList::getButton(int i)         { return thePlugin->button[i];          }

bool JahPluginList::getUseLabels(void)          { return thePlugin->uselabels;          }
int  JahPluginList::getNumberLabels(void)       { return thePlugin->numlabels;          }
QString JahPluginList::getLabel(int i)          { return thePlugin->label[i];           }

void JahPluginList::setBasePath(QString path)   { thePlugin->setBasePath(path);         }


