/*******************************************************************************
**
** The header file for the Jahshaka jahpreferences module 
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef JAHPREFS_H
#define JAHPREFS_H

#include    "qstring.h"
#include    "qsettings.h"

class JahPrefs
{

    public:

        //for some reason we have to do this public on windows vc6++
        #ifdef WIN32
    ~JahPrefs() throw() ;
        #endif

    private:

        static JahPrefs _instance ;

        // Hide these 'cause this is a singleton.       
        JahPrefs() throw() ; 

        #ifndef WIN32
    ~JahPrefs() throw() ;    
        #endif

        JahPrefs( JahPrefs&);

        JahPrefs& operator = ( const JahPrefs&) ;

        /// the sole instance of this class
        QString JahVersion;
        QString BasePath;
        QString MediaPath;

        int     JahRenderQuality;
        int     JahRenderer;

        int     jahResolution;
        //int     XResolution;
        //int     YResolution;
        //QString Ratio;

        bool    menuBars;
        bool    dualHead;

        bool    videoHead;
        int     videoHeadX;
        int     videoHeadY;

        QString  language;

        bool    rememberGeometry;
        int     XPos;    
        int     YPos;     
        int     JahWidth;    
        int     JahHeight;    

        int     XPos2;    
        int     YPos2;     
        int     JahWidth2;    
        int     JahHeight2;    

        bool    HwFog;
        bool    HwAliasing;
        
        bool    vertexshaders;
        bool    gpusupport;
        bool    nv_gpusupport;

        bool    autoLaunchserver;

        bool    foundPrefs;

        bool    wideScreen;

        bool    jahstyle;
        int     themestyle;
        int     themecolor;

        int     max_vertex_program_instructions_ARB;
        int     max_fragment_program_instructions_ARB;

    public:
        //gets a instance of this object
        static JahPrefs& getInstance() throw();

        //the main prefs read and write routines
        void savePreferences(void) throw();
        void checkVersion(void) throw();
        void saveVideoheadPreferences(void) throw();
        void loadPreferences(void) throw();

        //routines to set varialbes
        void setJahVersion(QString jahver);

        void setJahResolution ( int xres );
        //void setXRes ( int xres );
        //void setYRes ( int xres );
        //void setRatio( QString ratio );

        void setBasePath ( QString path ); 
        void setMediaPath( QString path );

        void setMenubars( bool value ); 
        void setDualhead( bool value );

        void setThemestyle( int value );
        void setThemecolor( int value );

        void setVideohead( bool value ); 
        void setVideoheadX( int value ); 
        void setVideoheadY( int value ); 

        void setRememberGeometry( bool value );
        void setGeometryX( int value ); 
        void setGeometryY( int value ); 
        void setGeometryWidth( int value ); 
        void setGeometryHeight( int value );

        void setGeometryX2( int value ); 
        void setGeometryY2( int value ); 
        void setGeometryWidth2( int value ); 
        void setGeometryHeight2( int value );

        void setLanguage( QString value );

        void setHwFog( bool value );
        void setHwAliasing( bool value ); 
        void setVShading( bool value );
        void setGpusupport (bool value );
        void setNVGpusupport (bool value );

        void setMaxVertexInstructionsARB(int value);
        void setMaxFragmentInstructionsARB(int value);

        void setRenderer( int value );
        void setRenderQuality( int value );

        void setWidescreen (bool value );

        void setStyle      (bool value );
        void setStyleTheme (int value );
        void setStyleColor (int value );

        void setAutoLaunchServer (bool value );

        ///////////////////////////////////
        //routines to get variables

        bool getFoundPrefs(void);

        QString getJahVersion( void );

        int getJahResolution( void );
        //int getXRes( void );
        //int getYRes( void );
        //QString getRatio( void );

        QString getBasePath ( void ); 
        QString getMediaPath( void ); 

        bool getMenubars( void ); 
        bool getDualhead( void );

        int  getThemestyle( void );
        int  getThemecolor( void );

        bool getVideohead( void ); 
        int  getVideoheadX( void ); 
        int  getVideoheadY( void ); 

        bool getRememberGeometry( void ); 
        int  getGeometryX( void ); 
        int  getGeometryY( void ); 
        int  getGeometryWidth( void ); 
        int  getGeometryHeight( void );

        int  getGeometryX2( void ); 
        int  getGeometryY2( void ); 
        int  getGeometryWidth2( void ); 
        int  getGeometryHeight2( void );

        QString  getLanguage( void );

        bool getHwFog( void ); 
        bool getHwAliasing( void ); 
        bool getVShading( void );
        bool getGpusupport( void );
        bool getNVGpusupport( void );

        int getMaxVertexInstructionsARB( void );
        int getMaxFragmentInstructionsARB( void );

        int  getRenderer( void );
        int  getRenderQuality( void );

        bool getWidescreen( void );

        bool getStyle (void );
        int  getStyleTheme (void );
        int  getStyleColor (void ); 

        bool getAutoLaunchServer (void );

};

#endif

