/*******************************************************************************
**
** The source file for the Jahshaka mediaobject library
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "mediaobject.h"

////////////////////////////////////////
// constructors and destructors

mediaData::mediaData(void)
{
    mediaopen = false;

    jahmpegsupport = false;
    winavisupport  = false;
    linavisupport  = false;

    mpegobject     = false;
    winaviobject   = false;
    linaviobject   = false;


    #ifdef JAHMPEGDECODER
    jahmpegsupport = true;
    #endif

    #ifdef WINAVISUPPORT  
    winavisupport = true;
    #endif

    #ifdef LINUXAVISUPPORT  
    linavisupport = true;
    #endif


}

mediaData::~mediaData(void)
{
    if (jahmpegsupport && mpegobject)
    {
        if (mediaopen) 
        { 
            jahmpegobject->closeMedia(); 
            mediaopen = false;
        }

        delete jahmpegobject;
    }

    if (winavisupport && winaviobject)
    {
        if (mediaopen) 
        { 
            jahwinaviobject->closeMedia(); 
            mediaopen = false;
        }

        delete jahwinaviobject;
    }

    if (linavisupport && linaviobject)
    {
        if (mediaopen) 
        { 
            jahlinaviobject->closeMedia();
            mediaopen = false;
        }

        delete jahlinaviobject;
    }

}

////////////////////////////////////////
// main code only uses AVI for now

void  mediaData::printinfo(void)
{
    if (jahmpegsupport && mpegobject)
        jahmpegobject->printinfo();

    if (winavisupport && winaviobject)
        jahwinaviobject->printinfo();

    if (linavisupport && linaviobject)
        jahlinaviobject->printinfo();
};


bool mediaData::openMedia(QString mediafile) 
{
    //we need to know here what the filetype is
    //and the os/media type support
    //so we can use the correct libraries
    //debug("mediadata::opening media");

	//would save time to check if file exists...
    QFile f( mediafile );

    if (!f.exists())
    {
        return false;
	}

    bool mpeg = false;
    bool avi = false;

    //now check for known extensions
    //currently we support
    //avi and mpg

    mediafile.lower();

    avi  = mediafile.endsWith("avi");

    if (!avi)
        mpeg = mediafile.endsWith("mpg");

    if (!mpeg)
        mpeg = mediafile.endsWith("mpeg");



    //in order to open media we need to create the necessary object first
    bool status = false;

    #ifdef JAHMPEGDECODER
    if (mpeg)
    {
        if (jahmpegsupport)
        {
            jahmpegobject = new mediaObjectMpeg;
            status = jahmpegobject->openMedia(mediafile);
            mpegobject = true;
        }
    }
    #endif

    #ifdef WINAVISUPPORT
    if (avi)
    {
        if (winavisupport)
        {
            jahwinaviobject = new mediaObjectWinavi;
            status = jahwinaviobject->openMedia(mediafile);
            winaviobject = true;

        }
    }
    #endif

    #ifdef LINUXAVISUPPORT  
    if (avi)
    {
        if (linavisupport)
        {
            jahlinaviobject = new mediaObjectLinavi;
            status = jahlinaviobject->openMedia(mediafile);
            linaviobject = true;
        }
    }
    #endif

    if (status)
    {
        mediaopen = true;
    }

    //need to poll object that was created to make sure we were successful before
    //returning true here
    return status;
};

bool mediaData::closeMedia(void)
{
    bool status = false;

    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            status = jahmpegobject->closeMedia();

        if (winavisupport && winaviobject)
            status = jahwinaviobject->closeMedia();

        if (linavisupport && linaviobject)
            status = jahlinaviobject->closeMedia();
    }

    if (status)
    {
        mediaopen = false;
    }

    return status;
};

QImage mediaData::getVideoFrame(int frame)
{
    QImage thedata;

    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            return jahmpegobject->getVideoFrame(frame);

        if (winavisupport && winaviobject)
            return jahwinaviobject->getVideoFrame(frame);

        if (linavisupport && linaviobject)
            return jahlinaviobject->getVideoFrame(frame);
    }

    return thedata;

};

int mediaData::getVideoLength()
{
    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            return jahmpegobject->getVideoLength();

        if (winavisupport && winaviobject)
            return jahwinaviobject->getVideoLength();

        if (linavisupport && linaviobject)
            return jahlinaviobject->getVideoLength();
    }
    
    return 0;
};



void  mediaData::getAudioFrame(int frame)
{
};

bool  mediaData::isMediaOpen(void)
{
    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            return jahmpegobject->isMediaOpen();

        if (winavisupport && winaviobject)
            return jahwinaviobject->isMediaOpen();

        if (linavisupport && linaviobject)
            return jahlinaviobject->isMediaOpen();
    }

    return false;
};


QString mediaData::getFileName()
{
    QString loadfile;

    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            return jahmpegobject->getFileName();

        if (winavisupport && winaviobject)
            return jahwinaviobject->getFileName();

        if (linavisupport && linaviobject)
            return jahlinaviobject->getFileName();
    }

    return loadfile;
};

int  mediaData::getNumberFrames()
{
    if (mediaopen)
    {
        if (jahmpegsupport && mpegobject)
            return jahmpegobject->getNumberFrames();

        if (winavisupport && winaviobject)
            return jahwinaviobject->getNumberFrames();

        if (linavisupport && linaviobject)
            return jahlinaviobject->getNumberFrames();
    }

    return 0;
};


