/*******************************************************************************
**
** The header file for the Jahshaka OpenMediaLib library
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef MEDIADATA_H
#define MEDIADATA_H

#include "qstring.h"
#include "qimage.h"

//we can get rid of these later on
//in order to make this a standalone Qt library
#include "jahpreferences.h"
#include "jahtracer.h"


#ifdef JAHMPEGDECODER  
#include "mediaobject_mpeg.h"
#endif

#ifdef WINAVISUPPORT
#include "mediaobject_win.h"
#endif

#ifdef LINUXAVISUPPORT
#include "mediaobject_lin.h"
#endif

//for unsupported platforms
#include "mediaobject_tmp.h"

//in case we want to add audio support in here...
//maybe better to create a audio object?
//#ifdef AUDIOSUPPORT
//#include "mediaobjectaudio_lin.h"
//#endif

////////////////////////////////////////////////////////
//The core controlling mediaobject class

class mediaData {
    
public:

	mediaData(void);
	~mediaData(void);

	bool    openMedia(QString mediafile);
	bool    closeMedia(void);

	QImage  getVideoFrame(int frame);
	int     getVideoLength();

	void    getAudioFrame(int frame); //not yet implemented

	bool    isMediaOpen();

	QString getFileName();
	int     getNumberFrames();

	void    printinfo(void);


private:

	//support for the different libraries
	bool    jahmpegsupport;
	bool    winavisupport;
	bool    linavisupport;

	bool    mpegobject;
	bool    winaviobject;
	bool    linaviobject;

    bool    mediaopen;

	QString JahBasePath;

	//objects for the different encoders
	//we can make these plugins later on

	#ifdef JAHMPEGDECODER  
	mediaObjectMpeg		* jahmpegobject;
	#else
	mediaObjectTemp		* jahmpegobject;
	#endif

	#ifdef WINAVISUPPORT
	mediaObjectWinavi	* jahwinaviobject;
	#else
	mediaObjectTemp		* jahwinaviobject;
	#endif

	#ifdef LINUXAVISUPPORT
	mediaObjectLinavi	* jahlinaviobject;
	#else
	mediaObjectTemp		* jahlinaviobject;
	#endif



};

#endif



