/*******************************************************************************
**
** The header file for the Jahshaka network module header
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef NETWORKING_H
#define NETWORKING_H

#include <interfaceheaders.h>
#include <httpd.h>
#include <qsocketdevice.h>
#include <assetobject/videodata.h>
#include <qfile.h>
#include <qwidget.h>
#include <qtabwidget.h>
#include <qlistview.h>
#include <qcombobox.h>

#include <qhostaddress.h>

#include "giftconnection.h"

#include "apollonqueryview.h"
#include "apollontransfertab.h"

#include "jahfileframe.h"


extern QString JahMediaPath;

class JahNet :  public  QWidget {
  Q_OBJECT

public:


JahNet( QWidget* parent, QHBox* parent2, const char* name, QHBox* controls, QHBox* controller,
                int* globalclipnumber )
{

    JahPrefs& jprefs  = JahPrefs::getInstance();
    jprefs.setAutoLaunchServer(autoLaunchServer);
	
	serverLaunched = FALSE;

    buildInterface(controls);           //  builds the network interfaces
    buildController(controller);        //  draw slider interface

    buildNetframe(parent);              //  draw p2p interface

    //initialize variables
    clipnumber = globalclipnumber;

    initializeObjects();

};


  ~JahNet() { }


    QWidget* parentobject;
    int*  clipnumber;

    bool NetStatus;

    void buildInterface( QHBox* qtinterface);
    void buildController( QHBox* controller);
    void buildNetframe( QWidget* controller);
    void initializeObjects();
    void initializeUiOptions();

    void ServerInfo();
    void launchServer();

public slots:

    void setConnectServer();

    void newConnect();
    void echoText(QString);
    void requestStats();
    void giftStatusReply(QString status);

    void giftsocketError(int error);

    void clientinterface();
    void serverinterface();
    void p2pinterface();

    void languageChange();

    void createNewClient();
    void createNewClient(QHostAddress);

    void  addClip(videoData,int);
    void  GrabDesktopClip();

    void recirprocateConnection(QHostAddress addy);

    void updatetheDesktop( videoData );

    //used for the p2p engine
    void startDaemon(  );
    void initializeGift();

    // tries to connect to a remote gift daemon
    void retryRemoteConnection();

    // we've just been connected with the giFT daemon
    void connected();

    void searchGiftNetwork(  );
    void getGiftNetworkStats(  );
    void startQuery(const int &index, const QString &text);

    void setupTransferTab();

    void sendClipToDesktop();

	void slotGetHash(QString hash);
	void slotDeleteHash(QString hash);

private slots:

    void closeConnection();
    void sendToServer();
    void sendMediaToServer(videoData,int);
    void socketReadyRead();
    void socketConnected();
    void socketConnectionClosed();
    void socketClosed();
    void socketError( int e );

    //ui switching
    void togglep2plocalview(QWidget*);

signals:

    void GrabClip();
    void updateDesktop( videoData );

    void updateDesktopString(QString);


	void giftItemReply(QString);

    void sigSendHash(QString);
	void sigDeleteHash(QString hash); // sends the hash which was removed from the list

    //used for main ui switching
    void switchToNet();
    void switchToNetP2P();

protected:

private:

	bool autoLaunchServer;
	bool serverLaunched;

    giFTConnection* jahGift;

    ApollonQueryView* kqv;

    ApollonTransferTab   *m_transTab;


    QValueList<QString>* m_hashList;  // a pointer to this list exists also in apollonqueryview!!

    int          m_numStatUpdates;   // tells how many times the stats have been updated
	int          m_tries;            // number of tries to start the giFT daemon
	QTextEdit   *m_infoBox;          // the one and only info box which shows the stats

    QTimer *m_timer;

    FileMainFrame* theSharedFolder;


    SimpleServer *server ;
    SimpleServer *imageserver ;

    QSocket *socket; QSocket *datasocket;
    QSocket *recipsocket;

    QTextView *infoText;
    QTextView *infoText2;

    QHBox *hb;
    QLineEdit *inputText;
    QPushButton *send;
	QPushButton *close;
    QCheckBox* autoConnect;

    // QHBox* JahP2PSearch;
    QFrame* JahP2PSearch;

    QFrame* mainsearchframe;
    //QFrame* mainshareframe;
    QHBox* mainshareframe;

    QGridLayout* mainnetLayout;
    QGridLayout* jahshakanetLayout;


    QVBox *netbox;
    QHBox*        ClientBox;
    QVBox*        ServerBox;

    QFrame* clientBox2;
    QFrame* clientBox1;

    QFrame* clientframe;
    QLabel* ClientLabel;
    QLabel* serverlabel;
    QLabel* texteditLabel;

    QLineEdit* serveredit;


    QLineEdit* texteditbox;
    QPushButton* sendbutton, * sendimagebutton, * connectbutton,* closebutton;
    QTextView* clienttextfeedback;


    ////////////////////////////////////////////////////////////////////////////
    //interface variables
    QFrame*      LayersFrame;

    QPushButton*  AddClient;
    QPushButton*  AddServer;

    QPushButton*  AddP2P;
    QFrame*       P2PBox;




	QWidgetStack*  NetInterface;



    QPushButton*   ResetButton;             QPushButton*   SaveButton;    QPushButton*   LoadButton;
	  QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;

    QFrame*        ControlFrame_2;
    QWidgetStack*  fxoptionals;
    QHBox*         fxoptionals_clipone;     QHBox*         fxoptionals_cliptwo;
    QHBox*         fxoptionals_object;      QHBox*         fxoptionals_world;
    QHBox*         fxoptionals_text;

    QFrame*        ControlFrame1;
    QLabel*        RotateText_1;
    QLabel*        TranslateText_1;
    QLabel*        ScaleText_1;


    //for jahgift interface

    QTabWidget* jahp2p;
    QWidget* tab;
    QLineEdit* usersonlinebox_2;
    QLabel* giftserveridtext;
    QPushButton* connectgiftbutton;
    QPushButton* disconnectgiftbutton;
    QLabel* giftportidtext;
    QLineEdit* usersonlinebox_2_2;
    QLineEdit* shareduserbox;
    QLabel* textLabel3_4;
    QLineEdit* localsharesbox;
    QLabel* textLabel3_3;
    QLineEdit* totalsharedbox;
    QLabel* textLabel3_2;
    QLabel* textLabel3;
    QLineEdit* usersonlinebox;
    QLabel* giftportidtext_2;
    QLabel* giftstatuslable;
    QLineEdit* gidtstatusbox;
    QWidget* tab_2;
    QListView* searchresultes;
    QFrame* searchframe;
    QLabel* searchforlabel;
    QLineEdit* searchforlineedit;
    QPushButton* searchbutton;
    QLabel* mediatypelabel;
    QComboBox* mediatypebox;
    QComboBox* mediatypebox_2;
    QLabel* mediacategorylabel;
    QWidget* tab_3;
    QLabel* textLabel5_2;
    QLabel* textLabel5;
    QListView* outgoingtraffic;
    //QListView* incomingtraffic;
    QHBox* incomingtraffic;
    QWidget* tab_4;
    QTextEdit* textEdit2;
    QWidget* tab_5;
    QLabel* publicsharelabel;
    QLineEdit* sharelocation;
    QPushButton* setshare;
    QPushButton* syncshares;

    QWidgetStack* theSharedNetLayout;

    //for gift app
     QDialog * MaDialog;
     QTabWidget * viewtab;
};

#endif























