/*******************************************************************************
**
** The source file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

/* contains code from flip - pageflipper for TIFF animations
 * Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
 *
 * "flip" is free software; you can redistribute it and/or use
 * it and/or modify it under the terms of the "Artistic License"
 */

//we need to update tiffflip routines to use videoData objects
//and use the assetexchange the same way that the diskplayer does

#ifndef AUDIOSUPPORT
#include "MainPlayer.h"
#else
#include "MainPlayerAudio.h"
#endif

double MainPlayer::fpsVal[NumFPS];

static const unsigned long slider_res = 100;	// slider resolution

//==============================================================================
MainPlayer::MainPlayer( QWidget *parent, const char* name, QHBox* control, QHBox* controler,
                        QHBox* leftcontroller, QHBox* rightcontroller) : QWidget (parent, name)
#ifdef AUDIOSUPPORT
, _is_audio (false),
    _frame_to_set (0),
    _is_slider_pressed (false)
#endif
{
    //initalize the global vars from the prefs
    JahPrefs& jprefs = JahPrefs::getInstance();
    JahBasePath  = jprefs.getBasePath();

    //set up main interface
    buildInterfaceMenus(leftcontroller,rightcontroller);
    buildController(controler);

    //restore the preferences note that its not a dialog
    loadPreferences();

    //create the timer
    timer = new QTimer(this);

    //////////////////////////////////////////
    //this allows us to open the player output in a separate window
    //we need to check video head support here
    if (videohead)
        launchVideoHead();

    //on with the rest of it now...
    mainworldLayout = new QHBoxLayout( parent, 11, 6, "mainworldLayout");
    player = new QFrame( parent, "player" );
    mainworldLayout->addWidget( player );

    //this is where we initialize both the disk player
    jahplayer = new JahPlayer(player, 0);
    jahplayer->hide();

    //this creates the player autoloading mechanisims
    tiffFlip = new TiffFlip(player, 0);
    tiffFlip->hide();

    //this has to be done after creating tiffflip and diskplayer objects
    if (videohead)
        activateVideoHead();

    //ok we need to set this one up now and we are cool mon!
    buildInterface( control );

    //////////////////////////////////////////////////////////////////////////
    // connect the slots

    //connect player drag n drop to player load signal
    connect(jahplayer, SIGNAL(draggedAImage()), this, SLOT(draggedAImage()) );

    connect(jahplayer, SIGNAL(resizePlayer(mediaSize)), this, SLOT(ResizeDisplay(mediaSize)) );
    connect(jahplayer, SIGNAL(LoadingFrame(int)), this, SLOT(UpdateLoadStatus(int)) );

    connect(tiffFlip, SIGNAL(resizePlayer(mediaSize)), this, SLOT(ResizeDisplay(mediaSize)) );
    connect(tiffFlip, SIGNAL(LoadingFrame(int)), this, SLOT(UpdateLoadStatus(int)) );
    connect(tiffFlip, SIGNAL(Done(int, bool)), this, SLOT(LoadSequenceCompleted(int, bool)) );

    //set up timer
    connect(timer, SIGNAL(timeout()), this, SLOT(Action()) );

    //used for the standalone JahPlayer
    #ifdef JAHPLAYER
    connect( toolthumbview,   SIGNAL(clicked()), this, SLOT(toggleDesktopImages() )  );
    connect( toolbinview,     SIGNAL(clicked()), this, SLOT(toggleDesktopTable() )  );
    #endif
}

//----------------------------------------------------------

void	MainPlayer::loadPreferences()
{
    //initalize the global vars from the prefs
    JahPrefs& jprefs = JahPrefs::getInstance();

    /////////////////////////////////////////////////////////
    //from the prefs
    JahBasePath  = jprefs.getBasePath();
    videohead    = jprefs.getVideohead();

    /////////////////////////////////////////////////////////
    //for the videhead resolution
	//start up using default resolution
	int vhres = jprefs.getJahResolution();

	projectData thedata;
	vhXres = thedata.getXres(vhres);
    vhYres = thedata.getYres(vhres);

    /////////////////////////////////////////////////////////
    // defaults
    fpsVal[F60] = 60.0;
    fpsVal[F48] = 48.0;
    fpsVal[F30] = 30.0;
    fpsVal[F24] = 24.0;
    fpsVal[F20] = 20.0;
    fpsVal[F15] = 15.0;
    fpsVal[F12] = 12.0;
    fpsVal[F10] = 10.0;

    minFrameInput = maxFrameInput = 0;
    curFrame	=  0;
    pingpong	=  frameStep = 1;

    //prevents a crash on startup
    _is_slider_pressed = true;

    //set the default speed for the player
    currentSpeed = fpsVal[30];

    videoheadInitialized = false;
}

//--------------------------------------------------------------------
MainPlayer::~MainPlayer()
{
    //if opening the player in its own window we need to close it out on exit
    if (videohead) 
    {
        Stop();

        //save out videohead settings
        JahPrefs& jprefs = JahPrefs::getInstance();

        int x = playerdialog->x();
        int y = playerdialog->y();

        jprefs.setVideoheadX(x);
        jprefs.setVideoheadY(y);

        jprefs.saveVideoheadPreferences();

        playerdialog->close();

    }
}

////////////////////////////////////////////////////////////////////
// routines for videohead support
//need to replace 720x486 with the default Resolution...
void	MainPlayer::launchVideoHead()
{
    if (videohead && !videoheadInitialized)
    {
        videoheadInitialized = true;

        JahPrefs& jprefs = JahPrefs::getInstance();

        int videoheadx = jprefs.getVideoheadX();
        int videoheady = jprefs.getVideoheadY();

        ///////////////////////////////////////////

        playerdialog = new QWidget( 0, "Jahshaka VideoHead" );
        playerdialog->setPaletteBackgroundColor ( QColor( 0, 0, 0 ) );
        playerdialog->setFocusPolicy  (QWidget::NoFocus  );
        playerdialog->setFixedSize( QSize( vhXres, vhYres ) );

        mainplayerLayout = new QHBoxLayout( playerdialog, 0, 0, "mainworldLayout");

        //this is the one that we bitBlt to with the picture
        playerClip = new QWidget(playerdialog, 0);

        mainplayerLayout->addWidget( playerClip );

        playerdialog->move(videoheadx,videoheady); //got this from preferences

    }
}

//--------------------------------------------------------------------
void	MainPlayer::UpdatePlayer2(QPixmap thepix)
{	
    static int dWidth  = 0;
    static int dHeight = 0;

    if ( dWidth != thepix.width() ||  (dHeight != thepix.height()) ) 
    {
        dWidth  = thepix.width();  
        if (dWidth >vhXres) dWidth = vhXres;

        dHeight = thepix.height(); 
        if (dHeight >vhYres) dHeight = vhYres;

        playerClip->setMinimumSize( dWidth, dHeight  );
        playerClip->setMaximumSize( dWidth, dHeight  );
    }

    bitBlt(playerClip, 0, 0, &thepix, 0, 0, -1, -1);

}

void	MainPlayer::activateVideoHead()
{
    if (videohead)
    {
        jahplayer->videohead = videohead;
        tiffFlip->videohead = videohead;

        connect(jahplayer, SIGNAL(updateplayer2(QPixmap)), this, SLOT(UpdatePlayer2(QPixmap)) );
        connect(tiffFlip, SIGNAL(updateplayer2(QPixmap)), this, SLOT(UpdatePlayer2(QPixmap)) );

        playerdialog->show();
    }
}

void	MainPlayer::deactivateVideoHead()
{
        videohead = false;

        jahplayer->videohead = false;
        tiffFlip->videohead = false;

        disconnect(jahplayer, SIGNAL(updateplayer2(QPixmap)), this, SLOT(UpdatePlayer2(QPixmap)) );
        disconnect(tiffFlip, SIGNAL(updateplayer2(QPixmap)), this, SLOT(UpdatePlayer2(QPixmap)) );

        playerdialog->hide();
}

//////////////////////////////////////////////////////////////////
// for drag n drop

void	MainPlayer::draggedAImage()
{
    emit playerDraggedAImage();
}
//used in the player to flip desktop display modes
void MainPlayer::toggleDesktopImages() 
{

    debug("toggling desktop");
    emit toggleTheDesktopImages();
}

void MainPlayer::toggleDesktopTable() 
{

    debug("toggling desktop");
    emit toggleTheDesktopTable();
}

//--------------------------------------------------------------------
void	MainPlayer::SetFrameNumber(int i)
{    
    //set up tracer
    //JahTrace& jtrace = JahTrace::getInstance();

    // This slot tells player to draw a certain frame
    QString rere;
    //jtrace.debug("MainPlayer::", "SetFrameNumber() setting framenumber to:",i);
    curFrame = i;

    if (!_is_audio)
    {

    frameNumString.setNum(curFrame-minFrame+1);
    //jtrace.debug("MainPlayer::","framenumber is now:"+frameNumString);

    animframe->setValue( curFrame-minFrame+1 );

        if (playfromdisk)
        {
            if (assetData.theType == SEQUENCE)
            { 
                //debug("its a sequence... hack the framenumber");
                int skip = assetData.videoext.toInt();
                int theframe = (curFrame-startFrame)+skip;
                //debug(">>Mainwindow Setframenumber startFrame:"+rere.setNum(theframe));

                jahplayer->LoadTheFrame(theframe);
            }
            else
            {
            jahplayer->LoadTheFrame(curFrame-startFrame); //+imagestartframe if its a sequence
            }
        }
        else
        {
            tiffFlip->SetFrame(curFrame-startFrame);
        }
    }
    else
    //its audio
    {
        #ifdef AUDIOSUPPORT
        _frame_to_set = i;
        #endif

        jahplayer->LoadTheFrame(1);
    }

}

void MainPlayer::sliderValueChanged(int val)
{

    if (!_is_slider_pressed && !timer->isActive())
    {
        //debug("slider value changed");

        int theframe;
        theframe = controllerslider->value();
        SetFrameNumber(theframe);
    }
}


void MainPlayer::LoadMySequence(videoData desktopclip)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.info( "Initializing the disk player");

    //prevents a crash on startup
    _is_slider_pressed = true;

    //the local instance of the asset
    assetData = desktopclip;

    //default settings
    playfromdisk = true;
    _is_audio = false;

    //cant play audio if theres no audio support
    #ifndef AUDIOSUPPORT
    if (assetData.theCategory == AUDIO) return;
    #endif

    //hide everythings and show the disk player
    jahplayer->hide();
    tiffFlip->hide();
    jahplayer->show();

    //just to make sure
    Clear();
    //reset things properly
    tiffFlip->Reset();
    jahplayer->Reset();

    //zero the sliders and settings
    curFrame	=  0;
    pingpong	=  frameStep = 1;

    controllerslider->setRange	(0, 0);
    minFrameInput->setActive(false);
    maxFrameInput->setActive(false);
    statusline->setText("using play from disk");
    animframe->setValue( 1 );
    actualFps->setText(" ");
    controllerslider->setRange	(assetData.startframe, assetData.endframe);
    minFrameInput->display(" ");
    maxFrameInput->display(" ");


    //initialize MaxFrames for RamPlayer
    int MaxFrames = assetData.getMaxFrames();
    startFrame = assetData.startframe;

    #ifdef AUDIOSUPPORT
    if (assetData.theCategory == AUDIO)
    {
        jtrace.debug("loading a audio clip");
        playfromdisk = true;
        AudioData audio_data;

        audio_data.clipname = assetData.clipname; //filename alone
        audio_data.filename = assetData.location+assetData.filename+assetData.extension;  //name+path in full
        audio_data.sndfile_handle = 0;
        audio_data.file_lenght = 0;

        //we will do this in the player mon
        audio_data.sndfile_handle = sf_open (audio_data.filename.ascii (), SFM_READ, &audio_data.file_info);
        if (audio_data.sndfile_handle == 0) {
            jtrace.error (" !!! Soundfile open error:");
            jtrace.error (audio_data.filename);
            return;
        }

        //we dont need to do this until its time to play the clip
        sf_seek(audio_data.sndfile_handle, 0, SEEK_SET) ;

        LoadAudio(audio_data);

    }
    #endif

    //this sets up the interface values
    LoadSequenceCompleted(startFrame, false);

    //initializes the disk player
    startFrame = jahplayer->LoadImages(assetData);

    //SetFrameNumber(curFrame);
    //a messy hack for still image loading loads using assetexchange default
    if (assetData.theCategory==IMAGE || assetData.theCategory==AUDIO || assetData.theCategory==CLIP )
    {
        startFrame = 1;
    }

    SetFrameNumber(startFrame);

    //default to 30fps but we should check speed for streams...
    switchFps(30);

    //updating asset display data
    UpdateAssetData(desktopclip);

}


////////////////////////////////////////////////////
// used to blast a QImage to the 2nd video head
// from other modules

void	MainPlayer::BlastImage(const QImage& newimage)
{


    Stop();
    if(tiffFlip->isLoading()) { tiffFlip->abortLoad(); }

    controllerslider->setRange	(0, 0);
    minFrameInput->setActive(false);
    maxFrameInput->setActive(false);
    tiffFlip->Reset();
    statusline->setText("Still loaded.");
    animframe->setValue( 1 );
    actualFps->setText(" ");

    controllerslider->setRange	(1, 1);
    minFrameInput->display("1");
    maxFrameInput->display("1");

    ClearOut();
    tiffFlip->GetImages(newimage, 1, 1 );

    startFrame = 1;

}

//--------------------------------------------------------------------
void	MainPlayer::Clear()
{
    Stop();
    if(tiffFlip->isLoading()) {	tiffFlip->abortLoad(); }

    if (!playfromdisk)
    {
        tiffFlip->Reset();
        statusline->setText("no frames loaded.");
        actualFps->setText(" ");
        QObject::disconnect(scrubstop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );

    }
    else
    {
        jahplayer->Reset();
        statusline->setText("no frames loaded.");
        actualFps->setText(" ");
    }

    // audio hack
    #ifdef AUDIOSUPPORT
    if (_oss_player.Is_Started ()) {
        _oss_player.Stop ();
    }
    if (_oss_player.Is_Open ())  {
        _oss_player.Close ();
    }
    #endif

    _is_slider_pressed = true;
    _is_audio = false;

}

//--------------------------------------------------------------------
//used for external access to plyaer
void	MainPlayer::ClearOut()
{
    Clear();
}


//--------------------------------------------------------------------
void	MainPlayer::UpdateLoadStatus(int i)
{
    static char	message[128];
    if (!playfromdisk) 
    {
        sprintf(message, "loading frame %d of %d", i+1, tiffFlip->GetMaxFrames() );
    }

    statusline->setText(message);
}



//--------------------------------------------------------------------
void	MainPlayer::LoadSequenceCompleted(int sf, bool aborted)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    QString rere;

    if (playfromdisk)
    {
        jtrace.debug("LoadSequenceCompleted() in Diskplayer");
        QString rere;

        startFrame = assetData.startframe;
        minFrame = startFrame;
        maxFrame = assetData.endframe;

        controllerslider->setRange	(minFrame, maxFrame);
        jahplayer->setCurMinFrame(minFrame);
        jahplayer->setCurMaxFrame(maxFrame);

        minFrameInput->setValue (minFrame);
        minFrameInput->setMinMaxInt	(minFrame, maxFrame-1);
        maxFrameInput->setMinMaxInt	(minFrame+1, maxFrame);
        maxFrameInput->setValue (maxFrame);

        curFrame = 0;
        controllerslider->setValue(curFrame);

        statusline->setText("Player Ready");

        //default to 30fps but we should check speed for streams...
        currentSpeed = fpsVal[30];

        //QObject::disconnect(scrubstop, SIGNAL(clicked()), jahplayer, SLOT(abortLoad()) );
    }
    else
    {
        jtrace.debug("LoadSequenceCompleted() in Ramplayer");
        startFrame = sf;
        minFrame = startFrame;
        maxFrame = tiffFlip->GetNumFrames()-1+startFrame;

        //frameScroll->setRange	(minFrame, maxFrame);
        controllerslider->setRange	(minFrame, maxFrame);
        tiffFlip->setCurMinFrame(minFrame);
        tiffFlip->setCurMaxFrame(maxFrame);

        minFrameInput->setValue (minFrame);
        minFrameInput->setMinMaxInt	(minFrame, maxFrame-1);
        maxFrameInput->setMinMaxInt	(minFrame+1, maxFrame);
        maxFrameInput->setValue (maxFrame);

        curFrame = 0;
        //frameScroll->setValue(curFrame);
        controllerslider->setValue(curFrame);
        SetFrameNumber(minFrame);

        char message[128];
        sprintf(message, "loaded %d frames at %dx%d.", tiffFlip->GetNumFrames(), tiffFlip->GetWidth(), tiffFlip->GetHeight() );
        statusline->setText(message);

        QObject::disconnect(scrubstop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
        if(!aborted)
            LoopForward();
    }

}

//actually we should use this routine for single frames as well
//for less confusion...
//and we can streamline the loader... in tiffflipp
//since all the necessary data is already in the desktopclip
//--------------------------------------------------------------------
void	MainPlayer::UpdateAssetData(videoData &desktopclip)
{
    QString data1,data2;
    playerOutput->setText("--------------------------------");
    if (desktopclip.theCategory == IMAGE) { data1 = "Type Image :"; }
    if (desktopclip.theCategory == CLIP ) { data1 = "Type Clip  :"; }
    playerOutput->insertLine(" Clipname      :" + desktopclip.clipname, -1 );
    playerOutput->insertLine( "File Location :" + desktopclip.location, -1 );
    playerOutput->insertLine( "File Name     :" + desktopclip.filename +desktopclip.extension, -1 );
    playerOutput->insertLine( "Start :" + data1.setNum(desktopclip.startframe) + " End :" + data2.setNum(desktopclip.endframe), -1 );
    playerOutput->insertLine("-Resolution---------------------", -1);

    assetExchange exchange;
    QImage theimage; 
    theimage = exchange.getImage(desktopclip);

    playerOutput->insertLine( "X Width     :" + data1.setNum( theimage.width()), -1 );
    playerOutput->insertLine( "Y Width     :" + data1.setNum( theimage.height()), -1 );

    playerOutput->insertLine("--------------------------------", -1);

}

//--------------------------------------------------------------------
void	MainPlayer::SetMinFrame(int val)
{
    if (!playfromdisk) 
    {

    maxFrameInput->setMinInt( (minFrame = val) + 1 );
    controllerslider->setRange(minFrame, maxFrame);
    tiffFlip->setCurMinFrame(minFrame);
    tiffFlip->setCurMaxFrame(maxFrame);
    //controllerslider->setValue(minFrame);
    }
    else
    {
    maxFrameInput->setMinInt( (minFrame = val) + 1 );
    controllerslider->setRange(minFrame, maxFrame);
    jahplayer->setCurMinFrame(minFrame);
    jahplayer->setCurMaxFrame(maxFrame);
    //controllerslider->setValue(minFrame);    
    }
}
void	MainPlayer::SetMaxFrame(int val)
{
    if (!playfromdisk) 
    {
    minFrameInput->setMaxInt( (maxFrame = val) - 1 );
    controllerslider->setRange(minFrame, maxFrame);
    tiffFlip->setCurMinFrame(minFrame);
    //controllerslider->setValue(maxFrame);
    }
    else
    {
    minFrameInput->setMaxInt( (maxFrame = val) - 1 );
    controllerslider->setRange(minFrame, maxFrame);
    jahplayer->setCurMinFrame(minFrame);
    //controllerslider->setValue(maxFrame);
    }
}

//--------------------------------------------------------------------
void	MainPlayer::LoopForward()
{
    if (timer->isActive())  timer->stop();

    //debug(">>>>>>in loop forward");

    if (_is_audio!=true)
    {
    //debug(">>not audio");

    bool loopstatus = forcedplay->isChecked(); // status...

    playMode = loopstatus;
    pingpong = loopstatus;

    //timer speeds here
    timer->start((int)(1000.0/currentSpeed));
    }
        #ifdef AUDIOSUPPORT
    if (_is_audio) {
    //debug(">>is audio");
        if (_oss_player.Is_Started ()) return;

        _oss_player.Start (); return;

        if (!_oss_player.Is_Started ()) {
            debug (" Error starting player thread.");
            return;
        }
    }
    #endif



}
//--------------------------------------------------------------------
void	MainPlayer::PingPong()
{
    bool isact = false;

    if (timer->isActive())
        { timer->stop(); isact=true; }

    //#ifdef AUDIOSUPPORT
    //if (_is_audio) {
    //	return;
    //}
    //#endif



    bool loopstatus = forcedplay->isChecked(); // status...

    playMode = loopstatus;
    pingpong = loopstatus;

    if (isact) { timer->start((int)(1000.0/currentSpeed)); }
}


//--------------------------------------------------------------------
void	MainPlayer::Stop()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Stopping player");

    if (timer->isActive())
        timer->stop();

    #ifdef AUDIOSUPPORT
    if (_is_audio) {
        if (_oss_player.Is_Started ()) {
            _oss_player.Stop ();
            curFrame = minFrame;
            _audio_lock.Lock ();
            sf_seek (_audio_data.sndfile_handle, 0, SEEK_SET);
            _audio_lock.Unlock ();
        }
    }
    #endif

    actualFps->setText(" ");
    pingpong = 0;
}




//--------------------------------------------------------------------
void	MainPlayer::FastForward()
{

        #ifdef AUDIOSUPPORT
    if (_is_audio) {
        _audio_lock.Lock ();
        curFrame = 0;
        sf_seek (_audio_data.sndfile_handle, 0, SEEK_END);
        _audio_lock.Unlock ();

        controllerslider->setValue (0);
        SetFrameNumber(0);
        return;
    }
        #endif
    // </audio hack>

    controllerslider->setValue( curFrame = maxFrame );
    SetFrameNumber(curFrame);
}

void	MainPlayer::FrameForward()
{

    #ifdef AUDIOSUPPORT

    if (_is_audio) {
    }

    #endif

    int value = controllerslider->value();
    value+=1;

    if (value>= maxFrame) value = maxFrame;

    controllerslider->setValue( value );
    SetFrameNumber(value);
}

void	MainPlayer::FrameBackward()
{

    #ifdef AUDIOSUPPORT

    if (_is_audio) {
    }

    #endif

    int value = controllerslider->value();
    value-=1;

    if (value<0) value = 0;

    controllerslider->setValue( value );
    SetFrameNumber(value);
}

//--------------------------------------------------------------------
void	MainPlayer::Rewind()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    #ifdef AUDIOSUPPORT
    // <audio hack>
    if (_is_audio) {
        jtrace.debug("audio rewind");
        _audio_lock.Lock ();
        curFrame = maxFrame;
        sf_seek (_audio_data.sndfile_handle, 0, SEEK_SET);
        _audio_lock.Unlock ();

        controllerslider->setValue (0);
        return;
    }
        #endif
    // </audio hack>

    controllerslider->setValue( curFrame = minFrame );
    SetFrameNumber(curFrame);
}

//--------------------------------------------------------------------
void	MainPlayer::Action()
{

    static int		counter = 1;
    static int		elapsed;
    static int		accumulated;
    static double	framesPerSec;
    static char		labelText[64];


    switch(playMode)
    {
        case LOOP_FORWARD:
            ActionLoopForward();
            break;
        case PING_PONG:
            ActionPingPong();
            break;
        default:
            warning ("MainPlayer::playMode is set incorrectly.\nplayMode = %d", playMode);
            timer->stop();
    }

    #ifndef _WIN32
    gettimeofday(&curTime, 0);
    #endif

    elapsed = (int)curTime.tv_usec - (int)oldTime.tv_usec;
    oldTime.tv_usec = curTime.tv_usec;
    if(elapsed < 0)
        elapsed += 1000000;

    accumulated += elapsed;
    if (counter >= 10)
    {
        framesPerSec = 1000*1.0/((double)accumulated*0.0001);
        counter = 0;
        accumulated = 0;
        sprintf(labelText, "%3.2f", framesPerSec);
        actualFps->setText(labelText);
    }
    counter++;
}
//- - - - - - - - - - - - - - - - - - - - - - -
void	MainPlayer::ActionLoopForward()
{

#ifdef AUDIOSUPPORT
    if (_is_audio && _is_slider_pressed) 
    {
        return;
    }
#endif
    // when audio this is made in the player callback
    if (!_is_audio)
    {

        curFrame += frameStep;

        if (curFrame > maxFrame)
            curFrame = minFrame + 1;

        controllerslider->setValue(curFrame);
        SetFrameNumber(curFrame);
    }
    else
    {
        //jtrace.debug("action loop audio");
        controllerslider->setValue(curFrame*slider_res/maxFrame);
        SetFrameNumber(curFrame*slider_res/maxFrame);
    }
        qApp->processEvents();
}

void	MainPlayer::ActionPingPong()
{
    curFrame += pingpong;
    if (curFrame > maxFrame)
    {
        curFrame = maxFrame;
        pingpong = -frameStep;
    }
    else if (curFrame < minFrame)
    {
        curFrame = minFrame;
        pingpong = frameStep;
    }

    controllerslider->setValue(curFrame);
    SetFrameNumber(curFrame);
    qApp->processEvents();
}


//--------------------------------------------------------------------
void	MainPlayer::switchFps(int fpsval)
{

    switch(fpsval) {
        case 60  : { SetFPS(F60); break; }
        case 48  : { SetFPS(F48); break; }
        case 30  : { SetFPS(F30); break; }
        case 24  : { SetFPS(F24); break; }
        case 20  : { SetFPS(F20); break; }
        case 15  : { SetFPS(F15); break; }
        case 12  : { SetFPS(F12); break; }
        case 10  : { SetFPS(F10); break; }
        default  : { SetFPS(F30); break; }
    }


}

//--------------------------------------------------------------------
void	MainPlayer::SetFPS(FPS index)
{
    for(int i = 0; i < NumFPS; i++)

    currentSpeed = fpsVal[index];

    if (timer->isActive())
    {
        timer->changeInterval((int)(1000.0/currentSpeed));
    }
}

//--------------------------------------------------------------------

void	MainPlayer::SetStep(int index)
{

    switch(index)
    {
        case 1:	frameStep = 1;
            break;
        case 2:	frameStep = 2;
            break;
        case 3:	frameStep = 3;
            break;
        case 4:	frameStep = 4;
            break;
        case 5:	frameStep = 5;
            break;
        case 6:	frameStep = 6;
            break;
    }
}

//--------------------------------------------------------------------

void	MainPlayer::SetZoom(int index)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Calling setzoom");

    //rplace with a switch
    float thezoom;
    if (index == 1) thezoom = 0.5;
    if (index == 2) thezoom = 1.0;
    if (index == 3) thezoom = 2.0;
    if (index <1 || index >3) thezoom = 1.0;

    if (!playfromdisk)
        tiffFlip->SetDisplayZoom(thezoom);
    else
        jahplayer->SetDisplayZoom(thezoom);

    int theframe;
    theframe = controllerslider->value();
    SetFrameNumber(theframe);

}

void	MainPlayer::ResizeDisplay(mediaSize thesize)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    //jtrace.debug( "Calling Resize display");

    player->setMinimumSize( QSize( thesize.width, thesize.height ) );
    player->setMaximumSize( QSize( thesize.width, thesize.height ) );
}
//----------------------------------------------------------


//we are using this to launch the ram player
//using existing asset
void	MainPlayer::launchRamPlayer()
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();

    //ram player doesnt support images just clips
    if (assetData.theCategory==IMAGE) return;

    //this is where we toggle the player type
    playfromdisk = !playfromdisk;

    //////////////////////////////////////////////////////////
    //hide everything and show what we need
    jahplayer->hide();
    tiffFlip->hide();

    if (playfromdisk)
        jahplayer->show();
    else
        tiffFlip->show();

    if(tiffFlip->isLoading())
    {
        tiffFlip->abortLoad();
    }

    //////////////////////////////////////////////////////////
    //just to make sure we stop whatever is going on
    Stop();

    //////////////////////////////////////////////////////////
    //if its back to diskmode then we go back to disk mode
    //not sure if all of this is necessary
    if (playfromdisk)
    {
        jahplayer->Reset();
        animframe->setValue( 1 );
        actualFps->setText(" ");
        controllerslider->setRange	(assetData.startframe, assetData.endframe);

        minFrameInput->display(" ");
        maxFrameInput->display(" ");

        LoadSequenceCompleted(assetData.startframe, false);

        //if its play from disk we send the videodata object to the disk player
        startFrame = jahplayer->LoadImages(assetData);

        return;
    }

    ////////////////////////////////////////////////////////
    //we move ahead with the ram player here

    jtrace.debug("Initializing Ram player");

    //zero these sliders and settings
    controllerslider->setRange	(0, 0);
    minFrameInput->setActive(false);
    maxFrameInput->setActive(false);

    tiffFlip->Reset();
    //jahplayer->Reset();

    switchFps(30);

    statusline->setText("Loading Asset");

    animframe->setValue( 1 );
    actualFps->setText(" ");

    if (!playfromdisk) 
    {
        QObject::connect(scrubstop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
    }

    controllerslider->setRange	(assetData.startframe, assetData.endframe);
    minFrameInput->display(" ");
    maxFrameInput->display(" ");

    int MaxFrames = assetData.getMaxFrames();

    #ifdef WIN32
    //theres a issue on windows with loading more than 98 frames in the ram player
    //related to this command pixlist[0]->setOptimization(QPixmap::NoOptim);
    if (!playfromdisk)
    {
        if (MaxFrames>98) { MaxFrames=98; }
    }
    #endif


    if (!playfromdisk) 
    {
        startFrame = tiffFlip->LoadImages( assetData.getFullFileName(), MaxFrames, 1);
    }

}


// ------------
// audio hack
#ifdef AUDIOSUPPORT
void MainPlayer::LoadAudio (AudioData audio_data)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "Loading Audio into player");


    //if (audio_data.file_info.format & SF_FORMAT_TYPEMASK != SF_FORMAT_WAV) {
        // doesn't play anything except wave for now
    //	debug ("file has unsupported format.");
    //	return;
    //}

    int bps;
    // we should determine file format for proper playing
    switch (audio_data.file_info.format & SF_FORMAT_SUBMASK) {
    case SF_FORMAT_PCM_S8:
        bps = 1;
        break;
    case SF_FORMAT_PCM_16:
        bps = 2;
        break;
    case SF_FORMAT_PCM_24:
        bps = 3;
        break;
    case SF_FORMAT_PCM_32:
        bps = 4;
        break;
    default:
        jtrace.error("Audiofile has unsupported sample format.");
        return;
    }

    const int num_samples = 512;

    _audio_data = audio_data;
    _is_audio = true;

    // calculate samples
    curFrame  = 0;
    frameStep = num_samples/bps/_audio_data.file_info.channels;
    maxFrame  = _audio_data.file_info.frames - 1;
    minFrame  = 0;

    //frameScroll->setRange (0, slider_res);	// QSlider doesn't handle a big ranges
    controllerslider->setRange (0, slider_res);	// QSlider doesn't handle a big ranges
    minFrameInput->display((int) minFrame);
    maxFrameInput->display((int) maxFrame);

    // set up player

    _oss_player.NumBlocks (0); // let OSS automatically set number of blocks
    _oss_player.Set_Channels (_audio_data.file_info.channels);
    _oss_player.NumBlocks (bps * num_samples);
    _oss_player.Set_Master (true);
    _oss_player.Set_Callback (_PlayerCallback, this);
    _oss_player.SetBPS (bps*8);

    if (_oss_player.Open () != 0)  {
        jtrace.debug ("===============            ");
        jtrace.debug (" OSS player opening error. ");
        jtrace.debug ("===============            ");
        Clear ();
        return;
    }        

    sf_seek (_audio_data.sndfile_handle, 0, SEEK_SET);
}

//this is really part of the timer loop
unsigned long MainPlayer::PlayerCallback (float* buf, unsigned long size)
{

    _audio_lock.Lock ();

    sf_count_t rd = sf_read_float(_audio_data.sndfile_handle, buf, size);

    if (rd < size) {		
        // loop the track
        sf_seek (_audio_data.sndfile_handle, 0, SEEK_SET);
        sf_count_t rd = sf_read_float(_audio_data.sndfile_handle, buf, size);
        curFrame = minFrame;		
    }		

    curFrame += frameStep;	
    if (curFrame > maxFrame)
        curFrame = minFrame;
    _audio_lock.Unlock ();

    return rd;
}

/*
void MainPlayer::RecordFrame(int frame)
{
    _frame_to_set = frame;
}
*/
#endif

void MainPlayer::JumpToFrame ()
{
    if (_is_audio)
    {
        #ifdef AUDIOSUPPORT
        _audio_lock.Lock ();
        curFrame = _frame_to_set*maxFrame/slider_res;
        sf_seek (_audio_data.sndfile_handle, curFrame, SEEK_SET);
        _audio_lock.Unlock ();
        #endif
    }
    else
    {
        int theframe;
        theframe = controllerslider->value();
        SetFrameNumber(theframe);
    }
    _is_slider_pressed = false;
}

void MainPlayer::ScrollPressed ()
{
    _is_slider_pressed = true;
}
