/*******************************************************************************
**
** The header file for the Jahshaka player module
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

/* contains code from flip - pageflipper for TIFF animations
 * Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
 *
 * "flip" is free software; you can redistribute it and/or use
 * it and/or modify it under the terms of the "Artistic License"
 */

#ifndef MainWindowAudio_h
#define MainWindowAudio_h


#ifndef _WIN32
#include <unistd.h>
#include <sys/time.h>
#else
#include <sys/timeb.h>
#endif

#include <stdio.h>

#include <interfaceheaders.h>

#include <qsettings.h>
#include <qtimer.h>
#include <qmultilineedit.h>
#include <qkeycode.h>
#include <qtooltip.h>
#include <qiconset.h>
#include <qmessagebox.h>
#include <qtabwidget.h>


//used for videodata support
#include "videodata.h"
#include "projectdata.h"

//custom ui objects
#include "widget.h"
#include "InputLCD.h"

//included from libraries/jahplayer
#include "TiffFlip.h"
#include "JahPlayer.h"

//for the tracer
#include "jahtracer.h"

//for preferences global
#include "jahpreferences.h"

/////////////////////////
// for audio support
#include "Audio_OSS.h"
#include "Lock.h"

#define SIZE_50         0
#define SIZE_100        1
#define SIZE_200        2
#define LOOP_FORWARD    0
#define PING_PONG       1

//====================================================================
class MainPlayer : public QWidget
{
        Q_OBJECT

public:
    MainPlayer(QWidget *parent, const char* name,
    QHBox* control, QHBox* controler, QHBox* leftcontroller, QHBox* rightcontroller );
    ~MainPlayer();

    //the asset to be loaded in the player
    videoData assetData;

    enum FPS { F60, F48, F30, F24, F20, F15, F12, F10,NumFPS };

    static double fpsVal[NumFPS];

    void	loadPreferences();

    void	ClearOut();
    void	BlastImage(const QImage&);
    void	LoadMySequence(videoData desktopclip); //was just videoData

    //added for default to play from disk
    void	LoadDiskSequence();
    void	PlayDiskSequence(videoData &desktopclip);

    bool playfromdisk;

    QHBoxLayout * mainplayerLayout;
    QFrame * playerframe;
    QFrame * mainplayerframe;
    QWidget * playerClip;

    //preferences variables
    bool videohead;
    //int videoheadx, videoheady;
	int  vhXres, vhYres;

    QGridLayout * jahshakaLayout;
    QFrame * mainworld;
    QHBoxLayout * mainworldLayout;
    QFrame * player;
    QHBox * playerh;
    QVBox * playerv;

    //scene control variables
    QWidgetStack* controlpanel;
    QFrame* scene_controls;
    QFrame* object_controls;

    //builds the controler
    void addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                            int w, int h, QString iconOff, QString iconOn );

    void    addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range);
    void    buildController( QHBox* controller);
    void    buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller);
    void    buildInterface( QHBox* qtinterface );

    void	UpdateAssetData(videoData &desktopclip);

    // audio
    void LoadAudio (AudioData audio_data);

    //for videohead support
    void	launchVideoHead(void);
    void	activateVideoHead(void);
    void	deactivateVideoHead(void);

protected slots:

    void	UpdatePlayer2(QPixmap thepix);

    //used to flip desktop display modes in player
    void    toggleDesktopImages();
    void    toggleDesktopTable();

    void	UpdateLoadStatus(int);
    void	LoadSequenceCompleted(int sf, bool aborted);

    void	SetFrameNumber(int);
    void    switchModeFps(void);
    void	SetFPS(FPS);
    void	switchFps(int fpsval);
    void    switchModeStep(void);
    void	SetStep(int);
    void	SetZoom(int);
    void	ResizeDisplay(mediaSize);

    void	SetMinFrame(int);
    void	SetMaxFrame(int);

    void	Clear();

    //encoding
    void	launchRamPlayer();

    void    draggedAImage();

    void    sliderValueChanged(int val);

    void JumpToFrame ();	// jump to frame when the user releases the slider
    //void RecordFrame(int frame);	// record current scroll pos
    void ScrollPressed ();	// sets the flag that user pressed the slider

public slots:

    void	LoopForward();
    void	PingPong();
    void	Stop();
    void	FastForward();
    void	FrameForward();
    void	FrameBackward();
    void	Rewind();
    void	Action();
    void	ActionLoopForward();
    void	ActionPingPong();

signals:
    void toggleTheDesktopImages();
    void toggleTheDesktopTable();

    void playerDraggedAImage();


private:
    QString JahBasePath;

    bool videoheadInitialized;

    TiffFlip		*tiffFlip;
    JahPlayer		*jahplayer;

    int			fpsID[7];

    //QPopupMenu	*stepPopup;
    int			step1_ID;
    int			step2_ID;
    int			step3_ID;
    int			step4_ID;
    int			step5_ID;
    int			step6_ID;
    //QPopupMenu	*zoomPopup;
    int			zoom1_ID;
    int			zoom2_ID;
    int			playMode;

    QTimer		*timer;

    unsigned long	curFrame; // in audio: current file pointer (in samples)
    unsigned long	minFrame; // in audio: 0
    unsigned long	maxFrame; // in audio: max samples
    int             startFrame;
    int             pingpong;
    double          currentSpeed;
    unsigned long	frameStep;// in audio: buffer size in samples (1 channel)


    struct	timeval	oldTime;
    struct	timeval	curTime;

    //////////////////////////////////////////////////////////////////////////
    // audio hack starts here

    IO::AUDIO_OSS _oss_player; // oss audio device representation
    bool _is_audio;		// object to play is audio

    AudioData _audio_data;	// info about audio file
    MutexLock _audio_lock;	// audio player mutex lock

    static unsigned long  _PlayerCallback (float* buf, unsigned long size, void* arg) {
        return reinterpret_cast<MainPlayer*>(arg)->PlayerCallback (buf, size);
    }

    // this method is called by player in order to get a new block
    // of file to play.
    unsigned long PlayerCallback (float* buf, unsigned long size);

    int _frame_to_set;	// when the user drags the slider - it's value stored there
    bool _is_slider_pressed; // indicates that the user has pressed the slider

    ///////////////////////////////////////////////////////////////////////////
    // audio hack ends here



    //////////////////////////////////////////////////////////////////////
    //player interface objects

    QFrame* maincontroller;
    QWidgetStack* MainController;
    QWidget* page;
    QFrame* forceplayframe;
    QCheckBox* forceplaycheckbox;
    QFrame* controllerframe;
    QFrame* TimecodeFrame;

    QPushButton* controllerpreviousbutton;
    QPushButton* controllerstopbutton;
    QPushButton* controllerplaybutton;
    QPushButton* controllerffworardbutton;
    QPushButton* controllernextbutton;
    QPushButton* controllerrewindbutton;

    QSlider* controllerslider;
    QFrame* timecodeframe;
    QPushButton* keyframebutton;
    QLCDNumber* timecodedisplay;
    QFrame* imagingframe;
    QPushButton* imagingphotobutton;
    QPushButton* imagingrenderbutton;

    //protected:
    QHBoxLayout* Form1Layout;
    QHBoxLayout* maincontrollerLayout;
    QHBoxLayout* pageLayout;

    //////////////////////////////////////////////////////////////////////
    //controller variables
    int    Astartframe;  int  Aendframe;  int  Aanimframe; int  MaxFrames;

    //for the left and right menus
    QFrame         *glworldLeft;
    JahToolButton   *tooltranslate, *toolscale, *toolrotate;
    JahToolButton  *toolthumbview, *toolbinview;

    QFrame         *glworldRight;
    JahToolButton  *toolzoomin, *toolzoomout;
    QSlider        *zoom;

    //for the player
    QString		frameNumString;
    QLabel		*statusline;
    QLabel		*frameNum;
    QLabel		*actualFps;

    QFrame* controls;
    QHBox* controlers;

    //this is to put the player in a 2nd head
    QWidget * playerdialog;

    QPushButton* rewButton;
    QPushButton* pingButton;
    QPushButton* stopButton;

    QMenuBar	*menuBar;
    QPopupMenu	*filePopup;
    QPopupMenu	*utilPopup;

    QVBoxLayout	*topLayout;
    QHBoxLayout	*middle;
    QVBoxLayout	*midR;
    QVBoxLayout	*midM;
    QVBoxLayout	*midL;
    QHBoxLayout	*buttons;
    QHBoxLayout	*status;


    QScrollBar	*frameScroll;

    InputLCD	*minFrameInput;
    InputLCD	*maxFrameInput;


    QCheckBox    * forcedplay;

    JahToolButton  * scrubfirst, * scrubprevious, * scrubstop, * scrubplay, * scrubnext, * scrublast;
    QPushButton  * scrubkey, * scrubrender, * scrubrenderAll;

    //for the rest of the interface
    QHBox*       MiddleFrame;		QSlider *     Slider2;
    InputLCD*    animframe;		QLabel *      aframetext;  //was QLCDNumber
    QLabel*      sframetext;		InputLCD *    lcdstartframe;
    QLabel*      eframetext;		InputLCD *    lcdendframe;

    QLabel* maxtext, *mintext, *statustext, *fpstext, *mpegtext, *avitext, *playeroutputtext;

    //for the new interface
    QFrame*      LayersFrame;

    QWidgetStack*  EffectsMain;
    QFrame*        SceneFrame;
    QFrame*        EffectsFrame;
    bool           EffectsStatus;

    QTabWidget*        tabframe;
    QWidget*        axiscontrols;
    QWidget*        objectcontrols;
    QWidget*        texturecontrols;
    QWidget*        scenecontrols;
    QWidget*        effectscontrols;

    QFrame*        ControlFrame0;
    QFrame*        ControlFrame1;
    QFrame*        ControlFrame2;
    QFrame*        sceneframe;

    QMultiLineEdit* playerOutput;

    //for the mpeg encoder
    QPushButton * mpegPlay, *aviEncode;

};

#endif // MainWindow_h



