/*******************************************************************************
**
** The source file for the Jahshaka core file file
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "MainPlayer.h"
#include <qstylefactory.h>
#include "InputLCD.h"

void MainPlayer::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) {


    QStyle *platinum=QStyleFactory::create("platinum");

    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");

    #ifdef JAHPLAYER
    //we are only using these 2 for now but we may use the others later on
    toolthumbview = new JahToolButton( glworldLeft, "thumbview" );
    addJahPlayerButton( toolthumbview, 0, 205, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/thumbview.png", JahBasePath+"Pixmaps/desktop/thumbview.png" );

    toolbinview =   new JahToolButton( glworldLeft, "binview" );
    addJahPlayerButton( toolbinview,   0, 235, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/binview.png", JahBasePath+"Pixmaps/desktop/binview.png" );
    #endif

    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );

    toolzoomin =   new JahToolButton( glworldRight, "zoomin" );
    addJahPlayerButton( toolzoomin,   0, 355, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomin.png", JahBasePath+"Pixmaps/desktop/zoomin.png" );

    zoom = new QSlider( glworldRight, "scrubslider" );
    if (platinum) zoom->setStyle( platinum );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( 1 );    zoom->setMaxValue( 3 );    zoom->setValue   ( 2 );

    toolzoomout =   new JahToolButton( glworldRight, "zoomout" );
    addJahPlayerButton( toolzoomout,   0, 115, 30, 27 ,    24, 28, JahBasePath+"Pixmaps/desktop/zoomout.png", JahBasePath+"Pixmaps/desktop/zoomout.png" );

    connect( zoom,         SIGNAL(sliderMoved(int)), this, SLOT(SetZoom(int) )  );

}


void MainPlayer::buildController( QHBox* controller) {

//////////////////////////////////////////////////////////////
// This is the control bar
// please clean me up and make me look cool
// also get rid of constraints on startframe and endframe
// and use the resizeable lcds in timecode format
// for these variables display

Astartframe=1;  Aendframe=100;  Aanimframe=1; MaxFrames = 100000;

    Form1Layout = new QHBoxLayout( controller, 11, 6, "Form1Layout");

    maincontroller = new QFrame( controller, "maincontroller" );
    maincontroller->setMinimumSize( QSize( 0, 35 ) );
    maincontroller->setMaximumSize( QSize( 32767, 35 ) );
    maincontroller->setPaletteBackgroundColor( QColor( 100, 100, 100 ) );
    maincontroller->setFrameShape( QFrame::Box );
    maincontroller->setFrameShadow( QFrame::Plain );
    maincontrollerLayout = new QHBoxLayout( maincontroller, 0, 0, "maincontrollerLayout");

    MainController = new QWidgetStack( maincontroller, "MainController" );

    page = new QWidget( MainController, "page" );
    pageLayout = new QHBoxLayout( page, 0, 0, "pageLayout");
    QSpacerItem* spacer = new QSpacerItem( 30, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer );

    forceplayframe = new QFrame( page, "forceplayframe" );
    forceplayframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, forceplayframe->sizePolicy().hasHeightForWidth() ) );
    forceplayframe->setMinimumSize( QSize( 90, 20 ) );
    forceplayframe->setMaximumSize( QSize( 90, 20 ) );
    forceplayframe->setFrameShape( QFrame::StyledPanel );
    forceplayframe->setFrameShadow( QFrame::Raised );

    forcedplay = new QCheckBox( forceplayframe, "forceplaycheckbox" );
    forcedplay->setGeometry( QRect( 0, 0, 91, 21 ) );
    forcedplay->setText("Loop" );
    pageLayout->addWidget( forceplayframe );
    connect( forcedplay,  SIGNAL(clicked()), this, SLOT(PingPong())  );

    QSpacerItem* spacer_2 = new QSpacerItem( 16, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_2 );
    QSpacerItem* spacer_3 = new QSpacerItem( 16, 16, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_3 );

    controllerframe = new QFrame( page, "controllerframe" );
    controllerframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, controllerframe->sizePolicy().hasHeightForWidth() ) );
    controllerframe->setMinimumSize( QSize( 170, 30 ) );
    controllerframe->setMaximumSize( QSize( 170, 30 ) );
    controllerframe->setFrameShape( QFrame::NoFrame );
    controllerframe->setFrameShadow( QFrame::Plain );

    //////////////////////////////////////////////////////////////////
    // set up the main playbar buttons

    scrubfirst = new JahToolButton( controllerframe, "controllerrewindbutton" );
    addJahPlayerButton( scrubfirst, 0, 1, 27, 29,   28, 30, JahBasePath+"Pixmaps/player/rewind_off.png", JahBasePath+"Pixmaps/player/rewind_on.png" );
    connect( scrubfirst,  SIGNAL(clicked()), this, SLOT(Rewind())  );

    scrubprevious = new JahToolButton( controllerframe, "controllerpreviousbutton" );
    addJahPlayerButton( scrubprevious, 27, 1, 27, 29,  24, 28, JahBasePath+"Pixmaps/player/frewind_off.png", JahBasePath+"Pixmaps/player/frewind_on.png" );
    connect( scrubprevious,  SIGNAL(clicked()), SLOT(FrameBackward())  );

    scrubstop = new JahToolButton( controllerframe, "controllerstopbutton" );
    addJahPlayerButton( scrubstop, 54, 1, 27, 29,    24, 28, JahBasePath+"Pixmaps/player/stop_off.png", JahBasePath+"Pixmaps/player/stop_on.png" );
    connect( scrubstop,  SIGNAL(clicked()), this, SLOT(Stop())  );

    scrubplay = new JahToolButton( controllerframe, "play" );
    addJahPlayerButton( scrubplay, 81, 1, 27, 29 ,    24, 28, JahBasePath+"Pixmaps/player/play_off.png", JahBasePath+"Pixmaps/player/play_on.png" );
    connect( scrubplay,  SIGNAL(clicked()), this, SLOT(LoopForward())  );

    scrubnext = new JahToolButton( controllerframe, "controllernextbutton" );
    addJahPlayerButton( scrubnext, 108, 1, 27, 29,    24, 28, JahBasePath+"Pixmaps/player/ffoward_off.png", JahBasePath+"Pixmaps/player/ffoward_on.png" );
    connect( scrubnext,  SIGNAL(clicked()), SLOT(FrameForward())  );

    scrublast = new JahToolButton( controllerframe, "controllerffworardbutton" );
    addJahPlayerButton( scrublast,  135, 1, 27, 29,   24, 28, JahBasePath+"Pixmaps/player/foward_off.png",  JahBasePath+"Pixmaps/player/foward_on.png" );
    connect( scrublast,  SIGNAL(clicked()), this, SLOT(FastForward())  );

    pageLayout->addWidget( controllerframe );

    controllerslider = new QSlider( page, "controllerslider" );
    //make sure platinum is loaded
    QStyle *platinum=QStyleFactory::create("platinum");
    //if(platinum) qApp->setStyle(platinum);
    if(platinum) controllerslider->setStyle( platinum );
    controllerslider->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)0, 0, 0, controllerslider->sizePolicy().hasHeightForWidth() ) );

    #ifdef JAHPLAYER
    controllerslider->setMinimumSize( QSize( 20, 0 ) );
    #else
    controllerslider->setMinimumSize( QSize( 490, 0 ) );
    #endif

    controllerslider->setMaximumSize( QSize( 500, 32767 ) );
    controllerslider->setOrientation( QSlider::Horizontal );
    controllerslider->setTickmarks( QSlider::Right );


    QObject::connect(controllerslider, SIGNAL(sliderMoved(int)), this, SLOT(SetFrameNumber(int)) );
    QObject::connect(controllerslider, SIGNAL(sliderPressed()), this, SLOT(ScrollPressed()) );
    QObject::connect(controllerslider, SIGNAL(sliderReleased()), this, SLOT(JumpToFrame()) );

    QObject::connect(controllerslider, SIGNAL(valueChanged(int)), this, SLOT(sliderValueChanged(int)) );

#ifdef AUDIOSUPPORT
    //QObject::connect(controllerslider, SIGNAL(sliderMoved(int)), this, SLOT(RecordFrame(int)) );
    //moved the RecordFrame call into SetFrameNumber
#endif

    pageLayout->addWidget( controllerslider );

    timecodeframe = new QFrame( page, "timecodeframe" );
    timecodeframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0,
                                timecodeframe->sizePolicy().hasHeightForWidth() ) );
    timecodeframe->setMinimumSize( QSize( 125, 30 ) );
    timecodeframe->setMaximumSize( QSize( 125, 30 ) );
    timecodeframe->setFrameShape( QFrame::NoFrame );
    timecodeframe->setFrameShadow( QFrame::Raised );

    animframe = new InputLCD( timecodeframe, "timecodedisplay",9 );
    addSliderLcd(animframe, 5, 0, 90, 31, 100);
    animframe->setValue( 1 );
    connect( controllerslider,   SIGNAL(valueChanged(int)),   animframe, SLOT(setValue(int)) );
    //connect( animframe, SIGNAL(valueChanged(int)),   SLOT(updateanimframe(int)) );

    pageLayout->addWidget( timecodeframe );

    QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_4 );

    imagingframe = new QFrame( page, "imagingframe" );
    imagingframe->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0,
                                imagingframe->sizePolicy().hasHeightForWidth() ) );
    imagingframe->setMinimumSize( QSize( 105, 20 ) );
    imagingframe->setMaximumSize( QSize( 105, 20 ) );
    imagingframe->setFrameShape( QFrame::NoFrame );
    imagingframe->setFrameShadow( QFrame::Raised );

    scrubrender = new QPushButton( imagingframe, "imagingphotobutton" );
    scrubrender->setGeometry( QRect( 0, 0, 50, 20 ) );
    scrubrender->setMinimumSize( QSize( 50, 20 ) );
    scrubrender->setMaximumSize( QSize( 50, 20 ) );
    scrubrender->setText( tr( "FPS"  ) );
    //connect( scrubrender, SIGNAL(clicked()), SLOT( Render()) );   //change this mon
    connect( scrubrender,  SIGNAL(clicked()), SLOT( switchModeFps())  );

    scrubrenderAll = new QPushButton( imagingframe, "imagingrenderbutton" );
    scrubrenderAll->setGeometry( QRect( 55, 0, 50, 20 ) );
    scrubrenderAll->setMinimumSize( QSize( 50, 20 ) );
    scrubrenderAll->setMaximumSize( QSize( 50, 20 ) );
    scrubrenderAll->setText( tr( "Step"  ) );
    //connect( scrubrenderAll, SIGNAL(clicked()), SLOT( RenderScene()) );   //change this mon
    connect( scrubrenderAll,  SIGNAL(clicked()), SLOT( switchModeStep())  );

    pageLayout->addWidget( imagingframe );

    QSpacerItem* spacer_5 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    pageLayout->addItem( spacer_5 );

    MainController->addWidget( page, 0 );
    maincontrollerLayout->addWidget( MainController );
    Form1Layout->addWidget( maincontroller );
    MainController->raiseWidget(0);
}

void MainPlayer::buildInterface( QHBox* qtinterface ) {

QStyle *platinum=QStyleFactory::create("platinum");

///////////////////////////////////////////////////////////
//The layers interface
LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
#ifndef JAHPLAYER
LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 140
LayersFrame->setMaximumSize( QSize( 60, 195 ) );
#endif
//////////////////////////////////////////////////////////////
// set up the scene options, or prefernces
EffectsMain = new QWidgetStack(qtinterface);
SceneFrame   = new QFrame(EffectsMain);
EffectsFrame = new QFrame(EffectsMain);

EffectsMain->addWidget(SceneFrame,0);
EffectsMain->addWidget(EffectsFrame,1);

#ifndef JAHPLAYER
EffectsMain->setGeometry( QRect( 0, 0, 855, 195) );
#endif

EffectsStatus = FALSE;
EffectsMain->raiseWidget(1);

    //////////////////////////////////////////////////////////
    // set up options here
    controlpanel = new QWidgetStack(EffectsFrame);

    //this may need to be QHBox as it is below...
    scene_controls  = new QFrame(controlpanel);     controlpanel->addWidget(scene_controls,0);
    object_controls = new QFrame(controlpanel);     controlpanel->addWidget(object_controls,1);

    #ifndef JAHPLAYER
    controlpanel->setGeometry( QRect( 1, 1, 700, 195) );
    #endif

/////////////////////////////////////////////////////////
//Set up scene interface
sceneframe = new QFrame( scene_controls, "scenecontrols" );

#ifndef JAHPLAYER
sceneframe->setGeometry( QRect( 1, 1, 700, 195 ) );
#endif

if (platinum) sceneframe->setStyle( platinum );


/////////////////////////////////////////////////////////
//Set up tabbed interface

// tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );
tabframe = new QTabWidget( object_controls, "axiscontrols" );
tabframe->setGeometry( QRect( 1, 2, 700, 190 ) );
tabframe->setTabShape( QTabWidget::Triangular );
tabframe->setAutoMask( FALSE );
if (platinum) tabframe->setStyle( platinum );


//this has become the settings panel need to update variable names
scenecontrols = new QWidget( EffectsFrame, "scenesettings" );
tabframe->insertTab( scenecontrols, tr( "  MEDIA  " ) );

objectcontrols = new QWidget( EffectsFrame, "objectcontrols" );
tabframe->insertTab( objectcontrols, tr( "  ASSET  " ) );

axiscontrols = new QWidget( EffectsFrame, "utilitycontrols" );
tabframe->insertTab( axiscontrols, tr( "  UTILITY  " ) );


/////////////////////////////////////////////////////////////////////
// the layout boxes for different types of layers
// these should be changed and regroups to represent tabs above
// these should be arrays

// the axis control tab bar
// this is just really world and layer now
// we can get rid of the rest

    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////
    // the main player options

    //mover this into the layers...
    ControlFrame0 = new QFrame( scenecontrols, "ControlFrame3" );
    ControlFrame0->setGeometry( QRect( 0, 0, 660, 170 ) );

    //-- LCD Inputs
    mintext = new QLabel( ControlFrame0, "sframetext" );
    mintext->setGeometry( QRect( 15, 5, 100, 25 ) );
    mintext->setText( tr( "STARTFRAME" ) );
    mintext->setAlignment( int( QLabel::AlignLeft ) );
    QFont mintext_font(  mintext->font() );
    mintext_font.setPointSize( 9 );
    mintext->setFont( mintext_font );

    minFrameInput = new InputLCD(ControlFrame0); //controlers
    addSliderLcd(minFrameInput, 120, 5, 45, 25, 0);

    connect(minFrameInput, SIGNAL(valueChanged(int)), SLOT(SetMinFrame(int)) );

    maxtext = new QLabel( ControlFrame0, "eframetext" );
    maxtext->setGeometry( QRect( 15, 45, 100, 25 ) );
    maxtext->setText( tr( "ENDFRAME"  ) );
    maxtext->setAlignment( int( QLabel::AlignLeft ) );
    QFont maxtext_font(  maxtext->font() );
    maxtext_font.setPointSize( 9 );
    maxtext->setFont( maxtext_font );

    maxFrameInput = new InputLCD(ControlFrame0); //controlers
    addSliderLcd(maxFrameInput, 120, 45, 45, 25, 0);

    connect(maxFrameInput, SIGNAL(valueChanged(int)), SLOT(SetMaxFrame(int)) );

    //fps data
    fpstext = new QLabel( ControlFrame0, "eframetext" );
    fpstext->setGeometry( QRect( 200, 5, 100, 25 ) );
    fpstext->setText( tr( "ActualFPS"  ) );
    fpstext->setAlignment( int( QLabel::AlignLeft ) );
    QFont fpstext_font(  fpstext->font() );
    fpstext_font.setPointSize( 9 );
    fpstext->setFont( fpstext_font );

    actualFps	= new QLabel(ControlFrame0);
    actualFps->setText(" ");
    actualFps->setFrameStyle(QFrame::Sunken | QFrame::Box);
    actualFps->setAlignment( AlignCenter );
    actualFps->setGeometry( QRect( 320, 5, 45, 25 ) );

    //-- status line
    statustext = new QLabel( ControlFrame0, "eframetext" );
    statustext->setGeometry( QRect( 15, 100, 100, 25 ) );
    statustext->setText( tr( "Player Status"  ) );
    statustext->setAlignment( int( QLabel::AlignLeft ) );
    QFont statustext_font(  statustext->font() );
    statustext_font.setPointSize( 9 );
    statustext->setFont( statustext_font );

    statusline	= new QLabel(ControlFrame0);
    statusline->setText("no frames loaded.");
    statusline->setFrameStyle(QFrame::Sunken | QFrame::Box);
    statusline->setAlignment( AlignCenter );
    statusline->setFont ( QFont("Helvetica", 9, QFont::Bold) );
    statusline->setGeometry( QRect( 120, 100, 250, 25 ) );

    //for the ram player
    avitext = new QLabel( ControlFrame0, "eframetext" );
    avitext->setGeometry( QRect( 420, 10, 100, 25 ) );
    avitext->setText( tr( "RAM Player"  ) );
    avitext->setAlignment( int( QLabel::AlignLeft ) );
    QFont avitext_font(  avitext->font() );
    avitext_font.setPointSize( 9 );
    avitext->setFont( avitext_font );

    aviEncode = new QPushButton( ControlFrame0, "aviencode" );
    aviEncode->setGeometry( QRect( 420, 40, 86, 25 ) );
    aviEncode->setText( tr( "Toggle RAM player"  ) );
    connect( aviEncode, SIGNAL(clicked()), SLOT(launchRamPlayer()) );


    ///////////////////////////////////////////////////////////////////////
    //set up asset data menus

    ControlFrame1 = new QFrame( objectcontrols, "ControlFrame1" );
    ControlFrame1->setGeometry( QRect( 0, 0, 660, 170 ) );    // was 510

    playeroutputtext = new QLabel( ControlFrame1, "sframetext" );
    playeroutputtext->setGeometry( QRect( 15, 5, 100, 25 ) );
    playeroutputtext->setText( tr( "Asset Data" ) );
    playeroutputtext->setAlignment( int( QLabel::AlignLeft ) );
    QFont playeroutputtext_font(  playeroutputtext->font() );
    playeroutputtext_font.setPointSize( 9 );
    playeroutputtext->setFont( playeroutputtext_font );


    playerOutput = new QMultiLineEdit(ControlFrame1, "assetdatabox");
    playerOutput->setGeometry( QRect( 120, 5, 550, 169 ) );    // was 510

    ///////////////////////////////////////////////////////////////////////
    //raise main widget here

    controlpanel->raiseWidget(1);

}


//this was taken from World/glworldinterface since we dont share code with it
//we could break out all interface building code into its own library
//and put it in libraries folder!

void MainPlayer::addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range) {

    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    lcd->setGeometry( QRect( x, y, w, h ) );
    lcd->setPalette( p, TRUE );
    lcd->setLineWidth( 1 );
    lcd->setSegmentStyle( QLCDNumber::Flat );  lcd->setActive	( true);
    lcd->setValue( 0 );

}

void MainPlayer::addJahPlayerButton( JahToolButton * button, int x1, int y1, int x2, int y2,
                                int w, int h, QString iconOff, QString iconOn ) {

    button->setToggleButton(false);
    //button->setFixedSize(w,h);
    button->setUsesBigPixmap(true);
    button->setGeometry( QRect( x1, y1, x2, y2 ) );

    QIconSet *icoSet;
    icoSet = new QIconSet( QPixmap(iconOff) );
    icoSet->setPixmap    ( QPixmap(iconOn), QIconSet::Large, QIconSet::Active);
    icoSet->setPixmap    ( QPixmap(iconOff), QIconSet::Large, QIconSet::Disabled);
    icoSet->setPixmap    ( QPixmap(iconOn), QIconSet::Small, QIconSet::Active);
    icoSet->setPixmap    ( QPixmap(iconOff), QIconSet::Small, QIconSet::Disabled);
    button->setIconSet(*icoSet);
    //delete icoSet;

}

void MainPlayer::switchModeFps(void)
{
    int fpsval;

QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( "15",4,0 );
    menu->insertItem( "24",3,0 );
    menu->insertItem( "30",2,0 );
    menu->insertItem( "60",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( scrubrender->mapToGlobal( QPoint( 0, scrubrender->height() + 1 ) ) );

    if ( id != -1 ) {

    switch(id) {
        case 1  : { fpsval = 60; break; }
        case 2  : { fpsval = 30; break; }
        case 3  : { fpsval = 24; break; }
        case 4  : { fpsval = 15; break; }
        default : { fpsval = 30; break; }
    }
    }

    delete menu;

    switchFps(fpsval);

}

void MainPlayer::switchModeStep(void)
{
    int stepval;

    QPopupMenu *menu = new QPopupMenu( 0 );

    menu->insertItem( "1",4,0 );
    menu->insertItem( "2",3,0 );
    menu->insertItem( "3",2,0 );
    menu->insertItem( "5",1,0 );

    menu->setMouseTracking( TRUE );
    int id = menu->exec( scrubrenderAll->mapToGlobal( QPoint( 0, scrubrenderAll->height() + 1 ) ) );

    if ( id != -1 ) {
    //if (numberLayers==NUMBER_LAYERS) return;
    //numberLayers+=1; layNum.setNum(numberLayers);

    switch(id) {
        case 1  : { stepval = 5; break; }
        case 2  : { stepval = 3; break; }
        case 3  : { stepval = 2; break; }
        case 4  : { stepval = 1; break; }
        default : { stepval = 1; break; }
    }
    }

    delete menu;

    SetStep (stepval);
}
