/* flip - pageflipper for TIFF animations
 * Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
 *
 * "flip" is free software; you can redistribute it and/or use
 * it and/or modify it under the terms of the "Artistic License"
 */

#include "InputLCD.h"
#include "qdatetime.h"
#include "jahcalc.h"

#include <limits.h>
#include <math.h>

InputLCD::InputLCD (QWidget* parent, const char* name, const int theval)
	: QLCDNumber (parent, name)
{
    setFrameStyle( QFrame::Sunken | QFrame::Panel );
    setNumDigits( theval );
    setMode( QLCDNumber::DEC );
    setCursor (sizeHorCursor);

    minInt		= -(INT_MAX-1);
    maxInt		=  (INT_MAX-1);
    lcdval=0;
    LCDactive	= FALSE;
    display (" ");
    increment = false;
    lcdsize = theval;
}

//----------------------------------------------------------------
void	InputLCD::setActive (bool status)
{
	LCDactive = status;
	if (LCDactive == FALSE)
	{
		display (" ");
	}
}	

//----------------------------------------------------------------
void	InputLCD::setIncrement (int val)
{

    increment=true;
    incrementVal = val;

}	

//----------------------------------------------------------------
void	InputLCD::setValue (int val)
{

	LCDactive	= TRUE;

  //we turned off error checking here...
	//if (val < minInt)	val = minInt;
	//if (val > maxInt)	val = maxInt;
	
  //ok, we have to hardcode our time display since qt doesnt do 30 frames per second
  //it only does milliseconds
    if (increment==true)
    { 
        QString i; i.setNum(val*incrementVal);
        //debug("incremented:"+i);
        display (i); lcdval = val;
    }
    else
    {
        if (lcdsize==4 ) { display (val); }
        else 
        {
            int hh=0; int mm=0; int ss=0; int ms=0;

            for (int x=1; x<val; x++) 
            {
                ms+=1;
                if (ms==30) { ms=0; ss+=1; }
                if (ss==60) { ss=0; mm+=1; }
                if (mm==60) { mm=0; hh+=1; }
            }
            
            QString s;
            s.sprintf( "%02d:%02d:%02d", mm, ss,ms );
            display( s );
            lcdval = val;
        }
    }
}
//----------------------------------------------------------------
void	InputLCD::setMinInt (int val)
{
	minInt = val;
}
void	InputLCD::setMaxInt (int val)
{
	maxInt = val;
}

void	InputLCD::setMinMaxInt (int min, int max)
{
	minInt = min;
	maxInt = max;
}

//----------------------------------------------------------------
void InputLCD::mousePressEvent ( QMouseEvent *msEv )
{   
    //ButtonState QMouseEvent::button () const
    if (msEv->button()== LeftButton)
    {
        lastXPos	= msEv->x();
    }
    else if (msEv->button()== RightButton)
    {
          
        int result;

        JahCalc *  thecalc = new JahCalc (this,"jahcalc",TRUE);

        if ( thecalc->exec() == QDialog::Accepted ) 
        {
            result = (int)thecalc->returnResult;

            emit	valueChanged(result);
            display (result);
        }

        delete thecalc;
    }
}

void InputLCD::mouseDoubleClickEvent ( QMouseEvent *msEv )
{ //debug("mouse double clicked pressed in lcd");
	//set value to zero
  emit	valueChanged(0);
  display (0);
}


//----------------------------------------------------------------
void InputLCD::mouseMoveEvent ( QMouseEvent *msEv )
{   

    int tempInt;
    int newInt;

    if (LCDactive)
    { //debug("and lcd is active");
        tempInt = msEv->x()-lastXPos;
        
        switch (msEv->state())
        {
            case (LeftButton):
                break;
            case (LeftButton | ControlButton) :
                tempInt *= 10;
                break;
            default :
                tempInt = 0;
        }

        

        int thecurrentvalue;
        //if we are doing timecode we use the timecode value
        if (lcdsize==4) { thecurrentvalue=intValue(); }
        else            { thecurrentvalue=lcdval; }

        newInt = thecurrentvalue + tempInt;


        if (thecurrentvalue != newInt )
        {
            if (increment==true) 
            {        
                //needs to be rounded to the next IncrementVal
                //so we can drag to the right
                int theint = newInt/incrementVal; 
                
                if (newInt > (lcdval * incrementVal)) { theint+=1; }   
                setValue (theint);
                emit    incrementValueChanged(theint);
                //lcdval = newInt;

            }
            else
            {   
                //min fix for timecode
                if (lcdsize!=4 && newInt<=0) { newInt = 1; }
            
                emit	valueChanged(newInt);
                setValue (newInt);
            }
	    }
    
    }
    
	lastXPos = msEv->x();
}

