/*******************************************************************************
**
** The source file for the Jahshaka jahnodes class
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** Released under the GNU General Public License
**
*******************************************************************************/

#include "keyframer.h"

JahKeyframer::JahKeyframer(QWidget* theparent, QWidget* parent, const char* name, WFlags f) :
QWidget(parent,name,f)  
{

    keyframerui = new QFrame( this, "keyframerui" );
    keyframerui->setGeometry( QRect( 0, 0, 664, 170 ) );
    keyframeruiLayout = new QHBoxLayout( keyframerui, 11, 6, "keyframeruiLayout"); 

    keyframer = new QFrame( keyframerui, "keyframer" );
    keyframeruiLayout->addWidget( keyframer );

    //canvas = new QCanvas(980,306);
    canvas = new QCanvas(keyframer);

    QColor back;
    back.setHsv(35,35,35);
    canvas->setBackgroundColor(back);
    canvas->resize( 700, 200 );

    keyeditor = new KeyframeEditor(canvas,keyframer);
    keyeditor->resize( 700, 200 );
    keyeditor->setHScrollBarMode( KeyframeEditor::AlwaysOff ); //AlwaysOn
    keyeditor->setVScrollBarMode( KeyframeEditor::AlwaysOff ); //AlwaysOff

    keycontrols = new QFrame( keyframerui, "keycontrols" );
    keycontrols->setMinimumSize( QSize( 34, 0 ) );
    keycontrols->setMaximumSize( QSize( 34, 32767 ) );

    keycontrolsLayout = new QVBoxLayout( keycontrols, 0, 0, "keycontrolsLayout");
    QSpacerItem* spacer = new QSpacerItem( 21, 41, QSizePolicy::Minimum, QSizePolicy::Expanding );
    keycontrolsLayout->addItem( spacer );

    addKeyframe = new QPushButton( keycontrols, "addKeyframe" );
    addKeyframe->setMinimumSize( QSize( 32, 32 ) );
    addKeyframe->setMaximumSize( QSize( 32, 32 ) );
    keycontrolsLayout->addWidget( addKeyframe );
    connect( addKeyframe, SIGNAL(clicked()), SLOT(emitAddKeyframe() ) );

    removeKeyframe = new QPushButton( keycontrols, "removeKeyframe" );
    removeKeyframe->setMinimumSize( QSize( 32, 32 ) );
    removeKeyframe->setMaximumSize( QSize( 32, 32 ) );
    keycontrolsLayout->addWidget( removeKeyframe );
    QSpacerItem* spacer_2 = new QSpacerItem( 21, 41, QSizePolicy::Minimum, QSizePolicy::Expanding );
    keycontrolsLayout->addItem( spacer_2 );
    connect( removeKeyframe, SIGNAL(clicked()), SLOT(emitRemoveKeyframe() ) );

    prevKeyframe = new QPushButton( keycontrols, "prevKeyframe" );
    prevKeyframe->setMinimumSize( QSize( 32, 32 ) );
    prevKeyframe->setMaximumSize( QSize( 32, 32 ) );
    keycontrolsLayout->addWidget( prevKeyframe );
    connect( prevKeyframe, SIGNAL(clicked()), SLOT(emitPrevKeyframe() ) );

    nextKeyframe = new QPushButton( keycontrols, "nextKeyframe" );
    nextKeyframe->setMinimumSize( QSize( 32, 32 ) );
    nextKeyframe->setMaximumSize( QSize( 32, 32 ) );
    keycontrolsLayout->addWidget( nextKeyframe );
    QSpacerItem* spacer_3 = new QSpacerItem( 21, 51, QSizePolicy::Minimum, QSizePolicy::Expanding );
    keycontrolsLayout->addItem( spacer_3 );
    keyframeruiLayout->addWidget( keycontrols );
    connect( nextKeyframe, SIGNAL(clicked()), SLOT(emitNextKeyframe() ) );

    languageChange();

    //this flips the world view matrix vertically so we are drawing up instead of down
    //QWMatrix m = keyeditor->worldMatrix();
    //m.scale( 1, -1 );
    //keyeditor->setWorldMatrix( m );
    //keyeditor->resize( 700, 200 );

    //can be used later on to change keyframes when they are changed in the keyeditor
    //connect( editor,   SIGNAL(updateEditanimframe(int)),   theparent,   SLOT(updateEditanimframe(int)) );
    //connect( editor,   SIGNAL(updateSetinFrames(int,int)), theparent,   SLOT(updateSetinFrames(int,int)) );

    static int r=24;
    srand(++r);

    clear();

    buildInterface();

}

void JahKeyframer::languageChange()
{
    //setCaption( tr( "Form1" ) );
    addKeyframe->setText( tr( "+" ) );
    removeKeyframe->setText( tr( "-" ) );
    prevKeyframe->setText( tr( "<<" ) );
    nextKeyframe->setText( tr( ">>" ) );
}

JahKeyframer::~JahKeyframer()
{   //delete printer; 
}

void JahKeyframer::clear() 
{   
    //debug("clearing editor");
    keyeditor->cleareditor();
    //debug("cleared editor");
}

void JahKeyframer::enlarge()
{    
    //canvas->resize(canvas.width()*4/3, canvas.height()*4/3);
}

void JahKeyframer::shrink()
{    
    //canvas->resize(canvas.width()*3/4, canvas.height()*3/4);
}

void JahKeyframer::zoomIn()
{   
    QWMatrix m = keyeditor->worldMatrix();    
    m.scale( 2.0, 2.0 );    
    keyeditor->setWorldMatrix( m );
}

void JahKeyframer::zoomOut()
{    
    QWMatrix m = keyeditor->worldMatrix();    
    m.scale( 0.5, 0.5 );    
    keyeditor->setWorldMatrix( m );
}


//qt signals are here
void JahKeyframer::emitAddKeyframe(void)    { emit AddKeyframe(); }
void JahKeyframer::emitRemoveKeyframe(void) { emit RemoveKeyframe(); }
void JahKeyframer::emitPrevKeyframe(void)   { emit PrevKeyframe(); }
void JahKeyframer::emitNextKeyframe(void)   { emit NextKeyframe(); }

//this one draws random keyframes for testing purposes
void JahKeyframer::updateKeyframes()
{ 
    clear();

    buildInterface();

    buildKeyframes();

    canvas->setAllChanged();
    canvas->update();

    keyeditor->update();
}

void JahKeyframer::updateKeyframes(int numFrames, QPtrList<motionNode> * nodelist)
{   
    clear();

    buildInterface();

    buildKeyframes(numFrames, nodelist);

    canvas->setAllChanged();
    canvas->update();

    keyeditor->update();
}



void JahKeyframer::buildInterface()
{
    QCanvasLine* i = new QCanvasLine(canvas);
    i->setPen( QPen(Qt::green) );
    i->setPoints( 0, 143,	  800, 143 );
    i->setZ(128);
    i->show();

    QCanvasLine* y = new QCanvasLine(canvas);
    y->setPen( QPen(Qt::green) );
    y->setPoints( 2, 0,	  2, 155 );
    y->setZ(128);
    y->show();

}

void JahKeyframer::drawLine(int x1, int y1, int x2, int y2)
{
    QCanvasLine* i = new QCanvasLine(canvas);
    i->setPoints( x1, y1,	  x2,y2 );
    i->setPen( QPen(Qt::white) );
    i->setZ(128);
    i->show();

}

//////////////////////////////////////////
// adds the playbar vertical line
void JahKeyframer::addLine()
{
    QCanvasLine* i = new SliderItem(canvas);
    i->setPoints(canvas->width()/2, 0,
                canvas->width()/2, canvas->height() );

    i->setPen( QPen(QColor(100,0,0), 6) );
    i->setZ(3);
    i->show();
}

//initially this will be display only :)
//with zoom and scroll and timeline 
void JahKeyframer::buildKeyframes()
{
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug("JahKeyframer::buildKeyframes()","in empty buildkeyframes");
}

//initially this will be display only :)
//with zoom and scroll and timeline 
void JahKeyframer::buildKeyframes(int numFrames, QPtrList<motionNode> * nodelist)
{
    //set up tracer
    JahTrace& jtrace = JahTrace::getInstance();
    jtrace.debug( "JahKeyframer::","running buildKeyframes");
    jtrace.debug( "numnodes",numFrames);

    //the keyframer widget is using its own colors for now
    //if ( !tb ) tb = new QBrush( Qt::red );
    //if ( !tp ) tp = new QPen( Qt::white );

    int nodecount = 0;

    //int w = canvas->width();
    //int h = canvas->height();

    //number of nodes
    int   nodes = numFrames;

    //use an array is cool as we use an array for nodes right now
    QMemArray<KeyframeItem*> lastRow(nodes);

    KeyframeItem *prev = 0;

    for ( int i = 0; i < nodes; i++ ) 
    {
        KeyframeItem *el = new KeyframeItem( canvas );
        nodecount++;

        int xval; int yval;

        xval = nodelist->at(i)->nodenum*4;
        yval = (int)nodelist->at(i)->tx/4;
        yval +=50;

        //jtrace.debug( "keyframe",xval);
        //jtrace.debug( "x val",yval);


        el->move( xval ,yval );

        if ( prev ) { (new KeyEdgeItem( prev, el, canvas ))->show();  }
        if ( i > 0 ) lastRow[i-1] = prev;
        prev = el;

        el->show();

        //if i=0 first keyframe draw line from offscreen infinity to curent point
        if (i==0)  { (new KeyEdgeItem( 0, yval, el, canvas ))->show(); }

        //if i=nodes last keyframe draw line to offscreen infinity from curent point
        if (i==nodes-1) { (new KeyEdgeItem( el, 800, yval, canvas ))->show(); }

    }

    //tb->setColor( Qt::red );
    //tp->setColor( Qt::blue );

    //nprev = 0;
    nodecount = 0;

    //number of nodes
    nodes = numFrames;

    //use an array is cool as we use an array for nodes right now
    QMemArray<KeyframeItem*> lastRow2(nodes);

    KeyframeItem *prev2 = 0;


    for ( int ii = 0; ii < nodes; ii++ ) 
    {
        KeyframeItem *el = new KeyframeItem( canvas );
        nodecount++;

        int xval; int yval;

        xval = (int)nodelist->at(ii)->nodenum*4;
        yval = (int)nodelist->at(ii)->ty/4;
        yval +=50;

        //jtrace.debug( "keyframe",xval);
        //jtrace.debug( "y val",yval);

        el->move( xval ,yval );

        if ( prev2 ) { (new KeyEdgeItem( prev2, el, canvas ))->show();  }
        if ( ii > 0 ) lastRow2[ii-1] = prev2;
        prev2 = el;

        el->show();

        //if i=0 first keyframe draw line from offscreen infinity to curent point
        if (ii==0)
        {
            (new KeyEdgeItem( 0, yval, el, canvas ))->show();
        }

        //if i=nodes last keyframe draw line to offscreen infinity from curent point
        if (ii==nodes-1)
        {
            (new KeyEdgeItem( el, 800, yval, canvas ))->show();
        }

    }

    nodecount = 0;

    //number of nodes
    nodes = numFrames;

    //use an array is cool as we use an array for nodes right now
    QMemArray<KeyframeItem*> lastRow3(nodes);

    KeyframeItem *prev3 = 0;


    for ( int iii = 0; iii < nodes; iii++ ) 
    {
        KeyframeItem *el = new KeyframeItem( canvas );
        nodecount++;

        int xval; int yval;

        xval = nodelist->at(iii)->nodenum*4;
        yval = (int)nodelist->at(iii)->tz/4;
        yval +=50;

        //jtrace.debug( "keyframe",xval);
        //jtrace.debug( "z val",yval);

        el->move( xval ,yval );

        if ( prev3 ) { (new KeyEdgeItem( prev3, el, canvas ))->show();  }
        if ( iii > 0 ) lastRow3[iii-1] = prev3;
        prev3 = el;

        el->show();

        //if i=0 first keyframe draw line from offscreen infinity to curent point
        if (iii==0)
        {
            (new KeyEdgeItem( 0, yval, el, canvas ))->show();
        }

        //if i=nodes last keyframe draw line to offscreen infinity from curent point
        if (iii==nodes-1)
        {
            (new KeyEdgeItem( el, 800, yval, canvas ))->show();
        }

    }
}
