/*******************************************************************************
**
** A Jahshaka RasFx Plugin
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** This is Open Source Software Released under the LGPL License
**
*******************************************************************************/

#include "rfxnvblur.h"
#include <math.h>

void    MyPlugin::isComplexArbSupported(void)
{
	QString sourceName = tempBasePath+"plugins/rfxplugins/rfxnvblur/blur3_frag_arb_gpu.fp";
	HASARB =  checkComplexArbSupport(sourceName, max_fp_instructions, max_vp_instructions );

    if (max_fp_instructions < 200)
    {
        HASARB = false;  // More resource checking required - FIXME
    }
}


void    MyPlugin::processGpuArbFx(void)
{
    float                                       horizontal;
    float                                       vertical;
    float                                       stretch;
    float                                       quality;
    float                                       weight;

    GLint                                       width;
    GLint                                       height;

    GLuint                                      fragment_program_handle;

    unsigned char*                              fragment_program;

//     fxtimer->starttime();

    width = (GLint)gpuwidth;
    height = (GLint)gpuheight;

	//debug("ARB blur");

    float float_width = (float)width;
    float float_height = (float)height;

    horizontal = (slider[1] / 10.0) + 1.0;
    vertical   = slider[2] / 10.0;

    stretch = (slider[4] / 10000.0) + 0.003;
    quality = slider[5] / 10.0;
    weight  = (slider[6] / 10.0) + 0.5;

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texid);

    QString qualityshader;

#if 0
    if      (quality < 1.0) { qualityshader="blur3_frag_arb_gpu.fp";  }
    else if (quality < 2.0) { qualityshader="blur7_frag_arb_gpu.fp";  }
    else if (quality < 3.0) { qualityshader="blur10_frag_arb_gpu.fp"; }
    else                    { qualityshader="blur15_frag_arb_gpu.fp"; }
    //fprintf(stderr, "qualityshader = %s\n", qualityshader.ascii());
#else
    // For some reason the other shaders produce a GL_INVALID_VALUE
    qualityshader="blur3_frag_arb_gpu.fp";
#endif

    fragment_program = loadshaderfile(JahBasePath + "rfxnvblur/" + qualityshader);

    glGenProgramsARB(1, &fragment_program_handle);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, fragment_program_handle);
    glProgramStringARB( GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB,
                        strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION)
    {
        find_shader_program_error(fragment_program, (char*)qualityshader.ascii());
    }

    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, horizontal, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 1, vertical, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 2, stretch, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 3, weight, 0.0f, 0.0f, 0.0f);

    glEnable(GL_FRAGMENT_PROGRAM_ARB);

    glBegin(GL_QUADS);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, 0.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio, 0.0, 0.0, 1.0); 
            glVertex2f( float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio, 
                                texheightratio, 0.0, 1.0);
            glVertex2f( float_width / 2.0, float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, texheightratio, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, float_height / 2.0);

    glEnd();

    glBindTexture(GL_TEXTURE_2D,texid);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (renderwidth - width) / 2,
                        (renderheight - height) / 2, width, height);

    glDisable(GL_FRAGMENT_PROGRAM_ARB);

    delete [] fragment_program;

    glDeleteProgramsARB(1, &fragment_program_handle);

//     fxtimer->endtime();
//     fxtimer->printtime("Time for ARB Blur: ");
}


/*
void    MyPlugin::processGpuArbFx(void)
{
    ////////////////////////////////////////////
    //main imageprocessing loop

    float                                       horizontal;
    float                                       vertical;
    float                                       stretch;
    float                                       quality;
    float                                       weight;

    GLint                                       width;
    GLint                                       height;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    static int                                  count = 0;

    //////////////////////////////////////////////////////////
    //we need to send over...

    width = (GLint)gpuwidth;
    height = (GLint)gpuheight;

    float float_width = (float)width;
    float float_height = (float)height;

    //////////////////////////////////////////////////////////

    horizontal = (slider[1] / 10.0) + 1.0;
    vertical = slider[2] / 10.0;

    stretch = (slider[3] / 10000.0) + 0.003;
    quality = slider[4] / 10.0;

    weight = (slider[5] / 10.0) + 0.5;

    //////////////////////////////////////////////////////////


    //if (count == 0)
    {
        vertex_program = loadshaderfile(JahBasePath+"rfxcore/jahshaka_basic_vert_arb.vp");
    }

    loadJahshakaBasicArb(width, height,camera_distance, vertex_program, vertex_program_handle);

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texid);

	char				fragment_program_name[500];

    if (quality < 1.0)
    {
		strcpy(fragment_program_name, JahBasePath+"rfxnvblur/blur3_frag_arb_gpu.fp");
    }
    else if (quality < 2.0)
    {
		strcpy(fragment_program_name, JahBasePath+"rfxnvblur/blur7_frag_arb_gpu.fp");
    }
    else if (quality < 3.0)
    {
		strcpy(fragment_program_name, JahBasePath+"rfxnvblur/blur10_frag_arb_gpu.fp");
    }
    else
    {
		strcpy(fragment_program_name, JahBasePath+"rfxnvblur/blur15_frag_arb_gpu.fp");
    }

    fragment_program = loadshaderfile(fragment_program_name);

    glGenProgramsARB(1, &fragment_program_handle);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, fragment_program_handle);
    glProgramStringARB( GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB,
                        strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION)
    {
        find_shader_program_error(fragment_program, fragment_program_name);
    }


    /////////////////////////////////////////////////////////

    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, horizontal, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 1, vertical, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 2, stretch, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 3, weight, 0.0f, 0.0f, 0.0f);

    glEnable(GL_FRAGMENT_PROGRAM_ARB);

    //////////////////////////////////////////////////////////////////////////
    glBegin(GL_QUADS);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, 0.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio, 0.0, 0.0, 1.0);
            glVertex2f( float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio,
                                texheightratio, 0.0, 1.0);
            glVertex2f( float_width / 2.0, float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0,
                                texheightratio, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, float_height / 2.0);


    glEnd(); //GL_QUADS

    ////////////////////////////////////////////////////////////////////////

    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (renderwidth - width) / 2,
                        (renderheight - height) / 2, width, height);

    /////////////////////////////////////////////////////////////////////

    glDisable(GL_VERTEX_PROGRAM_ARB);
    glDisable(GL_FRAGMENT_PROGRAM_ARB);

	delete vertex_program;
	delete fragment_program;

    count++;

    glDeleteProgramsARB(1, &vertex_program_handle);
    glDeleteProgramsARB(1, &fragment_program_handle);
}

*/

