/*******************************************************************************
**
** A Jahshaka RasFx Plugin
** The Jahshaka Project
** Copyright (C) 2000-2004 The Jahshaka Project
** This is Open Source Software Released under the LGPL License
**
*******************************************************************************/

#include "rfxnvedgedetect.h"
#include <math.h>

void    MyPlugin::isComplexArbSupported(void)
{
	QString sourceName = tempBasePath+"plugins/rfxplugins/rfxnvedgedetect/edgedetect_frag_arb_gpu.fp";

	HASARB =  checkComplexArbSupport(sourceName, max_fp_instructions, max_vp_instructions );

    if (max_fp_instructions < 200)
    {
        HASARB = false;  // More resource checking required - FIXME
    }

    HASARB = false;  // Something is broken - getting GL ERROR GL_INVALID_VALUE
}


void    MyPlugin::processGpuArbFx(void)
{
    float                                       horizontal;
    float                                       vertical;
    float                                       stretch;
    float                                       quality;
    float                                       weight;

    float                                       transparency;
    float                                       blend;

    GLint                                       width;
    GLint                                       height;

    GLuint                                      fragment_program_handle;

    unsigned char*                              fragment_program;

//     fxtimer->starttime();

//     horizontal = (layerlist->at(effectobj)->thelayer->node->tx / 10.0) + 2.0;
//     vertical = layerlist->at(effectobj)->thelayer->node->ty / 10.0;
    horizontal = 2.0;
    vertical   = 0.0;

    stretch      = (slider[1] / 10000.0) + 0.002;
    blend        = (slider[2] / 500.0) + 1.0;
    transparency = (slider[3] / 100.0) + 0.05;
    quality      = 0.0;
    weight       = (slider[4]/ 10000.0) - 0.30;

    float float_width = (float)width;
    float float_height = (float)height;

	debug("ARB edgedetect");

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texid);

    fragment_program = loadshaderfile(JahBasePath+"rfxnvedgedetect/edgedetect_frag_arb_gpu.fp");

    glGenProgramsARB(1, &fragment_program_handle);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, fragment_program_handle);
    glProgramStringARB( GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB,
                        strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION)
    {
        find_shader_program_error(fragment_program, "edgedetect_frag_arb_gpu.fp");
    }

    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, horizontal, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 1, vertical, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 2, stretch, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 3, blend, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 4, transparency, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 5, weight, 0.0f, 0.0f, 0.0f);

    glEnable(GL_FRAGMENT_PROGRAM_ARB);

    glBegin(GL_QUADS);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, 0.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                                texwidthratio, 0.0, 0.0, 1.0); 
            glVertex2f( float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 
                                texwidthratio, 
                                texheightratio, 0.0, 1.0);
            glVertex2f( float_width / 2.0, float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, texheightratio, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, float_height / 2.0);

    glEnd();

    glBindTexture(GL_TEXTURE_2D,texid);
    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (renderwidth - width) / 2, 
                        (renderheight - height) / 2, width, height);

    glDisable(GL_FRAGMENT_PROGRAM_ARB);

    delete [] fragment_program;

    glDeleteProgramsARB(1, &fragment_program_handle);

//     fxtimer->endtime();
//     fxtimer->printtime("Time for ARB Edge Detect: ");
}


/*
void    MyPlugin::processGpuArbFx(void)
{
    ////////////////////////////////////////////
    //main imageprocessing loop

    float                                       horizontal;
    float                                       vertical;
    float                                       stretch;
    float                                       quality;
    float                                       weight;

    float                                       transparency;
    float                                       blend;

    GLint                                       width;
    GLint                                       height;

    GLuint                                      vertex_program_handle;
    GLuint                                      fragment_program_handle;

    unsigned char*                              vertex_program;
    unsigned char*                              fragment_program;

    static int                                  count = 0;

    //////////////////////////////////////////////////////////
    //we need to send over...

    width  = (GLint)gpuwidth;
    height = (GLint)gpuheight;

    float float_width = (float)width;
    float float_height = (float)height;

    //////////////////////////////////////////////////////////

    horizontal = 2.0;
    vertical = 0.0;

    stretch = (slider[1] / 10000.0) + 0.002;
    blend = (slider[2] / 500.0) + 1.0;
    transparency = (slider[3] / 100.0) + 0.05;

    quality = 0.0;
    weight = (slider[4] / 10.0) - 0.30;

    //////////////////////////////////////////////////////////

    //if (count == 0)
    {
        vertex_program = loadshaderfile(JahBasePath+"rfxcore/jahshaka_basic_vert_arb.vp");
    }

    loadJahshakaBasicArb(width, height,camera_distance, vertex_program, vertex_program_handle);

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texid);

    //if (count == 0)
    {
        fragment_program = loadshaderfile(JahBasePath+"rfxnvedgedetect/edgedetect_frag_arb_gpu.fp");
    }

    glGenProgramsARB(1, &fragment_program_handle);
    glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, fragment_program_handle);
    glProgramStringARB( GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB,
                        strlen((char*)fragment_program), (const GLubyte*)fragment_program);

    if (glGetError() == GL_INVALID_OPERATION)
    {
        find_shader_program_error(fragment_program, "edgedetect_frag_arb_gpu.fp");
    }

    /////////////////////////////////////////////////////////

    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, horizontal, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 1, vertical, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 2, stretch, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 3, blend, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 4, transparency, 0.0f, 0.0f, 0.0f);
    glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 5, weight, 0.0f, 0.0f, 0.0f);

    glEnable(GL_FRAGMENT_PROGRAM_ARB);

    //////////////////////////////////////////////////////////////////////////
    glBegin(GL_QUADS);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0, 0.0, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio, 0.0, 0.0, 1.0);
            glVertex2f( float_width / 2.0, -float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB,
                                texwidthratio,
                                texheightratio, 0.0, 1.0);
            glVertex2f( float_width / 2.0, float_height / 2.0);

            glMultiTexCoord4fARB(GL_TEXTURE0_ARB, 0.0,
                                texheightratio, 0.0, 1.0);
            glVertex2f( -float_width / 2.0, float_height / 2.0);


    glEnd(); //GL_QUADS

    ////////////////////////////////////////////////////////////////////////

    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, (renderwidth - width) / 2,
                        (renderheight - height) / 2, width, height);

    /////////////////////////////////////////////////////////////////////

    glDisable(GL_VERTEX_PROGRAM_ARB);
    glDisable(GL_FRAGMENT_PROGRAM_ARB);

	delete vertex_program;
	delete fragment_program;

    count++;

    glDeleteProgramsARB(1, &vertex_program_handle);
    glDeleteProgramsARB(1, &fragment_program_handle);
}


*/
